{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_assembly_arp;

interface

uses base_structures,
     definition_ip_addresses,
     class_parser_arp,
     class_assembly_baseclass;

type tARPAssembler  = class (tFrameAssembler)

                            constructor create;

                            private
                                   priv_HardwareType : word;
                                   priv_ProtocolType : word;
                                   priv_HardwareSize : byte;
                                   priv_ProtocolSize : byte;
                                   priv_OpCode       : word;
                                   priv_SenderMAC    : tMACAddress;
                                   priv_SenderIP     : tIPv4Address;
                                   priv_TargetMAC    : tMACAddress;
                                   priv_TargetIP     : tIPv4Address;

                            protected
                                     // called from base class from GetAssembly

                                     procedure DetermineAssemblySize; override;
                                     procedure Assemble; override;

                            public
                                  property HardwareType : word write priv_HardwareType;
                                  property ProtocolType : word write priv_ProtocolType;
                                  property HardwareSize : byte write priv_HardwareSize;
                                  property ProtocolSize : byte write priv_ProtocolSize;
                                  property OpCode : word write priv_OpCode;
                                  property SenderMAC : tMACAddress write priv_SenderMAC;
                                  property SenderIP : tIPv4Address write priv_SenderIP;
                                  property TargetMAC : tMACAddress write priv_TargetMAC;
                                  property TargetIP : tIPv4Address write priv_TargetIP;

                                  procedure CopyFromParser (Parser : tARPParser);
                      end;


implementation

uses toolkit;

(* ------------------------------------------- *)
{ tARPAssembler }
(* ------------------------------------------- *)

constructor tARPAssembler.create;

begin
     inherited;

     priv_HardwareType := 0;
     priv_ProtocolType := 0;
     priv_HardwareSize := 0;
     priv_ProtocolSize := 0;
     priv_OpCode := 0;
     priv_SenderMAC.initialize;
     priv_SenderIP.initialize;
     priv_TargetMAC.initialize;
     priv_TargetIP.initialize;
end;

(* ------------------------------------------- *)

procedure tARPAssembler.DetermineAssemblySize;

begin
     prot_length := sizeOf (tARPStruct);
end;

(* ------------------------------------------- *)

procedure tARPAssembler.Assemble;

var pData  : pARPStruct;

begin
     pData := pARPStruct (prot_data);

     pData^.HardwareType := Swap16 (priv_HardwareType);
     pData^.ProtocolType := Swap16 (priv_ProtocolType);
     pData^.HardwareSize := priv_HardwareSize;
     pData^.ProtocolSize := priv_ProtocolSize;
     pData^.Opcode := Swap16 (priv_OpCode);
     pData^.SenderMAC := priv_SenderMAC.RangeBytes;
     pData^.SenderIP := swap32 (priv_SenderIP.ValueAsLong);
     pData^.TargetMAC := priv_TargetMAC.RangeBytes;
     pData^.TargetIP := swap32 (priv_TargetIP.ValueAsLong);
end;

(* ------------------------------------------- *)

procedure tARPAssembler.CopyFromParser(Parser: tARPParser);

begin
     priv_HardwareType := Parser.HardwareType;
     priv_ProtocolType := Parser.ProtocolType;
     priv_HardwareSize := Parser.HardwareSize;
     priv_ProtocolSize := Parser.ProtocolSize;
     priv_Opcode := Parser.OpCode;
     priv_SenderMAC := Parser.SenderMAC;
     priv_SenderIP := Parser.SenderIP;
     priv_TargetMAC := Parser.TargetMAC;
     priv_TargetIP := Parser.TargetIP;
end;

(* ------------------------------------------- *)

end.
