{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_assembly_baseclass;

interface

uses SysUtils,
     base_structures;

type EAssemblerException = class (Exception) // alle Exceptions beginnen per Konvention mit einem E; Die Klasse Exception ist Vorfahr aller Exceptions

                               private
                                      priv_ErrorMessage : string;
                               public
                                     constructor Create (Msg: string);
                                     property Message : string read priv_ErrorMessage write priv_ErrorMessage;
                               end;

     tFrameAssembler = class (tObject)

                             constructor create;
                             Destructor Destroy; override;

                             private
 //                                 procedure InitMem;

                             protected
                                      prot_data         : pByte;
                                      prot_length       : integer;

                                      procedure DetermineAssemblySize; virtual; abstract;
                                      procedure Assemble; virtual; abstract;

                             public
                                   function GetAssembly : tFrameChunk;
                       end;

implementation

(* ------------------------------------------- *)
{ EAssemblerException }
(* ------------------------------------------- *)

constructor EAssemblerException.Create(Msg: string);

begin
     inherited Create (Msg);
end;

(* ------------------------------------------- *)
{ tFrameParser }
(* ------------------------------------------- *)

constructor tFrameAssembler.create;

begin
     inherited create;

     prot_length := 0;
end;

(* ------------------------------------------- *)

destructor tFrameAssembler.Destroy;

begin
//   freemem (priv_data, priv_length);         // Disabled because we still need to work with it. tFramechunks will be freed later

     inherited Destroy;
end;

(* ------------------------------------------- *)
{
procedure tFrameAssembler.InitMem;

var i     : integer;
    pData : pByte;

begin
     getMem (prot_data, prot_length);

     // zero the new buffer

     pData := prot_data;

     for i := 0 to prot_length - 1 do
         begin
              pData^ := 0;

              inc (pData);
         end;
end;
}
(* ------------------------------------------- *)

function tFrameAssembler.GetAssembly : tFrameChunk;

begin
     DetermineAssemblySize;

     assert (prot_length > 0, 'Assembly length is zero, cannot assemble (tFrameAssembler.GetAssembly)');

     // get memory for the new assembly

//   InitMem;
     prot_data := AllocMem (prot_length);

     // construct the result

     Assemble;

     // And... return it

     result.Len := prot_length;
     result.Data := prot_data;
end;

(* ------------------------------------------- *)

end.
