{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_assembly_ethernet;

interface

uses base_structures,
     definition_ip_addresses,
     class_parser_ethernet,
     class_parser_llc,
     class_assembly_baseclass;

type tEthernetAssembler  = class (tFrameAssembler)

                                 constructor create;

                                 private
                                        priv_ethsrc          : tMACAddress;
                                        priv_ethdst          : tMACAddress;
                                        priv_ethertypelength : word;

                                 protected
                                          // called from base class from GetAssembly

                                          procedure DetermineAssemblySize; override;
                                          procedure Assemble; override;

                                 public
                                       property SourceAddress : tMACAddress read priv_ethsrc write priv_ethsrc;
                                       property DestinationAddress : tMACAddress read priv_ethdst write priv_ethdst;
                                       property EtherTypeLength : word read priv_ethertypelength write priv_ethertypelength;

                                       procedure CopyFromParser (Parser : tEthernetParser);
                           end;


implementation

uses toolkit;

(* ------------------------------------------- *)
{ tEthernetAssembler }
(* ------------------------------------------- *)

constructor tEthernetAssembler.create;

begin
     inherited;

     priv_ethsrc.initialize;
     priv_ethdst.initialize;
     priv_ethertypelength := 0;
end;

(* ------------------------------------------- *)

procedure tEthernetAssembler.DetermineAssemblySize;

begin
     prot_length := 14;
end;

(* ------------------------------------------- *)

procedure tEthernetAssembler.Assemble;

var pData  : pStruct_Ethernet;

begin
     pData := pStruct_Ethernet (prot_data);

     pData^.dst := priv_ethdst.RangeBytes;
     pData^.src := priv_ethsrc.RangeBytes;
     pData^.ethtypelength := swap16 (priv_ethertypelength);
end;

(* ------------------------------------------- *)

procedure tEthernetAssembler.CopyFromParser(Parser: tEthernetParser);

begin
     priv_ethdst := Parser.DestinationAddress;
     priv_ethsrc := Parser.SourceAddress;
     priv_ethertypelength := Parser.EtherTypeLength;
end;

(* ------------------------------------------- *)

end.
