{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_assembly_gre;

interface

uses base_structures,
     class_parser_gre,
     class_assembly_baseclass;


type tGREAssembler = class (tFrameAssembler)

                                constructor create;

                                private
                                       priv_ChecksumPresent   : boolean;
                                       priv_RoutingPresent    : boolean;
                                       priv_KeyPresent        : boolean;
                                       priv_SequencePresent   : boolean;
                                       priv_strictSourceRoute : boolean;
                                       priv_recursioncontrol  : byte;
                                       priv_flags             : byte;
                                       priv_versionnumber     : byte;
                                       priv_checksum          : word;
                                       priv_key               : longword;
                                       priv_sequence          : longword;
                                       priv_protocoltype      : word;

                                protected
                                         // called from base class from GetAssembly

                                         procedure DetermineAssemblySize; override;
                                         procedure Assemble; override;

                                public
                                      property CheckSumPresent : boolean read priv_ChecksumPresent write priv_ChecksumPresent;
                                      property RoutingPresent : boolean read priv_RoutingPresent write priv_RoutingPresent;
                                      property KeyPresent : boolean read priv_KeyPresent write priv_KeyPresent;
                                      property SequencePresent : boolean read priv_SequencePresent write priv_SequencePresent;
                                      property StrictSourceRoute : boolean read priv_strictSourceRoute write priv_strictSourceRoute;
                                      property RecursionControl : byte read priv_recursioncontrol write priv_recursioncontrol;
                                      property Flags : byte read priv_flags write priv_flags;
                                      property VersionNumber : byte read priv_versionnumber write priv_versionnumber;
                                      property Checkum : word read priv_checksum write priv_checksum;
                                      property Key : longword read priv_key write priv_key;
                                      property Sequence : longword read priv_sequence write priv_sequence;
                                      property ProtocolType : word read priv_protocoltype write priv_protocoltype;

                                      procedure CopyFromParser (Parser : tGREParser);
                           end;


implementation

uses toolkit;

(* ------------------------------------------- *)
{ tGREAssembler }
(* ------------------------------------------- *)

constructor tGREAssembler.create;

begin
     inherited create;

     priv_ChecksumPresent := false;
     priv_RoutingPresent := false;
     priv_KeyPresent := false;
     priv_SequencePresent := false;
     priv_strictSourceRoute := false;

     priv_recursioncontrol := 0;
     priv_flags := 0;
     priv_versionnumber := 0;
     priv_checksum := 0;
     priv_key := 0;
     priv_sequence := 0;
     priv_protocoltype := 0;
end;

(* ------------------------------------------- *)

procedure tGREAssembler.DetermineAssemblySize;

begin
     prot_length := sizeOf (tGREStruct);

     if priv_ChecksumPresent then
        inc (prot_length, 4);

     if priv_KeyPresent then
        inc (prot_length, 4);

     if priv_SequencePresent then
        inc (prot_length, 4);
end;

(* ------------------------------------------- *)

procedure tGREAssembler.Assemble;

var pData     : pGREStruct;
    help      : word;
    help2     : word;
    pLongHelp : pLongword;
    LongHelp  : Longword;
    offset    : integer;

begin
     pData := pGREStruct (prot_data);

     help := 0;

     if priv_ChecksumPresent then
        help := help or $8000;

     if priv_RoutingPresent then
        help := help or $4000;

     if priv_KeyPresent then
        help := help or $2000;

     if priv_SequencePresent then
        help := help or $1000;

     if priv_StrictSourceRoute then
        help := help or $800;

     help2 := priv_recursioncontrol shl 8;
     help := help or help2;

     help2 := priv_flags shl 3;
     help := help or help2;

     help := help or priv_versionnumber;

     pData^.FlagsVersion := swap16 (help);
     pData^.ProtocolType := swap16 (priv_protocoltype);

     offset := sizeOf (tGREStruct);

     if priv_checksumpresent then
        begin
//           LongHelp := (priv_checksum shl 16);
             LongHelp := priv_checksum;

             pLongHelp := pLongword (prot_data + offset);
             pLongHelp^ := Swap32 (LongHelp);

             inc (offset, 4);
        end;

     if priv_keypresent then
        begin
             pLongHelp := pLongword (prot_data + offset);
             pLongHelp^ := Swap32 (priv_key);

             inc (offset, 4);
        end;

     if priv_sequencepresent then
        begin
             pLongHelp := pLongword (prot_data + offset);
             pLongHelp^ := Swap32 (priv_sequence);

             inc (offset, 4);
        end;

     assert (offset = prot_length, 'Memory offset check mismatch (tGREAssembler.Assemble)');
end;

(* ------------------------------------------- *)

procedure tGREAssembler.CopyFromParser (Parser: tGREParser);

begin
     priv_ChecksumPresent := Parser.CheckSumPresent;
     priv_RoutingPresent := Parser.RoutingPresent;
     priv_KeyPresent := Parser.KeyPresent;
     priv_SequencePresent := Parser.SequencePresent;

     priv_strictSourceRoute := Parser.StrictSourceRoute;
     priv_recursioncontrol := Parser.RecursionControl;
     priv_flags := Parser.Flags;
     priv_versionnumber := Parser.VersionNumber;
     priv_checksum := Parser.Checksum;
     priv_key := Parser.Key;
     priv_sequence := Parser.Sequence;
     priv_protocoltype := Parser.ProtocolType;
end;

(* ------------------------------------------- *)

end.
