{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_assembly_ipv6_fragmentationheader;

interface

uses base_structures,
     definition_ip_addresses,
     class_parser_ipv6_fragmentationheader,
     class_assembly_baseclass;

type tIPv6FragHdrAssembler  = class (tFrameAssembler)

                             constructor create;

                                    private
                                           priv_flags          : tIPFlagSet;
                                           priv_fragmentoffset : word;
                                           priv_identification : longword;
                                           priv_nextheader     : byte;

                                    protected
                                             // called from base class from GetAssembly

                                             procedure DetermineAssemblySize; override;
                                             procedure Assemble; override;

                                    public
                                          property Offset : word read priv_fragmentoffset write priv_fragmentoffset;
                                          property Flags : tIPFlagSet read priv_flags write priv_flags;
                                          property Identification : longword read priv_identification write priv_identification;
                                          property NextHeader : byte read priv_nextheader write priv_nextheader;

                                          {$REGION 'Documentation'}
                                          ///	<summary>
                                          ///	  copies all values from a parser object
                                          ///	  to the assembler
                                          ///	</summary>
                                          {$ENDREGION}
                                          procedure CopyFromParser (Parser : tIPv6FragmentationParser);
                              end;


implementation

uses Toolkit;

(* ------------------------------------------- *)
{ tIPv6Assembler }
(* ------------------------------------------- *)

constructor tIPv6FragHdrAssembler.create;

begin
     inherited create;

     priv_flags := [];
     priv_fragmentoffset := 0;
     priv_identification := 0;
     priv_nextheader := 0;
end;

(* ------------------------------------------- *)

procedure tIPv6FragHdrAssembler.DetermineAssemblySize;

begin
     prot_length := sizeOf (tIPv6FragmentationStruct);
end;

(* ------------------------------------------- *)

procedure tIPv6FragHdrAssembler.Assemble;

var pData  : pIPv6FragmentationStruct;
    wHelp  : word;

begin
     pData := pIPv6FragmentationStruct (prot_data);

     wHelp := priv_fragmentoffset shl 3;

     if ipfMoreFragments in priv_flags then
        wHelp := wHelp or $1;

     pData^.NextHeader := priv_nextheader;
     pData^.Reserved := 0;
     pData^.OffsetAndFlags := swap16 (wHelp);
     pData^.Identification := swap32 (priv_identification);
end;

(* ------------------------------------------- *)

procedure tIPv6FragHdrAssembler.CopyFromParser (Parser: tIPv6FragmentationParser);

begin
     priv_fragmentoffset := Parser.Offset;
     priv_flags := Parser.Flags;
     priv_identification := Parser.Identification;
     priv_nextheader := Parser.NextAnchor.HeaderType;
end;

(* ------------------------------------------- *)

end.


