{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit base_messages;

// This unit provides constants for each message handle to allow coding checks which would not work if
// we'd use the handle strings instead. This unit is exported from MessageEdit.

interface

const MSG_LOADER_PREMATURE_EOF = 100;

      MSG_WRITER_INTERFACE_UNDEFINED = 101;
      MSG_WRITER_LINKID_MISMATCH = 102;
      MSG_WRITER_ERROR_WRITING_FRAME = 103;

      MSG_PARSER_PARSING_ABORTED = 200;
      MSG_PARSER_OUT_OF_DATA = 201;
      MSG_PARSER_OUT_OF_DATA_FOR_LENGTH = 202;

      MSG_IP_TOTAL_LENGTH_ZERO = 300;

      MSG_TCP_BAD_MSS_OPTION_LEN = 400;
      MSG_TCP_BAD_SACK_OPTION_LEN = 401;
      MSG_TCP_MULTIPLE_EOL_OPTION_SPECIFIED = 402;
      MSG_TCP_MULTIPLE_MSS_OPTIONS_SPECIFIED = 403;
      MSG_TCP_MULTIPLE_SACK_OPTIONS_SPECIFIED = 404;
      MSG_TCP_MULTIPLE_SCALE_OPTIONS_SPECIFIED = 405;
      MSG_TCP_MULTIPLE_TS_OPTIONS_SPECIFIED = 406;
      MSG_TCP_OPTION_CODE_INVALID = 407;
      MSG_TCP_OPTION_DATA_AFTER_EOL = 408;
      MSG_TCP_OPTION_NOT_IMPLEMENTED = 409;
      MSG_TCP_OPTION_SIZE_EXCEEDS_HEADER_LENGTH = 410;
      MSG_TCP_BAD_FASTOPEN_OPTION_LEN = 411;
      MSG_TCP_OPTION_SACK_BLOCKS_TRUNCATED = 412;

      MSG_DNS_ANSWER_PARSE_ERROR = 500;
      MSG_DNS_BROKEN_IPV4_RECORD = 501;
      MSG_DNS_BROKEN_IPV6_RECORD = 502;
      MSG_DNS_FQDN_OFFSET_OUT_OF_BOUNDS = 503;
      MSG_DNS_PARSE_ERROR = 504;
      MSG_DNS_POINTER_LOOP_EXCEEDED = 505;
      MSG_DNS_RECORD_TYPE_SKIPPED = 506;
      MSG_DNS_TXT_RECORD_LENGTH_INVALID = 507;

      MSG_FLOWHNDLR_PARSEFAILED = 600;

      function GetMessageHandle (const MessageID : integer) : string;
      function GetDiagMessageBaseText (const MessageID : integer) : string;

implementation

uses SysUtils;

(* ------------------------------------------- *)

function GetMessageHandle (const MessageID : integer) : string;

begin
     case MessageID of
      MSG_LOADER_PREMATURE_EOF                  : result := 'MSG_LOADER_PREMATURE_EOF';

      MSG_WRITER_INTERFACE_UNDEFINED            : result := 'MSG_WRITER_INTERFACE_UNDEFINED';
      MSG_WRITER_LINKID_MISMATCH                : result := 'MSG_WRITER_LINKID_MISMATCH';
      MSG_WRITER_ERROR_WRITING_FRAME            : result := 'MSG_WRITER_ERROR_WRITING_FRAME';

      MSG_PARSER_PARSING_ABORTED                : result := 'MSG_PARSER_PARSING_ABORTED';
      MSG_PARSER_OUT_OF_DATA                    : result := 'MSG_PARSER_OUT_OF_DATA';
      MSG_PARSER_OUT_OF_DATA_FOR_LENGTH         : result := 'MSG_PARSER_OUT_OF_DATA_FOR_LENGTH';

      MSG_IP_TOTAL_LENGTH_ZERO                  : result := 'MSG_IP_TOTAL_LENGTH_ZERO';

      MSG_TCP_BAD_MSS_OPTION_LEN                : result := 'MSG_TCP_BAD_MSS_OPTION_LEN';
      MSG_TCP_BAD_SACK_OPTION_LEN               : result := 'MSG_TCP_BAD_SACK_OPTION_LEN';
      MSG_TCP_MULTIPLE_EOL_OPTION_SPECIFIED     : result := 'MSG_TCP_MULTIPLE_EOL_OPTION_SPECIFIED';
      MSG_TCP_MULTIPLE_MSS_OPTIONS_SPECIFIED    : result := 'MSG_TCP_MULTIPLE_MSS_OPTIONS_SPECIFIED';
      MSG_TCP_MULTIPLE_SACK_OPTIONS_SPECIFIED   : result := 'MSG_TCP_MULTIPLE_SACK_OPTIONS_SPECIFIED';
      MSG_TCP_MULTIPLE_SCALE_OPTIONS_SPECIFIED  : result := 'MSG_TCP_MULTIPLE_SCALE_OPTIONS_SPECIFIED';
      MSG_TCP_MULTIPLE_TS_OPTIONS_SPECIFIED     : result := 'MSG_TCP_MULTIPLE_TS_OPTIONS_SPECIFIED';
      MSG_TCP_OPTION_CODE_INVALID               : result := 'MSG_TCP_OPTION_CODE_INVALID';
      MSG_TCP_OPTION_DATA_AFTER_EOL             : result := 'MSG_TCP_OPTION_DATA_AFTER_EOL';
      MSG_TCP_OPTION_NOT_IMPLEMENTED            : result := 'MSG_TCP_OPTION_NOT_IMPLEMENTED';
      MSG_TCP_OPTION_SIZE_EXCEEDS_HEADER_LENGTH : result := 'MSG_TCP_OPTION_SIZE_EXCEEDS_HEADER_LENGTH';

      MSG_DNS_ANSWER_PARSE_ERROR                : result := 'MSG_DNS_ANSWER_PARSE_ERROR';
      MSG_DNS_BROKEN_IPV4_RECORD                : result := 'MSG_DNS_BROKEN_IPV4_RECORD';
      MSG_DNS_BROKEN_IPV6_RECORD                : result := 'MSG_DNS_BROKEN_IPV6_RECORD';
      MSG_DNS_FQDN_OFFSET_OUT_OF_BOUNDS         : result := 'MSG_DNS_FQDN_OFFSET_OUT_OF_BOUNDS';
      MSG_DNS_PARSE_ERROR                       : result := 'MSG_DNS_PARSE_ERROR';
      MSG_DNS_POINTER_LOOP_EXCEEDED             : result := 'MSG_DNS_POINTER_LOOP_EXCEEDED';
      MSG_DNS_RECORD_TYPE_SKIPPED               : result := 'MSG_DNS_RECORD_TYPE_SKIPPED';
      MSG_DNS_TXT_RECORD_LENGTH_INVALID         : result := 'MSG_DNS_TXT_RECORD_LENGTH_INVALID';

      MSG_FLOWHNDLR_PARSEFAILED                 : result := 'MSG_FLOWHNDLR_PARSEFAILED';
     end;
end;

(* ------------------------------------------- *)

function GetDiagMessageBaseText (const MessageID : integer) : string;

begin
     case MessageID of
      MSG_LOADER_PREMATURE_EOF                  : result := 'MSG_LOADER_PREMATURE_EOF';

      MSG_WRITER_INTERFACE_UNDEFINED            : result := 'MSG_WRITER_INTERFACE_UNDEFINED';
      MSG_WRITER_LINKID_MISMATCH                : result := 'MSG_WRITER_LINKID_MISMATCH';
      MSG_WRITER_ERROR_WRITING_FRAME            : result := 'MSG_WRITER_ERROR_WRITING_FRAME';

      MSG_PARSER_PARSING_ABORTED                : result := 'MSG_PARSER_PARSING_ABORTED';
      MSG_PARSER_OUT_OF_DATA                    : result := 'Parser for %s had to abort parsing at offset %s due to lack of data. Last available offset was %s';
      MSG_PARSER_OUT_OF_DATA_FOR_LENGTH         : result := 'Parser for %s had to abort parsing at offset %s due to lack of data, trying to read %s more bytes. Last available offset was %s';

      MSG_IP_TOTAL_LENGTH_ZERO                  : result := 'IP total length is zero, which was probably caused by segment offloading';

      MSG_TCP_BAD_MSS_OPTION_LEN                : result := 'Bad length for "TCP MSS" option encountered. Should be %s bytes but was %s instead';
      MSG_TCP_BAD_SACK_OPTION_LEN               : result := 'Bad length for "TCP SACK permitted" option encountered. Should be %s bytes but was %s instead';
      MSG_TCP_MULTIPLE_EOL_OPTION_SPECIFIED     : result := 'Multiple TCP EOL options found';
      MSG_TCP_MULTIPLE_MSS_OPTIONS_SPECIFIED    : result := 'Multiple TCP MSS options found';
      MSG_TCP_MULTIPLE_SACK_OPTIONS_SPECIFIED   : result := 'Multiple TCP SACK allowed options found';
      MSG_TCP_MULTIPLE_SCALE_OPTIONS_SPECIFIED  : result := 'Multiple TCP Window scaling factor options found';
      MSG_TCP_MULTIPLE_TS_OPTIONS_SPECIFIED     : result := 'Multiple TCP timestamp options found';
      MSG_TCP_OPTION_CODE_INVALID               : result := 'Invalid TCP option code encountered: %s';
      MSG_TCP_OPTION_DATA_AFTER_EOL             : result := 'TCP option following EOL option';
      MSG_TCP_OPTION_NOT_IMPLEMENTED            : result := 'TCP option parsing not yet implemented for option code %s';
      MSG_TCP_OPTION_SIZE_EXCEEDS_HEADER_LENGTH : result := 'TCP option size of %s exceeds header length %s';

      MSG_DNS_ANSWER_PARSE_ERROR                : result := 'Error while parsing DNS answer: trying to read from offset %s exceeding available range of %s bytes';
      MSG_DNS_BROKEN_IPV4_RECORD                : result := 'MSG_DNS_BROKEN_IPV4_RECORD';
      MSG_DNS_BROKEN_IPV6_RECORD                : result := 'MSG_DNS_BROKEN_IPV6_RECORD';
      MSG_DNS_FQDN_OFFSET_OUT_OF_BOUNDS         : result := 'MSG_DNS_FQDN_OFFSET_OUT_OF_BOUNDS';
      MSG_DNS_PARSE_ERROR                       : result := 'Error while parsing DNS %s';
      MSG_DNS_POINTER_LOOP_EXCEEDED             : result := 'Error parsing FQDN - following too many pointers';
      MSG_DNS_RECORD_TYPE_SKIPPED               : result := 'Skipping DNS record type %s';
      MSG_DNS_TXT_RECORD_LENGTH_INVALID         : result := 'MSG_DNS_TXT_RECORD_LENGTH_INVALID';

      MSG_FLOWHNDLR_PARSEFAILED                 : result := 'MSG_FLOWHNDLR_PARSEFAILED';
     end;
end;

(* ------------------------------------------- *)

end.

