{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit base_structures;

interface

uses Contnrs,
     Vcl.Graphics;       // fr tColor

const maxFrameSize = 65535;
      maxInterfaces = 32;

      lwMaxLineSpeed = 4000000000;                                  // maximum linespeed we allow  (4Gbit/s)

      MicroSecondsPerDay = 86400000000;

      // Magic strings

      ENC_MagicString         = 'TRSNIFF data    '+chr(26);
      ENC_MagicStringLength   = 17;

      CAP_MagicString         = 'XCP' + chr(0);
      CAP_MagicStringLength   = 4;

      PCAP_MagicStringLength  = 4;

      PCAP_MagicStandard		  = $a1b2c3d4;
      PCAP_MagicStandardSwap  = $d4c3b2a1;
      PCAP_MagicModified      = $a1b2cd34;
      PCAP_MagicModified_Swap = $34cdb2a1;
      PCAP_NSEC_MAGIC			    = $a1b23c4d;
      PCAP_SWAPPED_NSEC_MAGIC	= $4d3cb2a1;

      PCAPNG_MagicString      = $0A0D0D0A;

      // Time Resolutions

      WTAP_FILE_TSPREC_MSEC   = 1;                                       // Time resolution in Milliseconds
      WTAP_FILE_TSPREC_USEC   = 2;                                       // Time resolution in Microseconds
      WTAP_FILE_TSPREC_NSEC   = 3;                                       // Time resolution in Nanoseconds

      // Capture Types

      CAPTYPE_UNKNOWN         = -1;
      CAPTYPE_NDIS	          =  0;	                                  // Capture on network interface using NDIS
      ETH_CAPTYPE_GIGPOD      =	 2;                                   // Gigabit Ethernet captured with pod
      ETH_CAPTYPE_OTHERPOD	  =  3;                                   // non-gigabit Ethernet captured with pod
      ETH_CAPTYPE_OTHERPOD2	  =  5;                                   // Gigabit Ethernet via pod ??
                                                                      // Captype 5 seen in capture from Distributed Sniffer with Version 4.50.211 software SysKonnect SK-9843 Gigabit Ethernet Server Adapter
      ETH_CAPTYPE_GIGPOD2	    =  6;                                   // Gigabit Ethernet captured with pod

      // PCAPNG Link Type Codes, see http://www.tcpdump.org/linktypes.html

      {$i linktypes.inc}

      {$i protocolheadertypes.inc}

      // Comments

      COMMENT_PSEUDOL2HDR = 'Pseudo L2 header added by TraceWrangler';

      // Return codes

      NoFrameTimeAvailable = 5000;
type
     // Enumeration Types

     tVersionStatus = (vsOkay, vsConnectionFail, vsProxyAutoConfigError, vsProxyScriptDownloadFail, vsWPADDetectionFail, vsFormatError, vsProxyFail);
     tTracefileFrameType = (tftNone, tftUnknown, tftENC, tftCAP, tftPCAP, tftPCAPngSimple, tftPCAPngDeprecated, tftPCAPngEnhanced);
     tCaptureFileType = (cfNone, cfUnknown, cfENC, cfCAP, cfPCAP, cfPCAPng);
     tDupeCheckMethod = (dcNone, dcFrameCRC, dcFullFrame);
     tCRCStatus = (crcUnknown, crcBad, crcGood);
     tErrorFlagsType = (efBadCRC, efBadPreamble, efPacketTooShort, efPacketTooLong, efCollision, efAlignment, efSymbolError, efBadStartFrameDelimiter, efWrongInterframeGap, efUnknown);
     tErrorFlagsSet = set of tErrorFlagsType;
     tPacketDirection = (pdUnknown, pdInbound, pdOutbound);
     tPacketReceptionType = (prtNotSpecified, prtUnicast, prtMulticast, prtBroadcast, prtPromiscuous);
     tMsgSeverity = (msNone, msDebug, msVerbose, msInfo, msWarning, msMajor, msCritical);
     tFileStatus = (fsPending, fsScanning, fsPreparsed, fsProcessing, fsComplete);
     tTracefileSortType = (stFilename, stFirstFrameTime);
     tProcessingStatus = (psNewFile, psScanningBasics, psScanningPCAPngStruct, psScanningConversations, psScanAborted, psReadingDatabase, psNoTaskAssigned, psTaskPending, psTaskProcessing, psTaskSkipped, psTaskComplete, psTaskAborted, psTaskFailed);
     tATErrorLocation = (ateUnknown, ateLoader, ateFrameProcessing, ateWriter);
     tFileTimestampMode = (tsmUnknown, tsmNone, tsmOriginalFile, tsmFirstFrame, tsmLastFrame, tsmSpecificDate);

//   tMultiWriteMode = (mwmNone, mwmAutomatic, mwmEtherConversation, mwmIPv4Conversation, mwmIPv6Conversation, mwmTCPConversation, mwmUDPConversation);
     tMultiWriteMode = (mwmNone, mwmEtherEndpoint, mwmIPEndpoint, mwmEtherConversation, mwmIPConversation, mwmL4Conversation);

     tReplacementEditType = (tETGeneric, tETMACAddress, tETVLANID, tETIPv4Address, tETIPv6Address, tETIPv4Subnet, tETIPv6Subnet, tETTCPPort, tETUDPPort, tETTCPSocketPort, tETUDPSocketPort, tETIPv4Conversation, tETIPv6Conversation, tETTCPConversation, tETUDPConversation);
     tReplacementEditMode = (tEMAdd, tEMEdit);

     tPseudoL2HeaderMode = (plmNone, plmSilent, plmWithFrameComment);

     tLayer4Protocol = (l4unknown, l4Any, l4TCP, l4UDP, l4ICMP, l4TCPUDP);

     tDNSOpCode = (dnsocQuery, dnsocInverseQuery, dnsocStatus, dnsocAvailable, dnsocNotify, dnsocUpdate, dnsocUnknown);
     tDNSReturnCode = (dnsrcNoError, dnsrcFormErr, dnsrcServFail, dnsrcNXDomain, dnsrcNotImp, dnsrcRefused, dnsrcYXDomain, dnsrcYXRRSet, dnsrcNXRRSet, dnsrcNotAuth, dnsrcNotZone, dnsrcBADVERS, dnsrcBADSIG, dnsrcBADKEY, dnsrcBADTIME, dnsrcBADMODE, dnsrcBADNAME, dnsrcBADALG, dnsrcAvailable, dnsrcPrivateUse);

     // WARNING: if adding more types, update Int64ToQueryTypes function!

     tDNSRecordType = (dnsrtA, dnsrtNS, dnsrtCNAME, dnsrtSOA, dnsrtPTR, dnsrtMX, dnsrtTXT, dnsrtAAAA, dnsrtSRV, dnsrtIXFR, dnsrtAFXR, dnsrtOPT, dnsrtDS, dnsrtNSEC3, dnsrtRRSIG, dnsrtSPF, dnsrtAFSDB, dnsrtAPL, dnsrtCERT, dnsrtDHCID, dnsrtDLV, dnsrtDNAME, dnsrtDNSKEY, dnsrtIPSECKEY, dnsrtKEY, dnsrtKX, dnsrtLOC, dnsrtNAPTR, dnsrtNSEC, dnsrtSIG, dnsrtTKEY, dnsrtTLSA, dnsrtSSHFP, dnsrtNSEC3PARAM, dnsrtHIP, dnsrtTSIG, dnsrtAny, dnsrtUnknown);
     tDNSRecordClass = (dnsrcInternet, dnsrcChaos, dnssrcNONE, dnssrcANY, dnsrcUnknown);
     tDNSRecordTypeSet = set of tDNSRecordType;

     tArraySanitizationMode = (asZero, asRandomize);

     tMarker = (mFoundByNRB, mFoundByEthernet, mFoundByARP, mFoundByIPv4, mFoundByIPv6, mFoundByICMP, mFoundByDNS, mFoundByNBNS, mFoundByDHCP, mFoundByExternalResearch);
     tMarkerSet = set of tMarker;

     tConversationAttribute = (caSingleFrame,caEthernetTrailerPresent,caTimeOutOfOrder,caOneSidedCommunication,caMultipleInterfaces,caSpansMultipleFiles,caIPFragmented,
                               caTCPMultiSession,caTCPSYNBlocked,caTCPMultiSYNRetry,caTCPSYNReset, caTCPPortReused, caAddressPairFlipped, caHardSliced, caNoQuickFilter, caMultiTTL);
     tConversationAttributeSet = set of tConversationAttribute;

     tConversationStoreRecType = (csrNone, csrMACEndpoint, csrIPv4Endpoint, csrIPv6Endpoint, csrMACConversation, csrIPv4Conversation, csrIPv6conversation, csrTCPConversation, csrUDPConversation, csrL4Conversation, csrNameResolution);
     tConversationStoreRecTypeSet = set of tConversationStoreRecType;

     tTunnelFlags = (tfTunnelCarrier, tfTunnelLayer, tfTunnelPayload, tfTunnelMPLS, tfTunnelGRE, tfTunnelGTPv1, tfTunnelAYIYA, tfTunnelVXLAN, tfTunnelERSPAN, tfTunnelCAPWAP, tfTunnelGeneve, tfTunnelTZSP);
     tTunnelFlagSet = set of tTunnelFlags;

     t802dot11FrameSubType = (mgmtftNone, mgmtftAssocReq, mgmtftAssocResp, mgmtftReassocReq, mgmtftReassocResp, mgmtftProbeReq, mgmtftProbeResp, mgmtftBeacon, mgmtftATIM, mgmtftDisassoc, mgmtftAuth, mgmtftDeAuth, mgmtftAction, mgmtftActionNoAck,
                              ctrlftNone, ctrlWrapper, ctrlftBlockAckReq, ctrlftBlockAck, ctrlftPSPoll, ctrlftRTS, ctrlftCTS, ctrlftACK, ctrlftCFend, ctrlftCFEndAck,
                              dftNone, dftData, dftDataCFAck, dftDataCFPoll, dftDataCFAckCFPoll, dftNull, dftCFAck, dftCFPoll, dftCFAckCFPoll, dftQoSData, dftQoSDataCFAck, dftQoSDataCFPoll, dftQoSCFAckCFPoll, dftQoSNull, dftQoSCFPollNoData, dftQoSCFAckNoData);

     tNavigationNodeStatus = (nsWarning, nsAlert, nsActive);
     tNavigationNodeStatusSet = set of tNavigationNodeStatus;

{
CS0 (Default) 	000 000 	0x00 							0
CS1 	001 000 	0x08 	8 	Scavenger 				YouTube, Gaming, P2P
CS2 	010 000 	0x10 	16 	OAM 							SNMP,SSH,Syslog
CS3 	011 000 	0x18 	24 	Signaling 				SCCP,SIP,H.323
CS4 	100 000 	0x20 	32 	Realtime 					TelePresence
CS5 	101 000 	0x28 	40 	Broadcast video 	Cisco IPVS
CS6 	110 000 	0x30 	48 	Network control 	EIGRP,OSPF,HSRP,IKE
CS7 	111 000 	0x38 	56
}

     tDSCPID = (dCS0, dCS1, dCS2, dCS3, dCP4, dCS5, dCS6, dCS7);
     tDSCPApplication = (dscpScavenger, dscpQAM, dscpSignaling, dscpRealtime, dscpBroadcastVideo, dscpNetworkControl);


     // Records and structures

     tANSIStringPair = record
                             original    : ANSIstring;
                             replacement : ANSIstring;

                             class operator equal (a, b : tANSIStringPair) : boolean;
                             class operator notequal (a, b : tANSIStringPair) : boolean;

                             procedure initialize; overload;
                             procedure initialize (const configvalue : string); overload;
                             function getLengthDelta : integer;
                             function getPair (const separator : string = '=') : ANSIstring;
                       end;

     tVersionStruct = record
                            version   : string;
                            changelog : string;
                            builddate : tDateTime;
                            status    : tVersionStatus;
                            content   : string;

                            procedure initialize;
                      end;

     tCounterStruct = record
                            CountStart : integer;
                            CountStep  : integer;
                            Digits     : byte;
                            Current    : integer;

                            procedure initialize;
                            procedure increment;
                            procedure reset;
                      end;

     tRenameParams = record
                           FilePattern           : string;
                           Counter               : tCounterStruct;
                           TimestampMode         : tFileTimestampMode;
                           TimestampPatternIndex : integer;

                           procedure initialize;
                     end;

     pEUIBytes = ^tEUIBytes;
     tEUIBytes = array [1..8] of byte;

     pEUIAddress =  ^tEUIAddress;
     tEUIAddress = record
                         public
                               RangeBytes : tEUIBytes;

                               class operator equal (a, b : tEUIAddress) : boolean;
                               class operator notequal (a, b : tEUIAddress) : boolean;

                               function asString (const Separator : string) : string;
                               procedure setfromString (const Address : string; const separator : string);
                   end;

     tIPID   = word;

     tDeltaTickTime = record
                            deltaDay : int64;
                            iHour    : int64;
                            iMin     : int64;
                            iSec     : int64;
                            iMicSec  : int64;

                            procedure initialize;
                      end;

     tFrameTimeStamp = record
                             public
                                   datetime    : tDateTime;
                                   nanoseconds : longint;

                                   procedure initialize;
                                   function asFullString : string;
                                   function asDateTimeString : string;
                                   procedure setFromString (Value : string);
                                   function DictionaryKey : string;

                                   // TODO: operators need to be improved, for example subtract

                                   class operator equal (a, b : tFrameTimeStamp) : boolean;
                                   class operator notequal (a, b : tFrameTimeStamp) : boolean;
                                   class operator greaterthan (a, b : tFrameTimeStamp) : boolean;
                                   class operator lessthan (a, b : tFrameTimeStamp) : boolean;
                                   class operator greaterthanorequal (a, b : tFrameTimeStamp) : boolean;
                                   class operator lessthanorequal (a, b : tFrameTimeStamp) : boolean;
                                   class operator subtract (a, b : tFrameTimeStamp) : tFrameTimeStamp;
                       end;

     tFrameDeltaTime = record
                             Hours        : longword;
                             Minutes      : word;
                             Seconds      : word;
                             Milliseconds : word;
                             Microseconds : word;
                             Nanoseconds  : word;

                             NanoTicks    : int64;

                             procedure initialize;
                             procedure DetermineDelta (const aNow, aThen : tFrameTimeStamp);
                             function toFloat : double;

                             class operator equal (a, b : tFrameDeltaTime) : boolean;
                             class operator notequal (a, b : tFrameDeltaTime) : boolean;
                             class operator greaterthan (a, b : tFrameDeltaTime) : boolean;
                             class operator lessthan (a, b : tFrameDeltaTime) : boolean;
                             class operator greaterthanorequal (a, b : tFrameDeltaTime) : boolean;
                             class operator lessthanorequal (a, b : tFrameDeltaTime) : boolean;
                       end;


     tFrameFlags = packed record                                    // Taken from PCAPng Specs
                                public
                                      Value   : longword;
                                      Present : boolean;

                                      function GetPacketDirection : tPacketDirection;
                                      function GetReceptionType : tPacketReceptionType;
                                      function GetFCSLength : byte;
                                      function GetErrorFlags : tErrorFlagsSet;

                                      procedure SetPacketDirection (Direction : tPacketDirection);
                                      procedure SetErrorFlags (FlagSet : tErrorFlagsSet);
                          end;

     tTimingDetails = record
                            AbsoluteTime  : tDateTime;              // Absolute DateTime, usually of the Trace (Date & Time of the actual Capture)
                            TraceTicks    : int64;                  // Microseconds since Midnight
                            NanoTicks     : word;                   // Additional Nanoseconds (if available)

                            procedure initialize;
                      end;


     tWireDetails  = record
                           LinkType        : longword;
                           WireSize        : longword;
                           CaptureSize     : longword;
                           InterfaceID     : longword;
                           LinkID          : tGUID;
//                         PacketDirection : tPacketDirection;
                           Dropped         : uint64;
                           FrameFlags      : tFrameFlags;
                           FrameType       : tTracefileFrameType;

                           procedure initialize;
                     end;

     tFileFrameLocation = record
                           FileIndex       : integer;               // Index of the capture file
                           FileFrameIndex  : int64;                 // Index of the frame within the capture file
                           FileOffset      : int64;                 // Offset within the file

                           class operator equal (a, b : tFileFrameLocation) : boolean;
                           class operator notequal (a, b : tFileFrameLocation) : boolean;
                           class operator greaterthan (a, b : tFileFrameLocation) : boolean;
                           class operator lessthan (a, b : tFileFrameLocation) : boolean;
                           class operator greaterthanorequal (a, b : tFileFrameLocation) : boolean;
                           class operator lessthanorequal (a, b : tFileFrameLocation) : boolean;

                           procedure initialize; overload;
                           /// <summary>
                           ///   Initializes the record structure with given
                           ///   values
                           /// </summary>
                           /// <param name="Index">
                           ///   FileIndex of the frame
                           /// </param>
                           /// <param name="FrameID">
                           ///   Index of the frame in the file
                           /// </param>
                           /// <param name="Offset">
                           ///   Offset of the frame in the file
                           /// </param>
                           procedure initialize (const Index : integer; const FrameID, Offset : int64); overload;
                           procedure UpdateIfLower (const NewFileIndex, NewFrameIndex : integer; const NewFileOffset : int64); overload;
                           procedure UpdateIfLower (const Location : tFileFrameLocation); overload;
                           procedure UpdateIfHigher (const NewFileIndex, NewFrameIndex : integer; const NewFileOffset : int64); overload;
                           procedure UpdateIfHigher (const Location : tFileFrameLocation); overload;

                           function isZero : boolean;
                           function toString : string;
                           function DictionaryKey : string;
                     end;

     tLocationType = (ltUnknown, ltSameDirectory, ltSubdirectory, ltAbsoluteDirectory);
     tOutputlocation = record
                             Location         : tLocationType;
                             OutputDir        : string;
                             OutputFile       : string;
                             SourceFile       : string;
                             TaskName         : string;
                             TaskNumber       : integer;
                             SectionID        : integer;
                             Count            : integer;
                             TimestampMode    : tFileTimestampMode;
                             OriginalFileTime : tDateTime;

                             procedure initialize (const LocationType : tLocationType; const SourceFilename, OutputDirectory, OutputFilename : string);
                             function getOutputfilename : string;
                       end;

     // Static frame buffer

     tRawBuffer = array [1..maxFrameSize + 1] of byte;
     pRawBuffer = ^tRawBuffer;

     // Payload Word Array for TCP & UDP payload

     {$REGION 'Documentation'}
     ///	<summary>
     ///	  Buffer containing parts of a frame in a word array. This is used for
     ///	  checksum calculation because checksums use this representation.
     ///	</summary>
     {$ENDREGION}
     tBufferWords = array of word;
     tBufferBytes = array of byte;

     tHexRepresentation = record
                                ValueAsString : ANSIString;

                                procedure initialize;
                                function GetHexByteLength : integer;
                                {$REGION 'Documentation'}
                                /// <summary>
                                ///   This method turns the Snort string
                                ///   representation into a "normal" string,
                                ///   with on character per byte
                                /// </summary>
                                {$ENDREGION}
                                function ValueAsBinaryString : ANSIString;
                                function isValid : boolean;
                          end;

     tOffsetList = array of integer;

     // Frame contents - more or less partial

     {$REGION 'Documentation'}
     ///	<summary>
     ///	  Buffer structure that holds parts of a frame. It is used when
     ///	  assembling protocol layers or when retrieving the remaining frame
     ///	  payload that cannot be parsed further
     ///	</summary>
     {$ENDREGION}
     tFrameChunk = record
                         Len        : integer;
                         Data       : pointer;

                         cutmarkers : tOffsetList;

                         class operator equal (a, b : tFrameChunk) : boolean;
                         class operator notequal (a, b : tFrameChunk) : boolean;

                         function getAsBufferWords : tBufferWords;
                         function getAsRawBuffer : tRawBuffer;
                         procedure initialize;
                         function getAsANSIString : ANSIString;
                         function getAsHEXString (const Separator : ANSIstring = '') : ANSIString;
                         function getCRC32 : longword;
                         function getByte (Offset : integer) : byte;
                         function getWord (Offset : integer; const Swapped : boolean = false) : word;
                         function getLongWord (Offset : integer; const Swapped : boolean = false) : longword;
                         /// <summary>
                         ///   Returns the number of cutmarkers available
                         /// </summary>
                         function SectionCount : integer;
                         procedure copyFrom (Chunk : tFrameChunk); overload;
                         procedure copyFrom (const DataPointer : pByte; const BufferLen : integer); overload;
                         procedure copyFrom (RawBuffer : tRawBuffer; BufferLen : integer); overload;
                         procedure copyFrom (const Value : ANSIString); overload;
                         procedure copyFrom (const Value : tHexRepresentation); overload;
                         {$REGION 'Documentation'}
                         /// <summary>
                         ///   Patches the byte specified by the Value
                         ///   parameter into the framechunk memory
                         /// </summary>
                         /// <param name="Offset">
                         ///   Offset at which to patch
                         /// </param>
                         /// <param name="Value">
                         ///   Value to patch into the framechunk
                         /// </param>
                         {$ENDREGION}
                         procedure Patch (const Offset : integer; const Value : byte); overload;
                         {$REGION 'Documentation'}
                         /// <summary>
                         ///   Patches the bytes specified by the Value
                         ///   parameter into the framechunk memory after byte
                         ///   swapping them
                         /// </summary>
                         /// <param name="Offset">
                         ///   Offset at which to patch
                         /// </param>
                         /// <param name="Value">
                         ///   Value to patch into the framechunk
                         /// </param>
                         {$ENDREGION}
                         procedure Patch (const Offset : integer; const Value : word; const DoNotSwap : boolean = false); overload;
                         {$REGION 'Documentation'}
                         /// <summary>
                         ///   Patches the bytes specified by the Value
                         ///   parameter into the framechunk memory after byte
                         ///   swapping them
                         /// </summary>
                         /// <param name="Offset">
                         ///   Offset at which to patch
                         /// </param>
                         /// <param name="Value">
                         ///   Value to patch into the framechunk
                         /// </param>
                         {$ENDREGION}
                         procedure Patch (const Offset : integer; const Value : longword; const DoNotSwap : boolean = false); overload;
                         {$REGION 'Documentation'}
                         /// <summary>
                         ///   Patches the bytes specified by the Value
                         ///   parameter into the framechunk memory after byte
                         ///   swapping them
                         /// </summary>
                         /// <param name="Offset">
                         ///   Offset at which to patch
                         /// </param>
                         /// <param name="Value">
                         ///   Value to patch into the framechunk
                         /// </param>
                         {$ENDREGION}
                         procedure Patch (const Offset : integer; const Value : uint64; const DoNotSwap : boolean = false); overload;
                         {$REGION 'Documentation'}
                         /// <summary>
                         ///   Patches the string specified by the Value
                         ///   parameter into the framechunk memory. Each
                         ///   character is converted to byte and then patched
                         /// </summary>
                         /// <param name="Offset">
                         ///   Offset at which to patch
                         /// </param>
                         /// <param name="Value">
                         ///   Value to patch into the framechunk
                         /// </param>
                         {$ENDREGION}
                         procedure Patch (const Offset : integer; const Value : string); overload;
                         procedure CreateZeroBuffer (BufferLen : integer);
                         procedure CreateFromPattern (Pattern : ansistring; BufferLen : integer);
                         procedure CreateFromLongword (const Value : LongWord);
                         /// <summary>
                         ///   This is used to add a CRC32 checksum to a chunk.
                         ///   Usually happens when an Ethernet frame had a FCS
                         ///   and the new one should have one, too
                         /// </summary>
                         /// <param name="SetBad">
                         ///   Forces if the CRC is set to an invalid value,
                         ///   e.g. when the original FCS was bad, too
                         /// </param>
                         procedure AttachCRC32 (const SetBad : boolean = False); overload;
                         procedure AttachCRC32 (const CRC32 : longword); overload;
                         procedure writeToDisk (Filename : string);
                         procedure clearCutMarkers;
                         procedure free;
                   end;

     // Interface ID

     tInterfaceID = longword;

     tGeneralTraceStats = record
                                FirstFrameTime    : tFrameTimeStamp;
                                FirstFrameOffset  : uint64;
                                LastFrameTime     : tFrameTimeStamp;
                                LastFrameOffset   : uint64;
                                MinFrameSize      : integer;
                                MaxFrameSize      : integer;
                                AverageFrameSize  : integer;
                                FrameCount        : integer;
                                TotalFrameBytes   : int64;
                                TotalWireBytes    : int64;
                                isSliced          : boolean;
                                SliceSizeConstant : boolean;
                                SliceSize         : integer;
                                TimeOrderCorrect  : boolean;
                                OutOfTimeFrames   : integer;

                                procedure initialize;
                          end;

     tMD5HashString = string[32];

     tTCPColorTable = array [0..11] of tColor;

     tConversationDialogSettings = record
                                         TCPColors           : tTCPColorTable;
                                         InitialPage         : integer;
                                         RememberPage        : boolean;
                                         ClearFilterOnChange : boolean;
                                   end;
     tICMPv4TypeCode = record
                             ICMPv4Type        : integer;
                             ICMPv4Code        : integer;

                             class operator equal (a, b : tICMPv4TypeCode) : boolean;
                             class operator notequal (a, b : tICMPv4TypeCode) : boolean;

                             procedure initialize; overload;
                             procedure initialize (const NewType, NewCode : integer); overload;
                             procedure initialize (const value : string); overload;
                             function toString (const separator : string = ':') : string;
                       end;

     tICMPv4TreeEntry = record
                              TypeCode          : tICMPv4TypeCode;
                              Description       : string;
                              ICMPShowType      : boolean;
                              ICMPShowCode      : boolean;
                        end;
     pICMPv4TreeEntry = ^tICMPv4TreeEntry;

     tTTLAnchorPair = record
                            SourceTTL      : byte;
                            DestinationTTL : byte;

                            class operator equal (a, b : tTTLAnchorPair) : boolean;
                            class operator notequal (a, b : tTTLAnchorPair) : boolean;
                            procedure initialize;
                      end;

     tMMAInt64   = record
                         Min       : int64;
                         Max       : int64;
                         Total     : int64;
                         NumValues : integer;

                         procedure initialize (const initialvalue : int64);
                         procedure addValue (const newvalue : int64);
                         function average : int64;
                   end;


const VALUE_ZERO_EUI       : tEUIAddress = (rangebytes : (0,0,0,0,0,0,0,0));

      // Complexitiy settings - used to help find faster filter matches (determined via parsemaster and stored on conversation scan)

      ComplexityUnknown   =  0;  // no information
      ComplexityL234      =  1;  // all L4 frames in file are Ethernet - IPv4/6 - TCP/UDP
      ComplexityL2Q34     =  2;  // all L4 frames in file are Ethernet - [VLAN] - IPv4/6 - TCP/UDP
      ComplexityHigh      = 10;  // all l4 frames in file have tunneled layers, IP over IP, etc

      // L4 protocol numbers

      protICMP = 1;
      protTCP = 6;
      protUDP = 17;

      function Get32BitAlignment (UnAlignedLength: integer) : integer; inline;

      {$REGION 'Documentation'}
      /// <summary>
      ///   Returns a link layer type index for a protocol type parser index.
      ///   This helps when stripping headers like the Juniper firewall header.
      /// </summary>
      /// <param name="ProtocolIndex">
      ///   Index of the parser protocol the link layer type is needed for
      /// </param>
      /// <returns>
      ///   Link layer value
      /// </returns>
      {$ENDREGION}
      function GetLinkTypeForProtocolIndex (const ProtocolIndex : integer) : integer;

      procedure InitializeRawBuffer (var buffer : tRawBuffer; const len : integer = maxFrameSize);
      function GetRawBufferAsWordBuffer (var buffer : tRawBuffer; const len : integer = maxFrameSize) : tBufferWords;

      /// <summary>
      ///   Returns a dynamic array of word values (as type tBufferWords) from
      ///   a memory location for a given offset and length. The array will be
      ///   sized accordingly. This is used in preparation of calculating
      ///   checksums, so the words are swapped for the array automatically.
      ///   Odd numbers of bytes will be padded according to RFC 786
      /// </summary>
      /// <param name="Buffer">
      ///   byte pointer to the memory range
      /// </param>
      /// <param name="Offset">
      ///   offset within the memory range
      /// </param>
      /// <param name="Len">
      ///   length of bytes to copy to word values
      /// </param>
      function getWordsStructFromOffset (Buffer: pByte; Offset, Len: integer): tBufferWords;

      {$REGION 'Documentation'}
      ///	<summary>
      ///	  Merges two chunks with each other by concatenating Chunk1 and
      ///	  Chunk2, creating a new combined chunk
      ///	</summary>
      ///	<param name="Chunk1">
      ///	  First chunk, will appear in front
      ///	</param>
      ///	<param name="Chunk2">
      ///	  Second chunk, will appear at the end
      ///	</param>
      ///	<returns>
      ///	  Merged frame chunks, looking like this: Chunk1Chunk2
      ///	</returns>
      ///	<remarks>
      ///	  Careful: mergeFrameChunks will not discard any of the input chunks,
      ///	  you have to do it yourself! Also note that you need to take care of
      ///	  the discard process of the merged chunk.
      ///	</remarks>
      {$ENDREGION}
      function mergeFrameChunks (Chunk1, Chunk2 : tFrameChunk) : tFrameChunk;
      /// <summary>
      ///   Returns the section delimited by the cutmakers.
      /// </summary>
      /// <param name="Chunk">
      ///   Framechunk to extract the section from
      /// </param>
      /// <param name="index">
      ///   section to be returned
      /// </param>
      /// <returns>
      ///   FrameChunk containing the section delimited by the relevant
      ///   cutmarkers
      /// </returns>
      /// <remarks>
      ///   Section 1 starts at offset 0 (implicit maker) and ends at cutmarker
      ///   1. Section 2 starts at cutmarker 1 and ends at cutmarker 2, etc.
      /// </remarks>
      function GetSectionFromFrameChunk (Chunk : tFrameChunk; const index : integer) : tFrameChunk;

      function CompareValue (value1, value2 : tFrameTimeStamp) : integer; overload;
      function CaptureFileTypeAsText (value : tCaptureFileType) : string;
      function getDiagnosticTextFromCode (MessageCode : integer) : string;
      function getStatusDescription (Status : tProcessingStatus) : string;
      function getLinkTypeName (LinkType : integer) : string;
      function isValidMAC (MACAddress : string) : boolean;

      function getBufferWordsAsHexString (Buffer : tBufferWords) : string;

      function CaptureDurationAsString (ANow, AThen : tFrameTimestamp) : string;
      function NanosecondsBetween (ANow, AThen : tFrameTimestamp) : int64;
      function MicrosecondsBetween (ANow, AThen : tFrameTimeStamp) : int64;
      function MillisecondsBetweenFrames (ANow, AThen : tFrameTimeStamp) : Int64;

      /// <summary>
      ///   Turns a NTP 64bit timestamp value into a UTC FrameTimeStamp value
      /// </summary>
      function NTPTimeStampToFrameTime (const value : uint64) : tFrameTimeStamp;

      procedure PatchMem (const BasePtr : pByte; const Offset : integer; const Value : byte); overload;
      procedure PatchMem (const BasePtr : pByte; const Offset : integer; const Value : word); overload;
      procedure PatchMem (const BasePtr : pByte; const Offset : integer; const Value : longword); overload;
      procedure PatchMem (const BasePtr : pByte; const Offset : integer; const Value : uint64); overload;
      procedure PatchMem (const BasePtr : pByte; const Offset : integer; const Value : string); overload;

      function Layer4ProtocolToStr (const Protocol : tLayer4Protocol) : string;
      function StrToLayer4Protocol (const ProtocolString : string) : tLayer4Protocol;
      function Layer4ProtocolToByte (const Protocol : tLayer4Protocol; const IPVersion : byte = 4) : byte;
      function ByteToLayer4Protocol (const ProtocolByte : byte) : tLayer4Protocol;

      function getCRC32FromPointer (const DataPtr : Pointer; Len : integer) : longword;

      procedure sanizizeByteArray (var buffer : array of byte; mode : tArraySanitizationMode);

//    function QueryTypesToLongword (TypeSet : tDNSRecordTypeSet) : longword;
//    function LongwordToQueryTypes (value : longword) : tDNSRecordTypeSet;

      function QueryTypesToInt64 (const TypeSet : tDNSRecordTypeSet) : uint64;
      function Int64ToQueryTypes (const value : uint64) : tDNSRecordTypeSet;
      function DNSReplyCodeFromWord (const ReplyCode : word) : tDNSReturnCode;
      function DNSReplyCodeToWord (const value : tDNSReturnCode) : word;
      function QueryTypeToString (const QueryType : tDNSRecordType) : string;
      function QueryTypeSetToString (const TypeSet : tDNSRecordTypeSet) : string;

      /// <summary>
      ///   Calculates an estimation of hops the packet has travelled
      /// </summary>
      function getHopDistanceEstimate (const TTL : byte) : byte;

      function TunnelFlagsToWord (TunnelFlags : tTunnelFlagSet) : word;
      function WordToTunnelFlags (value : word) : tTunnelFlagSet;

      /// <summary>
      ///   Checks if a string is a valid Stringpair in the format of
      ///   "value1"="value2".
      /// </summary>
      /// <param name="value">
      ///   String to check
      /// </param>
      /// <param name="CheckEmpty">
      ///   if true, value1 must not be empty to be valid
      /// </param>
      function isValidANSIStringPair (value : string; const CheckEmpty : boolean) : boolean;
      function isValidVLANIDPair (value : string) : boolean;
      function isValidICMPv4TypeCodePair (value : string) : boolean;
      function isValidPortPair (value : string) : boolean;

// Global Variables

var ZERO_GUID               : tGUID;
    TWWindowLabel           : string;
    LocalTraceIntelFilename : string;


implementation

uses SiAuto,
//   System.RegularExpressionsCore,
     RegularExpressions,
     SysUtils,
     StrUtils,
     Types,
     Math,
     Windows,
     DateUtils,
     Toolkit;

const CRCTable : array[0..255] of Longword = ($00000000, $77073096, $EE0E612C, $990951BA,
                                              $076DC419, $706AF48F, $E963A535, $9E6495A3,
                                              $0EDB8832, $79DCB8A4, $E0D5E91E, $97D2D988,
                                              $09B64C2B, $7EB17CBD, $E7B82D07, $90BF1D91,
                                              $1DB71064, $6AB020F2, $F3B97148, $84BE41DE,
                                              $1ADAD47D, $6DDDE4EB, $F4D4B551, $83D385C7,
                                              $136C9856, $646BA8C0, $FD62F97A, $8A65C9EC,
                                              $14015C4F, $63066CD9, $FA0F3D63, $8D080DF5,
                                              $3B6E20C8, $4C69105E, $D56041E4, $A2677172,
                                              $3C03E4D1, $4B04D447, $D20D85FD, $A50AB56B,
                                              $35B5A8FA, $42B2986C, $DBBBC9D6, $ACBCF940,
                                              $32D86CE3, $45DF5C75, $DCD60DCF, $ABD13D59,
                                              $26D930AC, $51DE003A, $C8D75180, $BFD06116,
                                              $21B4F4B5, $56B3C423, $CFBA9599, $B8BDA50F,
                                              $2802B89E, $5F058808, $C60CD9B2, $B10BE924,
                                              $2F6F7C87, $58684C11, $C1611DAB, $B6662D3D,

                                              $76DC4190, $01DB7106, $98D220BC, $EFD5102A,
                                              $71B18589, $06B6B51F, $9FBFE4A5, $E8B8D433,
                                              $7807C9A2, $0F00F934, $9609A88E, $E10E9818,
                                              $7F6A0DBB, $086D3D2D, $91646C97, $E6635C01,
                                              $6B6B51F4, $1C6C6162, $856530D8, $F262004E,
                                              $6C0695ED, $1B01A57B, $8208F4C1, $F50FC457,
                                              $65B0D9C6, $12B7E950, $8BBEB8EA, $FCB9887C,
                                              $62DD1DDF, $15DA2D49, $8CD37CF3, $FBD44C65,
                                              $4DB26158, $3AB551CE, $A3BC0074, $D4BB30E2,
                                              $4ADFA541, $3DD895D7, $A4D1C46D, $D3D6F4FB,
                                              $4369E96A, $346ED9FC, $AD678846, $DA60B8D0,
                                              $44042D73, $33031DE5, $AA0A4C5F, $DD0D7CC9,
                                              $5005713C, $270241AA, $BE0B1010, $C90C2086,
                                              $5768B525, $206F85B3, $B966D409, $CE61E49F,
                                              $5EDEF90E, $29D9C998, $B0D09822, $C7D7A8B4,
                                              $59B33D17, $2EB40D81, $B7BD5C3B, $C0BA6CAD,

                                              $EDB88320, $9ABFB3B6, $03B6E20C, $74B1D29A,
                                              $EAD54739, $9DD277AF, $04DB2615, $73DC1683,
                                              $E3630B12, $94643B84, $0D6D6A3E, $7A6A5AA8,
                                              $E40ECF0B, $9309FF9D, $0A00AE27, $7D079EB1,
                                              $F00F9344, $8708A3D2, $1E01F268, $6906C2FE,
                                              $F762575D, $806567CB, $196C3671, $6E6B06E7,
                                              $FED41B76, $89D32BE0, $10DA7A5A, $67DD4ACC,
                                              $F9B9DF6F, $8EBEEFF9, $17B7BE43, $60B08ED5,
                                              $D6D6A3E8, $A1D1937E, $38D8C2C4, $4FDFF252,
                                              $D1BB67F1, $A6BC5767, $3FB506DD, $48B2364B,
                                              $D80D2BDA, $AF0A1B4C, $36034AF6, $41047A60,
                                              $DF60EFC3, $A867DF55, $316E8EEF, $4669BE79,
                                              $CB61B38C, $BC66831A, $256FD2A0, $5268E236,
                                              $CC0C7795, $BB0B4703, $220216B9, $5505262F,
                                              $C5BA3BBE, $B2BD0B28, $2BB45A92, $5CB36A04,
                                              $C2D7FFA7, $B5D0CF31, $2CD99E8B, $5BDEAE1D,

                                              $9B64C2B0, $EC63F226, $756AA39C, $026D930A,
                                              $9C0906A9, $EB0E363F, $72076785, $05005713,
                                              $95BF4A82, $E2B87A14, $7BB12BAE, $0CB61B38,
                                              $92D28E9B, $E5D5BE0D, $7CDCEFB7, $0BDBDF21,
                                              $86D3D2D4, $F1D4E242, $68DDB3F8, $1FDA836E,
                                              $81BE16CD, $F6B9265B, $6FB077E1, $18B74777,
                                              $88085AE6, $FF0F6A70, $66063BCA, $11010B5C,
                                              $8F659EFF, $F862AE69, $616BFFD3, $166CCF45,
                                              $A00AE278, $D70DD2EE, $4E048354, $3903B3C2,
                                              $A7672661, $D06016F7, $4969474D, $3E6E77DB,
                                              $AED16A4A, $D9D65ADC, $40DF0B66, $37D83BF0,
                                              $A9BCAE53, $DEBB9EC5, $47B2CF7F, $30B5FFE9,
                                              $BDBDF21C, $CABAC28A, $53B39330, $24B4A3A6,
                                              $BAD03605, $CDD70693, $54DE5729, $23D967BF,
                                              $B3667A2E, $C4614AB8, $5D681B02, $2A6F2B94,
                                              $B40BBE37, $C30C8EA1, $5A05DF1B, $2D02EF8D);



(* ----------------------------------- *)

function Get32BitAlignment (UnAlignedLength: integer) : integer; inline;

begin
     if (UnAlignedLength mod 4) = 0 then
        result := UnAlignedLength
     else
         result := (4 - (UnAlignedLength MOD 4)) + UnAlignedLength;
end;

(* ----------------------------------- *)

function GetLinkTypeForProtocolIndex (const ProtocolIndex : integer) : integer;

begin
     case ProtocolIndex of
      NEXT_HEADER_ETHERNET : result := LINKTYPE_ETHERNET;
      NEXT_HEADER_IPv4     : result := LINKTYPE_IPV4;
     else
         result := LINKTYPE_NULL;
     end
end;

(* ----------------------------------- *)

function CaptureFileTypeAsText (value : tCaptureFileType) : string;

begin
     case value of
      cfNone    : result := 'None';
      cfUnknown : result := 'Unknown';
      cfENC     : result := 'ENC';
      cfCAP     : result := 'NAI CAP';
      cfPCAP    : result := 'PCAP';
      cfPCAPng  : result := 'PCAPng';
     end;
end;

(* ----------------------------------- *)

function getDiagnosticTextFromCode (MessageCode : integer) : string;

begin
     result := 'Unknown Message Code #' + IntToStr (MessageCode);

end;

(* ----------------------------------- *)

procedure InitializeRawBuffer (var buffer : tRawBuffer; const len : integer = maxFrameSize); inline;

var i : integer;

begin
{
     for i := 1 to len do
         buffer[i] := 0;
}
     ZeroMemory(@buffer[1],length(buffer));
end;

(* ------------------------------------------- *)

function GetRawBufferAsWordBuffer (var buffer : tRawBuffer; const len : integer = maxFrameSize) : tBufferWords;

var buf   : tBufferWords;
    index : integer;
    size  : integer;
    i     : integer;

begin
     if odd (len) then
        size := (len + 1) div 2
     else
         size := len div 2;

     setlength (buf, size);

     for i := 1 to len do
         begin
              index := (i - 1) div 2;

              if odd (i) then
                 buf[index] := buffer[i] shl 8
              else
                  buf[index] := buf[index] or buffer[i];
         end;

     result := buf;
end;

(* ------------------------------------------- *)

function getWordsStructFromOffset (Buffer : pByte; Offset, Len : integer): tBufferWords;

var iCount : integer;
    i      : integer;
    pHelp  : pWord;
    pHelp2 : pByte;
    wHelp  : word;
    OddLen : boolean;

begin
     // Prepare the section words

     iCount := Len div 2;

     if (Len mod 2) > 0 then
        begin
             inc (iCount);
             OddLen := true;
        end
     else
         OddLen := false;

     setLength (result, iCount);

     for i := 0 to iCount - 1 do
         begin
              if (i < (iCount -1)) or not oddlen then
                 begin
                      phelp := pWord (Buffer + Offset + (2*i));

                      result[i] := phelp^;
                      result[i] := Swap16 (result[i]);
                 end
              else
                  begin
                       // rfc 786: "If a segment contains an odd number of header and text octets
                       // to be checksummed, the last octet is padded on the right with zeros to
                       // form a 16 bit word for checksum purposes.

                       pHelp2 := pByte (Buffer + Offset + (2*i));
                       wHelp := pHelp2^;

                       result[i] := swap16 (wHelp);
                  end;
         end;
end;

(* ----------------------------------- *)

function mergeFrameChunks (Chunk1, Chunk2 : tFrameChunk) : tFrameChunk;

var data       : pByte;
    help       : PByte;
    size       : integer;
    cutmarkers : tOffsetList;
    len        : integer;
    i          : integer;

begin
     setLength (cutmarkers, 0);

     if (Chunk1.Len = 0) and (Chunk2.Len = 0) then
        begin
             result.Data := nil;
             result.Len := 0;

             SiMain.LogFatal ('Chunk1 and Chunk2 both empty on mergeFrameChunks, aborting merge');

             Exit;
        end;

     if Chunk1.Len = 0 then
        begin
             size := Chunk2.Len;

             // Reserve memory for second part

             getmem (data, Chunk2.Len);

             move (Chunk2.Data^, data^, Chunk2.Len);
        end
     else
         if Chunk2.Len = 0 then
            begin
                 size := Chunk1.Len;

                 // Reserve memory for first part

                 getmem (data, Chunk1.Len);

                 move (Chunk1.Data^, data^, Chunk1.Len);
            end
         else
             begin
                  // Determine length of both parts

                  size := Chunk1.Len + Chunk2.Len;

                  // Reserve memory for both parts

                  getmem (data, size);

                  // Position help at the offset where the second part will be copied to

                  help := PByte (data + Chunk1.Len);

                  // copy data

                  move (Chunk1.Data^, data^, Chunk1.Len);
                  move (Chunk2.Data^, help^, Chunk2.Len);

                  // Update cutmakers

                  len := length (Chunk1.cutmarkers);

                  // transfer existing Chunk1 markers first

                  if len > 0 then
                     cutmarkers := Chunk1.cutmarkers;

                  // prepare for at least one new marker and all from Chunk2

                  setLength (cutmarkers, len + length (Chunk2.cutmarkers) + 1);

                  // add new marker where Chunk2 starts

                  cutmarkers[len] := Chunk1.Len;

                  // Add existing cutmarkers from chunk 2

                  for i := 1 to length (Chunk2.cutmarkers) do
                      cutmarkers[len + i] := Chunk2.cutmarkers[i-1] + Chunk1.len;
             end;

     result.Data := data;
     result.Len := size;
     result.cutmarkers := cutmarkers;
end;

(* ----------------------------------- *)

function GetSectionFromFrameChunk (Chunk : tFrameChunk; const index : integer) : tFrameChunk;

var offsetLow  : integer;
    offsetHigh : integer;
    startpos   : PByte;
    count      : integer;
    data       : pByte;
    size       : integer;
    i          : integer;
    pFrom      : PByte;
    pTo        : PByte;

begin
     count := Chunk.SectionCount;

     assert ((index > 0) and (index <= count), 'Invalid section index ' + IntToStr (index) + ' specified (tFrameChunk.GetSection)');

     // Determine the cut markers we need

     if index = 1 then
        begin
             offsetLow := 0;
             offsetHigh := Chunk.cutmarkers[0];
        end
     else
         if index = count then
            begin
                 offsetLow := Chunk.cutmarkers[count-2];
                 offsetHigh := Chunk.Len;
            end
         else
             begin
                  offsetLow := Chunk.cutmarkers[index-2];
                  offsetHigh := Chunk.cutmarkers[index-1];
             end;

     // Determine length of the section

     size := offsetHigh - offsetLow;

     // Reserve memory

     getmem (data, size);

     // Copy the data

     pFrom := PByte (Chunk.Data) + offsetLow;
     pTo := data;

     for i := 1 to size do
         begin
              pTo^ := pFrom^;

              inc (pTo);
              inc (pFrom);
         end;

     // Return the chunk

     result.Data := data;
     result.Len := size;
     setLength (result.cutmarkers, 0);
end;

(* ----------------------------------- *)

function getBufferWordsAsHexString (Buffer : tBufferWords) : string;

var value : string;
    help  : string;
    i     : integer;

begin
     value := '';

     for i := 1 to length (Buffer) do
         begin
              help := IntToHex (Buffer[i-1], 4);
              insert (' ', help, 3);

              if i < length (Buffer) then
                 value := value + help + ' '
              else
                  value := value + help;
         end;

     result := value;
end;

(* ----------------------------------- *)

function NanosecondsBetween (ANow, AThen : tFrameTimestamp) : int64;

var ans   : int64;
    bns   : int64;
    secs  : int64;
    nsecs : int64;

begin
     secs := secondsBetween (ANow.datetime, AThen.datetime);

     nsecs := ANow.nanoseconds - AThen.nanoseconds;
     nsecs := nsecs + (1000000000 * secs);

     result := nsecs;
end;

(* ----------------------------------- *)

function MicrosecondsBetween (ANow, AThen : tFrameTimestamp) : int64;

var nsecs : int64;

begin
     nsecs := NanosecondsBetween (ANow, AThen);

     result := nsecs DIV 1000;
end;

(* ----------------------------------- *)

// Can't seem to overload MillisecondsBetween...

function MillisecondsBetweenFrames (ANow, AThen : tFrameTimestamp) : Int64;

var nsecs : int64;

begin
     nsecs := NanosecondsBetween (ANow, AThen);

     result := nsecs DIV 1000000;
end;

(* ----------------------------------- *)

function NTPTimeStampToFrameTime (const value : uint64) : tFrameTimeStamp;

var frametime : tFrameTimeStamp;
    help      : uint64;
    dhelp     : double;

begin
     frametime.datetime := encodeDateTime (1900, 1, 1, 0, 0, 0, 0);

     // seconds since 1900

     help := value and $ffffffff00000000 shr 32;
     frametime.datetime := IncSecond (frametime.datetime, help);

     // Second fractions

     help := value and $ffffffff;
     dhelp := help / 4294967296.0;
     frametime.nanoseconds := trunc (dHelp * 1000000000);

     result := frametime;
end;

(* ----------------------------------- *)

procedure PatchMem (const BasePtr : pByte; const Offset : integer; const Value : byte);

var help : pByte;

begin
     help := pByte (BasePtr + Offset);
     help^ := Value;
end;

(* ----------------------------------- *)

procedure PatchMem (const BasePtr : pByte; const Offset : integer; const Value : word);

var help : pWord;

begin
     help := pWord (BasePtr + Offset);
     help^ := Value;
end;

(* ----------------------------------- *)

procedure PatchMem (const BasePtr : pByte; const Offset : integer; const Value : longword);

var help : PLongword;

begin
     help := pLongword (BasePtr + Offset);
     help^ := Value;
end;

(* ----------------------------------- *)

procedure PatchMem (const BasePtr : pByte; const Offset : integer; const Value : uint64);

var help : pInt64;

begin
     help := pInt64 (BasePtr + Offset);
     help^ := Value;
end;

(* ----------------------------------- *)

procedure PatchMem (const BasePtr : pByte; const Offset : integer; const Value : string);

var help : pByte;
    i    : integer;

begin
      for i := 0 to length (Value) - 1 do
         begin
              help := (BasePtr + Offset + i);

              help^ := ord (value[i+1]);
         end;
end;

(* ----------------------------------- *)

function CaptureDurationAsString (ANow, AThen : tFrameTimestamp) : string;

var totalsecs : integer;
    hours     : integer;
    minutes   : integer;
    seconds   : integer;
    nanosecs  : integer;

begin
     totalsecs := SecondsBetween (ANow.datetime, AThen.datetime);
     nanosecs := ANow.nanoseconds - AThen.nanoseconds;

     if nanosecs < 0 then
        begin
             dec (totalsecs);

             nanosecs := 1000000000 + nanosecs;
        end;

     seconds := totalsecs mod 60;
     minutes := totalsecs div 60;
     hours := totalsecs div 3600;

     minutes := minutes - (hours * 60);

     if hours < 100 then
        result := Format('%2.2d:%2.2d:%2.2d.%2.9d',[hours, minutes, seconds, nanosecs])
     else
         result := Format('%2.3d:%2.2d:%2.2d.%2.9d',[hours, minutes, seconds, nanosecs])
end;

(* ----------------------------------- *)

procedure sanizizeByteArray (var buffer : array of byte; mode : tArraySanitizationMode);

var len : integer;
    i   : integer;

begin
     len := length (buffer);

     case mode of
      asZero      : begin
                         for i := 1 to len do
                             buffer[i-1] := 0;
                    end;
      asRandomize : begin
                         for i := 1 to len do
                             buffer[i-1] := random (255);
                    end;
     end;
end;

(* ----------------------------------- *)
{ tANSIStringPair }
(* ----------------------------------- *)

procedure tANSIStringPair.initialize;

begin
     original := '';
     replacement := '';
end;

(* ----------------------------------- *)

procedure tANSIStringPair.initialize (const configvalue : string);

begin
     original := AnsiDeQuotedStr (getEntry (configvalue, '=', 1), '"');
     replacement := AnsiDeQuotedStr (getEntry (configvalue, '=', 2), '"');
end;

(* ----------------------------------- *)

function tANSIStringPair.getLengthDelta: integer;

var delta : integer;
    olen  : integer;
    rlen  : integer;

begin
     // Check for Snort formatted hex representations

     if pos ('|', original) > 0 then
        olen := getSnortHexLen (original)
     else
         olen := length (original);

     if pos ('|', replacement) > 0 then
        rlen := getSnortHexLen (replacement)
     else
         rlen := length (replacement);

     // Determine length difference

     delta := olen - rlen;

     result := delta;
end;

(* ----------------------------------- *)

function tANSIStringPair.getPair (const separator : string = '='): ANSIstring;

begin
     result := AnsiQuotedStr (original, '"') + separator + AnsiQuotedStr (replacement, '"');
end;

(* ----------------------------------- *)

class operator tANSIStringPair.equal (a, b: tANSIStringPair): boolean;

begin
     result := (a.original = b.original) and (a.replacement = b.replacement);
end;

(* ----------------------------------- *)

class operator tANSIStringPair.notequal (a, b: tANSIStringPair): boolean;

begin
     result := not (a=b);
end;

(* ----------------------------------- *)
{ tCounterStruct }
(* ----------------------------------- *)

procedure tCounterStruct.initialize;

begin
     CountStart := 1;
     CountStep := 1;
     Digits := 1;
     Current := 1;
end;

(* ----------------------------------- *)

procedure tCounterStruct.increment;

begin
     Current := Current + CountStep;
end;

(* ----------------------------------- *)

procedure tCounterStruct.reset;

begin
     Current := CountStart;
end;

(* ----------------------------------- *)
{ tRenameParams }
(* ----------------------------------- *)

procedure tRenameParams.initialize;

begin
     FilePattern := '';
     Counter.initialize;
     TimestampMode := tsmNone;
     TimestampPatternIndex := 0;
end;

(* ----------------------------------- *)
{ tFrameTimeStamp }
(* ----------------------------------- *)

procedure tFrameTimeStamp.initialize;

begin
     datetime := 0;
     nanoseconds := 0;
end;

(* ----------------------------------- *)

function tFrameTimeStamp.asDateTimeString: string;

begin
     result := DateTimeToStr (datetime);
end;

(* ----------------------------------- *)

function tFrameTimeStamp.asFullString : string;

begin
     result := DateTimeToStr (datetime) + '.' + Format ('%.9d', [nanoseconds]);
end;

(* ----------------------------------- *)

function tFrameTimeStamp.DictionaryKey: string;

var value : string;

begin
     value := asFullString;

     value := stringReplace (value, ':', '', [rfReplaceAll]);
     value := stringReplace (value, ' ', '', [rfReplaceAll]);

     result := value;
end;

(* ----------------------------------- *)

procedure tFrameTimeStamp.setFromString (Value: string);

var datepart : string;
    timepart : string;
    nanopart : string;
    index    : integer;

begin
     index := pos (' ', value);

     if index > 0 then
        begin
             datepart := copy (value, 1, index - 1);
             timepart := copy (value, index + 1, length (Value) - index);
             nanopart := rightstring (timepart, '.');
             timepart := leftstring (timepart, '.');

             datetime := StrToDateTime (datepart + ' ' + timepart);
             tryStrToInt (nanopart, nanoseconds);
        end;
end;

(* ----------------------------------- *)

function CompareValue (value1, value2 : tFrameTimeStamp) : integer;

var compareresult : integer;

begin
     if value1.datetime > value2.datetime then
        compareresult := GreaterThanValue
     else
         if value1.datetime < value2.datetime then
            compareresult := LessThanValue
         else
             compareresult := CompareValue (value1.nanoseconds, value2.nanoseconds);

     result := compareresult;
end;

(* ----------------------------------- *)

class operator tFrameTimeStamp.equal (a, b: tFrameTimeStamp): boolean;

begin
     result := (a.datetime = b.datetime) and (a.nanoseconds = b.nanoseconds);
end;

(* ----------------------------------- *)

class operator tFrameTimeStamp.notequal(a, b: tFrameTimeStamp): boolean;

begin
     result := not (a=b);
end;

(* ----------------------------------- *)

class operator tFrameTimeStamp.GreaterThan(a, b: tFrameTimeStamp): boolean;

begin
     result := (a.datetime > b.datetime) or ((a.datetime = b.datetime) and (a.nanoseconds > b.nanoseconds));
end;

(* ----------------------------------- *)

class operator tFrameTimeStamp.GreaterThanOrEqual(a, b: tFrameTimeStamp): boolean;

begin
     result := (a=b) or (a > b);
end;

(* ----------------------------------- *)

class operator tFrameTimeStamp.LessThan(a, b: tFrameTimeStamp): boolean;

begin
     result := not (a >= b);
end;

(* ----------------------------------- *)

class operator tFrameTimeStamp.LessThanOrEqual(a, b: tFrameTimeStamp): boolean;

begin
     result := not (a > b);
end;

(* ----------------------------------- *)

class operator tFrameTimeStamp.subtract(a, b: tFrameTimeStamp): tFrameTimeStamp;

var help : tFrameTimeStamp;

begin
     help.datetime := (a.datetime - b.datetime);
     help.nanoseconds := (a.nanoseconds - b.nanoseconds);

     result := help;
end;

(* ----------------------------------- *)
{ tFrameDeltaTime }
(* ----------------------------------- *)

procedure tFrameDeltaTime.initialize;

begin
     Hours := 0;
     Minutes := 0;
     Seconds := 0;
     Milliseconds := 0;
     Microseconds := 0;
     Nanoseconds := 0;

     NanoTicks := 0;
end;

(* ----------------------------------- *)

procedure tFrameDeltaTime.DetermineDelta (const aNow, aThen: tFrameTimeStamp);

var iSecs  : int64;
    iNano  : int64;
    iMicro : int64;
    iMilli : int64;

begin
     NanoTicks := nanosecondsbetween (aNow, AThen);

     // Determine the other values from NanoTicks

     iSecs := NanoTicks DIV 1000000000;
     iNano := NanoTicks - (1000000000 * iSecs);
     NanoSeconds := iNano MOD 1000;

     iMicro := iNano DIV 1000;

     MicroSeconds := iMicro MOD 1000;
     MilliSeconds := iMicro DIV 1000;

     Seconds := iSecs;
     Minutes := Seconds DIV 60;
     Seconds := Seconds - (Minutes * 60);

     Hours := Minutes DIV 60;
     Minutes := Minutes - (Hours * 60);
 end;

(* ----------------------------------- *)

function tFrameDeltaTime.toFloat: double;

var h,m   : integer;
    iSecs : int64;
    iNano : int64;

begin
     h := hours;
     m := minutes;

     iSecs := (((h * 60) + m) * 60) + Seconds;
     iNano := NanoTicks - (iSecs * 1000000000);

     result := iSecs + (iNano / 1000000000);
end;

(* ----------------------------------- *)

class operator tFrameDeltaTime.equal (a, b: tFrameDeltaTime): boolean;

begin
     result := a.Nanoticks = b.NanoTicks;
end;

(* ----------------------------------- *)

class operator tFrameDeltaTime.greaterthan (a, b: tFrameDeltaTime): boolean;

begin
     result := a.NanoTicks > b.NanoTicks;
end;

(* ----------------------------------- *)

class operator tFrameDeltaTime.greaterthanorequal (a, b: tFrameDeltaTime): boolean;

begin
     result := a.NanoTicks >= b.NanoTicks;
end;

(* ----------------------------------- *)

class operator tFrameDeltaTime.lessthan (a, b: tFrameDeltaTime): boolean;

begin
     result := a.NanoTicks < b.NanoTicks;
end;

(* ----------------------------------- *)

class operator tFrameDeltaTime.lessthanorequal (a, b: tFrameDeltaTime): boolean;

begin
     result := a.NanoTicks <= b.NanoTicks;
end;

(* ----------------------------------- *)

class operator tFrameDeltaTime.notequal (a, b: tFrameDeltaTime): boolean;

begin
     result := not (a=b);
end;

(* ----------------------------------- *)
{ tEUIAddress }
(* ----------------------------------- *)

function tEUIAddress.asString (const Separator: string): string;

var i : integer;

begin
     result := '';

     for i := 1 to 8 do
         begin
              if i < 8 then
                 result := result + IntToHex (rangebytes[i], 2) + separator
              else
                  result := result + IntToHex (rangebytes[i], 2);
         end;
end;

(* ----------------------------------- *)

procedure tEUIAddress.setfromString(const Address, separator: string);

var i         : Integer;
    value     : string;
    temp      : integer;

begin
     for i := 1 to 8 do
         begin
              value := getEntry (Address, Separator, i);

              tryStrToInt (value, temp);
              rangebytes[i] := temp;
         end;
end;

(* ----------------------------------- *)

class operator tEUIAddress.equal(a, b: tEUIAddress): boolean;

var i : integer;

begin
     result := true;
     i := 1;

     for i := 1 to 8 do
         if not (a.rangebytes[i] = b.rangebytes[i]) then
            result := false;
end;

(* ----------------------------------- *)

class operator tEUIAddress.notequal(a, b: tEUIAddress): boolean;

begin
     result := not (a=b);
end;

(* ----------------------------------- *)
{ NGEPBFlags }
(* ----------------------------------- *)

function tFrameFlags.GetPacketDirection: tPacketDirection;

var help : longword;

begin
     help := Value and $3;           // get only bits 0 and 1

     result := pdUnknown;

     case help of
      1 : result := pdInbound;
      2 : result := pdOutbound;
     end;
end;

(* ----------------------------------- *)

function tFrameFlags.GetReceptionType: tPacketReceptionType;

var help : longword;

begin
     help := Value and $1c;           // get only bits 2,3,4
     help := help shr 2;

     result := prtNotSpecified;

     case help of
      1 : result := prtUnicast;
      2 : result := prtMulticast;
      3 : result := prtBroadcast;
      4 : result := prtPromiscuous;
     end;
end;

(* ----------------------------------- *)

function tFrameFlags.GetFCSLength: byte;

var help : longword;

begin
     help := Value and $1E0;           // get only bits 5 to 8
     help := help shr 5;

     result := byte (help);
end;

(* ----------------------------------- *)

function tFrameFlags.GetErrorFlags : tErrorFlagsSet;

var help : longword;

begin
     result := [];

     // Check Error Flags one by one and add the according markers

     if (Value and $80000000) = $80000000 then
        result := result + [efSymbolError];

     if (Value and $40000000) = $40000000 then
        result := result + [efBadPreamble];

     if (Value and $20000000) = $20000000 then
        result := result + [efBadStartFrameDelimiter];

     if (Value and $10000000) = $10000000 then
        result := result + [efAlignment];

     if (Value and $8000000) = $8000000 then
        result := result + [efWrongInterframeGap];

     if (Value and $4000000) = $4000000 then
        result := result + [efPacketTooShort];

     if (Value and $2000000) = $2000000 then
        result := result + [efPacketTooLong];

     if (Value and $1000000) = $1000000 then
        result := result + [efBadCRC];
end;

(* ----------------------------------- *)

procedure tFrameFlags.SetErrorFlags (FlagSet : tErrorFlagsSet);

begin
     if efSymbolError in FlagSet then
        Value := Value or $80000000;

     if efBadPreamble in FlagSet then
        Value := Value or $40000000;

     if efBadStartFrameDelimiter in FlagSet then
        Value := Value or $20000000;

     if efAlignment in FlagSet then
        Value := Value or $10000000;

     if efWrongInterframeGap in FlagSet then
        Value := Value or $8000000;

     if efPacketTooShort in FlagSet then
        Value := Value or $4000000;

     if efPacketTooLong in FlagSet then
        Value := Value or $2000000;

     if efBadCRC in FlagSet then
        Value := Value or $1000000;

     Present := true;
end;

(* ----------------------------------- *)

procedure tFrameFlags.SetPacketDirection (Direction: tPacketDirection);

var help : longword;

begin
     // Clear direction bits (0 and 1)

     Value := Value and $fffffffc;

     case Direction of
      pdInbound  : Value := Value or 1;
      pdOutbound : Value := Value or 2;
     end;

     Present := true;
end;

(* ----------------------------------- *)
{ tOutputlocation }
(* ----------------------------------- *)

procedure tOutputlocation.initialize (const LocationType : tLocationType; const SourceFilename, OutputDirectory, OutputFilename : string);

begin
     Location := LocationType;
     OutputDir := OutputDirectory;
     OutputFile := OutputFilename;
     SourceFile := SourceFilename;
     TimestampMode := tsmNone;
     OriginalFileTime := 0;

     TaskName := '';
     TaskNumber := 0;
     SectionID := 0;
     Count := 0;
end;

(* ----------------------------------- *)

function tOutputlocation.getOutputfilename: string;

var OutPath          : string;
    OriginalFilename : string;

begin
     // Get the directory

     case Location of
      ltSameDirectory     : begin
                                 OutPath := IncludeTrailingBackslash (ExtractFilePath (SourceFile));
                            end;
      ltSubdirectory      : begin
                                 OutPath := IncludeTrailingBackslash (ExtractFilePath (SourceFile)) + OutputDir;
                                 OutPath := IncludeTrailingBackslash (OutPath);
                            end;
      ltAbsoluteDirectory : begin
                                 OutPath := IncludeTrailingBackslash (OutputDir);
                            end;
     end;

     // Add the file

     OutPath := OutPath + OutputFile;

     // Do the replacements

     inc (Count);

     OriginalFilename := extractFilename (SourceFile);
     OriginalFilename := changeFileExt (OriginalFilename, '');

     OutPath := stringReplace (OutPath, '<taskname>', TaskName, [rfReplaceAll]);
     OutPath := stringReplace (OutPath, '<tasknumber>', IntToStr (TaskNumber), [rfReplaceAll]);
     OutPath := stringReplace (OutPath, '<date>', DateToStr (now), [rfReplaceAll]);
     OutPath := stringReplace (OutPath, '<datetime>', DateTimeToStr (now), [rfReplaceAll]);
     OutPath := stringReplace (OutPath, '<sectionid>', IntToStr (SectionID), [rfReplaceAll]);
     OutPath := stringReplace (OutPath, '<filename>', OriginalFilename, [rfReplaceAll]);
     OutPath := stringReplace (OutPath, '<count>', IntToStr (Count), [rfReplaceAll]);

     OutPath := OutPath + '.pcapng';

     result := OutPath;
end;

(* ----------------------------------- *)

function getStatusDescription (Status : tProcessingStatus) : string;

begin
     result := 'unknown';

     case Status of
      psNewFile               : result := 'New File';
      psScanningBasics        : result := 'Scanning Basics';
      psScanningPCAPngStruct  : result := 'Scanning PCAPng';
      psScanningConversations : result := 'Scanning Conversations';
      psScanAborted           : result := 'Scan aborted';
      psReadingDatabase       : result := 'Reading Database';
      psNoTaskAssigned        : result := 'No task assigned';
      psTaskPending           : result := 'Task pending';
      psTaskProcessing        : result := 'Processing task';
      psTaskSkipped           : result := 'Not relevant';
      psTaskComplete          : result := 'Task complete';
      psTaskAborted           : result := 'Task aborted';
      psTaskFailed            : result := 'Task failed';
     end;
end;

(* ----------------------------------- *)

function getLinkTypeName (LinkType : integer) : string;

begin
     case LinkType of
      LINKTYPE_NULL : result := 'NULL';
      LINKTYPE_ETHERNET : result := 'ETHERNET';
      LINKTYPE_TOKEN_RING : result := 'TOKEN_RING';
      LINKTYPE_SLIP : result := 'SLIP';
      LINKTYPE_PPP : result := 'PPP';
      LINKTYPE_RAW : result := 'RAW IP';
      LINKTYPE_IEEE802_11 : result := 'IEEE802_11';
      LINKTYPE_LOOP : result := 'LOOP';
      LINKTYPE_HDLC : result := 'HDLC';
      LINKTYPE_LINUX_SLL : result := 'LINUX_COOKED';
      LINKTYPE_JUNIPER : result := 'JUNIPER';
      LINKTYPE_CISCO_IOS : result := 'CISCO_IOS';
      LINKTYPE_AIRONET_HEADER : result := 'AIRONET_HEADER';
      LINKTYPE_IP_OVER_FC : result := 'IP_OVER_FC';
      LINKTYPE_IEEE802_11_RADIO : result := 'IEEE802_11';
      LINKTYPE_DOCSIS : result := 'DOCSIS';
      LINKTYPE_BLUETOOTH_HCI_H4 : result := 'BLUETOOTH_HCI_H4';
      LINKTYPE_USB_LINUX : result := 'USB_LINUX';
      LINKTYPE_IEEE802_15_4 : result := 'IEEE802_15_4';
      LINKTYPE_BLUETOOTH_HCI_H4_WITH_PHDR : result := 'BLUETOOTH_HCI_H4_WITH_PHDR';
      LINKTYPE_IEEE802_15_4_NONASK_PHY : result := 'IEEE802_15_4_NONASK_PHY';
      LINKTYPE_IPV4 : result := 'RAW IPV4';
      LINKTYPE_IPV6 : result := 'RAW IPV6';
      LINKTYPE_NETANALYZER : result := 'NETANALYZER';
      LINKTYPE_NETANALYZER_TRANSPARENT : result := 'NETANALYZER_TRANSPARENT';
     else
         result := 'unknown';
     end

end;

(* ----------------------------------- *)

function isValidMAC (MACAddress : string) : boolean;

var i     : integer;
    octet : string;
    value : integer;

begin
     // We allow : or -, but not both

     if pos (':', MACAddress) > 0 then
        MACAddress := StringReplace (MACAddress, ':', '', [rfReplaceAll])
     else
         if pos ('-', MACAddress) > 0 then
            MACAddress := StringReplace (MACAddress, '-', '', [rfReplaceAll]);

     // Check all hex octets if they're valid numbers

     if length (MACAddress) <> 12 then
        result := false
     else
         begin
              result := true;

              for i := 0 to 5 do
                  begin
                       octet := '$' + copy (MACAddress, (i*2) + 1, 2);

                       TryStrToInt (octet, value);

                       if value = 0 then
                          if octet <> '$00' then
                             result := false;
                  end;
         end;
end;

(* ----------------------------------- *)
{ tFrameChunk }
(* ----------------------------------- *)

procedure tFrameChunk.initialize;

begin
     Data := nil;
     Len := 0;

     setLength (cutmarkers, 0);
end;

(* ----------------------------------- *)

procedure tFrameChunk.CreateZeroBuffer (BufferLen : integer);

var i    : integer;
    help : PByte;

begin
     Len := BufferLen;

     Data := AllocMem (Len);
{
     getMem (Data, Len);

     help := PByte (Data);

     for i := 1 to Len do
         begin
              help^ := 0;
              inc (help);
         end;
}
end;

(* ----------------------------------- *)

procedure tFrameChunk.CreateFromPattern (Pattern: ANSIstring; BufferLen : integer);

var help  : ansistring;
    count : integer;
    i     : integer;

begin
     self.initialize;

     // Create a string that is at least as long as we need it

     count := (BufferLen div Length (Pattern)) + 1;
     help := DupeString (Pattern, count);

     // cut down to the exact length

     help := copy (help, 1, BufferLen);

     // Set Buffer

     self.copyFrom (help);
end;

(* ----------------------------------- *)

procedure tFrameChunk.copyFrom (Chunk: tFrameChunk);

begin
     if len > 0 then
        freemem (Data, len);

     len := Chunk.Len;

     // Reserve memory for first part

     getmem (data, Chunk.Len);

     move (Chunk.Data^, data^, Len);
end;

(* ----------------------------------- *)

procedure tFrameChunk.copyFrom (RawBuffer: tRawBuffer; BufferLen : integer);

begin
     if len > 0 then
        freemem (Data, len);

     len := BufferLen;

     // Reserve memory for first part

     getmem (data, Len);

     move (RawBuffer, data^, Len);
end;

(* ----------------------------------- *)

procedure tFrameChunk.copyFrom (const Value : ANSIString);

var help  : pANSIChar;
    i     : integer;

begin
     if len > 0 then
        freemem (Data, len);

     len := Length (Value);

     // Reserve memory for first part

     getmem (data, Len);

     help := pANSIChar (data);

     for i  := 1 to len do
         begin
              help^ := value[i];
              inc (help);
         end;
end;

(* ----------------------------------- *)

procedure tFrameChunk.copyFrom (const Value: tHexRepresentation);

var help : ANSIString;

begin
     // Get the content as binary string, then assign

     help := Value.ValueAsBinaryString;

     self.copyFrom(help);
end;

(* ----------------------------------- *)

procedure tFrameChunk.copyFrom (const DataPointer: pByte; const BufferLen: integer);

begin
     len := BufferLen;

     // Reserve memory for first part

     getmem (data, BufferLen);

     move (DataPointer^, data^, Len);
end;

(* ----------------------------------- *)

procedure tFrameChunk.AttachCRC32 (const CRC32 : longword);

var HelpChunk : tFrameChunk;
    OldData   : pointer;
    pCRCPos   : PByte;
    pCRC32    : pLongword;

begin
     // Build a new frame chunk

     OldData := Data;
     getmem (data, Len + 4);

     move (OldData^, data^, Len);

     // Position help at the offset where the second part will be copied to

     pCRCPos := pByte (Data) + Len;
     pCRC32 := pLongword (pCRCPos);
     pCRC32^ := CRC32;

     // Clean up

     freemem (OldData, Len);
     Len := Len + 4;
end;

(* ----------------------------------- *)

procedure tFrameChunk.AttachCRC32 (const SetBad: boolean);

var CRC32 : Longword;

begin
     CRC32 := self.getCRC32;

     if SetBad then
        if CRC32 < cardinal.MaxValue then
           inc (CRC32)
        else
            CRC32 := 0;

     AttachCRC32 (CRC32);
end;

(* ----------------------------------- *)

procedure tFrameChunk.Patch (const Offset : integer; const Value : byte);

begin
     if Offset + sizeOf (byte) > Len then
        raise Exception.Create('Error: invalid memory access while patching FrameChunk');

     PatchMem (pByte (Data), Offset, Value);
end;

(* ----------------------------------- *)

procedure tFrameChunk.Patch (const Offset : integer; const Value : word; const DoNotSwap : boolean = false);

var help : pWord;

begin
     if Offset + sizeOf (Word) > Len then
        raise Exception.Create('Error: invalid memory access while patching FrameChunk');

     if DoNotSwap then
        PatchMem (pByte (Data), Offset, Value)
     else
         PatchMem (pByte (Data), Offset, swap16 (Value));
end;

(* ----------------------------------- *)

procedure tFrameChunk.Patch (const Offset : integer; const Value : longword; const DoNotSwap : boolean = false);

var help : PLongword;

begin
     if Offset + sizeOf (Longword) > Len then
        raise Exception.Create('Error: invalid memory access while patching FrameChunk');

     if DoNotSwap then
        PatchMem (pByte (Data), Offset, Value)
     else
         PatchMem (pByte (Data), Offset, swap32 (Value));
end;

(* ----------------------------------- *)

procedure tFrameChunk.Patch (const Offset : integer; const Value : uint64; const DoNotSwap : boolean = false);

var help : puInt64;

begin
     if Offset + sizeOf (Int64) > Len then
        raise Exception.Create('Error: invalid memory access while patching FrameChunk');

     if DoNotSwap then
        PatchMem (pByte (Data), Offset, Value)
     else
         PatchMem (pByte (Data), Offset, swap64 (Value));
end;

(* ----------------------------------- *)

procedure tFrameChunk.Patch (const Offset : integer; const Value : string);

var help : pByte;
    i    : integer;

begin
     if Offset + length (Value) > Len then
        raise Exception.Create('Error: invalid memory access while patching FrameChunk');

     PatchMem (pByte (Data), Offset, Value);
end;

(* ----------------------------------- *)

function tFrameChunk.SectionCount: integer;

begin
     result := length (cutmarkers) + 1;
end;

(* ----------------------------------- *)

procedure tFrameChunk.clearCutMarkers;

begin
     setLength (cutmarkers, 0);
end;

(* ----------------------------------- *)

// Needed for FCS

procedure tFrameChunk.CreateFromLongword (const Value: LongWord);

begin
     Len := 4;
     getMem (Data, Len);

     Patch (0, Value);
end;

(* ----------------------------------- *)

function tFrameChunk.getAsANSIString : ANSIString;

var value : ANSIString;
    i     : integer;
    help  : pANSIChar;

begin
     value := '';
     help := pANSIChar (data);

     for i := 1 to len do
         begin
              value := value + help^;
              inc (help);
         end;

     result := value;
end;

(* ----------------------------------- *)

function tFrameChunk.getAsHEXString (const Separator : ANSIstring = '') : ANSIString;

var value : ANSIString;
    i     : integer;
    help  : pByte;
    ihelp : integer;

begin
     value := '';
     help := pByte (data);

     for i := 1 to len do
         begin
              ihelp := help^;
              value := value + IntToHex (ihelp, 2) + Separator;
              inc (help);
         end;

     result := value;
end;

(* ----------------------------------- *)

function tFrameChunk.getAsBufferWords: tBufferWords;

begin
     result := getWordsStructFromOffset (Data, 0, Len);
end;

(* ----------------------------------- *)

function tFrameChunk.getAsRawBuffer: tRawBuffer;

var buf : tRawBuffer;

begin
     InitializeRawBuffer (buf);

     move (data^, buf, len);

     result := buf;
end;

(* ----------------------------------- *)

function tFrameChunk.getByte (Offset: integer) : byte;

var help : pByte;

begin
     assert (Offset < len, 'Invalid Offset (tFrameChunk.getByte)');

     help := pByte (data) + Offset;

     result := help^;
end;

(* ----------------------------------- *)

function tFrameChunk.getWord (Offset: integer; const Swapped : boolean = false) : word;

var help : pByte;

begin
     assert (Offset < len-1, 'Invalid Offset (tFrameChunk.getByte)');

     help := pByte (data) + Offset;

     if Swapped then
        result := swap16 (pWord (help)^)
     else
         result := pWord (help)^;
end;

(* ----------------------------------- *)

function tFrameChunk.getLongWord (Offset: integer; const Swapped : boolean = false): longword;

var help : pByte;

begin
     assert (Offset < len-1, 'Invalid Offset (tFrameChunk.getByte)');

     help := pByte (data) + Offset;

     if Swapped then
        result := swap32 (pLongWord (help)^)
     else
         result := pLongWord (help)^;
end;

(* ----------------------------------- *)

function tFrameChunk.getCRC32: longword;

begin
     result := getCRC32FromPointer (Data, Len);
end;

(* ----------------------------------- *)

procedure tFrameChunk.writeToDisk (Filename: string);

var fHandle : file of byte;

begin
     assignfile (fHandle, Filename);
     rewrite (fHandle);

     blockwrite (fHandle, Data^, Len);

     closefile (fHandle);
end;

(* ----------------------------------- *)

procedure tFrameChunk.free;

begin
     freemem (Data, Len);
     Data := nil;
     Len := 0;
     setlength (cutmarkers, 0);
end;

(* ----------------------------------- *)

class operator tFrameChunk.equal (a, b : tFrameChunk) : boolean;

var value : boolean;
    aByte : pByte;
    bByte : pByte;
    i     : integer;

begin
     value := (a.Len = b.Len);

     if value then
        begin
             i := 0;
             aByte := pByte (a.Data);
             bByte := pByte (b.Data);

             // Scan until mismatch found or all bytes were compared

             while value and (i < a.len) do
                   begin
                        if aByte^ <> bByte^ then
                           value := false;

                        inc (aByte);
                        inc (bByte);
                        inc (i);
                   end;
        end;

     result := value;
end;

(* ----------------------------------- *)

class operator tFrameChunk.notequal (a, b : tFrameChunk) : boolean;

begin
     result := not (a=b);
end;

(* ----------------------------------- *)
{ tEtherConversation }
(* ----------------------------------- *)
{
procedure tEtherConversation.initialize;

begin
     DBIndex := 0;
     MACAddressPair.initialize;
     Address1DBIndex := 0;
     Address2DBIndex := 0;
     FirstFoundInFileID := 0;
     LastFoundInFileID := 0;
     FirstFoundInFrameID := 0;
     LastFoundInFrameID := 0;
     VLANID := 0;
     StartTimeStamp := encodeDatetime (1970,1,1,0,0,0,0);
     EndTimeStamp := encodeDatetime (1970,1,1,0,0,0,0);
     FrameCount := 0;
end;
}
(* ----------------------------------- *)
{ tVersionStruct }
(* ----------------------------------- *)

procedure tVersionStruct.initialize;

begin
     version   := '';
     changelog := '';
     builddate := EncodeDateTime (1970,1,1,0,0,0,0);
     status := vsOkay;
     content := '';
end;

(* ----------------------------------- *)
{ tWireDetails }
(* ----------------------------------- *)

procedure tWireDetails.initialize;

begin
     LinkType := 1;               // Ethernet
     WireSize := 0;
     CaptureSize := 0;
     InterfaceID := 0;
//   PacketDirection := pdUnknown;
     Dropped := 0;
     FrameFlags.Value := 0;
     FrameFlags.Present := false;
     FrameType := tftUnknown;
end;

(* ----------------------------------- *)
{ tFilePosition }
(* ----------------------------------- *)

class operator tFileFrameLocation.equal(a, b: tFileFrameLocation): boolean;

begin
     result := (a.FileIndex = b.FileIndex) and
               (a.FileFrameIndex = b.FileFrameIndex);
{
               (a.FileFrameIndex = b.FileFrameIndex) and
               (a.FileOffset = b.FileOffset);
}
end;

(* ----------------------------------- *)

class operator tFileFrameLocation.notequal(a, b: tFileFrameLocation): boolean;

begin
     result := not (a=b);
end;

(* ----------------------------------- *)

class operator tFileFrameLocation.greaterthan (a, b: tFileFrameLocation): boolean;

begin
     result := False;

     if (a.FileIndex > b.FileIndex) then
        result := true
     else
         if (a.FileIndex = b.FileIndex) then
            begin
                 if (a.FileFrameIndex > b.FileFrameIndex) then
                    result := True;
{
                 else
                     if (a.FileFrameIndex = b.FileFrameIndex) and (a.FileOffset > b.FileOffset) then
                        result := True;
}
            end;
end;

(* ----------------------------------- *)

class operator tFileFrameLocation.greaterthanorequal (a, b: tFileFrameLocation): boolean;

begin
     result := (a=b) or (a > b);
end;

(* ----------------------------------- *)

class operator tFileFrameLocation.lessthan (a, b: tFileFrameLocation): boolean;

begin
     result := not (a >= b);
end;

(* ----------------------------------- *)

class operator tFileFrameLocation.lessthanorequal (a, b: tFileFrameLocation): boolean;

begin
     result := not (a > b);
end;

(* ----------------------------------- *)

procedure tFileFrameLocation.initialize;

begin
     FileIndex       := 0;
     FileFrameIndex  := 0;
     FileOffset      := 0;
end;

(* ----------------------------------- *)

procedure tFileFrameLocation.initialize (const Index: integer; const FrameID, Offset: int64);

begin
     FileIndex := Index;
     FileFrameindex := FrameID;
     FileOffset := Offset;
end;

(* ----------------------------------- *)

procedure tFileFrameLocation.UpdateIfLower (const NewFileIndex, NewFrameIndex: integer; const NewFileOffset: int64);

begin
     if NewFileIndex < FileIndex then
        begin
             FileIndex := NewFileIndex;
             FileFrameindex := NewFrameIndex;
             FileOffset := NewFileOffset;
        end
     else
         if NewFileIndex = FileIndex then
            begin
                 if NewFrameIndex < FileFrameindex then
                    begin
                         FileFrameindex := NewFrameIndex;
                         FileOffset := NewFileOffset;
                    end;
            end;
end;

(* ----------------------------------- *)

procedure tFileFrameLocation.UpdateIfLower (const Location: tFileFrameLocation);

begin
     UpdateIfLower (Location.FileIndex, Location.FileFrameIndex, Location.FileOffset);
end;

(* ----------------------------------- *)

procedure tFileFrameLocation.UpdateIfHigher (const NewFileIndex, NewFrameIndex: integer; const NewFileOffset: int64);

begin
     if NewFileIndex > FileIndex then
        begin
             FileIndex := NewFileIndex;
             FileFrameindex := NewFrameIndex;
             FileOffset := NewFileOffset;
        end
     else
         if NewFileIndex = FileIndex then
            begin
                 if NewFrameIndex > FileFrameindex then
                    begin
                         FileFrameindex := NewFrameIndex;
                         FileOffset := NewFileOffset;
                    end;
            end;
end;

(* ----------------------------------- *)

procedure tFileFrameLocation.UpdateIfHigher (const Location: tFileFrameLocation);

begin
     UpdateIfHigher (Location.FileIndex, Location.FileFrameIndex, Location.FileOffset);
end;

(* ----------------------------------- *)

function tFileFrameLocation.isZero: boolean;

begin
     result := (FileIndex = 0) and (FileFrameIndex = 0) and (FileOffset = 0);
end;

(* ----------------------------------- *)

function tFileFrameLocation.toString: string;

begin
     result := FileIndex.ToString + ':' + FileFrameIndex.ToString + ':' + FileOffset.ToString;
end;

(* ----------------------------------- *)

function tFileFrameLocation.DictionaryKey: string;

begin
     result := FileIndex.ToString + ':' + FileFrameIndex.ToString + ':' + FileOffset.ToString;
end;

(* ----------------------------------- *)
{ tTimingDetails }
(* ----------------------------------- *)

procedure tTimingDetails.initialize;

begin
     AbsoluteTime  := 0;
     TraceTicks    := 0;
     NanoTicks     := 0;
end;

(* ----------------------------------- *)
{ tGeneralTraceStats }
(* ----------------------------------- *)

procedure tGeneralTraceStats.initialize;

begin
     FirstFrameTime.initialize;
     FirstFrameOffset := 0;
     LastFrameTime.initialize;
     LastFrameOffset := 0;
     MinFrameSize := 10000;
     MaxFrameSize := 0;
     AverageFrameSize := 0;
     FrameCount := 0;
     TotalFrameBytes := 0;
     TotalWireBytes := 0;
     isSliced := false;
     SliceSizeConstant := true;
     SliceSize := 0;
     TimeOrderCorrect := true;
     OutOfTimeFrames := 0;
end;

(* ----------------------------------- *)
{ tDeltaTickTime }
(* ----------------------------------- *)

procedure tDeltaTickTime.initialize;

begin
     deltaDay := 0;
     iHour := 0;
     iMin := 0;
     iSec := 0;
     iMicSec := 0;
end;

(* ----------------------------------- *)
{ tHexRepresentation }
(* ----------------------------------- *)

procedure tHexRepresentation.initialize;

begin
     ValueAsString := '';
end;

(* ----------------------------------- *)

function tHexRepresentation.isValid: boolean;

begin
     result := isValidSnortHexString (ValueAsString);
end;

(* ----------------------------------- *)

function tHexRepresentation.GetHexByteLength: integer;

begin
     result := getSnortHexLen (ValueAsString);
end;

(* ----------------------------------- *)

function tHexRepresentation.ValueAsBinaryString: ANSIString;

var value : ANSIString;
    count : integer;
    sub   : string;
    i     : integer;

begin
     if pos ('|', ValueAsString) = 0 then
        value := ValueAsString
     else
         begin
              value := '';
              count := numEntries (ValueAsString, '|');

              for i := 1 to count do
                  begin
                       sub := getEntry (ValueAsString, '|', i);

                       if odd (i) then
                          value := value + sub
                       else
                           value := value + getSnortHexAsString (sub);
                  end;
         end;

     result := value;
end;

(* ------------------------------------------- *)

function Layer4ProtocolToStr (const Protocol : tLayer4Protocol) : string;

begin
     case Protocol of
      l4TCP    : result := 'TCP';
      l4UDP    : result := 'UDP';
      l4ICMP   : result := 'ICMP';
      l4Any    : result := 'Any';
      l4TCPUDP : result := 'TCP/UDP';
     else
         result := 'unknown';
     end;
end;

(* ------------------------------------------- *)

function StrToLayer4Protocol (const ProtocolString : string) : tLayer4Protocol;

var help : string;

begin
     help := UpperCase (ProtocolString);

     if help = 'TCP' then
        result := l4TCP
     else
     if help = 'UDP' then
        result := l4UDP
     else
     if help = 'ICMP' then
        result := l4ICMP
     else
     if help = 'Any' then
        result := l4Any
     else
     if help = 'TCP/UDP' then
        result := l4TCPUDP
     else
         result := l4unknown;
end;

(* ----------------------------------- *)

function Layer4ProtocolToByte (const Protocol : tLayer4Protocol; const IPVersion : byte = 4) : byte;

begin
     case Protocol of
      l4TCP  : result := 6;
      l4UDP  : result := 17;
      l4ICMP : begin
                    if IPVersion = 6 then
                       result := 58
                    else
                        result := 1;
               end
     else
         result := 0;
     end;
end;

(* ----------------------------------- *)

function ByteToLayer4Protocol (const ProtocolByte : byte) : tLayer4Protocol;

begin
     result := l4Any;

     case ProtocolByte of
       1,
      58 : result := l4ICMP;
       6 : result := l4TCP;
      17 : result := l4UDP;
     end;
end;

(* ----------------------------------- *)

function getCRC32FromPointer (const DataPtr : Pointer; Len : integer) : longword;

var p   : PByte;
    i   : Integer;
    crc : longword;

begin
     crc := $ffffffff;

     p := PByte (DataPtr);

     // We calculate the CRC over the full length minus the last 4 bytes because
     // sometimes traces still contain the FCS which compensates the CRC and gives
     // false results

     for i := 1 to Len do
         begin
              crc := (crc shr 8)  xor  crcTable[p^ xor (crc and $000000FF)];
              inc(p);
         end;

     crc := not crc;

     result := crc;
end;

(* ----------------------------------- *)
{ tTTLAnchorPair }
(* ----------------------------------- *)

class operator tTTLAnchorPair.equal(a, b: tTTLAnchorPair): boolean;

begin
     result := (a.SourceTTL = b.SourceTTL) and (a.DestinationTTL = b.DestinationTTL);
end;

(* ----------------------------------- *)

class operator tTTLAnchorPair.notequal(a, b: tTTLAnchorPair): boolean;

begin
     result := not (a=b);
end;

(* ----------------------------------- *)

procedure tTTLAnchorPair.initialize;

begin
     SourceTTL := 0;
     DestinationTTL := 0;
end;

(* ----------------------------------- *)

function QueryTypesToInt64 (const TypeSet : tDNSRecordTypeSet) : uint64;

var value : uint64;
    qtype : tDNSRecordType;
    index : integer;
    help  : uint64;

begin
     value := 0;

     for qtype in TypeSet do
         begin
              index := ord (qtype);
              help := trunc (power (2, index));

              value := value or help;
         end;

     result := value;
end;

(* ----------------------------------- *)

function Int64ToQueryTypes (const value : uint64) : tDNSRecordTypeSet;

var TypeSet : tDNSRecordTypeSet;
    help    : uint64;
    i       : integer;

begin
     TypeSet := [];

     // we got 38 types... can't count to 64 because it'll crash

     for i := 0 to 38 do
         begin
              help := trunc (power (2, i));

              if (value and help) = help then
                 TypeSet := TypeSet + [tDNSRecordType(i)];
         end;

     result := TypeSet;
end;

(* ----------------------------------- *)

function DNSReplyCodeFromWord (const ReplyCode : word) : tDNSReturnCode;

begin
      case ReplyCode of
      0 : result := dnsrcNoError;
      1 : result := dnsrcFormErr;
      2 : result := dnsrcServFail;
      3 : result := dnsrcNXDomain;
      4 : result := dnsrcNotImp;
      5 : result := dnsrcRefused;
      6 : result := dnsrcYXDomain;
      7 : result := dnsrcYXRRSet;
      8 : result := dnsrcNXRRSet;
      9 : result := dnsrcNotAuth;
      10 : result := dnsrcNotZone;
      11..15 : result := dnsrcAvailable;
      16 : result := dnsrcBADSIG;
      17 : result := dnsrcBADKEY;
      18 : result := dnsrcBADTIME;
      19 : result := dnsrcBADMODE;
      20 : result := dnsrcBADNAME;
      21 : result := dnsrcBADALG;
      22..3840 : result := dnsrcAvailable;
      3851..4095 : result := dnsRCPrivateUse;
      4096..65535 : result := dnsrcAvailable;
     end;
end;

(* ----------------------------------- *)

function DNSReplyCodeToWord (const value : tDNSReturnCode) : word;

begin
     case value of
      dnsrcNoError    : result := 0;
      dnsrcFormErr    : result := 1;
      dnsrcServFail   : result := 2;
      dnsrcNXDomain   : result := 3;
      dnsrcNotImp     : result := 4;
      dnsrcRefused    : result := 5;
      dnsrcYXDomain   : result := 6;
      dnsrcYXRRSet    : result := 7;
      dnsrcNXRRSet    : result := 8;
      dnsrcNotAuth    : result := 9;
      dnsrcNotZone    : result := 10;
      dnsrcBADVERS    : result := 16;
      dnsrcBADSIG     : result := 16;
      dnsrcBADKEY     : result := 17;
      dnsrcBADTIME    : result := 18;
      dnsrcBADMODE    : result := 19;
      dnsrcBADNAME    : result := 20;
      dnsrcBADALG     : result := 21;
      dnsrcAvailable  : result := 11;
      dnsrcPrivateUse : result := 3851;
     end;
end;

(* ----------------------------------- *)

function QueryTypeToString (const QueryType : tDNSRecordType) : string;

begin
     case QueryType of
      dnsrtA          : result := 'A';
      dnsrtNS         : result := 'NS';
      dnsrtCNAME      : result := 'CNAME';
      dnsrtSOA        : result := 'SOA';
      dnsrtPTR        : result := 'PTR';
      dnsrtMX         : result := 'MX';
      dnsrtTXT        : result := 'TXT';
      dnsrtAAAA       : result := 'AAAA';
      dnsrtSRV        : result := 'SRV';
      dnsrtIXFR       : result := 'IXFR';
      dnsrtAFXR       : result := 'AFXR';
      dnsrtOPT        : result := 'OPT';
      dnsrtDS         : result := 'DS';
      dnsrtNSEC3      : result := 'NSEC3';
      dnsrtRRSIG      : result := 'RRSIG';
      dnsrtSPF        : result := 'SPF';
      dnsrtAFSDB      : result := 'AFSDB';
      dnsrtAPL        : result := 'APL';
      dnsrtCERT       : result := 'CERT';
      dnsrtDHCID      : result := 'DHCID';
      dnsrtDLV        : result := 'DLV';
      dnsrtDNAME      : result := 'DNAME';
      dnsrtDNSKEY     : result := 'DNSKEY';
      dnsrtIPSECKEY   : result := 'IPSECKEY';
      dnsrtKEY        : result := 'KEY';
      dnsrtKX         : result := 'KX';
      dnsrtLOC        : result := 'LOC';
      dnsrtNAPTR      : result := 'NAPTR';
      dnsrtNSEC       : result := 'NSEC';
      dnsrtSIG        : result := 'SIC';
      dnsrtTKEY       : result := 'TKEY';
      dnsrtTLSA       : result := 'TLSA';
      dnsrtSSHFP      : result := 'SSHFP';
      dnsrtNSEC3PARAM : result := 'NSEC3PARAM';
      dnsrtHIP        : result := 'HIP';
      dnsrtTSIG       : result := 'TSIG';
      dnsrtAny        : result := 'ANY';
      dnsrtUnknown    : result := 'Unknown';
     else
         result := 'Unknown';
     end;
end;

(* ----------------------------------- *)

function QueryTypeSetToString (const TypeSet : tDNSRecordTypeSet) : string;

var recType : tDNSRecordType;
    value   : string;

begin
//   tDNSRecordType = (dnsrtA, dnsrtNS, dnsrtCNAME, dnsrtSOA, dnsrtPTR, dnsrtMX, dnsrtTXT, dnsrtAAAA, dnsrtSRV, dnsrtIXFR, dnsrtAFXR, dnsrtOPT, dnsrtDS, dnsrtNSEC3, dnsrtRRSIG, dnsrtSPF, dnsrtAFSDB, dnsrtAPL, dnsrtCERT, dnsrtDHCID, dnsrtDLV, dnsrtDNAME, dnsrtDNSKEY, dnsrtIPSECKEY, dnsrtKEY, dnsrtKX, dnsrtLOC, dnsrtNAPTR, dnsrtNSEC, dnsrtSIG, dnsrtTKEY, dnsrtTLSA, dnsrtSSHFP, dnsrtNSEC3PARAM, dnsrtHIP, dnsrtTSIG, dnsrtAny, dnsrtUnknown);

     value := '';

     for recType in TypeSet do
         begin
              value := value + QueryTypeToString (recType) + ' ';
         end;

     result := stringReplace (trim (value), ' ', ',', [rfReplaceAll]);
end;

(* ----------------------------------- *)

function getHopDistanceEstimate (const TTL : byte) : byte;

var OriginTTL : byte;

begin
     if TTL > 128 then
        OriginTTL := 255
     else
         if TTL > 64 then
            OriginTTL := 128
         else
             if TTL > 32 then
                OriginTTL := 64
             else
                 OriginTTL := 32;

     result := OriginTTL - TTL;
end;

(* ----------------------------------- *)
{ tTCPRTTStats }
(* ----------------------------------- *)

procedure tMMAInt64.initialize (const initialvalue : int64);

begin
     if initialvalue = 0 then
        begin
             Min := 0;
             Max := 0;
             Total := 0;
             NumValues := 0;
        end
     else
         begin
             Min := initialvalue;
             Max := initialvalue;
             Total := initialvalue;
             NumValues := 1;
         end;
end;

(* ----------------------------------- *)

procedure tMMAInt64.addValue (const newvalue: int64);

begin
     if (newvalue < Min) or (Min = 0) then
        Min := newvalue;

     if (newvalue > Max) then
        Max := newvalue;

     Total := Total + newvalue;
     inc (NumValues);
end;

(* ----------------------------------- *)

function tMMAInt64.average: int64;

begin
     result := Total div NumValues;
end;

(* ----------------------------------- *)

function TunnelFlagsToWord (TunnelFlags : tTunnelFlagSet) : word;

var value : word;
    help  : word;

begin
     // Set bits according to flags

     if tfTunnelCarrier in Tunnelflags then
        value := 1
     else
         value := 0;

     if tfTunnelLayer in Tunnelflags then
        value := value + 2;

     if tfTunnelPayload in Tunnelflags then
        value := value + 4;

     if tfTunnelPayload in Tunnelflags then
        value := value + 8;

     if tfTunnelMPLS in Tunnelflags then
        value := value + 16;

     if tfTunnelGRE in Tunnelflags then
        value := value + 32;

     if tfTunnelGTPv1 in Tunnelflags then
        value := value + 64;

     if tfTunnelAYIYA in Tunnelflags then
        value := value + 128;

     if tfTunnelVXLAN in Tunnelflags then
        value := value + 256;

     if tfTunnelERSPAN in Tunnelflags then
        value := value + 512;

     if tfTunnelCAPWAP in Tunnelflags then
        value := value + 1024;

     if tfTunnelGeneve in Tunnelflags then
        value := value + 2048;

     // Check if there's unknown flags we forgot to handle

     assert ((value > 0) or (Tunnelflags = []), 'Unknown tunnel flag encountered (TunnelFlagsToWord)');

     result := value;
end;

(* ----------------------------------- *)

function WordToTunnelFlags (value : word) : tTunnelFlagSet;

var flagset : tTunnelFlagSet;

begin
     if (Value and 1) = 1 then
        flagset := [tfTunnelCarrier]
     else
         flagset := [];

     if (Value and 2) = 2 then
        flagset := flagset + [tfTunnelLayer];

     if (Value and 4) = 4 then
        flagset := flagset + [tfTunnelPayload];

     if (Value and 8) = 8 then
        flagset := flagset + [tfTunnelPayload];

     if (Value and 16) = 16 then
        flagset := flagset + [tfTunnelMPLS];

     if (Value and 32) = 32 then
        flagset := flagset + [tfTunnelGRE];

     if (Value and 64) = 64 then
        flagset := flagset + [tfTunnelGTPv1];

     if (Value and 128) = 128 then
        flagset := flagset + [tfTunnelAYIYA];

     if (Value and 256) = 256 then
        flagset := flagset + [tfTunnelVXLAN];

     if (Value and 512) = 512 then
        flagset := flagset + [tfTunnelERSPAN];

     if (Value and 1024) = 1024 then
        flagset := flagset + [tfTunnelCAPWAP];

     if (Value and 2048) = 2048 then
        flagset := flagset + [tfTunnelGeneve];

     result := flagset;
end;

(* ----------------------------------- *)
{ tICMPv4TypeCode }
(* ----------------------------------- *)

procedure tICMPv4TypeCode.initialize;

begin
     ICMPv4Type := 0;
     ICMPv4Code := 0;
end;

(* ----------------------------------- *)

procedure tICMPv4TypeCode.initialize (const NewType, NewCode: integer);

begin
     ICMPv4Type := NewType;
     ICMPv4Code := NewCode;
end;

(* ----------------------------------- *)

procedure tICMPv4TypeCode.initialize(const value: string);

var dummy    : integer;
    help     : string;

begin
     if numEntries (value, ':') = 2 then
        begin
             help := getEntry (value, ':', 1);
             val (help, ICMPv4Type, dummy);

             help := getEntry (value, ':', 2);
             val (help, ICMPv4Code, dummy);
        end;
end;

(* ----------------------------------- *)

class operator tICMPv4TypeCode.equal (a, b: tICMPv4TypeCode): boolean;

begin
     result := (a.ICMPv4Type = b.ICMPv4Type) and (a.ICMPv4Code = b.ICMPv4Code);
end;

(* ----------------------------------- *)

class operator tICMPv4TypeCode.notequal (a, b: tICMPv4TypeCode): boolean;

begin
     result := not (a=b);
end;

(* ----------------------------------- *)

function tICMPv4TypeCode.toString (const separator : string = ':') : string;

begin
     result := ICMPv4Type.ToString + separator + ICMPv4Code.ToString;
end;

(* ----------------------------------- *)
{ Configuration value sanity checks }
(* ----------------------------------- *)

function isValidANSIStringPair (value : string; const CheckEmpty : boolean) : boolean;

var regEx : tRegEx;
    help  : string;

begin
     regEx := tRegEx.Create ('^".*"=".*"$');

     result := regEx.IsMatch (value);

     if result and CheckEmpty then
        begin
             help := getEntry (value, '"="', 1);

             result := not ((help = '"') or (help = ''));
        end;
end;

(* ----------------------------------- *)

function isValidVLANIDPair (value : string) : boolean;

var id    : integer;
    dummy : integer;
    help  : string;

begin
     if numEntries (value, '=') = 2 then
        begin
             help := getEntry (value, '=', 1);
             val (help, id, dummy);

             result := (help = '0') or ((id > 0) and (id < 4096));

             if result then
                begin
                     help := getEntry (value, '=', 2);
                     val (help, id, dummy);

                     result := (help = '0') or ((id > 0) and (id < 4096));
                end;
        end
     else
         result := false;
end;

(* ----------------------------------- *)

function isValidICMPv4TypeCodePair (value : string) : boolean;

var icmptype : integer;
    icmpcode : integer;
    dummy    : integer;
    help     : string;

begin
     if numEntries (value, ':') = 2 then
        begin
             help := getEntry (value, ':', 1);
             val (help, icmptype, dummy);

             result := (help = '0') or ((icmptype > 0) and (icmptype < 19));

             if result then
                begin
                     help := getEntry (value, ':', 2);
                     val (help, icmpcode, dummy);

                     case icmptype of
                       3 : result := (help = '0') or ((icmpcode > 0) and (icmpcode < 16));
                       5 : result := (help = '0') or ((icmpcode > 0) and (icmpcode < 4));
                      11 : result := (help = '0') or ((icmpcode > 0) and (icmpcode < 2));
                     else
                         result := (help = '0');
                     end;
                end;
        end
     else
         result := false;
end;

(* ----------------------------------- *)

function isValidPortPair (value : string) : boolean;

var id    : integer;
    dummy : integer;
    help  : string;

begin
     if numEntries (value, '=') = 2 then
        begin
             help := getEntry (value, '=', 1);
             val (help, id, dummy);

             result := (help = '0') or ((id > 0) and (id < 65536));

             if result then
                begin
                     help := getEntry (value, '=', 2);
                     val (help, id, dummy);

                     result := (help = '0') or ((id > 0) and (id < 65536));
                end;
        end
     else
         result := false;
end;

(* ----------------------------------- *)

initialization
              ZERO_GUID := tGUID.Empty;
              TWWindowLabel := '';
              LocalTraceIntelFilename := 'traceintel.db';
end.
