{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_formstorage;

interface

uses Controls,
     Forms,
     Grids,
     SysUtils,
     StdCtrls,
     ComCtrls,
     ExtCtrls,
     IniFiles,
     Classes,
     ValEdit,

{$ifdef VER230}

     // AnyDAC

     uADStanIntf, uADStanOption, uADStanError, uADGUIxIntf,
     uADPhysIntf, uADStanDef, uADStanPool,
     uADStanAsync, uADPhysManager, uADStanParam, uADDatSManager, uADDAptIntf,
     uADDAptManager, uADStanExprFuncs, Vcl.StdCtrls, uADPhysSQLite, Data.DB,
     uADCompDataSet, uADCompClient, uADGUIxFormsWait, uADPhysSQLiteWrapper,

{$else}

     // FireDAC

     FireDAC.Stan.Intf, FireDAC.Stan.Option, FireDAC.Stan.Error, FireDAC.UI.Intf,
     FireDAC.Phys.Intf, FireDAC.Stan.Def, FireDAC.Stan.Pool, FireDAC.dAPT,
     FireDAC.Stan.Async, FireDAC.Phys, FireDAC.Stan.ExprFuncs,
     FireDAC.VCLUI.Wait, FireDAC.Comp.UI, FireDAC.Phys.SQLite, FireDAC.Phys.SQLiteWrapper,
     Data.DB, FireDAC.Comp.Client, FireDAC.Stan.Param,
{$endif}

     // Smartinspect

     SiAuto;

type tWipeScope = (wpForm, wpAll);

     tVisualStatus = record
                           keepdefault  : boolean;
                           enabled      : boolean;
                           visible      : boolean;
                     end;

     tFormStorageManager = class (tObject)

                                 constructor create (AssociatedForm : tForm); overload; virtual;
                                 constructor create (AssociatedForm : tForm; DBConnection : tFDConnection); overload; virtual;
                                 procedure free;

                                 private
                                        procedure setDBFile(const Value: string);


                                 protected
                                          priv_DBInit      : boolean;

                                          priv_formname    : string;
                                          priv_comment     : string;
                                          priv_version     : string;

                                          priv_FormToStore : tForm;
                                          priv_database    : tFDConnection;
                                          priv_query       : tFDQuery;
                                          priv_dbfile      : string;

                                          priv_PrefFile    : tIniFile;

                                          function getDBStructure : string; virtual;

                                          procedure WipeDatabase (scope : tWipeScope);
                                          procedure StoreProgramVersionToDB;
                                          function ConnectToDatabase : boolean;

                                          function GetDBElementID (name : string) : integer;
                                          function ElementVisualStatus(elementID : integer) : tVisualStatus;
                                          function updateDBElement (item : tControl; itemtype : integer) : integer;
                                          function IntFromBool (value : boolean) : integer;
                                          function BoolFromInt (value : integer) : boolean;

                                          procedure storeEditToDB (item : tEdit);
                                          procedure storeRadioButtonToDB (item : tRadioButton);
                                          procedure storeCheckboxToDB (item : tCheckBox);
                                          procedure storeComboboxToDB (item : tComboBox);
                                          procedure storeTreeViewToDB (item : tTreeView);
                                          procedure storeListViewToDB (item : tListView);
                                          procedure storeButtonToDB(item: tButton);
                                          procedure storeMemoToDB (item : tMemo);
                                          procedure storeListBoxToDB (item : tListbox);
                                          procedure storeLabelToDB (item : tLabel);
                                          procedure storePanelToDB (item : tPanel);
                                          procedure storeGroupBoxToDB (item : tGroupbox);
                                          procedure storeUpDownToDB (item : tUpDown);
                                          procedure storeValueListEditorToDB (item : tValueListEditor);

                                          procedure StoreAdditionalComponentsToDB (Component : TComponent); virtual;

                                          procedure retrieveEditFromDB (elementID : integer; item : tEdit);
                                          procedure retrieveRadioButtonFromDB (elementID : integer; item : tRadioButton);
                                          procedure retrieveCheckboxFromDB (elementID : integer; item : tCheckBox);
                                          procedure retrieveComboboxFromDB (elementID : integer; item : tComboBox);
                                          procedure retrieveTreeViewFromDB (elementID : integer; item : tTreeView);
                                          procedure retrieveListViewFromDB (elementID : integer; item : tListView);
                                          procedure retrieveButtonFromDB(elementID : integer; item: tButton);
                                          procedure retrieveMemoFromDB (elementID : integer; item : tMemo);
                                          procedure retrieveListBoxFromDB (elementID : integer; item : tListbox);
                                          procedure retrieveLabelFromDB (elementID : integer; item : tLabel);
                                          procedure retrievePanelFromDB (elementID : integer; item : tPanel);
                                          procedure retrieveGroupBoxFromDB (elementID : integer; item : tGroupbox);
                                          procedure retrieveUpDownFromDB (elementID : integer; item : tUpDown);
                                          procedure retrieveValueListEditorFromDB (elementID : integer; item : tValueListEditor);

                                          procedure RetrieveAdditionalComponentsFromDB (Component : TComponent); virtual;

                                 public
                                          property Database : tFDConnection read priv_database write priv_database;
                                          property DatabaseFile : string read priv_dbfile write setDBFile;
                                          property DatabaseInitialized : boolean read priv_DBInit;

                                          function storeFormToDB (DatabaseFile : string; const Comment : string = '') : integer; virtual;
                                          function retrieveFormFromDB (DatabaseFile : string) : integer; virtual;

                                          procedure storeFormToIniFile (storagefile : string);
                                          procedure retrieveFormFromIniFile (storagefile : string);

                                          {$REGION 'Documentation'}
                                          ///	<remarks>
                                          ///	  This one is public to allow
                                          ///	  reading name/value pairs from
                                          ///	  the general table on demand
                                          ///	</remarks>
                                          {$ENDREGION}
                                          procedure storeNameValueToDB (ParamName, ParamValue : string);
                                          {$REGION 'Documentation'}
                                          ///	<remarks>
                                          ///	  This one is public to allow
                                          ///	  reading name/value pairs from
                                          ///	  the general table on demand
                                          ///	</remarks>
                                          {$ENDREGION}
                                          function retrieveValueFroMDB (Name : string) : string;
                       end;

     function GetFormStorageGenerationIndex (Filename : string) : integer;

implementation

uses Toolkit;

const CR = chr(13);
      LF = chr(10);
      CRLF = CR + LF;

      IdEdit = 1;
      IdRadioButton = 2;
      IdCheckbox = 3;
      IdComboBox = 4;
      IdTreeView = 5;
      IdListBox = 6;
      IdListView = 7;
      IdMemo = 8;
      IdButton = 9;
      IdLabel = 10;
      IdPanel = 11;
      IdGroupBox = 12;
      IdVirtualTreeView = 13;
      IdUpDown = 14;
      IdValueListEditor = 15;
      IdVirtualStringTree = 16;

      newID = 0;

      cDBstruct = 'CREATE TABLE "formButtons" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL , "elementid" INTEGER NOT NULL , "caption" CHAR NOT NULL );' + CRLF +
                  'CREATE TABLE "formCheckboxes" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL , "elementid" INTEGER NOT NULL , "checked" INTEGER NOT NULL );' + CRLF +
                  'CREATE TABLE "formComboboxes" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL , "elementid" INTEGER NOT NULL , "itemcount" INTEGER NOT NULL  DEFAULT 0, "itemindex" INTEGER NOT NULL  DEFAULT 0);' + CRLF +
                  'CREATE TABLE "formComboboxitems" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL , "elementid" INTEGER NOT NULL , "itemindex" INTEGER NOT NULL , "itemvalue" VARCHAR);' + CRLF +
                  'CREATE TABLE "formEditfields" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL ,"elementid" INTEGER NOT NULL ,"value" CHAR NOT NULL );' + CRLF +
                  'CREATE TABLE "formElements" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL , "name" CHAR NOT NULL , "type" INTEGER NOT NULL , "enabled" INTEGER DEFAULT 1, "visible" INTEGER DEFAULT 1, "ignore" INTEGER DEFAULT 0);' + CRLF +
                  'CREATE TABLE "formGeneral" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL , "name" VARCHAR NOT NULL , "value" VARCHAR);' + CRLF +
                  'CREATE TABLE "formGroupboxes" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL , "elementid" INTEGER NOT NULL , "caption" VARCHAR);' + CRLF +
                  'CREATE TABLE "formLabels" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL , "elementid" INTEGER NOT NULL , "caption" CHAR NOT NULL );' + CRLF +
                  'CREATE TABLE "formListboxes" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL , "elementid" INTEGER NOT NULL , "itemcount" INTEGER NOT NULL  DEFAULT 0, "itemselected" INTEGER NOT NULL  DEFAULT 0);' + CRLF +
                  'CREATE TABLE "formListboxitems" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL , "elementid" INTEGER NOT NULL , "itemindex" INTEGER NOT NULL , "itemvalue" VARCHAR);' + CRLF +
                  'CREATE TABLE "formListviewitems" ("id" INTEGER PRIMARY KEY  NOT NULL ,"elementid" INTEGER NOT NULL ,"itemindex" INTEGER NOT NULL ,"checked" INTEGER NOT NULL  DEFAULT 0 , "rowitems" VARCHAR);' + CRLF +
                  'CREATE TABLE "formListviews" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL , "elementid" INTEGER NOT NULL , "itemcount" INTEGER NOT NULL  DEFAULT 0, "checkboxes" INTEGER NOT NULL  DEFAULT 0);' + CRLF +
                  'CREATE TABLE "formMemos" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL , "elementid" INTEGER NOT NULL , "content" TEXT);' + CRLF +
                  'CREATE TABLE "formPanels" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL , "elementid" INTEGER NOT NULL , "caption" CHAR);' + CRLF +
                  'CREATE TABLE "formRadiobuttons" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL , "elementid" INTEGER NOT NULL , "checked" INTEGER NOT NULL );' + CRLF +
                  'CREATE TABLE "formUpDowns" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL  UNIQUE , "elementid" INTEGER NOT NULL , "min" INTEGER NOT NULL  DEFAULT 0, "max" INTEGER NOT NULL  DEFAULT 100, "position" INTEGER NOT NULL  DEFAULT 0);' + CRLF +
                  'CREATE TABLE "formValueListEditorItems" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL  UNIQUE , "elementid" INTEGER NOT NULL , "itemindex" INTEGER NOT NULL , "name" VARCHAR, "value" VARCHAR);' + CRLF +
                  'CREATE TABLE "formValueListEditors" ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL  UNIQUE , "elementid" INTEGER NOT NULL , "stringcount" INTEGER NOT NULL  DEFAULT 0);' + CRLF +
                  'CREATE UNIQUE INDEX "elementname" ON "formElements" ("name" ASC, "ignore" ASC);';

(* -------------------------------------------- *)

function GetFormStorageGenerationIndex (Filename : string) : integer;

var DBCon : TFDConnection;
    Query : TFDQuery;
    value : integer;
    help  : string;

begin
     value := -1;   // fail

     // Create SQLite DB connector and Query

     DBCon := tFDConnection.Create(nil);
     DBCon.Params.Clear;
     DBCon.Params.Add('DriverID=SQLite');
     DBCon.Params.Add('Database=' + Filename);

     try
        DBCon.Open;

        Query := TFDQuery.Create(nil);
        Query.Connection := DBCon;

        Query.SQL.Text := 'SELECT * FROM formEditfields WHERE elementid=(SELECT id FROM formElements WHERE name="EditFormGenerationIndex")';
        Query.Open;

        if Query.RecordCount = 1 then
           begin
                help := Query.FieldByName ('value').AsString;

                TryStrToInt (help, value);
           end
        else
            value := 0;

     finally
            Query.Free;

            DBCon.Close;
            DBCon.Free;
     end;

     result := value;
end;

(* -------------------------------------------- *)
{ tStorageManager }
(* -------------------------------------------- *)

constructor tFormStorageManager.create (AssociatedForm : tForm);

begin
     inherited create;

     priv_FormToStore := AssociatedForm;

     // Create SQLite DB connector

     priv_database := tFDConnection.Create(nil);
     priv_database.Params.Clear;
     priv_database.Params.Add('DriverID=SQLite');

     // Prepare query for action

     priv_query := tFDQuery.Create(nil);
     priv_query.Connection := priv_database;
end;

(* -------------------------------------------- *)

constructor tFormStorageManager.create (AssociatedForm: tForm; DBConnection: tFDConnection);

begin
     inherited create;

     priv_FormToStore := AssociatedForm;

     // Create SQLite DB connector

     priv_database := DBConnection;
     priv_database.Params.Clear;
     priv_database.Params.Add('DriverID=SQLite');

     // Prepare query for action

     priv_query := tFDQuery.Create(nil);
     priv_query.Connection := priv_database;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.free;

begin
     priv_query.Free;
     priv_database.Free;

     inherited free;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.setDBFile (const Value : string);

begin
     priv_dbfile := Value;
     priv_database.Params.Clear;
     priv_database.Params.Add('DriverID=SQLite');
     priv_database.Params.Add('Database=' + priv_dbfile);
end;

(* -------------------------------------------- *)

function tFormStorageManager.IntFromBool(value: boolean): integer;

begin
     if value then
        result := 1
     else
         result := 0;
end;

(* -------------------------------------------- *)

function tFormStorageManager.BoolFromInt(value: integer): boolean;

begin
     result := (value = 1);
end;

(* -------------------------------------------- *)

function tFormStorageManager.getDBStructure: string;

begin
     result := cDBStruct;
end;

(* -------------------------------------------- *)

// This procedure clears out the database by looking for all available tables
// and then truncating them

procedure tFormStorageManager.WipeDatabase (scope : tWipeScope);

var sSQL   : string;
    sTable : string;
    i      : integer;

begin
     // Scan the database for tables

     sSQL := 'SELECT tbl_name FROM "main"."sqlite_master" WHERE type="table"';

     priv_query.SQL.Clear;
     priv_query.SQL.Add(sSQL);
     priv_query.Open;

     sSQL := '';

     // Prepare truncating statement

     for i := 1 to priv_query.RecordCount do
         begin
              sTable := priv_query.FieldByName ('tbl_name').AsString;

              // If table is a formstorage table or all should be wiped, go ahead

              if (scope = wpAll) or (pos ('form', sTable) = 1) then
                 sSQL := sSQL + 'DELETE FROM "main"."' + sTable + '";';

              priv_query.Next;
         end;

     priv_query.SQL.Clear;
     priv_query.SQL.Add (sSQL);
     priv_query.ExecSQL;

     // Reset AutoIndexes

     priv_query.SQL.Text := 'update sqlite_sequence set seq=0;';
     priv_query.ExecSQL;
end;

(* -------------------------------------------- *)

function tFormStorageManager.ConnectToDatabase : boolean;

begin
     // If we have a connection, kick it

     if priv_database.Connected then
        priv_database.Close;

     result := true;
     priv_DBInit := false;

     if not FileExists (priv_dbfile) then
        begin
             // SQlite db does not exist, try to create it

             try
                priv_database.Open;

                priv_query.SQL.Clear;
                priv_query.SQL.Text := getDBStructure;
                priv_query.ExecSQL;

                priv_DBInit := true;
             except
                   result := false;
             end;
        end
     else
         begin
              try
                 priv_database.Open;
              except
                    result := false;
              end;
         end;
end;

(* -------------------------------------------- *)

function tFormStorageManager.GetDBElementID(name: string): integer;

var sSQL : string;

begin
     sSQL := 'SELECT id FROM formElements WHERE name="' + name + '";';

     priv_Query.SQL.Clear;
     priv_query.SQL.Add (sSQL);
     priv_query.Open;

     if priv_query.RecordCount > 0 then
        result := priv_query.FieldByName ('id').AsInteger
     else
         result := newID;

     priv_query.Close;
end;

(* -------------------------------------------- *)

// Retrieves additional status information that all elements can have

function tFormStorageManager.ElementVisualStatus(elementID : integer) : tVisualStatus;

var subQuery : tFDQuery;
    sSQL     : string;

begin
     result.keepdefault := true;

     subQuery := tFDQuery.Create(nil);
     subQuery.Connection := priv_database;

     sSQL := 'SELECT * FROM formElements WHERE id=' + intToStr (elementID) + ';';

     subQuery.SQL.Clear;
     subQuery.SQL.Add(sSQL);

     subQuery.Open;

     if subQuery.RecordCount > 0 then
        begin
             result.keepdefault := false;

             result.enabled := BoolFromInt (subQuery.FieldByName('enabled').AsInteger);
             result.visible := BoolFromInt (subQuery.FieldByName('visible').AsInteger);
        end;

     subQuery.Free;
end;

(* -------------------------------------------- *)

// This function adds or updates an Element and returns the DB index of the element afterwards

function tFormStorageManager.updateDBElement(item: tControl; itemtype : integer) : integer;

var iElementID : integer;
    sVisible   : string;
    sEnabled   : string;
    sIgnore    : string;
    sItemType  : string;
    sSQL       : string;

begin
     iElementID := GetDBElementID (item.Name);
     sItemType := IntToStr (itemtype);
     sVisible := IntToStr (IntFromBool (item.Visible));
     sEnabled := IntToStr (IntFromBool (item.Enabled));
     sIgnore := IntToStr (IntFromBool (item.Tag = -1));

     // if element id exists, we have to update it, otherwise we create a new entry

     if iElementID = newID then
        sSQL := 'INSERT INTO "main"."formElements" ("name","type","enabled","visible","ignore") VALUES (''' + item.Name + ''',''' + sItemType + ''',''' + sEnabled + ''',''' + sVisible + ''',''' + sIgnore + ''');'
     else
         sSQL := 'UPDATE "main"."formElements" SET "enabled"=''' + sEnabled + ''',"visible"=''' + sVisible + ''',"ignore"=''' + sIgnore + ''' WHERE id=' + IntToStr (iElementID) + ';';

     priv_query.SQL.Clear;
     priv_query.SQL.Add(sSQL);
     priv_query.ExecSQL;

     result := iElementID;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.storeNameValueToDB (ParamName, ParamValue: string);

var sSQL : string;

begin
     sSQL := 'SELECT id FROM formGeneral WHERE name="' + ParamName + '";';

     priv_query.SQL.Clear;
     priv_query.SQL.Add(sSQL);
     priv_query.Open;

     if priv_query.RecordCount = 0 then
        sSQL := 'INSERT INTO formGeneral (name,value) VALUES (:name,:value);'
     else
         sSQL := 'UPDATE formGeneral SET value=:value WHERE name=:name;';

     priv_query.SQL.Text := sSQL;
     priv_query.ParamByName ('name').AsString := ParamName;
     priv_query.ParamByName ('value').AsString := ParamValue;
     priv_query.ExecSQL;
end;

(* -------------------------------------------- *)

function tFormStorageManager.retrieveValueFroMDB (Name: string): string;

var wasOpen : boolean;

begin
     wasOpen := priv_database.Connected;

     if not wasOpen then
        priv_database.Open;

     priv_query.SQL.Text := 'SELECT value FROM "main"."formGeneral" WHERE name=:name;';
     priv_query.ParamByName ('name').AsString := Name;
     priv_query.Open;

     if priv_query.RecordCount > 0 then
        result := priv_query.FieldByName('value').AsString
     else
         result := '';

     if not wasOpen then
        priv_database.Close;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.StoreProgramVersionToDB;

var sVersion : string;

begin
     sVersion := getExeVersion (ParamStr(0));
     priv_version := sVersion;

     storeNameValueToDB ('ExeVersion',sVersion);
end;

(* -------------------------------------------- *)

// Stores an Edit field to the database

procedure tFormStorageManager.storeEditToDB (item: tEdit);

var iElementID : integer;
    sSQL       : string;

begin
     if (item.tag >= newID) then
        begin
             iElementID := updateDBElement (item, IdEdit);

             // If we created a new element, get the ID and create a new entry

             if iElementID = newID then
                begin
                     iElementID := GetDBElementID (item.Name);

//                   sSQL := 'INSERT INTO "main"."formEditfields" ("elementid","value") VALUES (''' + IntToStr (iElementID) + ''', ''' + item.Text + ''');';

                     priv_query.SQL.Text := 'INSERT INTO "main"."formEditfields" ("elementid","value") VALUES (:elementid,:value);';
                     priv_query.ParamByName ('elementid').AsInteger := iElementID;
                     priv_query.ParamByName ('value').AsString := item.Text;
                     priv_query.ExecSQL;
                end
             else
                 begin
//                    sSQL := 'UPDATE "main"."formEditfields" SET "value"=''' + item.Text + ''' WHERE "elementid"=' + IntToStr (iElementID) + ';';

                     priv_query.SQL.Text := 'UPDATE "main"."formEditfields" SET "value"=:value WHERE "elementid"=:elementid;';
                     priv_query.ParamByName ('elementid').AsInteger := iElementID;
                     priv_query.ParamByName ('value').AsString := item.Text;
                     priv_query.ExecSQL;
                 end;
{
             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.ExecSQL;
}
        end;
end;

(* -------------------------------------------- *)

// Stores a radiobutton item to the database

procedure tFormStorageManager.storeRadioButtonToDB(item: tRadioButton);

var iElementID : integer;
    sSQL       : string;

begin
     if (item.tag >= 0) then
        begin
             iElementID := updateDBElement (item, IdRadioButton);

             // If we created a new element, get the ID and create a new entry

             if iElementID = newID then
                begin
                     iElementID := GetDBElementID (item.Name);

                     sSQL := 'INSERT INTO "main"."formRadiobuttons" ("elementid","checked") VALUES (''' + IntToStr (iElementID) + ''', ''' + IntToStr (IntFromBool (item.Checked)) + ''');';
                end
             else
                 begin
                      sSQL := 'UPDATE "main"."formRadiobuttons" SET "checked"=''' + IntToStr (IntFromBool (item.Checked)) + ''' WHERE "elementid"=' + IntToStr (iElementID) + ';';
                 end;

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.ExecSQL;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.storeCheckboxToDB(item: tCheckBox);

var iElementID : integer;
    sSQL       : string;

begin
     if (item.tag >= 0) then
        begin
             iElementID := updateDBElement (item, IdCheckBox);

             // If we created a new element, get the ID and create a new entry

             if iElementID = newID then
                begin
                     iElementID := GetDBElementID (item.Name);

                     sSQL := 'INSERT INTO "main"."formCheckboxes" ("elementid","checked") VALUES (''' + IntToStr (iElementID) + ''', ''' + IntToStr (IntFromBool (item.Checked)) + ''');';
                end
             else
                 begin
                      sSQL := 'UPDATE "main"."formCheckboxes" SET "checked"=''' + IntToStr (IntFromBool (item.Checked)) + ''' WHERE "elementid"=' + IntToStr (iElementID) + ';';
                 end;

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.ExecSQL;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.storeComboboxToDB(item: tComboBox);

var iElementID : integer;
    i          : integer;
    sSQL       : string;

begin
     if (item.tag >= 0) then
        begin
             iElementID := updateDBElement (item, IdComboBox);

             // If we created a new element, get the ID and create a new entry
             // First, we store the number of items in the combobox

             if iElementID = newID then
                begin
                     iElementID := GetDBElementID (item.Name);

                     // sSQL := 'INSERT INTO "main"."formComboboxes" ("elementid","itemcount","itemindex") VALUES (''' + IntToStr (iElementID) + ''', ''' + IntToStr (item.Items.Count) + ''', ''' + IntToStr (item.ItemIndex) + ''');';
                     sSQL := 'INSERT INTO "main"."formComboboxes" ("elementid","itemcount","itemindex") VALUES (:elementid,:itemcount,:itemindex);';
                end
             else
                 begin
                      // sSQL := 'UPDATE "main"."formComboboxes" SET "itemcount"=''' + IntToStr (item.Items.Count) + ''', "itemindex"=''' + IntToStr (item.ItemIndex) + ''' WHERE "elementid"=' + IntToStr (iElementID) + ';';

                      sSQL := 'UPDATE "main"."formComboboxes" SET "itemcount"=:itemcount, "itemindex"=:itemindex WHERE "elementid"=:elementid;';
                 end;

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);

             priv_query.ParamByName ('elementid').AsInteger := iElementID;
             priv_query.ParamByName ('itemcount').AsInteger := item.Items.Count;
             priv_query.ParamByName ('itemindex').AsInteger := item.ItemIndex;

             priv_query.ExecSQL;

             // First, we have to clear all old items the combobox may have had

             priv_query.SQL.Clear;
             // priv_query.SQL.Add ('DELETE from "main"."formComboboxitems" WHERE "elementid"=' + IntToStr (iElementID));
             priv_query.SQL.Add ('DELETE from "main"."formComboboxitems" WHERE "elementid"=:elementid;');
             priv_query.ParamByName ('elementid').AsInteger := iElementID;
             priv_query.ExecSQL;

             // Next, we store all items to the combobox item table
{
             sSQL := '';

             if item.Items.Count > 0 then
                begin
                     for i := 1 to item.Items.Count do
                         sSQL := sSQL + 'INSERT INTO "main"."formComboboxitems" ("elementid","itemindex","itemvalue") VALUES (''' + IntToStr (iElementID) + ''', ''' + intToStr (i) + ''',''' + item.Items[i-1] + ''');';

                     priv_query.SQL.Clear;
                     priv_query.SQL.Add(sSQL);
                     priv_query.ExecSQL;
                end;
}
             if item.Items.Count > 0 then
                begin
                     priv_query.SQL.Text := 'INSERT INTO "main"."formComboboxitems" ("elementid","itemindex","itemvalue") VALUES (:elementid,:itemindex,:itemvalue);';

                     priv_query.Params.ArraySize := item.Items.Count;

                     for i := 1 to item.Items.Count do
                         begin
                              priv_query.ParamByName('elementid').AsIntegers[i-1] := iElementID;
                              priv_query.ParamByName('itemindex').AsIntegers[i-1] := i;
                              priv_query.ParamByName('itemvalue').AsWideStrings[i-1] := item.Items[i-1];
                         end;

                     priv_query.Execute (item.Items.Count,0);
                end;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.storeGroupBoxToDB(item: tGroupbox);

var iElementID : integer;
    sSQL       : string;

begin
     if (item.tag > newID) then
        begin
             iElementID := updateDBElement (item, IdGroupBox);

             // If we created a new element, get the ID and create a new entry

             if iElementID = newID then
                begin
                     iElementID := GetDBElementID (item.Name);

//                   sSQL := 'INSERT INTO "main"."formGroupboxes" ("elementid","caption") VALUES (''' + IntToStr (iElementID) + ''', ''' + item.caption + ''');';
                     sSQL := 'INSERT INTO "main"."formGroupboxes" ("elementid","caption") VALUES (:elementid,:caption);';
                end
             else
                 begin
//                    sSQL := 'UPDATE "main"."formGroupboxes" SET "caption"=''' + item.Caption + ''' WHERE "elementid"=' + IntToStr (iElementID) + ';';
                      sSQL := 'UPDATE "main"."formGroupboxes" SET "caption"=:caption WHERE "elementid"=:elementid;';
                 end;

             priv_query.ParamByName ('elementid').AsInteger := iElementID;
             priv_query.ParamByName ('caption').AsWideString := item.Caption;

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.ExecSQL;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.storeLabelToDB(item: tLabel);

var iElementID : integer;
    sSQL       : string;

begin
     if (item.tag > newID) then
        begin
             iElementID := updateDBElement (item, IdLabel);

             // If we created a new element, get the ID and create a new entry

             if iElementID = newID then
                begin
                     iElementID := GetDBElementID (item.Name);

                     sSQL := 'INSERT INTO "main"."formLabels" ("elementid","caption") VALUES (''' + IntToStr (iElementID) + ''', ''' + item.caption + ''');';
                end
             else
                 begin
                      sSQL := 'UPDATE "main"."formLabels" SET "caption"=''' + item.Caption + ''' WHERE "elementid"=' + IntToStr (iElementID) + ';';
                 end;

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.ExecSQL;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.storeListBoxToDB(item: tListbox);

var iElementID : integer;
    i          : integer;
    sSQL       : string;

begin
     if (item.tag >= 0) then
        begin
             iElementID := updateDBElement (item, IdListbox);

             // If we created a new element, get the ID and create a new entry
             // First, we store the number of items in the listview

             if iElementID = newID then
                begin
                     iElementID := GetDBElementID (item.Name);

                     sSQL := 'INSERT INTO "main"."formListboxes" ("elementid","itemcount","itemselected") VALUES (''' + IntToStr (iElementID) + ''', ''' + IntToStr (item.Items.Count) + ''', ''' + IntToStr (item.ItemIndex) + ''');';
                end
             else
                 begin
                      sSQL := 'UPDATE "main"."formListboxes" SET "itemcount"=''' + IntToStr (item.Items.Count) + ''', "itemselected"=''' + IntToStr (item.ItemIndex) + ''' WHERE "elementid"=' + IntToStr (iElementID) + ';';
                 end;

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.ExecSQL;

             // Now put the items into their table

             if item.Items.Count > 0 then
                for i := 1 to item.Items.Count do
                    begin
                         // write the row into database

                         sSQL := 'INSERT INTO "main"."formListboxitems" ("elementid","itemindex","itemvalue") VALUES ("' + IntToStr (iElementID) + '","' + intToStr (i) + '","' + item.Items[i-1] + '");';

                         if sSQL <> '' then
                            begin
                                 priv_query.SQL.Clear;
                                 priv_query.SQL.Add(sSQL);
                                 priv_query.ExecSQL;
                            end;
                    end;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.storeButtonToDB(item: tButton);

var iElementID : integer;
    sSQL       : string;

begin
     if (item.tag > newID) then
        begin
             iElementID := updateDBElement (item, IdButton);

             // If we created a new element, get the ID and create a new entry

             if iElementID = newID then
                begin
                     iElementID := GetDBElementID (item.Name);

                     sSQL := 'INSERT INTO "main"."formButtons" ("elementid","caption") VALUES (''' + IntToStr (iElementID) + ''', ''' + item.caption + ''');';
                end
             else
                 begin
                      sSQL := 'UPDATE "main"."formButtons" SET "caption"=''' + item.Caption + ''' WHERE "elementid"=' + IntToStr (iElementID) + ';';
                 end;

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.ExecSQL;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.storeListViewToDB(item: tListView);

var iElementID : integer;
    i,j        : integer;
    sSQL       : string;
    sHelp      : string;

begin
     if (item.tag >= 0) then
        begin
             iElementID := updateDBElement (item, IdListview);

             // If we created a new element, get the ID and create a new entry
             // First, we store the number of items in the listview

             if iElementID = newID then
                begin
                     iElementID := GetDBElementID (item.Name);

                     sSQL := 'INSERT INTO "main"."formListviews" ("elementid","itemcount","checkboxes") VALUES (''' + IntToStr (iElementID) + ''', ''' + IntToStr (item.Items.Count) + ''', ''' + IntToStr (IntFromBool (item.Checkboxes)) + ''');';
                end
             else
                 begin
                      sSQL := 'UPDATE "main"."formListViews" SET "itemcount"=''' + IntToStr (item.Items.Count) + ''', "checkboxes"=''' + IntToStr (IntFromBool (item.Checkboxes)) + ''' WHERE "elementid"=' + IntToStr (iElementID) + ';';
                 end;

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.ExecSQL;

             // Now put the items into their table

             if item.Items.Count > 0 then
                for i := 1 to item.Items.Count do
                    begin
                         sSQL := '';

                         // Gather all row items into one string

                         sHelp := item.Items[i-1].Caption;

                         for j := 1 to item.Items[i-1].SubItems.Count do
                             sHelp := sHelp + ',' + item.Items[i-1].SubItems[j-1];

                         // write the row into database

                         sSQL := sSQL + 'INSERT INTO "main"."formListviewitems" ("elementid","itemindex","checked","rowitems") VALUES ("' + IntToStr (iElementID) + '","' + intToStr (i) + '","' + IntToStr (IntFromBool (item.Items[i-1].Checked)) + '","' + sHelp + '");';

                         if sSQL <> '' then
                            begin
                                 priv_query.SQL.Clear;
                                 priv_query.SQL.Add(sSQL);
                                 priv_query.ExecSQL;
                            end;
                    end;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.storeMemoToDB(item: tMemo);

var iElementID : integer;
    sSQL       : string;

begin
     if (item.tag >= newID) then
        begin
             iElementID := updateDBElement (item, IdMemo);

             // If we created a new element, get the ID and create a new entry

             if iElementID = newID then
                begin
                     iElementID := GetDBElementID (item.Name);

//                   sSQL := 'INSERT INTO "main"."formMemos" ("elementid","content") VALUES (''' + IntToStr (iElementID) + ''', ''' + item.Text + ''');';

                     priv_query.SQL.Text := 'INSERT INTO "main"."formMemos" ("elementid","content") VALUES (:elementid,:content);';
                     priv_query.ParamByName ('elementid').AsInteger := iElementID;
                     priv_query.ParamByName ('content').AsString := item.Text;
                     priv_query.ExecSQL;

                end
             else
                 begin
//                    sSQL := 'UPDATE "main"."formMemos" SET "content"=''' + item.Text + ''' WHERE "elementid"=' + IntToStr (iElementID) + ';';

                     priv_query.SQL.Text := 'UPDATE "main"."formMemos" SET "content"=:content WHERE "elementid"=:elementid;';
                     priv_query.ParamByName ('elementid').AsInteger := iElementID;
                     priv_query.ParamByName ('content').AsString := item.Text;
                     priv_query.ExecSQL;
                 end;
{
             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.ExecSQL;
}
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.storePanelToDB(item: tPanel);

var iElementID : integer;
    sSQL       : string;

begin
     if (item.tag > newID) then
        begin
             iElementID := updateDBElement (item, IdButton);

             // If we created a new element, get the ID and create a new entry

             if iElementID = newID then
                begin
                     iElementID := GetDBElementID (item.Name);

                     sSQL := 'INSERT INTO "main"."formPanels" ("elementid","caption") VALUES (''' + IntToStr (iElementID) + ''', ''' + item.caption + ''');';
                end
             else
                 begin
                      sSQL := 'UPDATE "main"."formPanels" SET "caption"=''' + item.Caption + ''' WHERE "elementid"=' + IntToStr (iElementID) + ';';
                 end;

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.ExecSQL;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.storeTreeViewToDB(item: tTreeView);

begin
     if (item.tag >= newID) then
        begin
             updateDBElement (item, IdTreeView);
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.storeUpDownToDB(item: tUpDown);

var iElementID : integer;
    sSQL       : string;

begin
     if (item.tag >= newID) then
        begin
             iElementID := updateDBElement (item, IdUpDown);

             // If we created a new element, get the ID and create a new entry

             if iElementID = newID then
                begin
                     iElementID := GetDBElementID (item.Name);

                     priv_query.SQL.Text := 'INSERT INTO "main"."formUpDowns" ("elementid","min","max","position") VALUES (:elementid,:min,:max,:position);';
                end
             else
                 begin
                      priv_query.SQL.Text := 'UPDATE "main"."formUpDowns" SET "min"=:min,"max"=:max,"position"=:position WHERE "elementid"=:elementid;';
                 end;

             priv_query.ParamByName ('elementid').AsInteger := iElementID;
             priv_query.ParamByName ('min').AsInteger := item.Min;
             priv_query.ParamByName ('max').AsInteger := item.Max;
             priv_query.ParamByName ('position').AsInteger := item.Position;

             priv_query.ExecSQL;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.storeValueListEditorToDB (item: tValueListEditor);

var iElementID : integer;
    sSQL       : string;
    i          : integer;

begin
     if (item.tag >= newID) then
        begin
             iElementID := updateDBElement (item, IdUpDown);

             // If we created a new element, get the ID and create a new entry

             if iElementID = newID then
                begin
                     iElementID := GetDBElementID (item.Name);

                     priv_query.SQL.Text := 'INSERT INTO "main"."formValueListEditors" ("elementid","stringcount") VALUES (:elementid,:stringcount);';
                end
             else
                 begin
                      priv_query.SQL.Text := 'UPDATE "main"."formValueListEditors" SET "stringcount"=:stringcount WHERE "elementid"=:elementid;';
                 end;

             priv_query.ParamByName ('elementid').AsInteger := iElementID;
             priv_query.ParamByName ('stringcount').AsInteger := item.Strings.Count;

             priv_query.ExecSQL;

             // Now put the items into their table

             if item.Strings.Count > 0 then
                for i := 1 to item.Strings.Count do
                    begin
                         // write the row into database

                         priv_query.SQL.Text := 'INSERT INTO "main"."formValueListEditorItems" ("elementid","itemindex","name","value") VALUES (:elementid,:itemindex,:name,:value);';
                         priv_query.ParamByName('elementid').AsInteger := iElementID;
                         priv_query.ParamByName('itemindex').AsInteger := i;
                         priv_query.ParamByName('name').AsString := (item.Strings as tStringlist).Names[i-1];
                         priv_query.ParamByName('value').AsString := (item.Strings as tStringlist).ValueFromIndex[i-1];
                         priv_query.ExecSQL;
                    end;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.StoreAdditionalComponentsToDB (Component: TComponent);

begin
     // Default: nothing here. Should be implemented in inherited classes
end;

(* -------------------------------------------- *)

function tFormStorageManager.storeFormToDB (DatabaseFile: string; const Comment : string = '') : integer;

var ComponentHandled : boolean;
    Name             : string;
    i                : integer;

begin
     result := 0;
     ComponentHandled := false;

     setDBFile (DatabaseFile);

     // Connect, and if successfull, store the stuff

     if connectToDatabase then
        begin
             // First, remove everything because its outdated anyway

             WipeDatabase (wpForm);

             // Write Version String

             StoreProgramVersionToDB;

             // Write Comment

             if Comment <> '' then
                begin
                     priv_comment := Comment;

                     storeNameValueToDB ('Comment', Comment);
                end;

             // Write Form Name

             priv_formname := priv_FormToStore.Name;
             storeNameValueToDB ('FormName', priv_formname);

             // Store all elements

             for i := 0 to priv_FormToStore.ComponentCount - 1 do
                 begin
                      Name := priv_FormToStore.Components[i].Name;

                      // tEdit

                      if priv_FormToStore.Components[i].ClassType = tEdit then
                         begin
                              storeEditToDB (priv_FormToStore.Components[i] as tEdit);
                              ComponentHandled := true;
                         end;

                      // Radiobutton

                      if priv_FormToStore.Components[i].ClassType = tRadioButton then
                         begin
                              storeRadioButtonToDB (priv_FormToStore.Components[i] as tRadioButton);
                              ComponentHandled := true;
                         end;

                      // Checkbox

                      if priv_FormToStore.Components[i].ClassType = tCheckBox then
                         begin
                              storeCheckboxToDB (priv_FormToStore.Components[i] as tCheckBox);
                              ComponentHandled := true;
                         end;

                      // ComboBox

                      if priv_FormToStore.Components[i].ClassType = tComboBox then
                         begin
                              storeComboboxToDB (priv_FormToStore.Components[i] as tComboBox);
                              ComponentHandled := true;
                         end;

                      // TreeView

                      if priv_FormToStore.Components[i].ClassType = tTreeView then
                         begin
                              storeTreeViewToDB (priv_FormToStore.Components[i] as tTreeView);
                              ComponentHandled := true;
                         end;

                      // ListView

                      if priv_FormToStore.Components[i].ClassType = tListView then
                         begin
                              storeListViewToDB (priv_FormToStore.Components[i] as tListView);
                              ComponentHandled := true;
                         end;

                      // Memo

                      if priv_FormToStore.Components[i].ClassType = tMemo then
                         begin
                              storeMemoToDB (priv_FormToStore.Components[i] as tMemo);
                              ComponentHandled := true;
                         end;

                      // Listbox

                      if priv_FormToStore.Components[i].ClassType = tListbox then
                         begin
                              storeListBoxToDB (priv_FormToStore.Components[i] as tListBox);
                              ComponentHandled := true;
                         end;

                      // Button

                      if priv_FormToStore.Components[i].ClassType = tButton then
                         begin
                              storeButtonToDB (priv_FormToStore.Components[i] as tButton);
                              ComponentHandled := true;
                         end;

                      // Label

                      if priv_FormToStore.Components[i].ClassType = tLabel then
                         begin
                              storeLabelToDB (priv_FormToStore.Components[i] as tLabel);
                              ComponentHandled := true;
                         end;

                      // Panel

                      if priv_FormToStore.Components[i].ClassType = tPanel then
                         begin
                              storePanelToDB (priv_FormToStore.Components[i] as tPanel);
                              ComponentHandled := true;
                         end;

                      // Groupbox

                      if priv_FormToStore.Components[i].ClassType = tGroupBox then
                         begin
                              storeGroupBoxToDB (priv_FormToStore.Components[i] as tGroupBox);
                              ComponentHandled := true;
                         end;

                      // UpDown

                      if priv_FormToStore.Components[i].ClassType = tUpDown then
                         begin
                              storeUpDownToDB (priv_FormToStore.Components[i] as tUpDown);
                              ComponentHandled := true;
                         end;

                      // ValueListEditor

                      if priv_FormToStore.Components[i].ClassType = tValueListEditor then
                         begin
                              storeValueListEditorToDB (priv_FormToStore.Components[i] as tValueListEditor);
                              ComponentHandled := true;
                         end;

                      if not ComponentHandled then
                         StoreAdditionalComponentsToDB (priv_FormToStore.Components[i]);
                 end;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.retrieveButtonFromDB(elementID : integer; item: tButton);

var sSQL       : string;

begin
     if (item.tag > newID) then
        begin
             sSQL := 'SELECT value FROM "main"."formButtons" WHERE elementid=' + IntToStr (ElementID) + ';';

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.Open;

             if priv_query.RecordCount > 0 then
                item.caption := priv_query.FieldByName ('caption').AsString;
        end;
end;


(* -------------------------------------------- *)

procedure tFormStorageManager.retrieveCheckboxFromDB(elementID : integer; item: tCheckBox);

var sSQL       : string;

begin
     if (item.tag >= 0) then
        begin
             sSQL := 'SELECT checked FROM "main"."formCheckboxes" WHERE elementid=' + IntToStr (ElementID) + ';';

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.Open;

             if priv_query.RecordCount > 0 then
                item.Checked := BoolFromInt (priv_query.FieldByName ('checked').AsInteger);
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.retrieveComboboxFromDB(elementID : integer; item: tComboBox);

var iItemIndex : integer;
    i          : integer;
    sSQL       : string;

begin
     if (item.tag >= 0) then
        begin
             // First, we get the item that is selected

             sSQL := 'SELECT itemindex FROM "main"."formComboboxes" WHERE elementid=' + IntToStr (ElementID) + ';';

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.Open;

             if priv_query.RecordCount > 0 then
                iItemIndex := priv_query.FieldByName ('itemindex').AsInteger
             else
                 iItemIndex := -1;

             // Now, lets get all items and add them

             item.Items.Clear;

             sSQL := 'SELECT itemvalue FROM "main"."formComboboxitems" WHERE elementid=' + IntToStr (ElementID) + ' ORDER BY itemindex ASC;';

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.Open;

             for i := 1 to priv_query.RecordCount do
                 begin
                      item.Items.Add(priv_query.FieldByName ('itemvalue').AsString);

                      priv_query.Next;
                 end;

             item.ItemIndex := iItemIndex;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.retrieveEditFromDB(elementID : integer; item: tEdit);

var sSQL       : string;

begin
     if (item.tag >= newID) then
        begin
             sSQL := 'SELECT value FROM "main"."formEditfields" WHERE elementid=' + IntToStr (ElementID) + ';';

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.Open;

             if priv_query.RecordCount > 0 then
                item.Text := priv_query.FieldByName ('value').AsString;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.retrieveGroupBoxFromDB(elementID : integer; item: tGroupbox);

var sSQL       : string;

begin
     if (item.tag > newID) then
        begin
             sSQL := 'SELECT caption FROM "main"."formGroupboxes" WHERE elementid=' + IntToStr (ElementID) + ';';

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.Open;

             if priv_query.RecordCount > 0 then
                item.caption := priv_query.FieldByName ('caption').AsString;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.retrieveLabelFromDB(elementID : integer; item: tLabel);

var sSQL       : string;

begin
     if (item.tag > newID) then
        begin
             sSQL := 'SELECT value FROM "main"."formLabels" WHERE elementid=' + IntToStr (ElementID) + ';';

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.Open;

             if priv_query.RecordCount > 0 then
                item.caption := priv_query.FieldByName ('caption').AsString;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.retrieveListBoxFromDB(elementID : integer; item: tListbox);

var iItemIndex : integer;
    i          : integer;
    sSQL       : string;

begin
     if (item.tag >= 0) then
        begin
             // First, we get the item that is selected

             sSQL := 'SELECT itemselected FROM "main"."formListboxes" WHERE elementid=' + IntToStr (ElementID) + ';';

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.Open;

             if priv_query.RecordCount > 0 then
                iItemIndex := priv_query.FieldByName ('itemselected').AsInteger
             else
                 iItemIndex := -1;

             // Now, lets get all items and add them

             item.Items.Clear;

             sSQL := 'SELECT itemvalue FROM "main"."formListboxitems" WHERE elementid=' + IntToStr (ElementID) + ' ORDER BY itemindex ASC;';

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.Open;

             for i := 1 to priv_query.RecordCount do
                 begin
                      item.Items.Add(priv_query.FieldByName ('itemvalue').AsString);

                      priv_query.Next;
                 end;

             item.ItemIndex := iItemIndex;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.retrieveListViewFromDB(elementID : integer; item: tListView);

var iColCount    : integer;
    iRowCount    : integer;
    i,j          : integer;
    sSQL         : string;
    boCheckboxes : boolean;
    listitem     : tListItem;
    sHelp        : string;

begin
     if (item.tag >= 0) then
        begin
             // First, we get the item that is selected

             sSQL := 'SELECT * FROM "main"."formListviews" WHERE elementid=' + IntToStr (ElementID) + ';';

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.Open;

             if priv_query.RecordCount > 0 then
                begin
                     iRowCount := priv_query.FieldByName ('itemcount').AsInteger;
                     boCheckboxes := BoolFromInt (priv_query.FieldByName ('checkboxes').AsInteger);
                end
             else
                 begin
                      iRowCount := 0;
                      boCheckboxes := false;
                 end;

             // Now, lets get all items and add them

             item.Clear;

             if iRowCount > 0 then
                begin
                     sSQL := 'SELECT * FROM "main"."formListviewitems" WHERE elementid=' + IntToStr (ElementID) + ' ORDER BY itemindex ASC;';

                     priv_query.SQL.Clear;
                     priv_query.SQL.Add(sSQL);
                     priv_query.Open;

                     for i := 1 to priv_query.RecordCount do
                         begin
                              sHelp := priv_query.fieldByName ('rowitems').AsString;
                              iColCount := numEntries (sHelp, ',');

                              listitem := item.Items.Add;
                              listitem.Caption := getEntry (sHelp, ',', 1);

                              // Add subitems

                              for j := 2 to iColCount do
                                  listitem.SubItems.Add(getEntry (sHelp, ',', j));

                              if boCheckboxes then
                                 ListItem.Checked := (priv_query.fieldByname ('checked').AsInteger = 1);

                              priv_query.Next;
                         end;
                end;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.retrieveMemoFromDB(elementID : integer; item: tMemo);

var sSQL       : string;

begin
     if (item.tag >= newID) then
        begin
             sSQL := 'SELECT content FROM "main"."formMemos" WHERE elementid=' + IntToStr (ElementID) + ';';

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.Open;

             if priv_query.RecordCount > 0 then
                item.Text := priv_query.FieldByName ('content').AsString;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.retrievePanelFromDB(elementID : integer; item: tPanel);

var sSQL       : string;

begin
     if (item.tag > newID) then
        begin
             sSQL := 'SELECT value FROM "main"."formPanels" WHERE elementid=' + IntToStr (ElementID) + ';';

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.Open;

             if priv_query.RecordCount > 0 then
                item.caption := priv_query.FieldByName ('caption').AsString;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.retrieveRadioButtonFromDB(elementID : integer; item: tRadioButton);

var sSQL : string;

begin
     if (item.tag >= 0) then
        begin
             sSQL := 'SELECT checked FROM "main"."formRadiobuttons" WHERE elementid=' + IntToStr (ElementID) + ';';

             priv_query.SQL.Clear;
             priv_query.SQL.Add(sSQL);
             priv_query.Open;

             if priv_query.RecordCount > 0 then
                item.Checked := BoolFromInt (priv_query.FieldByName ('checked').AsInteger);
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.retrieveTreeViewFromDB(elementID : integer; item: tTreeView);

begin
     // Not done yet
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.retrieveUpDownFromDB(elementID: integer; item: tUpDown);

begin
     if (item.tag >= 0) then
        begin
             priv_query.SQL.Text := 'SELECT * FROM "main"."formUpDowns" WHERE elementid=:elementid;';
             priv_query.ParamByName ('elementid').AsInteger := elementID;
             priv_query.Open;

             if priv_query.RecordCount > 0 then
                begin
                     item.Min := priv_query.FieldByName('min').AsInteger;
                     item.Max := priv_query.FieldByName('max').AsInteger;
                     item.Position := priv_query.FieldByName('position').AsInteger;
                end;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.retrieveValueListEditorFromDB (elementID: integer; item: tValueListEditor);

var name  : string;
    value : string;
    index : integer;
    i     : integer;

begin
     if (item.tag >= 0) then
        begin
//           item.Strings.Clear;

             priv_query.SQL.Text := 'SELECT * FROM "main"."formValueListEditors" WHERE elementid=:elementid;';
             priv_query.ParamByName ('elementid').AsInteger := elementID;
             priv_query.Open;
             priv_query.FetchAll;

             if priv_query.RecordCount > 0 then
                begin
                     // Get Items

                     priv_query.SQL.Text := 'SELECT * FROM "main"."formValueListEditorItems" WHERE elementid=:elementid ORDER BY itemindex ASC;';
                     priv_query.ParamByName ('elementid').AsInteger := elementID;
                     priv_query.Open;
                     priv_query.FetchAll;

                     for i := 1 to priv_query.RecordCount do
                         begin
                              name := priv_query.FieldByName('name').AsString;
                              value := priv_query.FieldByName('value').AsString;

                              // Find the key and assign the value if found

                              index := item.Strings.IndexOfName(name);

                              if index > -1 then
                                 item.Strings.ValueFromIndex[index] := value
                              else
                                  item.Strings.Add(name + '=' + value);

                              priv_query.Next;
                         end;
                end;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.RetrieveAdditionalComponentsFromDB (Component: TComponent);

begin
     // Default: nothing here. Should be implemented in inherited classes
end;

(* -------------------------------------------- *)

function tFormStorageManager.retrieveFormFromDB (DatabaseFile: string) : integer;

var iElementID       : integer;
    i                : integer;
    vStatus          : tVisualStatus;
    ComponentHandled : boolean;

begin
     result := 0;
     ComponentHandled := false;

     setDBFile (DatabaseFile);

     // Connect, and if successfull, retrieve the stuff, but only if the
     // DB wasn't just created (because then we would read empty stuff and
     // clear out our form, and that would be... less than optimal.

     if connectToDatabase and not priv_DBInit then
        begin
             // Get Version and Comment

             priv_version := retrieveValueFromDB ('ExeVersion');
             priv_formname := retrieveValueFromDB ('FormName');
             priv_comment := retrieveValueFromDB ('Comment');

             for i := 0 to priv_FormToStore.ComponentCount - 1 do
                 begin
                      iElementID := GetDBElementID (priv_FormToStore.Components[i].Name);

                      if iElementID > 0 then
                         begin
                              vStatus := ElementVisualStatus(iElementID);

                              // tEdit

                              if priv_FormToStore.Components[i].ClassType = tEdit then
                                 begin
                                      if not vStatus.keepdefault then
                                         begin
                                              (priv_FormToStore.Components[i] as tEdit).Enabled := vStatus.enabled;
                                              (priv_FormToStore.Components[i] as tEdit).Visible := vStatus.visible;
                                         end;

                                      retrieveEditFromDB (iElementID, priv_FormToStore.Components[i] as tEdit);
                                      ComponentHandled := true;
                                 end;

                              // Radiobutton

                              if priv_FormToStore.Components[i].ClassType = tRadioButton then
                                 begin
                                      if not vStatus.keepdefault then
                                         begin
                                              (priv_FormToStore.Components[i] as tRadioButton).Enabled := vStatus.enabled;
                                              (priv_FormToStore.Components[i] as tRadioButton).Visible := vStatus.visible;
                                         end;

                                      retrieveRadioButtonFromDB (iElementID, priv_FormToStore.Components[i] as tRadioButton);
                                      ComponentHandled := true;
                                 end;

                              // Checkbox

                              if priv_FormToStore.Components[i].ClassType = tCheckBox then
                                 begin
                                      if not vStatus.keepdefault then
                                         begin
                                              (priv_FormToStore.Components[i] as tCheckBox).Enabled := vStatus.enabled;
                                              (priv_FormToStore.Components[i] as tCheckBox).Visible := vStatus.visible;
                                         end;

                                      retrieveCheckboxFromDB (iElementID, priv_FormToStore.Components[i] as tCheckBox);
                                      ComponentHandled := true;
                                 end;

                              // ComboBox

                              if priv_FormToStore.Components[i].ClassType = tComboBox then
                                 begin
                                      if not vStatus.keepdefault then
                                         begin
                                              (priv_FormToStore.Components[i] as tComboBox).Enabled := vStatus.enabled;
                                              (priv_FormToStore.Components[i] as tComboBox).Visible := vStatus.visible;
                                         end;

                                      retrieveComboboxFromDB (iElementID, priv_FormToStore.Components[i] as tComboBox);
                                      ComponentHandled := true;
                                 end;

                              // TreeView

                              if priv_FormToStore.Components[i].ClassType = tTreeView then
                                 begin
                                      if not vStatus.keepdefault then
                                         begin
                                              (priv_FormToStore.Components[i] as tTreeView).Enabled := vStatus.enabled;
                                              (priv_FormToStore.Components[i] as tTreeView).Visible := vStatus.visible;
                                         end;

                                      retrieveTreeViewFromDB (iElementID, priv_FormToStore.Components[i] as tTreeView);
                                      ComponentHandled := true;
                                 end;

                              // ListView

                              if priv_FormToStore.Components[i].ClassType = tListView then
                                 begin
                                      if not vStatus.keepdefault then
                                         begin
                                              (priv_FormToStore.Components[i] as tListView).Enabled := vStatus.enabled;
                                              (priv_FormToStore.Components[i] as tListView).Visible := vStatus.visible;
                                         end;

                                      retrieveListViewFromDB (iElementID, priv_FormToStore.Components[i] as tListView);
                                      ComponentHandled := true;
                                 end;

                              // Memo

                              if priv_FormToStore.Components[i].ClassType = tMemo then
                                 begin
                                      if not vStatus.keepdefault then
                                         begin
                                              (priv_FormToStore.Components[i] as tMemo).Enabled := vStatus.enabled;
                                              (priv_FormToStore.Components[i] as tMemo).Visible := vStatus.visible;
                                         end;

                                      retrieveMemoFromDB (iElementID, priv_FormToStore.Components[i] as tMemo);
                                      ComponentHandled := true;
                                 end;

                              // Listbox

                              if priv_FormToStore.Components[i].ClassType = tListbox then
                                 begin
                                      if not vStatus.keepdefault then
                                         begin
                                              (priv_FormToStore.Components[i] as tListbox).Enabled := vStatus.enabled;
                                              (priv_FormToStore.Components[i] as tListbox).Visible := vStatus.visible;
                                         end;

                                      retrieveListBoxFromDB (iElementID, priv_FormToStore.Components[i] as tListBox);
                                      ComponentHandled := true;
                                 end;

                              // Button

                              if priv_FormToStore.Components[i].ClassType = tButton then
                                 begin
                                      if not vStatus.keepdefault then
                                         begin
                                              (priv_FormToStore.Components[i] as tButton).Enabled := vStatus.enabled;
                                              (priv_FormToStore.Components[i] as tButton).Visible := vStatus.visible;
                                         end;

                                      retrieveButtonFromDB (iElementID, priv_FormToStore.Components[i] as tButton);
                                      ComponentHandled := true;
                                 end;

                              // Label

                              if priv_FormToStore.Components[i].ClassType = tLabel then
                                 begin
                                      if not vStatus.keepdefault then
                                         begin
                                              (priv_FormToStore.Components[i] as tLabel).Enabled := vStatus.enabled;
                                              (priv_FormToStore.Components[i] as tLabel).Visible := vStatus.visible;
                                         end;

                                      retrieveLabelFromDB (iElementID, priv_FormToStore.Components[i] as tLabel);
                                      ComponentHandled := true;
                                 end;

                              // Panel

                              if priv_FormToStore.Components[i].ClassType = tPanel then
                                 begin
                                      if not vStatus.keepdefault then
                                         begin
                                              (priv_FormToStore.Components[i] as tPanel).Enabled := vStatus.enabled;
                                              (priv_FormToStore.Components[i] as tPanel).Visible := vStatus.visible;
                                         end;

                                      retrievePanelFromDB (iElementID, priv_FormToStore.Components[i] as tPanel);
                                      ComponentHandled := true;
                                 end;

                              // Groupbox

                              if priv_FormToStore.Components[i].ClassType = tGroupBox then
                                 begin
                                      if not vStatus.keepdefault then
                                         begin
                                              (priv_FormToStore.Components[i] as tGroupBox).Enabled := vStatus.enabled;
                                              (priv_FormToStore.Components[i] as tGroupBox).Visible := vStatus.visible;
                                         end;

                                      retrieveGroupBoxFromDB (iElementID, priv_FormToStore.Components[i] as tGroupBox);
                                      ComponentHandled := true;
                                 end;

                              // UpDown

                              if priv_FormToStore.Components[i].ClassType = tUpDown then
                                 begin
                                      if not vStatus.keepdefault then
                                         begin
                                              (priv_FormToStore.Components[i] as tUpDown).Enabled := vStatus.enabled;
                                              (priv_FormToStore.Components[i] as tUpDown).Visible := vStatus.visible;
                                         end;

                                      retrieveUpDownFromDB (iElementID, priv_FormToStore.Components[i] as tUpDown);
                                      ComponentHandled := true;
                                 end;

                              // ValueListEditors

                              if priv_FormToStore.Components[i].ClassType = tValueListEditor then
                                 begin
                                      if not vStatus.keepdefault then
                                         begin
                                              (priv_FormToStore.Components[i] as tValueListEditor).Enabled := vStatus.enabled;
                                              (priv_FormToStore.Components[i] as tValueListEditor).Visible := vStatus.visible;
                                         end;

                                      retrieveValueListEditorFromDB (iElementID, priv_FormToStore.Components[i] as tValueListEditor);
                                      ComponentHandled := true;
                                 end;

                              // Handle if not already done and event assigned

                              if not ComponentHandled then
                                 RetrieveAdditionalComponentsFromDB (priv_FormToStore.Components[i]);
                         end;
                 end;
        end;
end;

(* -------------------------------------------- *)

procedure tFormStorageManager.storeFormToIniFile(storagefile: string);

var i,j,k        : integer;
    helpEdit     : tEdit;
    helpUpDown   : tUpDown;
    helpRadio    : tRadioButton;
    helpCheck    : tCheckBox;
    helpCombo    : tComboBox;
    helpTree     : tTreeView;
    helpMemo     : tMemo;
    helpButton   : tButton;
    helpLabel    : tLabel;
    helpList     : tListView;
    helpPanel    : tPanel;
    helpGroup    : tGroupBox;
    helpListbox  : tListBox;
    helpDTPicker : tDateTimePicker;
    ListCount    : integer;
    ListBoxCount : integer;
    TreeCount    : integer;
    ComboCount   : integer;
    sHelp        : string;

begin
     try
        if fileexists (StorageFile) then
           deletefile (StorageFile);
     except

     end;

     priv_PrefFile := tIniFile.Create(StorageFile);

     ListCount := 0;
     TreeCount := 0;
     ComboCount := 0;
     ListBoxCount := 0;

     for i := 0 to priv_FormToStore.ComponentCount - 1 do
         if not (priv_FormToStore.Components[i].Tag = -1) then
            begin
                 if priv_FormToStore.Components[i].ClassType = tEdit then
                    begin
                         helpEdit := priv_FormToStore.Components[i] as tEdit;

                         priv_PrefFile.WriteString ('EditFields', helpEdit.Name, helpEdit.Text);
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tUpDown then
                    begin
                         helpUpDown := priv_FormToStore.Components[i] as tUpDown;

                         priv_PrefFile.WriteInteger ('UpDownSelectors', helpUpDown.Name, helpUpDown.position);
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tRadioButton then
                    begin
                         helpRadio := priv_FormToStore.Components[i] as tRadioButton;

                         priv_PrefFile.WriteBool ('RadioButtons', helpRadio.Name, helpRadio.Checked);
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tCheckBox then
                    begin
                         helpCheck := priv_FormToStore.Components[i] as tCheckBox;

                         priv_PrefFile.WriteBool('CheckBoxes', helpCheck.Name, helpCheck.Checked);
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tComboBox then
                    begin
                         inc (ComboCount);

                         helpCombo := priv_FormToStore.Components[i] as tComboBox;

                         priv_PrefFile.WriteString ('ComboBoxes', 'ComboBox' + IntToStr (ComboCount), helpCombo.Name);

                         priv_PrefFile.WriteInteger('ComboBoxListIndex', helpCombo.Name , ComboCount);
                         priv_PrefFile.WriteString ('ComboBox' + IntToStr (ComboCount), 'Value', helpCombo.Text);
                         priv_PrefFile.WriteInteger('ComboBox' + IntToStr (ComboCount), 'ItemIndex', helpCombo.ItemIndex);
                         priv_PrefFile.WriteInteger('ComboBox' + IntToStr (ComboCount), 'ItemCount', helpCombo.Items.Count);

                         for j := 1 to helpCombo.Items.Count do
                             begin
                                  sHelp := helpCombo.Items[j-1];

                                  priv_PrefFile.WriteString ('ComboBox' + IntToStr (ComboCount), 'Item' + IntToStr (j), sHelp);
                             end;
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tTreeView then
                    begin
                         helpTree := priv_FormToStore.Components[i] as tTreeView;
                         inc (TreeCount);

                         priv_PrefFile.WriteString('TreeViews', 'Tree' + IntToStr (TreeCount), helpTree.Name);
                         priv_PrefFile.WriteInteger('TreeListIndex', helpTree.Name, TreeCount);

                         priv_PrefFile.WriteInteger('TreeView' + IntToStr (TreeCount), 'ItemCount', helpTree.Items.Count);

                         for j := 1 to helpTree.Items.Count do
                             begin
                                  sHelp := helpTree.Items[j-1].Text;
                                  sHelp := sHelp + ',' + IntToStr (helpTree.Items[j-1].Level);

                                  priv_PrefFile.WriteString ('TreeView' + IntToStr (TreeCount), 'Item' + IntToStr(j), sHelp);
                             end;
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tListView then
                    begin
                         helpList := priv_FormToStore.Components[i] as tListView;
                         inc (ListCount);

                         priv_PrefFile.WriteString('ListViews', 'List' + IntToStr (ListCount), helpList.Name);
                         priv_PrefFile.WriteInteger( 'ListViewIndex', helpList.Name,ListCount);

                         priv_PrefFile.WriteInteger('ListView' + IntToStr (ListCount), 'ItemCount', helpList.Items.Count);
                         priv_PrefFile.WriteBool('ListView' + IntToStr (ListCount), 'CheckBoxes', helpList.Checkboxes);

                         // Saving the items will only work if list is not set to readonly mode

                         for j := 1 to helpList.Items.Count do
                             begin
                                  sHelp := helpList.Items[j-1].Caption;

                                  for k := 1 to helpList.Items[j-1].SubItems.Count do
                                      sHelp := sHelp + ',' + helpList.Items[j-1].SubItems[k-1];

                                  if helpList.Checkboxes then
                                     if helpList.Items[j-1].Checked then
                                        sHelp := sHelp + ',true'
                                     else
                                         sHelp := sHelp + ',false';

                                  priv_PrefFile.WriteString('ListView' + IntToStr (ListCount), 'Line' + IntToStr (j), sHelp);
                             end;
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tMemo then
                    begin
                         helpMemo := priv_FormToStore.Components[i] as tMemo;
                         sHelp := stringReplace (helpMemo.Text, CR+LF, '|', [rfReplaceAll]);

                         priv_PrefFile.WriteString('Memos', helpMemo.Name, sHelp);
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tListbox then
                    begin
                         helpListbox := priv_FormToStore.Components[i] as tListBox;
                         inc (ListBoxCount);

                         priv_PrefFile.WriteString('ListBoxes', 'ListBox' + IntToStr (ListBoxCount), helpListbox.Name);
                         priv_PrefFile.WriteInteger( 'ListBoxIndex', helpListbox.Name, ListBoxCount);

                         priv_PrefFile.WriteInteger('ListBox' + IntToStr (ListBoxCount), 'ItemCount', helpListbox.Items.Count);

                         // Saving the items will only work if list is not set to readonly mode

                         for j := 1 to helpListbox.Items.Count do
                             begin
                                  sHelp := helpListBox.Items[j-1];

                                  priv_PrefFile.WriteString('ListBox' + IntToStr (ListCount), 'Line' + IntToStr (j), sHelp);
                             end;
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tButton then
                    begin
                         helpButton := priv_FormToStore.Components[i] as tButton;

                         if helpButton.Tag = 1 then
                            priv_PrefFile.WriteString('Buttons', helpButton.Name, helpButton.Caption);
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tLabel then
                    begin
                         helpLabel := priv_FormToStore.Components[i] as tLabel;

                         if helpLabel.Tag = 1 then
                            priv_PrefFile.WriteString('Labels', helpLabel.Name, helpLabel.Caption);
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tPanel then
                    begin
                         helpPanel := priv_FormToStore.Components[i] as tPanel;

                         if helpPanel.Tag = 1 then
                            priv_PrefFile.WriteString('Panels', helpPanel.Name, helpPanel.Caption);
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tGroupBox then
                    begin
                         helpGroup := priv_FormToStore.Components[i] as tGroupBox;

                         if helpGroup.Tag = 1 then
                            priv_PrefFile.WriteString('GroupBoxes', helpGroup.Name, helpGroup.Caption);
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tDateTimePicker then
                    begin
                         helpDTPicker := priv_FormToStore.Components[i] as tDateTimePicker;

                         if helpDTPicker.Tag = 1 then
                            priv_PrefFile.WriteDateTime('DateTimePicker', helpDTPicker.Name, helpDTPicker.DateTime);
                    end
                 else
                     sHelp := 'Unknown Component: "' + priv_FormToStore.Components[i].ClassName + '"';
            end;

     priv_PrefFile.Free;
end;

(* -------------------------------------------- *)

// Gets saved values for all form items if not tag = -1
// Exceptions are tbuttons, tlabel, tgroupbox and tpanel - they are never
// loaded except if their tag is 1.

procedure tFormStorageManager.retrieveFormFromIniFile(storagefile: string);

var i,j,k        : integer;
    helpEdit     : tEdit;
    helpRadio    : tRadioButton;
    helpCheck    : tCheckBox;
    helpCombo    : tComboBox;
    helpTree     : tTreeView;
    helpMemo     : tMemo;
    helpList     : tListView;
    helpButton   : tButton;
    helpLabel    : tLabel;
    helpPanel    : tPanel;
    helpGroup    : tGroupBox;
    helpDTPicker : tDateTimePicker;
    helpListbox  : tListBox;
    helpUpDown   : tUpDown;
    sHelp        : string;
    sHelp2       : string;
    iHelp        : integer;
    iItems       : integer;
    boHelp       : boolean;
    iCount       : integer;
    listItem     : tListItem;

begin
     if not FileExists (StorageFile) then
        exit;

     priv_PrefFile := tIniFile.Create(StorageFile);

     for i := 0 to priv_FormToStore.ComponentCount - 1 do
         if not (priv_FormToStore.Components[i].Tag = -1) then
            begin
                 if priv_FormToStore.Components[i].ClassType = tEdit then
                    begin
                         helpEdit := priv_FormToStore.Components[i] as tEdit;

                         if priv_PrefFile.ValueExists ('EditFields', helpEdit.Name) then
                            helpEdit.Text := priv_PrefFile.ReadString ('EditFields', helpEdit.Name, '');
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tUpDown then
                    begin
                         helpUpDown := priv_FormToStore.Components[i] as tUpDown;

                         helpUpDown.Position := priv_PrefFile.ReadInteger('UpDownSelectors', helpUpDown.Name, helpUpDown.Min);
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tRadioButton then
                    begin
                         helpRadio := priv_FormToStore.Components[i] as tRadioButton;

                         if priv_PrefFile.ValueExists ('RadioButtons', helpRadio.Name) then
                            helpRadio.Checked := priv_PrefFile.ReadBool('RadioButtons', helpRadio.Name, false);
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tCheckBox then
                    begin
                         helpCheck := priv_FormToStore.Components[i] as tCheckBox;

                         if priv_PrefFile.ValueExists ('CheckBoxes', helpCheck.Name) then
                            helpCheck.Checked := priv_PrefFile.ReadBool('CheckBoxes', helpCheck.Name, false);
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tComboBox then
                    begin
                         helpCombo := priv_FormToStore.Components[i] as tComboBox;

                         if priv_PrefFile.ValueExists ('ComboBoxListIndex', helpCombo.Name) then
                            begin
                                 helpCombo.Items.Clear;

                                 iHelp := priv_PrefFile.ReadInteger('ComboBoxListIndex', helpCombo.Name, 0);
                                 iCount := priv_PrefFile.ReadInteger('ComboBox' + IntToStr (iHelp), 'ItemCount', 0);
                                 helpCombo.Items.Capacity := iCount;

                                 for j := 1 to iCount do
                                     begin
                                          sHelp := priv_PrefFile.ReadString ('ComboBox' + IntToStr (iHelp), 'Item' + IntToStr(j), '');
                                          helpCombo.Items.Add(sHelp);
                                     end;

                                 helpCombo.Text := priv_PrefFile.ReadString ('ComboBox' + IntToStr (iHelp), 'Value', '');
                                 helpCombo.ItemIndex := priv_PrefFile.ReadInteger('ComboBox' + IntToStr (iHelp), 'ItemIndex', -1);
                            end;
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tTreeView then
                    begin
                         helpTree := priv_FormToStore.Components[i] as tTreeView;

                         if priv_PrefFile.ValueExists ('TreeListIndex', helpTree.Name) then
                            begin
                                 helpTree.Items.Clear;

                                 iHelp := priv_PrefFile.ReadInteger('TreeListIndex', helpTree.Name, 0);
                                 iCount := priv_PrefFile.ReadInteger('TreeView' + IntToStr (iHelp), 'ItemCount', 0);

                                 for j := 1 to iCount do
                                     begin
                                          sHelp := priv_PrefFile.ReadString('TreeView' + IntToStr (iHelp), 'Item' + IntToStr (j), '');

                                          // TODO: need to reestablish the Tree items
                                     end;
                            end;
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tListView then
                    begin
                         helpList := priv_FormToStore.Components[i] as tListView;

                         if priv_PrefFile.ValueExists ('ListViewIndex', helpList.Name) then
                            begin
                                 helpList.Items.Clear;

                                 iHelp := priv_PrefFile.ReadInteger('ListViewIndex', helpList.Name, 0);
                                 iCount := priv_PrefFile.ReadInteger('ListView' + IntToStr (iHelp), 'ItemCount', 0);
                                 boHelp := priv_PrefFile.ReadBool('ListView' + IntToStr (iHelp), 'CheckBoxes', true);

                                 for j := 1 to iCount do
                                     begin
                                          sHelp := priv_PrefFile.ReadString('ListView' + IntToStr (iHelp), 'Line' + IntToStr (j), '');

                                          iItems := NumEntries (sHelp, ',');

                                          ListItem := helpList.Items.Add;
                                          ListItem.Caption := getEntry (sHelp, ',', 1);

                                          if boHelp then
                                             begin
                                                  for k := 2 to (iItems - 1) do
                                                      ListItem.SubItems.Add (getEntry (sHelp, ',', k));

                                                  sHelp2 := getEntry (sHelp, ',', iItems);

                                                  ListItem.Checked := sHelp2 = 'true';
                                             end
                                          else
                                              begin
                                                   for k := 2 to iItems do
                                                       ListItem.SubItems.Add (getEntry (sHelp, ',', k));
                                              end;
                                     end;
                            end;
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tListBox then
                    begin
                         helpListBox := priv_FormToStore.Components[i] as tListBox;

                         if priv_PrefFile.ValueExists ('ListBoxIndex', helpListBox.Name) then
                            begin
                                 helpListBox.Items.Clear;

                                 iHelp := priv_PrefFile.ReadInteger('ListBoxIndex', helpListBox.Name, 0);
                                 iCount := priv_PrefFile.ReadInteger('ListBox' + IntToStr (iHelp), 'ItemCount', 0);

                                 for j := 1 to iCount do
                                     begin
                                          sHelp := priv_PrefFile.ReadString('ListBox' + IntToStr (iHelp), 'Line' + IntToStr (j), '');

                                          helpListBox.Items.Add(sHelp);
                                     end;
                            end;
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tMemo then
                    begin
                         helpMemo := priv_FormToStore.Components[i] as tMemo;

                         sHelp := priv_PrefFile.ReadString('Memos', helpMemo.Name, '');
                         sHelp := stringReplace (sHelp, '|', CR+LF, [rfReplaceAll]);

                         helpMemo.Text := sHelp;
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tButton then
                    begin
                         helpButton := priv_FormToStore.Components[i] as tButton;

                         if helpButton.Tag = 1 then
                            helpButton.Caption := priv_PrefFile.ReadString('Buttons', helpButton.Caption, '');
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tLabel then
                    begin
                         helpLabel := priv_FormToStore.Components[i] as tLabel;

                         if helpLabel.Tag = 1 then
                            helpLabel.Caption := priv_PrefFile.ReadString('Labels', helpLabel.Caption, '');
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tPanel then
                    begin
                         helpPanel := priv_FormToStore.Components[i] as tPanel;

                         if helpPanel.Tag = 1 then
                            helpPanel.Caption := priv_PrefFile.ReadString('Panels', helpPanel.Caption, '');
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tGroupBox then
                    begin
                         helpGroup := priv_FormToStore.Components[i] as tGroupBox;

                         if helpGroup.Tag = 1 then
                            helpGroup.Caption := priv_PrefFile.ReadString('GroupBoxes', helpGroup.Caption, '');
                    end
                 else
                 if priv_FormToStore.Components[i].ClassType = tDateTimePicker then
                    begin
                         helpDTPicker := priv_FormToStore.Components[i] as tDateTimePicker;

                         if helpDTPicker.Tag = 1 then
                            helpDTPicker.DateTime := priv_PrefFile.ReadDateTime('DateTimePicker', helpDTPicker.Name, now);
                    end
                 else
                     sHelp := 'Unknown Component: "' + priv_FormToStore.Components[i].ClassName + '"';
            end;

     priv_PrefFile.Free;
end;

(* -------------------------------------------- *)


end.
