{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit definition_anonymization_struct;

interface

uses base_structures,
     definition_ip_addresses,
     class_inteldb_manager;

type tPayloadRemovalMode = (prmUnknown, prmCut, prmZero, prmString);
     tSanitizeCommentsScope = (scsUnknown, scsNone, scsFile, scsPacket, scsBoth);
     tSanitizeCommentsMode = (scmUnknown, scmNone, scmRemoveAll, scmAsk);
     tSanitizeIHBMode = (simUnknown, simNone, simReplaceGeneric, simSetToBlank, simAsk);
     tSanitizeCaptureFilterMode = (cfmUnknown, cfmNone, cfmRemoveFilter, cfmKeepFilter);
     tSanitizeNRBMode = (snmUnknown, snmNone, snmReplaceIPs, snmRemoveBlock, snmAsk);
     tVLANRemoveMode = (vrmUnknown, vrmNone, vrmRemoveAll, vrmRemoveAllListed, vrmRemoveAllUnlisted);
     tPrivateAddressRandomMode = (parUnknown, parRandomizeToPrivate, parRandomizeFullRange, parKeepOriginal);
     tRecalculateCRCMode = (rcmUnknown, rcmKeepCRC, rcmKeepBadCRC, rcmFixZeroCRC, rcmFixAll);
     tIPv6RandomizationMode = (iprUnknown, iprNone, iprRandomizeAndKeepPrefix, iprRandomizeBothAndKeepPrefix, iprFullStatelessRandomization, iprBlackMarker);
     tAYIYAIdentSanitization = (aisUnknown, aisNone, aisZero);
     tAYIYASignatureSanitization = (assUnknown, assNone, assZero);
     tDHCPv4ReplaceHostNameMode = (rhnUnknown, rhnKeepHostname, rhnSetToZeroString, rhnUsePlaceholders);

     tBoolParam = (bpUnknown, bpTrue, bpFalse);

     tMACPairList = array of tMACAddressPair;
     tVLANPairList = array of tWordPair;
     tIPv4AddressList = array of tIPv4AddressPair;
     tIPv6AddressList = array of tIPv6AddressPair;
     tICMPv4TypeCodeList = array of tICMPv4TypeCode;
     tPortList = array of tPortPair;
//   tSocketPortList = array of tSocketPortPair;
     tSocketPortList = array of tIPSocketPair;
     tVLANRemoveList = array [0..4096] of boolean;
     tFQDNList = array of tFQDNPair;

     tAnonymizeParameters = record
                                  // File Details

                                  TracefileListIndex               : integer;                                 // Helper to keep track of the current file
                                  KeepOriginalApplicationString    : boolean;                                 // for PCAPng Application strings
                                  OverwriteExistingTarget          : boolean;                                 // set from general preferences (advanced settings)

                                  // Payload

                                  PayloadTruncateUnknownLayers     : boolean;
                                  PayloadRemovalMode               : tPayloadRemovalMode;
                                  PayloadOverwritePattern          : ANSIstring;
                                  PayloadReAddPaddingIfSliced      : boolean;

                                  PayloadDoTruncateAfterLayer      : boolean;
                                  PayloadTruncateAfterLayer        : byte;
                                  PayloadDoTruncateAtOffset        : boolean;
                                  PayloadTruncateAtOffset          : word;
                                  PayLoadDoReplaceStrings          : boolean;
                                  PayLoadDoReplaceCaseInsensitve   : boolean;
                                  PayloadDoNotEnforceSameLength    : boolean;
                                  PayloadReplacementStrings        : array of tANSIStringPair;

                                  // PCAPNG

                                  PCAPngDoReplace                  : boolean;
                                  PCAPngSanitizeCommentsScope      : tSanitizeCommentsScope;
                                  PCAPngSanitizeCommentsMode       : tSanitizeCommentsMode;
                                  PCAPngIHBDoReplace               : boolean;
                                  PCAPngSanitizeIHBInterfaceName   : tSanitizeIHBMode;
                                  PCAPngSanitizeIHBCaptureFilter   : tSanitizeCaptureFilterMode;
                                  PCAPngSanitizeNRB                : tSanitizeNRBMode;
                                  PCAPngSanitizePacketOptions      : boolean;

                                  // 802.11

                                  WiFiDoReplace                    : boolean;
                                  WiFiReplaceBySSIDList            : boolean;
                                  WiFiReplaceRemainingSSIDs        : boolean;
                                  WiFiReplaceRemainingSSIDsPrefix  : string;
                                  WiFiKeepUnknownTaggedParams      : boolean;
                                  WiFiSSIDReplacementStrings       : array of tANSIStringPair;

                                  // Ethernet

                                  EthernetDoReplace                : boolean;
                                  EthernetDoOnlyDBBasedReplacement : boolean;                                 // internally used, not configurable (mostly for constistent IPv6 replacements for EUI addresses)
                                  EthernetReplaceByMACList         : boolean;
                                  EthernetMACList                  : tMACPairList;
                                  EthernetRandomizeMACAddress      : boolean;
                                  EthernetBlackMarkerMACAddress    : boolean;
                                  EthernetLeaveVendorIntact        : boolean;
                                  EthernetRecalculateFCSMode       : tRecalculateCRCMode;
                                  EthernetPreserveTrailingBytes    : boolean;

                                  // VLAN

                                  VLANDoReplace                    : boolean;
                                  VLANDoReplaceVLANByIDList        : boolean;
                                  VLANReplaceByVLANIDList          : tVLANPairList;
                                  VLANRemoveMode                   : tVLANRemoveMode;
                                  VLANRemoveList                   : tVLANRemoveList;
                                  VLANRandomizeVLANIDs             : boolean;

                                  // Tunneling

                                  TunnelDoReplace                  : boolean;
                                  TunnelAYIYASanitizeIdentity      : tAYIYAIdentSanitization;
                                  TunnelAYIYASanitizeSignature     : tAYIYASignatureSanitization;
                                  MPLSRandomizeLabel               : boolean;
                                  MPLSMaximumLabelValue            : longword;

                                  // IPv4

                                  IPv4DoReplace                    : boolean;
                                  IPv4DoReplaceByIPList            : boolean;
                                  IPv4ReplaceByIPList              : boolean;
                                  IPv4AddressList                  : tIPv4AddressList;
                                  IPV4AddressAutoMode              : boolean;
                                  IPv4ReplaceBySubnetlist          : boolean;
                                  IPv4KeepHostPart                 : boolean;
                                  IPv4SubnetList                   : tIPv4AddressList;     // Good enough since every IP address has a mask bit count
                                  IPv4RandomizeIPAddress           : boolean;
                                  IPv4BlackMarkerIPAddress         : boolean;
                                  IPv4RandomizeMulticast           : boolean;
                                  IPv4RandomizeAPIPA               : boolean;
                                  IPv4RandomizeDocumentation       : boolean;
                                  IPv4PrivateAddressRandomizeMode  : tPrivateAddressRandomMode;
                                  IPv4RandomizeIPID                : boolean;
                                  IPv4RecalculateCRCMode           : tRecalculateCRCMode;
                                  IPV4StoreGeneratedSubnetIPsToDB  : boolean;

                                  // IPv6

                                  IPv6DoReplace                    : boolean;
                                  IPv6DoReplaceByIPList            : boolean;
                                  IPv6AddressList                  : tIPv6AddressList;
                                  IPV6AddressAutoMode              : boolean;
                                  IPv6ReplaceBySubnetlist          : boolean;
                                  IPv6KeepHostPart                 : boolean;
                                  IPv6SubnetList                   : tIPv6AddressList;     // Good enough since every IP address has a mask bit count
                                  IPv6RandomizeIPAddress           : boolean;
//                                IPv6BlackMarkerIPAddress         : boolean;              now in randomizeaddressmode
                                  IPv6RandomizeAddressMode         : tIPv6RandomizationMode;
                                  IPv6RandomizeDocumentationAddr   : boolean;
                                  IPV6StoreGeneratedSubnetIPsToDB  : boolean;

                                  // ICMPv4

                                  ICMPv4DoReplace                  : boolean;
                                  ICMPv4RemovePingPayload          : boolean;
                                  ICMPv4RemoveICMPTypeCodes        : boolean;
                                  ICMPv4RemoveByTypeCode           : tICMPv4TypeCodeList;
                                  ICMPv4RecalculateCRCMode         : tRecalculateCRCMode;

                                  // ICMPv6

                                  ICMPv6DoReplace                  : boolean;
                                  ICMPv6RemovePingPayload          : boolean;
                                  ICMPv6RecalculateCRCMode         : tRecalculateCRCMode;
                                  ICMPv6RemoveDNSFQDNsFromOptions  : boolean;

                                  // TCP

                                  TCPDoReplace                     : boolean;
                                  TCPReplaceSocketsByList          : boolean;
                                  TCPSocketPortList                : tSocketPortList;
                                  TCPReplacePortsByList            : boolean;
                                  TCPPortList                      : tPortList;
                                  TCPRandomizePorts                : boolean;
                                  TCPIgnoreSystemPorts             : boolean;
                                  TCPRecalculateCRC                : tRecalculateCRCMode;

                                  // UDP

                                  UDPDoReplace                     : boolean;
                                  UDPReplaceSocketsByList          : boolean;
                                  UDPSocketPortList                : tSocketPortList;
                                  UDPReplacePortsByList            : boolean;
                                  UDPPortList                      : tPortList;
                                  UDPRandomizePorts                : boolean;
                                  UDPIgnoreSystemPorts             : boolean;
                                  UDPRecalculateCRC                : tRecalculateCRCMode;

                                  // DHCPv4

                                  DHCPv4DoReplace                  : boolean;
                                  DHCPv4ReplaceHostName            : boolean;
                                  DHCPv4ReplaceHostnameMode        : tDHCPv4ReplaceHostNameMode;
                                  DHCPv4HostnameList               : tFQDNList;

                                  // Application

                                  RTPSDoReplace                    : boolean;
                                  RTPSReplacementBlackMarker       : boolean;

                                  // Output (used for text based config files as of now only)

                                  OutputTargetLocation             : tLocationType;
                                  OutputPath                       : string;
                                  OutputFilename                   : string;
                                  OutputTimestamp                  : tFileTimestampMode;

                                  // Settings

//                                ReplacementDBFile                : string;
//                                ReplacementDBCreateInMemory      : boolean;
                                  ReplacementDBUseDiskFile         : boolean;
                                  ReplacementDBKeepDiskFile        : boolean;
                                  ReplacementDBWipeOnStart         : boolean;
                                  ReplacementDBForcePath           : boolean;
                                  ReplacementDBFilePath            : string;

                                  // Advanced

                                  AdvDoNotReassembleIPFragments    : boolean;
                                  AdvReassemblyChainTimeoutMS      : integer;

                                  // Helper Functions

                                  /// <summary>
                                  ///   Tells if the sanitized frame payload is
                                  ///   shortened by any setting, e.g. removing
                                  ///   unknown layers by slicing, layer
                                  ///   truncate or others.
                                  /// </summary>
                                  function ShortenPayload : boolean;

                                  procedure initialize;
                                  function SaveToConfigFile (const Filename : string; const StripComments : boolean) : boolean;
                                  function LoadFromConfigFile (const Filename : string) : string;
                            end;

     function getBool (const value : string) : tBoolParam;

     function TargetLocationToString (const value : tLocationType) : string;
     function TargetTimestampToString (const value : tFileTimestampMode) : string;
     function PayloadRemovalModeToString (const value : tPayloadRemovalMode) : string;
     function SanitizeCommentsScopeToString (const value : tSanitizeCommentsScope) : string;
     function SanitizeCommentsModeToString (const value : tSanitizeCommentsMode) : string;
     function SanitizeIHBModeToString (const value : tSanitizeIHBMode) : string;
     function SanitizeIHBCaptureFilterModeToString (const value : tSanitizeCaptureFilterMode) : string;
     function SanitizeNRBModeToString (const value : tSanitizeNRBMode) : string;
     function VLANRemoveModeToString (const value : tVLANRemoveMode) : string;
     function PrivateAddressRandomModeToString (const value : tPrivateAddressRandomMode) : string;
     function RecalculateCRCModeToString (const value : tRecalculateCRCMode) : string;
     function IPv6RandomizationModeToString (const value : tIPv6RandomizationMode) : string;
     function AYIYAIdentSanitizationToString (const value : tAYIYAIdentSanitization) : string;
     function AYIYASignatureSanitizationToString (const value : tAYIYASignatureSanitization) : string;
     function DHCPv4ReplaceHostNameModeToString (const value : tDHCPv4ReplaceHostNameMode) : string;

     function StringToTargetLocation (const value : string) : tLocationType;
     function StringToTargetTimestamp (const value : string) : tFileTimestampMode;
     function StringToPayloadRemovalMode (const value : string) : tPayloadRemovalMode;
     function StringToSanitizeCommentsScope (const value : string) : tSanitizeCommentsScope;
     function StringToSanitizeCommentsMode (const value : string) : tSanitizeCommentsMode;
     function StringToSanitizeIHBMode (const value : string) : tSanitizeIHBMode;
     function StringToSanitizeIHBCaptureFilterMode (const value : string) : tSanitizeCaptureFilterMode;
     function StringToSanitizeNRBMode (const value : string) : tSanitizeNRBMode;
     function StringToVLANRemoveMode (const value : string) : tVLANRemoveMode;
     function StringPrivateAddressRandomMode (const value : string) : tPrivateAddressRandomMode;
     function StringToRecalculateCRCMode (const value : string) : tRecalculateCRCMode;
     function StringToIPv6RandomizationMode (value : string) : tIPv6RandomizationMode;
     function StringToAYIYAIdentSanitization (const value : string) : tAYIYAIdentSanitization;
     function StringToAYIYASignatureSanitization (const value : string) : tAYIYASignatureSanitization;
     function StringToDHCPv4ReplaceHostNameMode (const value : string) : tDHCPv4ReplaceHostNameMode;


implementation

uses SysUtils,
     Types,
     Classes,
     StrUtils,
     toolkit;

const // Read tokens

      // File Details

      cTracefileListIndex = 1;
      cKeepOriginalApplicationString = 2;
      cOverwriteExistingTarget = 3;

      // DB Settings

      cReplacementMgrDBFile = 11;
      cCreateDBInMemory = 12;

      // Payload

      cPayloadTruncateUnknownLayers = 21;
      cPayloadRemovalMode = 22;
      cPayloadOverwritePattern = 23;
      cPayloadReAddPaddingIfSliced = 24;

      cPayloadDoTruncateAfterLayer = 25;
      cPayloadTruncateAfterLayer = 26;
      cPayloadDoTruncateAtOffset = 27;
      cPayloadTruncateAtOffset = 28;
      cPayLoadDoReplaceStrings  = 29;
      cPayLoadDoReplaceCaseInsensitve = 30;
      cPayloadDoNotEnforceSameLength = 31;
      cPayloadReplacementStrings = 32;

      // PCAPNG

      cPCAPngDoReplace = 41;
      cPCAPngSanitizeCommentsScope = 42;
      cPCAPngSanitizeCommentsMode = 43;
      cPCAPngIHBDoReplace = 44;
      cPCAPngSanitizeIHBInterfaceName = 45;
      cPCAPngSanitizeIHBCaptureFilter = 46;
      cPCAPngSanitizeNRB = 47;
      cPCAPngSanitizePacketOptions = 48;

      // 802.11

      cWiFiDoReplace = 61;
      cWiFiReplaceBySSIDList = 62;
      cWiFiReplaceRemainingSSIDs = 63;
      cWiFiReplaceRemainingSSIDsPrefix = 64;
      cWiFiKeepUnknownTaggedParams = 65;
      cWiFiSSIDReplacementStrings = 66;

      // Ethernet

      cEthernetDoReplace = 71;
      cEthernetDoOnlyDBBasedReplacement = 72;
      cEthernetReplaceByMACList = 73;
      cEthernetMACList = 74;
      cEthernetRandomizeMACAddress = 75;
      cEthernetBlackMarkerMACAddress = 76;
      cEthernetLeaveVendorIntact = 77;
      cEthernetRecalculateFCSMode = 78;
      cEthernetPreserveTrailingBytes = 79;

      // VLAN

      cVLANDoReplace = 91;
      cVLANDoReplaceVLANByIDList = 92;
      cVLANReplaceByVLANIDList = 93;
      cVLANRemoveMode = 94;
      cVLANRemoveList = 95;
      cVLANRandomizeVLANIDs = 96;

      // IPv4

      cIPv4DoReplace = 111;
      cIPv4DoReplaceByIPList = 112;
      cIPv4ReplaceByIPList = 113;
      cIPv4AddressList = 114;
      cIPv4ReplaceBySubnetlist = 115;
      cIPv4KeepHostPart = 116;
      cIPv4SubnetList = 117;
      cIPv4RandomizeIPAddress = 118;
      cIPv4BlackMarkerIPAddress = 119;
      cIPv4RandomizeMulticast = 120;
      cIPv4RandomizeAPIPA = 121;
      cIPv4RandomizeDocumentation = 122;
      cIPv4PrivateAddressRandomizeMode = 123;
      cIPv4RandomizeIPID = 124;
      cIPv4RecalculateCRCMode = 126;
      cIPV4StoreGeneratedSubnetIPsToDB = 127;

      // IPv6

      cIPv6DoReplace = 141;
      cIPv6DoReplaceByIPList = 142;
      cIPv6AddressList = 143;
      cIPv6ReplaceBySubnetlist = 144;
      cIPv6KeepHostPart = 145;
      cIPv6SubnetList = 146;
      cIPv6RandomizeIPAddress = 147;
      cIPv6BlackMarkerIPAddress = 148;
      cIPv6RandomizeAddressMode = 149;
      cIPv6RandomizeDocumentationAddr = 150;
      cIPV6StoreGeneratedSubnetIPsToDB = 151;
      cIPv64RecalculateCRCMode = 152;

      // ICMPv4

      cICMPv4DoReplace = 161;
      cICMPv4RemovePingPayload = 162;
      cICMPv4RemoveByTypeCode = 163;
      cICMPv4RecalculateCRCMode = 164;

      // ICMPv6

      cICMPv6DoReplace = 171;
      cICMPv6RemovePingPayload = 172;
      cICMPv6RecalculateCRCMode = 173;
      cICMPv6RemoveDNSFQDNsFromOptions = 174;

      // TCP

      cTCPDoReplace = 181;
      cTCPReplaceSocketsByList = 182;
      cTCPSocketPortList = 183;
      cTCPReplacePortsByList = 184;
      cTCPPortList = 185;
      cTCPRandomizePorts = 186;
      cTCPIgnoreSystemPorts = 187;
      cTCPRecalculateCRC = 188;

      // UDP

      cUDPDoReplace = 201;
      cUDPReplaceSocketsByList = 202;
      cUDPSocketPortList = 203;
      cUDPReplacePortsByList = 204;
      cUDPPortList = 205;
      cUDPRandomizePorts = 206;
      cUDPIgnoreSystemPorts = 207;
      cUDPRecalculateCRC = 208;

      // Tunneling

      cTunnelDoReplace = 221;
      cTunnelAYIYASanitizeIdentity = 222;
      cTunnelAYIYASanitizeSignature = 223;

      cMPLSRandomizeLabel = 224;
      cMPLSMaximumLabelValue = 225;

      // DHCPv4

      cDHCPv4DoReplace = 241;
      cDHCPv4ReplaceHostName = 242;
      cDHCPv4ReplaceHostnameMode = 243;
      cDHCPv4HostnameList = 244;

      // Application

      cRTPSDoReplace = 251;
      cRTPSReplacementBlackMarker = 252;

      // Settings

      cReplacementDBUseDiskFile = 261;
      cReplacementDBKeepDiskFile = 262;
      cReplacementDBWipeOnStart = 263;
      cReplacementDBForcePath = 264;
      cReplacementDBFilePath = 265;

      // Advanced

      cAdvDoNotReassembleIPFragments = 271;
      cAdvReassemblyChainTimeoutMS = 272;

      // Presets

      cDefaultPayloadPattern = 'Payload removed by TraceWrangler!';
      cDefaultWiFiSSIDPrefix = 'Redacted SSID #';
      cDefaultRelativePath = 'Anonymized\';
      cDefaultOutputFilename = '<filename>_anon';

(* ------------------------------------------- *)

function getBool (const value : string) : tBoolParam;

begin
     if lowercase (value) = 'true' then
        result := bpTrue
     else
         if lowercase (value) = 'false' then
            result := bpFalse
         else
             result := bpUnknown;
end;

(* ------------------------------------------- *)

function TargetLocationToString (const value : tLocationType) : string;

begin
     case value of
      ltSameDirectory     : result := 'SamePath';
      ltSubdirectory      : result := 'SubPath';
      ltAbsoluteDirectory : result := 'AbsolutePath';
     end;
end;

(* ------------------------------------------- *)

function TargetTimestampToString (const value : tFileTimestampMode) : string;

begin
     case value of
      tsmNone         : result := 'KeepAsIs';
      tsmOriginalFile : result := 'Original';
      tsmFirstFrame   : result := 'FirstFrame';
      tsmLastFrame    : result := 'LastFrame';
     end;
end;

(* ------------------------------------------- *)

function PayloadRemovalModeToString (const value : tPayloadRemovalMode) : string;

begin
     case value of
      prmCut    : result := 'Cut';
      prmZero   : result := 'Zero';
      prmString : result := 'String';
     end;
end;

(* ------------------------------------------- *)

function SanitizeCommentsScopeToString (const value : tSanitizeCommentsScope) : string;

begin
     case value of
      scsNone   : result := 'None';
      scsFile   : result := 'File';
      scsPacket : result := 'Packet';
      scsBoth   : result := 'Both';
     end;
end;

(* ------------------------------------------- *)

function SanitizeCommentsModeToString (const value : tSanitizeCommentsMode) : string;

begin
     case value of
      scmNone      : result := 'None';
      scmRemoveAll : result := 'RemoveAll';
      scmAsk       : result := 'Ask';
     end;
end;

(* ------------------------------------------- *)

function SanitizeIHBModeToString (const value : tSanitizeIHBMode) : string;

begin
     case value of
      simNone           : result := 'None';
      simReplaceGeneric : result := 'GenerateName';             // was ReplaceGeneric
      simSetToBlank     : result := 'SetToBlank';
      simAsk            : result := 'Ask';
     end;
end;

(* ------------------------------------------- *)

function SanitizeIHBCaptureFilterModeToString (const value : tSanitizeCaptureFilterMode) : string;

begin
     case value of
      cfmNone         : result := 'None';
      cfmRemoveFilter : result := 'RemoveFilter';
      cfmKeepFilter   : result := 'KeepAsIs';
     end;
end;

(* ------------------------------------------- *)

function SanitizeNRBModeToString (const value : tSanitizeNRBMode) : string;

begin
     case value of
      snmNone        : result := 'None';
      snmReplaceIPs  : result := 'ReplaceIPs';
      snmRemoveBlock : result := 'RemoveBlock';
      snmAsk         : result := 'Ask';
     end;
end;

(* ------------------------------------------- *)

function VLANRemoveModeToString (const value : tVLANRemoveMode) : string;

begin
     case value of
      vrmNone              : result := 'None';
      vrmRemoveAll         : result := 'RemoveAll';
      vrmRemoveAllListed   : result := 'RemoveAllListed';
      vrmRemoveAllUnlisted : result := 'RemoveAllUnlisted';
     end;
end;

(* ------------------------------------------- *)

function PrivateAddressRandomModeToString (const value : tPrivateAddressRandomMode) : string;

begin
     case value of
      parRandomizeToPrivate : result := 'RandomizeToPrivate';
      parRandomizeFullRange : result := 'RandomizeFullRange';
      parKeepOriginal       : result := 'KeepOriginal';
     end;
end;

(* ------------------------------------------- *)

function RecalculateCRCModeToString (const value : tRecalculateCRCMode) : string;

begin
     case value of
      rcmKeepCRC    : result := 'KeepCRC';
      rcmKeepBadCRC : result := 'KeepBadCRC';
      rcmFixZeroCRC : result := 'FixZeroCRC';
      rcmFixAll     : result := 'FixAll';
     end;
end;

(* ------------------------------------------- *)

function IPv6RandomizationModeToString (const value : tIPv6RandomizationMode) : string;

begin
     case value of
      iprNone                       : result := 'None';
      iprRandomizeAndKeepPrefix     : result := 'RandomizeAndKeepPrefix';
      iprRandomizeBothAndKeepPrefix : result := 'RandomizeBothAndKeepPrefix';
      iprFullStatelessRandomization : result := 'FullStatelessRandomization';
      iprBlackMarker                : result := 'BlackMarker';
     end;
end;

(* ------------------------------------------- *)

function AYIYAIdentSanitizationToString (const value : tAYIYAIdentSanitization) : string;

begin
     case value of
      aisNone : result := 'None';
      aisZero : result := 'Zero';
     end;
end;

(* ------------------------------------------- *)

function AYIYASignatureSanitizationToString (const value : tAYIYASignatureSanitization) : string;

begin
     case value of
      assNone : result := 'None';
      assZero : result := 'Zero';
     end;
end;

(* ------------------------------------------- *)

function DHCPv4ReplaceHostNameModeToString (const value : tDHCPv4ReplaceHostNameMode) : string;

begin
     case value of
      rhnSetToZeroString : result := 'SetToZeroString';
      rhnUsePlaceholders : result := 'UsePlaceHolders';
     end;
end;

(* ------------------------------------------- *)

function StringToTargetLocation (const value : string) : tLocationType;

begin
     if lowercase (value) = 'samepath' then
        result := ltSameDirectory
     else
     if lowercase (value) = 'subpath' then
        result := ltSubDirectory
     else
     if lowercase (value) = 'absolutepath' then
        result := ltAbsoluteDirectory
     else
     result := ltUnknown;
end;

(* ------------------------------------------- *)

function StringToTargetTimestamp (const value : string) : tFileTimestampMode;

begin
     if lowercase (value) = 'keepasis' then
        result := tsmNone
     else
     if lowercase (value) = 'original' then
        result := tsmOriginalFile
     else
     if lowercase (value) = 'firstframe' then
        result := tsmFirstFrame
     else
     if lowercase (value) = 'lastframe' then
        result := tsmLastFrame
     else
     result := tsmUnknown;
end;

(* ------------------------------------------- *)

function StringToPayloadRemovalMode (const value : string) : tPayloadRemovalMode;

begin
     if lowercase (value) = 'cut' then
        result := prmCut
     else
     if lowercase (value) = 'zero' then
        result := prmZero
     else
     if lowercase (value) = 'string' then
        result := prmString
     else
         result := prmUnknown;
end;

(* ------------------------------------------- *)

function StringToSanitizeCommentsScope (const value : string) : tSanitizeCommentsScope;

begin
     if lowercase (value) = 'none' then
        result := scsNone
     else
     if lowercase (value) = 'file' then
        result := scsfile
     else
     if lowercase (value) = 'packet' then
        result := scsPacket
     else
     if lowercase (value) = 'both' then
        result := scsBoth
     else
         result := scsUnknown;
end;

(* ------------------------------------------- *)

function StringToSanitizeCommentsMode (const value : string) : tSanitizeCommentsMode;

begin
     if lowercase (value) = 'removeall' then
        result := scmRemoveAll
     else
     if lowercase (value) = 'ask' then
        result := scmAsk
     else
         result := scmUnknown;
end;

(* ------------------------------------------- *)

function StringToSanitizeIHBMode (const value : string) : tSanitizeIHBMode;

begin
     if lowercase (value) = 'none' then
        result := simNone
     else
     if (lowercase (value) = 'replacegeneric') or (lowercase (value) = 'generatename') then         // both, because replacegeneric is deprecated but may appear in old files
        result := simReplaceGeneric
     else
     if lowercase (value) = 'settoblank' then
        result := simSetToBlank
     else
     if lowercase (value) = 'ask' then
        result := simAsk
     else
         result := simUnknown;
end;

(* ------------------------------------------- *)

function StringToSanitizeIHBCaptureFilterMode (const value : string) : tSanitizeCaptureFilterMode;

begin
     if lowercase (value) = 'none' then
        result := cfmNone
     else
     if lowercase (value) = 'removefilter' then
        result := cfmRemoveFilter
     else
     if lowercase (value) = 'KeepAsIs' then
        result := cfmKeepFilter
     else
         result := cfmUnknown;
end;

(* ------------------------------------------- *)

function StringToSanitizeNRBMode (const value : string) : tSanitizeNRBMode;

begin
     if lowercase (value) = 'none' then
        result := snmNone
     else
     if lowercase (value) = 'replaceips' then
        result := snmReplaceIPs
     else
     if lowercase (value) = 'removeblock' then
        result := snmRemoveBlock
     else
     if lowercase (value) = 'ask' then
        result := snmAsk
     else
         result := snmUnknown;
end;

(* ------------------------------------------- *)

function StringToVLANRemoveMode (const value : string) : tVLANRemoveMode;

begin
     if lowercase (value) = 'none' then
        result := vrmNone
     else
     if lowercase (value) = 'removeall' then
        result := vrmRemoveAll
     else
     if lowercase (value) = 'removealllisted' then
        result := vrmRemoveAllListed
     else
     if lowercase (value) = 'removeallunlisted' then
        result := vrmRemoveAllUnlisted
     else
         result := vrmUnknown;
end;

(* ------------------------------------------- *)

function StringPrivateAddressRandomMode (const value : string) : tPrivateAddressRandomMode;

begin
     if lowercase (value) = 'randomizetoprivate' then
        result := parRandomizeToPrivate
     else
     if lowercase (value) = 'randomizefullrange' then
        result := parRandomizeFullRange
     else
     if lowercase (value) = 'keeporiginal' then
        result := parKeepOriginal
     else
         result := parUnknown;
end;

(* ------------------------------------------- *)

function StringToRecalculateCRCMode (const value : string) : tRecalculateCRCMode;

begin
     if lowercase (value) = 'keepcrc' then
        result := rcmKeepCRC
     else
     if lowercase (value) = 'keepbadcrc' then
        result := rcmKeepBadCRC
     else
     if lowercase (value) = 'fixzerocrc' then
        result := rcmFixZeroCRC
     else
     if lowercase (value) = 'fixall' then
        result := rcmFixAll
     else
         result := rcmUnknown;
end;

(* ------------------------------------------- *)

function StringToIPv6RandomizationMode (value : string) : tIPv6RandomizationMode;

begin
     if (lowercase (value) = 'none') or (lowercase (value) = 'keepasis') then
        result := iprNone
     else
     if lowercase (value) = 'randomizeandkeepprefix' then
        result := iprRandomizeAndKeepPrefix
     else
     if lowercase (value) = 'randomizebothandkeepprefix' then
        result := iprRandomizeBothAndKeepPrefix
     else
     if lowercase (value) = 'fullstatelessrandomization' then
        result := iprFullStatelessRandomization
     else
     if lowercase (value) = 'BlackMarker' then
        result := iprBlackMarker
     else
         result := iprUnknown;
end;

(* ------------------------------------------- *)

function StringToAYIYAIdentSanitization (const value : string) : tAYIYAIdentSanitization;

begin
     if lowercase (value) = 'none' then
        result := aisNone
     else
     if lowercase (value) = 'zero' then
        result := aisZero
     else
         result := aisUnknown;
end;

(* ------------------------------------------- *)

function StringToAYIYASignatureSanitization (const value : string) : tAYIYASignatureSanitization;

begin
     if lowercase (value) = 'none' then
        result := assNone
     else
     if lowercase (value) = 'zero' then
        result := assZero
     else
         result := assUnknown;
end;

(* ------------------------------------------- *)

function StringToDHCPv4ReplaceHostNameMode (const value : string) : tDHCPv4ReplaceHostNameMode;

begin
     if lowercase (value) = 'setztozerostring' then
        result := rhnSetToZeroString
     else
     if lowercase (value) = 'useplaceholders' then
        result := rhnUsePlaceholders
     else
         result := rhnUnknown;
end;

(* ------------------------------------------- *)

function AnonTokenToParameterName (const TokenID : integer) : string;

begin
     case TokenID of
//    cTracefileListIndex               : result := '';
      cKeepOriginalApplicationString    : result := 'KeepOriginalApplicationString';
      cOverwriteExistingTarget          : result := 'OverwriteExistingTarget';

      // DB Settings

      cReplacementMgrDBFile             : result := 'ReplacementMgrDBFile';
      cCreateDBInMemory                 : result := 'reateDBInMemory';

      // Payload

      cPayloadTruncateUnknownLayers     : result := 'PayloadTruncateUnknownLayers';
      cPayloadRemovalMode               : result := 'PayloadRemovalMode';
      cPayloadOverwritePattern          : result := 'PayloadOverwritePattern';
      cPayloadReAddPaddingIfSliced      : result := 'PayloadReAddPaddingIfSliced';

      cPayloadDoTruncateAfterLayer      : result := 'PayloadDoTruncateAfterLayer';
      cPayloadTruncateAfterLayer        : result := 'PayloadTruncateAfterLayer';
      cPayloadDoTruncateAtOffset        : result := 'PayloadDoTruncateAtOffset';
      cPayloadTruncateAtOffset          : result := 'PayloadTruncateAtOffset';
      cPayLoadDoReplaceStrings          : result := 'PayLoadDoReplaceStrings';
      cPayLoadDoReplaceCaseInsensitve   : result := 'PayLoadDoReplaceCaseInsensitve';
      cPayloadDoNotEnforceSameLength    : result := 'PayloadDoNotEnforceSameLength';
      cPayloadReplacementStrings        : result := 'PayloadReplacementStrings';

      // PCAPNG

      cPCAPngDoReplace                  : result := 'PCAPngDoReplace';
      cPCAPngSanitizeCommentsScope      : result := 'PCAPngSanitizeCommentsScope';
      cPCAPngSanitizeCommentsMode       : result := 'PCAPngSanitizeCommentsMode';
      cPCAPngIHBDoReplace               : result := 'PCAPngIHBDoReplace';
      cPCAPngSanitizeIHBInterfaceName   : result := 'PCAPngSanitizeIHBInterfaceName';
      cPCAPngSanitizeIHBCaptureFilter   : result := 'PCAPngSanitizeIHBCaptureFilter';
      cPCAPngSanitizeNRB                : result := 'PCAPngSanitizeNRB';
      cPCAPngSanitizePacketOptions      : result := 'PCAPngSanitizePacketOptions';

      // 802.11

      cWiFiDoReplace                    : result := 'WiFiDoReplace';
      cWiFiReplaceBySSIDList            : result := 'WiFiReplaceBySSIDList';
      cWiFiReplaceRemainingSSIDs        : result := 'WiFiReplaceRemainingSSIDs';
      cWiFiReplaceRemainingSSIDsPrefix  : result := 'WiFiReplaceRemainingSSIDsPrefix';
      cWiFiKeepUnknownTaggedParams      : result := 'WiFiKeepUnknownTaggedParams';
      cWiFiSSIDReplacementStrings       : result := 'WiFiSSIDReplacementStrings';

      // Ethernet

      cEthernetDoReplace                : result := 'EthernetDoReplace';
      cEthernetDoOnlyDBBasedReplacement : result := 'EthernetDoOnlyDBBasedReplacement';
      cEthernetReplaceByMACList         : result := 'EthernetReplaceByMACList';
      cEthernetMACList                  : result := 'EthernetMACList';
      cEthernetRandomizeMACAddress      : result := 'EthernetRandomizeMACAddress';
      cEthernetBlackMarkerMACAddress    : result := 'EthernetBlackMarkerMACAddress';
      cEthernetLeaveVendorIntact        : result := 'EthernetLeaveVendorIntact';
      cEthernetRecalculateFCSMode       : result := 'EthernetRecalculateFCSMode';
      cEthernetPreserveTrailingBytes    : result := 'EthernetPreserveTrailingBytes';

      // VLAN

      cVLANDoReplace                    : result := 'VLANDoReplace';
      cVLANDoReplaceVLANByIDList        : result := 'VLANDoReplaceVLANByIDList';
      cVLANReplaceByVLANIDList          : result := 'VLANReplaceByVLANIDList';
      cVLANRemoveMode                   : result := 'VLANRemoveMode';
      cVLANRemoveList                   : result := 'VLANRemoveList';
      cVLANRandomizeVLANIDs             : result := 'VLANRandomizeVLANIDs';

      // IPv4

      cIPv4DoReplace                    : result := 'IPv4DoReplace';
      cIPv4DoReplaceByIPList            : result := 'IPv4DoReplaceByIPList';
      cIPv4ReplaceByIPList              : result := 'IPv4ReplaceByIPList';
      cIPv4AddressList                  : result := 'IPv4AddressList';
      cIPv4ReplaceBySubnetlist          : result := 'IPv4ReplaceBySubnetlist';
      cIPv4KeepHostPart                 : result := 'IPv4KeepHostPart';
      cIPv4SubnetList                   : result := 'IPv4SubnetList';
      cIPv4RandomizeIPAddress           : result := 'IPv4RandomizeIPAddress';
      cIPv4BlackMarkerIPAddress         : result := 'IPv4BlackMarkerIPAddress';
      cIPv4RandomizeMulticast           : result := 'IPv4RandomizeMulticast';
      cIPv4RandomizeAPIPA               : result := 'IPv4RandomizeAPIPA';
      cIPv4RandomizeDocumentation       : result := 'IPv4RandomizeDocumentation';
      cIPv4PrivateAddressRandomizeMode  : result := 'IPv4PrivateAddressRandomizeMode';
      cIPv4RandomizeIPID                : result := 'IPv4RandomizeIPID';
      cIPv4RecalculateCRCMode           : result := 'IPv4RecalculateCRCMode';
      cIPV4StoreGeneratedSubnetIPsToDB  : result := 'IPV4StoreGeneratedSubnetIPsToDB';

      // IPv6

      cIPv6DoReplace                    : result := 'IPv6DoReplace';
      cIPv6DoReplaceByIPList            : result := 'IPv6DoReplaceByIPList';
      cIPv6AddressList                  : result := 'IPv6AddressList';
      cIPv6ReplaceBySubnetlist          : result := 'IPv6ReplaceBySubnetlist';
      cIPv6KeepHostPart                 : result := 'IPv6KeepHostPart';
      cIPv6SubnetList                   : result := 'IPv6SubnetList';
      cIPv6RandomizeIPAddress           : result := 'IPv6RandomizeIPAddress';
      cIPv6BlackMarkerIPAddress         : result := 'IPv6BlackMarkerIPAddress';
      cIPv6RandomizeAddressMode         : result := 'IPv6RandomizeAddressMode';
      cIPv6RandomizeDocumentationAddr   : result := 'IPv6RandomizeDocumentationAddr';
      cIPV6StoreGeneratedSubnetIPsToDB  : result := 'IPV6StoreGeneratedSubnetIPsToDB';
      cIPv64RecalculateCRCMode          : result := 'IPv64RecalculateCRCMode';

      // ICMPv4

      cICMPv4DoReplace                  : result := 'ICMPv4DoReplace';
      cICMPv4RemovePingPayload          : result := 'ICMPv4RemovePingPayload';
      cICMPv4RemoveByTypeCode           : result := 'ICMPv4RemoveByTypeCode';
      cICMPv4RecalculateCRCMode         : result := 'ICMPv4RecalculateCRCMode';

      // ICMPv6

      cICMPv6DoReplace                  : result := 'ICMPv6DoReplace';
      cICMPv6RemovePingPayload          : result := 'ICMPv6RemovePingPayload';
      cICMPv6RecalculateCRCMode         : result := 'ICMPv6RecalculateCRCMode';
      cICMPv6RemoveDNSFQDNsFromOptions  : result := 'ICMPv6RemoveDNSFQDNsFromOptions';

      // TCP

      cTCPDoReplace                     : result := 'TCPDoReplace';
      cTCPReplaceSocketsByList          : result := 'TCPReplaceSocketsByList';
      cTCPSocketPortList                : result := 'TCPSocketPortList';
      cTCPReplacePortsByList            : result := 'TCPReplacePortsByList';
      cTCPPortList                      : result := 'TCPPortList';
      cTCPRandomizePorts                : result := 'TCPRandomizePorts';
      cTCPIgnoreSystemPorts             : result := 'TCPIgnoreSystemPorts';
      cTCPRecalculateCRC                : result := 'TCPRecalculateCRC';

      // UDP

      cUDPDoReplace                     : result := 'UDPDoReplace';
      cUDPReplaceSocketsByList          : result := 'UDPReplaceSocketsByList';
      cUDPSocketPortList                : result := 'UDPSocketPortList';
      cUDPReplacePortsByList            : result := 'UDPReplacePortsByList';
      cUDPPortList                      : result := 'UDPPortList';
      cUDPRandomizePorts                : result := 'UDPRandomizePorts';
      cUDPIgnoreSystemPorts             : result := 'UDPIgnoreSystemPorts';
      cUDPRecalculateCRC                : result := 'UDPRecalculateCRC';

      // Tunneling

      cTunnelDoReplace                  : result := 'TunnelDoReplace';
      cTunnelAYIYASanitizeIdentity      : result := 'TunnelAYIYASanitizeIdentity';
      cTunnelAYIYASanitizeSignature     : result := 'TunnelAYIYASanitizeSignature';

      cMPLSRandomizeLabel               : result := 'MPLSRandomizeLabel';
      cMPLSMaximumLabelValue            : result := 'MPLSMaximumLabelValue';

      // DHCPv4

      cDHCPv4DoReplace                  : result := 'DHCPv4DoReplace';
      cDHCPv4ReplaceHostName            : result := 'DHCPv4ReplaceHostName';
      cDHCPv4ReplaceHostnameMode        : result := 'DHCPv4ReplaceHostnameMode';
      cDHCPv4HostnameList               : result := 'DHCPv4HostnameList';

      // Application

      cRTPSDoReplace                    : result := 'RTPSDoReplace';
      cRTPSReplacementBlackMarker       : result := 'RTPSReplacementBlackMarker';

      // Settings

      cReplacementDBUseDiskFile         : result := 'ReplacementDBUseDiskFile';
      cReplacementDBKeepDiskFile        : result := 'ReplacementDBKeepDiskFile';
      cReplacementDBWipeOnStart         : result := 'ReplacementDBWipeOnStart';
      cReplacementDBForcePath           : result := 'ReplacementDBForcePath';
      cReplacementDBFilePath            : result := 'ReplacementDBFilePath';

      // Advanced

      cAdvDoNotReassembleIPFragments    : result := 'AdvDoNotReassembleIPFragments';
      cAdvReassemblyChainTimeoutMS      : result := 'AdvReassemblyChainTimeoutMS';
     else
         result := '';
     end;

     assert (result <> '', 'Unkown Setting token (AnonTokenToParameterName)');
end;

(* ------------------------------------------- *)
{ tAnonymizeParameters }
(* ------------------------------------------- *)

procedure tAnonymizeParameters.initialize;

var i : Integer;

begin
     TracefileListIndex := 0;
     KeepOriginalApplicationString := false;
     OverwriteExistingTarget := true;

     // Payload

     PayloadTruncateUnknownLayers := true;
     PayloadRemovalMode := prmCut;
     PayloadOverwritePattern := cDefaultPayloadPattern;
     PayloadReAddPaddingIfSliced := false;

     PayloadDoTruncateAfterLayer := false;
     PayloadTruncateAfterLayer := 4;
     PayloadDoTruncateAtOffset := false;
     PayloadTruncateAtOffset := 54;
     PayLoadDoReplaceStrings := false;
     PayLoadDoReplaceCaseInsensitve := true;
     PayloadDoNotEnforceSameLength := false;
     setLength (PayloadReplacementStrings, 0);

     // PCAPNG

     PCAPngDoReplace := true;
     PCAPngSanitizeCommentsScope := scsBoth;
     PCAPngSanitizeCommentsMode := scmRemoveAll;
     PCAPngIHBDoReplace := true;
     PCAPngSanitizeIHBInterfaceName := simReplaceGeneric;
     PCAPngSanitizeIHBCaptureFilter := cfmRemoveFilter;
     PCAPngSanitizeNRB := snmRemoveBlock;
     PCAPngSanitizePacketOptions := false;

     // 802.11

     WiFiDoReplace := true;
     WiFiReplaceBySSIDList := false;
     WiFiReplaceRemainingSSIDs := true;
     WiFiReplaceRemainingSSIDsPrefix := cDefaultWiFiSSIDPrefix;
     WiFiKeepUnknownTaggedParams := false;
     setLength (WiFiSSIDReplacementStrings, 0);

     // Ethernet

     EthernetDoReplace := true;
     EthernetDoOnlyDBBasedReplacement := false;
     EthernetReplaceByMACList := false;
     setLength (EthernetMACList, 0);
     EthernetRandomizeMACAddress := true;
     EthernetBlackMarkerMACAddress := false;
     EthernetLeaveVendorIntact := false;
     EthernetRecalculateFCSMode := rcmKeepBadCRC;
     EthernetPreserveTrailingBytes := false;

     // VLAN

     VLANDoReplace := true;
     VLANDoReplaceVLANByIDList := false;
     setLength (VLANReplaceByVLANIDList, 0);
     VLANRemoveMode := vrmNone;

     for i := 0 to 4096 do
         VLANRemoveList[i] := false;

     VLANRandomizeVLANIDs := true;

     // Tunneling

     TunnelDoReplace := true;
     TunnelAYIYASanitizeIdentity := aisZero;
     TunnelAYIYASanitizeSignature := assZero;

     MPLSRandomizeLabel := true;
     MPLSMaximumLabelValue := 1048575;

     // IPv4

     IPv4DoReplace := true;
     IPv4DoReplaceByIPList := false;
     IPv4ReplaceByIPList := false;
     setLength (IPv4AddressList, 0);
     IPv4ReplaceBySubnetlist := false;
     IPv4KeepHostPart := true;
     setLength (IPv4SubnetList, 0);
     IPv4RandomizeIPAddress := true;
     IPv4BlackMarkerIPAddress := false;
     IPv4RandomizeMulticast := false;
     IPv4RandomizeAPIPA := false;
     IPv4RandomizeDocumentation := false;
     IPv4PrivateAddressRandomizeMode := parRandomizeToPrivate;
     IPv4RandomizeIPID := false;
     IPv4RecalculateCRCMode := rcmKeepBadCRC;
     IPV4StoreGeneratedSubnetIPsToDB := true;

     // IPv6

     IPv6DoReplace := true;
     IPv6DoReplaceByIPList := false;
     setLength (IPv6AddressList, 0);
     IPv6ReplaceBySubnetlist := false;
     IPv6KeepHostPart := true;
     setLength (IPv6SubnetList, 0);
     IPv6RandomizeIPAddress := true;
     IPv6RandomizeAddressMode := iprRandomizeAndKeepPrefix;
     IPv6RandomizeDocumentationAddr := false;
     IPV6StoreGeneratedSubnetIPsToDB := true;

     // ICMPv4

     ICMPv4DoReplace := true;
     ICMPv4RemovePingPayload := true;
     ICMPv4RemoveICMPTypeCodes := false;
     setLength (ICMPv4RemoveByTypeCode, 0);
     ICMPv4RecalculateCRCMode := rcmKeepBadCRC;

     // ICMPv6

     ICMPv6DoReplace := true;
     ICMPv6RemoveDNSFQDNsFromOptions := true;
     ICMPv6RemovePingPayload := true;
     ICMPv6RecalculateCRCMode := rcmKeepBadCRC;

     // TCP

     TCPDoReplace := true;
     TCPReplaceSocketsByList := false;
     setLength (TCPSocketPortList, 0);
     TCPReplacePortsByList := false;
     setLength (TCPPortList, 0);
     TCPRandomizePorts := false;
     TCPIgnoreSystemPorts := true;
     TCPRecalculateCRC := rcmKeepBadCRC;

     // UDP

     UDPDoReplace := true;
     UDPReplaceSocketsByList := false;
     setLength (UDPSocketPortList, 0);
     UDPReplacePortsByList := false;
     setLength (UDPPortList, 0);
     UDPRandomizePorts := false;
     UDPIgnoreSystemPorts := true;
     UDPRecalculateCRC := rcmKeepBadCRC;

     // DHCPv4

     DHCPv4DoReplace := true;
     DHCPv4ReplaceHostName := true;
     DHCPv4ReplaceHostnameMode := rhnUsePlaceholders;
     setLength (DHCPv4HostnameList, 0);

     // Application

     RTPSDoReplace := true;
     RTPSReplacementBlackMarker := true;

     // Settings

     ReplacementDBUseDiskFile := false;
     ReplacementDBKeepDiskFile := false;
     ReplacementDBWipeOnStart := true;
     ReplacementDBForcePath := false;
     ReplacementDBFilePath := '';

     // Output

     OutputTargetLocation := ltSameDirectory;
     OutputPath := cDefaultRelativePath;
     OutputFilename := cDefaultOutputFilename;
     OutputTimestamp := tsmOriginalFile;

     // Advanced

     AdvDoNotReassembleIPFragments := false;
     AdvReassemblyChainTimeoutMS := 100;
end;

(* ------------------------------------------- *)

function tAnonymizeParameters.SaveToConfigFile (const Filename: string; const StripComments : boolean) : boolean;

var template : tStringlist;
    config   : tStringlist;
    rStream  : TResourceStream;
    skipadd  : boolean;
    sLine    : string;
    param    : string;
    value    : string;
    count    : integer;
    i, j     : Integer;

begin
     result := false;

     template := tStringlist.Create;

     // Get structure from Ressource (configured at Project -> Resources and Images)

     rStream := TResourceStream.Create (HInstance, 'AnonTaskTemplate', RT_RCDATA);
     template.LoadFromStream (rStream);
     rStream.Free;

     // build config from template

     config := tStringlist.Create;

     for i := 1 to template.Count do
         begin
              sLine := template[i-1];

              if sLine <> '' then
                 begin
                      if pos ('#', trim (sLine)) = 1 then
                         begin
                              if not StripComments then
                                  config.Add (sLine);
                         end
                      else
                          begin
                               skipadd := false;

                               // so this is not a comment, but we have to do something :-)

                               param := getEntry (sLine, ' ', 1);
                               value := getEntry (sLine, ' ', 2);

                               // now this is not elegant, but somehow it needs to be done, doesn't it?

                               if param = 'TargetLocation' then
                                  value := TargetLocationToString (OutputTargetLocation)
                               else
                               if param = 'TargetPath' then
                                  begin
                                       if OutputTargetlocation = ltSameDirectory then
                                          value := AnsiQuotedStr (cDefaultRelativePath, '"')
                                       else
                                           value := AnsiQuotedStr (OutputPath, '"');
                                  end
                               else
                               if param = 'TargetFilename' then
                                  value := AnsiQuotedStr (OutputFilename, '"')
                               else
                               if param = 'TargetTimestamp' then
                                  value := TargetTimestampToString (OutputTimestamp)
                               else
                               if param = 'ReplacementDBUseDiskFile' then
                                  value := lowercase (BoolToStr (ReplacementDBUseDiskFile, true))
                               else
                               if param = 'ReplacementDBKeepDiskFile' then
                                  value := lowercase (BoolToStr (ReplacementDBKeepDiskFile, true))
                               else
                               if param = 'ReplacementDBWipeOnStart' then
                                  value := lowercase (BoolToStr (ReplacementDBWipeOnStart, true))
                               else
                               if param = 'ReplacementDBForcePath' then
                                  value := lowercase (BoolToStr (ReplacementDBForcePath, true))
                               else
                               if param = 'ReplacementDBFilePath' then
                                  value := ReplacementDBFilePath
                               else
                               if param = 'PayloadTruncateUnknownLayers' then
                                  value := lowercase (BoolToStr (PayloadTruncateUnknownLayers, true))
                               else
                               if param = 'PayloadRemovalMode' then
                                  value := PayloadRemovalModeToString (PayloadRemovalMode)
                               else
                               if param = 'PayloadOverwritePattern' then
                                  value := PayloadOverwritePattern
                               else
                               if param = 'PayloadReAddPaddingIfSliced' then
                                  value := lowercase (BoolToStr (PayloadReAddPaddingIfSliced, true))
                               else
                               if param = 'PayloadTruncateAfterLayer' then
                                  begin
                                       if PayloadDoTruncateAfterLayer then
                                          value := IntToStr (PayloadTruncateAfterLayer)
                                       else
                                           value := '0';
                                  end
                               else
                               if param = 'PayloadTruncateAtOffset' then
                                  begin
                                       if PayloadDoTruncateAtOffset then
                                          value := IntToStr (PayloadTruncateAtOffset)
                                       else
                                           value := '0';
                                  end
                               else
                               if param = 'PayLoadReplaceArbitraryStrings' then
                                  value := lowercase (BoolToStr (PayLoadDoReplaceStrings, true))
                               else
                               if param = 'PayLoadReplaceCaseInsensitve' then
                                  value := lowercase (BoolToStr (PayLoadDoReplaceCaseInsensitve, true))
                               else
                               if param = 'PayloadDoNotEnforceSameLength' then
                                  value := lowercase (BoolToStr (PayloadDoNotEnforceSameLength, true))
                               else
                               if param = 'PayloadReplacementString' then
                                  begin
                                       value := '';
                                       count := length (PayloadReplacementStrings);

                                       // put all string pairs in one line, comma separated

                                       for j := 1 to count do
                                           begin
                                                if j < count then
                                                   value := value + PayloadReplacementStrings[j-1].getPair + ','
                                                else
                                                    value := value + PayloadReplacementStrings[j-1].getPair
                                           end;
                                  end
                               else
                               if param = 'PCAPngSanitizeCommentsScope' then
                                  value := SanitizeCommentsScopeToString (PCAPngSanitizeCommentsScope)
                               else
                               if param = 'PCAPngSanitizeCommentsMode' then
                                  value := SanitizeCommentsModeToString (PCAPngSanitizeCommentsMode)
                               else
                               if param = 'PCAPngSanitizeIHBInterfaceName' then
                                  begin
                                       if PCAPngIHBDoReplace then
                                          value := SanitizeIHBModeToString (PCAPngSanitizeIHBInterfaceName)
                                       else
                                           value := 'KeepAsIs';
                                  end
                               else
                               if param = 'PCAPngSanitizeIHBCaptureFilter' then
                                  begin
                                       if PCAPngIHBDoReplace then
                                          value := SanitizeIHBCaptureFilterModeToString (PCAPngSanitizeIHBCaptureFilter)
                                       else
                                           value := 'KeepAsIs';
                                  end
                               else
                               if param = 'PCAPngSanitizeNRB' then
                                  if PCAPngSanitizeNRB = snmNone then
                                     value := 'KeepAsIs'
                                  else
                                      value := SanitizeNRBModeToString (PCAPngSanitizeNRB)
                               else
                               if param = 'WiFiSSIDReplacementString' then
                                  begin
                                       value := '';
                                       count := length (WiFiSSIDReplacementStrings);

                                       // put all string pairs in one line, comma separated

                                       for j := 1 to count do
                                           begin
                                                if j < count then
                                                   value := value + WiFiSSIDReplacementStrings[j-1].getPair + ','
                                                else
                                                    value := value + WiFiSSIDReplacementStrings[j-1].getPair
                                           end;
                                  end
                               else
                               if param = 'WiFiReplaceRemainingSSIDsPrefix' then
                                  begin
                                       if WiFiReplaceRemainingSSIDs then
                                          value := AnsiQuotedStr (WiFiReplaceRemainingSSIDsPrefix, '"')
                                       else
                                           value := ''
                                  end
                               else
                               if param = 'WiFiKeepUnknownTaggedParams' then
                                  value := lowercase (BoolToStr (WiFiKeepUnknownTaggedParams, true))
                               else
                               if param = 'EthernetReplaceByMACPair' then
                                  begin
                                       if EthernetReplaceByMACList then
                                          begin
                                               value := '';
                                               count := length (EthernetMACList);

                                               for j := 1 to count do
                                                   begin
                                                        if j < count then
                                                           value := value + EthernetMACList[j-1].Source.asString + '=' + EthernetMACList[j-1].Destination.asString + ','
                                                        else
                                                            value := value + EthernetMACList[j-1].Source.asString + '=' + EthernetMACList[j-1].Destination.asString
                                                   end;
                                          end
                                       else
                                           value := '';
                                  end
                               else
                               if param = 'EthernetSanitizeMACAddresses' then
                                  begin
                                       if EthernetRandomizeMACAddress then
                                          begin
                                               if EthernetBlackMarkerMACAddress then
                                                  value := 'BlackMarker'
                                               else
                                                   value := 'Randomize';

                                               if EthernetLeaveVendorIntact then
                                                  value := 'KeepVendor';
                                          end
                                       else
                                           value := 'KeepAsIs';
                                  end
                               else
                               if param = 'EthernetRecalculateFCSMode' then
                                  value := RecalculateCRCModeToString (EthernetRecalculateFCSMode)
                               else
                               if param = 'EthernetPreserveTrailingBytes' then
                                  value := lowercase (BoolToStr (EthernetPreserveTrailingBytes, true))
                               else
                               if param = 'VLANReplaceByIDPair' then
                                  begin
                                       if VLANDoReplaceVLANByIDList then
                                          begin
                                               value := '';
                                               count := length (VLANReplaceByVLANIDList);

                                               for j := 1 to count do
                                                   begin
                                                        if j < count then
                                                           value := value + VLANReplaceByVLANIDList[j-1].Value1.toString + '=' + VLANReplaceByVLANIDList[j-1].Value2.toString + ','
                                                        else
                                                            value := value + VLANReplaceByVLANIDList[j-1].Value1.toString + '=' + VLANReplaceByVLANIDList[j-1].Value2.toString
                                                   end;
                                          end
                                       else
                                           value := '';
                                  end
                               else
                               if param = 'VLANRemovalMode' then
                                  value := VLANRemoveModeToString (VLANRemoveMode)
                               else
                               if param = 'VLANIDList' then
                                  begin
                                       value := '';

                                       for j := 0 to 4096 do
                                           begin
                                                if VLANRemoveList[j] then
                                                   value := value + j.ToString + ' ';
                                           end;

                                       value := stringReplace (trim (value), ' ', ',', [rfReplaceAll]);
                                  end
                               else
                               if param = 'VLANRandomizeVLANIDs' then
                                  value := lowercase (BoolToStr (VLANRandomizeVLANIDs, true))
                               else
                               if param = 'TunnelAYIYASanitizeID' then
                                  value := lowercase (BoolToStr (TunnelDoReplace and (TunnelAYIYASanitizeIdentity = aisZero), true))
                               else
                               if param = 'MPLSRandomizeLabel' then
                                  value := lowercase (BoolToStr (MPLSRandomizeLabel, true))
                               else
                               if param = 'MPLSMaximumLabelValue' then
                                  value := MPLSMaximumLabelValue.ToString
                               else
                               if param = 'IPv4ReplaceByAddressPair' then
                                  begin
                                       if IPv4DoReplaceByIPList then
                                          begin
                                               value := '';
                                               count := length (IPv4AddressList);

                                               for j := 1 to count do
                                                   begin
                                                        if j < count then
                                                           value := value + IPv4AddressList[j-1].Source.ValueAsString + '=' + IPv4AddressList[j-1].Destination.ValueAsString + ','
                                                        else
                                                            value := value + IPv4AddressList[j-1].Source.ValueAsString + '=' + IPv4AddressList[j-1].Destination.ValueAsString;
                                                   end;
                                          end
                                       else
                                           value := '';
                                  end
                               else
                               if param = 'IPv4EnableAutoMode' then
                                  value := lowercase (BoolToStr (IPV4AddressAutoMode, true))
                               else
                               if param = 'IPv4ReplaceBySubnetPair' then
                                  begin
                                       if IPv4ReplaceBySubnetlist then
                                          begin
                                               value := '';
                                               count := length (IPv4SubnetList);

                                               for j := 1 to count do
                                                   begin
                                                        if j < count then
                                                           value := value + IPv4SubnetList[j-1].Source.ValueAsCIDRString + '=' + IPv4SubnetList[j-1].Destination.ValueAsCIDRString + ','
                                                        else
                                                            value := value + IPv4SubnetList[j-1].Source.ValueAsCIDRString + '=' + IPv4SubnetList[j-1].Destination.ValueAsCIDRString;
                                                   end;
                                          end
                                       else
                                           value := '';
                                  end
                               else
                               if param = 'IPv4SubnetReplacementKeepHostBits' then
                                  value := lowercase (BoolToStr (IPv4KeepHostPart, true))
                               else
                               if param = 'IPv4SanitizeAddresses' then
                                  begin
                                       if IPv4RandomizeIPAddress then
                                          value := 'Randomize'
                                       else
                                           value := 'KeepAsIs';

                                       if IPv4BlackMarkerIPAddress then
                                          value := 'BlackMarker';
                                  end
                               else
                               if param = 'IPv4KeepSpecialAddresses' then
                                  begin
                                       value := '';

                                       if IPv4RandomizeMulticast and IPv4RandomizeAPIPA and IPv4RandomizeDocumentation then
                                          value := 'None'
                                       else
                                           if not (IPv4RandomizeMulticast or IPv4RandomizeAPIPA or IPv4RandomizeDocumentation) then
                                              value := 'All'
                                           else
                                               begin
                                                    if IPv4RandomizeMulticast then
                                                       value := 'MCast';

                                                    if IPv4RandomizeAPIPA then
                                                       value := value + 'APIPA';

                                                    if IPv4RandomizeDocumentation then
                                                       value := value + 'Doc';
                                               end;
                                  end
                               else
                               if param = 'IPv4PrivateAddressRandomizeMode' then
                                  value := PrivateAddressRandomModeToString (IPv4PrivateAddressRandomizeMode)
                               else
                               if param = 'IPv4RandomizeIPID' then
                                  value := lowercase (BoolToStr (IPv4RandomizeIPID, true))
                               else
                               if param = 'IPv4RecalculateCRCMode' then
                                  value := RecalculateCRCModeToString (IPv4RecalculateCRCMode)
                               else
                               if param = 'IPv6ReplaceByAddressPair' then
                                  begin
                                       if IPv6DoReplaceByIPList then
                                          begin
                                               value := '';
                                               count := length (IPv6AddressList);

                                               for j := 1 to count do
                                                   begin
                                                        if j < count then
                                                           value := value + IPv6AddressList[j-1].Source.ValueAsString + '=' + IPv6AddressList[j-1].Destination.ValueAsString + ','
                                                        else
                                                            value := value + IPv6AddressList[j-1].Source.ValueAsString + '=' + IPv6AddressList[j-1].Destination.ValueAsString;
                                                   end;
                                          end
                                       else
                                           value := '';
                                  end
                               else
                               if param = 'IPv6EnableAutoMode' then
                                  value := lowercase (BoolToStr (IPV6AddressAutoMode, true))
                               else
                               if param = 'IPv6ReplaceByPrefixPair' then
                                  begin
                                       if IPv6ReplaceBySubnetlist then
                                          begin
                                               value := '';
                                               count := length (IPv6SubnetList);

                                               for j := 1 to count do
                                                   begin
                                                        if j < count then
                                                           value := value + IPv6SubnetList[j-1].Source.ValueAsString + '=' + IPv6SubnetList[j-1].Destination.ValueAsString + ','
                                                        else
                                                            value := value + IPv6SubnetList[j-1].Source.ValueAsString + '=' + IPv6SubnetList[j-1].Destination.ValueAsString;
                                                   end;
                                          end
                                       else
                                           value := '';
                                  end
                               else
                               if param = 'IPv6SanitizeAddresses' then
                                  begin
                                       if IPv6RandomizeIPAddress then
                                          value := IPv6RandomizationModeToString (IPv6RandomizeAddressMode)
                                       else
                                           value := 'KeepAsIs';
                                  end
                               else
                               if param = 'IPv6KeepSpecialAddresses' then
                                  begin
                                       if IPv6RandomizeDocumentationAddr then
                                          value := 'None'
                                       else
                                           value := 'Doc';
                                  end
                               else
                               if param = 'ICMPv4RemovePingPayload' then
                                  value := lowercase (BoolToStr (ICMPv4RemovePingPayload, true))
                               else
                               if param = 'ICMPv4RemovePacketByTypeCode' then
                                  begin
                                       if ICMPv4RemoveICMPTypeCodes then
                                          begin
                                               value := '';
                                               count := length (ICMPv4RemoveByTypeCode);

                                               for j := 1 to count do
                                                   begin
                                                        if j < count then
                                                           value := value + ICMPv4RemoveByTypeCode[j-1].toString + ','
                                                        else
                                                            value := value + ICMPv4RemoveByTypeCode[j-1].toString;
                                                   end;
                                          end
                                       else
                                           value := '';
                                  end
                               else
                               if param = 'ICMPv4RecalculateCRCMode' then
                                  value := RecalculateCRCModeToString (ICMPv4RecalculateCRCMode)
                               else
                               if param = 'ICMPv6RemoveDNSFQDNsFromOptions' then
                                  value := lowercase (BoolToStr (ICMPv6RemoveDNSFQDNsFromOptions, true))
                               else
                               if param = 'ICMPv6RemovePingPayload' then
                                  value := lowercase (BoolToStr (ICMPv6RemovePingPayload, true))
                               else
                               if param = 'ICMPv6RecalculateCRCMode' then
                                  value := RecalculateCRCModeToString (ICMPv6RecalculateCRCMode)
                               else
                               if param = 'TCPReplaceSocketsByList' then
                                  begin
                                       if TCPReplaceSocketsByList then
                                          begin
                                               value := '';
                                               count := length (TCPSocketPortList);

                                               for j := 1 to count do
                                                   begin
                                                        if j < count then
                                                           value := value + TCPSocketPortList[j-1].Source.ValueAsString + '=' + TCPSocketPortList[j-1].Destination.ValueAsString + ','
                                                        else
                                                            value := value + TCPSocketPortList[j-1].Source.ValueAsString + '=' + TCPSocketPortList[j-1].Destination.ValueAsString;
                                                   end;
                                          end
                                       else
                                           value := '';
                                  end
                               else
                               if param = 'TCPReplacePortsByList' then
                                  begin
                                       if TCPReplacePortsByList then
                                          begin
                                               value := '';
                                               count := length (TCPPortList);

                                               for j := 1 to count do
                                                   begin
                                                        if j < count then
                                                           value := value + TCPPortList[j-1].Value1.toString + '=' + TCPPortList[j-1].Value2.toString + ','
                                                        else
                                                            value := value + TCPPortList[j-1].Value1.toString + '=' + TCPPortList[j-1].Value2.toString;
                                                   end;
                                          end
                                       else
                                           value := '';
                                  end
                               else
                               if param = 'TCPRandomizePorts' then
                                  begin
                                       if TCPRandomizePorts then
                                          begin
                                               value := 'RandomizeAll';

                                               if TCPIgnoreSystemPorts then
                                                  value := 'IgnoreSystem';
                                          end
                                       else
                                           value := 'KeepAsIs';
                                  end
                               else
                               if param = 'TCPRecalculateCRC' then
                                  value := RecalculateCRCModeToString (TCPRecalculateCRC)
                               else
                               if param = 'UDPReplaceSocketsByList' then
                                  begin
                                       if UDPReplaceSocketsByList then
                                          begin
                                               value := '';
                                               count := length (UDPSocketPortList);

                                               for j := 1 to count do
                                                   begin
                                                        if j < count then
                                                           value := value + UDPSocketPortList[j-1].Source.ValueAsString + '=' + UDPSocketPortList[j-1].Destination.ValueAsString + ','
                                                        else
                                                            value := value + UDPSocketPortList[j-1].Source.ValueAsString + '=' + UDPSocketPortList[j-1].Destination.ValueAsString;
                                                   end;
                                          end
                                       else
                                           value := '';
                                  end
                               else
                               if param = 'UDPReplacePortsByList' then
                                  begin
                                       if UDPReplacePortsByList then
                                          begin
                                               value := '';
                                               count := length (UDPPortList);

                                               for j := 1 to count do
                                                   begin
                                                        if j < count then
                                                           value := value + UDPPortList[j-1].Value1.toString + '=' + UDPPortList[j-1].Value2.toString + ','
                                                        else
                                                            value := value + UDPPortList[j-1].Value1.toString + '=' + UDPPortList[j-1].Value2.toString;
                                                   end;
                                          end
                                       else
                                           value := '';
                                  end
                               else
                               if param = 'UDPRandomizePorts' then
                                  begin
                                       if UDPRandomizePorts then
                                          begin
                                               value := 'RandomizeAll';

                                               if UDPIgnoreSystemPorts then
                                                  value := 'IgnoreSystem';
                                          end
                                       else
                                           value := 'KeepAsIs';
                                  end
                               else
                               if param = 'UDPRecalculateCRC' then
                                  value := RecalculateCRCModeToString (UDPRecalculateCRC)
                               else
                               if param = 'DHCPv4ReplaceHostName' then
                                  begin
                                       if DHCPv4ReplaceHostName then
                                          value := DHCPv4ReplaceHostNameModeToString (DHCPv4ReplaceHostnameMode)
                                       else
                                           value := 'KeepAsIs';
                                  end
                               else
                               if param = 'RTPSSanitizeData' then
                                  begin
                                       if RTPSDoReplace then
                                          begin
                                               if RTPSReplacementBlackMarker then
                                                  value := 'BlackMarker'
                                               else
                                                   value := 'Randomize';
                                          end
                                       else
                                           value := 'KeepAsIs';
                                  end
                               else
                               if param = 'AdvDoNotReassembleIPFragments' then
                                  value := lowercase (BoolToStr (AdvDoNotReassembleIPFragments, true))
                               else
                               if param = 'AdvReassemblyChainTimeoutMS' then
                                  value := AdvReassemblyChainTimeoutMS.ToString;

                               // Create output line, unless we did that already, e.g. in a loop for multiple values for the same keyword

                               if not skipadd then
                                  config.Add (param + ' ' + value);
                          end;
                 end
              else
                  begin
                       // If we strip comments, we also remove empty lines

                       if not StripComments then
                          config.Add ('');
                  end;
         end;

     try
        config.SaveToFile (Filename);

        result := true;
     finally
            template.Free;
            config.Free;
     end;
end;

(* ------------------------------------------- *)

function tAnonymizeParameters.LoadFromConfigFile (const Filename : string) : string;

var config : tStringlist;
    sLine  : string;
    param  : string;
    value  : string;
    entry  : string;
    help   : string;
    count  : integer;
    dummy  : integer;
    iHelp  : integer;
    i, j   : integer;

begin
     result := 'General Failure';
     assert (fileexists (Filename), 'File does not exist (tAnonymizeParameters.LoadFromConfigFile)');

     // preset defaults

     initialize;

     // load config

     config := tStringlist.Create;
     config.LoadFromFile (Filename);

     try
        for i := 1 to config.Count do
            begin
                 sLine := config[i-1];

                 if (sLine <> '') and (pos ('#', trim (sLine)) <> 1) then
                    begin
                         // get param and value

                         param := lowercase (getEntry (sLine, ' ', 1));
                         value := rightstring (trim (sLine), ' ');

                         // Check if there is a comment at the end. Since the # could be part of
                         // a string, check if there's a quotation mark behind it. If not, strip
                         // the comment

                         j := pos ('#', value);

                         if j > 0 then
                            begin
                                 if PosEx ('"', value, j) = 0 then
                                    value := trim (leftstring (value, '#', true));
                            end;

                         // parse parameters and assign values, if possible

                         if param = 'tasktype' then
                            begin
                                 if LowerCase (value) <> 'anontask' then
                                    exit ('Invalid Task File');
                            end
                         else
                         if param = 'targetlocation' then
                            begin
                                 if StringToTargetLocation (value) <> ltUnknown then
                                    OutputTargetlocation := StringToTargetLocation (value)
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid location type');
                            end
                         else
                         if param = 'targetpath' then
                            OutputPath := AnsiDeQuotedStr (value, '"')
                         else
                         if param = 'targetfilename' then
                            OutputFilename := AnsiDeQuotedStr (value, '"')
                         else
                         if param = 'targettimestamp' then
                            begin
                                 if StringToTargetTimestamp (value) <> tsmUnknown then
                                    OutputTimestamp := StringToTargetTimestamp (value)
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid timestamp type');
                            end
                         else
                         if param = 'replacementdbusediskfile' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : ReplacementDBUseDiskFile := true;
                                  bpFalse   : ReplacementDBUseDiskFile := false;
                                 end;
                            end
                         else
                         if param = 'replacementdbkeepdiskfile' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : ReplacementDBKeepDiskFile := true;
                                  bpFalse   : ReplacementDBKeepDiskFile := false;
                                 end;
                            end
                         else
                         if param = 'replacementdbwipeonstart' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : ReplacementDBWipeOnStart := true;
                                  bpFalse   : ReplacementDBWipeOnStart := false;
                                 end;
                            end
                         else
                         if param = 'replacementdbforcepath' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : ReplacementDBForcePath := true;
                                  bpFalse   : ReplacementDBForcePath := false;
                                 end;
                            end
                         else
                         if param = 'replacementdbfilepath' then
                            OutputFilename := AnsiDeQuotedStr (value, '"')
                         else
                         if param = 'payloadtruncateunknownlayers' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : PayloadTruncateUnknownLayers := true;
                                  bpFalse   : PayloadTruncateUnknownLayers := false;
                                 end;
                            end
                         else
                         if param = 'payloadremovalmode' then
                            begin
                                 if StringToPayloadRemovalMode (value) <> prmUnknown then
                                    PayloadRemovalMode := StringToPayloadRemovalMode (value)
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param = 'payloadoverwritepattern' then
                            PayloadOverwritePattern := value
                         else
                         if param = 'payloadreaddpaddingifsliced' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : PayloadReAddPaddingIfSliced := true;
                                  bpFalse   : PayloadReAddPaddingIfSliced := false;
                                 end;
                            end
                         else
                         if param = 'payloadtruncateafterlayer' then
                            begin
                                 val (value, iHelp, dummy);

                                 if (iHelp = 0) or ((iHelp > 1) and (iHelp < 5)) then
                                    begin
                                         PayloadDoTruncateAfterLayer := (iHelp > 0);
                                         PayloadTruncateAfterLayer := iHelp;
                                    end
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param = 'payloadtruncateatoffset' then
                            begin
                                 val (value, iHelp, dummy);

                                 if (iHelp = 0) or ((iHelp > 13) and (iHelp < 65536)) then
                                    begin
                                         PayloadDoTruncateAtOffset := (iHelp > 0);
                                         PayloadTruncateAtOffset := iHelp;
                                    end
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param = 'payloadreplacearbitrarystrings' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : PayLoadDoReplaceStrings := true;
                                  bpFalse   : PayLoadDoReplaceStrings := false;
                                 end;
                            end
                         else
                         if param = 'payloadreplacecaseinsensitve' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : PayLoadDoReplaceCaseInsensitve := true;
                                  bpFalse   : PayLoadDoReplaceCaseInsensitve := false;
                                 end;
                            end
                         else
                         if param = 'payloaddonotenforcesamelength' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : PayloadDoNotEnforceSameLength := true;
                                  bpFalse   : PayloadDoNotEnforceSameLength := false;
                                 end;
                            end
                         else
                         if param = 'payloadreplacementstring' then
                            begin
                                 count := numEntries (value, ',');
                                 setLength (PayloadReplacementStrings, length (PayloadReplacementStrings) + count);

                                 for j := 1 to count do
                                     begin
                                          entry := trim (getEntry (value, ',', j));

                                          if not isValidANSIStringPair (entry, true) then
                                             begin
                                                  setLength (PayloadReplacementStrings, j-1);
                                                  exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                             end;

                                          // add the string

                                          PayloadReplacementStrings[j-1].initialize (entry);
                                     end;
                            end
                         else
                         if param='pcapngsanitizecommentsscope' then
                            begin
                                 if StringToSanitizeCommentsScope (value) <> scsUnknown then
                                    PCAPngSanitizeCommentsScope :=  StringToSanitizeCommentsScope (value)
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='pcapngsanitizecommentsmode' then
                            begin
                                 if StringToSanitizeCommentsMode (value) <> scmUnknown then
                                    PCAPngSanitizeCommentsMode :=  StringToSanitizeCommentsMode (value)
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='pcapngsanitizeihbinterfacename' then
                            begin
                                 if StringToSanitizeIHBMode (value) <> simUnknown then
                                    PCAPngSanitizeIHBInterfaceName :=  StringToSanitizeIHBMode (value)
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='pcapngsanitizeihbcapturefilter' then
                            begin
                                 if StringToSanitizeIHBCaptureFilterMode (value) <> cfmUnknown then
                                    PCAPngSanitizeIHBCaptureFilter :=  StringToSanitizeIHBCaptureFilterMode (value)
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='pcapngsanitizenrb' then
                            begin
                                 if StringToSanitizeNRBMode (value) <> snmUnknown then
                                    PCAPngSanitizeNRB :=  StringToSanitizeNRBMode (value)
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='wifissidreplacementstring' then
                            begin
                                 count := numEntries (value, ',');
                                 setLength (WiFiSSIDReplacementStrings, length (WiFiSSIDReplacementStrings) + count);

                                 for j := 1 to count do
                                     begin
                                          entry := trim (getEntry (value, ',', j));

                                          if not isValidANSIStringPair (entry, true) then
                                             begin
                                                  setLength (WiFiSSIDReplacementStrings, j-1);
                                                  exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                             end;

                                          // add the string

                                          WiFiSSIDReplacementStrings[j-1].initialize (entry);
                                     end;
                            end
                         else
                         if param='wifireplaceremainingssidsprefix' then
                            WiFiReplaceRemainingSSIDsPrefix := value
                         else
                         if param='wifikeepunknowntaggedparams' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : WiFiKeepUnknownTaggedParams := true;
                                  bpFalse   : WiFiKeepUnknownTaggedParams := false;
                                 end;
                            end
                         else
                         if param='ethernetreplacebymacpair' then
                            begin
                                 count := numEntries (value, ',');

                                 if count > 0 then
                                    EthernetReplaceByMACList := true;

                                 setLength (EthernetMACList, length (EthernetMACList) + count);

                                 for j := 1 to count do
                                     begin
                                          entry := trim (getEntry (value, ',', j));

                                          if not isValidMACaddressPair (entry, false) then
                                             begin
                                                  setLength (EthernetMACList, j-1);
                                                  exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                             end;

                                          // add the string

                                          EthernetMACList[j-1].initialize (entry);
                                     end;
                            end
                         else
                         if param='ethernetsanitizemacaddresses' then
                            begin
                                 help := lowercase (value);

                                 if (help = 'randomize') or (help = 'blackmarker') or (help = 'keepvendor') then
                                    begin
                                         EthernetRandomizeMACAddress := (help = 'randomize') or (help = 'keepvendor');
                                         EthernetBlackMarkerMACAddress := (help = 'blackmarker');
                                         EthernetLeaveVendorIntact := (help = 'keepvendor');
                                    end
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='ethernetrecalculatefcsmode' then
                            begin
                                 if StringToRecalculateCRCMode (value) <> rcmUnknown then
                                    EthernetRecalculateFCSMode := StringToRecalculateCRCMode (value)
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='ethernetpreservetrailingbytes' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : EthernetPreserveTrailingBytes := true;
                                  bpFalse   : EthernetPreserveTrailingBytes := false;
                                 end;
                            end
                         else
                         if param='vlanreplacebyidpair' then
                            begin
                                 count := numEntries (value, ',');

                                 if count > 0 then
                                    VLANDoReplaceVLANByIDList := true;

                                 setLength (VLANReplaceByVLANIDList, length (VLANReplaceByVLANIDList) + count);

                                 for j := 1 to count do
                                     begin
                                          entry := trim (getEntry (value, ',', j));

                                          if not isValidVLANIDPair (entry) then
                                             begin
                                                  setLength (VLANReplaceByVLANIDList, j-1);
                                                  exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                             end;

                                          // add the string

                                          VLANReplaceByVLANIDList[j-1].initialize (entry);
                                     end;
                            end
                         else
                         if param='vlanremovalmode' then
                            begin
                                 if StringToVLANRemoveMode (value) = vrmUnknown then
                                    exit ('Error in task config file in line ' + i.ToString + ': invalid value')
                                 else
                                     VLANRemoveMode := StringToVLANRemoveMode (value);
                            end
                         else
                         if param='vlanidlist' then
                            begin
                                 count := numEntries (value, ',');

                                 for j := 1 to count do
                                     begin
                                          entry := trim (getEntry (value, ',', j));
                                          val (entry, iHelp, dummy);

                                          if ((iHelp = 0) and (entry='0')) or ((iHelp > 0) and (iHelp < 4097)) then
                                             VLANRemoveList[iHelp] := true
                                          else
                                              exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                     end;
                            end
                         else
                         if param='vlanrandomizevlanids' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : VLANRandomizeVLANIDs := true;
                                  bpFalse   : VLANRandomizeVLANIDs := false;
                                 end;
                            end
                         else
                         if param='tunnelayiyasanitizeid' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : begin
                                                   TunnelDoReplace := true;
                                                   TunnelAYIYASanitizeIdentity := aisZero;
                                              end;
                                  bpFalse   : begin
                                                   TunnelAYIYASanitizeIdentity := aisNone;
                                              end;
                                 end;
                            end
                         else
                         if param='mplsrandomizelabel' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : MPLSRandomizeLabel := true;
                                  bpFalse   : MPLSRandomizeLabel := false;
                                 end;
                            end
                         else
                         if param='mplsmaximumlabelvalue' then
                            begin
                                 val (value, iHelp, dummy);

                                 if (iHelp > 31) and (iHelp < 1048576) then
                                    MPLSMaximumLabelValue := iHelp
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='ipv4replacebyaddresspair' then
                            begin
                                 count := numEntries (value, ',');

                                 if count > 0 then
                                    IPv4DoReplaceByIPList := true;

                                 setLength (IPv4AddressList, length (IPv4AddressList) + count);

                                 for j := 1 to count do
                                     begin
                                          entry := trim (getEntry (value, ',', j));

                                          if not isValidIPv4AddressPair (entry, false) then
                                             begin
                                                  setLength (IPv4AddressList, j-1);
                                                  exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                             end;

                                          // add the string

                                          IPv4AddressList[j-1].initialize (entry, false);
                                     end;
                            end
                         else
                         if param='ipv4enableautomode' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : IPV4AddressAutoMode := true;
                                  bpFalse   : IPV4AddressAutoMode := false;
                                 end;
                            end
                         else
                         if param='ipv4replacebysubnetpair' then
                            begin
                                 count := numEntries (value, ',');

                                 if count > 0 then
                                    IPv4ReplaceBySubnetlist := true;

                                 setLength (IPv4SubnetList, length (IPv4SubnetList) + count);

                                 for j := 1 to count do
                                     begin
                                          entry := trim (getEntry (value, ',', j));

                                          if not isValidIPv4SubnetPair (entry, false) then
                                             begin
                                                  setLength (IPv4SubnetList, j-1);
                                                  exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                             end;

                                          // add the string

                                          IPv4SubnetList[j-1].initialize (entry, true);
                                     end;
                            end
                         else
                         if param='ipv4subnetreplacementkeephostbits' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : IPv4KeepHostPart := true;
                                  bpFalse   : IPv4KeepHostPart := false;
                                 end;
                            end
                         else
                         if param='ipv4sanitizeaddresses' then
                            begin
                                 help := lowercase (value);

                                 IPv4RandomizeIPAddress := (help = 'randomize');
                                 IPv4BlackMarkerIPAddress := (help = 'blackmarker');
                            end
                         else
                         if param='ipv4keepspecialaddresses' then
                            begin
                                 help := lowercase (value);

                                 IPv4RandomizeMulticast := (help = 'apipa') or (help = 'apipadoc') or (help = 'doc') or (help = 'none');
                                 IPv4RandomizeAPIPA := (help = 'mcast') or (help = 'mcastdoc') or (help = 'doc') or (help = 'none');
                                 IPv4RandomizeDocumentation := (help = 'mcast') or (help = 'mcastapipa') or (help = 'apipa') or (help = 'none');
                            end
                         else
                         if param='ipv4privateaddressrandomizemode' then
                            begin
                                 if StringPrivateAddressRandomMode (value) = parUnknown then
                                    exit ('Error in task config file in line ' + i.ToString + ': invalid value')
                                 else
                                     IPv4PrivateAddressRandomizeMode := StringPrivateAddressRandomMode (value);
                            end
                         else
                         if param='ipv4randomizeipid' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : IPv4RandomizeIPID := true;
                                  bpFalse   : IPv4RandomizeIPID := false;
                                 end;
                            end
                         else
                         if param='ipv4recalculatecrcmode' then
                            begin
                                 if StringToRecalculateCRCMode (value) <> rcmUnknown then
                                    IPv4RecalculateCRCMode := StringToRecalculateCRCMode (value)
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='ipv6replacebyaddresspair' then
                            begin
                                 count := numEntries (value, ',');

                                 if count > 0 then
                                    IPv6DoReplaceByIPList := true;

                                 setLength (IPv6AddressList, length (IPv6AddressList) + count);

                                 for j := 1 to count do
                                     begin
                                          entry := trim (getEntry (value, ',', j));

                                          if not isValidIPv6AddressPair (entry, false) then
                                             begin
                                                  setLength (IPv6AddressList, j-1);
                                                  exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                             end;

                                          // add the string

                                          IPv6AddressList[j-1].initialize (entry, false);
                                     end;
                            end
                         else
                         if param='ipv6enableautomode' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : IPV6AddressAutoMode := true;
                                  bpFalse   : IPV6AddressAutoMode := false;
                                 end;
                            end
                         else
                         if param='ipv6replacebyprefixpair' then
                            begin
                                 count := numEntries (value, ',');

                                 if count > 0 then
                                    IPv6ReplaceBySubnetlist := true;

                                 setLength (IPv6SubnetList, length (IPv6SubnetList) + count);

                                 for j := 1 to count do
                                     begin
                                          entry := trim (getEntry (value, ',', j));

                                          if not isValidIPv6SubnetPair (entry, false) then
                                             begin
                                                  setLength (IPv6SubnetList, j-1);
                                                  exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                             end;

                                          // add the string

                                          IPv6SubnetList[j-1].initialize (entry, true);
                                     end;
                            end
                         else
                         if param='ipv6sanitizeaddresses' then
                            begin
                                 if StringToIPv6RandomizationMode (value) <> iprUnknown then
                                    IPv6RandomizeAddressMode := StringToIPv6RandomizationMode (value)
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='ipv6keepspecialaddresses' then
                            begin
                                 value := lowercase (value);

                                 if (value = 'none') then
                                    IPv6RandomizeDocumentationAddr := true
                                 else
                                     if (value = 'doc') then
                                        IPv6RandomizeDocumentationAddr := false
                                     else
                                         exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='icmpv4removepingpayload' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : ICMPv4RemovePingPayload := true;
                                  bpFalse   : ICMPv4RemovePingPayload := false;
                                 end;
                            end
                         else
                         if param='icmpv4removepacketbytypecode' then
                            begin
                                 count := numEntries (value, ',');

                                 if count > 0 then
                                    ICMPv4RemoveICMPTypeCodes := true;

                                 setLength (ICMPv4RemoveByTypeCode, length (ICMPv4RemoveByTypeCode) + count);

                                 for j := 1 to count do
                                     begin
                                          entry := trim (getEntry (value, ',', j));

                                          if not isValidICMPv4TypeCodePair (entry) then
                                             begin
                                                  setLength (ICMPv4RemoveByTypeCode, j-1);
                                                  exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                             end;

                                          // add the string

                                          ICMPv4RemoveByTypeCode[j-1].initialize (value);
                                     end;
                            end
                         else
                         if param='icmpv4recalculatecrcmode' then
                            begin
                                 if StringToRecalculateCRCMode (value) <> rcmUnknown then
                                    ICMPv4RecalculateCRCMode := StringToRecalculateCRCMode (value)
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='icmpv6removednsfqdnsfromoptions' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : ICMPv6RemoveDNSFQDNsFromOptions := true;
                                  bpFalse   : ICMPv6RemoveDNSFQDNsFromOptions := false;
                                 end;
                            end
                         else
                         if param='icmpv6removepingpayload' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : ICMPv6RemovePingPayload := true;
                                  bpFalse   : ICMPv6RemovePingPayload := false;
                                 end;
                            end
                         else
                         if param='icmpv6recalculatecrcmode' then
                            begin
                                 if StringToRecalculateCRCMode (value) <> rcmUnknown then
                                    ICMPv6RecalculateCRCMode := StringToRecalculateCRCMode (value)
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='tcpreplacesocketsbylist' then
                            begin
                                 count := numEntries (value, ',');

                                 if count > 0 then
                                    TCPReplaceSocketsByList := true;

                                 setLength (TCPSocketPortList, length (TCPSocketPortList) + count);

                                 for j := 1 to count do
                                     begin
                                          entry := trim (getEntry (value, ',', j));

                                          if not isValidSocketPair (entry) then
                                             begin
                                                  setLength (TCPSocketPortList, j-1);
                                                  exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                             end;

                                          // add the string

                                          TCPSocketPortList[j-1].initialize (entry);
                                     end;
                            end
                         else
                         if param='tcpreplaceportsbylist' then
                            begin
                                 count := numEntries (value, ',');

                                 if count > 0 then
                                    TCPReplacePortsByList := true;

                                 setLength (TCPPortList, length (TCPPortList) + count);

                                 for j := 1 to count do
                                     begin
                                          entry := trim (getEntry (value, ',', j));

                                          if not isValidPortPair (entry) then
                                             begin
                                                  setLength (TCPPortList, j-1);
                                                  exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                             end;

                                          // add the string

                                          TCPPortList[j-1].initialize (value);
                                     end;
                            end
                         else
                         if param='tcprandomizeports' then
                            begin
                                 value := lowercase (value);

                                 if (value = 'randomizeall') or (value = 'ignoresystem') or (value = 'keepasis') then
                                    begin
                                         TCPIgnoreSystemPorts := (value = 'ignoresystem');
                                         TCPRandomizePorts := TCPIgnoreSystemPorts or (value = 'randomizeall');
                                    end
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='tcprecalculatecrc' then
                            begin
                                 if StringToRecalculateCRCMode (value) <> rcmUnknown then
                                    TCPRecalculateCRC := StringToRecalculateCRCMode (value)
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='udpreplacesocketsbylist' then
                            begin
                                 count := numEntries (value, ',');

                                 if count > 0 then
                                    UDPReplaceSocketsByList := true;

                                 setLength (UDPSocketPortList, length (UDPSocketPortList) + count);

                                 for j := 1 to count do
                                     begin
                                          entry := trim (getEntry (value, ',', j));

                                          if not isValidSocketPair (entry) then
                                             begin
                                                  setLength (UDPSocketPortList, j-1);
                                                  exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                             end;

                                          // add the string

                                          UDPSocketPortList[j-1].initialize (entry);
                                     end;
                                end
                         else
                         if param='udpreplaceportsbylist' then
                            begin
                                 count := numEntries (value, ',');

                                 if count > 0 then
                                    UDPReplacePortsByList := true;

                                 setLength (UDPPortList, length (TCPPortList) + count);

                                 for j := 1 to count do
                                     begin
                                          entry := trim (getEntry (value, ',', j));

                                          if not isValidPortPair (entry) then
                                             begin
                                                  setLength (UDPPortList, j-1);
                                                  exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                             end;

                                          // add the string

                                          UDPPortList[j-1].initialize (value);
                                     end;
                            end
                         else
                         if param='udprandomizeports' then
                            begin
                                 value := lowercase (value);

                                 if (value = 'randomizeall') or (value = 'ignoresystem') or (value = 'keepasis') then
                                    begin
                                         UDPIgnoreSystemPorts := (value = 'ignoresystem');
                                         UDPRandomizePorts := UDPIgnoreSystemPorts or (value = 'randomizeall');
                                    end
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='udprecalculatecrc' then
                            begin
                                 if StringToRecalculateCRCMode (value) <> rcmUnknown then
                                    UDPRecalculateCRC := StringToRecalculateCRCMode (value)
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='dhcpv4replacehostname' then
                            begin
                                 if StringToDHCPv4ReplaceHostNameMode (value) <> rhnUnknown then
                                    DHCPv4ReplaceHostnameMode := StringToDHCPv4ReplaceHostNameMode (value)
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param='rtpssanitizedata' then
                            begin
                                 value := lowercase (value);

                                 if (value = 'randomize') or (value = 'blackmarker') or (value = 'keepasis') then
                                    begin
                                         RTPSReplacementBlackMarker := (value = 'blackmarker');
                                         RTPSDoReplace := RTPSReplacementBlackMarker or (value = 'randomize');
                                    end
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                         if param = 'advdonotreassembleipfragments' then
                            begin
                                 case getBool (value) of
                                  bpUnknown : exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                                  bpTrue    : AdvDoNotReassembleIPFragments := true;
                                  bpFalse   : AdvDoNotReassembleIPFragments := false;
                                 end;
                            end
                         else
                         if param = 'advreassemblychaintimeoutms' then
                            begin
                                 val (value, iHelp, dummy);

                                 if (iHelp >= 100) and (iHelp <= 10000) then
                                    AdvReassemblyChainTimeoutMS := iHelp
                                 else
                                     exit ('Error in task config file in line ' + i.ToString + ': invalid value');
                            end
                         else
                             exit ('Error in task config file in line ' + i.ToString + ': unknown parameter "' + param + '"');
                    end;

            end;

        assert (false, 'Reminder: need to set all "DoReplace" variables according to active settings');

        result := 'Success';

     finally
            config.Free;
     end;
end;

(* ------------------------------------------- *)

function tAnonymizeParameters.ShortenPayload: boolean;

begin
     result := (PayloadTruncateUnknownLayers and (PayloadRemovalMode = prmCut)) or
               PayloadDoTruncateAfterLayer or PayloadDoTruncateAtOffset;
end;

(* ------------------------------------------- *)

end.
