{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit definition_conversation_types;

interface

uses VirtualTrees,
     base_structures,
     definition_interface_mapping_types,
     definition_ip_addresses,
     definition_tcp_structures;

type // Endpoints

     tMACEndpoint = record
                          MACAddress        : tMACAddress;
                          FramesReceived    : integer;
                          FramesSent        : integer;
                          BytesReceived     : int64;
                          BytesSent         : int64;
                          FirstFrameIndex   : integer;
                          LastFrameIndex    : integer;
                          FirstFrameOffset  : int64;
                          LastFrameOffset   : int64;
                          FirstFrameTime    : tFrameTimeStamp;
                          LastFrameTime     : tFrameTimeStamp;
                          FirstFileID       : int64;
                          LastFileID        : int64;
                          Markers           : tMarkerSet;
                          InterfaceList     : tInterfaceFileIDList;
                          Attributes        : tConversationAttributeSet;
                          IPAddresses       : integer;
                          FlowCount         : integer;

                          // Helpers

                          OriginFileIndex   : integer;
                          OriginTimestamp   : tFrameTimeStamp;

                          procedure initialize (const FileIndex : integer = 0);
                          function DictionaryKey : string;
                          function durationseconds : integer;
                    end;

     tMACEndpointList = array of tMACEndpoint;

     tIPv4Endpoint = record
                           IPv4Address       : tIPv4Address;
                           FramesReceived    : integer;
                           FramesSent        : integer;
                           BytesReceived     : int64;
                           BytesSent         : int64;
                           FirstFrame        : int64;
                           LastFrame         : int64;
                           FirstFrameOffset  : int64;
                           LastFrameOffset   : int64;
                           FirstFrameTime    : tFrameTimeStamp;
                           LastFrameTime     : tFrameTimeStamp;
                           FirstFileID       : integer;
                           LastFileID        : integer;
                           Markers           : tMarkerSet;
                           InterfaceList     : tInterfaceFileIDList;
                           Attributes        : tConversationAttributeSet;
                           PartnerCount      : integer;
                           FlowCount         : integer;
                           ClientCount       : integer;
                           TTL               : byte;

                           // Helpers

                           OriginFileIndex   : integer;
                           OriginTimestamp   : tFrameTimeStamp;

                           procedure initialize (const FileIndex : integer = 0);
                           function DictionaryKey : string;
                           function durationseconds : integer;
                     end;

     tIPv4EndpointList = array of tIPv4Endpoint;

     tIPv6Endpoint = record
                           IPv6Address       : tIPv6Address;
                           FramesReceived    : integer;
                           FramesSent        : integer;
                           BytesReceived     : int64;
                           BytesSent         : int64;
                           FirstFrame        : int64;
                           LastFrame         : int64;
                           FirstFrameOffset  : int64;
                           LastFrameOffset   : int64;
                           FirstFrameTime    : tFrameTimeStamp;
                           LastFrameTime     : tFrameTimeStamp;
                           FirstFileID       : integer;
                           LastFileID        : integer;
                           Markers           : tMarkerSet;
                           InterfaceList     : tInterfaceFileIDList;
                           Attributes        : tConversationAttributeSet;
                           PartnerCount      : integer;
                           Flowcount         : integer;
                           ClientCount       : integer;
                           HopLimit          : byte;

                           // Helpers

                           OriginFileIndex   : integer;
                           OriginTimestamp   : tFrameTimeStamp;

                           procedure initialize (const FileIndex : integer = 0);
                           function DictionaryKey : string;
                           function durationseconds : integer;
                     end;

     tIPv6EndpointList = array of tIPv6Endpoint;

     // Conversations

     tMACConversation = record
                              MACPair           : tMACAddressPair;
                              FramesSrcToDst    : integer;
                              FramesDstToSrc    : integer;
                              BytesSrcToDst     : int64;
                              BytesDstToSrc     : int64;
                              FirstFrame        : int64;
                              LastFrame         : int64;
                              FirstFrameOffset  : int64;
                              LastFrameOffset   : int64;
                              FirstFrameTime    : tFrameTimeStamp;
                              LastFrameTime     : tFrameTimeStamp;
                              FirstFileID       : integer;
                              LastFileID        : integer;
                              Markers           : tMarkerSet;
                              InterfaceList     : tInterfaceFileIDList;
                              Attributes        : tConversationAttributeSet;
                              FlowCount         : integer;

                              // Helpers

                              OriginFileIndex   : integer;
                              OriginTimestamp   : tFrameTimeStamp;
                              OriginAddressPair : tMACAddressPair;

                              procedure initialize (const FileIndex : integer = 0);
                              function DictionaryKey : string;
                              function durationseconds : integer;
                              function durationasstring : string;
                        end;

     tMACConversationList = array of tMACConversation;

     tIPv4Conversation = record
                               IPv4AddressPair   : tIPv4AddressPair;
                               FramesSrcToDst    : integer;
                               FramesDstToSrc    : integer;
                               BytesSrcToDst     : int64;
                               BytesDstToSrc     : int64;
                               FirstFrame        : int64;
                               LastFrame         : int64;
                               FirstFrameOffset  : int64;
                               LastFrameOffset   : int64;
                               FirstFrameTime    : tFrameTimeStamp;
                               LastFrameTime     : tFrameTimeStamp;
                               FirstFileID       : integer;
                               LastFileID        : integer;
                               Markers           : tMarkerSet;
                               InterfaceList     : tInterfaceFileIDList;
                               Attributes        : tConversationAttributeSet;

                               // Calculcated Values

                               MinRTTns          : int64;
                               MaxRTTns          : int64;
                               AggregatedRTTns   : int64;                    // all RTTs added
                               RTTSampleCount    : integer;                  // number of RTT samples, needed to caluclate average
                               FlowCount         : integer;

                               // Helpers

                               OriginFileIndex   : integer;
                               OriginTimestamp   : tFrameTimeStamp;
                               OriginAddressPair : tIPv4AddressPair;

                               procedure initialize (const FileIndex : integer = 0);
                               function DictionaryKey : string;
                               function durationseconds : integer;
                               function durationasstring : string;
                         end;

     tIPv4ConversationList = array of tIPv4Conversation;

     tIPv6Conversation = record
                               IPv6AddressPair   : tIPv6AddressPair;
                               FramesSrcToDst    : integer;
                               FramesDstToSrc    : integer;
                               BytesSrcToDst     : int64;
                               BytesDstToSrc     : int64;
                               FirstFrame        : int64;
                               LastFrame         : int64;
                               FirstFrameOffset  : int64;
                               LastFrameOffset   : int64;
                               FirstFrameTime    : tFrameTimeStamp;
                               LastFrameTime     : tFrameTimeStamp;
                               FirstFileID       : integer;
                               LastFileID        : integer;
                               Markers           : tMarkerSet;
                               InterfaceList     : tInterfaceFileIDList;
                               Attributes        : tConversationAttributeSet;

                               // Calculcated Values

                               MinRTTns          : int64;
                               MaxRTTns          : int64;
                               AggregatedRTTns   : int64;                    // all RTTs added
                               RTTSampleCount    : integer;                  // number of RTT samples, needed to caluclate average
                               FlowCount         : integer;

                               // Helpers

                               OriginFileIndex   : integer;
                               OriginTimestamp   : tFrameTimeStamp;
                               OriginAddressPair : tIPv6AddressPair;

                               procedure initialize (const FileIndex : integer = 0);
                               function DictionaryKey : string;
                               function durationseconds : integer;
                               function durationasstring : string;
                         end;

     tIPv6ConversationList = array of tIPv6Conversation;

     tTCPConversation = record
                              SocketPair         : tIPSocketPair;
                              FramesSrcToDst     : integer;
                              FramesDstToSrc     : integer;
                              BytesSrcToDst      : int64;
                              BytesDstToSrc      : int64;
                              FirstFrame         : int64;
                              LastFrame          : int64;
                              FirstFrameOffset   : int64;
                              LastFrameOffset    : int64;
                              FirstFrameTime     : tFrameTimeStamp;
                              LastFrameTime      : tFrameTimeStamp;
                              FirstFileID        : integer;
                              LastFileID         : integer;
                              SynSeen            : boolean;
                              FinSeen            : boolean;
                              RstSeen            : boolean;
                              Markers            : tMarkerSet;
                              Flags              : tTCPFlagSet;
                              InterfaceList      : tInterfaceFileIDList;
                              Attributes         : tConversationAttributeSet;
                              InitialRTTns       : int64;
                              SessionStatus      : byte;
                              Tunnelflags        : tTunnelFlagSet;
//                            MiscInfo           : string;

                              // Helpers

                              OriginFileIndex    : integer;
                              OriginTimestamp    : tFrameTimeStamp;
                              OriginAddressPair  : tIPSocketPair;
                              ClientIP           : tIPAddress;       // if we see the SYN we know which of the two addresses is the client
                              InitialSequenceNo  : longword;
                              LastSeenSequence   : longword;
                              SynTimestamp       : tFrameTimeStamp;
                              FinTimestamp       : tFrameTimeStamp;
                              HandshakeThird     : longword;           // sequence number of the expected third handshake packet
                              HandshakeThirdTime : tFrameTimeStamp;    // Timestamp of the third handshake packet
                              SynRetries         : integer;
                              HandshakeCount     : integer;
                              PayloadBytes       : integer;
                              ClientPayloadBytes : int64;
                              ServerPayloadBytes : int64;

                              FilterMatchText    : string;
                              Marked             : boolean;

                              procedure initialize (const FileIndex : integer = 0); overload;
                              procedure initialize (IPv4Conversation : tIPv4Conversation); overload;
                              procedure initialize (IPv6Conversation : tIPv6Conversation); overload;
                              procedure UpdateFilterMatchText;
                              function DictionaryKey : string;
                              function durationseconds : integer;
                              function durationasstring : string;
                        end;

     tTCPConversationList = array of tTCPConversation;

     tUDPConversation = record
                               SocketPair        : tIPSocketPair;
                               FramesSrcToDst    : integer;
                               FramesDstToSrc    : integer;
                               BytesSrcToDst     : int64;
                               BytesDstToSrc     : int64;
                               FirstFrame        : int64;
                               LastFrame         : int64;
                               FirstFrameOffset  : int64;
                               LastFrameOffset   : int64;
                               FirstFrameTime    : tFrameTimeStamp;
                               LastFrameTime     : tFrameTimeStamp;
                               FirstFileID       : integer;
                               LastFileID        : integer;
                               Markers           : tMarkerSet;
                               InterfaceList     : tInterfaceFileIDList;
                               Attributes        : tConversationAttributeSet;
                               Tunnelflags       : tTunnelFlagSet;

                               // Helpers

                               OriginFileIndex   : integer;
                               OriginTimestamp   : tFrameTimeStamp;
                               OriginAddressPair : tIPSocketPair;

                               procedure initialize (const FileIndex : integer = 0); overload;
                               procedure initialize (IPv4Conversation : tIPv4Conversation); overload;
                               procedure initialize (IPv6Conversation : tIPv6Conversation); overload;
                               function DictionaryKey : string;
                               function durationseconds : integer;
                               function durationasstring : string;
                         end;
     tUDPConversationList = array of tUDPConversation;

     tNameResolutionEntry = record
                                  FQDN              : tFQDN;
                                  isAnswer          : boolean;
                                  AliasFQDN         : tFQDN;
                                  Answer            : tNameResolutionAnswer;
                                  ReplyCode         : tDNSReturnCode;
                                  FrameCount        : integer;
                                  FirstFrame        : int64;
                                  LastFrame         : int64;
                                  FirstFrameOffset  : int64;
                                  LastFrameOffset   : int64;
                                  FirstFrameTime    : tFrameTimeStamp;
                                  LastFrameTime     : tFrameTimeStamp;
                                  FirstFileID       : integer;
                                  LastFileID        : integer;
                                  QueryTypes        : tDNSRecordTypeSet;
                                  QueryOnly         : boolean;
                                  UniqueClients     : integer;
                                  Markers           : tMarkerSet;
                                  InterfaceList     : tInterfaceFileIDList;
                                  Attributes        : tConversationAttributeSet;

                                  procedure initialize (const FileIndex : integer = 0);
                                  /// <summary>
                                  ///   Returns the dictionary key, consisting
                                  ///   of FQDN, Answer and DNS Replycode
                                  /// </summary>
                                  function DictionaryKey : string;
                            end;

     tNameResolutionList = array of tNameResolutionEntry;

     tConversationCountTable = record
                                     EtherAddressCount : integer;
                                     IPv4AddressCount  : integer;
                                     IPv6AddressCount  : integer;
                                     EtherPairCount    : integer;
                                     IPv4PairCount     : integer;
                                     IPv6PairCount     : integer;
                                     TCPPairCount      : integer;
                                     UDPPairCount      : integer;

                                     procedure initialize;
                                     procedure accumulate (SubCounters : tConversationCountTable);
                               end;

     // Filter stuff

     tConversationFilter = record
                                 ExcludeUpperLayerAddresses : boolean;                            // e.g. do not show IPs that only show up in name resolution etc.

                                 procedure initialize;
                           end;

     // *** Virtual Tree View display entries ***

     tMACEndpointTreeEntry = record
                                   index                  : integer;
                                   Address                : tMACAddress;
                                   Frames                 : integer;
                                   Bytes                  : int64;
                                   FramesSent             : integer;
                                   FramesReceived         : integer;
                                   BytesSent              : int64;
                                   BytesReceived          : int64;
                                   HelpFirstSeen          : tFileFrameLocation;
                                   HelpLastSeen           : tFileFrameLocation;
                                   InterfaceList          : tInterfaceFileIDList;
                                   Attributes             : tConversationAttributeSet;
                                   IPAddresses            : integer;
                                   Flowcount              : integer;

                                   procedure CopyFrom (Value : tMACEndpoint);
                             end;
     pMACEndpointTreeEntry = ^tMACEndpointTreeEntry;

     tIPv4EndpointTreeEntry = record
                                    index                  : integer;
                                    Address                : tIPv4Address;
                                    Packets                : integer;
                                    Bytes                  : int64;
                                    PacketsSent            : integer;
                                    PacketsReceived        : integer;
                                    BytesSent              : int64;
                                    BytesReceived          : int64;
                                    HelpFirstSeen          : tFileFrameLocation;
                                    HelpLastSeen           : tFileFrameLocation;
                                    InterfaceList          : tInterfaceFileIDList;
                                    Attributes             : tConversationAttributeSet;
                                    PartnerCount           : integer;
                                    Flowcount              : integer;

                                    procedure CopyFrom (Value : tIPv4Endpoint);
                              end;
     pIPv4EndpointTreeEntry = ^tIPv4EndpointTreeEntry;

     tIPv6EndpointTreeEntry = record
                                    index                  : integer;
                                    Address                : tIPv6Address;
                                    Packets                : integer;
                                    Bytes                  : int64;
                                    PacketsSent            : integer;
                                    PacketsReceived        : integer;
                                    BytesSent              : int64;
                                    BytesReceived          : int64;
                                    HelpFirstSeen          : tFileFrameLocation;
                                    HelpLastSeen           : tFileFrameLocation;
                                    InterfaceList          : tInterfaceFileIDList;
                                    Attributes             : tConversationAttributeSet;
                                    PartnerCount           : integer;
                                    Flowcount              : integer;

                                    procedure CopyFrom (Value : tIPv6Endpoint);
                              end;
     pIPv6EndpointTreeEntry = ^tIPv6EndpointTreeEntry;

     tMACConversationTreeEntry = record
                                       index               : integer;
                                       AddressA            : tMACAddress;
                                       AddressB            : tMACAddress;
                                       Frames              : integer;
                                       Bytes               : int64;
                                       FramesAtoB          : integer;
                                       FramesBtoA          : integer;
                                       BytesAtoB           : int64;
                                       BytesBtoA           : int64;
                                       DurationSeconds     : integer;
                                       DurationString      : string;
                                       MBpSAtoB            : double;
                                       MBpSBtoA            : double;
                                       HelpFirstSeen       : tFileFrameLocation;
                                       HelpLastSeen        : tFileFrameLocation;
                                       InterfaceList       : tInterfaceFileIDList;
                                       Attributes          : tConversationAttributeSet;
                                       Flowcount           : integer;

                                       procedure CopyFrom (Value : tMACConversation);
                                 end;
     pMACConversationTreeEntry = ^tMACConversationTreeEntry;

     tIPv4ConversationTreeEntry = record
                                        index               : integer;
                                        AddressA            : tIPv4Address;
                                        AddressB            : tIPv4Address;
                                        Packets             : integer;
                                        Bytes               : int64;
                                        PacketsAtoB         : integer;
                                        PacketsBtoA         : integer;
                                        BytesAtoB           : int64;
                                        BytesBtoA           : int64;
                                        DurationSeconds     : integer;
                                        DurationString      : string;
                                        MBpSAtoB            : double;
                                        MBpSBtoA            : double;
                                        MinRTTns            : int64;
                                        MaxRTTns            : int64;
                                        AverageRTTns        : int64;
                                        FlowCount           : integer;
                                        HelpFirstSeen       : tFileFrameLocation;
                                        HelpLastSeen        : tFileFrameLocation;
                                        InterfaceList       : tInterfaceFileIDList;
                                        Attributes          : tConversationAttributeSet;

                                        procedure CopyFrom (Value : tIPv4Conversation);
                                  end;
     pIPv4ConversationTreeEntry = ^tIPv4ConversationTreeEntry;

     tIPv6ConversationTreeEntry = record
                                        index               : integer;
                                        AddressA            : tIPv6Address;
                                        AddressB            : tIPv6Address;
                                        Packets             : integer;
                                        Bytes               : int64;
                                        PacketsAtoB         : integer;
                                        PacketsBtoA         : integer;
                                        BytesAtoB           : int64;
                                        BytesBtoA           : int64;
                                        DurationSeconds     : integer;
                                        DurationString      : string;
                                        MBpSAtoB            : double;
                                        MBpSBtoA            : double;
                                        MinRTTns            : int64;
                                        MaxRTTns            : int64;
                                        AverageRTTns        : int64;
                                        FlowCount           : integer;
                                        HelpFirstSeen       : tFileFrameLocation;
                                        HelpLastSeen        : tFileFrameLocation;
                                        InterfaceList       : tInterfaceFileIDList;
                                        Attributes          : tConversationAttributeSet;

                                        procedure CopyFrom (Value : tIPv6Conversation);
                                  end;
     pIPv6ConversationTreeEntry = ^tIPv6ConversationTreeEntry;

     tTCPConversationTreeEntry = record
                                       index               : integer;
//                                     IPVersion           : byte;
//                                     AddressIPv4A        : tIPv4Address;
//                                     AddressIPv6A        : tIPv6Address;
//                                     AddressIPv4B        : tIPv4Address;
//                                     AddressIPv6B        : tIPv6Address;
                                       AddressA            : tIPAddress;
                                       AddressB            : tIPAddress;
                                       PortA               : word;
                                       PortB               : word;
                                       Packets             : integer;
                                       Bytes               : int64;
                                       PacketsAtoB         : integer;
                                       PacketsBtoA         : integer;
                                       BytesAtoB           : int64;
                                       BytesBtoA           : int64;
                                       PacketRatioAtoB     : integer;
                                       DurationSeconds     : integer;
                                       DurationString      : string;
                                       InitialRTTns        : int64;
                                       SessionStatus       : byte;
                                       MBpSAtoB            : double;
                                       MBpSBtoA            : double;
                                       SynSeen             : boolean;
                                       FinSeen             : boolean;
                                       RstSeen             : boolean;
                                       Flags               : tTCPFlagSet;
                                       FirstTimestamp      : tFrameTimestamp;
                                       LastTimestamp       : tFrameTimestamp;
                                       HelpFirstSeen       : tFileFrameLocation;
                                       HelpLastSeen        : tFileFrameLocation;
                                       InterfaceList       : tInterfaceFileIDList;
                                       Attributes          : tConversationAttributeSet;
                                       Tunnelflags         : tTunnelFlagSet;

                                       IsParent            : boolean;
                                       Marked              : boolean;
//                                     nextMarked          : PVirtualNode;
//                                     previousMarked      : PVirtualNode;

                                       procedure CopyFrom (Value : tTCPConversation; const ItemIndex : integer = 0);
//                                     procedure CopyFrom (Value : tTCPIPv4Conversation; const ItemIndex : integer = 0); Overload;
//                                     procedure CopyFrom (Value : tTCPIPv6Conversation; const ItemIndex : integer = 0); Overload;

                                       function GetIPv4SocketPair : tIPv4SocketPair;
                                       function GetIPv6SocketPair : tIPv6SocketPair;
                                 end;
     pTCPConversationTreeEntry = ^tTCPConversationTreeEntry;

     tUDPConversationTreeEntry = record
                                       index                : integer;
//                                     IPVersion            : byte;
//                                     AddressIPv4A         : tIPv4Address;
//                                     AddressIPv6A         : tIPv6Address;
                                       AddressA             : tIPAddress;
                                       PortA                : word;
//                                     AddressIPv4B         : tIPv4Address;
//                                     AddressIPv6B         : tIPv6Address;
                                       AddressB             : tIPAddress;
                                       PortB                : word;
                                       Packets              : integer;
                                       Bytes                : int64;
                                       PacketsAtoB          : integer;
                                       PacketsBtoA          : integer;
                                       BytesAtoB            : int64;
                                       BytesBtoA            : int64;
                                       DurationSeconds      : integer;
                                       DurationString       : string;
                                       MBpSAtoB             : double;
                                       MBpSBtoA             : double;
                                       PacketsPerSecondAtoB : integer;
                                       PacketsPerSecondBtoA : integer;
                                       DeltaTime            : tFrameDeltaTime;
                                       FirstTimestamp       : tFrameTimestamp;
                                       LastTimestamp        : tFrameTimestamp;
                                       HelpFirstSeen        : tFileFrameLocation;
                                       HelpLastSeen         : tFileFrameLocation;
                                       InterfaceList        : tInterfaceFileIDList;
                                       Attributes           : tConversationAttributeSet;
                                       IsParent             : boolean;
                                       Tunnelflags          : tTunnelFlagSet;

                                       procedure CopyFrom (Value : tUDPConversation); Overload;

                                       function GetIPv4SocketPair : tIPv4SocketPair;
                                       function GetIPv6SocketPair : tIPv6SocketPair;
                                 end;
     pUDPConversationTreeEntry = ^tUDPConversationTreeEntry;

     tSocketDetails        = record
                                   IPSocket      : tIPSocket;
                                   UseCount      : integer;
                                   NumPacketsIn  : longword;
                                   NumPacketsOut : longword;
                                   Firstseen     : tFrametimestamp;
                                   LastSeen      : tFrametimestamp;

                                   procedure initialize;
                             end;
     pSocketDetails = ^tSocketDetails;


     function containsLiveNetworkMarker (MarkerSet : tMarkerSet) : boolean;
     function GetAttributeDescription (const attributes : tConversationAttributeSet) : string;
     function GetTCPSessionText (const SessionStatus : byte) : string;
     function GetTCPStatusDescription (const SessionStatus : byte; const attributes : tConversationAttributeSet) : string;
     function AttributesToLong (Attributes : tConversationAttributeSet) : longword;
     function LongToAttributes (Value : Longword) : tConversationAttributeSet;


const TCPHandshakeNone = 0;
      TCPHandshakeSYNSent = 1;
      TCPHandshakeSYNACKSent = 2;
      TCPHandshakeSYNTwoWay = 3;
      TCPHandshakeHalfOpen = 4;
      TCPHandshakeSYNReset = 5;
      TCPHandshakeComplete = 6;
      TCPHandshakeAndTeardown = 7;
      TCPTeardown = 8;
      TCPHalfOpenReset = 9;
      TCPSocketRefused = 10;
      TCPSocketCloseByInactivity = 11;

implementation

uses SysUtils,
     DateUtils,
     Math,
     Toolkit;

const
//    MByteDivider = 1048576;
      MByteDivider = 1000000;

(* ------------------------------------------- *)

function containsLiveNetworkMarker (MarkerSet : tMarkerSet) : boolean;

begin
     result := (mFoundByEthernet in MarkerSet) or
               (mFoundByIPv4 in MarkerSet) or
               (mFoundByIPv6 in MarkerSet);
end;

(* ------------------------------------------- *)

function GetAttributeDescription (const attributes : tConversationAttributeSet) : string;

var value : string;

begin
     if (caTimeOutOfOrder in Attributes) then
        value := 'Frames out-of-order'
     else
         value := '';

     if (caSpansMultipleFiles in Attributes) then
        begin
             if value = '' then
                value := 'spans multiple files'
             else
                 value := value + ', spans multiple files';
        end;

     if (caEthernetTrailerPresent in Attributes) then
        begin
             if value = '' then
                value := 'has Ethernet trailer'
             else
                 value := value + ', has Ethernet trailer';
        end;

     if (caIPFragmented in Attributes) then
        begin
             if value = '' then
                value := 'IP fragmented'
             else
                 value := value + ', IP fragmented';
        end;

     if (caHardSliced in Attributes) then
             if value = '' then
                value := 'HardSliced'
             else
                 value := value + ', HardSliced';

     result := value;
end;

(* ------------------------------------------- *)

function GetTCPSessionText (const SessionStatus : byte) : string;

var value : string;

begin
     value := '';

     case SessionStatus of
      TCPHandshakeNone           : value := 'No Handshake or teardown';
      TCPHandshakeSYNSent        : value := 'SYN sent';
      TCPHandshakeSYNACKSent     : value := 'SYN/ACK sent';
      TCPHandshakeSYNTwoWay      : value := 'SYN Two Way';
      TCPHandshakeHalfOpen       : value := 'TCP half-open';
      TCPHandshakeSYNReset       : value := 'SYN rejected';
      TCPHandshakeComplete       : value := 'Handshake complete';
      TCPHandshakeAndTeardown    : value := 'Handshake and Teardown';
      TCPTeardown                : value := 'Teardown';
      TCPHalfOpenReset           : value := 'TCP half-open then reset';
      TCPSocketRefused           : value := 'Client IP refused by server application';
      TCPSocketCloseByInactivity : value := 'New socket closed due to client inactivity';
     end;

     result := value;
end;

(* ------------------------------------------- *)

function GetTCPStatusDescription (const SessionStatus : byte; const attributes : tConversationAttributeSet) : string;

var value : string;

begin
     if (caTimeOutOfOrder in Attributes) then
        value := 'Frames out-of-order'
     else
         value := GetTCPSessionText (SessionStatus);

     if (caTCPSYNReset in Attributes) then
        begin
             case SessionStatus of
              TCPHandshakeComplete    : value := 'Handshake complete after initial SYN reject';
              TCPHandshakeAndTeardown : value := 'Handshake and Teardown after initial SYN reject';
             end;
        end
     else
         if (caTCPMultiSYNRetry in Attributes) then
            begin
                 case SessionStatus of
                  TCPHandshakeComplete    : value := 'Handshake complete after multi SYN retry';
                  TCPHandshakeAndTeardown : value := 'Handshake and Teardown after multi SYN retry';
                 end;
            end;

     if (caTCPSYNBlocked in attributes) then
        value := value + ', SYN blocked';

     if (caTCPPortReused in Attributes) then
        value := value + ', port reused';

     if (caSpansMultipleFiles in Attributes) then
        value := value + ', spans multiple files';

     if (caEthernetTrailerPresent in Attributes) then
        value := value + ', has Ethernet trailer';

     if (caIPFragmented in Attributes) then
        value := value + ', IP fragmented';

     if (caHardSliced in Attributes) then
        value := value + ', HardSliced';

     if (caMultiTTL in Attributes) then
        value := value + ', MultiTTL';

{
     if (caAddressPairFlipped in Attributes) then               disabled because we show it in the correct order (flipped or not, as found first)
        value := value + ', flipped';
}
     result := value;
end;

(* ------------------------------------------- *)

function AttributesToLong (Attributes : tConversationAttributeSet) : longword;

var value : longword;

begin
     // some attributes will not be stored to DB so we don't need it, like
     // caSelected,caSingleFrame

     if caTimeOutOfOrder in Attributes then
        value := 1
     else
         value := 0;

     if caMultipleInterfaces in Attributes then
        value := value + 2;

     if caOneSidedCommunication in Attributes then
        value := value + 4;

     if caSpansMultipleFiles in Attributes then
        value := value + 8;

     if caEthernetTrailerPresent in Attributes then
        value := value + 16;

     if caIPFragmented in Attributes then
        value := value + 32;

     if caTCPMultiSession in Attributes then
        value := value + 64;

     if caTCPMultiSYNRetry in Attributes then
        value := value + 128;

     if caTCPSYNReset in Attributes then
        value := value + 256;

     if caAddressPairFlipped in Attributes then
        value := value + 512;

     if caHardSliced in Attributes then
        value := value + 1024;

     if caTCPPortReused in Attributes then
        value := value + 2048;

     if caTCPSYNBlocked in Attributes then
        value := value + 4096;

     if caNoQuickFilter in Attributes then
        value := value + 8192;

     if caMultiTTL in Attributes then
        value := value + 16384;

     if caSingleFrame in Attributes then
        value := value + 32768;

     // Check if there's unknown flags we forgot to handle

     assert ((value > 0) or (Attributes = []), 'Unknown attribute flag encountered (AttributesToLong)');

     result := value;
end;

(* ------------------------------------------- *)

function LongToAttributes (Value : Longword) : tConversationAttributeSet;

var attr : tConversationAttributeSet;

begin
     if (Value and 1) = 1 then
        attr := [caTimeOutOfOrder]
     else
         attr := [];

     if (Value and 2) = 2 then
        attr := attr + [caMultipleInterfaces];

     if (Value and 4) = 4 then
        attr := attr + [caOneSidedCommunication];

     if (Value and 8) = 8 then
        attr := attr + [caSpansMultipleFiles];

     if (Value and 16) = 16 then
        attr := attr + [caEthernetTrailerPresent];

     if (Value and 32) = 32 then
        attr := attr + [caIPFragmented];

     if (value and 64) = 64 then
        attr := attr + [caTCPMultiSession];

     if (value and 128) = 128 then
        attr := attr + [caTCPMultiSYNRetry];

     if (value and 256) = 256 then
        attr := attr + [caTCPSYNReset];

     if (value and 512) = 512 then
        attr := attr + [caAddressPairFlipped];

     if (value and 1024) = 1024 then
        attr := attr + [caHardSliced];

     if (value and 2048) = 2048 then
        attr := attr + [caTCPPortReused];

     if (value and 4096) = 4096 then
        attr := attr + [caTCPSYNBlocked];

     if (value and 8192) = 8192 then
        attr := attr + [caNoQuickFilter];

     if (value and 16384) = 16384 then
        attr := attr + [caMultiTTL];

     if (value and 32768) = 32768 then
        attr := attr + [caSingleFrame];

     result := attr;
end;

(* ------------------------------------------- *)
{ tMACEndpoint }
(* ------------------------------------------- *)

procedure tMACEndpoint.initialize (const FileIndex : integer = 0);

begin
     MACAddress.initialize;
     FramesReceived := 0;
     FramesSent := 0;
     BytesReceived := 0;
     BytesSent := 0;
     FirstFrameIndex := 0;
     LastFrameIndex := 0;
     FirstFrameOffset := 0;
     LastFrameOffset := 0;
     FirstFrameTime.initialize;
     LastFrameTime.initialize;
     FirstFileID := FileIndex;
     LastFileID := FileIndex;
     Markers := [];
     setLength (InterfaceList, 0);
     Attributes := [];
     IPAddresses := 0;
     Flowcount := 0;
     OriginFileIndex := FileIndex;
     OriginTimestamp.initialize;
end;

(* ------------------------------------------- *)

function tMACEndpoint.DictionaryKey: string;

begin
     result := MACAddress.DictionaryKey;
end;

(* ------------------------------------------- *)

function tMACEndpoint.durationseconds: integer;

begin
     result := secondsbetween (LastFrameTime.datetime, FirstFrameTime.datetime);
end;

(* ------------------------------------------- *)
{ tIPv4Endpoint }
(* ------------------------------------------- *)

procedure tIPv4Endpoint.initialize (const FileIndex : integer = 0);

begin
     IPv4Address.initialize;
     FramesReceived := 0;
     FramesSent := 0;
     BytesReceived := 0;
     BytesSent := 0;
     FirstFrame := 0;
     LastFrame := 0;
     FirstFrameOffset := 0;
     LastFrameOffset := 0;
     FirstFrameTime.initialize;
     LastFrameTime.initialize;
     FirstFileID := FileIndex;
     LastFileID := FileIndex;
     Markers := [];
     setLength (InterfaceList, 0);
     Attributes := [];
     PartnerCount := 0;
     Flowcount := 0;
     ClientCount := 0;
     TTL := 0;
     OriginFileIndex := FileIndex;
     OriginTimestamp.initialize;
end;

(* ------------------------------------------- *)

function tIPv4Endpoint.DictionaryKey: string;

begin
     result := IPv4Address.DictionaryKey;
end;

(* ------------------------------------------- *)

function tIPv4Endpoint.durationseconds: integer;

begin
     result := secondsbetween (LastFrameTime.datetime, FirstFrameTime.datetime);
end;

(* ------------------------------------------- *)
{ tIPv6Endpoint }
(* ------------------------------------------- *)

procedure tIPv6Endpoint.initialize (const FileIndex : integer = 0);

begin
     IPv6Address.initialize;
     FramesReceived := 0;
     FramesSent := 0;
     BytesReceived := 0;
     BytesSent := 0;
     FirstFrame := 0;
     LastFrame := 0;
     FirstFrameOffset := 0;
     LastFrameOffset := 0;
     FirstFrameTime.initialize;
     LastFrameTime.initialize;
     FirstFileID := FileIndex;
     LastFileID := FileIndex;
     Markers := [];
     setLength (InterfaceList, 0);
     Attributes := [];
     PartnerCount := 0;
     Flowcount := 0;
     ClientCount := 0;
     HopLimit := 0;
     OriginFileIndex := FileIndex;
     OriginTimestamp.initialize;
end;

(* ------------------------------------------- *)

function tIPv6Endpoint.DictionaryKey: string;

begin
     result := IPv6Address.DictionaryKey;
end;

(* ------------------------------------------- *)

function tIPv6Endpoint.durationseconds: integer;

begin
     result := secondsbetween (LastFrameTime.datetime, FirstFrameTime.datetime);
end;

(* ------------------------------------------- *)
{ tMACConversation }
(* ------------------------------------------- *)

procedure tMACConversation.initialize (const FileIndex : integer = 0);

begin
     MACPair.initialize;
     FramesSrcToDst := 0;
     FramesDstToSrc := 0;
     BytesSrcToDst := 0;
     BytesDstToSrc := 0;
     FirstFrame := 0;
     LastFrame := 0;
     FirstFrameOffset := 0;
     LastFrameOffset := 0;
     FirstFrameTime.initialize;
     LastFrameTime.initialize;
     FirstFileID := FileIndex;
     LastFileID := FileIndex;
     Markers := [];
     setLength (InterfaceList, 0);
     Attributes := [];
     OriginFileIndex := FileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;
     FlowCount := 0;
end;

(* ------------------------------------------- *)

function tMACConversation.durationseconds: integer;

begin
     result := secondsbetween (LastFrameTime.datetime, FirstFrameTime.datetime);
end;

(* ------------------------------------------- *)

function tMACConversation.DictionaryKey: string;

begin
     result := MACPair.DictionaryKey;
end;

(* ------------------------------------------- *)

function tMACConversation.durationasstring: string;

begin
     result := CaptureDurationAsString (LastFrameTime, FirstFrameTime);
end;

(* ------------------------------------------- *)
{ tIPv4Conversation }
(* ------------------------------------------- *)

procedure tIPv4Conversation.initialize (const FileIndex : integer = 0);

begin
     IPv4AddressPair.initialize;
     FramesSrcToDst := 0;
     FramesDstToSrc := 0;
     BytesSrcToDst := 0;
     BytesDstToSrc := 0;
     FirstFrame := 0;
     LastFrame := 0;
     FirstFrameOffset := 0;
     LastFrameOffset := 0;
     FirstFrameTime.initialize;
     LastFrameTime.initialize;
     FirstFileID := FileIndex;
     LastFileID := FileIndex;
     Markers := [];
     setLength (InterfaceList, 0);
     Attributes := [];
     MinRTTns := 0;
     MaxRTTns := 0;
     AggregatedRTTns := 0;
     RTTSampleCount := 0;
     FlowCount := 0;
     OriginFileIndex := FileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;
end;

(* ------------------------------------------- *)

function tIPv4Conversation.DictionaryKey: string;

begin
     result := IPv4AddressPair.DictionaryKey;
end;

(* ------------------------------------------- *)

function tIPv4Conversation.durationasstring: string;

begin
     result := CaptureDurationAsString (LastFrameTime, FirstFrameTime);
end;

(* ------------------------------------------- *)

function tIPv4Conversation.durationseconds: integer;

begin
     result := secondsbetween (LastFrameTime.datetime, FirstFrameTime.datetime);
end;

(* ------------------------------------------- *)
{ tIPv6Conversation }
(* ------------------------------------------- *)

procedure tIPv6Conversation.initialize (const FileIndex : integer = 0);

begin
     IPv6AddressPair.initialize;
     FramesSrcToDst := 0;
     FramesDstToSrc := 0;
     BytesSrcToDst := 0;
     BytesDstToSrc := 0;
     FirstFrame := 0;
     LastFrame := 0;
     FirstFrameOffset := 0;
     LastFrameOffset := 0;
     FirstFrameTime.initialize;
     LastFrameTime.initialize;
     FirstFileID := FileIndex;
     LastFileID := FileIndex;
     Markers := [];
     setLength (InterfaceList, 0);
     Attributes := [];
     MinRTTns := 0;
     MaxRTTns := 0;
     AggregatedRTTns := 0;
     RTTSampleCount := 0;
     FlowCount := 0;
     OriginFileIndex := FileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;
end;

(* ------------------------------------------- *)

function tIPv6Conversation.DictionaryKey: string;

begin
     result := IPv6AddressPair.DictionaryKey;
end;

(* ------------------------------------------- *)

function tIPv6Conversation.durationasstring: string;

begin
     result := CaptureDurationAsString (LastFrameTime, FirstFrameTime);
end;

(* ------------------------------------------- *)

function tIPv6Conversation.durationseconds: integer;

begin
     result := secondsbetween (LastFrameTime.datetime, FirstFrameTime.datetime);
end;

(* ------------------------------------------- *)
{ tITCPIPv4Conversation }
(* ------------------------------------------- *)
{
procedure tTCPIPv4Conversation.initialize (const FileIndex : integer = 0);

begin
     SocketPair.initialize;
     FramesSrcToDst := 0;
     FramesDstToSrc := 0;
     BytesSrcToDst := 0;
     BytesDstToSrc := 0;
     FirstFrame := 0;
     LastFrame := 0;
     FirstFrameOffset := 0;
     LastFrameOffset := 0;
     FirstFrameTime.initialize;
     LastFrameTime.initialize;
     FirstFileID := FileIndex;
     LastFileID := FileIndex;
     SynSeen := false;
     FinSeen := false;
     RstSeen := false;
     Markers := [];
     Flags := [];
     setLength (InterfaceList, 0);
     Attributes := [];
     InitialRTTns := 0;
//   MiscInfo := '';
     SessionStatus := 0;

     ClientIP.initialize;
     SynTimestamp.initialize;
     FinTimestamp.initialize;
     InitialSequenceNo := 0;
     LastSeenSequence := 0;
     HandshakeThird := 0;
     HandshakeThirdTime.initialize;
     HandshakeCount := 0;
     SynRetries := 0;
     OriginFileIndex := FileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;

     PayloadBytes := 0;
     ClientPayloadBytes := 0;
     ServerPayloadBytes := 0;

     FilterMatchText := '';
     Marked := False;
end;

(* ------------------------------------------- *)

procedure tTCPIPv4Conversation.initialize (IPv4Conversation: tIPv4Conversation);

begin
     self.initialize;
     SocketPair.Source.IPAddress := IPv4Conversation.IPv4AddressPair.Source;
     SocketPair.Destination.IPAddress := IPv4Conversation.IPv4AddressPair.Destination;

     FramesSrcToDst := IPv4Conversation.FramesSrcToDst;
     FramesDstToSrc := IPv4Conversation.FramesDstToSrc;
     BytesSrcToDst := IPv4Conversation.BytesSrcToDst;
     BytesDstToSrc := IPv4Conversation.BytesDstToSrc;
     FirstFrame := IPv4Conversation.FirstFrame;
     LastFrame := IPv4Conversation.LastFrame;
     FirstFrameTime := IPv4Conversation.FirstFrameTime;
     LastFrameTime := IPv4Conversation.LastFrameTime;
     FirstFileID := IPv4Conversation.FirstFileID;
     LastFileID := IPv4Conversation.LastFileID;
     copyInterfaceEntryList (IPv4Conversation.InterfaceList, InterfaceList);
     Markers := IPv4Conversation.Markers;
     OriginFileIndex := IPv4Conversation.OriginFileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;

     PayloadBytes := 0;
     ClientPayloadBytes := 0;
     ServerPayloadBytes := 0;
end;

(* ------------------------------------------- *)

procedure tTCPIPv4Conversation.UpdateFilterMatchText;

begin
     FilterMatchText := SocketPair.ValueAsString (true) + '-' + lowercase (GetTCPStatusDescription (SessionStatus, Attributes));
end;

(* ------------------------------------------- *)

function tTCPIPv4Conversation.DictionaryKey: string;

begin
     result := SocketPair.DictionaryKey;
end;

(* ------------------------------------------- *)

function tTCPIPv4Conversation.durationasstring: string;

begin
     result := CaptureDurationAsString (LastFrameTime, FirstFrameTime);
end;

(* ------------------------------------------- *)

function tTCPIPv4Conversation.durationseconds: integer;

begin
     result := secondsbetween (LastFrameTime.datetime, FirstFrameTime.datetime);
end;
}
(* ------------------------------------------- *)
{ tTCPIPv6Conversation }
(* ------------------------------------------- *)
{
procedure tTCPIPv6Conversation.initialize (const FileIndex : integer = 0);

begin
     SocketPair.initialize;
     FramesSrcToDst := 0;
     FramesDstToSrc := 0;
     BytesSrcToDst := 0;
     BytesDstToSrc := 0;
     FirstFrameTime.initialize;
     LastFrameTime.initialize;
     FirstFileID := FileIndex;
     LastFileID := FileIndex;
     SynSeen := false;
     FinSeen := false;
     RstSeen := false;
     Markers := [];
     Flags := [];
     setLength (InterfaceList, 0);
     Attributes := [];
     InitialRTTns := 0;
     SessionStatus := 0;
//   MiscInfo := '';

     ClientIP.initialize;
     SynTimestamp.initialize;
     FinTimestamp.initialize;
     InitialSequenceNo := 0;
     LastSeenSequence := 0;
     HandshakeThird := 0;
     HandshakeThirdTime.initialize;
     HandshakeCount := 0;
     SynRetries := 0;
     OriginFileIndex := FileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;

     PayloadBytes := 0;
     ClientPayloadBytes := 0;
     ServerPayloadBytes := 0;

     FilterMatchText := '';

     Marked := False;
end;

(* ------------------------------------------- *)

procedure tTCPIPv6Conversation.initialize (IPv6Conversation: tIPv6Conversation);

begin
     self.initialize;
     SocketPair.Source.IPAddress := IPv6Conversation.IPv6AddressPair.Source;
     SocketPair.Destination.IPAddress := IPv6Conversation.IPv6AddressPair.Destination;

     FramesSrcToDst := IPv6Conversation.FramesSrcToDst;
     FramesDstToSrc := IPv6Conversation.FramesDstToSrc;
     BytesSrcToDst := IPv6Conversation.BytesSrcToDst;
     BytesDstToSrc := IPv6Conversation.BytesDstToSrc;
     FirstFrame := IPv6Conversation.FirstFrame;
     LastFrame := IPv6Conversation.LastFrame;
     FirstFrameTime := IPv6Conversation.FirstFrameTime;
     LastFrameTime := IPv6Conversation.LastFrameTime;
     FirstFileID := IPv6Conversation.FirstFileID;
     LastFileID := IPv6Conversation.LastFileID;
     copyInterfaceEntryList (IPv6Conversation.InterfaceList, InterfaceList);
     Markers := IPv6Conversation.Markers;
     OriginFileIndex := IPv6Conversation.OriginFileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;

     PayloadBytes := 0;
     ClientPayloadBytes := 0;
     ServerPayloadBytes := 0;
end;

(* ------------------------------------------- *)

procedure tTCPIPv6Conversation.UpdateFilterMatchText;

begin
     FilterMatchText := SocketPair.ValueAsString (true) + '-' + lowercase (GetTCPStatusDescription (SessionStatus, Attributes));
end;

(* ------------------------------------------- *)

function tTCPIPv6Conversation.DictionaryKey: string;

begin
     result := SocketPair.DictionaryKey;
end;

(* ------------------------------------------- *)

function tTCPIPv6Conversation.durationasstring: string;

begin
     result := CaptureDurationAsString (LastFrameTime, FirstFrameTime);
end;

(* ------------------------------------------- *)

function tTCPIPv6Conversation.durationseconds: integer;

begin
     result := secondsbetween (LastFrameTime.datetime, FirstFrameTime.datetime);
end;
}
(* ------------------------------------------- *)
{ tTCPConversation }
(* ------------------------------------------- *)

procedure tTCPConversation.initialize (const FileIndex: integer);

begin
     SocketPair.initialize;
     FramesSrcToDst := 0;
     FramesDstToSrc := 0;
     BytesSrcToDst := 0;
     BytesDstToSrc := 0;
     FirstFrame := 0;
     LastFrame := 0;
     FirstFrameOffset := 0;
     LastFrameOffset := 0;
     FirstFrameTime.initialize;
     LastFrameTime.initialize;
     FirstFileID := FileIndex;
     LastFileID := FileIndex;
     SynSeen := false;
     FinSeen := false;
     RstSeen := false;
     Markers := [];
     Flags := [];
     setLength (InterfaceList, 0);
     Attributes := [];
     InitialRTTns := 0;
//   MiscInfo := '';
     SessionStatus := 0;
     Tunnelflags := [];

     ClientIP.initialize;
     SynTimestamp.initialize;
     FinTimestamp.initialize;
     InitialSequenceNo := 0;
     LastSeenSequence := 0;
     HandshakeThird := 0;
     HandshakeThirdTime.initialize;
     HandshakeCount := 0;
     SynRetries := 0;
     OriginFileIndex := FileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;

     PayloadBytes := 0;
     ClientPayloadBytes := 0;
     ServerPayloadBytes := 0;

     FilterMatchText := '';
     Marked := False;
end;

(* ------------------------------------------- *)

procedure tTCPConversation.initialize (IPv4Conversation: tIPv4Conversation);

begin
     self.initialize;
     SocketPair.Source.IPAddress.initialize (IPv4Conversation.IPv4AddressPair.Source);
     SocketPair.Destination.IPAddress.initialize (IPv4Conversation.IPv4AddressPair.Destination);

     FramesSrcToDst := IPv4Conversation.FramesSrcToDst;
     FramesDstToSrc := IPv4Conversation.FramesDstToSrc;
     BytesSrcToDst := IPv4Conversation.BytesSrcToDst;
     BytesDstToSrc := IPv4Conversation.BytesDstToSrc;
     FirstFrame := IPv4Conversation.FirstFrame;
     LastFrame := IPv4Conversation.LastFrame;
     FirstFrameTime := IPv4Conversation.FirstFrameTime;
     LastFrameTime := IPv4Conversation.LastFrameTime;
     FirstFileID := IPv4Conversation.FirstFileID;
     LastFileID := IPv4Conversation.LastFileID;
     copyInterfaceEntryList (IPv4Conversation.InterfaceList, InterfaceList);
     Markers := IPv4Conversation.Markers;
     OriginFileIndex := IPv4Conversation.OriginFileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;
end;

(* ------------------------------------------- *)

procedure tTCPConversation.initialize (IPv6Conversation: tIPv6Conversation);

begin
     self.initialize;
     SocketPair.Source.IPAddress.initialize (IPv6Conversation.IPv6AddressPair.Source);
     SocketPair.Destination.IPAddress.initialize (IPv6Conversation.IPv6AddressPair.Destination);

     FramesSrcToDst := IPv6Conversation.FramesSrcToDst;
     FramesDstToSrc := IPv6Conversation.FramesDstToSrc;
     BytesSrcToDst := IPv6Conversation.BytesSrcToDst;
     BytesDstToSrc := IPv6Conversation.BytesDstToSrc;
     FirstFrame := IPv6Conversation.FirstFrame;
     LastFrame := IPv6Conversation.LastFrame;
     FirstFrameTime := IPv6Conversation.FirstFrameTime;
     LastFrameTime := IPv6Conversation.LastFrameTime;
     FirstFileID := IPv6Conversation.FirstFileID;
     LastFileID := IPv6Conversation.LastFileID;
     copyInterfaceEntryList (IPv6Conversation.InterfaceList, InterfaceList);
     Markers := IPv6Conversation.Markers;
     OriginFileIndex := IPv6Conversation.OriginFileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;
end;

(* ------------------------------------------- *)

function tTCPConversation.DictionaryKey: string;

begin
     result := SocketPair.DictionaryKey;
end;

(* ------------------------------------------- *)

function tTCPConversation.durationasstring: string;

begin
     result := CaptureDurationAsString (LastFrameTime, FirstFrameTime);
end;

(* ------------------------------------------- *)

function tTCPConversation.durationseconds: integer;

begin
     result := secondsbetween (LastFrameTime.datetime, FirstFrameTime.datetime);
end;

(* ------------------------------------------- *)

procedure tTCPConversation.UpdateFilterMatchText;

begin
     FilterMatchText := SocketPair.ValueAsString (true) + '-' + lowercase (GetTCPStatusDescription (SessionStatus, Attributes));
end;

(* ------------------------------------------- *)
{ tUDPIPv4Conversation }
(* ------------------------------------------- *)
{
procedure tUDPIPv4Conversation.initialize (const FileIndex : integer = 0);

begin
     SocketPair.initialize;
     FramesSrcToDst := 0;
     FramesDstToSrc := 0;
     BytesSrcToDst := 0;
     BytesDstToSrc := 0;
     FirstFrameTime.initialize;
     LastFrameTime.initialize;
     FirstFileID := FileIndex;
     LastFileID := FileIndex;
     Markers := [];
     setLength (InterfaceList, 0);
     Attributes := [];
     OriginFileIndex := FileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;
end;

(* ------------------------------------------- *)

procedure tUDPIPv4Conversation.initialize (IPv4Conversation: tIPv4Conversation);

begin
     SocketPair.initialize;
     SocketPair.Source.IPAddress := IPv4Conversation.IPv4AddressPair.Source;
     SocketPair.Destination.IPAddress := IPv4Conversation.IPv4AddressPair.Destination;

     FramesSrcToDst := IPv4Conversation.FramesSrcToDst;
     FramesDstToSrc := IPv4Conversation.FramesDstToSrc;
     BytesSrcToDst := IPv4Conversation.BytesSrcToDst;
     BytesDstToSrc := IPv4Conversation.BytesDstToSrc;
     FirstFrame := IPv4Conversation.FirstFrame;
     LastFrame := IPv4Conversation.LastFrame;
     FirstFrameTime := IPv4Conversation.FirstFrameTime;
     LastFrameTime := IPv4Conversation.LastFrameTime;
     FirstFileID := IPv4Conversation.FirstFileID;
     LastFileID := IPv4Conversation.LastFileID;
     copyInterfaceEntryList (IPv4Conversation.InterfaceList, InterfaceList);
     Markers := IPv4Conversation.Markers;
     OriginFileIndex := IPv4Conversation.OriginFileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;

end;

(* ------------------------------------------- *)

function tUDPIPv4Conversation.DictionaryKey: string;

begin
     result := SocketPair.DictionaryKey;
end;

(* ------------------------------------------- *)

function tUDPIPv4Conversation.durationasstring: string;

begin
     result := CaptureDurationAsString (LastFrameTime, FirstFrameTime);
end;

(* ------------------------------------------- *)

function tUDPIPv4Conversation.durationseconds: integer;

begin
     result := secondsbetween (LastFrameTime.datetime, FirstFrameTime.datetime);
end;
}
(* ------------------------------------------- *)
{ tTCPIPv6Conversation }
(* ------------------------------------------- *)
{
procedure tUDPIPv6Conversation.initialize (const FileIndex : integer = 0);

begin
     SocketPair.initialize;
     FramesSrcToDst := 0;
     FramesDstToSrc := 0;
     BytesSrcToDst := 0;
     BytesDstToSrc := 0;
     FirstFrameTime.initialize;
     LastFrameTime.initialize;
     FirstFileID := FileIndex;
     LastFileID := FileIndex;
     Markers := [];
     setLength (InterfaceList, 0);
     Attributes := [];
     OriginFileIndex := FileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;
end;

(* ------------------------------------------- *)

procedure tUDPIPv6Conversation.initialize (IPv6Conversation: tIPv6Conversation);

begin
     SocketPair.initialize;
     SocketPair.Source.IPAddress := IPv6Conversation.IPv6AddressPair.Source;
     SocketPair.Destination.IPAddress := IPv6Conversation.IPv6AddressPair.Destination;

     FramesSrcToDst := IPv6Conversation.FramesSrcToDst;
     FramesDstToSrc := IPv6Conversation.FramesDstToSrc;
     BytesSrcToDst := IPv6Conversation.BytesSrcToDst;
     BytesDstToSrc := IPv6Conversation.BytesDstToSrc;
     FirstFrame := IPv6Conversation.FirstFrame;
     LastFrame := IPv6Conversation.LastFrame;
     FirstFrameTime := IPv6Conversation.FirstFrameTime;
     LastFrameTime := IPv6Conversation.LastFrameTime;
     FirstFileID := IPv6Conversation.FirstFileID;
     LastFileID := IPv6Conversation.LastFileID;
     copyInterfaceEntryList (IPv6Conversation.InterfaceList, InterfaceList);
     Markers := IPv6Conversation.Markers;
     OriginFileIndex := IPv6Conversation.OriginFileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;
end;

(* ------------------------------------------- *)

function tUDPIPv6Conversation.DictionaryKey: string;

begin
     result := SocketPair.DictionaryKey;
end;

(* ------------------------------------------- *)

function tUDPIPv6Conversation.durationasstring: string;

begin
     result := CaptureDurationAsString (LastFrameTime, FirstFrameTime);
end;

(* ------------------------------------------- *)

function tUDPIPv6Conversation.durationseconds: integer;

begin
     result := secondsbetween (LastFrameTime.datetime, FirstFrameTime.datetime);
end;
}
(* ------------------------------------------- *)
{ tNameResolutionEntry }
(* ------------------------------------------- *)

function tNameResolutionEntry.DictionaryKey: string;

begin
     if isAnswer then
        result := lowercase (FQDN.FullFQDN) + '-1-' + Answer.DictionaryKey + '-' + IntToStr (DNSReplyCodeToWord (ReplyCode))
     else
         result := lowercase (FQDN.FullFQDN) + '-0-' + Answer.DictionaryKey + '-' + IntToStr (DNSReplyCodeToWord (ReplyCode));
end;

(* ------------------------------------------- *)

procedure tNameResolutionEntry.initialize (const FileIndex : integer = 0);

begin
     FQDN.initialize;
     isAnswer := false;
     AliasFQDN.initialize;
     Answer.initialize;
     ReplyCode := dnsrcNoError;
     FrameCount := 0;
     FirstFrame := 0;
     LastFrame := 0;
     FirstFrameOffset := 0;
     LastFrameOffset := 0;
     FirstFrameTime.initialize;
     LastFrameTime.initialize;
     FirstFileID := FileIndex;
     LastFileID := FileIndex;
     UniqueClients := 0;
     Markers := [];
     setLength (InterfaceList, 0);
     Attributes := [];
end;

(* ------------------------------------------- *)
{ tConversationCountTable }
(* ------------------------------------------- *)

procedure tConversationCountTable.initialize;

begin
     EtherAddressCount := 0;
     IPv4AddressCount := 0;
     IPv6AddressCount := 0;
     EtherPairCount := 0;
     IPv4PairCount := 0;
     IPv6PairCount := 0;
     TCPPairCount := 0;
     UDPPairCount := 0;
end;

(* ------------------------------------------- *)

procedure tConversationCountTable.accumulate (SubCounters : tConversationCountTable);

begin
     EtherAddressCount := EtherAddressCount + SubCounters.EtherAddressCount;
     IPv4AddressCount := IPv4AddressCount + SubCounters.IPv4AddressCount;
     IPv6AddressCount := IPv6AddressCount + SubCounters.IPv6AddressCount;
     EtherPairCount := EtherPairCount + SubCounters.EtherPairCount;
     IPv4PairCount := IPv4PairCount + SubCounters.IPv4PairCount;
     IPv6PairCount := IPv6PairCount + SubCounters.IPv6PairCount;
     TCPPairCount := TCPPairCount + SubCounters.TCPPairCount;
     UDPPairCount := UDPPairCount + SubCounters.UDPPairCount;
end;

(* ------------------------------------------- *)
{ tMACEndpointTreeEntry }
(* ------------------------------------------- *)

procedure tMACEndpointTreeEntry.CopyFrom (Value: tMACEndpoint);

begin
     index := 0;
     Address := Value.MACAddress;
     Frames := Value.FramesReceived + Value.FramesSent;
     Bytes := Value.BytesReceived + value.BytesSent;
     FramesSent := Value.FramesSent;
     FramesReceived := Value.FramesReceived;
     BytesSent := value.BytesSent;
     BytesReceived := value.BytesReceived;

     HelpFirstSeen.FileIndex := value.FirstFileID;
     HelpFirstSeen.FileFrameIndex := value.FirstFrameIndex;
     HelpFirstSeen.FileOffset := 0;
     HelpLastSeen.FileIndex := value.LastFileID;
     HelpLastSeen.FileFrameIndex := value.LastFrameIndex;
     HelpLastSeen.FileOffset := 0;

     copyInterfaceEntryList (Value.InterfaceList, InterfaceList);

     Attributes := value.Attributes;
     IPAddresses := Value.IPAddresses;
     Flowcount := Value.Flowcount;
end;

(* ------------------------------------------- *)
{ tIPv4EndpointTreeEntry }
(* ------------------------------------------- *)

procedure tIPv4EndpointTreeEntry.CopyFrom (Value: tIPv4Endpoint);

begin
     index := 0;
     Address := Value.IPv4Address;
     Packets := Value.FramesReceived + Value.FramesSent;
     Bytes := Value.BytesReceived + value.BytesSent;
     PacketsSent := Value.FramesSent;
     PacketsReceived := Value.FramesReceived;
     BytesSent := value.BytesSent;
     BytesReceived := value.BytesReceived;

     HelpFirstSeen.FileIndex := value.FirstFileID;
     HelpFirstSeen.FileFrameIndex := value.FirstFrame;
     HelpFirstSeen.FileOffset := 0;
     HelpLastSeen.FileIndex := value.LastFileID;
     HelpLastSeen.FileFrameIndex := value.LastFrame;
     HelpLastSeen.FileOffset := 0;

     copyInterfaceEntryList (Value.InterfaceList, InterfaceList);

     Attributes := value.Attributes;
     PartnerCount := Value.PartnerCount;
     Flowcount := Value.Flowcount;
end;

(* ------------------------------------------- *)
{ tIPv6EndpointTreeEntry }
(* ------------------------------------------- *)

procedure tIPv6EndpointTreeEntry.CopyFrom (Value: tIPv6Endpoint);

begin
     index := 0;
     Address := Value.IPv6Address;
     Packets := Value.FramesReceived + Value.FramesSent;
     Bytes := Value.BytesReceived + value.BytesSent;
     PacketsSent := Value.FramesSent;
     PacketsReceived := Value.FramesReceived;
     BytesSent := value.BytesSent;
     BytesReceived := value.BytesReceived;

     HelpFirstSeen.FileIndex := value.FirstFileID;
     HelpFirstSeen.FileFrameIndex := value.FirstFrame;
     HelpFirstSeen.FileOffset := 0;
     HelpLastSeen.FileIndex := value.LastFileID;
     HelpLastSeen.FileFrameIndex := value.LastFrame;
     HelpLastSeen.FileOffset := 0;

     copyInterfaceEntryList (Value.InterfaceList, InterfaceList);

     PartnerCount := Value.PartnerCount;
     Attributes := value.Attributes;
     Flowcount := Value.Flowcount;
end;

(* ------------------------------------------- *)
{ tMACConversationTreeEntry }
(* ------------------------------------------- *)

procedure tMACConversationTreeEntry.CopyFrom (Value: tMACConversation);

begin
     index := 0;
     AddressA := value.MACPair.Source;
     AddressB := value.MACPair.Destination;
     Frames := value.FramesSrcToDst + value.FramesDstToSrc;
     Bytes := value.BytesSrcToDst + value.BytesDstToSrc;
     FramesAtoB := value.FramesSrcToDst;
     FramesBtoA := value.FramesDstToSrc;
     BytesAtoB := value.BytesSrcToDst;
     BytesBtoA := value.BytesDstToSrc;
     DurationSeconds := value.durationseconds;
     DurationString := CaptureDurationAsString (value.LastFrameTime, value.FirstFrameTime);

     if DurationSeconds > 0 then
        begin
             MBpSAtoB := (8 * value.BytesSrcToDst / DurationSeconds) / MByteDivider;
             MBpSBtoA := (8 * value.BytesDstToSrc / DurationSeconds) / MByteDivider;
        end
     else
         begin
              MBpSAtoB := 0;
              MBpSBtoA := 0;
         end;

     HelpFirstSeen.FileIndex := value.FirstFileID;
     HelpFirstSeen.FileFrameIndex := value.FirstFrame;
     HelpFirstSeen.FileOffset := 0;
     HelpLastSeen.FileIndex := value.LastFileID;
     HelpLastSeen.FileFrameIndex := value.LastFrame;
     HelpLastSeen.FileOffset := 0;

     copyInterfaceEntryList (Value.InterfaceList, InterfaceList);

     Attributes := value.Attributes;
     Flowcount := Value.Flowcount;
end;

(* ------------------------------------------- *)
{ tIPv4ConversationTreeEntry }
(* ------------------------------------------- *)

procedure tIPv4ConversationTreeEntry.CopyFrom (Value: tIPv4Conversation);

begin
     index := 0;
     AddressA := value.IPv4AddressPair.Source;
     AddressB := value.IPv4AddressPair.Destination;
     Packets := value.FramesSrcToDst + value.FramesDstToSrc;
     Bytes := value.BytesSrcToDst + value.BytesDstToSrc;
     PacketsAtoB := value.FramesSrcToDst;
     PacketsBtoA := value.FramesDstToSrc;
     BytesAtoB := value.BytesSrcToDst;
     BytesBtoA := value.BytesDstToSrc;
     DurationSeconds := value.durationseconds;
     DurationString := CaptureDurationAsString (value.LastFrameTime, value.FirstFrameTime);

     if DurationSeconds > 0 then
        begin
//           MBpSAtoB := (value.BytesSrcToDst / Duration) / MByteDivider;
//           MBpSBtoA := (value.BytesDstToSrc / Duration) / MByteDivider;
             MBpSAtoB := (8 * value.BytesSrcToDst / DurationSeconds) / MByteDivider;
             MBpSBtoA := (8 * value.BytesDstToSrc / DurationSeconds) / MByteDivider;
        end
     else
         begin
              MBpSAtoB := 0;
              MBpSBtoA := 0;
         end;

     MinRTTns := Value.MinRTTns;
     MaxRTTns := Value.MaxRTTns;

     if Value.RTTSampleCount > 0 then
        AverageRTTns := Value.AggregatedRTTns div Value.RTTSampleCount
     else
         AverageRTTns := 0;

     FlowCount := Value.FlowCount;

     HelpFirstSeen.FileIndex := value.FirstFileID;
     HelpFirstSeen.FileFrameIndex := value.FirstFrame;
     HelpFirstSeen.FileOffset := 0;
     HelpLastSeen.FileIndex := value.LastFileID;
     HelpLastSeen.FileFrameIndex := value.LastFrame;
     HelpLastSeen.FileOffset := 0;

     copyInterfaceEntryList (Value.InterfaceList, InterfaceList);

     Attributes := value.Attributes;
end;

(* ------------------------------------------- *)
{ tIPv6ConversationTreeEntry }
(* ------------------------------------------- *)

procedure tIPv6ConversationTreeEntry.CopyFrom (Value: tIPv6Conversation);

begin
     index := 0;
     AddressA := value.IPv6AddressPair.Source;
     AddressB := value.IPv6AddressPair.Destination;
     Packets := value.FramesSrcToDst + value.FramesDstToSrc;
     Bytes := value.BytesSrcToDst + value.BytesDstToSrc;
     PacketsAtoB := value.FramesSrcToDst;
     PacketsBtoA := value.FramesDstToSrc;
     BytesAtoB := value.BytesSrcToDst;
     BytesBtoA := value.BytesDstToSrc;
     DurationSeconds := value.durationseconds;
     DurationString := CaptureDurationAsString (value.LastFrameTime, value.FirstFrameTime);

     if DurationSeconds > 0 then
        begin
             MBpSAtoB := (8 * value.BytesSrcToDst / DurationSeconds) / MByteDivider;
             MBpSBtoA := (8 * value.BytesDstToSrc / DurationSeconds) / MByteDivider;
        end
     else
         begin
              MBpSAtoB := 0;
              MBpSBtoA := 0;
         end;

     MinRTTns := Value.MinRTTns;
     MaxRTTns := Value.MaxRTTns;

     if Value.RTTSampleCount > 0 then
        AverageRTTns := Value.AggregatedRTTns div Value.RTTSampleCount
     else
         AverageRTTns := 0;

     FlowCount := Value.FlowCount;

     HelpFirstSeen.FileIndex := value.FirstFileID;
     HelpFirstSeen.FileFrameIndex := value.FirstFrame;
     HelpFirstSeen.FileOffset := 0;
     HelpLastSeen.FileIndex := value.LastFileID;
     HelpLastSeen.FileFrameIndex := value.LastFrame;
     HelpLastSeen.FileOffset := 0;

     copyInterfaceEntryList (Value.InterfaceList, InterfaceList);

     Attributes := value.Attributes;
end;

(* ------------------------------------------- *)
{ tTCPConversationTreeEntry }
(* ------------------------------------------- *)

procedure tTCPConversationTreeEntry.CopyFrom (Value: tTCPConversation; const ItemIndex : integer = 0);

var dHelp : double;

begin
     index := ItemIndex;
     AddressA := value.SocketPair.Source.IPAddress;
     AddressB := value.SocketPair.Destination.IPAddress;
     PortA := Value.SocketPair.Source.Port;
     PortB := Value.SocketPair.Destination.Port;
     Packets := Value.FramesSrcToDst + value.FramesDstToSrc;
     Bytes := Value.BytesSrcToDst + Value.BytesDstToSrc;
     PacketsAtoB := Value.FramesSrcToDst;
     PacketsBtoA := Value.FramesDstToSrc;
     BytesAtoB := Value.BytesSrcToDst;
     BytesBtoA := Value.BytesDstToSrc;
     DurationSeconds := value.durationseconds;
     DurationString := CaptureDurationAsString (value.LastFrameTime, value.FirstFrameTime);

     dHelp := 100 * PacketsAtoB / Packets;
     PacketRatioAtoB := trunc (dHelp);

     InitialRTTns := value.InitialRTTns;

     if DurationSeconds > 0 then
        begin
             MBpSAtoB := (8 * value.BytesSrcToDst / DurationSeconds) / MByteDivider;
             MBpSBtoA := (8 * value.BytesDstToSrc / DurationSeconds) / MByteDivider;
        end
     else
         begin
              MBpSAtoB := 0;
              MBpSBtoA := 0;
         end;

     SessionStatus := Value.SessionStatus;
     SynSeen := Value.SynSeen;
     FinSeen := Value.FinSeen;
     RstSeen := Value.RstSeen;

     FirstTimestamp := value.FirstFrameTime;
     LastTimestamp := value.LastFrameTime;

     HelpFirstSeen.FileIndex := value.FirstFileID;
     HelpFirstSeen.FileFrameIndex := value.FirstFrame;
     HelpFirstSeen.FileOffset := 0;
     HelpLastSeen.FileIndex := value.LastFileID;
     HelpLastSeen.FileFrameIndex := value.LastFrame;
     HelpLastSeen.FileOffset := 0;

     copyInterfaceEntryList (Value.InterfaceList, InterfaceList);

     Attributes := value.Attributes;

     Flags := Value.Flags;
     Tunnelflags := Value.Tunnelflags;

     Marked := Value.Marked;

     IsParent := false;

//   nextMarked := nil;
//   previousMarked := nil;
end;

(* ------------------------------------------- *)
{
procedure tTCPConversationTreeEntry.CopyFrom (Value: tTCPIPv4Conversation; const ItemIndex : integer = 0);

begin
     index := ItemIndex;
//   IPVersion := 4;
//   AddressIPv4A := Value.SocketPair.Source.IPAddress;
//   AddressIPv6A.initialize;
//   AddressIPv4B := Value.SocketPair.Destination.IPAddress;
//   AddressIPv6B.initialize;
     AddressA.initialize (value.SocketPair.Source.IPAddress);
     AddressB.initialize (value.SocketPair.Destination.IPAddress);
     PortA := Value.SocketPair.Source.Port;
     PortB := Value.SocketPair.Destination.Port;
     Packets := Value.FramesSrcToDst + value.FramesDstToSrc;
     Bytes := Value.BytesSrcToDst + Value.BytesDstToSrc;
     PacketsAtoB := Value.FramesSrcToDst;
     PacketsBtoA := Value.FramesDstToSrc;
     BytesAtoB := Value.BytesSrcToDst;
     BytesBtoA := Value.BytesDstToSrc;
     DurationSeconds := value.durationseconds;
     DurationString := CaptureDurationAsString (value.LastFrameTime, value.FirstFrameTime);
     InitialRTTns := value.InitialRTTns;

     if DurationSeconds > 0 then
        begin
             MBpSAtoB := (8 * value.BytesSrcToDst / DurationSeconds) / MByteDivider;
             MBpSBtoA := (8 * value.BytesDstToSrc / DurationSeconds) / MByteDivider;
        end
     else
         begin
              MBpSAtoB := 0;
              MBpSBtoA := 0;
         end;

     SessionStatus := Value.SessionStatus;
     SynSeen := Value.SynSeen;
     FinSeen := Value.FinSeen;
     RstSeen := Value.RstSeen;

     HelpFirstSeen.FileIndex := value.FirstFileID;
     HelpFirstSeen.FrameIndex := value.FirstFrame;
     HelpFirstSeen.FileOffset := 0;
     HelpLastSeen.FileIndex := value.LastFileID;
     HelpLastSeen.FrameIndex := value.LastFrame;
     HelpLastSeen.FileOffset := 0;

     copyInterfaceEntryList (Value.InterfaceList, InterfaceList);

     Attributes := value.Attributes;

     Flags := Value.Flags;

     Marked := Value.Marked;

     nextMarked := nil;
     previousMarked := nil;
end;

(* ------------------------------------------- *)

procedure tTCPConversationTreeEntry.CopyFrom (Value: tTCPIPv6Conversation; const ItemIndex : integer = 0);

begin
     index := ItemIndex;
//   IPVersion := 6;
//   AddressIPv4A.initialize;
//   AddressIPv6A := Value.SocketPair.Source.IPAddress;
//   AddressIPv4B.initialize;
//   AddressIPv6B := Value.SocketPair.Destination.IPAddress;
     AddressA.initialize (value.SocketPair.Source.IPAddress);
     AddressB.initialize (value.SocketPair.Destination.IPAddress);
     PortA := Value.SocketPair.Source.Port;
     PortB := Value.SocketPair.Destination.Port;
     Packets := Value.FramesSrcToDst + value.FramesDstToSrc;
     Bytes := Value.BytesSrcToDst + Value.BytesDstToSrc;
     PacketsAtoB := Value.FramesSrcToDst;
     PacketsBtoA := Value.FramesDstToSrc;
     BytesAtoB := Value.BytesSrcToDst;
     BytesBtoA := Value.BytesDstToSrc;
     DurationSeconds := value.durationseconds;
     DurationString := CaptureDurationAsString (value.LastFrameTime, value.FirstFrameTime);
     InitialRTTns := value.InitialRTTns;

     if DurationSeconds > 0 then
        begin
             MBpSAtoB := (8 * value.BytesSrcToDst / DurationSeconds) / MByteDivider;
             MBpSBtoA := (8 * value.BytesDstToSrc / DurationSeconds) / MByteDivider;
        end
     else
         begin
              MBpSAtoB := 0;
              MBpSBtoA := 0;
         end;

     SessionStatus := Value.SessionStatus;
     SynSeen := Value.SynSeen;
     FinSeen := Value.FinSeen;
     RstSeen := Value.RstSeen;

     HelpFirstSeen.FileIndex := value.FirstFileID;
     HelpFirstSeen.FrameIndex := value.FirstFrame;
     HelpFirstSeen.FileOffset := 0;
     HelpLastSeen.FileIndex := value.LastFileID;
     HelpLastSeen.FrameIndex := value.LastFrame;
     HelpLastSeen.FileOffset := 0;

     copyInterfaceEntryList (Value.InterfaceList, InterfaceList);

     Attributes := value.Attributes;
     Flags := Value.Flags;

     Marked := Value.Marked;

     nextMarked := nil;
     previousMarked := nil;
end;
}
(* ------------------------------------------- *)

function tTCPConversationTreeEntry.GetIPv4SocketPair: tIPv4SocketPair;

begin
     result.initialize;

     assert (AddressA.IPVersion = 4);

     result.Source.IPAddress := AddressA.IPv4Address;
     result.Source.Port := PortA;
     result.Destination.IPAddress := AddressB.IPv4Address;
     result.Destination.Port := PortB;
end;

(* ------------------------------------------- *)

function tTCPConversationTreeEntry.GetIPv6SocketPair: tIPv6SocketPair;

begin
     result.initialize;

     assert (AddressA.IPVersion = 6);

     result.Source.IPAddress := AddressA.IPv6Address;
     result.Source.Port := PortA;
     result.Destination.IPAddress := AddressB.IPv6Address;
     result.Destination.Port := PortB;
end;

(* ------------------------------------------- *)
{ tUDPConversationTreeEntry }
(* ------------------------------------------- *)

procedure tUDPConversationTreeEntry.CopyFrom (Value: tUDPConversation);

var durationmsec : int64;

begin
     index := 0;
     AddressA := Value.SocketPair.Source.IPAddress;
     PortA := Value.SocketPair.Source.Port;
     AddressB := Value.SocketPair.Destination.IPAddress;
     PortB := Value.SocketPair.Destination.Port;
     Packets := Value.FramesSrcToDst + value.FramesDstToSrc;
     Bytes := Value.BytesSrcToDst + Value.BytesDstToSrc;
     PacketsAtoB := Value.FramesSrcToDst;
     PacketsBtoA := Value.FramesDstToSrc;
     BytesAtoB := Value.BytesSrcToDst;
     BytesBtoA := Value.BytesDstToSrc;
     DurationSeconds := value.durationseconds;
     DurationString := CaptureDurationAsString (value.LastFrameTime, value.FirstFrameTime);

     if value.durationseconds > 0 then
        begin
             MBpSAtoB := (8 * value.BytesSrcToDst / value.durationseconds) / MByteDivider;
             MBpSBtoA := (8 * value.BytesDstToSrc / value.durationseconds) / MByteDivider;
        end
     else
         begin
              MBpSAtoB := 0;
              MBpSBtoA := 0;
         end;

     // Calculate packets per second

     durationmsec := MillisecondsBetweenFrames (value.LastFrameTime, value.FirstFrameTime);

     if durationmsec > 0 then
        begin
             PacketsPerSecondAtoB := round ((PacketsAtoB * 1000) / durationmsec);
             PacketsPerSecondBtoA := round ((PacketsBtoA * 1000) / durationmsec);
        end
     else
         begin
              PacketsPerSecondAtoB := 0;
              PacketsPerSecondBtoA := 0;
         end;

     // Calculate Delta time, if there are only 2 packets, one in each direction

     if (PacketsAtoB = 1) and (PacketsBtoA = 1) then
        DeltaTime.DetermineDelta (value.LastFrameTime, value.FirstFrameTime)
     else
         DeltaTime.initialize;

     // Other stuff

     FirstTimestamp := value.FirstFrameTime;
     LastTimestamp := value.LastFrameTime;

     HelpFirstSeen.FileIndex := value.FirstFileID;
     HelpFirstSeen.FileFrameIndex := value.FirstFrame;
     HelpFirstSeen.FileOffset := 0;
     HelpLastSeen.FileIndex := value.LastFileID;
     HelpLastSeen.FileFrameIndex := value.LastFrame;
     HelpLastSeen.FileOffset := 0;

     copyInterfaceEntryList (Value.InterfaceList, InterfaceList);

     Tunnelflags := Value.Tunnelflags;

     Attributes := value.Attributes;
end;

(* ------------------------------------------- *)
{
procedure tUDPConversationTreeEntry.CopyFrom (Value: tUDPIPv4Conversation);

begin
     index := 0;
     IPVersion := 4;
     AddressIPv4A := Value.SocketPair.Source.IPAddress;
     AddressIPv6A.initialize;
     PortA := Value.SocketPair.Source.Port;
     AddressIPv4B := Value.SocketPair.Destination.IPAddress;
     AddressIPv6B.initialize;
     PortB := Value.SocketPair.Destination.Port;
     Packets := Value.FramesSrcToDst + value.FramesDstToSrc;
     Bytes := Value.BytesSrcToDst + Value.BytesDstToSrc;
     PacketsAtoB := Value.FramesSrcToDst;
     PacketsBtoA := Value.FramesDstToSrc;
     BytesAtoB := Value.BytesSrcToDst;
     BytesBtoA := Value.BytesDstToSrc;
     DurationSeconds := value.durationseconds;
     DurationString := CaptureDurationAsString (value.LastFrameTime, value.FirstFrameTime);

     if value.durationseconds > 0 then
        begin
             MBpSAtoB := (8 * value.BytesSrcToDst / value.durationseconds) / MByteDivider;
             MBpSBtoA := (8 * value.BytesDstToSrc / value.durationseconds) / MByteDivider;
        end
     else
         begin
              MBpSAtoB := 0;
              MBpSBtoA := 0;
         end;

     FirstTimestamp := value.FirstFrameTime;
     LastTimestamp := value.LastFrameTime;

     HelpFirstSeen.FileIndex := value.FirstFileID;
     HelpFirstSeen.FrameIndex := value.FirstFrame;
     HelpFirstSeen.FileOffset := 0;
     HelpLastSeen.FileIndex := value.LastFileID;
     HelpLastSeen.FrameIndex := value.LastFrame;
     HelpLastSeen.FileOffset := 0;

     copyInterfaceEntryList (Value.InterfaceList, InterfaceList);

     Attributes := value.Attributes;
end;

(* ------------------------------------------- *)

procedure tUDPConversationTreeEntry.CopyFrom (Value: tUDPIPv6Conversation);

begin
     index := 0;
     IPVersion := 6;
     AddressIPv4A.initialize;
     AddressIPv6A := Value.SocketPair.Source.IPAddress;
     PortA := Value.SocketPair.Source.Port;
     AddressIPv4B.initialize;
     AddressIPv6B := Value.SocketPair.Destination.IPAddress;
     PortB := Value.SocketPair.Destination.Port;
     Packets := Value.FramesSrcToDst + value.FramesDstToSrc;
     Bytes := Value.BytesSrcToDst + Value.BytesDstToSrc;
     PacketsAtoB := Value.FramesSrcToDst;
     PacketsBtoA := Value.FramesDstToSrc;
     BytesAtoB := Value.BytesSrcToDst;
     BytesBtoA := Value.BytesDstToSrc;
     DurationSeconds := value.durationseconds;
     DurationString := CaptureDurationAsString (value.LastFrameTime, value.FirstFrameTime);

     if value.durationseconds > 0 then
        begin
             MBpSAtoB := (8 * value.BytesSrcToDst / value.durationseconds) / MByteDivider;
             MBpSBtoA := (8 * value.BytesDstToSrc / value.durationseconds) / MByteDivider;
        end
     else
         begin
              MBpSAtoB := 0;
              MBpSBtoA := 0;
         end;

     FirstTimestamp := value.FirstFrameTime;
     LastTimestamp := value.LastFrameTime;

     HelpFirstSeen.FileIndex := value.FirstFileID;
     HelpFirstSeen.FrameIndex := value.FirstFrame;
     HelpFirstSeen.FileOffset := 0;
     HelpLastSeen.FileIndex := value.LastFileID;
     HelpLastSeen.FrameIndex := value.LastFrame;
     HelpLastSeen.FileOffset := 0;

     copyInterfaceEntryList (Value.InterfaceList, InterfaceList);

     Attributes := value.Attributes;
end;
}
(* ------------------------------------------- *)

function tUDPConversationTreeEntry.GetIPv4SocketPair: tIPv4SocketPair;

begin
     result.initialize;

     assert (AddressA.IPVersion = 4);

     result.Source.IPAddress := AddressA.IPv4Address;
     result.Source.Port := PortA;
     result.Destination.IPAddress := AddressB.IPv4Address;
     result.Destination.Port := PortB;
end;

(* ------------------------------------------- *)

function tUDPConversationTreeEntry.GetIPv6SocketPair: tIPv6SocketPair;

begin
     result.initialize;

     assert (AddressA.IPVersion = 6);

     result.Source.IPAddress := AddressA.IPv6Address;
     result.Source.Port := PortA;
     result.Destination.IPAddress := AddressB.IPv6Address;
     result.Destination.Port := PortB;
end;

(* ------------------------------------------- *)
{ tConversationFilter }
(* ------------------------------------------- *)

procedure tConversationFilter.initialize;

begin
     ExcludeUpperLayerAddresses := false;
end;

(* ------------------------------------------- *)
{ tSocketDetails }
(* ------------------------------------------- *)

procedure tSocketDetails.initialize;

begin
     IPSocket.initialize;
     UseCount := 0;
     NumPacketsIn := 0;
     NumPacketsOut := 0;
     Firstseen.initialize;
     LastSeen.initialize;
end;

(* ------------------------------------------- *)
{ tUDPConversation }
(* ------------------------------------------- *)

procedure tUDPConversation.initialize (const FileIndex: integer);

begin
     SocketPair.initialize;
     FramesSrcToDst := 0;
     FramesDstToSrc := 0;
     BytesSrcToDst := 0;
     BytesDstToSrc := 0;
     FirstFrameTime.initialize;
     LastFrameTime.initialize;
     FirstFileID := FileIndex;
     LastFileID := FileIndex;
     Markers := [];
     setLength (InterfaceList, 0);
     Attributes := [];
     OriginFileIndex := FileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;
     Tunnelflags := [];
end;

(* ------------------------------------------- *)

procedure tUDPConversation.initialize (IPv4Conversation: tIPv4Conversation);

begin
     self.initialize;
     SocketPair.Source.IPAddress.initialize (IPv4Conversation.IPv4AddressPair.Source);
     SocketPair.Destination.IPAddress.initialize (IPv4Conversation.IPv4AddressPair.Destination);

     FramesSrcToDst := IPv4Conversation.FramesSrcToDst;
     FramesDstToSrc := IPv4Conversation.FramesDstToSrc;
     BytesSrcToDst := IPv4Conversation.BytesSrcToDst;
     BytesDstToSrc := IPv4Conversation.BytesDstToSrc;
     FirstFrame := IPv4Conversation.FirstFrame;
     LastFrame := IPv4Conversation.LastFrame;
     FirstFrameTime := IPv4Conversation.FirstFrameTime;
     LastFrameTime := IPv4Conversation.LastFrameTime;
     FirstFileID := IPv4Conversation.FirstFileID;
     LastFileID := IPv4Conversation.LastFileID;
     copyInterfaceEntryList (IPv4Conversation.InterfaceList, InterfaceList);
     Markers := IPv4Conversation.Markers;
     OriginFileIndex := IPv4Conversation.OriginFileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;
end;

(* ------------------------------------------- *)

procedure tUDPConversation.initialize (IPv6Conversation: tIPv6Conversation);

begin
     self.initialize;
     SocketPair.Source.IPAddress.initialize (IPv6Conversation.IPv6AddressPair.Source);
     SocketPair.Destination.IPAddress.initialize (IPv6Conversation.IPv6AddressPair.Destination);

     FramesSrcToDst := IPv6Conversation.FramesSrcToDst;
     FramesDstToSrc := IPv6Conversation.FramesDstToSrc;
     BytesSrcToDst := IPv6Conversation.BytesSrcToDst;
     BytesDstToSrc := IPv6Conversation.BytesDstToSrc;
     FirstFrame := IPv6Conversation.FirstFrame;
     LastFrame := IPv6Conversation.LastFrame;
     FirstFrameTime := IPv6Conversation.FirstFrameTime;
     LastFrameTime := IPv6Conversation.LastFrameTime;
     FirstFileID := IPv6Conversation.FirstFileID;
     LastFileID := IPv6Conversation.LastFileID;
     copyInterfaceEntryList (IPv6Conversation.InterfaceList, InterfaceList);
     Markers := IPv6Conversation.Markers;
     OriginFileIndex := IPv6Conversation.OriginFileIndex;
     OriginTimestamp.initialize;
     OriginAddressPair.initialize;
end;

(* ------------------------------------------- *)

function tUDPConversation.durationasstring: string;

begin
     result := CaptureDurationAsString (LastFrameTime, FirstFrameTime);
end;

(* ------------------------------------------- *)

function tUDPConversation.durationseconds: integer;

begin
     result := secondsbetween (LastFrameTime.datetime, FirstFrameTime.datetime);
end;

(* ------------------------------------------- *)

function tUDPConversation.DictionaryKey: string;

begin
     result := SocketPair.DictionaryKey;
end;

(* ------------------------------------------- *)

end.
