{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit definition_filter_structures;

interface

{$i 'jedi.inc')}

uses DBXJSON,
{$ifdef DELPHI20_UP}
     System.JSON,
{$endif}
     System.Generics.Collections,
     base_structures,
     definition_ip_addresses,
     definition_tcp_structures,
     definition_5tuple;

type tFilterType = (ftIgnore, ftSingle, ftRange);
     tICMPFilterType = (IFTIgnore, IFTReject, IFTAny, IFTType);
     tFilterScope = (fsAny, fsSource, fsDestination, fsBoth);

     tFilterStatistics     = record
                                   TotalCompares      : int64;
                                   TotalHits          : int64;
                                   TotalMisses        : int64;
                                   TupleHits          : int64;

                                   procedure initialize;
                             end;

     tMACAddressFilter = record
                               AddressFilterType      : tFilterType;
                               AddressScope           : tFilterScope;

                               SingleAddress          : tMACAddress;
                               SourceAddress          : tMACAddress;
                               DestinationAddress     : tMACAddress;

                               class operator equal (a, b : tMACAddressFilter) : boolean;
                               class operator notequal (a, b : tMACAddressFilter) : boolean;
                               class operator greaterthan (a, b : tMACAddressFilter) : boolean;
                               class operator lessthan (a, b : tMACAddressFilter) : boolean;
                               class operator greaterthanorequal (a, b : tMACAddressFilter) : boolean;
                               class operator lessthanorequal (a, b : tMACAddressFilter) : boolean;


                               procedure initialize; overload;
                               procedure initialize (Settings : tJSONObject); overload;
                               function toJSON : tJSONObject;
                               function asDisplayFilter (const Bracketed : boolean = false) : string;
                         end;

     tMACFilterList = array of tMACAddressFilter;

     tIPv4AddressFilter = record
                                AddressFilterType      : tFilterType;
                                AddressScope           : tFilterScope;

                                SingleAddress          : tIPv4Address;
                                SourceAddress          : tIPv4Address;
                                DestinationAddress     : tIPv4Address;
                                FirstAddress           : tIPv4Address;
                                LastAddress            : tIPv4Address;

                                class operator equal (a, b : tIPv4AddressFilter) : boolean;
                                class operator notequal (a, b : tIPv4AddressFilter) : boolean;
                                class operator greaterthan (a, b : tIPv4AddressFilter) : boolean;
                                class operator lessthan (a, b : tIPv4AddressFilter) : boolean;
                                class operator greaterthanorequal (a, b : tIPv4AddressFilter) : boolean;
                                class operator lessthanorequal (a, b : tIPv4AddressFilter) : boolean;

                                procedure initialize; overload;
                                procedure initialize (Settings : tJSONObject); overload;
                                procedure initialize (const IPAddress : tIPv4Address); overload;
                                procedure initialize (const IPPair : tIPv4AddressPair); overload;
                                procedure initialize (const SocketPair : tIPv4SocketPair); overload;
                                function toJSON : tJSONObject;
                                function asDisplayFilter (const Bracketed : boolean = false) : string;
                          end;

     tIPv4FilterList = array of tIPv4AddressFilter;

     tIPv6AddressFilter = record
                                AddressFilterType      : tFilterType;
                                AddressScope           : tFilterScope;

                                SingleAddress          : tIPv6Address;
                                SourceAddress          : tIPv6Address;
                                DestinationAddress     : tIPv6Address;
                                FirstAddress           : tIPv6Address;
                                LastAddress            : tIPv6Address;

                                class operator equal (a, b : tIPv6AddressFilter) : boolean;
                                class operator notequal (a, b : tIPv6AddressFilter) : boolean;
                                class operator greaterthan (a, b : tIPv6AddressFilter) : boolean;
                                class operator lessthan (a, b : tIPv6AddressFilter) : boolean;
                                class operator greaterthanorequal (a, b : tIPv6AddressFilter) : boolean;
                                class operator lessthanorequal (a, b : tIPv6AddressFilter) : boolean;

                                procedure initialize; overload;
                                procedure initialize (Settings : tJSONObject); overload;
                                procedure initialize (const IPAddress : tIPv6Address); overload;
                                procedure initialize (const IPPair : tIPv6AddressPair); overload;
                                procedure initialize (const SocketPair : tIPv6SocketPair); overload;
                                function toJSON : tJSONObject;
                                function asDisplayFilter (const Bracketed : boolean = false) : string;
                          end;

     tIPv6FilterList = array of tIPv6AddressFilter;

     tPortFilter = record
                         Layer4Protocol                : tLayer4Protocol;
                         PortFilterType                : tFilterType;
                         PortScope                     : tFilterScope;

                         SinglePort                    : integer;
                         SourcePort                    : integer;
                         DestinationPort               : integer;
                         FirstPort                     : integer;
                         LastPort                      : integer;

                         class operator equal (a, b : tPortFilter) : boolean;
                         class operator notequal (a, b : tPortFilter) : boolean;
                         class operator greaterthan (a, b : tPortFilter) : boolean;
                         class operator lessthan (a, b : tPortFilter) : boolean;
                         class operator greaterthanorequal (a, b : tPortFilter) : boolean;
                         class operator lessthanorequal (a, b : tPortFilter) : boolean;

                         procedure initialize; overload;
                         procedure initialize (Settings : tJSONObject); overload;
                         procedure initialize (const Port : integer); overload;
                         procedure initialize (const SocketPair : tIPv4SocketPair; const Layer4Prot : tLayer4Protocol); overload;
                         procedure initialize (const SocketPair : tIPv6SocketPair; const Layer4Prot : tLayer4Protocol); overload;
                         function toJSON : tJSONObject;
                         function asDisplayFilter (const Bracketed : boolean = false) : string;
                   end;

     tTCPFilter = record
                        TCPFilterType                  : tFilterType;
                        Flags                          : tTCPFlagSet;
                        TCPHandshake                   : boolean;

                        class operator equal (a, b : tTCPFilter) : boolean;
                        class operator notequal (a, b : tTCPFilter) : boolean;

                        procedure initialize; overload;
                        procedure initialize (Settings : tJSONObject); overload;
                        function toJSON : tJSONObject;
                        function asDisplayFilter (const Bracketed : boolean = false) : string;

                  end;

     tICMPFilter = record
                         ICMPFilterType                 : tICMPFilterType;
                         ICMPType                       : byte;
                         ICMPCode                       : byte;

                         class operator equal (a, b : tICMPFilter) : boolean;
                         class operator notequal (a, b : tICMPFilter) : boolean;

                         procedure initialize; overload;
                         procedure initialize (Settings : tJSONObject); overload;
                         function toJSON : tJSONObject;
                         function asDisplayFilter (const Bracketed : boolean = false) : string;

                   end;

     tPortFilterList = array of tPortFilter;

     tFrameFilter = record
                          MACFilter   : tMACAddressFilter;
                          IPv4Filter  : tIPv4AddressFilter;
                          IPv6Filter  : tIPv6AddressFilter;
                          ICMPFilter  : tICMPFilter;
                          PortFilter  : tPortFilter;
                          TCPFilter   : tTCPFilter;
                          FilterLabel : string;

                          // Helpers to indicate file location of first/last frame, and other details, if available
                          // won't be included in any class operator or JSON/display stuff

                          HelpRecType           : tConversationStoreRecType;              // What kind of record we need to pull from the conversation store to determine first/last frame etc
                          HelpFirstSeen         : tFileFrameLocation;                     // File location of where the first match will be found (taken from conversation store)
                          HelpLastSeen          : tFileFrameLocation;                     // File location of where the last match will be found (taken from conversation store)
                          HelpConversationIndex : integer;                                // Index as seen in conversation dialog

                          class operator equal (a, b : tFrameFilter) : boolean;
                          class operator notequal (a, b : tFrameFilter) : boolean;
                          class operator greaterthan (a, b : tFrameFilter) : boolean;
                          class operator lessthan (a, b : tFrameFilter) : boolean;
                          class operator greaterthanorequal (a, b : tFrameFilter) : boolean;
                          class operator lessthanorequal (a, b : tFrameFilter) : boolean;

                          procedure initialize; overload;
                          procedure initialize (FilterSettings : tJSONObject); overload;
                          procedure initialize (FilterSettings : string); overload;
                          procedure initialize (const Tuple : t5Tuple; const NewLabel : string = ''); overload;
                          procedure updateRecType;
                          function isUnspecified : boolean;
                          /// <summary>
                          ///   This creates a new JSON object and returns it.
                          ///   Freeing it is the responsibility of the calling
                          ///   routine.
                          /// </summary>
                          /// <param name="IncludeNonFilterItems">
                          ///   if true, things like the filter label are
                          ///   included
                          /// </param>
                          function toJSON (const IncludeNonFilterItems : boolean = True) : tJSONObject;
                          function asDisplayFilter : string;
                          function asSocketString : string;
                          function as5Tuple : t5Tuple;
                          function SectionCount : integer;

                          function DictionaryKey : string;
                      end;

     tOnAddFilterToSet = procedure (Sender : tObject; var Filter : tFrameFilter) of object;

     tFilterSet = class (tObject)

                        constructor create;

                        private
                               EventOnAddFilter  : tOnAddFilterToSet;

                               priv_filterlist   : tDictionary<string, tFrameFilter>;
                               priv_filterarray  : array of tFrameFilter;

                               priv_currentindex : integer;

                               function getFilterCount : integer;

                        public
                              property count : integer read getFilterCount;

                              property OnAddFilterToSet : tOnAddFilterToSet read EventOnAddFilter write EventOnAddFilter;

                              /// <summary>
                              ///   Adds a filter to the filterset if it isn't
                              ///   part of it yet
                              /// </summary>
                              /// <param name="Filter">
                              ///   New Filter to be added
                              /// </param>
                              /// <returns>
                              ///   true if the filter was added
                              /// </returns>
                              function AddFilter (Filter : tFrameFilter) : boolean;
                              /// <summary>
                              ///   Adds filters from another filterlist to the
                              ///   filterlist if they didn't exist yet
                              /// </summary>
                              /// <param name="FilterSet">
                              ///   Filters to be added to the current set
                              /// </param>
                              /// <returns>
                              ///   Number of filters added
                              /// </returns>
                              function AddFilterSet (const FilterSet : tFilterSet) : integer;
                              procedure Clear;
                              procedure Trim;
                              function getFirst : tFrameFilter;
                              function getNext : tFrameFilter;
                              function getFilterByKey (const DictionaryKey : string) : tFrameFilter;
                              procedure RemoveFilter (const DictionaryKey : string);

                              destructor Destroy; override;
                  end;


     function FilterTypeToStr (FilterType : tFilterType) : string;
     function StrToFilterType (FilterString : string) : tFilterType;
     function FilterScopeToStr (FilterScope : tFilterScope) : string;
     function StrToFilterScope (ScopeString : string): tFilterScope;
     function ICMPFilterTypeToStr (FilterType : tICMPFilterType) : string;
     function StrToICMPFilterType (FilterString : string) : tICMPFilterType;


const MaxFilterLabelSize = 64;

implementation

uses SiAuto,
     SysUtils,
     toolkit;

const FilterVersion = '1.0';

(* ------------------------------------------- *)
{ tFilterStatistics }
(* ------------------------------------------- *)

procedure tFilterStatistics.initialize;

begin
     TotalCompares := 0;
     TotalHits := 0;
     TotalMisses := 0;
     TupleHits := 0;
end;

(* ------------------------------------------- *)

function FilterTypeToStr (FilterType : tFilterType) : string;

begin
     case FilterType of
      ftSingle : result := 'Single';
      ftRange  : result := 'Range';
     else
         result := 'Ignore';
     end
end;

(* ------------------------------------------- *)

function StrToFilterType (FilterString : string) : tFilterType;

begin
     if FilterString = 'Single' then
        result := ftSingle
     else
     if FilterString = 'Range' then
        result := ftRange
     else
         result := ftIgnore;
end;

(* ------------------------------------------- *)

function ICMPFilterTypeToStr (FilterType : tICMPFilterType) : string;

begin
     case FilterType of
      IFTIgnore : result := 'Ignore';
      IFTReject : result := 'Reject';
      IFTAny    : result := 'Any';
      IFTType   : result := 'ICMPType';
     end;
end;

(* ------------------------------------------- *)

function StrToICMPFilterType (FilterString : string) : tICMPFilterType;

begin
     if FilterString = 'Ignore' then
        result := IFTIgnore
     else
     if FilterString = 'Reject' then
        result := IFTReject
     else
     if FilterString = 'Any' then
        result := IFTAny
     else
     if FilterString = 'ICMPType' then
        result := IFTType
     else
         result := IFTIgnore;
end;

(* ------------------------------------------- *)

function FilterScopeToStr (FilterScope : tFilterScope) : string;

begin
     case FilterScope of
      fsSource      : result := 'Source';
      fsDestination : result := 'Destination';
      fsBoth        : result := 'Both';
     else
         result := 'Any';
     end;
end;

(* ------------------------------------------- *)

function StrToFilterScope (ScopeString : string): tFilterScope;

begin
     if ScopeString = 'Any' then
        result := fsAny
     else
     if ScopeString = 'Source' then
        result := fsSource
     else
     if ScopeString = 'Destination' then
        result := fsDestination
     else
     if ScopeString = 'Both' then
        result := fsBoth;
end;

(* ------------------------------------------- *)
{
procedure mergeFilterToSet (const filter : tFrameFilter; var filterset : tFilterSet);

var found : boolean;
    len   : integer;
    i     : integer;

begin
     found := false;
     len := length (filterset);

     if len > 0 then
        begin
             i := 0;

             repeat
                   if filter = filterset[i] then
                      begin
                           found := true;

//                         SiMain.LogVerbose ('Filter match found in list: "' + filter.asSocketString + '" is the same as existing filter "' + filterset[i].asSocketString + '"');
                      end;

                   inc (i);

             until found or (i >= len);
        end;

     if not found then
        begin
             setLength (filterset, len + 1);
             filterset[len] := Filter;
        end;
end;

(* ------------------------------------------- *)

procedure mergeFilterSetToSet (const newfilters : tFilterSet; var filterset : tFilterSet);

var filter : tFrameFilter;
    len    : integer;
    i      : integer;

begin
     len := length (newfilters);

     for i := 1 to len do
         begin
              filter := newfilters[i-1];

              mergeFilterToSet (filter, filterset);
         end;
end;

(* ------------------------------------------- *)

procedure deleteFilterFromSet (const Index : integer; var filterset : tFilterSet);

var len : integer;
    i   : integer;

begin
     len := length(filterset);

     Assert(len > 0);
     Assert(Index < len);

     for i := Index + 1 to len - 1 do
         filterset[i-1] := filterset[i];

     SetLength (filterset, len - 1);
end;
}
(* ------------------------------------------- *)
{ tMACAddressFilter }
(* ------------------------------------------- *)

procedure tMACAddressFilter.initialize;

begin
     AddressFilterType := ftIgnore;
     AddressScope := fsAny;

     SingleAddress.initialize;
     SourceAddress.initialize;
     DestinationAddress.initialize;
end;

(* ------------------------------------------- *)

procedure tMACAddressFilter.initialize (Settings: tJSONObject);

var JVal : tJSONValue;

begin
     initialize;

     JVal := Settings.GetValue ('FilterType');
     assert (assigned (JVal), 'FilterType JSON failure (tMACAddressFilter.initialize)');
     AddressFilterType := StrToFilterType (StripQuotes (JVal.Value));

     JVal := Settings.GetValue ('Scope');
     assert (assigned (JVal), 'Scope JSON failure (tMACAddressFilter.initialize)');
     AddressScope := StrToFilterScope (StripQuotes (JVal.ToString));

     JVal := Settings.GetValue('SingleAddress');
     assert (assigned (JVal), 'SingleAddress JSON failure (tMACAddressFilter.initialize)');
     SingleAddress.setfromString (StripQuotes (JVal.ToString), ':');

     JVal := Settings.GetValue('SourceAddress');
     assert (assigned (JVal), 'SourceAddress JSON failure (tMACAddressFilter.initialize)');
     SourceAddress.setfromString (StripQuotes (JVal.ToString), ':');

     JVal := Settings.GetValue('DestinationAddress');
     assert (assigned (JVal), 'DestinationAddress JSON failure (tMACAddressFilter.initialize)');
     DestinationAddress.setfromString (StripQuotes (JVal.ToString), ':');
end;

(* ------------------------------------------- *)

class operator tMACAddressFilter.equal (a, b: tMACAddressFilter): boolean;

begin
     result := (a.AddressFilterType = b.AddressFilterType) and
               (a.AddressScope = b.AddressScope) and
               (a.SingleAddress = b.SingleAddress) and
               (a.SourceAddress = b.SourceAddress) and
               (a.DestinationAddress = b.DestinationAddress);
end;

(* ------------------------------------------- *)

class operator tMACAddressFilter.greaterthan (a, b: tMACAddressFilter): boolean;

begin
     if not (a = b) then
        begin
             if (a.SingleAddress = b.SingleAddress) then
                begin
                     if (a.SourceAddress = b.SourceAddress) then
                        result := (a.DestinationAddress > b.DestinationAddress)
                     else
                         result := (a.SourceAddress > b.SourceAddress);
                end
             else
                 result := a.SingleAddress > b.SingleAddress;
        end
     else
         result := false;
end;

(* ------------------------------------------- *)

class operator tMACAddressFilter.greaterthanorequal (a, b: tMACAddressFilter): boolean;

begin
     result := (a = b) or (a > b);
end;

(* ------------------------------------------- *)

class operator tMACAddressFilter.lessthan (a, b: tMACAddressFilter): boolean;

begin
     result := not (a >= b);
end;

(* ------------------------------------------- *)

class operator tMACAddressFilter.lessthanorequal (a, b: tMACAddressFilter): boolean;

begin
     result := not (a > b);
end;

(* ------------------------------------------- *)

class operator tMACAddressFilter.notequal (a, b: tMACAddressFilter): boolean;

begin
     result := not (a=b);
end;

(* ------------------------------------------- *)

function tMACAddressFilter.toJSON: tJSONObject;

var value : tJSONObject;

begin
     value := tJSONObject.Create;

     value.AddPair (TJSONPair.Create ('FilterType', tJSONString.Create (FilterTypeToStr (AddressFilterType))));
     value.AddPair (TJSONPair.Create ('Scope', tJSONString.Create (FilterScopeToStr (AddressScope))));
     value.AddPair (TJSONPair.Create ('SingleAddress', tJSONString.Create (SingleAddress.asString(':'))));
     value.AddPair (TJSONPair.Create ('SourceAddress', tJSONString.Create (SourceAddress.asString(':'))));
     value.AddPair (TJSONPair.Create ('DestinationAddress', tJSONString.Create (DestinationAddress.asString(':'))));

     result := value;
end;

(* ------------------------------------------- *)

function tMACAddressFilter.asDisplayFilter (const Bracketed : boolean = false) : string;

var value : string;

begin
     case AddressScope of
      fsAny         : value := 'eth.addr==' + SingleAddress.asString;
      fsSource      : value := 'eth.src==' + SourceAddress.asString;
      fsDestination : value := 'eth.dst==' + SourceAddress.asString;
      fsBoth        : begin
                           value := 'eth.addr==' + SourceAddress.asString + ' and eth.addr==' + destinationaddress.asString;

                           if Bracketed then
                              value := '(eth.addr==' + SourceAddress.asString + ' and eth.addr==' + destinationaddress.asString + ')'
                           else
                               value := 'eth.addr==' + SourceAddress.asString + ' and eth.addr==' + destinationaddress.asString;
                      end;
     end;

     result := value;
end;

(* ------------------------------------------- *)
{ tIPv4AddressFilter }
(* ------------------------------------------- *)

procedure tIPv4AddressFilter.initialize;

begin
     AddressFilterType := ftIgnore;
     AddressScope := fsAny;

     SingleAddress.initialize;
     SourceAddress.initialize;
     DestinationAddress.initialize;
     FirstAddress.initialize;
     LastAddress.initialize;
end;

(* ------------------------------------------- *)

procedure tIPv4AddressFilter.initialize (Settings: tJSONObject);

var JVal  : tJSONValue;
    help  : string;

begin
     initialize;

     JVal := Settings.GetValue ('FilterType');
     assert (assigned (JVal), 'FilterType JSON failure (tIPv4AddressFilter.initialize)');
     AddressFilterType := StrToFilterType (StripQuotes (JVal.Value));

     JVal := Settings.GetValue ('Scope');
     assert (assigned (JVal), 'Scope JSON failure (tIPv4AddressFilter.initialize)');
     AddressScope := StrToFilterScope (StripQuotes (JVal.ToString));

     JVal := Settings.GetValue('SingleAddress');
     assert (assigned (JVal), 'SingleAddress JSON failure (tIPv4AddressFilter.initialize)');
     help := stripquotes (JVal.ToString);
     SingleAddress.setValueFromCIDRString (help);

     JVal := Settings.GetValue('SourceAddress');
     assert (assigned (JVal), 'SourceAddress JSON failure (tIPv4AddressFilter.initialize)');
     help := stripquotes (JVal.ToString);
     SourceAddress.setValueFromCIDRString (help);

     JVal := Settings.GetValue('DestinationAddress');
     assert (assigned (JVal), 'DestinationAddress JSON failure (tIPv4AddressFilter.initialize)');
     help := stripquotes (JVal.ToString);
     DestinationAddress.setValueFromCIDRString (help);

     JVal := Settings.GetValue('FirstAddress');
     assert (assigned (JVal), 'FirstAddress JSON failure (tIPv4AddressFilter.initialize)');
     help := stripquotes (JVal.ToString);
     FirstAddress.setValueFromCIDRString (help);

     JVal := Settings.GetValue('LastAddress');
     assert (assigned (JVal), 'LastAddress JSON failure (tIPv4AddressFilter.initialize)');
     help := stripquotes (JVal.ToString);
     LastAddress.setValueFromCIDRString (help);
end;

(* ------------------------------------------- *)

procedure tIPv4AddressFilter.initialize(const IPAddress: tIPv4Address);

begin
     AddressFilterType := ftSingle;
     AddressScope := fsAny;
     SingleAddress := IPAddress;
end;

(* ------------------------------------------- *)

procedure tIPv4AddressFilter.initialize(const IPPair: tIPv4AddressPair);

begin
     AddressFilterType := ftSingle;
     AddressScope := fsBoth;
     SourceAddress := IPPair.Source;
     DestinationAddress := IPPair.Destination;
end;

(* ------------------------------------------- *)

procedure tIPv4AddressFilter.initialize (const SocketPair: tIPv4SocketPair);

begin
     initialize (SocketPair.IPAddressPair);
end;

(* ------------------------------------------- *)

class operator tIPv4AddressFilter.equal (a, b: tIPv4AddressFilter): boolean;

var isSame : boolean;

begin
     isSame := false;

     if (a.AddressFilterType = b.AddressFilterType) and (a.AddressScope = b.AddressScope) then
        begin
             // check if we have the same filter, e.g. when source/destinations are swapped

             case a.AddressScope of
              fsAny         : begin
                                   // any checks if source or destination address are a match

                                   isSame := (a.SingleAddress = b.SingleAddress);
                              end;
              fsSource      : begin
                                   // Checks the source address

                                   isSame := (a.SourceAddress = b.SourceAddress);
                              end;
              fsDestination : begin
                                   // Checks the destination address

                                   isSame := (a.DestinationAddress = b.DestinationAddress);
                              end;
              fsBoth        : begin
                                   // Checks source and destination to be a match

                                   isSame := ((a.SourceAddress = b.SourceAddress) and (a.DestinationAddress = b.DestinationAddress)) or
                                             ((a.SourceAddress = b.DestinationAddress) and (a.DestinationAddress = b.SourceAddress));
                              end;
             end;
        end;

     result := isSame;
end;

(* ------------------------------------------- *)

class operator tIPv4AddressFilter.greaterthan (a, b: tIPv4AddressFilter): boolean;

begin
     if not (a = b) then
        begin
             if (a.SingleAddress = b.SingleAddress) then
                begin
                     if (a.SourceAddress = b.SourceAddress) then
                        result := (a.DestinationAddress > b.DestinationAddress)
                     else
                         result := (a.SourceAddress > b.SourceAddress);
                end
             else
                 result := a.SingleAddress > b.SingleAddress;
        end
     else
         result := false;
end;

(* ------------------------------------------- *)

class operator tIPv4AddressFilter.greaterthanorequal (a, b: tIPv4AddressFilter): boolean;

begin
     result := (a = b) or (a > b);
end;

(* ------------------------------------------- *)

class operator tIPv4AddressFilter.lessthan (a, b: tIPv4AddressFilter): boolean;

begin
     result := not (a >= b);
end;

(* ------------------------------------------- *)

class operator tIPv4AddressFilter.lessthanorequal (a, b: tIPv4AddressFilter): boolean;

begin
     result := not (a > b);
end;

(* ------------------------------------------- *)

class operator tIPv4AddressFilter.notequal (a, b: tIPv4AddressFilter): boolean;

begin
     result := not (a=b);
end;

(* ------------------------------------------- *)

function tIPv4AddressFilter.toJSON: tJSONObject;

var value : tJSONObject;

begin
     value := tJSONObject.Create;

     value.AddPair (TJSONPair.Create ('FilterType', tJSONString.Create (FilterTypeToStr (AddressFilterType))));
     value.AddPair (TJSONPair.Create ('Scope', tJSONString.Create (FilterScopeToStr (AddressScope))));
     value.AddPair (TJSONPair.Create ('SingleAddress', tJSONString.Create (SingleAddress.ValueAsCIDRString)));
     value.AddPair (TJSONPair.Create ('SourceAddress', tJSONString.Create (SourceAddress.ValueAsCIDRString)));
     value.AddPair (TJSONPair.Create ('DestinationAddress', tJSONString.Create (DestinationAddress.ValueAsCIDRString)));
     value.AddPair (TJSONPair.Create ('FirstAddress', tJSONString.Create (FirstAddress.ValueAsCIDRString)));
     value.AddPair (TJSONPair.Create ('LastAddress', tJSONString.Create (LastAddress.ValueAsCIDRString)));

     result := value;
end;

(* ------------------------------------------- *)

function tIPv4AddressFilter.asDisplayFilter (const Bracketed : boolean = false): string;

var value : string;

begin
     case AddressScope of
      fsAny         : begin
                           if SingleAddress.MaskBitCount = 32 then
                              value := 'ip.addr==' + SingleAddress.ValueAsString
                           else
                               value := 'ip.addr==' + SingleAddress.ValueAsCIDRString;
                      end;
      fsSource      : begin
                           if SourceAddress.MaskBitCount = 32 then
                              value := 'ip.src==' + SourceAddress.ValueAsString
                           else
                               value := 'ip.addr==' + SourceAddress.ValueAsCIDRString;
                      end;
      fsDestination : begin
                           if DestinationAddress.MaskBitCount = 32 then
                              value := 'ip.dst==' + DestinationAddress.ValueAsString
                           else
                               value := 'ip.dst==' + DestinationAddress.ValueAsCIDRString;
                      end;
      fsBoth        : begin
                           if SourceAddress.MaskBitCount = 32 then
                              value := 'ip.addr==' + SourceAddress.ValueAsString
                           else
                               value := 'ip.addr==' + SourceAddress.ValueAsCIDRString;

                           value := value + ' and ';

                           if DestinationAddress.MaskBitCount = 32 then
                              value := value + 'ip.addr==' + DestinationAddress.ValueAsString
                           else
                               value := value + 'ip.addr==' + DestinationAddress.ValueAsCIDRString;

                           if Bracketed then
                              value := '(' + value + ')';
                      end;
     end;

     result := value;
end;

(* ------------------------------------------- *)
{ tIPv6AddressFilter }
(* ------------------------------------------- *)

procedure tIPv6AddressFilter.initialize;

begin
     AddressFilterType := ftIgnore;
     AddressScope := fsAny;

     SingleAddress.initialize;
     SourceAddress.initialize;
     DestinationAddress.initialize;
     FirstAddress.initialize;
     LastAddress.initialize;
end;

(* ------------------------------------------- *)

procedure tIPv6AddressFilter.initialize (Settings: tJSONObject);

var JVal  : tJSONValue;
    help  : string;

begin
     initialize;

     JVal := Settings.GetValue ('FilterType');
     assert (assigned (JVal), 'FilterType JSON failure (tIPv6AddressFilter.initialize)');
     AddressFilterType := StrToFilterType (StripQuotes (JVal.Value));

     JVal := Settings.GetValue ('Scope');
     assert (assigned (JVal), 'Scope JSON failure (tIPv6AddressFilter.initialize)');
     AddressScope := StrToFilterScope (StripQuotes (JVal.ToString));

     JVal := Settings.GetValue('SingleAddress');
     assert (assigned (JVal), 'SingleAddress JSON failure (tIPv6AddressFilter.initialize)');
     help := stripquotes (JVal.ToString);
     SingleAddress.setValueFromStringWithMask (help);

     JVal := Settings.GetValue('SourceAddress');
     assert (assigned (JVal), 'SourceAddress JSON failure (tIPv6AddressFilter.initialize)');
     help := stripquotes (JVal.ToString);
     SourceAddress.setValueFromStringWithMask (help);

     JVal := Settings.GetValue('DestinationAddress');
     assert (assigned (JVal), 'DestinationAddress JSON failure (tIPv6AddressFilter.initialize)');
     help := stripquotes (JVal.ToString);
     DestinationAddress.setValueFromStringWithMask (help);

     JVal := Settings.GetValue('FirstAddress');
     assert (assigned (JVal), 'FirstAddress JSON failure (tIPv6AddressFilter.initialize)');
     help := stripquotes (JVal.ToString);
     FirstAddress.setValueFromStringWithMask (help);

     JVal := Settings.GetValue('LastAddress');
     assert (assigned (JVal), 'LastAddress JSON failure (tIPv6AddressFilter.initialize)');
     help := stripquotes (JVal.ToString);
     LastAddress.setValueFromStringWithMask (help);
end;

(* ------------------------------------------- *)

procedure tIPv6AddressFilter.initialize (const IPAddress: tIPv6Address);

begin
     AddressFilterType := ftSingle;
     AddressScope := fsAny;
     SingleAddress := IPAddress;
end;

(* ------------------------------------------- *)

procedure tIPv6AddressFilter.initialize (const IPPair: tIPv6AddressPair);

begin
     AddressFilterType := ftSingle;
     AddressScope := fsBoth;
     SourceAddress := IPPair.Source;
     DestinationAddress := IPPair.Destination;
end;

(* ------------------------------------------- *)

procedure tIPv6AddressFilter.initialize (const SocketPair: tIPv6SocketPair);

begin
     initialize (SocketPair.IPAddressPair);
end;

(* ------------------------------------------- *)

class operator tIPv6AddressFilter.equal (a, b: tIPv6AddressFilter): boolean;

var isSame : boolean;

begin
     isSame := false;

     if (a.AddressFilterType = b.AddressFilterType) and (a.AddressScope = b.AddressScope) then
        begin
             // check if we have the same filter, e.g. when source/destinations are swapped

             case a.AddressScope of
              fsAny         : begin
                                   // any checks if source or destination address are a match

                                   isSame := (a.SingleAddress = b.SingleAddress);
                              end;
              fsSource      : begin
                                   // Checks the source address

                                   isSame := (a.SourceAddress = b.SourceAddress);
                              end;
              fsDestination : begin
                                   // Checks the destination address

                                   isSame := (a.DestinationAddress = b.DestinationAddress);
                              end;
              fsBoth        : begin
                                   // Checks source and destination to be a match

                                   isSame := ((a.SourceAddress = b.SourceAddress) and (a.DestinationAddress = b.DestinationAddress)) or
                                             ((a.SourceAddress = b.DestinationAddress) and (a.DestinationAddress = b.SourceAddress));
                              end;
             end;
        end;

     result := isSame;

{
     result := (a.AddressFilterType = b.AddressFilterType) and
               (a.AddressScope = b.AddressScope) and
               (a.SingleAddress = b.SingleAddress) and
               (a.SourceAddress = b.SourceAddress) and
               (a.DestinationAddress = b.DestinationAddress) and
               (a.FirstAddress = b.FirstAddress) and
               (a.LastAddress = b.LastAddress);
}
end;

(* ------------------------------------------- *)

class operator tIPv6AddressFilter.notequal (a, b: tIPv6AddressFilter): boolean;

begin
     result := not (a=b);
end;

(* ------------------------------------------- *)

class operator tIPv6AddressFilter.greaterthan (a, b: tIPv6AddressFilter): boolean;

begin
     if not (a = b) then
        begin
             if (a.SingleAddress = b.SingleAddress) then
                begin
                     if (a.SourceAddress = b.SourceAddress) then
                        result := (a.DestinationAddress > b.DestinationAddress)
                     else
                         result := (a.SourceAddress > b.SourceAddress);
                end
             else
                 result := a.SingleAddress > b.SingleAddress;
        end
     else
         result := false;
end;

(* ------------------------------------------- *)

class operator tIPv6AddressFilter.greaterthanorequal (a, b: tIPv6AddressFilter): boolean;

begin
     result := (a = b) or (a > b);
end;

(* ------------------------------------------- *)

class operator tIPv6AddressFilter.lessthan (a, b: tIPv6AddressFilter): boolean;

begin
     result := not (a >= b);
end;

(* ------------------------------------------- *)

class operator tIPv6AddressFilter.lessthanorequal (a, b: tIPv6AddressFilter): boolean;

begin
     result := not (a > b);
end;

(* ------------------------------------------- *)

function tIPv6AddressFilter.toJSON: tJSONObject;

var value : tJSONObject;

begin
     value := tJSONObject.Create;

     value.AddPair (TJSONPair.Create ('FilterType', tJSONString.Create (FilterTypeToStr (AddressFilterType))));
     value.AddPair (TJSONPair.Create ('Scope', tJSONString.Create (FilterScopeToStr (AddressScope))));
     value.AddPair (TJSONPair.Create ('SingleAddress', tJSONString.Create (SingleAddress.ValueAsSlashedNotationString)));
     value.AddPair (TJSONPair.Create ('SourceAddress', tJSONString.Create (SourceAddress.ValueAsSlashedNotationString)));
     value.AddPair (TJSONPair.Create ('DestinationAddress', tJSONString.Create (DestinationAddress.ValueAsSlashedNotationString)));
     value.AddPair (TJSONPair.Create ('FirstAddress', tJSONString.Create (FirstAddress.ValueAsSlashedNotationString)));
     value.AddPair (TJSONPair.Create ('LastAddress', tJSONString.Create (LastAddress.ValueAsSlashedNotationString)));

     result := value;
end;

(* ------------------------------------------- *)

function tIPv6AddressFilter.asDisplayFilter (const Bracketed : boolean = false) : string;

var value : string;

begin
     case AddressScope of
      fsAny         : begin
                           if SingleAddress.IsPrefixInformation then
                              value := 'ip.addr==' + SingleAddress.ValueAsSlashedNotationString
                           else
                               value := 'ip.addr==' + SingleAddress.ValueAsString;
                      end;
      fsSource      : begin
                           if SourceAddress.IsPrefixInformation then
                              value := 'ip.src==' + SourceAddress.ValueAsSlashedNotationString
                           else
                               value := 'ip.addr==' + SourceAddress.ValueAsString;
                      end;
      fsDestination : begin
                           if DestinationAddress.IsPrefixInformation then
                              value := 'ip.dst==' + DestinationAddress.ValueAsSlashedNotationString
                           else
                               value := 'ip.dst==' + DestinationAddress.ValueAsString;
                      end;
      fsBoth        : begin
                           if SourceAddress.IsPrefixInformation then
                              value := 'ip.addr==' + SourceAddress.ValueAsSlashedNotationString
                           else
                               value := 'ip.addr==' + SourceAddress.ValueAsString;

                           value := value + ' and ';

                           if DestinationAddress.IsPrefixInformation then
                              value := value + 'ip.addr==' + DestinationAddress.ValueAsSlashedNotationString
                           else
                               value := value + 'ip.addr==' + DestinationAddress.ValueAsString;

                           if Bracketed then
                              value := '(' + value + ')';
                      end;
     end;

     result := value;
end;

(* ------------------------------------------- *)
{ tICMPFilter }
(* ------------------------------------------- *)

procedure tICMPFilter.initialize;

begin
     ICMPFilterType := IFTIgnore;
     ICMPType := 0;
     ICMPCode := 0;
end;

(* ------------------------------------------- *)

procedure tICMPFilter.initialize (Settings: tJSONObject);

var JVal  : tJSONValue;
    JPair : tJSONPair;
    JNum  : tJSONNumber;
    help  : string;

begin
     initialize;

     JVal := Settings.GetValue ('FilterType');
     ICMPFilterType := StrToICMPFilterType (StripQuotes (JVal.Value));

     JPair := Settings.Get ('ICMPType');
     assert (assigned (JPair), 'ICMPType JSON failure (tICMPFilter.initialize)');
     JNum := JPair.JsonValue as tJSONNUmber;
     ICMPType := JNum.AsInt;

     JPair := Settings.Get ('ICMPCode');
     assert (assigned (JPair), 'ICMPCode JSON failure (tICMPFilter.initialize)');
     JNum := JPair.JsonValue as tJSONNUmber;
     ICMPCode := JNum.AsInt;
end;

(* ------------------------------------------- *)

class operator tICMPFilter.equal (a, b: tICMPFilter): boolean;

begin
     result := (a.ICMPFilterType = b.ICMPFilterType) and
               (a.ICMPType = b.ICMPType) and
               (a.ICMPCode = b.ICMPCode);
end;

(* ------------------------------------------- *)

class operator tICMPFilter.notequal (a, b: tICMPFilter): boolean;

begin
     result := not (a=b);
end;

(* ------------------------------------------- *)

function tICMPFilter.toJSON: tJSONObject;

var value : tJSONObject;

begin
     value := tJSONObject.Create;

     value.AddPair (TJSONPair.Create ('FilterType', tJSONString.Create (ICMPFilterTypeToStr (ICMPFilterType))));
     value.AddPair (TJSONPair.Create ('ICMPType', tJSONNumber.Create (ICMPType)));
     value.AddPair (TJSONPair.Create ('ICMPCode', tJSONNumber.Create (ICMPCode)));

     result := value;
end;

(* ------------------------------------------- *)

function tICMPFilter.asDisplayFilter (const Bracketed: boolean) : string;

var value : string;

begin
     case ICMPFilterType of
      IFTReject : result := 'not icmp';
      IFTAny    : result := 'icmp';
      IFTType   : begin
                       result := 'icmp.type==' + ICMPType.ToString;

                       if Bracketed then
                          result := '(' + result  + ')';
                  end;
     end;
end;

(* ------------------------------------------- *)
{ tPortFilter }
(* ------------------------------------------- *)

procedure tPortFilter.initialize;

begin
     Layer4Protocol := l4Any;
     PortFilterType := ftIgnore;
     PortScope := fsAny;

     SinglePort := 0;
     SourcePort := 0;
     DestinationPort := 0;
     FirstPort := 0;
     LastPort := 0;
end;

(* ------------------------------------------- *)

procedure tPortFilter.initialize (Settings: tJSONObject);

var JVal  : tJSONValue;
    JPair : tJSONPair;
    JNum  : tJSONNumber;
    help  : string;

begin
     initialize;

     JVal := Settings.GetValue ('Layer4Protocol');
     assert (assigned (JVal), 'Layer4Protocol JSON failure (tPortFilter.initialize)');
     Layer4Protocol := StrToLayer4Protocol (StripQuotes (JVal.Value));

     JVal := Settings.GetValue ('FilterType');
     assert (assigned (JVal), 'FilterType JSON failure (tPortFilter.initialize)');
     PortFilterType := StrToFilterType (StripQuotes (JVal.Value));

     JVal := Settings.GetValue ('Scope');
     assert (assigned (JVal), 'Scope JSON failure (tPortFilter.initialize)');
     PortScope := StrToFilterScope (StripQuotes (JVal.ToString));

     JPair := Settings.Get ('SinglePort');
     assert (assigned (JPair), 'SinglePort JSON failure (tPortFilter.initialize)');
     JNum := JPair.JsonValue as tJSONNUmber;
     SinglePort := JNum.AsInt;

     JPair := Settings.Get ('SourcePort');
     assert (assigned (JPair), 'SourcePort JSON failure (tPortFilter.initialize)');
     JNum := JPair.JsonValue as tJSONNUmber;
     SourcePort := JNum.AsInt;

     JPair := Settings.Get ('DestinationPort');
     assert (assigned (JPair), 'DestinationPort JSON failure (tPortFilter.initialize)');
     JNum := JPair.JsonValue as tJSONNUmber;
     DestinationPort := JNum.AsInt;

     JPair := Settings.Get ('FirstPort');
     assert (assigned (JPair), 'FirstPort JSON failure (tPortFilter.initialize)');
     JNum := JPair.JsonValue as tJSONNUmber;
     FirstPort := JNum.AsInt;

     JPair := Settings.Get ('LastPort');
     assert (assigned (JPair), 'LastPort JSON failure (tPortFilter.initialize)');
     JNum := JPair.JsonValue as tJSONNUmber;
     LastPort := JNum.AsInt;
end;

(* ------------------------------------------- *)

procedure tPortFilter.initialize (const SocketPair: tIPv4SocketPair; const Layer4Prot : tLayer4Protocol);

begin
     Layer4Protocol := Layer4Prot;
     PortScope := fsBoth;
     PortFilterType := ftSingle;
     SourcePort := socketpair.Source.Port;
     DestinationPort := socketpair.Destination.Port;
end;

(* ------------------------------------------- *)

procedure tPortFilter.initialize(const Port: integer);

begin
     Layer4Protocol := l4Any;
     PortScope := fsAny;
     PortFilterType := ftSingle;
     SinglePort := Port;
end;

(* ------------------------------------------- *)

procedure tPortFilter.initialize (const SocketPair: tIPv6SocketPair; const Layer4Prot : tLayer4Protocol);

begin
     Layer4Protocol := Layer4Prot;
     PortScope := fsBoth;
     PortFilterType := ftSingle;
     SourcePort := socketpair.Source.Port;
     DestinationPort := socketpair.Destination.Port;
end;

(* ------------------------------------------- *)

class operator tPortFilter.equal(a, b: tPortFilter): boolean;

var isSame : boolean;

begin
     isSame := false;

     if (a.Layer4Protocol = b.Layer4Protocol) and (a.PortFilterType = b.PortFilterType) and (a.PortScope = b.PortScope) then
        begin
             // check if we have the same filter, e.g. when source/destinations are swapped

             case a.PortScope of
              fsAny         : begin
                                   // any checks if source or destination address are a match

                                   isSame := (a.SinglePort = b.SinglePort);
                              end;
              fsSource      : begin
                                   // Checks the source address

                                   isSame := (a.SourcePort = b.SourcePort);
                              end;
              fsDestination : begin
                                   // Checks the destination address

                                   isSame := (a.DestinationPort = b.DestinationPort);
                              end;
              fsBoth        : begin
                                   // Checks source and destination to be a match

                                   isSame := ((a.SourcePort = b.SourcePort) and (a.DestinationPort = b.DestinationPort)) or
                                             ((a.SourcePort = b.DestinationPort) and (a.DestinationPort = b.SourcePort));
                              end;
             end;
        end;

     result := isSame;
{
     result := (a.Layer4Protocol = b.Layer4Protocol) and
               (a.PortFilterType = b.PortFilterType) and
               (a.PortScope = b.PortScope) and
               (a.SinglePort = b.SinglePort) and
               (a.SourcePort = b.SourcePort) and
               (a.DestinationPort = b.DestinationPort) and
               (a.FirstPort = b.FirstPort) and
               (a.LastPort = b.LastPort);
}
end;

(* ------------------------------------------- *)

class operator tPortFilter.notequal(a, b: tPortFilter): boolean;

begin
     result := not (a=b);
end;

(* ------------------------------------------- *)

class operator tPortFilter.greaterthan (a, b: tPortFilter): boolean;

begin
     if not (a = b) then
        begin
             if (a.Layer4Protocol = b.Layer4Protocol) then
                begin
                     if (a.SinglePort = b.SinglePort) then
                        begin
                             if (a.SourcePort = b.SourcePort) then
                                begin
                                     if (a.DestinationPort = b.DestinationPort) then
                                        begin
                                             if (a.FirstPort = b.FirstPort) then
                                                result := (a.LastPort > b.LastPort)
                                             else
                                                 result := (a.FirstPort > b.FirstPort);
                                        end
                                     else
                                         result := (a.DestinationPort > b.DestinationPort);
                                end
                             else
                                 result := (a.SourcePort > b.SourcePort);
                        end
                     else
                         result := (a.SinglePort > b.SinglePort);
                end
             else
                 result := (a.Layer4Protocol > b.Layer4Protocol);
        end
     else
         result := false;
end;

(* ------------------------------------------- *)

class operator tPortFilter.greaterthanorequal (a, b: tPortFilter): boolean;

begin
     result := (a = b) or (a > b);
end;

(* ------------------------------------------- *)

class operator tPortFilter.lessthan (a, b: tPortFilter): boolean;

begin
     result := not (a >= b);
end;

(* ------------------------------------------- *)

class operator tPortFilter.lessthanorequal (a, b: tPortFilter): boolean;

begin
     result := not (a > b);
end;

(* ------------------------------------------- *)

function tPortFilter.toJSON: tJSONObject;

var value : tJSONObject;

begin
     value := tJSONObject.Create;

     value.AddPair (TJSONPair.Create ('Layer4Protocol', tJSONString.Create (Layer4ProtocolToStr (Layer4Protocol))));
     value.AddPair (TJSONPair.Create ('FilterType', tJSONString.Create (FilterTypeToStr (PortFilterType))));
     value.AddPair (TJSONPair.Create ('Scope', tJSONString.Create (FilterScopeToStr (PortScope))));
     value.AddPair (TJSONPair.Create ('SinglePort', tJSONNumber.Create (SinglePort)));
     value.AddPair (TJSONPair.Create ('SourcePort', tJSONNumber.Create (SourcePort)));
     value.AddPair (TJSONPair.Create ('DestinationPort', tJSONNumber.Create (DestinationPort)));
     value.AddPair (TJSONPair.Create ('FirstPort', tJSONNumber.Create (FirstPort)));
     value.AddPair (TJSONPair.Create ('LastPort', tJSONNumber.Create (LastPort)));

     result := value;
end;

(* ------------------------------------------- *)

function tPortFilter.asDisplayFilter (const Bracketed : boolean = false) : string;

var value : string;

begin
     case Layer4Protocol of
      l4Any,
      l4TCPUDP : begin
                      case PortScope of
                       fsAny         : value := 'tcp.port==' + IntToStr (SinglePort) + ' or ' + 'udp.port==' + IntToStr (SinglePort);
                       fsSource      : value := 'tcp.srcport==' + IntToStr (SourcePort) + ' or ' + 'udp.srcport==' + IntToStr (SourcePort);
                       fsDestination : value := 'tcp.dstport==' + IntToStr (DestinationPort) + ' or ' + 'udp.dstport==' + IntToStr (DestinationPort);
                       fsBoth        : begin
                                            value := '(tcp.port==' + IntToStr (SourcePort) + ' and tcp.port==' + IntToStr (DestinationPort) + ') or ' +
                                                     '(udp.port==' + IntToStr (SourcePort) + ' and udp.port==' + IntToStr (DestinationPort) + ')';

                                            if Bracketed then
                                               value := '(' + value + ')';
                                       end;
                      end;
                 end;
      l4TCP    : begin
                      case PortScope of
                       fsAny         : value := 'tcp.port==' + IntToStr (SinglePort);
                       fsSource      : value := 'tcp.srcport==' + IntToStr (SourcePort);
                       fsDestination : value := 'tcp.dstport==' + IntToStr (DestinationPort);
                       fsBoth        : begin
                                            if Bracketed then
                                               value := '(tcp.port==' + IntToStr (SourcePort) + ' and tcp.port==' + IntToStr (DestinationPort) + ')'
                                            else
                                                value := 'tcp.port==' + IntToStr (SourcePort) + ' and tcp.port==' + IntToStr (DestinationPort);
                                       end;
                      end;
                 end;
      l4UDP    : begin
                      case PortScope of
                       fsAny         : value := 'udp.port==' + IntToStr (SinglePort);
                       fsSource      : value := 'udp.srcport==' + IntToStr (SourcePort);
                       fsDestination : value := 'udp.dstport==' + IntToStr (DestinationPort);
                       fsBoth        : begin
                                            if Bracketed then
                                               value := '(udp.port==' + IntToStr (SourcePort) + ' and udp.port==' + IntToStr (DestinationPort) + ')'
                                            else
                                                value := 'udp.port==' + IntToStr (SourcePort) + ' and udp.port==' + IntToStr (DestinationPort);
                                       end;
                      end;
                 end;
      l4ICMP   : begin
                      value := 'icmp';
                 end;
     end;

     result := value;
end;

(* ------------------------------------------- *)
{ tTCPFilter }
(* ------------------------------------------- *)

procedure tTCPFilter.initialize;

begin
     TCPFilterType := ftIgnore;
     flags := [];
end;

(* ------------------------------------------- *)

procedure tTCPFilter.initialize (Settings: tJSONObject);

var JVal  : tJSONValue;
    JPair : tJSONPair;
    JArr  : tJSONArray;
    flag  : tTCPFlags;
    help  : string;
    i     : Integer;

begin
     initialize;

     JVal := Settings.GetValue ('FilterType');
     assert (assigned (JVal), 'FilterType JSON failure (tTCPFilter.initialize)');
     TCPFilterType := StrToFilterType (StripQuotes (JVal.Value));

     JPair := Settings.Get ('Flags') as TJSONPair;
     assert (assigned (JPair), 'Flags JSON failure (tTCPFilter.initialize)');
     JArr := JPair.JSONValue as tJSONArray;

     for i := 1 to JArr.Size do
         begin
              help := JArr.Get(i-1).ToString;
              flag := StringToTCPFlag (stripquotes (help));

              flags := flags + [flag];
         end;

     JPair := Settings.Get ('TCPHandshake');
     assert (assigned (JPair), 'TCPHandshake JSON failure (tTCPFilter.initialize)');
     TCPHandshake := JPair.JSONValue is tJSONTrue;
end;

(* ------------------------------------------- *)

class operator tTCPFilter.equal(a, b: tTCPFilter): boolean;

begin
     result := (a.TCPFilterType = b.TCPFilterType) and
               (a.Flags = b.Flags) and
               (a.TCPHandshake = b.TCPHandshake);
end;

(* ------------------------------------------- *)

class operator tTCPFilter.notequal(a, b: tTCPFilter): boolean;

begin
     result := not (a=b);
end;

(* ------------------------------------------- *)

function tTCPFilter.toJSON: tJSONObject;

var value     : tJSONObject;
    flagvalue : tJSONArray;
    flag      : tTCPFlags;

begin
     value := tJSONObject.Create;

     value.AddPair (TJSONPair.Create ('FilterType', tJSONString.Create (FilterTypeToStr (TCPFilterType))));

     if flags <> [] then
        begin
             flagvalue := tJSONArray.Create;

             for flag in flags do
                 flagvalue.AddElement(tJSONString.Create (TCPFlagToString (flag)));

             value.AddPair ('Flags', flagvalue);
        end;

     if TCPHandshake then
        value.AddPair ('TCPHandshake', tJSONTrue.Create)
     else
         value.AddPair ('TCPHandshake', tJSONFalse.Create);

     result := value;
end;

(* ------------------------------------------- *)

function tTCPFilter.asDisplayFilter (const Bracketed : boolean = false) : string;

var value : string;
    flag  : tTCPFlags;

begin
     value := '';

     for flag in flags do
         begin
              case flag of
               tcpSYN : value := 'tcp.flags.syn==1';
               tcpACK : begin
                             if value = '' then
                                value := 'tcp.flags.ack==1'
                             else
                                 value := value + ' and tcp.flags.ack==1'
                        end;
               tcpFIN : begin
                             if value = '' then
                                value := 'tcp.flags.ack==1'
                             else
                                 value := value + ' and tcp.flags.ack==1'
                        end;
               tcpRST : begin
                             if value = '' then
                                value := 'tcp.flags.reset==1'
                             else
                                 value := value + ' and tcp.flags.reset==1'
                        end;
               tcpPSH : begin
                             if value = '' then
                                value := 'tcp.flags.psh==1'
                             else
                                 value := value + ' and tcp.flags.psh==1'
                        end;
               tcpURG : begin
                             if value = '' then
                                value := 'tcp.flags.urg==1'
                             else
                                 value := value + ' and tcp.flags.urg==1'
                        end;
               tcpECN : begin
                             if value = '' then
                                value := 'tcp.flags.ecn==1'
                             else
                                 value := value + ' and tcp.flags.ecn==1'
                        end;
               tcpCWR : begin
                             if value = '' then
                                value := 'tcp.flags.cwr==1'
                             else
                                 value := value + ' and tcp.flags.cwr==1'
                        end;
              end;
         end;

     if TCPHandshake then
        begin
             if value = '' then
                value := 'tcp.handshake'
             else
                 value := value + 'and tcp.handshake';
        end;

     if Bracketed and (pos (' and ', value) > 0) then
        value := '(' + value + ')';

     result := value;
end;

(* ------------------------------------------- *)
{ tFrameFilter }
(* ------------------------------------------- *)

procedure tFrameFilter.initialize;

begin
     MACFilter.initialize;
     IPv4Filter.initialize;
     IPv6Filter.initialize;
     ICMPFilter.initialize;
     PortFilter.initialize;
     TCPFilter.initialize;

     FilterLabel := '';

     HelpRecType := csrNone;
     HelpFirstSeen.initialize;
     HelpLastSeen.initialize;
     HelpConversationIndex := 0;
end;

(* ------------------------------------------- *)

procedure tFrameFilter.initialize (FilterSettings : tJSONObject);

var subFilter : tJSONObject;
    JVal      : tJSONValue;

begin
     subFilter := FilterSettings.GetValue ('MACFilter') as tJSONObject;

     if subfilter <> nil then
        MACFilter.initialize (subFilter)
     else
         MACFilter.initialize;

     subFilter := FilterSettings.GetValue ('IPv4Filter') as tJSONObject;

     if subfilter <> nil then
        IPv4Filter.initialize (subFilter)
     else
         IPv4Filter.initialize;

     subFilter := FilterSettings.GetValue ('IPv6Filter') as tJSONObject;

     if subfilter <> nil then
        IPv6Filter.initialize (subFilter)
     else
         IPv6Filter.initialize;

     subFilter := FilterSettings.GetValue ('ICMPFilter') as tJSONObject;

     if subfilter <> nil then
        ICMPFilter.initialize (subfilter)
     else
         ICMPFilter.initialize;

     subFilter := FilterSettings.GetValue ('PortFilter') as tJSONObject;

     if subfilter <> nil then
        PortFilter.initialize (subFilter)
     else
         PortFilter.initialize;

     subFilter := FilterSettings.GetValue ('TCPFilter') as tJSONObject;

     if subfilter <> nil then
        TCPFilter.initialize (subFilter)
     else
         TCPFilter.initialize;

     JVal := FilterSettings.GetValue ('FilterLabel');
     FilterLabel := StripQuotes (JVal.ToString);

     HelpRecType := csrNone;
     HelpFirstSeen.initialize;
     HelpLastSeen.initialize;
     HelpConversationIndex := 0;
end;

(* ------------------------------------------- *)

procedure tFrameFilter.initialize (FilterSettings : string);

var json : tJSONObject;

begin
     json := TJSONObject.ParseJSONValue (TEncoding.ASCII.GetBytes (FilterSettings),0) as TJSONObject;

     initialize (json);

     json.Free;
end;

(* ------------------------------------------- *)

procedure tFrameFilter.initialize (const Tuple: t5Tuple; const NewLabel : string = '');

begin
     initialize;

     case Tuple.IPVersion of
      4 : begin
               IPv4Filter.initialize (Tuple.IPv4SocketPair);
               PortFilter.initialize (Tuple.IPv4SocketPair, Tuple.Layer4Protocol);
          end;
      6 : begin
               IPv6Filter.initialize (Tuple.IPv6SocketPair);
               PortFilter.initialize (Tuple.IPv6SocketPair, Tuple.Layer4Protocol);
          end;
     end;

     FilterLabel := NewLabel;
end;

(* ------------------------------------------- *)

procedure tFrameFilter.updateRecType;

begin
     if (IPv4Filter.AddressFilterType = ftSingle) then
        begin
             // This is at least an IPv4 Address filter

             case IPv4Filter.AddressScope of
              fsAny,
              fsSource,
              fsDestination : begin
                                   HelpRecType := csrIPv4Endpoint;

                                   // Theoretically we could determine a Socket endpoint, but we don't have a type like that atm
                              end;
              fsBoth        : begin
                                   HelpRecType := csrIPv4Conversation;

                                   // Check if it's a socket pair

                                   if PortFilter.PortScope = fsBoth then
                                      begin
                                           case PortFilter.Layer4Protocol of
                                            l4ICMP,
                                            l4Any  : HelpRecType := csrL4Conversation;
                                            l4TCP  : HelpRecType := csrTCPConversation;
                                            l4UDP  : HelpRecType := csrUDPConversation;
                                           end;
                                      end;
                              end;
             end;
        end
     else
         if (IPv6Filter.AddressFilterType = ftSingle) then
            begin
                 // This is at least an IPv4 Address filter

                 case IPv6Filter.AddressScope of
                  fsAny,
                  fsSource,
                  fsDestination : begin
                                       HelpRecType := csrIPv6Endpoint;

                                       // Theoretically we could determine a Socket endpoint, but we don't have a type like that atm
                                  end;
                  fsBoth        : begin
                                       HelpRecType := csrIPv6Conversation;

                                       // Check if it's a socket pair

                                       if PortFilter.PortScope = fsBoth then
                                          begin
                                               case PortFilter.Layer4Protocol of
                                                l4ICMP,
                                                l4Any  : HelpRecType := csrL4Conversation;
                                                l4TCP  : HelpRecType := csrTCPConversation;
                                                l4UDP  : HelpRecType := csrUDPConversation;
                                               end;
                                          end;
                                  end;
                 end;
            end
         else
             begin
                  // must be Ethenet stuff, if anything

                  if MACFilter.AddressFilterType = ftSingle then
                     begin
                          HelpRecType := csrMACEndpoint;

                          if MACFilter.AddressScope = fsBoth then
                             HelpRecType := csrMACConversation;
                     end
                  else
                      HelpRecType := csrNone;
             end;
end;

(* ------------------------------------------- *)

class operator tFrameFilter.equal (a, b: tFrameFilter): boolean;

begin
     // The FilterLabel is not checked as it is not part of the filtering itself

     result := (a.MACFilter = b.MACFilter) and
               (a.IPv4Filter = b.IPv4Filter) and
               (a.IPv6Filter = b.IPv6Filter) and
               (a.PortFilter = b.PortFilter) and
               (a.TCPFilter = b.TCPFilter);
end;

(* ------------------------------------------- *)

class operator tFrameFilter.notequal(a, b: tFrameFilter): boolean;

begin
     result := not (a=b);
end;

(* ------------------------------------------- *)

function tFrameFilter.isUnspecified: boolean;

begin
     result := (SectionCount = 0)
end;

(* ------------------------------------------- *)

class operator tFrameFilter.greaterthan (a, b: tFrameFilter): boolean;

begin
     if (a.MACFilter = b.MACFilter) then
        begin
             if (a.IPv4Filter = b.IPv4Filter) then
                begin
                     if (a.IPv6Filter = b.IPv6Filter) then
                        result := (a.PortFilter > b.PortFilter)
                     else
                         result := (a.IPv6Filter > b.IPv6Filter);
                end
             else
                 result := (a.IPv4Filter > b.IPv4Filter);
        end
     else
         result := false;
end;

(* ------------------------------------------- *)

class operator tFrameFilter.greaterthanorequal (a, b: tFrameFilter): boolean;

begin
     result := (a = b) or (a > b);
end;

(* ------------------------------------------- *)

class operator tFrameFilter.lessthan (a, b: tFrameFilter): boolean;

begin
     result := not (a >= b);
end;

(* ------------------------------------------- *)

class operator tFrameFilter.lessthanorequal (a, b: tFrameFilter): boolean;

begin
     result := not (a > b);
end;

(* ------------------------------------------- *)

function tFrameFilter.SectionCount: integer;

var subcount : integer;

begin
     // Count the number of sub items

     subcount := 0;

     if MACFilter.AddressFilterType <> ftIgnore then
        inc (subcount);

     if IPv4Filter.AddressFilterType <> ftIgnore then
        inc (subcount);

     if IPv6Filter.AddressFilterType <> ftIgnore then
        inc (subcount);

     if ICMPFilter.ICMPFilterType <> IFTIgnore then
        inc (subcount);

     if PortFilter.PortFilterType <> ftIgnore then
        inc (subcount);

     if TCPFilter.TCPFilterType <> ftIgnore then
        inc (subcount);

     result := subcount;
end;

(* ------------------------------------------- *)

function tFrameFilter.toJSON (const IncludeNonFilterItems : boolean = True) : tJSONObject;

var value : tJSONObject;

begin
     value := tJSONObject.Create;

     value.AddPair (TJSONPair.Create ('FilterVersion', tJSONString.Create (FilterVersion)));

     if MACFilter.AddressFilterType <> ftIgnore then
        value.AddPair ('MACFilter', MACFilter.toJSON);

     if IPv4Filter.AddressFilterType <> ftIgnore then
        value.AddPair ('IPv4Filter', IPv4Filter.toJSON);

     if IPv6Filter.AddressFilterType <> ftIgnore then
        value.AddPair ('IPv6Filter', IPv6Filter.toJSON);

     if ICMPFilter.ICMPFilterType <> IFTIgnore then
        value.AddPair ('ICMPFilter', ICMPFilter.toJSON);

     if PortFilter.PortFilterType <> ftIgnore then
        value.AddPair ('PortFilter', PortFilter.toJSON);

     if TCPFilter.TCPFilterType <> ftIgnore then
        value.AddPair ('TCPFilter', TCPFilter.toJSON);

     // We leave out not frame matching relevant items if required, in case we need to compare
     // filters for matching functionality only

     if IncludeNonFilterItems then
        begin
             value.AddPair (TJSONPair.Create ('FilterLabel', tJSONString.Create (FilterLabel)));
        end;

     result := value;
end;

(* ------------------------------------------- *)

function tFrameFilter.DictionaryKey : string;

var help : TJSONObject;

begin
     help := self.toJSON(False);
     result := help.ToString;

     help.Free;
end;

(* ------------------------------------------- *)

function tFrameFilter.asDisplayFilter: string;

var value     : string;
    boBracket : boolean;
    selcount  : integer;

begin
     selcount := SectionCount;

     if selcount > 0 then
        begin
             boBracket := (selcount > 1);

             // Construct display filter string

             if MACFilter.AddressFilterType <> ftIgnore then
                value := MACFilter.asDisplayFilter (boBracket)
             else
                 value := '';

             if IPv4Filter.AddressFilterType <> ftIgnore then
                if value = '' then
                   value := IPv4Filter.asDisplayFilter (boBracket)
                else
                    value := value + ' and ' + IPv4Filter.asDisplayFilter (boBracket);

             if IPv6Filter.AddressFilterType <> ftIgnore then
                if value = '' then
                   value := IPv6Filter.asDisplayFilter (boBracket)
                else
                    value := value + ' and ' + IPv6Filter.asDisplayFilter (boBracket);

             if ICMPFilter.ICMPFilterType <> IFTIgnore then
                if value = '' then
                   value := ICMPFilter.asDisplayFilter (boBracket)
                else
                    value := value + ' and ' + ICMPFilter.asDisplayFilter (boBracket);

             if PortFilter.PortFilterType <> ftIgnore then
                if value = '' then
                   value := PortFilter.asDisplayFilter (boBracket)
                else
                    value := value + ' and ' + PortFilter.asDisplayFilter (boBracket);

             if TCPFilter.TCPFilterType <> ftIgnore then
                if value = '' then
                   value := TCPFilter.asDisplayFilter (boBracket)
                else
                    value := value + ' and ' + TCPFilter.asDisplayFilter (boBracket);
        end
     else
         value := '';

     result := value;
end;

(* ------------------------------------------- *)

function tFrameFilter.asSocketString: string;

var value     : string;
    selcount  : integer;

begin
     selcount := SectionCount;

     if selcount > 0 then
        begin
             if IPv4Filter.AddressFilterType <> ftIgnore then
                if value = '' then
                   value := IPv4Filter.SourceAddress.ValueAsString + ':' + IntToStr (PortFilter.SourcePort) + '-' + IPv4Filter.DestinationAddress.ValueAsString + ':' + IntToStr (PortFilter.DestinationPort);

             if IPv6Filter.AddressFilterType <> ftIgnore then
                value := IPv6Filter.SourceAddress.ValueAsString + ':' + IntToStr (PortFilter.SourcePort) + '-' + IPv6Filter.DestinationAddress.ValueAsString + ':' + IntToStr (PortFilter.DestinationPort);
        end
     else
         value := '';

     result := value;
end;

(* ------------------------------------------- *)

function tFrameFilter.as5Tuple: t5Tuple;

var value     : t5Tuple;
    boOkay    : boolean;

begin
     value.initialize;

     if HelpRecType = csrNone then
        updateRecType;

     // Check if this is a tuple filter at all
{
     boOkay := ((IPv4Filter.AddressFilterType = ftSingle) or (IPv6Filter.AddressFilterType = ftSingle)) and
                (PortFilter.PortFilterType = ftSingle);

     if boOkay then
        begin
             if (IPv4Filter.AddressFilterType = ftSingle) then
                if (IPv4Filter.AddressScope <> fsBoth) then
                   boOkay := False;

             if (IPv6Filter.AddressFilterType = ftSingle) then
                if (IPv6Filter.AddressScope <> fsBoth) then
                   boOkay := False;

             if (PortFilter.PortScope <> fsBoth) then
                boOkay := False;
        end;

     // If so, build the tuple

     if boOkay then
}
     if HelpRecType in [csrTCPConversation, csrUDPConversation] then
        begin
             if IPv4Filter.AddressFilterType <> ftIgnore then
                begin
                     value.IPVersion := 4;
                     value.Layer4Protocol := PortFilter.Layer4Protocol;
                     value.IPv4SocketPair.Source.IPAddress := IPv4Filter.SourceAddress;
                     value.IPv4SocketPair.Destination.IPAddress := IPv4Filter.DestinationAddress;

                     case PortFilter.Layer4Protocol of
                      l4Any,
                      l4ICMP : begin
                                    value.IPv4SocketPair.Source.Port := 0;
                                    value.IPv4SocketPair.Destination.Port := 0;
                               end;
                      l4TCP,
                      l4UDP  : begin
                                    value.IPv4SocketPair.Source.Port := PortFilter.SourcePort;
                                    value.IPv4SocketPair.Destination.Port := PortFilter.DestinationPort;
                               end;
                     end;
                end;

             if IPv6Filter.AddressFilterType <> ftIgnore then
                begin
                     value.IPVersion := 6;
                     value.Layer4Protocol := PortFilter.Layer4Protocol;
                     value.IPv6SocketPair.Source.IPAddress := IPv6Filter.SourceAddress;
                     value.IPv6SocketPair.Destination.IPAddress := IPv6Filter.DestinationAddress;

                     case PortFilter.Layer4Protocol of
                      l4Any,
                      l4ICMP : begin
                                    value.IPv6SocketPair.Source.Port := 0;
                                    value.IPv6SocketPair.Destination.Port := 0;
                               end;
                      l4TCP,
                      l4UDP  : begin
                                    value.IPv6SocketPair.Source.Port := PortFilter.SourcePort;
                                    value.IPv6SocketPair.Destination.Port := PortFilter.DestinationPort;
                               end;
                     end;
                end;
        end;

     result := value;
end;

(* ------------------------------------------- *)
{ tFilterSet }
(* ------------------------------------------- *)

constructor tFilterSet.create;

begin
     inherited;

     priv_filterlist := tDictionary<string, tFrameFilter>.create;
     priv_currentindex := 0;
end;

(* ------------------------------------------- *)

destructor tFilterSet.Destroy;

begin
     priv_filterlist.Free;

     inherited;
end;

(* ------------------------------------------- *)

procedure tFilterSet.Clear;

begin
     priv_filterlist.Clear;
     priv_currentindex := 0;
     setlength (priv_filterarray, 0);
end;

(* ------------------------------------------- *)

function tFilterSet.getFilterCount: integer;

begin
     result := priv_filterlist.Count;
end;

(* ------------------------------------------- *)

function tFilterSet.getFirst: tFrameFilter;

var HelpArray : tArray<string>;
    key       : string;
    index     : integer;

begin
     if priv_filterlist.Count > 0 then
        begin
             HelpArray := priv_filterlist.Keys.ToArray;
             TArray.Sort<string>(HelpArray);

             setLength (priv_filterarray, priv_filterlist.Count);
             index := 0;

             for key in HelpArray do
                 begin
                      inc (index);
                      priv_filterarray[index-1] := priv_filterlist.Items[key];
                 end;

             // set the index to the first entry

             priv_currentindex := 1;

             result := priv_filterarray[priv_currentindex-1];
        end
     else
         result.initialize;
end;

(* ------------------------------------------- *)

function tFilterSet.getNext: tFrameFilter;

begin
     assert (priv_currentindex > 0, 'GetNext failed with not prior GetFirst (tFilterSet.getNext)');

     inc (priv_currentindex);

     if priv_currentindex <= priv_filterlist.Count then
        result := priv_filterarray[priv_currentindex-1]
     else
         result.initialize;
end;

(* ------------------------------------------- *)

function tFilterSet.getFilterByKey (const DictionaryKey: string): tFrameFilter;

begin
     if priv_filterlist.ContainsKey (DictionaryKey) then
        result := priv_filterlist.Items[DictionaryKey]
     else
         result.initialize;
end;

(* ------------------------------------------- *)

function tFilterSet.AddFilter (Filter: tFrameFilter) : boolean;

var key : string;

begin
     key := Filter.DictionaryKey;

     if not priv_filterlist.ContainsKey (key) then
        begin
             // Allow last second modification of filters, e.g.
             // to set helper variables like first/last frame

             if assigned (EventOnAddFilter) then
                EventOnAddFilter (self, Filter);

             // Add to list

             priv_filterlist.Add (key, Filter);
             result := true
        end
     else
         result := false;
end;

(* ------------------------------------------- *)

function tFilterSet.AddFilterSet (const FilterSet: tFilterSet) : integer;

var Filter : tFrameFilter;
    key    : string;
    count  : integer;

begin
     count := 0;

     for key in FilterSet.priv_filterlist.Keys do
         begin
              Filter := FilterSet.priv_filterlist.Items[key];

              if AddFilter (Filter) then
                 inc (Count);
         end;

     result := count;
end;

(* ------------------------------------------- *)

procedure tFilterSet.RemoveFilter (const DictionaryKey: string);

begin
     assert (priv_filterlist.ContainsKey (DictionaryKey), 'Error: filter not found (tFilterSet.RemoveFilter)');

     priv_filterlist.Remove (DictionaryKey);
end;

(* ------------------------------------------- *)

procedure tFilterSet.Trim;

begin
     priv_filterlist.TrimExcess;
end;

(* ------------------------------------------- *)

end.
