{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit definition_http_structures;

interface

type tHTTPMethod = (HTTPMethodUnknown, HTTPMethodGet, HTTPMethodPost, HTTPMethodHead, HTTPMethodPut, HTTPMethodDelete, HTTPMethodConnect, HTTPMethodOptions, HTTPMethodTrace);
     tHTTPConnectionState = (HTTPConnectionClose, HTTPConnectionKeepAlive);

     tHTTPHeaderEntry   = record
                                Name           : ANSIString;
                                Value          : ANSIString;

                                procedure initialize;
                                procedure setfromString (NewHeader : string);
                                function asString : string;
                                function ValueAsInteger : integer;
                          end;

     tHTTPHeaderList = array of tHTTPHeaderEntry;

     function StrToHTTPMethod (Method : string) : tHTTPMethod;

const HTTPHeaderDelimiter = ': ';


implementation

uses SysUtils,
     Toolkit;

(* ------------------------------------------- *)

function StrToHTTPMethod (Method : string) : tHTTPMethod;

begin
     Method := trim (UpperCase (Method));

     result := HTTPMethodUnknown;

     if Method = 'GET' then
        result := HTTPMethodGet;

     if Method = 'POST' then
        result := HTTPMethodPost;

     if Method = 'HEAD' then
        result := HTTPMethodHead;

     if Method = 'PUT' then
        result := HTTPMethodPut;

     if Method = 'DELETE' then
        result := HTTPMethodDelete;

     if Method = 'CONNECT' then
        result := HTTPMethodConnect;

     if Method = 'OPTIONS' then
        result := HTTPMethodOptions;

     if Method = 'TRACE' then
        result := HTTPMethodTrace;
end;

(* ------------------------------------------- *)
{ tHTTPHeaderEntry }
(* ------------------------------------------- *)

procedure tHTTPHeaderEntry.initialize;

begin
     Name := '';
     Value := '';
end;

(* ------------------------------------------- *)

procedure tHTTPHeaderEntry.setfromString (NewHeader: string);

begin
     Name := trim (leftstring (NewHeader, HTTPHeaderDelimiter));
     Value := trim (rightstring (NewHeader, HTTPHeaderDelimiter));
end;

(* ------------------------------------------- *)

function tHTTPHeaderEntry.asString: string;

begin
     result := Name + HTTPHeaderDelimiter + Value;
end;

(* ------------------------------------------- *)

function tHTTPHeaderEntry.ValueAsInteger: integer;

var help  : integer;
    dummy : integer;

begin
     val (Value, help, dummy);

     result := help;
end;

(* ------------------------------------------- *)

end.
