{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit definition_simple_nameresolution;

interface

uses base_structures,
     definition_ip_addresses;

type
     // Simple Name Resolution stuff

     tSimpleIPv4NameResolutionEntry = record
                                            IPv4Address    : tIPv4Address;
                                            FQDN           : string;
                                      end;

     tSimpleIPv6NameResolutionEntry = record
                                            IPv6Address    : tIPv6Address;
                                            FQDN           : string;
                                      end;

     tSimpleIPv4NameResolutionList  = record
                                            IPv4NameList : array of tSimpleIPv4NameResolutionEntry;

                                            procedure AddEntry (NewEntry : tSimpleIPv4NameResolutionEntry);
                                            procedure AddIPandFQDN (IPAddress : tIPv4Address; FQDN : string);
                                            function GetEntry (index : integer) : tSimpleIPv4NameResolutionEntry;
                                            function count : integer;
                                            procedure initialize;
                                      end;

     tSimpleIPv6NameResolutionList  = record
                                            IPv6NameList : array of tSimpleIPv6NameResolutionEntry;

                                            procedure AddEntry (NewEntry : tSimpleIPv6NameResolutionEntry);
                                            procedure AddIPandFQDN (IPAddress : tIPv6Address; FQDN : string);
                                            function GetEntry (index : integer) : tSimpleIPv6NameResolutionEntry;
                                            function count : integer;
                                            procedure initialize;
                                      end;


implementation


(* ----------------------------------- *)
{ SimpleIPv4NameResolutionList }
(* ----------------------------------- *)

procedure tSimpleIPv4NameResolutionList.AddEntry (NewEntry: tSimpleIPv4NameResolutionEntry);

var count : integer;

begin
     count := length (IPv4NameList);
     setLength (IPv4NameList, count + 1);
     IPv4NameList[count] := NewEntry;
end;

(* ----------------------------------- *)

procedure tSimpleIPv4NameResolutionList.AddIPandFQDN (IPAddress: tIPv4Address; FQDN: string);

var Entry : tSimpleIPv4NameResolutionEntry;

begin
     Entry.IPv4Address := IPAddress;
     Entry.FQDN := FQDN;

     AddEntry (Entry);
end;

(* ----------------------------------- *)

function tSimpleIPv4NameResolutionList.count: integer;

begin
     result := length (IPv4NameList);
end;

(* ----------------------------------- *)

function tSimpleIPv4NameResolutionList.GetEntry (index: integer): tSimpleIPv4NameResolutionEntry;

begin
     result.IPv4Address.ValueAsLong := 0;
     result.FQDN := '';

     if (index > 0) and (index <= length (IPv4NameList)) then
        result := IPv4NameList[index - 1];
end;

(* ----------------------------------- *)

procedure tSimpleIPv4NameResolutionList.initialize;

begin
     setlength (IPv4NameList, 0);
end;

(* ----------------------------------- *)
{ SimpleIPv6NameResolutionList }
(* ----------------------------------- *)

procedure tSimpleIPv6NameResolutionList.AddEntry (NewEntry: tSimpleIPv6NameResolutionEntry);

var count : integer;

begin
     count := length (IPv6NameList);
     setLength (IPv6NameList, count + 1);
     IPv6NameList[count] := NewEntry;
end;

(* ----------------------------------- *)

procedure tSimpleIPv6NameResolutionList.AddIPandFQDN (IPAddress: tIPv6Address; FQDN: string);

var Entry : tSimpleIPv6NameResolutionEntry;

begin
     Entry.IPv6Address := IPAddress;
     Entry.FQDN := FQDN;

     AddEntry (Entry);
end;

(* ----------------------------------- *)

function tSimpleIPv6NameResolutionList.count: integer;

begin
     result := length (IPv6NameList);
end;

(* ----------------------------------- *)

function tSimpleIPv6NameResolutionList.GetEntry (index: integer): tSimpleIPv6NameResolutionEntry;

begin
     result.IPv6Address.ValueAsWordStruct := IPv6_NODE_UNSPECIFIED;
     result.FQDN := '';

     if (index > 0) and (index <= length (IPv6NameList)) then
        result := IPv6NameList[index - 1];
end;

(* ----------------------------------- *)

procedure tSimpleIPv6NameResolutionList.initialize;

begin
     setlength (IPv6NameList, 0);
end;

(* ----------------------------------- *)


end.
