{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit definition_tcptuple;

interface

uses base_structures,
     definition_ip_addresses,
     definition_tcp_structures,
     definition_5tuple;


type tTCPBaseTuple = record
                           Anchor        : tConversationAnchor;
                           Tuple         : t5Tuple;

                           procedure initialize; overload;
                           procedure initialize (ConAnchor : tConversationAnchor; Value : t5Tuple); overload;
                           procedure initialize (Value : t5Tuple); overload;

                           function DictionaryKey : string;
                     end;

     tTCPTuple  = record
                        Base          : tTCPBaseTuple;
                        ISN           : tTCPSequence;
                        ISNTimestamp  : tFrameTimestamp;

                        procedure initialize; overload;
                        procedure initialize (Value : t5Tuple); overload;
                        function isUndefined : boolean;

                        /// <summary>
                        ///   The dictionary key consists of the concatenated
                        ///   dictionary keys of the anchor and the tuple,
                        ///   plus the ISN. The index is not included because
                        ///   it is ephemeral. The socket pairs in the tuple
                        ///   are already normalized by calling the
                        ///   dictionary key function of the 5tuple, which is
                        ///   doing that.
                        /// </summary>
                        /// <returns>
                        ///   normalized dictionary key
                        /// </returns>
                        function DictionaryKey : string;
                  end;
     tTCPTupleList = array of tTCPTuple;


implementation

(* ------------------------------------------- *)
{ tTCPBaseTuple }
(* ------------------------------------------- *)

procedure tTCPBaseTuple.initialize;

begin
     Anchor.initialize;
     Tuple.initialize;
end;

(* ------------------------------------------- *)

procedure tTCPBaseTuple.initialize (Value: t5Tuple);

begin
     Anchor.initialize;
     Tuple := Value;
end;

(* ------------------------------------------- *)

procedure tTCPBaseTuple.initialize (ConAnchor : tConversationAnchor; Value : t5Tuple);

begin
     Anchor := ConAnchor;
     Tuple := Value;
end;

(* ------------------------------------------- *)

function tTCPBaseTuple.DictionaryKey: string;

begin
     result := Anchor.DictionaryKey + '-' + Tuple.DictionaryKey;
end;

(* ------------------------------------------- *)
{ tTCPTuple }
(* ------------------------------------------- *)

procedure tTCPTuple.initialize;

begin
     Base.initialize;
     ISN.initialize;
     ISNTimestamp.initialize;
end;

(* ------------------------------------------- *)

procedure tTCPTuple.initialize (Value: t5Tuple);

begin
     Base.initialize (Value);
     ISN.initialize;
     ISNTimestamp.initialize;
end;

(* ------------------------------------------- *)

function tTCPTuple.isUndefined: boolean;

begin
     result := (Base.Tuple.IPVersion = 0);
end;

(* ------------------------------------------- *)

function tTCPTuple.DictionaryKey: string;

begin
///  result := Anchor.DictionaryKey + '-' + Tuple.DictionaryKey + '-' + ISN.DictionaryKey;
//   result := Anchor.DictionaryKey + '-' + Tuple.DictionaryKey;
     result := Base.DictionaryKey;
end;

(* ------------------------------------------- *)

end.
