{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit toolkit;

interface

uses System.Classes,
     System.SysUtils,
     WinAPI.Windows,
     WinAPI.ShFolder;



const FileSignatureLength = 65535;

      IPv6UNCRegEx = '^\\\\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|' +
                     ':((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|' +
                     '(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|' +
                     '((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)' +
                     '(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*\\$';


type tSigBuffer = array [0..FileSignatureLength-1] of byte;

     tFileDetails = record
                          FileSize          : int64;
                          FileCreated       : tDateTime;
                          FileModified      : tDateTime;
                          FileAccessed      : tDateTime;
                          FileSignature     : string;
                          FileScanned       : tDatetime;

                          class operator equal (a, b : tFileDetails) : boolean;
                          class operator notequal (a, b : tFileDetails) : boolean;

                          procedure initialize;
                          procedure Determine (const Filename : string; const SkipFileSig : boolean = false);
                          function NewestDate : tDateTime;
                          function OldestDate : tDateTime;
                          function NewestWriteDate : tDateTime;
                          function OldestWriteDate : tDateTime;
                    end;

         /// <summary>
         ///   Compares two Unicode strings. Digits in the strings are
         ///   considered as numerical content rather than text. This test is
         ///   not case-sensitive.
         /// </summary>
         function StrCmpLogical (const s1, s2: string): Integer;
         /// <summary>
         ///   For strings in a TStringList instance you can use its CustomSort
         ///   method
         /// </summary>
         function StringListCompareLogical(List: TStringList; Index1, Index2: Integer): Integer;
         function rightstring (const sLine, sSubString : string; const IgnoreCase : boolean = true) : string; overload;
         function rightstring (const sLine : string; const iStartpos : integer) : string; overload;
         function leftstring (const sLine, sSubString : string; const MatchLastOccurence : boolean = false; const IgnoreCase : boolean = true) : string; overload;
         function leftstring (const sLine : string; const iEndpos : integer) : string; overload;
         {$REGION 'Documentation'}
         /// <summary>
         ///   Removes embracing quotation marks from strings
         /// </summary>
         /// <param name="SLine">
         ///   string to be stripped
         /// </param>
         /// <example>
         ///   "this is a test" -&gt; this is a test<br />"this is a test -&gt;
         ///   this is a test<br />this is a test" -&gt; this is a test<br />this
         ///   is "a" test -&gt; this is "a" test
         /// </example>
         {$ENDREGION}
         function stripquotes (const SLine : string) : string;
         {$REGION 'Documentation'}
         /// <summary>
         ///   Reduces a long string to one with dots in the middle
         /// </summary>
         /// <param name="InSpace">
         ///   The "InSpace" is an approximate number of charecters the
         ///   intended space can hold.
         /// </param>
         {$ENDREGION}
         function mince (StringToMince : string; InSpace : integer): String;

         // Substring Entries

         function NumEntries (const sLine, sSeparator : string) : word; overload;
         function NumEntries (const sLine, sSeparator : string; const SkipMultiple : boolean) : word; overload;
         function GetEntry (const sLine, sSeparator : string; const iNumber : integer) : string; overload;
         function GetEntry (const sLine, sSeparator : string; const iNumber : integer; const SkipMultiple : boolean) : string; overload;

         // XML

         function getTaggedValue (sLine, sTag : STRING) : STRING;
         function getTagParameter (sLine, sTag, sParameter : STRING) : STRING;
         function getTagBlock (const sLine, sTag : string) : string;

         // Sanitazion

         function SanitizeHTMLChars (sLine : string) : string;
         function SanitizeFilename (sLine : string) : string;
         function GetMinimalCommonPath (sPath1, sPath2 : string) : string;
         function GetRelativeFilename (BasePath, FileWithAbsolutePath : string; ReturnFullPathIfOutOfScope : boolean = false) : string;

         // OS and User related

         function GetFileSize (const FileName : string) : Int64;
         function FilesAreIdentical (const Filename1, Filename2 : string) : boolean;
         function IsDirectoryWriteable (const Filepath : string) : boolean;
         function DirectoryIsEmpty (const Directory: string; const includeSubdirectories : boolean = true): Boolean;
         /// <summary>
         ///   Deletes all files and subdirectories including the directory
         ///   specified in the parameter
         /// </summary>
         function DeleteDirectory (const Directory : string) : boolean;
         /// <summary>
         ///   Returns the absolute filename for the file specified by the
         ///   parameter, to allow specifying relative files and automatically
         ///   adding the current working directory if required
         /// </summary>
         function isDriveRoot (const DirectoryName : string) : boolean;
         function DetermineFilenameFromParam (const Parameter : string) : string;
         function GetExeVersion(Datei: string): string;
         function GetComputerNetName: string;
         Function GetUserFromWindows: string;
         function ProcessExists (exeFilename : string) : boolean;
         function GetPathOfGlobalAppData : string;
         function GetPathOfRoamingAppData : string;
         function GetPathOfTempFiles : string;

         // Integer to String with advanced Formats

         function intToCool(Number : Int64) : STRING;
         function CoolVal(Cool : String) : Real;
         function pointtext (value : int64) : string;

         // Workarounds for Uint64, which does not exists in XE2, and is broken with int64
         // Both routines taken from http://stackoverflow.com/questions/6077258/theres-a-uinttostr-in-delphi-to-let-you-display-uint64-values-but-where-is-strt

         function TryStrToUINT64(StrValue:String; var uValue:UInt64 ) : Boolean;
         function StrToUINT64 (Value:String) : UInt64;

         // IP Conversion

         function LongToIP (IP : Longword) : string;
         function IPToLong (sIPString : String) : Longword;

         // Encoding/Decoding

         function Encode64(S: string): string;
         function Decode64(S: string): string;

         // Byte swap routines

         function Swap16 (const value : SmallInt) : SmallInt; overload;
         function Swap16 (const value : word) : word; overload;
         function Swap32 (const value : integer) : integer; overload;
         function Swap32 (const value : dword) : dword; overload;
         function Swap64 (const value : int64) : int64; overload;
         function Swap64 (const value : uint64) : uint64; overload;

         // Number validation routines

         function ValueIsAValidNumber (sValue : string) : boolean; overload;
         function ValueIsAValidNumber (iValue : integer) : boolean; overload;
         function ValueIsAValidNumber (sValue : string; min, max : integer) : boolean; overload;
         function ValueIsAValidNumber (iValue : integer; min, max : integer) : boolean; overload;
         function ValueIsAValidFloat (sValue : string) : boolean;

         {$REGION 'Documentation'}
         /// <summary>
         ///   Checks if a string contains only hex digits (no $, x or space)
         ///   <br />
         /// </summary>
         /// <param name="sValue">
         ///   string containing hex numbers
         /// </param>
         /// <returns>
         ///   true if only hex digits are found
         /// </returns>
         {$ENDREGION}
         function StringIsValidHexNumber (const sValue : ANSIString) : boolean;
         {$REGION 'Documentation'}
         /// <summary>
         ///   checks if a string is a valid Snort formated hex string
         /// </summary>
         /// <param name="value">
         ///   string containing snort formated hex
         /// </param>
         /// <returns>
         ///   true if at least one block of "||" is found
         /// </returns>
         {$ENDREGION}
         function isValidSnortHexString (const value : ANSIString) : boolean;
         {$REGION 'Documentation'}
         /// <summary>
         ///   returns the length of a string containing Snort binary hex and/
         ///   or ascii
         /// </summary>
         /// <param name="value">
         ///   snort filter content string
         /// </param>
         /// <returns>
         ///   total length of all content strings, hex or asci
         /// </returns>
         {$ENDREGION}
         function getSnortHexLen (const value : ANSIString) : integer;
         {$REGION 'Documentation'}
         /// <summary>
         ///   turns a snort hex representation into a real string where each
         ///   character represents the byte of the byte in hex representation
         /// </summary>
         {$ENDREGION}
         function getSnortHexAsString (const value : ANSIString) : ANSIString;
         function getMD5ForString (const value : string) : string;

         // DateTime stuff

         function SecondsToTimeStr (Seconds : Integer): string;
         function DateTimeToFlatStr (Value : tDateTime) : string;
         function FlatStrToDateTime (Value : string) : tDateTime;

         // Memory stuff

         procedure FreeStringlistObjects (const strings: TStrings) ;
         function MemoryUsed : cardinal;


type TFileAttributes = set of (atrReadOnly, atrHidden, atrSysFile, atrVolumeId, atrDirectory, atrArchive, atrAnyFile);

const CRLF = chr(13) + chr(10);

implementation

uses System.StrUtils,
     System.DateUtils,
     System.Character,
     WinAPI.TlHelp32,
     RegularExpressions,
     IdHashMessageDigest,
     IdHash,
     IdGlobal;


const Codes64 = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz+/';
      Version = '3.1';

function StrCmpLogicalW(psz1, psz2: PWideChar): Integer; stdcall; external 'shlwapi.dll';

(* -------------------------------------- *)
{ tFileDetails }
(* -------------------------------------- *)

procedure tFileDetails.initialize;

begin
     FileSize := 0;
     FileCreated := 0;
     FileModified := 0;
     FileAccessed := 0;
     FileSignature := '';       // Used externally
     FileScanned := 0;
end;

(* -------------------------------------- *)

class operator tFileDetails.equal (a, b: tFileDetails): boolean;

begin
     result := (a.FileSize = b.FileSize) and
               (a.FileSignature = b.FileSignature);
end;

(* -------------------------------------- *)

class operator tFileDetails.notequal (a, b: tFileDetails): boolean;

begin
     result := not (a=b);
end;

(* -------------------------------------- *)

function tFileDetails.OldestWriteDate: tDateTime;

begin
     if (FileCreated < FileModified) then
        result := FileCreated
     else
         result := FileModified;
end;

(* -------------------------------------- *)

function tFileDetails.NewestWriteDate: tDateTime;

begin
     if (FileCreated > FileModified) then
        result := FileCreated
     else
         result := FileModified;
end;

(* -------------------------------------- *)

function tFileDetails.OldestDate: tDateTime;

var WriteDate : tDateTime;

begin
     WriteDate := OldestWriteDate;

     if WriteDate > FileModified then
        result := WriteDate
     else
         result := FileModified;
end;

(* -------------------------------------- *)

function tFileDetails.NewestDate: tDateTime;

var WriteDate : tDateTime;

begin
     WriteDate := OldestWriteDate;

     if WriteDate < FileModified then
        result := WriteDate
     else
         result := FileModified;
end;

(* ------------------------------------ *)

function getFileSignature (const Filename : string) : string;

var size    : int64;
    stream  : tFileStream;
    engine  : tIdHashMessageDigest5;
    sigpos  : int64;
    sigbuf  : tSigBuffer;
    buf     : TIdBytes;
    help    : int64;
    readlen : int64;
    value   : string;

begin
     // Get file size

     size := getFileSize (Filename);
     sigpos := (size div 2) - (FileSignatureLength div 2);

     // Get bytes from the middle of the file

     stream := tFileStream.Create (Filename, fmOpenRead or fmShareDenyNone);
     stream.Seek (sigpos, soFromBeginning);

     readlen := stream.Read (sigbuf, FileSignatureLength);
     stream.Free;

//   setlength (buf, FileSignatureLength);
//   move (sigbuf[low(sigbuf)], buf[0], FileSignatureLength);

     if readlen > 8 then
        begin
             setlength (buf, readlen);
             move (sigbuf[low(sigbuf)], buf[0], readlen);
        end
     else
         setlength (buf, 8);

     // Overwrite the first 8 bytes with the bytes of the byte swapped file size

     size := swap64 (size);

     help := (size and $ff00000000000000) shr 56;
     buf[0] := help;

     help := (size and $ff000000000000) shr 48;
     buf[1] := help;

     help := (size and $ff0000000000) shr 40;
     buf[2] := help;

     help := (size and $ff00000000) shr 32;
     buf[3] := help;

     help := (size and $ff000000) shr 24;
     buf[4] := help;

     help := (size and $ff0000) shr 16;
     buf[5] := help;

     help := (size and $ff00) shr 8;
     buf[6] := help;

     help := (size and $ff);
     buf[7] := help;

     engine := tIdHashMessageDigest5.Create;
     value := engine.HashBytesAsHex (buf);
     engine.Free;

     result := lowercase (value);
end;

(* -------------------------------------- *)

procedure tFileDetails.Determine (const Filename: string; const SkipFileSig : boolean = false);

var info       : TWin32FileAttributeData;
    low        : int64;
    high       : int64;
    SystemTime : TSystemTime;
    LocalTime  : TSystemTime;

begin
     if FileExists (Filename) then
        try
           if GetFileAttributesEx(PChar(FileName), GetFileExInfoStandard, @info) then
              begin
                   high := info.nFileSizeHigh;
                   low := info.nFileSizeLow;

                   FileSize := low or (high shl 32);

                   // Determine Create time stamp

                   FileTimeToSystemTime (info.ftCreationTime, SystemTime);
                   SystemTimeToTzSpecificLocalTime(nil, SystemTime, LocalTime);

                   FileCreated := SystemTimeToDateTime(LocalTime);

                   // Determine Modified time stamp

                   FileTimeToSystemTime (info.ftLastWriteTime, SystemTime);
                   SystemTimeToTzSpecificLocalTime(nil, SystemTime, LocalTime);

                   FileModified := SystemTimeToDateTime(LocalTime);

                   // Determine Accessed time stamp

                   FileTimeToSystemTime (info.ftLastAccessTime, SystemTime);
                   SystemTimeToTzSpecificLocalTime(nil, SystemTime, LocalTime);

                   FileAccessed := SystemTimeToDateTime(LocalTime);
              end;

           if not SkipFileSig then
              begin
                   FileSignature := getFileSignature (Filename);
                   FileScanned := now;
              end;

        except
              initialize;
        end
     else
         initialize;
end;

(* -------------------------------------- *)

function FilesAreIdentical (const Filename1, Filename2 : string) : boolean;

var details1 : tFileDetails;
    details2 : tFileDetails;

begin
     // If any of both files is missing, consider the copy failed

     if not (fileexists (Filename1) and fileexists (Filename2)) then
        result := false
     else
         begin
              // Get file details and compare

              try
                 details1.Determine (Filename1);
                 details2.Determine (Filename2);

                 if (details1.FileSignature <> '') or (details2.FileSignature <> '') then
                    begin
                         result := (details1.FileSize = details2.FileSize) and (details1.FileSignature = details2.FileSignature);

                         // If any of both files is empty, consider the copy failed

                         if (details1.FileSize = 0) or (details2.FileSize = 0) then
                            result := false;
                    end
                 else
                     result := false;

              except
                    result := false;
              end;
         end;
end;

(* -------------------------------------- *)

function IsDirectoryWriteable (const Filepath : string) : boolean;

var FileName : String;
    H        : THandle;

begin
     FileName := IncludeTrailingPathDelimiter (Filepath) + 'writetest.tmp';

     H := CreateFile (PChar (FileName), GENERIC_READ or GENERIC_WRITE, 0, nil, CREATE_NEW, FILE_ATTRIBUTE_TEMPORARY or FILE_FLAG_DELETE_ON_CLOSE, 0);

     Result := H <> INVALID_HANDLE_VALUE;

     if Result then
        CloseHandle(H);
end;

(* -------------------------------------- *)

function DirectoryIsEmpty (const Directory: string; const includeSubdirectories : boolean = true): Boolean;

var path : string;
    help : string;
    SR   : TSearchRec;
    i    : Integer;

begin
     path := IncludeTrailingPathDelimiter (Directory);

     if not includeSubdirectories then
        begin
             Result := False;

             try
                FindFirst (path + '*', faAnyFile, SR);

                for i := 1 to 2 do
                    if (SR.Name = '.') or (SR.Name = '..') then
                       Result := FindNext (SR) <> 0;
             finally
                    System.SysUtils.FindClose (SR);
             end;
        end
     else
         begin
              result := true;

              if FindFirst (path + '*', faAnyFile, SR) = 0 then
                 try
                    repeat
                          if (SR.Name <> '.') and (SR.Name <> '..') then
                             begin
                                  // Check if all we find are directories, otherwise fail

                                  if ((SR.attr and faDirectory) = faDirectory) then
                                     begin
                                          help := IncludeTrailingPathDelimiter (path + SR.Name);

                                          if not DirectoryIsEmpty (help, true) then
                                             result := false;
                                     end
                                  else
                                      result := false;
                             end;

                    until (FindNext(SR) <> 0) or (result = false);
                 finally
                        System.SysUtils.FindClose(SR);
                 end;
         end;
end;

(* -------------------------------------- *)

function DeleteDirectory (const Directory : string) : boolean;

var path : string;
    help : string;
    SR   : TSearchRec;
    i    : Integer;

begin
     result := true;

     path := IncludeTrailingPathDelimiter (Directory);

     if FindFirst (path + '*', faAnyFile, SR) = 0 then
        try
           repeat
                 if (SR.Name <> '.') and (SR.Name <> '..') then
                    begin
                         if ((SR.attr and faDirectory) = faDirectory) then
                            begin
                                 help := IncludeTrailingPathDelimiter (path + SR.Name);

                                 // Delete the subdirectory recursively, then the directory itself

                                 if not DeleteDirectory (help) then
                                    result := false
                                 else
                                     if not RemoveDir (help) then
                                        result := false;
                            end
                         else
                             result := DeleteFile (PChar (path + SR.Name));
                    end;

           until (FindNext(SR) <> 0) or (result = false);
        finally
               System.SysUtils.FindClose(SR);
        end;

     // If the recursive stuff was okay, delete the directory itself

     if result then
        result := RemoveDir (Directory);
end;

(* -------------------------------------- *)

function isDriveRoot (const DirectoryName : string) : boolean;

var RegEx : tRegEx;
    count : integer;

begin
     RegEx := tRegEx.Create ('\\{2}(((\d{1,3}\.){3}\d{1,3})|\w+)\\');   // \\192.168.0.1\ or \\sentinel2012\

     result := ((pos (':\', DirectoryName) = 2) and (length (DirectoryName) = 3));

     if not result then
        begin
             count := numEntries (DirectoryName, '\');

             result := RegEx.IsMatch (DirectoryName) and (count < 5);

             if not result then
                begin
                     RegEx.Create (IPv6UNCRegEx);

                     result := RegEx.IsMatch (DirectoryName);
                end;

        end;

end;

(* -------------------------------------- *)

function DetermineFilenameFromParam (const Parameter : string) : string;

var path : string;

begin
     result := '';

     if pos (':\', Parameter) = 0 then
        begin
             path := IncludeTrailingPathdelimiter (GetCurrentDir);

             if fileexists (path + Parameter) then
                result := path + Parameter;
        end
     else
         if fileexists (Parameter) then
            result := Parameter;
end;

(* -------------------------------------- *)

function StrCmpLogical (const s1, s2: string): Integer;

begin
     Result := StrCmpLogicalW(PChar(s1), PChar(s2));
end;

(* -------------------------------------- *)

function StringListCompareLogical (List: TStringList; Index1, Index2: Integer) : Integer;

begin
     Result := StrCmpLogical(List[Index1], List[Index2]);
end;

(* -------------------------------------- *)

// Extracts the right part of a string starting right after the given substring

function rightstring (const sLine, sSubString : string; const IgnoreCase : boolean = true) : string; overload;

var sHelp : string;
    iLen  : integer;
    iPos  : integer;

begin
     sHelp := '';
     iLen := length (sLine);

     if IgnoreCase then
        iPos := pos (lowercase (sSubString), lowercase (sLine))
     else
         iPos := pos (sSubString, sLine);

     if (iPos > 0) and (iPos <= length (sLine)) then
        sHelp := copy (sLine, iPos + length (sSubstring), (iLen - iPos + 1));

     result := sHelp;
end;

(* -------------------------------------- *)

// Extracts the right part of a string from a given position

function rightstring (const sLine : string; const iStartpos : integer) : string; overload;

var sHelp : string;
    iLen  : integer;

begin
     if iStartpos <= length (sLine) then
        begin
             iLen := length (sLine);
             shelp := copy (sLine, iStartpos, (iLen - iStartpos + 1));
             result := sHelp;
        end
     else
         result := '';
end;

(* -------------------------------------- *)

// Extracts the left part of a string starting right before the given substring

function leftstring (const sLine, sSubString : string; const MatchLastOccurence : boolean = false; const IgnoreCase : boolean = true) : string; overload;

var sHelp : string;
    wPos  : integer;
    lPos  : integer;

begin
     sHelp := '';

     if IgnoreCase then
        wPos := pos (lowercase (sSubString), lowercase (sLine))
     else
         wPos := pos (sSubString, sLine);

     if wPos > 0 then
        begin
             if not MatchLastOccurence then
                sHelp := copy (sLine, 1, wPos - 1)
             else
                 begin
                      repeat
                            lPos := wPos;

                            if IgnoreCase then
                               wPos := posEx (lowercase (sSubString), lowercase (sLine), lPos + 1)
                            else
                                wPos := posEx (sSubString, sLine, lPos + 1);

                      until wPos = 0;

                      sHelp := copy (sLine, 1, lPos - 1)
                 end;
        end
     else
         sHelp := sLine;

     result := sHelp;
end;

(* -------------------------------------- *)

// Extracts the left part of a string to a given position

function leftstring (const sLine : string; const iEndpos : integer) : string; overload;

var sHelp : string;

begin
     sHelp := '';

     if iEndPos >= 1 then
        shelp := copy (sLine, 1, iEndpos);

     result := sHelp;
end;

(* -------------------------------------- *)

function stripquotes (const sLine : string) : string;

var len : integer;

begin
     len := length (sLine);
     result := sLine;

     if pos ('"', sLine) = 1 then
        begin
             if sLine[len] = '"' then
                begin
                     result := copy (sLine, 2, len - 2);
                end
             else
                 result := copy (sLine, 2, len - 1);
        end
     else
         if (len > 0) and (sLine[len] = '"') then
            result := copy (sLine, 1, len - 1);
end;

(* -------------------------------------- *)

function mince (StringToMince : string; InSpace :integer): string;

var TotalLength, FLength : Integer;

begin
     TotalLength := Length(StringToMince) ;

     if TotalLength > InSpace then
        begin
             FLength := (Inspace Div 2) - 2;

             Result := Copy(StringToMince, 0, fLength)
                       + '...'
                       + Copy(StringToMince, TotalLength-fLength, TotalLength) ;
        end
     else
         Result := StringToMince;
end;

(* -------------------------------------- *)

// Tells how many entries exists in a line.
// Example: numentries ('1,2,3,4', ',') results in 4, because there are 4 entries seperated by ","

function NumEntries (const sLine, sSeparator : string) : word; overload;

var count : longword;
    sHelp : string;

begin
     count := 0;
     sHelp := sLine;

     if (trim (sLine) <> '') and (sLine <> sSeparator) then
        begin
             count := 1;

             while pos (sSeparator, sHelp) > 0 do
                   begin
                        inc (count);

                        sHelp := rightstring (sHelp, pos(sSeparator, sHelp) + length (sSeparator));
                   end;
        end;

     result := count;
end;

(* -------------------------------------- *)

// Tells how many entries exists in a line. SkipMultiple will treat multiple separators following
// each other as one if set to true.
// Example: numentries ('1,2,3,,,4', ',', true) results in 4, because there are 4 entries seperated by "," and
// the three "," between 3 and 4 are reduced to one.

function NumEntries (const sLine, sSeparator : string; const SkipMultiple : boolean) : word; overload;

var count : longword;
    sHelp : string;

begin
     count := 0;
     sHelp := sLine;

     if SkipMultiple then
        while pos (sSeparator + sSeparator, sHelp) > 0 do
              begin
                   sHelp := stringreplace (sHelp, sSeparator + sSeparator, sSeparator, [rfReplaceAll])
              end;

     if (trim (sLine) <> '') and (sLine <> sSeparator) then
        begin
             count := 1;

             while pos (sSeparator, sHelp) > 0 do
                   begin
                        inc (count);

                        sHelp := rightstring (sHelp, pos(sSeparator, sHelp) + length (sSeparator));
                   end;
        end;

     result := count;
end;

(* -------------------------------------- *)

// returns a specified entry from sLine. Use numentries to get the entry count first.

function GetEntry (const sLine, sSeparator : string; const iNumber : integer) : string; overload;

var sReststring : string;
    sResult     : string;
    iPos        : longint;
    i           : longint;

begin
     sResult := '';
     sReststring := sLine;

     i := 0;

     while (i < iNumber) do
           begin
                inc (i);

                iPos := pos (sSeparator, sRestString);

                if iPos > 0 then
                   begin
                        sResult := copy (sReststring, 1, iPos - 1);
                        sReststring := rightstring (sReststring, iPos + length (sSeparator));
                   end
                else
                    begin
                         sResult := sReststring;
                         sRestString := '';
                    end;
           end;

     trim (sResult);
     result := sResult;
end;

(* -------------------------------------- *)

// returns a specified entry from sLine. Use numentries to get the entry count first.

function GetEntry (const sLine, sSeparator : string; const iNumber : integer; const SkipMultiple : boolean) : string; overload;

var sReststring : string;
    sResult     : string;
    iPos        : longint;
    i           : longint;

BEGIN
     sResult := '';
     sReststring := sLine;

     if SkipMultiple then
        while pos (sSeparator + sSeparator, sRestString) > 0 do
              begin
                   sRestString := stringreplace (sRestString, sSeparator + sSeparator, sSeparator, [rfReplaceAll])
              end;

     i := 0;

     while (i < iNumber) DO
           begin
                inc (i);

                iPos := pos (sSeparator, sRestString);

                if iPos > 0 then
                   begin
                        sResult := copy (sReststring, 1, iPos - 1);
                        sReststring := rightstring (sReststring, iPos + length (sSeparator));
                   end
                else
                    begin
                         sResult := sReststring;
                         sRestString := '';
                    end;
           end;

     trim (sResult);
     result := sResult;
END;

(* -------------------------------------- *)

// Returns the value between two tags (opening and closing)
// example: getTaggedValue ('<test>abc</test>', 'test') -> 'abc'
//          getTaggedValue ('<test name="abc">def</test>', test) -> 'def'

function getTaggedValue (sLine, sTag : STRING) : STRING;

var sHelp     : string;
    iCount    : integer;
    sStartTag : string;
    sEndTag   : string;

begin
     // If the tag contains additional parameters, strip them to get the end tag

     if pos ('=', sTag) > 0 then
        begin
             sEndTag := getentry (sTag, '=', 1);
             sEndTag := getentry (sEndTag, ' ', 1);
        end;

     // Check if the tag is specified without additional parameters but the one in the given line has some

     if pos ('<' + sTag + '>', sLine) = 0 then
        begin
             // We did not find the tag at all, it probably has params
             // So we just grab the right one from the result line

             sStartTag := getEntry (sLine, '>', 1);
             sStartTag := rightstring (sStartTag, '<');

             sEndTag := getentry (sTag, '=', 1);
             sEndTag := getentry (sEndTag, ' ', 1);
        end
     else
         begin
              sStartTag := sTag;
              sEndTag := sTag;
         end;

     sHelp := rightstring (sLine, '<' + sStartTag + '>');
     result := leftstring (sHelp, '</' + sEndTag + '>');
end;

(* -------------------------------------- *)

// Returns a parameter value.
// Example: getTagParameter ('<test value="abc">', 'test', 'value') -> 'abc'
//          getTagParameter ('<name="test">, '', 'name') -> 'test'

function getTagParameter (sLine, sTag, sParameter : STRING) : STRING;

var sHelp : STRING;

begin
     sHelp := stringReplace (sLine, '''', '"', [rfReplaceAll]);       // Replace single quotation marks with doubles

     if sTag <> '' then
        sHelp := rightstring (sLine, '<' + sTag);                     // Removes the tag header when we do not need it

     sHelp := rightstring (sHelp, sParameter + '=');
     sHelp := leftstring (sHelp, ' ');

     // If values are in quotation marks, strip them, otherwise simply return the value

     if pos ('"', sHelp) > 0 then
        result := getentry (sHelp, '"', 2)
     else
         result := trim (leftstring (sHelp, '>'));
end;

(* -------------------------------------- *)

// Returns the text between the opening and closing tag
// Example: getTagBlock ('<br><br><tr id="1"><td><a href="...">Test</a></td></tr><br>','tr') -> '<tr id="1"><td><a href="...">Test</a></td></tr>'

function getTagBlock (const sLine, sTag : string) : string;

var iStartPos : integer;
    iEndPos   : integer;
    sSub      : string;

begin
     iStartPos := pos ('<' + lowercase (sTag) + ' ', lowercase (sLine));

     if iStartPos = 0 then
        iStartPos := pos ('<' + lowercase (sTag) + '>', lowercase (sLine));

     // Remove the parts before the match, because the end tag must be found behind the start tag

     sSub := copy (sLine, iStartPos, length (sLine) - iStartPos);

     iEndPos := pos ('</' + lowercase (sTag) + '>', lowercase (sSub));

     if (iStartPos > 0) then
        begin
             if (iEndPos > 0) then
                result := copy (sSub, iStartPos, iEndPos - iStartPos)
             else
                 result := sSub;
        end
     else
         result := '';
end;

(* -------------------------------------- *)

// returns file size in bytes or -1 if not found.

function GetFileSize (const FileName : string) : Int64;

var info  : TWin32FileAttributeData;
    low   : int64;
    high  : int64;
    value : int64;

begin
     result := -1;

     if not GetFileAttributesEx(PWideChar(FileName), GetFileExInfoStandard, @info) then
        exit;

     high := info.nFileSizeHigh;
     low := info.nFileSizeLow;

     value := low or (high shl 32);

     result := value;
end;

(* -------------------------------------- *)

function GetExeVersion(Datei: string): string;

   var aFileName: array [0..MAX_PATH] of Char;
       pdwHandle: DWORD;
       nInfoSize: DWORD;
       pFileInfo: Pointer;
       pFixFInfo: PVSFixedFileInfo;
       nFixFInfo: DWORD;

begin
     //Gibt Versionsnummer zurck

     StrPCopy(aFileName,Datei);
     pdwHandle := 0;
     nInfoSize := GetFileVersionInfoSize(aFileName, pdwHandle);
     result:='0';

     if nInfoSize <> 0 then
        pFileInfo := GetMemory(nInfoSize)
     else
         pFileInfo := nil;

     if Assigned(pFileInfo) then
        try
           if GetFileVersionInfo(aFileName, pdwHandle, nInfoSize, pFileInfo) then
              begin
                   pFixFInfo := nil;
                   nFixFInfo := 0;

                   if VerQueryValue(pFileInfo, '\', Pointer(pFixFInfo), nFixFInfo) then
                      begin
                           result := Format('%d.%d.%d build %d',[HiWord(pFixFInfo^.dwFileVersionMS),
                           LoWord(pFixFInfo^.dwFileVersionMS),HiWord(pFixFInfo^.dwFileVersionLS),
                           LoWord(pFixFInfo^.dwFileVersionLS)]);
                      end;
              end;
        finally
               FreeMemory(pFileInfo);
        end;
end;

(* -------------------------------------- *)

// Replaces international HTML character representations

function SanitizeHTMLChars (sLine : string) : string;

var sHelp : string;

begin
     sHelp := sLine;

     sHelp := StringReplace (sHelp, '’', '''', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, '…', '...', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, '“', '"', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, '”', '"', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, '‘', '''', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, '–', '-', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, '©', '(c)', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, 'ñ', '', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, 'é', '', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, 'ó', '', [rfReplaceAll]);

     result := sHelp;
end;

(* -------------------------------------- *)

// Replaces characters that may not be used in filenames

function SanitizeFilename (sLine : string) : string;

var sHelp : string;

begin
     sHelp := SanitizeHTMLChars (sLine);

     sHelp := StringReplace (sHelp, '/', '-', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, '\', '-', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, '*', '+', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, '?', '', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, '"', ' ', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, ':', '-', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, ':', '-', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, ';', '-', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, '[', '(', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, ']', ')', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, '<', '(', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, '>', ')', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, '|', '-', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, '=', '-', [rfReplaceAll]);
     sHelp := StringReplace (sHelp, CRLF, ' ', [rfReplaceAll]);

     result := sHelp;
end;

(* -------------------------------------- *)

// Compares two paths and returns the lowest common path. Make sure both paths are terminated by "\"!

function GetMinimalCommonPath (sPath1, sPath2 : string) : string;

var sDrive1    : string;
    sDrive2    : string;
    sSubPath1  : string;
    sSubPath2  : string;
    i          : integer;

begin
     sPath1 := lowercase (sPath1);
     sPath2 := lowercase (sPath2);

     // Check if both are the same

     if sPath1 = sPath2 then
        result := sPath1
     else
         begin
              // No, they're different, so lets see what they have in common

              result := '';

              sDrive1 := extractFileDrive (sPath1);
              sDrive2 := extractFileDrive (sPath2);

              if sDrive1 = sDrive2 then
                 begin
                      // The drive is the same

                      result := sDrive1 + '\';

                      sSubPath1 := rightstring (sPath1, sDrive1);
                      sSubPath2 := rightstring (sPath2, sDrive1);

                      // Add all subdirectories that are equal

                      if numEntries (sSubPath1, '\') <= numEntries (sSubPath2, '\') then
                         begin
                              for i := 2 to numEntries (sSubPath1, '\') do
                                  begin
                                       if getEntry (sSubPath1, '\', i) = getEntry (sSubPath2, '\', i) then
                                          result := result + getEntry (sSubPath1, '\', i) + '\';
                                  end;
                         end
                      else
                         begin
                              for i := 2 to numEntries (sSubPath2, '\') do
                                  begin
                                       if getEntry (sSubPath1, '\', i) = getEntry (sSubPath2, '\', i) then
                                          result := result + getEntry (sSubPath1, '\', i) + '\';
                                  end;
                         end;
                 end;
         end;
end;

(* -------------------------------------- *)

// Returns the relative path of a file

function GetRelativeFilename (BasePath, FileWithAbsolutePath : string; ReturnFullPathIfOutOfScope : boolean = false) : string;

var filename     : string;
    filepath     : string;
    commonpath   : string;
    relative1    : string;
    relative2    : string;
    i            : integer;

begin
     filename := extractFilename (FileWithAbsolutePath);
     filepath := lowercase (extractFilePath (FileWithAbsolutePath));

     basepath := IncludeTrailingBackslash (lowercase (basepath));

     // Check if the base path is equal to the beginning of the file path
     // if so, keep everything beyond the base path and add the filename

     if pos (BasePath, FilePath) = 1 then
        result := rightstring (FilePath, BasePath) + Filename
     else
         begin
              // Base path is not completely part of the file path

              if ReturnFullPathIfOutOfScope then
                 result := FileWithAbsolutePath
              else
                  begin
                       // find out if there is a common base path at all

                       commonpath := GetMinimalCommonPath (BasePath, FilePath);

                       if commonpath = '' then
                          result := FileWithAbsolutePath
                       else
                           begin
                                // First, lets get what both paths have on top of the common path

                                relative1 := rightstring (Basepath, Commonpath);
                                relative2 := rightstring (Filepath, Commonpath);

                                // Need to determine the amount of "..\" we need to add to the relative2 path.
                                // For that we count the number parts separated by "\"

                                for i := 1 to (numEntries (relative1, '\') - 1) do
                                    relative2 := '..\' + relative2;

                                result := relative2 + filename;
                           end;
                  end;
         end;
end;

(* -------------------------------------- *)

function GetComputerNetName: string;

var buffer : array[0..255] of char;
    size   : dword;
begin
     size := 256;

     if GetComputerName(buffer, size) then
        Result := buffer
     else
         Result := ''
end;

(* -------------------------------------- *)

Function GetUserFromWindows: string;

Var UserName : string;
    UserNameLen : Dword;

Begin
     UserNameLen := 255;
     SetLength(userName, UserNameLen) ;

     If GetUserName(PChar(UserName), UserNameLen) Then
        Result := Copy(UserName,1,UserNameLen - 1)
     Else
         Result := 'Unknown';
End;

(* -------------------------------------- *)

function processExists(exeFileName: string): Boolean;

var ContinueLoop: BOOL;
    FSnapshotHandle: THandle;
    FProcessEntry32: TProcessEntry32;

begin
     FSnapshotHandle := CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
     FProcessEntry32.dwSize := SizeOf(FProcessEntry32);
     ContinueLoop := Process32First(FSnapshotHandle, FProcessEntry32);
     Result := False;

     while Integer(ContinueLoop) <> 0 do
           begin
                if ((UpperCase(ExtractFileName(FProcessEntry32.szExeFile)) =
                  UpperCase(ExeFileName)) or (UpperCase(FProcessEntry32.szExeFile) =
                  UpperCase(ExeFileName))) then
                  begin
                       Result := True;
                  end;

                  ContinueLoop := Process32Next(FSnapshotHandle, FProcessEntry32);
           end;

     CloseHandle(FSnapshotHandle);
end;

(* -------------------------------------- *)

function GetPathOfGlobalAppData : string;

const SHGFP_TYPE_CURRENT = 0;

var path: array [0..MAX_PATH] of char;

begin
     if SUCCEEDED (SHGetFolderPath(0, CSIDL_COMMON_APPDATA, 0, SHGFP_TYPE_CURRENT, @path[0])) then
        result := IncludeTrailingBackslash (path)
     else
         result := '';
end;

(* -------------------------------------- *)

function GetPathOfRoamingAppData : string;

const SHGFP_TYPE_CURRENT = 0;

var path: array [0..MAX_PATH] of char;

begin
     if SUCCEEDED (SHGetFolderPath(0, CSIDL_APPDATA, 0, SHGFP_TYPE_CURRENT, @path[0])) then
        result := IncludeTrailingBackslash (path)
     else
         result := '';
end;

(* -------------------------------------- *)

function GetPathOfTempFiles : string;

var path : string;
    lng  : longword;

begin
     SetLength(Path, MAX_PATH) ;
     lng := GetTempPath(MAX_PATH, PChar(Path)) ;
     SetLength (Path, lng);

     result := path;
end;

(* -------------------------------------- *)

// Formats storage related sizes into a better readable form for humans

function intToCool (Number : Int64) : STRING;

begin
     result := inttoStr(Number);

     // Check up kBytes

     if Number > 1024 then
        result := format ('%1.2n k', [Number / 1024]);

     // Check up MBytes

     if Number > 1048576 then
        result := format ('%1.2n M', [Number / 1048576]);

     // Check up BBytes

     if Number > 1073741824 then
        result := format ('%1.2n G', [Number / 1073741824]);
end;

(* -------------------------------------- *)

// Formats storage related sizes into a better readable form for humans

function CoolVal (Cool : String) : Real;

var factor : Integer;
    Code   : Integer;

begin
     factor := 1;

     if lowercase (Cool[Length(Cool)]) = 'k' then
        begin
             factor := 1024;
             Cool := LeftString(Cool, Length(Cool)-1);
        end;

     if uppercase (Cool[Length(Cool)]) = 'M' then
        begin
             factor := 1048576;
             Cool := LeftString(Cool, Length(Cool)-1);
        end;

     if uppercase (Cool[Length(Cool)]) = 'G' then
        begin
             factor := 1073741824;
             Cool := LeftString(Cool, Length(Cool)-1);
        end;

     val(Cool, result, Code);
     result := result * factor;
end;

(* -------------------------------------- *)

// Converts Longint to string with dots to make large numbers more human readable

function PointText (value : int64) : string;

var s : string;
    i : integer;
    c : integer;
    l : integer;
    sep : char;

begin
     s := IntToStr (value);
     result := '';
     c := 0;
     l := length (IntToStr (abs (value)));

     // We run through the string from the back and add a dot every 3 digits, but only
     // there are more digits to come. Through making sure that our counter is lower than
     // the total length we avoid stuff like ".100.000" or "-.500.000"

     for i := length (s) downto 1 do
         begin
              result := s[i] + result;
              inc (c);

              if ((c mod 3) = 0) and (c < l) then
                 result := FormatSettings.ThousandSeparator + result;
         end;
end;

(* -------------------------------------- *)

function TryStrToUINT64 (StrValue:String; var uValue : System.UInt64 ):Boolean;

var Start,Base,Digit : Uint64;
    n                : integer;
    Nextvalue        : UInt64;

begin
     result := false;

     Base := 10;
     Start := 1;
     StrValue := Trim(UpperCase(StrValue));

     if StrValue='' then
        exit;

     if StrValue[1]='-' then
        exit;

     if StrValue[1]='$' then
        begin
             Base := 16;
             Start := 2;

             if Length(StrValue) > 17 then // $+16 hex digits = max hex length.
                exit;
        end;

     uValue := 0;

     for n := Start to Length(StrValue) do
         begin
              if System.Character.IsDigit(StrValue[n]) then
                 Digit := Ord(StrValue[n])-Ord('0')
              else
                  if  (Base=16) and (StrValue[n] >= 'A') and (StrValue[n] <= 'F') then
                      Digit := (Ord(StrValue[n])-Ord('A'))+10
                  else
                      exit;// invalid digit.

              Nextvalue := (uValue*base)+digit;

              if (Nextvalue<uValue) then
                 exit;

              uValue := Nextvalue;
         end;

     result := true; // success.
end;

(* -------------------------------------- *)

function StrToUINT64(Value:String):UInt64;

begin
     if not TryStrToUINT64(Value,result) then
        raise EConvertError.Create('Invalid uint64 value');
end;

(* -------------------------------------- *)

// converts an IP address in longword format to a human readable string value

function LongToIP(IP : Longword) : string;

var
   temp : Longword;
   oneval : Byte;
   i : Integer;

begin
     result := '';
     temp := IP;

     for i := 0 to 2 do
     begin
          oneval := temp mod 256;
          result := '.'+inttostr(oneval)+result;
          temp := temp div 256;
     end;

     result := inttostr(temp)+result;

end;

(* -------------------------------------- *)

// turns an IP string into its longword representation

function IPToLong(sIPString : String) : Longword;

var subbyte : byte;
    temp    : int64;

begin
     result := 0;

     if (sIPString <> '') then
        begin
             subbyte := StrToInt (getentry (sIPString, '.', 1));
             temp := subByte;
             temp := temp * 256 * 256 * 256;

             subbyte := StrToInt (getentry (sIPString, '.', 2));
             temp := temp + subByte * 256 * 256;

             subbyte := StrToInt (getentry (sIPString, '.', 3));
             temp := temp + subByte * 256;

             subbyte := StrToInt (getentry (sIPString, '.', 4));
             temp := temp + subByte;

             result := temp;
        end;
end;

(* -------------------------------------- *)

function Encode64(S: string): string;

var i : Integer;
    a : Integer;
    x : Integer;
    b : Integer;

begin
     Result := '';
     a := 0;
     b := 0;

     for i := 1 to Length(s) do
         begin
              x := Ord(s[i]);
              b := b * 256 + x;
              a := a + 8;

              while a >= 6 do
                    begin
                      a := a - 6;
                      x := b div (1 shl a);
                      b := b mod (1 shl a);
                      Result := Result + Codes64[x + 1];
                    end;
         end;

     if a > 0 then
        begin
             x := b shl (6 - a);
             Result := Result + Codes64[x + 1];
        end;
end;

(* -------------------------------------- *)

function Decode64(S: string): string;

var i : Integer;
    a : Integer;
    x : Integer;
    b : Integer;

begin
     Result := '';
     a := 0;
     b := 0;

     for i := 1 to Length(s) do
         begin
              x := Pos(s[i], codes64) - 1;

              if x >= 0 then
                 begin
                      b := b * 64 + x;
                      a := a + 6;

                      if a >= 8 then
                         begin
                           a := a - 8;
                           x := b shr a;
                           b := b mod (1 shl a);
                           x := x mod 256;

                           Result := Result + chr(x);
                         end;
                 end
                    else
                        Exit;
         end;
end;

(* -------------------------------------- *)

// Signed 16Bit Byte Swap for big/little endian conversions

function Swap16 (const value : SmallInt) : SmallInt; overload;

asm
   mov ax,value
    xchg al,ah
end;

(* -------------------------------------- *)

// Unsigned 16Bit Byte Swap for big/little endian conversions

function Swap16 (const value : word) : word; overload;

asm
   mov ax,value
   xchg al,ah
end;

(* -------------------------------------- *)

// 32Bit Byte Swap for big/little endian conversions

function Swap32 (const value : integer) : integer; overload;

asm
   mov eax, value
   bswap eax
end;

(* -------------------------------------- *)

// 32Bit Byte Swap for big/little endian conversions

function Swap32 (const value : dword) : dword; overload;

asm
   mov eax, value
   bswap eax
end;

(* -------------------------------------- *)

// 64Bit Byte Swap for big/little endian conversions

function Swap64 (const value : int64) : int64; overload;

var help1 : int64;
    help2 : longword;
    help3 : int64;

begin
     help1 := (value and $ffffffff);
     help2 := help1;
     help2 := swap32 (help2);
     help3 := help2;
     help3 := help3 shl 32;

     help1 := (value and $ffffffff00000000) shr 32;
     help2 := help1;
     help2 := swap32 (help2);
     help3 := help3 or help2;

     result := help3;
end;

(* -------------------------------------- *)

function Swap64 (const value : uint64) : uint64; overload;

var help1 : uint64;
    help2 : longword;
    help3 : uint64;

begin
     help1 := (value and $ffffffff);
     help2 := help1;
     help2 := swap32 (help2);
     help3 := help2;
     help3 := help3 shl 32;

     help1 := (value and $ffffffff00000000) shr 32;
     help2 := help1;
     help2 := swap32 (help2);
     help3 := help3 or help2;

     result := help3;
end;

(* -------------------------------------- *)
(* Number validation routines             *)
(* -------------------------------------- *)

function ValueIsAValidNumber (sValue : string) : boolean; overload;

var  iError: Integer;
     iDummy: Integer;

begin
     result := false;

     val(Trim(sValue), iDummy, iError);

     if iError = 0 then
        result := true;
end;

(* -------------------------------------- *)

function ValueIsAValidNumber (iValue : integer) : boolean; overload;

var  iError : Integer;
     iDummy : Integer;
     sHelp  : string;

begin
     result := false;
     sHelp := IntToStr (iValue);

     val(Trim(sHelp), iDummy, iError);

     if iError = 0 then
        result := true;
end;

(* -------------------------------------- *)

function ValueIsAValidNumber (sValue : string; min, max : integer) : boolean; overload;

var  iError: Integer;
     iDummy: Integer;

begin
     result := false;

     val(Trim(sValue), iDummy, iError);

     if iError = 0 then
        if (iDummy <= max) and (iDummy >= min) then
           result := true;
end;

(* -------------------------------------- *)

function ValueIsAValidNumber (iValue : integer; min, max : integer) : boolean; overload;

var  iError : Integer;
     iDummy : Integer;
     sHelp  : string;

begin
     result := false;
     sHelp := IntToStr (iValue);

     val(Trim(sHelp), iDummy, iError);

     if iError = 0 then
        if (iValue <= max) and (iValue >= min) then
           result := true;
end;

(* -------------------------------------- *)

function ValueIsAValidFloat (sValue : string) : boolean;

begin
     result := false;

     if numEntries (sValue, FormatSettings.decimalseparator) = 2 then
        if ValueIsAValidNumber (getEntry (sValue, FormatSettings.decimalseparator, 1)) and
           ValueIsAValidNumber (getEntry (sValue, FormatSettings.decimalseparator, 2)) then
           result := true;
end;

(* -------------------------------------- *)

function StringIsValidHexNumber (const sValue : ANSIString) : boolean;

var valid : boolean;
    len   : integer;
    i     : integer;

begin
     valid := true;

     i := 1;
     len := length (sValue);

     while (i <= len) and valid do
           begin
                valid := valid and (sValue[i] in ['0' .. '9', 'a'..'f','A'..'F']);

                inc (i)
           end;

     result := valid;
end;

(* -------------------------------------- *)

function isValidSnortHexString (const value : ANSIString) : boolean;

var valid   : boolean;
    content : string;
    count   : integer;
    i,j     : integer;

begin
     count := numEntries (value, '|');
     valid := odd (count);

     if valid and (length (value) > 0) then
        for i := 1 to count do
            begin
                 // look at the content between two | characters, so odd entries

                 if not odd (i) then
                    begin
                         content := getEntry (value, '|', i);
                         content := stringreplace (content, ' ', '', [rfReplaceAll]);

                         valid := valid and StringIsValidHexNumber (content);
                    end;
            end;

     result := valid;
end;

(* -------------------------------------- *)

function getSnortHexLen (const value : ANSIString) : integer;

var totallen : integer;
    content  : string;
    count    : integer;
    i,j      : integer;

begin
     totallen := 0;
     count := numEntries (value, '|');

     if odd (count) and (length (value) > 0) then
        for i := 1 to count do
            begin
                 content := getEntry (value, '|', i);

                 // if it is a string between |, count half because of each hex char taking two digits

                 if not odd (i) then
                    begin
                         if (content <> '') then
                            begin
                                 content := stringreplace (content, ' ', '', [rfReplaceAll]);

                                 totallen := totallen + (length (content) div 2);
                            end;
                    end
                 else
                     totallen := totallen + length (content);
            end;

     result := totallen;
end;

(* -------------------------------------- *)

function getSnortHexAsString (const value : ANSIString) : ANSIString;

var help  : ANSIString;
    entry : ANSIString;
    sub   : ANSIString;
    key   : char;
    num   : integer;
    i     : integer;

begin
     entry := '';

     help := stringReplace (value, ' ', '', [rfReplaceAll]);

     for i := 0 to (length (help) DIV 2) - 1 do
         begin
              sub := '$' + copy (help, (i*2) + 1, 2);

              tryStrToInt (sub, num);

              // Make sure we got it right

              if (num > 0) or (sub = '$00') then
                 entry := entry + chr (num);
         end;

     result := entry;
end;

(* -------------------------------------- *)

function getMD5ForString (const value : string) : string;

var md5    : TIdHashMessageDigest5;
    hash   : T4x4LongWordRecord;

begin
     md5 := TIdHashMessageDigest5.Create;

     try
        result := md5.HashStringAsHex (value);
     finally
            md5.Free;
     end;
 end;

(* -------------------------------------- *)

function SecondsToTimeStr (Seconds : Integer): string;

var H, M, S: string;
    ZH, ZM, ZS: Integer;

begin
     ZH := Seconds div 3600;
     ZM := Seconds div 60 - ZH * 60;
     ZS := Seconds - (ZH * 3600 + ZM * 60) ;

     result := Format ('%.2d:%.2d:%.2d', [ZH, ZM, ZS]);
end;

(* -------------------------------------- *)

function DateTimeToFlatStr (Value : tDateTime) : string;

begin
     result := format ('%.4d', [YearOf (Value)]) +
               format ('%.2d', [MonthOf (Value)]) +
               format ('%.2d', [DayOf (Value)]) +
               format ('%.2d', [HourOf (Value)]) +
               format ('%.2d', [MinuteOf (Value)]) +
               format ('%.2d', [SecondOf (Value)]);
end;

(* -------------------------------------- *)

function FlatStrToDateTime (Value : string) : tDateTime;

var year, month, day     : integer;
    minute, hour, second : integer;
    dummy                : integer;

begin
     assert (length (Value) = 14);

     val (copy (value, 1, 4), year, dummy);
     val (copy (value, 5, 2), month, dummy);
     val (copy (value, 7, 2), day, dummy);
     val (copy (value, 9, 2), hour, dummy);
     val (copy (value, 11, 2), minute, dummy);
     val (copy (value, 13, 2), second, dummy);

     result := encodeDateTime (year, month, day, hour, minute, second, 0);
end;

(* -------------------------------------- *)

procedure FreeStringlistObjects (const strings: TStrings) ;

var idx : integer;

begin
     for idx := 0 to Pred(strings.Count) do
         begin
              strings.Objects[idx].Free;
              strings.Objects[idx] := nil;
         end;
end;

(* -------------------------------------- *)

function MemoryUsed : cardinal;

var st: TMemoryManagerState;
    sb: TSmallBlockTypeState;

begin
     GetMemoryManagerState(st);
     result := st.TotalAllocatedMediumBlockSize + st.TotalAllocatedLargeBlockSize;

     for sb in st.SmallBlockTypeStates do
         begin
              result := result + sb.UseableBlockSize * sb.AllocatedBlockCount;
         end;
end;

(* -------------------------------------- *)


end.
