{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit vstedit_payloadreplacement;

// Taken from the VirtualTree edit demo

// Utility unit for the advanced Virtual Treeview demo application which contains the implementation of edit link
// interfaces used in other samples of the demo.

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, VirtualTrees, ExtDlgs, ImgList, Buttons, ExtCtrls, ComCtrls,
  Mask, base_structures;

(* ------------------------------------------- *)

type tReplacementEditorData = record
                                    original        : ANSIString;
                                    replacement     : ANSIString;
                                    deltalen        : integer;

                                    procedure initialize;
                              end;

     pReplacementEditorData = ^tReplacementEditorData;

     tVSTEditType = (mEdit);

     // Our own edit link to implement several different node editors.

     TPropertyEditLink = class (TInterfacedObject, IVTEditLink)

                               constructor create (EditType : tVSTEditType);

                               private
                                      priv_EditType : tVSTEditType;

                                      FEdit         : TWinControl;          // One of the property editor classes.
                                      FTree         : TVirtualStringTree;   // A back reference to the tree calling.
                                      FNode         : PVirtualNode;         // The node being edited.
                                      FColumn       : Integer;              // The column of the node being edited.

                               protected
                                        procedure EditKeyDown (Sender: TObject; var Key: Word; Shift: TShiftState);
                                        procedure EditKeyUp (Sender: TObject; var Key: Word; Shift: TShiftState);

                               public
                                     destructor Destroy; override;

                                     function BeginEdit: Boolean; stdcall;
                                     function CancelEdit: Boolean; stdcall;
                                     function EndEdit: Boolean; stdcall;
                                     function GetBounds: TRect; stdcall;
                                     function PrepareEdit (Tree: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex): Boolean; stdcall;
                                     procedure ProcessMessage (var Message: TMessage); stdcall;
                                     procedure SetBounds (R: TRect); stdcall;
                               end;


(* ------------------------------------------- *)

type PGridData = ^TGridData;
     TGridData = record
                       ValueType : array[0..3] of TValueType; // one for each column
                       Value     : array[0..3] of Variant;
                       Changed   : Boolean;
                 end;


(* ------------------------------------------- *)

implementation

{ tReplacementEditorData }

procedure tReplacementEditorData.initialize;

begin
     original := '';
     replacement := '';
     deltalen := 0;
end;

(* ------------------------------------------- *)

// This implementation is used in VST3 to make a connection beween the tree
// and the actual edit window which might be a simple edit, a combobox
// or a memo etc.

destructor TPropertyEditLink.Destroy;

begin
     //FEdit.Free; casues issue #357. Fix:

     if FEdit.HandleAllocated then
        PostMessage(FEdit.Handle, CM_RELEASE, 0, 0);

     inherited;
end;

(* ------------------------------------------- *)

procedure TPropertyEditLink.EditKeyDown (Sender: TObject; var Key: Word; Shift: TShiftState);

var CanAdvance: Boolean;

begin
     CanAdvance := true;

     case Key of
       VK_ESCAPE : begin
                        Key := 0;//ESC will be handled in EditKeyUp()
                   end;
       VK_RETURN : if CanAdvance then
                      begin
                           FTree.EndEditNode;
                           Key := 0;
                      end;
       VK_TAB    : if CanAdvance then
                      begin
                           FTree.EndEditNode;
                      end;
       VK_UP,
       VK_DOWN   : begin
                        // Consider special cases before finishing edit mode.
                        CanAdvance := Shift = [];

                        if FEdit is TComboBox then
                           CanAdvance := CanAdvance and not TComboBox(FEdit).DroppedDown;

                        if FEdit is TDateTimePicker then
                           CanAdvance :=  CanAdvance and not TDateTimePicker(FEdit).DroppedDown;

                        if CanAdvance then
                           begin
                                // Forward the keypress to the tree. It will asynchronously change the focused node.

                                PostMessage(FTree.Handle, WM_KEYDOWN, Key, 0);
                                Key := 0;
                           end;
                   end;
     end;
end;

(* ------------------------------------------- *)

procedure TPropertyEditLink.EditKeyUp (Sender: TObject; var Key: Word; Shift: TShiftState);

var value  : tANSIStringPair;
    Data   : pReplacementEditorData;

begin
     Data := FTree.GetNodeData (FNode);

     case FColumn of
      0 : begin
               value.original := (FEdit as TEdit).Text;
               value.replacement := data^.replacement;
          end;
      1 : begin
               value.original := data^.original;
               value.replacement := (FEdit as TEdit).Text;
          end;
     end;

     // Check key

     case Key of
       VK_ESCAPE: begin
                       value.original := Data^.original;
                       value.replacement := Data^.replacement;

                       FTree.CancelEditNode;
                       Key := 0;
                  end;
     end;

     // Update delta column

     data^.deltalen := value.getLengthDelta;

     // repaint

     FTree.RepaintNode (FNode);
end;

(* ------------------------------------------- *)

function TPropertyEditLink.BeginEdit : Boolean;

var EditOkay : boolean;

begin
     Result := True;
     FEdit.Show;
     FEdit.SetFocus;
end;

(* ------------------------------------------- *)

function TPropertyEditLink.CancelEdit : Boolean;

begin
     Result := True;
     FEdit.Hide;
end;

(* ------------------------------------------- *)

constructor TPropertyEditLink.create (EditType : tVSTEditType);

begin
     inherited create;

     priv_EditType := EditType;
end;

(* ------------------------------------------- *)

function TPropertyEditLink.GetBounds: TRect;

begin
     Result := FEdit.BoundsRect;
end;

(* ------------------------------------------- *)

function TPropertyEditLink.PrepareEdit (Tree: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex): Boolean;

// var Data: PPropertyData;
var Data : pReplacementEditorData;

begin
     Result := True;
     FTree := Tree as TVirtualStringTree;
     FNode := Node;
     FColumn := Column;

     // determine what edit type actually is needed

     FEdit.Free;
     FEdit := nil;

     Data := FTree.GetNodeData(Node);

     FEdit := tEdit.Create(nil);

     With FEdit as tEdit do
          begin
              Visible := False;
              Parent := Tree;
              OnKeyDown := EditKeyDown;
              OnKeyUp := EditKeyUp;

              case FColumn of
               0 : Text := Data^.original;
               1 : Text := Data^.replacement;
              end;
          end;
end;

(* ------------------------------------------- *)

function TPropertyEditLink.EndEdit: Boolean;

var Data   : pReplacementEditorData;
    Buffer : array[0..1024] of Char;
    help   : TEdit;
    value  : tANSIStringPair;

begin
     Result := True;

     help := (FEdit as TEdit);
     Data := FTree.GetNodeData(FNode);

     case FColumn of
      0 : begin
               if help.Text <> Data^.original then
                  Data^.original := help.Text;
          end;
      1 : begin
               if help.Text <> Data^.Replacement then
                  Data^.replacement := help.Text;
          end;
     end;

     // Verify and set delta length

     value.original := data^.original;
     value.replacement := data^.replacement;

     data^.deltalen := value.getLengthDelta;

     FEdit.Hide;
     FTree.SetFocus;
end;

(* ------------------------------------------- *)

procedure TPropertyEditLink.ProcessMessage (var Message: TMessage);

begin
     FEdit.WindowProc (Message);
end;

(* ------------------------------------------- *)

procedure TPropertyEditLink.SetBounds (R: TRect);

var Dummy: Integer;

begin
     // Since we don't want to activate grid extensions in the tree (this would influence how the selection is drawn)
     // we have to set the edit's width explicitly to the width of the column.

     FTree.Header.Columns.GetColumnBounds(FColumn, Dummy, R.Right);
     FEdit.BoundsRect := R;
end;

(* ------------------------------------------- *)


end.
