{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit dialog_anon_selectconversations;

// This is thought to be the selector tool for Larry Greenblatts "exchange source and destination MAC/IP" helper. But since it
// requires the conversation store to be able to associate IPs and MACs this is not used as of yet.

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VirtualTrees, Vcl.StdCtrls,

  definition_conversation_types;

type TFormSelectAnonConversations = class(TForm)
       ButtonOkay: TButton;
       ButtonCancel: TButton;
       VSTConversationList: TVirtualStringTree;
       procedure onCreate(Sender: TObject);
       procedure onGetText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType; var CellText: string);
       procedure onShow(Sender: TObject);
    procedure ButtonOkayClick(Sender: TObject);
    procedure ButtonCancelClick(Sender: TObject);
     private
            priv_ConversationList : tIPv4ConversationList;
            priv_ResultList       : tIPv4ConversationList;

            procedure UpdateTree;
     public
           property ResultList : tIPv4ConversationList read priv_Resultlist;

           procedure prepareConversationlist (List : tIPv4ConversationList);

     end;

var
  FormSelectAnonConversations: TFormSelectAnonConversations;

implementation

{$R *.dfm}

type pConversationTreeEntry = ^tConversationTreeEntry;
     tConversationTreeEntry = record
                                    Source       : string;
                                    Destination  : string;

                                    Index        : integer;
                              end;

(* ------------------------------------------- *)

procedure TFormSelectAnonConversations.ButtonCancelClick(Sender: TObject);

begin
     setLength (priv_Resultlist, 0);
end;

(* ------------------------------------------- *)

procedure TFormSelectAnonConversations.ButtonOkayClick(Sender: TObject);

var node  : pVirtualNode;
    data  : pConversationTreeEntry;
    count : integer;
    i     : integer;

begin
     setLength (priv_Resultlist, 0);

     count := VSTConversationList.SelectedCount;
     setLength (priv_Resultlist, count);

     node := VSTConversationList.GetFirstSelected;
     i := 0;

     while node <> nil do
           begin
                data := VSTConversationList.GetNodeData (node);
                priv_Resultlist[i] := priv_ConversationList[data^.Index];

                inc (i);
                node := VSTConversationList.GetNextSelected (node);
           end;
end;

(* ------------------------------------------- *)

procedure TFormSelectAnonConversations.onCreate (Sender: TObject);

begin
     // Workaround for scaling problems: look for /noscale command line option

     self.Scaled := FindCmdLineSwitch ('noscale');

     VSTConversationList.NodeDataSize := sizeOf (tConversationTreeEntry);
end;

(* ------------------------------------------- *)

procedure TFormSelectAnonConversations.onGetText (Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType; var CellText: string);

var data : pConversationTreeEntry;

begin
     data := VSTConversationList.GetNodeData (Node);

     case column of
      0 : CellText := data^.Source;
      1 : CellText := data^.Destination;
     end;
end;

(* ------------------------------------------- *)

procedure TFormSelectAnonConversations.prepareConversationlist (List: tIPv4ConversationList);

var count : integer;
    i     : integer;

begin
     count := length (List);
     setLength (priv_ConversationList, count);

     for i := 1 to count do
         priv_ConversationList[i-1] := List[i-1];
end;

(* ------------------------------------------- *)

procedure TFormSelectAnonConversations.UpdateTree;

var node  : pVirtualNode;
    data  : pConversationTreeEntry;
    entry : tIPv4Conversation;
    i     : integer;

begin
     VSTConversationList.BeginUpdate;
     VSTConversationList.Clear;

     for i := 1 to length (priv_ConversationList) do
         begin
              entry := priv_ConversationList[i-1];

              node := VSTConversationList.AddChild (nil);
              data := VSTConversationList.GetNodeData (node);

              data^.Source := entry.IPv4AddressPair.Source.ValueAsString;
              data^.Destination := entry.IPv4AddressPair.Destination.ValueAsString;
         end;

     VSTConversationList.EndUpdate;
end;

(* ------------------------------------------- *)

procedure TFormSelectAnonConversations.onShow (Sender: TObject);

begin
     UpdateTree;
end;

(* ------------------------------------------- *)

end.
