{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit dialog_configure_conversation_summary;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ComCtrls,
  Vcl.ExtCtrls, VirtualTrees,
  base_structures;

type

     TFormConfigureConversationDisplay = class(TForm)
       PageControl: TPageControl;
       TabSheetTCPConversations: TTabSheet;
       TabSheetGeneral: TTabSheet;
       ColorDialog: TColorDialog;
       ButtonOkay: TButton;
       ButtonCancel: TButton;
       VSTTCPListColors: TVirtualStringTree;
       LabelInitialPage: TLabel;
       ComboBoxInitialPage: TComboBox;
       CheckBoxGoToPageEveryTime: TCheckBox;
       CheckboxClearFilterOnPageChange: TCheckBox;
       procedure onCreate(Sender: TObject);
       procedure onShow(Sender: TObject);
       procedure onVSTTCPColorGetText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType; var CellText: string);
       procedure onVSTTCPColorPaintText(Sender: TBaseVirtualTree; const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType);
       procedure onVSTTCPListColorsDblClick(Sender: TObject);
     private
            priv_tcpcolortable : tTCPColorTable;
     public
           property ColorTable : tTCPColorTable read priv_tcpcolortable write priv_tcpcolortable;
     end;

var
  FormConfigureConversationDisplay: TFormConfigureConversationDisplay;

implementation

{$R *.dfm}

uses definition_conversation_types;

type tListColorEntry = record
                             index      : integer;
                             entrylabel : string;
                             entrycolor : tColor;
                       end;

     pListColorEntry = ^tListColorEntry;

(* ------------------------------------------- *)

procedure TFormConfigureConversationDisplay.onCreate(Sender: TObject);

begin
     // Workaround for scaling problems: look for /noscale command line option

     self.Scaled := FindCmdLineSwitch ('noscale');

     VSTTCPListColors.NodeDataSize := sizeOf (tListColorEntry);

     priv_tcpcolortable[TCPHandshakeNone] := clGrayText;
     priv_tcpcolortable[TCPHandshakeSYNSent] := clGrayText;
     priv_tcpcolortable[TCPHandshakeSYNACKSent] := clGrayText;
     priv_tcpcolortable[TCPHandshakeSYNTwoWay] := clGrayText;
     priv_tcpcolortable[TCPHandshakeHalfOpen] := $0045FF;
     priv_tcpcolortable[TCPHandshakeSYNReset] := clRed;
     priv_tcpcolortable[TCPHandshakeComplete] := clGrayText;
     priv_tcpcolortable[TCPHandshakeAndTeardown] := clWindowText;
     priv_tcpcolortable[TCPTeardown] := clGrayText;
     priv_tcpcolortable[TCPHalfOpenReset] := clRed;
end;

(* ------------------------------------------- *)

procedure TFormConfigureConversationDisplay.onShow(Sender: TObject);

var node : pVirtualNode;
    data : pListColorEntry;
    i    : integer;

begin
     VSTTCPListColors.BeginUpdate;
     VSTTCPListColors.Clear;

     for i := 0 to 8 do
         begin
              node := VSTTCPListColors.AddChild (nil);
              data := VSTTCPListColors.GetNodeData (node);

              data^.index := i;
              data^.entrylabel := GetTCPSessionText (i);
{
              case i of
               TCPHandshakeNone        : data^.entrylabel := 'No Handshake, no teardown';
               TCPHandshakeSYNSent     : data^.entrylabel := 'SYN seen without any answer';
               TCPHandshakeSYNACKSent  : data^.entrylabel := 'SYN/ACK seen but no SYN';
               TCPHandshakeSYNTwoWay   : data^.entrylabel := 'SYN seen coming from both nodes';
               TCPHandshakeHalfOpen    : data^.entrylabel := 'SYN and SYN/ACK seen (TCP half-open)';
               TCPHandshakeSYNReset    : data^.entrylabel := 'Session refused with Reset';
               TCPHandshakeComplete    : data^.entrylabel := 'Three Way Handshake seen';
               TCPHandshakeAndTeardown : data^.entrylabel := 'Three Way Handshake and Teardown seen';
               TCPTeardown             : data^.entrylabel := 'Teardown seen';
               TCPHalfOpenReset        : data^.entrylabel := 'Reset seen after TCP half-open';
               TCPSocketRefused        : data^.entrylabel := 'Active Socket refused by application';
              end;
}

              data^.entrycolor := priv_tcpcolortable[i];
         end;

     VSTTCPListColors.EndUpdate;
end;

(* ------------------------------------------- *)

procedure TFormConfigureConversationDisplay.onVSTTCPColorGetText (Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType; var CellText: string);

var data : pListColorEntry;

begin
     data := VSTTCPListColors.GetNodeData (node);

     CellText := data^.entrylabel;
end;

(* ------------------------------------------- *)

procedure TFormConfigureConversationDisplay.onVSTTCPColorPaintText (Sender: TBaseVirtualTree; const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType);

var data : pListColorEntry;

begin
     data := VSTTCPListColors.GetNodeData (node);

     TargetCanvas.Font.Color := data^.entrycolor;
end;

(* ------------------------------------------- *)

procedure TFormConfigureConversationDisplay.onVSTTCPListColorsDblClick (Sender: TObject);

var node : pVirtualNode;
    data : pListColorEntry;

begin
     if VSTTCPListColors.SelectedCount > 0 then
        begin
             node := VSTTCPListColors.GetFirstSelected;
             data := VSTTCPListColors.GetNodeData (node);

             Colordialog.Color := data^.entrycolor;

             if ColorDialog.Execute then
                begin
                     priv_tcpcolortable[data^.index] := Colordialog.Color;
                     data^.entrycolor := Colordialog.Color;

                     VSTTCPListColors.RepaintNode(node);
                end;

        end;
end;

(* ------------------------------------------- *)

end.
