{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit dialog_conversation_multifilesettings;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls;

type
  TFormMultiFileExtractSettings = class(TForm)
    GroupBoxTargetFileSettings: TGroupBox;
    ButtonOkay: TButton;
    ButtonCancel: TButton;
    LabelFileFormat: TLabel;
    EditFileFormat: TEdit;
    LabelTargetDirectory: TLabel;
    EditTargetDirectory: TEdit;
    ButtonSelectDirectory: TButton;
    MemoOutputHint: TMemo;
    ComboBoxTruncateAfterLayer: TComboBox;
    CheckBoxTruncateAfter: TCheckBox;
    procedure onChangeFileFormat(Sender: TObject);
    procedure ButtonSelectDirectoryClick(Sender: TObject);
    procedure onClickCheckboxes(Sender: TObject);
    procedure onCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FormMultiFileExtractSettings: TFormMultiFileExtractSettings;

implementation

{$R *.dfm}

uses Shlobj;

(* ------------------------------------------ *)

function BrowseDialog (const Title: string; const Flag: integer): string;

var lpItemID    : PItemIDList;
    BrowseInfo  : TBrowseInfo;
    DisplayName : array[0..MAX_PATH] of char;
    TempPath    : array[0..MAX_PATH] of char;

begin
     Result:='';
     FillChar(BrowseInfo, sizeof(TBrowseInfo), #0);

     with BrowseInfo do
          begin
               hwndOwner := Application.Handle;
               pszDisplayName := @DisplayName;
               lpszTitle := PChar(Title);
               ulFlags := Flag;
          end;

  lpItemID := SHBrowseForFolder(BrowseInfo);

  if lpItemId <> nil then
     begin
          SHGetPathFromIDList(lpItemID, TempPath);
          Result := TempPath;
          GlobalFreePtr(lpItemID);
     end;
end;

(* ------------------------------------------- *)

procedure TFormMultiFileExtractSettings.onCreate(Sender: TObject);

begin
     // Workaround for scaling problems: look for /noscale command line option

     self.Scaled := FindCmdLineSwitch ('noscale');
end;

(* ------------------------------------------- *)

procedure TFormMultiFileExtractSettings.ButtonSelectDirectoryClick (Sender: TObject);

var sPath : string;

begin
     sPath := BrowseDialog ('Select Path', BIF_RETURNONLYFSDIRS);

     EditTargetDirectory.Text := includetrailingpathdelimiter (sPath);
end;

(* ------------------------------------------- *)

procedure TFormMultiFileExtractSettings.onChangeFileFormat (Sender: TObject);

var valid : boolean;
    help  : char;
    test  : string;
    i     : integer;

begin
     valid := true;

     // remove known placeholders

     test := EditFileFormat.Text;

     test := stringreplace (test, '<filename>', '', [rfReplaceAll, rfIgnoreCase]);
     test := stringreplace (test, '<count>', '', [rfReplaceAll, rfIgnoreCase]);
     test := stringreplace (test, '<timestamp>', '', [rfReplaceAll, rfIgnoreCase]);
     test := stringreplace (test, '<sourcemac>', '', [rfReplaceAll, rfIgnoreCase]);
     test := stringreplace (test, '<destinationmac>', '', [rfReplaceAll, rfIgnoreCase]);
     test := stringreplace (test, '<sourceip>', '', [rfReplaceAll, rfIgnoreCase]);
     test := stringreplace (test, '<destinationip>', '', [rfReplaceAll, rfIgnoreCase]);
     test := stringreplace (test, '<sourceport>', '', [rfReplaceAll, rfIgnoreCase]);
     test := stringreplace (test, '<destinationport>', '', [rfReplaceAll, rfIgnoreCase]);

     // Check for invalid characters

     for i := 1 to length (test) do
         begin
              help := test[i];

              if help in ['<', '>', ':', '"', '/', '|', '?', '*'] then
                 valid := false;
         end;

     if valid then
        EditFileFormat.Font.Color := clWindowText
     else
         EditFileFormat.Font.Color := clRed;

     ButtonOkay.Enabled := valid;
end;

(* ------------------------------------------- *)

procedure TFormMultiFileExtractSettings.onClickCheckboxes(Sender: TObject);

begin
     ComboBoxTruncateAfterLayer.Enabled := CheckBoxTruncateAfter.Checked;
end;

(* ------------------------------------------- *)

end.
