{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit dialog_editfilter;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls,

  definition_filter_structures;

type tFilterPresetMode = (fsDefault, fsConversation);

  TFormEditFilter = class(TForm)
    ButtonOkay: TButton;
    ButtonCancel: TButton;
    GroupBoxEthernetSettings: TGroupBox;
    LabelSourceMAC: TLabel;
    LabelDestinationMAC: TLabel;
    EditSourceMAC: TEdit;
    EditDestinationMAC: TEdit;
    GroupBoxIPSettings: TGroupBox;
    LabelSourceIP: TLabel;
    LabelDestinationIP: TLabel;
    EditSourceIPAddress: TEdit;
    EditDestinationIPAddress: TEdit;
    ComboBoxSourceIPMaskBits: TComboBox;
    LabelMACMatchOn: TLabel;
    ComboBoxEthernetMatchOn: TComboBox;
    LabelIPMatchOn: TLabel;
    ComboBoxIPMatchOn: TComboBox;
    ComboBoxDestinationIPMaskBits: TComboBox;
    LabelMACAddress: TLabel;
    EditMAC: TEdit;
    LabelIPAddress: TLabel;
    EditIPAddress: TEdit;
    ComboBoxIPMaskBits: TComboBox;
    GroupBoxPortSettings: TGroupBox;
    LabelPortMatchOn: TLabel;
    ComboBoxPortsMatchOn: TComboBox;
    LabelPort: TLabel;
    EditPort: TEdit;
    LabelSourcePort: TLabel;
    EditSourcePort: TEdit;
    LabelDestinationPort: TLabel;
    EditDestinationPort: TEdit;
    ComboBoxPortProtocol: TComboBox;
    LabelTCPFlags: TLabel;
    CheckBoxFlagSYN: TCheckBox;
    CheckBoxFlagACK: TCheckBox;
    CheckBoxFlagPSH: TCheckBox;
    CheckBoxFlagRST: TCheckBox;
    CheckBoxFlagFIN: TCheckBox;
    CheckBoxFlagURG: TCheckBox;
    CheckBoxFlagECN: TCheckBox;
    CheckBoxFlagCWR: TCheckBox;
    LabelTCPSpecial: TLabel;
    CheckBoxTCPHandshake: TCheckBox;
    GroupBoxMetaData: TGroupBox;
    LabelFilterLabel: TLabel;
    EditFilterLabel: TEdit;
    GroupBoxICMPSettings: TGroupBox;
    LabelICMPType: TLabel;
    EditICMPType: TEdit;
    LabelMatchICMP: TLabel;
    ComboBoxMatchICMP: TComboBox;
    procedure onCreate(Sender: TObject);
    procedure onClickAddressScope(Sender: TObject);
    procedure onKeyPressEditMAC(Sender: TObject; var Key: Char);
    procedure onKeyPressEditIP(Sender: TObject; var Key: Char);
    procedure onKeyPressEditPort(Sender: TObject; var Key: Char);
    procedure onShow(Sender: TObject);
    procedure onKeyPressEditLabel(Sender: TObject; var Key: Char);
  private
    { Private declarations }

    procedure UpdateFormElements;

    procedure SetMACFilterItems (const Filter : tMACAddressFilter);
    procedure SetIPv4FilterItems (const Filter : tIPv4AddressFilter);
    procedure SetIPv6FilterItems (const Filter : tIPv6AddressFilter);
    procedure SetPortFilterItems (const Filter : tPortFilter);
    procedure SetTCPFilterItems (const Filter : tTCPFilter);

  public
        { Public declarations }

        procedure clear;
        procedure PresetFilterSettings (Mode : tFilterPresetMode);
        procedure PrepareForm (Filter : tFrameFilter);

        function getMACFilter : tMACAddressFilter;
        function getIPv4Filter : tIPv4AddressFilter;
        function getIPv6Filter : tIPv6AddressFilter;
        function getICMPFilter : tICMPFilter;
        function getPortFilter : tPortFilter;
        function getTCPFilter : tTCPFilter;

        function getFrameFilter : tFrameFilter;
  end;

var
  FormEditFilter: TFormEditFilter;

implementation

uses base_structures,
     definition_tcp_structures;

(* -------------------------------------------- *)
{$R *.dfm}
(* -------------------------------------------- *)

procedure TFormEditFilter.onCreate(Sender: TObject);

var i : integer;

begin
     // Workaround for scaling problems: look for /noscale command line option

     self.Scaled := FindCmdLineSwitch ('noscale');

     // Prepare Mask bit selectors

     for i := 128 downto 1 do
         begin
              ComboBoxIPMaskBits.Items.Add (IntToStr (i));
              ComboBoxSourceIPMaskBits.Items.Add (IntToStr (i));
              ComboBoxDestinationIPMaskBits.Items.Add (IntToStr (i));
         end;
end;

(* -------------------------------------------- *)

procedure TFormEditFilter.onShow(Sender: TObject);

begin
     UpdateFormElements;

end;

(* -------------------------------------------- *)

procedure TFormEditFilter.UpdateFormElements;

begin
     // Ethernet

     case ComboBoxEthernetMatchOn.ItemIndex of
      0 : begin
               LabelMACAddress.Enabled := true;
               LabelSourceMAC.Enabled := false;
               LabelDestinationMAC.Enabled := false;
          end;
      1 : begin
               LabelMACAddress.Enabled := false;
               LabelSourceMAC.Enabled := true;
               LabelDestinationMAC.Enabled := false;
          end;
      2 : begin
               LabelMACAddress.Enabled := false;
               LabelSourceMAC.Enabled := false;
               LabelDestinationMAC.Enabled := true;
          end;
      3 : begin
               LabelMACAddress.Enabled := false;
               LabelSourceMAC.Enabled := true;
               LabelDestinationMAC.Enabled := true;
          end;
     end;

     EditMAC.Enabled := LabelMACAddress.Enabled;
     EditSourceMAC.Enabled := LabelSourceMAC.Enabled;
     EditDestinationMAC.Enabled := LabelDestinationMAC.Enabled;

     // IP

     case ComboBoxIPMatchOn.ItemIndex of
      0 : begin
               LabelIPAddress.Enabled := true;
               LabelSourceIP.Enabled := false;
               LabelDestinationIP.Enabled := false;
          end;
      1 : begin
               LabelIPAddress.Enabled := false;
               LabelSourceIP.Enabled := true;
               LabelDestinationIP.Enabled := false;
          end;
      2 : begin
               LabelIPAddress.Enabled := false;
               LabelSourceIP.Enabled := false;
               LabelDestinationIP.Enabled := true;
          end;
      3 : begin
               LabelIPAddress.Enabled := false;
               LabelSourceIP.Enabled := true;
               LabelDestinationIP.Enabled := true;
          end;
     end;

     EditIPAddress.Enabled := LabelIPAddress.Enabled;
     EditSourceIPAddress.Enabled := LabelSourceIP.Enabled;
     EditDestinationIPAddress.Enabled := LabelDestinationIP.Enabled;

     ComboBoxIPMaskBits.Enabled := EditIPAddress.Enabled;
     ComboBoxSourceIPMaskBits.Enabled := EditSourceIPAddress.Enabled;
     ComboBoxDestinationIPMaskBits.Enabled := EditDestinationIPAddress.Enabled;


     // Ports

     case ComboBoxPortsMatchOn.ItemIndex of
      0 : begin
               LabelPort.Enabled := true;
               LabelSourcePort.Enabled := false;
               LabelDestinationPort.Enabled := false;
          end;
      1 : begin
               LabelPort.Enabled := false;
               LabelSourcePort.Enabled := true;
               LabelDestinationPort.Enabled := false;
          end;
      2 : begin
               LabelPort.Enabled := false;
               LabelSourcePort.Enabled := false;
               LabelDestinationPort.Enabled := true;
          end;
      3 : begin
               LabelPort.Enabled := false;
               LabelSourcePort.Enabled := true;
               LabelDestinationPort.Enabled := true;
          end;
     end;

     EditPort.Enabled := LabelPort.Enabled;
     EditSourcePort.Enabled := LabelSourcePort.Enabled;
     EditDestinationPort.Enabled := LabelDestinationPort.Enabled;
end;

(* -------------------------------------------- *)

procedure TFormEditFilter.onClickAddressScope(Sender: TObject);

begin
     UpdateFormElements;
end;

(* -------------------------------------------- *)

procedure TFormEditFilter.onKeyPressEditMAC (Sender: TObject; var Key: Char);

begin
     if not (key in [#1, #3, #8, #9, #$16, #$1a, '0'..'9','a'..'f','A'..'F', ':']) then
        key:=#0;
end;

(* -------------------------------------------- *)

procedure TFormEditFilter.onKeyPressEditIP (Sender: TObject; var Key: Char);

begin
     if not (key in [#1, #3, #8, #9, #$16, #$1a, '0'..'9','a'..'f','A'..'F', '.', ':']) then
        key:=#0;
end;

(* -------------------------------------------- *)

procedure TFormEditFilter.onKeyPressEditPort (Sender: TObject; var Key: Char);

begin
     if not (key in [#1, #3, #8, #9, #$16, #$1a, '0'..'9', '-', ',']) then
        key:=#0;
end;

(* -------------------------------------------- *)

procedure TFormEditFilter.onKeyPressEditLabel(Sender: TObject; var Key: Char);

begin
     if not (key in [#1, #3, #8, #9, #$16, #$1a, ' ', 'a'..'z', 'A'..'Z', '0'..'9', '-', '_', '#', '[', ']','(',')']) then
        key:=#0;
end;

(* -------------------------------------------- *)

procedure TFormEditFilter.clear;

begin
     ComboBoxEthernetMatchOn.ItemIndex := 0;
     EditMAC.Text := '';
     EditSourceMac.Text := '';
     EditDestinationMac.Text := '';

     ComboBoxIPMatchOn.ItemIndex := 0;
     EditIPAddress.Text := '';
     EditSourceIPAddress.Text := '';
     EditDestinationIPAddress.Text := '';

     ComboBoxIPMaskBits.ItemIndex := 128 - 32;
     ComboBoxSourceIPMaskBits.ItemIndex := 128 - 32;
     ComboBoxDestinationIPMaskBits.ItemIndex := 128 - 32;

     ComboBoxMatchICMP.ItemIndex := 0;
     EditICMPType.Text := '';

     ComboBoxPortsMatchOn.ItemIndex := 0;
     EditPort.Text := '';
     EditSourcePort.Text := '';
     EditDestinationPort.Text := '';

     ComboBoxPortProtocol.ItemIndex := 0;

     EditFilterLabel.Text := '';
end;

(* -------------------------------------------- *)

procedure TFormEditFilter.SetMACFilterItems (const Filter: tMACAddressFilter);

begin
     // Right now we only have ftSingle

     case Filter.AddressFilterType of
      ftIgnore : ;
      ftSingle : begin
                      case Filter.AddressScope of
                       fsAny         : begin
                                            ComboBoxEthernetMatchOn.ItemIndex := 0;
                                            EditMAC.Text := Filter.SingleAddress.asString;
                                       end;
                       fsSource      : begin
                                            ComboBoxEthernetMatchOn.ItemIndex := 1;
                                            EditSourceMAC.Text := Filter.SourceAddress.asString;
                                       end;
                       fsDestination : begin
                                            ComboBoxEthernetMatchOn.ItemIndex := 2;
                                            EditDestinationMAC.Text := Filter.DestinationAddress.asString;
                                       end;
                       fsBoth        : begin
                                            ComboBoxEthernetMatchOn.ItemIndex := 3;
                                            EditSourceMAC.Text := Filter.SourceAddress.asString;
                                            EditDestinationMAC.Text := Filter.DestinationAddress.asString;
                                       end;
                      end;
                 end;
      ftRange  : ;
     end;
end;

(* -------------------------------------------- *)

procedure TFormEditFilter.SetIPv4FilterItems (const Filter: tIPv4AddressFilter);

begin
     case Filter.AddressFilterType of
      ftIgnore : ;
      ftSingle : begin
                      case Filter.AddressScope of
                       fsAny         : begin
                                            ComboBoxIPMatchOn.ItemIndex := 0;
                                            EditIPAddress.Text := Filter.SingleAddress.ValueAsString;
                                            ComboBoxIPMaskBits.ItemIndex := 128 - Filter.SingleAddress.MaskBitCount;
                                       end;
                       fsSource      : begin
                                            ComboBoxIPMatchOn.ItemIndex := 1;
                                            EditSourceIPAddress.Text := Filter.SourceAddress.ValueAsString;
                                            ComboBoxSourceIPMaskBits.ItemIndex := 128 - Filter.SourceAddress.MaskBitCount;
                                       end;
                       fsDestination : begin
                                            ComboBoxIPMatchOn.ItemIndex := 2;
                                            EditDestinationIPAddress.Text := Filter.DestinationAddress.ValueAsString;
                                            ComboBoxDestinationIPMaskBits.ItemIndex := 128 - Filter.DestinationAddress.MaskBitCount;
                                       end;
                       fsBoth        : begin
                                            ComboBoxIPMatchOn.ItemIndex := 3;
                                            EditSourceIPAddress.Text := Filter.SourceAddress.ValueAsString;
                                            EditDestinationIPAddress.Text := Filter.DestinationAddress.ValueAsString;

                                            ComboBoxSourceIPMaskBits.ItemIndex := 128 - Filter.SourceAddress.MaskBitCount;
                                            ComboBoxDestinationIPMaskBits.ItemIndex := 128 - Filter.DestinationAddress.MaskBitCount;
                                       end;
                      end;
                 end;
      ftRange  : ;
     end;
end;

(* -------------------------------------------- *)

procedure TFormEditFilter.SetIPv6FilterItems (const Filter: tIPv6AddressFilter);

begin
     case Filter.AddressFilterType of
      ftIgnore : ;
      ftSingle : begin
                      case Filter.AddressScope of
                       fsAny         : begin
                                            ComboBoxIPMatchOn.ItemIndex := 0;
                                            EditIPAddress.Text := Filter.SingleAddress.ValueAsString;
                                            ComboBoxIPMaskBits.ItemIndex := 128 - Filter.SingleAddress.MaskBitCount;
                                       end;
                       fsSource      : begin
                                            ComboBoxIPMatchOn.ItemIndex := 1;
                                            EditSourceIPAddress.Text := Filter.SourceAddress.ValueAsString;
                                            ComboBoxSourceIPMaskBits.ItemIndex := 128 - Filter.SourceAddress.MaskBitCount;
                                       end;
                       fsDestination : begin
                                            ComboBoxIPMatchOn.ItemIndex := 2;
                                            EditDestinationIPAddress.Text := Filter.DestinationAddress.ValueAsString;
                                            ComboBoxDestinationIPMaskBits.ItemIndex := 128 - Filter.DestinationAddress.MaskBitCount;
                                       end;
                       fsBoth        : begin
                                            ComboBoxIPMatchOn.ItemIndex := 3;
                                            EditSourceIPAddress.Text := Filter.SourceAddress.ValueAsString;
                                            EditDestinationIPAddress.Text := Filter.DestinationAddress.ValueAsString;

                                            ComboBoxSourceIPMaskBits.ItemIndex := 128 - Filter.SourceAddress.MaskBitCount;
                                            ComboBoxDestinationIPMaskBits.ItemIndex := 128 - Filter.DestinationAddress.MaskBitCount;
                                       end;
                      end;
                 end;
      ftRange  : ;
     end;
end;

(* -------------------------------------------- *)

procedure TFormEditFilter.SetPortFilterItems (const Filter: tPortFilter);

begin
     case Filter.PortFilterType of
      ftIgnore : ;
      ftSingle : begin
                      case Filter.PortScope of
                       fsAny         : begin
                                            ComboBoxPortsMatchOn.ItemIndex := 0;
                                            EditPort.Text := IntToStr (Filter.SinglePort);
                                       end;
                       fsSource      : begin
                                            ComboBoxPortsMatchOn.ItemIndex := 1;
                                            EditSourcePort.Text := IntToStr (Filter.SourcePort);
                                       end;
                       fsDestination : begin
                                            ComboBoxPortsMatchOn.ItemIndex := 2;
                                            EditDestinationPort.Text := IntToStr (Filter.DestinationPort);
                                       end;
                       fsBoth        : begin
                                            ComboBoxPortsMatchOn.ItemIndex := 3;
                                            EditSourcePort.Text := IntToStr (Filter.SourcePort);
                                            EditDestinationPort.Text := IntToStr (Filter.DestinationPort);
                                       end;
                      end;

                      case Filter.Layer4Protocol of
                       l4TCPUDP : ComboBoxPortProtocol.ItemIndex := 0;
                       l4TCP    : ComboBoxPortProtocol.ItemIndex := 1;
                       l4UDP    : ComboBoxPortProtocol.ItemIndex := 2;
                      end;
                 end;
      ftRange  : ;
     end;
end;

(* -------------------------------------------- *)

procedure TFormEditFilter.SetTCPFilterItems (const Filter: tTCPFilter);

var flag : tTCPFlags;

begin
     for flag in Filter.Flags do
         begin
              case flag of
               tcpSYN : CheckboxFlagSYN.Checked := true;
               tcpACK : CheckboxFlagACK.Checked := true;
               tcpFIN : CheckboxFlagFIN.Checked := true;
               tcpRST : CheckboxFlagRST.Checked := true;
               tcpPSH : CheckboxFlagPSH.Checked := true;
               tcpURG : CheckboxFlagURG.Checked := true;
               tcpECN : CheckboxFlagECN.Checked := true;
               tcpCWR : CheckboxFlagCWR.Checked := true;
              end;
         end;

     CheckboxTCPHandshake.Checked := Filter.TCPHandshake;
end;

(* -------------------------------------------- *)

procedure TFormEditFilter.PrepareForm (Filter: tFrameFilter);

begin
     SetMACFilterItems (Filter.MACFilter);
     SetIPv4FilterItems (Filter.IPv4Filter);
     SetIPv6FilterItems (Filter.IPv6Filter);
     SetPortFilterItems (Filter.PortFilter);
     SetTCPFilterItems (Filter.TCPFilter);

     EditFilterLabel.Text := Filter.FilterLabel;
end;

(* -------------------------------------------- *)

procedure TFormEditFilter.PresetFilterSettings (Mode: tFilterPresetMode);

begin
     case Mode of
      fsDefault      : begin
                            ComboBoxEthernetMatchOn.ItemIndex := 0;
                            ComboBoxIPMatchOn.ItemIndex := 0;
                            ComboBoxIPMaskBits.ItemIndex := 128 - 32;
                            ComboBoxSourceIPMaskBits.ItemIndex := 128 - 32;
                            ComboBoxDestinationIPMaskBits.ItemIndex := 128 - 32;
                            ComboBoxPortsMatchOn.ItemIndex := 0;
                            ComboBoxPortProtocol.ItemIndex := 0;
                       end;
      fsConversation : begin
                            // Set combo boxes to IP pair and port pair

                            ComboBoxEthernetMatchOn.ItemIndex := 3; // Address Pair
                            ComboBoxIPMatchOn.ItemIndex := 3;       // Address Pair
                            ComboBoxPortsMatchOn.ItemIndex := 3;    // Port Pair
                            ComboBoxPortProtocol.ItemIndex := 1;    // TCP
                       end;
     end;
end;

(* -------------------------------------------- *)

function TFormEditFilter.getMACFilter: tMACAddressFilter;

var MACFilter : tMACAddressFilter;

begin
     MACFilter.initialize;

     // Get Ethernet values

     if (EditMAC.Text <> '') or (EditSourceMAC.Text <> '') or (EditDestinationMAC.Text <> '') then
        begin
             MacFilter.initialize;
             MACFilter.AddressFilterType := ftSingle;

             case ComboBoxEthernetMatchOn.ItemIndex of
              0 : begin
                       MACFilter.AddressScope := fsAny;
                       MACFilter.SingleAddress.setfromString (EditMAC.Text);
                  end;
              1 : begin
                       MACFilter.AddressScope := fsSource;
                       MACFilter.SourceAddress.setfromString (EditSourceMAC.Text);
                  end;
              2 : begin
                       MACFilter.AddressScope := fsDestination;
                       MACFilter.DestinationAddress.setfromString (EditDestinationMAC.Text);
                  end;
              3 : begin
                       MACFilter.AddressScope := fsBoth;
                       MACFilter.SourceAddress.setfromString (EditSourceMAC.Text);
                       MACFilter.DestinationAddress.setfromString (EditDestinationMAC.Text);
                  end;
             end;
        end;

     result := MACFilter;
end;

(* -------------------------------------------- *)

function TFormEditFilter.getIPv4Filter: tIPv4AddressFilter;

var IPv4Filter : tIPv4AddressFilter;

begin
     // IP values

     IPv4Filter.initialize;

     if (EditIPAddress.Text <> '') or (EditSourceIPAddress.Text <> '') or (EditDestinationIPAddress.Text <> '') then
        begin
             IPv4Filter.AddressFilterType := ftSingle;

             case ComboBoxIPMatchOn.ItemIndex of
              0 : begin
                       if pos (':', EditIPAddress.Text) = 0 then
                          begin
                               IPv4Filter.AddressScope := fsAny;

                               IPv4Filter.SingleAddress.setValueFromString(EditIPAddress.Text, 128 - ComboBoxIPMaskBits.ItemIndex);
                          end;
                  end;
              1 : begin
                       if pos (':', EditSourceIPAddress.Text) = 0 then
                          begin
                               IPv4Filter.AddressScope := fsSource;
                               IPv4Filter.SourceAddress.setValueFromString (EditSourceIPAddress.Text, 128 - ComboBoxSourceIPMaskBits.ItemIndex);
                          end;
                  end;
              2 : begin
                       if pos (':', EditDestinationIPAddress.Text) = 0 then
                          begin
                               IPv4Filter.AddressScope := fsDestination;

                               IPv4Filter.DestinationAddress.setValueFromString (EditDestinationIPAddress.Text, 128 - ComboBoxDestinationIPMaskBits.ItemIndex);
                          end;
                  end;
              3 : begin
                       if pos (':', EditDestinationIPAddress.Text) = 0 then
                          begin
                               IPv4Filter.AddressScope := fsBoth;

                               IPv4Filter.SourceAddress.setValueFromString (EditSourceIPAddress.Text, 128 - ComboBoxSourceIPMaskBits.ItemIndex);
                               IPv4Filter.DestinationAddress.setValueFromString (EditDestinationIPAddress.Text, 128 - ComboBoxDestinationIPMaskBits.ItemIndex);
                          end;
                  end;
             end;
        end;

     result := IPv4Filter;
end;

(* -------------------------------------------- *)

function TFormEditFilter.getIPv6Filter: tIPv6AddressFilter;

var IPv6Filter : tIPv6AddressFilter;

begin
     IPv6Filter.initialize;

     if (EditIPAddress.Text <> '') or (EditSourceIPAddress.Text <> '') or (EditDestinationIPAddress.Text <> '') then
        begin
             case ComboBoxIPMatchOn.ItemIndex of
              0 : begin
                       if pos (':', EditIPAddress.Text) <> 0 then
                          begin
                               IPv6Filter.AddressScope := fsAny;

                               IPv6Filter.SingleAddress.setValueFromString(EditIPAddress.Text, 129 - ComboBoxIPMaskBits.ItemIndex);
                          end;
                  end;
              1 : begin
                       if pos (':', EditSourceIPAddress.Text) <> 0 then
                          begin
                               IPv6Filter.AddressScope := fsSource;

                               IPv6Filter.SourceAddress.setValueFromString (EditSourceIPAddress.Text, 129 - ComboBoxSourceIPMaskBits.ItemIndex);
                          end;
                  end;
              2 : begin
                       if pos (':', EditDestinationIPAddress.Text) <> 0 then
                          begin
                               IPv6Filter.AddressScope := fsDestination;

                               IPv6Filter.DestinationAddress.setValueFromString (EditDestinationIPAddress.Text, 129 - ComboBoxDestinationIPMaskBits.ItemIndex);
                          end;
                  end;
              3 : begin
                       if pos (':', EditDestinationIPAddress.Text) <> 0 then
                          begin
                               IPv6Filter.AddressScope := fsBoth;

                               IPv6Filter.SourceAddress.setValueFromString (EditSourceIPAddress.Text, 129 - ComboBoxSourceIPMaskBits.ItemIndex);
                               IPv6Filter.DestinationAddress.setValueFromString (EditDestinationIPAddress.Text, 129 - ComboBoxDestinationIPMaskBits.ItemIndex);
                          end;
                  end;
             end;
        end;

     result := IPv6Filter;
end;

(* -------------------------------------------- *)

function TFormEditFilter.getICMPFilter: tICMPFilter;

var ICMPFilter : tICMPFilter;
    dummy      : integer;

begin
     ICMPFilter.initialize;

     case ComboBoxMatchICMP.ItemIndex of
      0 : begin
               // Match always

               ICMPFilter.ICMPFilterType := IFTIgnore;
          end;
      1 : begin
               // Specific ICMP Types

               ICMPFilter.ICMPFilterType := IFTType;

               val (EditICMPType.Text, ICMPFilter.ICMPType, dummy);

               if ICMPFilter.ICMPType = 0 then
                  ICMPFilter.ICMPFilterType := IFTIgnore;
          end;
      2 : begin
               // Refuse always

               ICMPFilter.ICMPFilterType := IFTReject;
          end;
     end;

     result := ICMPFilter;
end;

(* -------------------------------------------- *)

function TFormEditFilter.getPortFilter: tPortFilter;

var PortFilter : tPortFilter;

begin
     PortFilter.initialize;

     if (EditPort.Text <> '') or (EditSourcePort.Text <> '') or (EditDestinationPort.Text <> '') then
        begin
             PortFilter.PortFilterType := ftSingle;

             case ComboBoxPortProtocol.ItemIndex of
              0 : PortFilter.Layer4Protocol := l4TCPUDP;
              1 : PortFilter.Layer4Protocol := l4TCP;
              2 : PortFilter.Layer4Protocol := l4UDP;
             end;

             case ComboBoxPortsMatchOn.ItemIndex of
              0 : begin
                       PortFilter.PortScope := fsAny;
                       TryStrToInt (EditPort.Text, PortFilter.SinglePort);
                  end;
              1 : begin
                       PortFilter.PortScope := fsSource;
                       TryStrToInt (EditSourcePort.Text, PortFilter.SourcePort);
                  end;
              2 : begin
                       PortFilter.PortScope := fsDestination;
                       TryStrToInt (EditDestinationPort.Text, PortFilter.DestinationPort);
                  end;
              3 : begin
                       PortFilter.PortScope := fsBoth;
                       TryStrToInt (EditSourcePort.Text, PortFilter.SourcePort);
                       TryStrToInt (EditDestinationPort.Text, PortFilter.DestinationPort);
                  end;
             end;
        end;

     result := PortFilter;
end;

(* -------------------------------------------- *)

function TFormEditFilter.getTCPFilter: tTCPFilter;

var TCPFilter : tTCPFilter;

begin
     TCPFilter.initialize;

     if CheckBoxFlagSYN.Checked then
        TCPFilter.Flags := TCPFilter.Flags + [tcpSYN];

     if CheckBoxFlagACK.Checked then
        TCPFilter.Flags := TCPFilter.Flags + [tcpACK];

     if CheckBoxFlagPSH.Checked then
        TCPFilter.Flags := TCPFilter.Flags + [tcpPSH];

     if CheckBoxFlagRST.Checked then
        TCPFilter.Flags := TCPFilter.Flags + [tcpRST];

     if CheckBoxFlagFIN.Checked then
        TCPFilter.Flags := TCPFilter.Flags + [tcpFIN];

     if CheckBoxFlagURG.Checked then
        TCPFilter.Flags := TCPFilter.Flags + [tcpURG];

     if CheckBoxFlagECN.Checked then
        TCPFilter.Flags := TCPFilter.Flags + [tcpECN];

     if CheckBoxFlagCWR.Checked then
        TCPFilter.Flags := TCPFilter.Flags + [tcpCWR];

     TCPFilter.TCPHandshake := CheckboxTCPHandshake.Checked;

     // Set to active if anything is set

     if (TCPFilter.Flags <> []) or TCPFilter.TCPHandshake then
        TCPFilter.TCPFilterType := ftSingle;

     result := TCPFilter;
end;

(* -------------------------------------------- *)

function TFormEditFilter.getFrameFilter: tFrameFilter;

var filter : tFrameFilter;

begin
     filter.MACFilter := getMACFilter;
     filter.IPv4Filter := getIPv4Filter;
     filter.IPv6Filter := getIPv6Filter;
     filter.ICMPFilter := getICMPFilter;
     filter.PortFilter := getPortFilter;
     filter.TCPFilter := getTCPFilter;

     filter.FilterLabel := EditFilterLabel.Text;

     result := filter;
end;

(* -------------------------------------------- *)

end.
