{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit dialog_editmetadata;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls,

  class_parse_snortalertlog, Vcl.ComCtrls;

type
  TFormEditMetaData = class(TForm)
    GroupBoxMetaDetails: TGroupBox;
    ButtonOkay: TButton;
    ButtonCancel: TButton;
    LabelIPVersion: TLabel;
    ComboBoxIPVersion: TComboBox;
    LabelSourceIP: TLabel;
    EditSourceIP: TEdit;
    LabelSourcePort: TLabel;
    EditSourcePort: TEdit;
    LabelDestinationIP: TLabel;
    EditDestinationIP: TEdit;
    LabelDestinationPort: TLabel;
    EditDestinationPort: TEdit;
    LabelIPID: TLabel;
    EditIPID: TEdit;
    LabelTCPSequence: TLabel;
    EditTCPSequence: TEdit;
    LabelLayer4Protocol: TLabel;
    ComboBoxL4Protocol: TComboBox;
    UpDownIPID: TUpDown;
    LabelFrameComment: TLabel;
    EditFrameComment: TEdit;
    procedure onKeyDownEdit(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure onKeyPressIPAddress(Sender: TObject; var Key: Char);
    procedure onChangeComboboxes(Sender: TObject);
    procedure onChangeIPorPort(Sender: TObject);
    procedure onShow(Sender: TObject);
    procedure onCreate(Sender: TObject);
  private
         function GetDataEntry : tMetaDataEntry;
         procedure SetDataEntry (const value : tMetaDataEntry);

         procedure ValidateInputfields;
         procedure UpdateInputfieldStatus;

  public
        property DataEntry : tMetaDataEntry read GetDataEntry write SetDataEntry;
        procedure Clear;

  end;

var
  FormEditMetaData: TFormEditMetaData;

implementation

{$R *.dfm}

uses base_structures,
     definition_ip_addresses;

(* -------------------------------------- *)
{ TFormEditMetaData }
(* -------------------------------------- *)

procedure TFormEditMetaData.onCreate(Sender: TObject);

begin
     // Workaround for scaling problems: look for /noscale command line option

     self.Scaled := FindCmdLineSwitch ('noscale');
end;

(* -------------------------------------- *)

procedure TFormEditMetaData.onShow(Sender: TObject);

begin
     EditSourceIP.SetFocus;
end;

(* -------------------------------------- *)

function TFormEditMetaData.GetDataEntry: tMetaDataEntry;

var value : tMetaDataEntry;
    dummy : integer;

begin
     case ComboboxIPVersion.ItemIndex of
      0 : begin
               value.IPVersion := 4;

               value.IPv4SocketPair.Source.IPAddress.setValueFromString (EditSourceIP.Text);
               value.IPv4SocketPair.Destination.IPAddress.setValueFromString (EditDestinationIP.Text);

               value.IPID := UpDownIPID.Position;
          end;
      1 : begin
               value.IPVersion := 6;

               value.IPv6SocketPair.Source.IPAddress.setValueFromString (EditSourceIP.Text);
               value.IPv6SocketPair.Destination.IPAddress.setValueFromString (EditDestinationIP.Text);

               value.IPID :=  0;
          end;
     end;

     case ComboBoxL4Protocol.ItemIndex of
      0 : begin
               value.Layer4Protocol := l4Any;

               val (EditSourcePort.Text, value.IPv4SocketPair.Source.Port, dummy);
               val (EditDestinationPort.Text, value.IPv4SocketPair.Destination.Port, dummy);

               val (EditTCPSequence.Text, value.TCPSequenceNumber, dummy);
          end;
      1 : begin
               value.Layer4Protocol := l4TCP;

               val (EditSourcePort.Text, value.IPv4SocketPair.Source.Port, dummy);
               val (EditDestinationPort.Text, value.IPv4SocketPair.Destination.Port, dummy);

               val (EditTCPSequence.Text, value.TCPSequenceNumber, dummy);
          end;
      2 : begin
               value.Layer4Protocol := l4UDP;

               val (EditSourcePort.Text, value.IPv4SocketPair.Source.Port, dummy);
               val (EditDestinationPort.Text, value.IPv4SocketPair.Destination.Port, dummy);

               value.TCPSequenceNumber := 0;
          end;
      3 : begin
               value.Layer4Protocol := l4ICMP;

               value.IPv4SocketPair.Source.Port := 0;
               value.IPv4SocketPair.Destination.Port := 0;
          end;
     end;

     value.RuleDetails.Comment := EditFrameComment.Text;

     result := value;
end;

(* -------------------------------------- *)

procedure TFormEditMetaData.onChangeComboboxes (Sender: TObject);

begin
     UpdateInputFieldStatus;
end;

(* -------------------------------------- *)

procedure TFormEditMetaData.onChangeIPorPort(Sender: TObject);

begin
     ValidateInputfields;
end;

(* -------------------------------------- *)

procedure TFormEditMetaData.onKeyDownEdit(Sender: TObject; var Key: Word; Shift: TShiftState);

begin
     if Shift = [ssCtrl] then
        begin
             Case Key of
               Ord ('C') : (Sender as tEdit).CopyToClipboard;
               Ord ('V') : (Sender as tEdit).PasteFromClipboard;
               Ord ('X') : (Sender as tEdit).CutToClipboard;
             end;
        end;
end;

(* -------------------------------------- *)

procedure TFormEditMetaData.onKeyPressIPAddress(Sender: TObject; var Key: Char);

begin
     if not (key in [#8, #9, '0'..'9','a'..'f','A'..'F', '.', ':']) then
        key:=#0;
end;

(* -------------------------------------- *)

procedure TFormEditMetaData.ValidateInputfields;

var boOkay : boolean;
    help   : integer;

begin
     boOkay := True;

     case ComboboxIPVersion.ItemIndex of
      0 : begin
               // IPv4

               if not isValidIPv4String (EditSourceIP.Text) then
                  begin
                       EditSourceIP.Font.Color := clRed;
                       boOkay := False;
                  end
               else
                   EditSourceIP.Font.Color := clWindowText;

               if not isValidIPv4String (EditDestinationIP.Text) then
                  begin
                       EditDestinationIP.Font.Color := clRed;
                       boOkay := False;
                  end
               else
                   EditDestinationIP.Font.Color := clWindowText;
          end;
      1 : begin
               // IPv6

               if not isValidIPv6String (EditSourceIP.Text) then
                  begin
                       EditSourceIP.Font.Color := clRed;
                       boOkay := False;
                  end
               else
                   EditSourceIP.Font.Color := clWindowText;

               if not isValidIPv6String (EditDestinationIP.Text) then
                  begin
                       EditDestinationIP.Font.Color := clRed;
                       boOkay := False;
                  end
               else
                   EditDestinationIP.Font.Color := clWindowText;
          end;
     end;

     case ComboboxL4Protocol.ItemIndex of
      0,1,2 : begin
                   // Any

                   TryStrToInt (EditSourcePort.Text, help);

                   if (help = 0) or (help > 65535) then
                      begin
                           boOkay := False;
                           EditSourcePort.Font.Color := clRed;
                      end
                   else
                       EditSourcePort.Font.Color := clWindowText;

                   TryStrToInt (EditDestinationPort.Text, help);

                   if (help = 0) or (help > 65535) then
                      begin
                           boOkay := False;
                           EditDestinationPort.Font.Color := clRed;
                      end
                   else
                       EditDestinationPort.Font.Color := clWindowText;
              end;
     end;

     ButtonOkay.Enabled := boOkay;
end;

(* -------------------------------------- *)

procedure TFormEditMetaData.SetDataEntry(const value: tMetaDataEntry);

begin
     case value.IPVersion of
      4 : begin
               ComboboxIPVersion.ItemIndex := 0;

               EditSourceIP.Text := value.IPv4SocketPair.Source.IPAddress.ValueAsString;
               EditSourcePort.Text := IntToStr (value.IPv4SocketPair.Source.Port);
               EditDestinationIP.Text := value.IPv4SocketPair.Destination.IPAddress.ValueAsString;
               EditDestinationPort.Text := IntToStr (value.IPv4SocketPair.Destination.Port);

               LabelIPID.Enabled := true;
               EditIPID.Enabled := true;

               EditIPID.Text := IntToStr (value.IPID);
          end;
      6 : begin
               ComboboxIPVersion.ItemIndex := 1;

               EditSourceIP.Text := value.IPv6SocketPair.Source.IPAddress.ValueAsString;
               EditSourcePort.Text := IntToStr (value.IPv6SocketPair.Source.Port);
               EditDestinationIP.Text := value.IPv6SocketPair.Destination.IPAddress.ValueAsString;
               EditDestinationPort.Text := IntToStr (value.IPv6SocketPair.Destination.Port);

               LabelIPID.Enabled := false;
               EditIPID.Enabled := false;

               EditIPID.Text := '';
          end;
     end;

     case value.Layer4Protocol of
      l4Any  : begin
                    ComboBoxL4Protocol.ItemIndex := 0;

                    LabelSourcePort.Enabled := true;
                    EditSourcePort.Enabled := true;
                    LabelDestinationPort.Enabled := true;
                    EditDestinationPort.Enabled := true;
                    LabelTCPSequence.Enabled := true;
                    EditTCPSequence.Enabled := true;

                    EditTCPSequence.Text := IntToStr (value.TCPSequenceNumber);
               end;
      l4TCP  : begin
                    ComboBoxL4Protocol.ItemIndex := 1;

                    LabelSourcePort.Enabled := true;
                    EditSourcePort.Enabled := true;
                    LabelDestinationPort.Enabled := true;
                    EditDestinationPort.Enabled := true;
                    LabelTCPSequence.Enabled := true;
                    EditTCPSequence.Enabled := true;

                    EditTCPSequence.Text := IntToStr (value.TCPSequenceNumber);
               end;
      l4UDP  : begin
                    ComboBoxL4Protocol.ItemIndex := 2;

                    LabelSourcePort.Enabled := true;
                    EditSourcePort.Enabled := true;
                    LabelDestinationPort.Enabled := true;
                    EditDestinationPort.Enabled := true;
                    LabelTCPSequence.Enabled := false;
                    EditTCPSequence.Enabled := false;

                    EditTCPSequence.Text := '';
               end;
      l4ICMP : begin
                    ComboBoxL4Protocol.ItemIndex := 3;

                    EditSourcePort.Text := '';
                    EditDestinationPort.Text := '';
                    EditTCPSequence.Text := ''
               end;
     end;

     EditFrameComment.Text := value.RuleDetails.Comment;

     ValidateInputfields;
end;

(* -------------------------------------- *)

procedure TFormEditMetaData.UpdateInputfieldStatus;
begin
     case ComboboxIPVersion.ItemIndex of
      0 : begin
               // IPv4

               LabelIPID.Enabled := true;
               EditIPID.Enabled := true;
          end;
      1 : begin
               // IPv6

               LabelIPID.Enabled := false;
               EditIPID.Enabled := false;
          end;
     end;

     case ComboboxL4Protocol.ItemIndex of
      0 : begin
               // Any

               LabelSourcePort.Enabled := true;
               EditSourcePort.Enabled := true;
               LabelDestinationPort.Enabled := true;
               EditDestinationPort.Enabled := true;
               LabelTCPSequence.Enabled := true;
               EditTCPSequence.Enabled := true;

          end;
      1 : begin
               // TCP

               LabelSourcePort.Enabled := true;
               EditSourcePort.Enabled := true;
               LabelDestinationPort.Enabled := true;
               EditDestinationPort.Enabled := true;
               LabelTCPSequence.Enabled := true;
               EditTCPSequence.Enabled := true;

          end;
      2 : begin
               // UDP

               LabelSourcePort.Enabled := true;
               EditSourcePort.Enabled := true;
               LabelDestinationPort.Enabled := true;
               EditDestinationPort.Enabled := true;
               LabelTCPSequence.Enabled := false;
               EditTCPSequence.Enabled := false;

          end;
      3 : begin
               // ICMP

               LabelSourcePort.Enabled := false;
               EditSourcePort.Enabled := false;
               LabelDestinationPort.Enabled := false;
               EditDestinationPort.Enabled := false;
               LabelTCPSequence.Enabled := false;
               EditTCPSequence.Enabled := false;
          end;
     end;

     ValidateInputfields;
end;

(* -------------------------------------- *)

procedure TFormEditMetaData.Clear;

begin
     ComboboxIPVersion.ItemIndex := 0;

     EditSourceIP.Text := '';
     EditSourcePort.Text := '';
     EditDestinationIP.Text := '';
     EditDestinationPort.Text := '';
     EditSourcePort.Text := '';
     EditDestinationPort.Text := '';

     EditIPID.Text := '';
     EditTCPSequence.Text := '';

     UpdateInputFieldStatus;
end;

(* -------------------------------------- *)

end.
