{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit dialog_import_filter_from_file;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ComCtrls, Vcl.ExtCtrls,

  class_parse_csvfilterfile;

type tImportFileType = (iftUnknown, iftBeacon, iftSnortAlert, iftCSV);

     tImportOptionSet = record
                              IgnoreSYN          : boolean;
                              IgnoreICMP         : boolean;
                              AutoImportMetaData : boolean;
                              FilterLabel        : string;
                              CSVParseMode       : tCSVFilterParseMode;
                              CSVSeparator       : string;
                              SnortRuleFiles     : string;

                              procedure initialize;
                        end;

     TFormImportFilterFromFile = class (TForm)

                                        LabelFilename: TLabel;
                                        EditFilename: TEdit;
                                        ButtonSelectFile: TButton;
                                        ButtonOkay: TButton;
                                        ButtonCancel: TButton;
                                        OpenDialog: TOpenDialog;
                                        PageControlDetails: TPageControl;
                                        TabSheetBeaconFile: TTabSheet;
                                        TabSheetSnortAlert: TTabSheet;
                                        GroupBoxBeaconFile: TGroupBox;
                                        GroupBoxSnortAlertOptions: TGroupBox;
                                        TabSheetHint: TTabSheet;
                                        MemoMainStartHint: TMemo;
                                        CheckBoxSnortIgnoreSYN: TCheckBox;
                                        CheckBoxSnortAutoAddMetaData: TCheckBox;
                                        CheckBoxBeaconIgnoreSYN: TCheckBox;
                                        CheckBoxSnortIgnoreICMP: TCheckBox;
                                        CheckBoxBeaconIgnoreICMP: TCheckBox;
                                        LabeledEditFilterLabelTemplate: TLabeledEdit;
                                        TabSheetCSV: TTabSheet;
                                        GroupBox1: TGroupBox;
                                        LabelCSVLineFormat: TLabel;
                                        LabelCSVSeparator: TLabel;
                                        Label3: TLabel;
                                        MemoCSVRequiredLineFormat: TMemo;
                                        ComboBoxCSVFormat: TComboBox;
                                        EditCSVSeparator: TEdit;
                                        MemoFilterLabelHint: TMemo;
                                        LabeledEditRuleFile: TLabeledEdit;
                                        ButtonSelectSnortRuleFiles: TButton;
                                        OpenDialogSnortRules: TOpenDialog;

                                        procedure onShow(Sender: TObject);
                                        procedure ButtonSelectFileClick(Sender: TObject);
                                        procedure ButtonOkayClick(Sender: TObject);
                                        procedure ButtonCancelClick(Sender: TObject);
                                        procedure onChangeFilename(Sender: TObject);
                                        procedure onClickSnortAlertSettings(Sender: TObject);
                                        procedure onChangeCSVLineFormat(Sender: TObject);
                                        procedure ButtonSelectSnortRuleFilesClick(Sender: TObject);
    procedure onCreate(Sender: TObject);

                                        private
                                               priv_ImportFileType : tImportFileType;
                                               priv_optionSet      : tImportOptionSet;

                                               procedure CheckFileType (const Filename : string);

                                        public
                                              property ImportFileType : tImportFileType read priv_ImportFileType;
                                              property OptionSet : tImportOptionSet read priv_optionSet;

                                    end;

var
  FormImportFilterFromFile: TFormImportFilterFromFile;

implementation

{$R *.dfm}

uses base_structures,
     tracefile_utils,
     class_beaconfile,
     class_parse_snortalertlog;


(* ------------------------------------------- *)
{ tImportOptionSet }
(* ------------------------------------------- *)

procedure tImportOptionSet.initialize;

begin
     IgnoreSYN := false;
     IgnoreICMP := false;
     AutoImportMetaData := true;
     FilterLabel := DefaultSnortLabelTemplate;
     CSVParseMode := cpmNone;
     CSVSeparator := ',';
     SnortRuleFiles := '';
end;

(* ------------------------------------------- *)

procedure TFormImportFilterFromFile.onCreate(Sender: TObject);

begin
     // Workaround for scaling problems: look for /noscale command line option

     self.Scaled := FindCmdLineSwitch ('noscale');
end;

(* ------------------------------------------- *)

procedure TFormImportFilterFromFile.onChangeCSVLineFormat (Sender: TObject);

var header : string;
    line1  : string;
    line2  : string;

begin
     case ComboBoxCSVFormat.ItemIndex of
      0 : begin
               // IP Address

               LabelCSVSeparator.Enabled := False;

               header := 'IPAddress';
               line1 := '192.168.0.1';
               line2 := '2001:db8::1';
          end;
      1 : begin
               // IP Address Pair

               LabelCSVSeparator.Enabled := true;

               header := 'IPAddress1,IPAddress2';
               line1 := '192.168.0.1,10.0.0.10';
               line2 := '2001:db8::1,2001:db8::2';
          end;
      2 : begin
               // Socket

               LabelCSVSeparator.Enabled := true;

               header := 'IPAddress,Port';
               line1 := '192.168.0.1,1025';
               line2 := '2001:db8::1,1025';
          end;
      3 : begin
               // Socket Pair

               LabelCSVSeparator.Enabled := true;

               header := 'IPAddress1,Port,IPAddress2,Port';
               line1 := '192.168.0.1,1025,10.0.0.10,80';
               line2 := '2001:db8::1,1025,2001:db8::2,80';
          end;
      4 : begin
               // 5-Tuple

               LabelCSVSeparator.Enabled := true;

               header := 'IPAddress1,Port,IPAddress2,Port,Protocol';
               line1 := '192.168.0.1,1025,10.0.0.10,80,udp';
               line2 := '2001:db8::1,1025,2001:db8::2,80,tcp';
          end;
     end;

     header := StringReplace (header, ',', EditCSVSeparator.Text, [rfReplaceAll]);
     line1 := StringReplace (line1, ',', EditCSVSeparator.Text, [rfReplaceAll]);
     line2 := StringReplace (line2, ',', EditCSVSeparator.Text, [rfReplaceAll]);

     MemoCSVRequiredLineFormat.Clear;
     MemoCSVRequiredLineFormat.Lines.Add (header);
     MemoCSVRequiredLineFormat.Lines.Add (line1);
     MemoCSVRequiredLineFormat.Lines.Add (line2);

     EditCSVSeparator.Enabled := LabelCSVSeparator.Enabled;
end;

(* ------------------------------------------- *)

procedure TFormImportFilterFromFile.onChangeFilename (Sender: TObject);

begin
     // TODO: check if file is valid
end;

(* ------------------------------------------- *)

procedure TFormImportFilterFromFile.CheckFileType (const Filename: string);

var ext         : string;
    SnortParser : tSnortAlertParser;
    CSVParser   : tCSVFilterFile;
    CSVMode     : tCSVFilterParseMode;

begin
     ext := extractFileExt (Filename);

     if getTracefileType (Filename) = cfUnknown then
        begin
             if (ext = '.csv') then
                begin
                     priv_importfiletype := iftCSV;

                     PageControlDetails.ActivePageIndex := 3;

                     CSVMode := CSVParser.getFilterParseMode (Filename);

                     case CSVMode of
                      cpmNone,
                      cpmSingleIP     : ComboBoxCSVFormat.ItemIndex := 0;
                      cpmIPPair       : ComboBoxCSVFormat.ItemIndex := 1;
                      cpmSingleSocket : ComboBoxCSVFormat.ItemIndex := 2;
                      cpmSocketPair   : ComboBoxCSVFormat.ItemIndex := 3;
                      cpm5Tuple       : ComboBoxCSVFormat.ItemIndex := 4;
                     end;

                     // force update of the hint

                     onChangeCSVLineFormat (nil);
                end
             else
                 begin
                      SnortParser := tSnortAlertParser.create (Filename);

                      try
                         if SnortParser.isValidFile then
                            begin
                                 priv_importfiletype := iftSnortAlert;

                                 PageControlDetails.ActivePageIndex := 2;
                            end
                      finally
                             SnortParser.Free;
                      end;
                 end;

{
             if (ext = '.ids') then
                begin
                     SnortParser := tSnortAlertParser.create (Filename);

                     try
                        if SnortParser.isValidFile then
                           begin
                                priv_importfiletype := iftSnortAlert;

                                PageControlDetails.ActivePageIndex := 2;
                           end
                     finally
                            SnortParser.Free;
                     end;
                end;
}
        end
     else
         begin
              priv_importfiletype := iftBeacon;

              PageControlDetails.ActivePageIndex := 1;
         end;
end;

(* ------------------------------------------- *)

procedure TFormImportFilterFromFile.ButtonSelectFileClick (Sender: TObject);

begin
     if OpenDialog.Execute then
        begin
             EditFilename.Text := Opendialog.FileName;

             CheckFileType (OpenDialog.FileName);
        end;
end;

(* ------------------------------------------- *)

procedure TFormImportFilterFromFile.ButtonSelectSnortRuleFilesClick (Sender: TObject);

begin
     if OpenDialogSnortRules.Execute then
        begin
             LabeledEditRuleFile.Text := OpenDialogSnortRules.Files.CommaText;

        end;
end;

(* ------------------------------------------- *)

procedure TFormImportFilterFromFile.onShow (Sender: TObject);

var i : integer;

begin
     priv_importfiletype := iftUnknown;

     for i := 1 to PageControlDetails.PageCount do
         PageControlDetails.Pages[i-1].TabVisible := false;

     EditFilename.Clear;
     PageControlDetails.ActivePageIndex := 0;
end;

(* ------------------------------------------- *)

procedure TFormImportFilterFromFile.ButtonCancelClick(Sender: TObject);

begin
     priv_optionSet.initialize;
end;

(* ------------------------------------------- *)

procedure TFormImportFilterFromFile.ButtonOkayClick(Sender: TObject);

begin
     priv_optionSet.initialize;

     case priv_ImportFileType of
      iftBeacon     : begin
                           priv_optionSet.IgnoreSYN := CheckBoxBeaconIgnoreSYN.Checked;
                           priv_optionSet.IgnoreICMP := CheckBoxBeaconIgnoreICMP.Checked;
                      end;
      iftSnortAlert : begin
                           priv_optionSet.IgnoreSYN := CheckBoxSnortIgnoreSYN.Checked;
                           priv_optionSet.IgnoreICMP := CheckBoxSnortIgnoreICMP.Checked;
                           priv_optionSet.AutoImportMetaData := CheckBoxSnortAutoAddMetaData.Checked;

                           if LabeledEditFilterLabelTemplate.Enabled then
                              priv_optionSet.FilterLabel := LabeledEditFilterLabelTemplate.Text
                           else
                               priv_optionSet.FilterLabel :=  '';

                           priv_OptionSet.SnortRuleFiles := LabeledEditRuleFile.Text;
                      end;
      iftCSV        : begin
                           case ComboBoxCSVFormat.ItemIndex of
                            0 : priv_optionSet.CSVParseMode := cpmSingleIP;
                            1 : priv_optionSet.CSVParseMode := cpmIPPair;
                            2 : priv_optionSet.CSVParseMode := cpmSingleSocket;
                            3 : priv_optionSet.CSVParseMode := cpmSocketPair;
                            4 : priv_optionSet.CSVParseMode := cpm5Tuple;
                           end;
                      end;
     end;
end;

(* ------------------------------------------- *)

procedure TFormImportFilterFromFile.onClickSnortAlertSettings (Sender: TObject);

begin
     LabeledEditFilterLabelTemplate.Enabled := CheckBoxSnortAutoAddMetaData.Enabled and CheckBoxSnortAutoAddMetaData.Checked;
end;

(* ------------------------------------------- *)


end.
