{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}
 
unit form_preferences;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ComCtrls, Vcl.StdCtrls,
  Vcl.Grids, Vcl.ValEdit,

  base_structures,
  class_formstorage;

type
  TFormPreferences = class(TForm)
    PageControlPreferences: TPageControl;
    ButtonOkay: TButton;
    ButtonCancel: TButton;
    TabSheetAdvanced: TTabSheet;
    ValueListEditorAdvancedSettings: TValueListEditor;
    TabSheetStartup: TTabSheet;
    GroupBoxUpdate: TGroupBox;
    CheckBoxAutomaticCheckForUpdate: TCheckBox;
    LabelProxyServer: TLabel;
    LabelProxyPort: TLabel;
    EditProxyAddress: TEdit;
    EditProxyPort: TEdit;
    LabelProxyUser: TLabel;
    EditProxyUser: TEdit;
    LabelProxyPassword: TLabel;
    EditProxyPassword: TEdit;
    CheckBoxDoNotWarnIfNoConnection: TCheckBox;
    TabSheetFileHandling: TTabSheet;
    GroupBoxFileSettings: TGroupBox;
    CheckBoxWarnUnknownFileFormat: TCheckBox;
    CheckBoxWarnWhenAddingDirectories: TCheckBox;
    TabSheetDirectories: TTabSheet;
    GroupBox1: TGroupBox;
    LabelPathTaskFiles: TLabel;
    EditPathTaskFiles: TEdit;
    ButtonSelectTaskSettingsPath: TButton;
    CheckBoxWarnAlsoIfAddingFromDirectory: TCheckBox;
    Label1: TLabel;
    CheckBoxIncludeSubdirectories: TCheckBox;
    CheckBoxSkipWarningForCommandLine: TCheckBox;
    RadioButtonDirectConnetion: TRadioButton;
    RadioButtonUseProxyServer: TRadioButton;
    RadioButtonUseSystemProxy: TRadioButton;
    LabelProxyCredentials: TLabel;
    LabelProxyStatus: TLabel;
    TabSheetAutoScan: TTabSheet;
    GroupBoxAutoScanSettings: TGroupBox;
    LabelFileSizeThreshold: TLabel;
    LabelFileSizeThresholdMegabytes: TLabel;
    EditFileScanThreshold: TEdit;
    CheckBoxUseIntelDB: TCheckBox;
    CheckBoxMarkIntelDBFileInvisible: TCheckBox;
    CheckBoxDoNotWarnAboutScanThreshold: TCheckBox;
    CheckBoxSaveEssentialsToFile: TCheckBox;
    LabelTraceIntelDBFilename: TLabel;
    EditTraceIntelDBFilename: TEdit;
    RadioButtonUsePerDirectoryDatabase: TRadioButton;
    RadioButtonUseGlobalPathDatabases: TRadioButton;
    LabelTraceIntelGlobalDBPath: TLabel;
    EditGlobalIntelDBPath: TEdit;
    ButtonSelectGlobalIntelDBPath: TButton;
    procedure onShowFormPreferences(Sender: TObject);
    procedure onGetPicklistAdvancedSettings(Sender: TObject;
      const KeyName: string; Values: TStrings);
    procedure onCreateFormPreferences(Sender: TObject);
    procedure onKeyPressAdvancedSettings(Sender: TObject; var Key: Char);
    procedure onChangeStartupSettings(Sender: TObject);
    procedure ButtonSelectTaskSettingsPathClick(Sender: TObject);
    procedure onChangeFileSettings(Sender: TObject);
    procedure onChangeTaskSettingsDirectory(Sender: TObject);
    procedure onChangeScanSettings(Sender: TObject);
    procedure onKeyPressDBFilename(Sender: TObject; var Key: Char);
    procedure onExitDBFilename(Sender: TObject);
    procedure ButtonSelectGlobalIntelDBPathClick(Sender: TObject);
    procedure onKeyPressScanThreshold(Sender: TObject; var Key: Char);
  private
    { Private declarations }

         StorMan            : tFormStorageManager;

         priv_preffilename  : string;

         function BrowseDialog (const Title: string; const Flag: integer): string;
  public
    { Public declarations }

    function getAdvancedBoolSetting (const Key : string) : boolean; overload;
    function getAdvancedIntSetting (const Key : string) : integer; overload;

    procedure LoadPreferences (const Filename : string);
    procedure SavePreferences (const Filename : string);

    function HasTCPColorTable : boolean;
    function RetrieveConversationListSettings : tConversationDialogSettings;
    procedure StoreConversationListSettings (value : tConversationDialogSettings);


  end;

var
  FormPreferences: TFormPreferences;

const TracebusterDownloadURL = 'https://www.tracewrangler.com/download/tracewrangler.zip';
      TracebusterVersionURL = 'https://www.tracewrangler.com/download/version.xml?';


implementation

{$R *.dfm}

uses ShlObj,
     proxy_tools,
     Toolkit,
     definition_conversation_types;

(* ------------------------------------------ *)

function TFormPreferences.BrowseDialog (const Title: string; const Flag: integer): string;

var lpItemID    : PItemIDList;
    BrowseInfo  : TBrowseInfo;
    DisplayName : array[0..MAX_PATH] of char;
    TempPath    : array[0..MAX_PATH] of char;

begin
     Result:='';
     FillChar(BrowseInfo, sizeof(TBrowseInfo), #0);

     with BrowseInfo do
          begin
               hwndOwner := Application.Handle;
               pszDisplayName := @DisplayName;
               lpszTitle := PChar(Title);
               ulFlags := Flag;
          end;

  lpItemID := SHBrowseForFolder(BrowseInfo);

  if lpItemId <> nil then
     begin
          SHGetPathFromIDList(lpItemID, TempPath);
          Result := TempPath;
          GlobalFreePtr(lpItemID);
     end;
end;

(* ------------------------------------------- *)

procedure TFormPreferences.onCreateFormPreferences(Sender: TObject);

begin
     // Workaround for scaling problems: look for /noscale command line option

     self.Scaled := FindCmdLineSwitch ('noscale');

     // Prepare the storage manager and other objects we need for storage

     StorMan := tFormStorageManager.create (self);

     priv_preffilename := '';
end;

(* ------------------------------------------- *)

procedure TFormPreferences.onShowFormPreferences(Sender: TObject);

begin
     LabelProxyStatus.Caption := '';

     (ValueListEditorAdvancedSettings.Strings as tStringlist).Sort;

     PageControlPreferences.ActivePageIndex := 0;
end;

(* ------------------------------------------- *)

procedure TFormPreferences.LoadPreferences (const Filename: string);

begin
     priv_preffilename := Filename;

     StorMan.retrieveFormFromDB (Filename);

     // Update visual dependencies; this is not so elegant but should work

     onChangeFileSettings (nil);
     onChangeStartupSettings (nil);
end;

(* ------------------------------------------- *)

procedure TFormPreferences.SavePreferences (const Filename: string);

begin
     StorMan.storeFormToDB (Filename);
end;


(* ------------------------------------------- *)

procedure TFormPreferences.ButtonSelectTaskSettingsPathClick (Sender: TObject);

var sPath : string;

begin
     sPath := BrowseDialog ('Select Path', BIF_RETURNONLYFSDIRS);

     if DirectoryExists (sPath) then
        EditPathTaskFiles.Text := sPath;
end;

(* ------------------------------------------- *)

procedure TFormPreferences.ButtonSelectGlobalIntelDBPathClick (Sender: TObject);

var sPath : string;

begin
     sPath := BrowseDialog ('Select Path', BIF_RETURNONLYFSDIRS);

     if DirectoryExists (sPath) then
        EditGlobalIntelDBPath.Text := sPath;
end;

(* ------------------------------------------- *)

function TFormPreferences.getAdvancedBoolSetting(const Key: string): boolean;

var value : string;

begin
     value := (ValueListEditorAdvancedSettings.Strings as tStringList).Values[Key];

     result := lowercase (value) = 'true';
end;

(* ------------------------------------------- *)

function TFormPreferences.getAdvancedIntSetting(const Key: string): integer;

var value  : string;
    number : integer;

begin
     value := (ValueListEditorAdvancedSettings.Strings as tStringList).Values[Key];

     TryStrToInt (value, number);
     result := number;
end;

(* ------------------------------------------- *)

procedure TFormPreferences.onGetPicklistAdvancedSettings(Sender: TObject; const KeyName: string; Values: TStrings);

begin
     // Prepare true/false dropdown items for Advanced Setting List

     if pos ('Toggle', KeyName) > 0 then
        begin
             Values.Add('true');
             Values.Add('false');
        end;
end;

(* ------------------------------------------- *)

procedure TFormPreferences.onKeyPressAdvancedSettings(Sender: TObject; var Key: Char);

var Name : string;

begin
     Name := ValueListEditorAdvancedSettings.Strings.Names[ValueListEditorAdvancedSettings.Row-1];

     if pos ('Threshold', Name) > 0 then
        if not (Key in [#8, #9, '0' .. '9']) then
           Key := #0;
end;

(* ------------------------------------------- *)

procedure TFormPreferences.onKeyPressDBFilename(Sender: TObject; var Key: Char);

begin
     if not (key in [#8, #9, '.', '-', '_', '!', '0'..'9','a'..'z','A'..'Z']) then
        key:=#0;
end;

procedure TFormPreferences.onKeyPressScanThreshold(Sender: TObject; var Key: Char);

begin
     if not (key in [#8, #9, '0'..'9']) then
        key:=#0;
end;

(* ------------------------------------------- *)

procedure TFormPreferences.onExitDBFilename(Sender: TObject);

begin
     if pos ('\', EditTraceIntelDBFilename.Text) > 0 then
        EditTraceIntelDBFilename.Text := SanitizeFilename (getEntry (EditTraceIntelDBFilename.Text, '\', numEntries (EditTraceIntelDBFilename.Text, '\')));
end;

(* ------------------------------------------- *)

procedure TFormPreferences.onChangeFileSettings (Sender: TObject);

var help  : integer;
    dummy : integer;

begin
     CheckBoxWarnAlsoIfAddingFromDirectory.Enabled := CheckBoxWarnUnknownFileFormat.Checked;

     CheckBoxSkipWarningForCommandLine.Enabled := CheckBoxWarnWhenAddingDirectories.Checked;
     val (EditFileScanThreshold.Text, help, dummy);

     CheckBoxDoNotWarnAboutScanThreshold.Enabled := (help > 0);
end;

(* ------------------------------------------- *)

procedure TFormPreferences.onChangeScanSettings(Sender: TObject);

begin
     RadioButtonUsePerDirectoryDatabase.Enabled := CheckBoxUseIntelDB.Checked;
     LabelTraceIntelDBFilename.Enabled := RadioButtonUsePerDirectoryDatabase.Enabled and RadioButtonUsePerDirectoryDatabase.Checked;
     EditTraceIntelDBFilename.Enabled := LabelTraceIntelDBFilename.Enabled;
     CheckBoxMarkIntelDBFileInvisible.Enabled := LabelTraceIntelDBFilename.Enabled;

     RadioButtonUseGlobalPathDatabases.Enabled := CheckBoxUseIntelDB.Checked;
     LabelTraceIntelGlobalDBPath.Enabled := RadioButtonUseGlobalPathDatabases.Enabled and RadioButtonUseGlobalPathDatabases.Checked;
     EditGlobalIntelDBPath.Enabled := LabelTraceIntelGlobalDBPath.Enabled;
end;

(* ------------------------------------------- *)

procedure TFormPreferences.onChangeStartupSettings (Sender: TObject);

var TestResult : DWORD;
    ProxyInfo  : TProxyInfo;

begin
     LabelProxyServer.Enabled := RadioButtonUseProxyServer.Checked;
     EditProxyAddress.Enabled := RadioButtonUseProxyServer.Checked;
     LabelProxyPort.Enabled := RadioButtonUseProxyServer.Checked;
     EditProxyPort.Enabled := RadioButtonUseProxyServer.Checked;

     LabelProxyCredentials.Enabled := RadioButtonUseProxyServer.Checked or RadioButtonUseSystemProxy.Checked;

     LabelProxyUser.Enabled := LabelProxyCredentials.Enabled;
     EditProxyUser.Enabled := LabelProxyCredentials.Enabled;
     LabelProxyPassword.Enabled := LabelProxyCredentials.Enabled;
     EditProxyPassword.Enabled := LabelProxyCredentials.Enabled;

     CheckBoxDoNotWarnIfNoConnection.Enabled := CheckBoxAutomaticCheckForUpdate.Checked;

     // Do Proxy check if system settings are used

     if RadioButtonUseSystemProxy.Checked then
        begin
             TestResult := GetProxyInfo (TracebusterVersionURL, ProxyInfo);

             case TestResult of
               0: begin
                       LabelProxyStatus.Caption := 'Proxy autodetected: ' + BoolToStr(ProxyInfo.ProxyAutoDetected, True);

                       if pos (':', ProxyInfo.ProxyURL) > 0 then
                          begin
                               EditProxyAddress.Text := getEntry (ProxyInfo.ProxyURL, ':', 1);
                               EditProxyPort.Text := getEntry (ProxyInfo.ProxyURL, ':', 2);
                          end
                       else
                           begin
                                EditProxyAddress.Text := ProxyInfo.ProxyURL;
                                LabelProxyStatus.Caption := 'Unable to determine proxy port';
                           end;
                  end;
               12166: LabelProxyStatus.Caption := 'Error in proxy auto-config script code';
               12167: LabelProxyStatus.Caption := 'Unable to download proxy auto-config script';
               12180: LabelProxyStatus.Caption := 'WPAD detection failed';
             else
                 LabelProxyStatus.Caption := 'Last error: ' + IntToStr (TestResult);
             end;
        end
     else
         LabelProxyStatus.Caption := '';
end;

(* ------------------------------------------- *)

procedure TFormPreferences.onChangeTaskSettingsDirectory(Sender: TObject);

begin
     if DirectoryExists (EditPathTaskFiles.Text) then
        EditPathTaskFiles.Font.Color := clWindowText
     else
         EditPathTaskFiles.Font.Color := clRed;
end;

(* ------------------------------------------- *)

function TFormPreferences.HasTCPColorTable : boolean;

var wasOpen : boolean;

begin
     StorMan.DatabaseFile := priv_preffilename;

     wasOpen := StorMan.Database.Connected;

     try
        if not wasOpen then
           StorMan.Database.Open;

        result := (StorMan.retrieveValueFroMDB ('ColorTCPHandshakeNone') <> '');
     finally
            if not wasOpen then
               StorMan.Database.Close;
     end;
end;

(* ------------------------------------------- *)

function TFormPreferences.RetrieveConversationListSettings : tConversationDialogSettings;

var wasOpen : boolean;
    value   : tConversationDialogSettings;
    color   : string;
    help    : string;
    i       : integer;

begin
     for i := 0 to 8 do
         value.TCPColors[i] := clWindowText;

     if priv_preffilename <> '' then
        begin
             StorMan.DatabaseFile := priv_preffilename;

             wasOpen := StorMan.Database.Connected;

             if not wasOpen then
                StorMan.Database.Open;

             color := StorMan.retrieveValueFroMDB ('ColorTCPHandshakeNone');

             if color <> '' then
                value.TCPColors[TCPHandshakeNone] := StrToInt ('$' + color);

             color := StorMan.retrieveValueFroMDB ('ColorTCPHandshakeSYNSent');

             if color <> '' then
                value.TCPColors[TCPHandshakeSYNSent] := StrToInt ('$' + color);

             color := StorMan.retrieveValueFroMDB ('ColorTCPHandshakeSYNACKSent');

             if color <> '' then
                value.TCPColors[TCPHandshakeSYNACKSent] := StrToInt ('$' + color);

             color := StorMan.retrieveValueFroMDB ('ColorTCPHandshakeSYNTwoWay');

             if color <> '' then
                value.TCPColors[TCPHandshakeSYNTwoWay] := StrToInt ('$' + color);

             color := StorMan.retrieveValueFroMDB ('ColorTCPHandshakeHalfOpen');

             if color <> '' then
                value.TCPColors[TCPHandshakeHalfOpen] := StrToInt ('$' + color);

             color := StorMan.retrieveValueFroMDB ('ColorTCPHandshakeSYNReset');

             if color <> '' then
                value.TCPColors[TCPHandshakeSYNReset] := StrToInt ('$' + color);

             color := StorMan.retrieveValueFroMDB ('ColorTCPHandshakeComplete');

             if color <> '' then
                value.TCPColors[TCPHandshakeComplete] := StrToInt ('$' + color);

             color := StorMan.retrieveValueFroMDB ('ColorTCPHandshakeAndTeardown');

             if color <> '' then
                value.TCPColors[TCPHandshakeAndTeardown] := StrToInt ('$' + color);

             color := StorMan.retrieveValueFroMDB ('ColorTCPTeardown');

             if color <> '' then
                value.TCPColors[TCPTeardown] := StrToInt ('$' + color);

             color := StorMan.retrieveValueFroMDB ('ColorTCPHalfOpenReset');

             if color <> '' then
                value.TCPColors[TCPHalfOpenReset] := StrToInt ('$' + color)
             else
                 value.TCPColors[TCPHalfOpenReset] := $000000ff;

             color := StorMan.retrieveValueFroMDB ('ColorTCPSocketRefused');

             if color <> '' then
                value.TCPColors[TCPSocketRefused] := StrToInt ('$' + color)
             else
                 value.TCPColors[TCPSocketRefused] := $000000ff;

             color := StorMan.retrieveValueFroMDB ('ColorTCPSocketCloseByInactivity');

             if color <> '' then
                value.TCPColors[TCPSocketCloseByInactivity] := StrToInt ('$' + color)
             else
                 value.TCPColors[TCPSocketCloseByInactivity] := $000045ff;

             help := StorMan.retrieveValueFroMDB ('ConversationsInitialPage');
             tryStrToInt (help, value.InitialPage);

             help := StorMan.retrieveValueFroMDB ('ConversationsKeepPage');
             value.RememberPage := (help = '1');

             help := StorMan.retrieveValueFroMDB ('ClearFilterOnPageChange');
             value.ClearFilterOnChange := (help = '1');

             if not wasOpen then
                StorMan.Database.Close;
        end;

     result := value;
end;

(* ------------------------------------------- *)

procedure TFormPreferences.StoreConversationListSettings (value : tConversationDialogSettings);

var wasOpen : boolean;

begin
     if priv_preffilename <> '' then
        begin
             StorMan.DatabaseFile := priv_preffilename;

             wasOpen := StorMan.Database.Connected;

             if not wasOpen then
                StorMan.Database.Open;

             StorMan.storeNameValueToDB ('ColorTCPHandshakeNone', IntToHex (value.TCPColors[TCPHandshakeNone], 8));
             StorMan.storeNameValueToDB ('ColorTCPHandshakeSYNSent', IntToHex (value.TCPColors[TCPHandshakeSYNSent], 8));
             StorMan.storeNameValueToDB ('ColorTCPHandshakeSYNACKSent', IntToHex (value.TCPColors[TCPHandshakeSYNACKSent], 8));
             StorMan.storeNameValueToDB ('ColorTCPHandshakeSYNTwoWay', IntToHex (value.TCPColors[TCPHandshakeSYNTwoWay], 8));
             StorMan.storeNameValueToDB ('ColorTCPHandshakeHalfOpen', IntToHex (value.TCPColors[TCPHandshakeHalfOpen], 8));
             StorMan.storeNameValueToDB ('ColorTCPHandshakeSYNReset', IntToHex (value.TCPColors[TCPHandshakeSYNReset], 8));
             StorMan.storeNameValueToDB ('ColorTCPHandshakeComplete', IntToHex (value.TCPColors[TCPHandshakeComplete], 8));
             StorMan.storeNameValueToDB ('ColorTCPHandshakeAndTeardown', IntToHex (value.TCPColors[TCPHandshakeAndTeardown], 8));
             StorMan.storeNameValueToDB ('ColorTCPTeardown', IntToHex (value.TCPColors[TCPTeardown], 8));
             StorMan.storeNameValueToDB ('ColorTCPHalfOpenReset', IntToHex (value.TCPColors[TCPHalfOpenReset], 8));
             StorMan.storeNameValueToDB ('ColorTCPSocketRefused', IntToHex (value.TCPColors[TCPSocketRefused], 8));
             StorMan.storeNameValueToDB ('ColorTCPSocketCloseByInactivity', IntToHex (value.TCPColors[TCPSocketCloseByInactivity], 8));

             StorMan.storeNameValueToDB ('ConversationsInitialPage', IntToStr (value.InitialPage));

             if value.RememberPage then
                StorMan.storeNameValueToDB ('ConversationsKeepPage', '1')
             else
                 StorMan.storeNameValueToDB ('ConversationsKeepPage', '0');

             if value.ClearFilterOnChange then
                StorMan.storeNameValueToDB ('ClearFilterOnPageChange', '1')
             else
                 StorMan.storeNameValueToDB ('ClearFilterOnPageChange', '0');

             if not wasOpen then
                StorMan.Database.Close;
        end;
end;

(* ------------------------------------------- *)

end.
