{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_frameprocessor_juniper;

interface

uses base_structures,
     class_rawframe,
     class_frameprocessor_baseclass,
     class_parser_juniper;

type tFrameProcessorJuniper = class (tFrameProcessor)

                                    private
                                           priv_AddPseudoL2Header : tPseudoL2HeaderMode;

//                                         function getRawFrameFromFrameChunk (Frame : tRawFrame; PayLoad : tFrameChunk; FrameLengthModifier : integer; PacketDirection : tPacketDirection) : tRawFrame;
                                           function getRawFrameFromFrameChunk (Frame : tRawFrame; PayLoad : tFrameChunk; Parser : tJuniperParser) : tRawFrame;

                                    protected
                                             function getProcessedFrame (Frame : tRawFrame) : tRawFrame; override;

                                    public
                                          property AddPseudoL2Header : tPseudoL2HeaderMode read priv_AddPseudoL2Header write priv_AddPseudoL2Header;

                              end;

implementation

uses sysutils,
     class_parser_baseclass,
     common_parse_helpers,
     class_parser_ethernet,
     class_parsemaster,
     toolkit;

(* ------------------------------------------- *)
{ tFrameProcessorJuniper }
(* ------------------------------------------- *)

// function tFrameProcessorJuniper.getRawFrameFromFrameChunk (Frame: tRawFrame; PayLoad: tFrameChunk; FrameLengthModifier : integer; PacketDirection : tPacketDirection): tRawFrame;
function tFrameProcessorJuniper.getRawFrameFromFrameChunk (Frame: tRawFrame; PayLoad: tFrameChunk; Parser : tJuniperParser): tRawFrame;

var NewFrame     : tRawFrame;
    TimeDetails  : tTimingDetails;
    WireDetails  : tWireDetails;
    FilePosition : tFileFrameLocation;
    FrameBuffer  : tRawBuffer;
    FrameChunk   : tFrameChunk;
    L2Pseudo     : tFrameChunk;
    FinalLen     : integer;
    i            : integer;

begin
     TimeDetails.AbsoluteTime := Frame.Timestamp.datetime;
     TimeDetails.TraceTicks := Frame.Ticks;
     TimeDetails.NanoTicks := (Frame.Timestamp.nanoseconds mod 1000);            // because right now, Nanosecs also contain microsecs

     WireDetails.WireSize := Frame.WireSize - Parser.LayerSize;
     WireDetails.CaptureSize := Payload.Len;

     WireDetails.InterfaceID := Frame.InterfaceID;
     WireDetails.Dropped := Frame.DropCount;
     WireDetails.FrameFlags := Frame.FrameFlags;
     WireDetails.FrameFlags.SetPacketDirection (Parser.Direction);
     WireDetails.FrameType := Frame.FrameFileType;

     FilePosition.FileIndex := 0;
     FilePosition.FileFrameIndex := 0;
     FilePosition.FileOffset := 0;

     // Set the link layer type based on the next protocol following the Juniper header

     WireDetails.LinkType := GetLinkTypeForProtocolIndex (Parser.NextAnchor.HeaderType);

     // Add dummy Ethernet header if necessary

     if Parser.Layer2HeaderStripped then
        begin
             if priv_AddPseudoL2Header <> plmNone then
                begin
                     // Create zeroed Pseudo Ethernet header and patch the header
                     // type into the EtherType bytes

                     L2Pseudo.CreateZeroBuffer (14);
//                   L2Pseudo.Patch (12, swap16 (Parser.NextAnchor.HeaderType));        // according to tests with CAPWAP processor, swapping gives wrong results here
                     L2Pseudo.Patch (12, word (Parser.NextAnchor.HeaderType));

                     FrameChunk := mergeFrameChunks (L2Pseudo, Payload);

                     L2Pseudo.free;

                     FrameBuffer := FrameChunk.getAsRawBuffer;
                     FrameChunk.free;

                     // Force Linktype to Ethernet and increase the frame size

                     WireDetails.WireSize := WireDetails.WireSize + 14;
                     WireDetails.CaptureSize := WireDetails.CaptureSize + 14;

                     WireDetails.LinkType := LINKTYPE_ETHERNET;
                end
             else
                 begin
                      FrameBuffer := Payload.getAsRawBuffer;
                 end;
        end
     else
         begin
              FrameBuffer := Payload.getAsRawBuffer;
         end;

     NewFrame := tRawFrame.create (TimeDetails, WireDetails, FilePosition, FrameBuffer);

     for i := 1 to Frame.Commentlist.Count do
         NewFrame.Commentlist.Add (Frame.Commentlist[i-1]);

     if Parser.Layer2HeaderStripped and (priv_AddPseudoL2Header = plmWithFrameComment) then
        NewFrame.Commentlist.Add (COMMENT_PSEUDOL2HDR);

     result := NewFrame;
end;

(* ------------------------------------------- *)

function tFrameProcessorJuniper.getProcessedFrame (Frame: tRawFrame): tRawFrame;

var
//  ParseMaster : tParseMaster;
    NewFrame    : tRawFrame;
    HeaderType  : tHeaderType;
    JParser     : tJuniperParser;
    EthParser   : tEthernetParser;
    Chunk       : tFrameChunk;

begin
     // Process if it is a juniper frame

     if Frame.LinkType = LINKTYPE_JUNIPER then
        begin
             if not assigned (prot_ParseMaster) then
                prot_ParseMaster := tParseMaster.create(Frame, ppmParseAll);

             // Get the Juniper Header

             HeaderType := prot_ParseMaster.getParserTypeByIndex(1);
             JParser := prot_ParseMaster.getParserByIndex(1) as tJuniperParser;

             // Get the frame starting right after the Juniper Header

             Chunk := Frame.FrameDataAsChunk (JParser.NextAnchor.Offset);

             // Create

             if Chunk.Len > 0 then
                begin
                     // Create new Frame from existing frame but without the Juniper stuff

                     NewFrame := getRawFrameFromFrameChunk (Frame, Chunk, JParser);

                     // Discard original

                     Frame.free;
                     Frame := nil;

                     // Set frame to modified to force discarding the parsemaster object

                     prot_framewasmodified := true;
                end
             else
                 begin
                      if assigned (prot_OnProcessingError) then
                         prot_OnProcessingError (self, 'Error: parsing failed, cannot process frame #' + IntToStr (Frame.FileFrameID));

                      NewFrame := Frame;
                 end;

             // Discard Chunk

             Chunk.free;
        end
     else
         NewFrame := Frame;

     result := NewFrame;
end;

(* ------------------------------------------- *)

end.
