{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_frameprocessor_metaenhancer;

interface

uses System.Generics.Collections,
     base_structures,
     class_rawframe,
     class_tracewriter,
     class_simpleframestorage,
     class_frameprocessor_baseclass,
     class_parse_snortalertlog;

type tOnMetaInfoEnhance = procedure (Sender : tObject; const SetSize, MatchCount : integer; const Entry : tMetaDataEntry) of Object;

     tFrameProcessorMetaEnhancer = class (tFrameProcessor)

                                         constructor create (const Name : string; const MetaInfo : tMetaDataSet; const TraceWriter : tTraceWriter = nil; const MaxFrameCountBeforeWrite : integer = 10000);

                                         private
                                                priv_OnMetaInfoMatch    : tOnMetaInfoEnhance;

                                                priv_metadataset        : tDictionary<string, tMetaDataEntry>;

                                                priv_MetaInfoLastMatch  : tMetaDataEntry;
                                                priv_MetaInfoMatchCount : integer;

                                                procedure AddMetaDataSet (const DataSet : tMetaDataSet);
                                                procedure EnhanceFrame (Frame : tRawFrame);

                                                function getMetaDataCount : integer;

                                         protected

                                         public
                                               destructor Destroy; override;

                                               property MetaDataCount : integer read getMetaDataCount;

                                               property OnMetaInfoEnhance : tOnMetaInfoEnhance read priv_OnMetaInfoMatch write priv_OnMetaInfoMatch;

                                               function GetProcessedFrame (Frame : tRawFrame) : tRawFrame; override;
                                   end;


implementation

uses definition_tcp_structures,
     class_ParseMaster;

(* ------------------------------------------- *)
{ tFrameProcessorMetaEnhander }
(* ------------------------------------------- *)

constructor tFrameProcessorMetaEnhancer.create (const Name: string; const MetaInfo: tMetaDataSet; const TraceWriter: tTraceWriter; const MaxFrameCountBeforeWrite: integer);

begin
     inherited create (Name, TraceWriter, MaxFrameCountBeforeWrite);

     priv_metadataset := tDictionary<string, tMetaDataEntry>.create (length (MetaInfo));
     priv_MetaInfoLastMatch.initialize;
     priv_MetaInfoMatchCount := 0;

     // Add metadata to set

     AddMetaDataSet (MetaInfo);
end;

(* ------------------------------------------- *)

destructor tFrameProcessorMetaEnhancer.Destroy;

begin
     priv_metadataset.Free;

     inherited;
end;

(* ------------------------------------------- *)

function tFrameProcessorMetaEnhancer.getMetaDataCount: integer;

begin
     result := priv_metadataset.Count;
end;

(* ------------------------------------------- *)

procedure tFrameProcessorMetaEnhancer.AddMetaDataSet (const DataSet : tMetaDataSet);

var entry    : tMetaDataEntry;
    oldentry : tMetaDataEntry;
    key      : string;
    i        : integer;

begin
     for i := 1 to length (DataSet) do
         begin
              entry := DataSet[i-1];

              // Generate entry key with a static SID, because we need to be able to find the
              // entry without one

              key := entry.DictionaryKey (UNKNOWN_SID);

              // Add the alert to the alert list

              if not priv_metadataset.ContainsKey (key) then
                 priv_metadataset.Add (key, entry)
              else
                  begin
                       oldentry := priv_metadataset.Items[key];

                       // TODO: Handle collisions, which can happen when a single frame has hits on multiple snort signatures



                  end;
         end;
end;

(* ------------------------------------------- *)

procedure tFrameProcessorMetaEnhancer.EnhanceFrame (Frame : tRawFrame);

var entry  : tMetaDataEntry;
    l4prot : tLayer4Protocol;
    key    : string;

begin
     l4prot := l4Any;

     if ppTCP in prot_ParseMaster.ParsedProtocols then
        l4prot := l4TCP
     else
     if ppUDP in prot_ParseMaster.ParsedProtocols then
        l4prot := l4UDP
     else
     if (ppICMPv4 in prot_ParseMaster.ParsedProtocols) or (ppICMPv6 in prot_ParseMaster.ParsedProtocols) then
        l4prot := l4ICMP;

     // To avoid commenting TCP ACK packets we require a non-zero TCP payload lenght, or no TCP

     if not ((l4prot = l4TCP) and (tcpACK in prot_ParseMaster.TCPFlags) and (prot_ParseMaster.TCPPayloadLength = 0)) then
        begin
             // We don't know the SID (as we're matching on flows), so using UNKNOWN_SID

             if prot_ParseMaster.FirstIPLayerVersion = 4 then
                key := getDictionaryKeyForMetaDataEntry (prot_ParseMaster.IPv4SocketPair, prot_ParseMaster.Identification, l4prot, prot_ParseMaster.TCPSequenceNo.Value, UNKNOWN_SID)
             else
                 key := getDictionaryKeyForMetaDataEntry (prot_ParseMaster.IPv6SocketPair, prot_ParseMaster.Identification, l4prot, prot_ParseMaster.TCPSequenceNo.Value, UNKNOWN_SID);

             // get the metadata entry if available

             if priv_metadataset.ContainsKey (key) then
                begin
                     entry := priv_metadataset.Items[key];

                     Frame.Commentlist.Add (entry.RuleDetails.Comment);

                     // Update Statistics

                     priv_MetaInfoLastMatch := entry;
                     inc (priv_MetaInfoMatchCount);

                     if assigned (priv_OnMetaInfoMatch) then
                        priv_OnMetaInfoMatch (self, priv_metadataset.Count, priv_MetaInfoMatchCount, entry);
                end;

        end;
end;

(* ------------------------------------------- *)

function tFrameProcessorMetaEnhancer.getProcessedFrame (Frame: tRawFrame): tRawFrame;

begin
     // Create ParseMaster if not handed over

     if not assigned (prot_ParseMaster) then
        prot_ParseMaster := tParseMaster.create (Frame, ppmUpToLayer4);

     // Check if we have a metadata entry to enhance the frame with

     EnhanceFrame (Frame);

     result := Frame;
end;

(* ------------------------------------------- *)

end.
