{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_interfacelist;

interface

uses Contnrs,
     base_structures,
     pcapng_structures,
     definition_ip_addresses,
     definition_simple_nameresolution,
     class_interfacestatistics;

const MaxQuickListEntries = 256;

type tInterfaceOrigin = (ioUnknown, ioFromFile, ioAutogenerated, ioUserCreated);

     // *** When changing the interface object make sure that it is stored correctly in tInterfaceMapper.StoreToDatabase ***

     tCaptureInterface = class (tObject)

                               constructor create (Index : integer; LinkType : longword; Origin : tInterfaceOrigin; OwnsItsStatisticsRecords : boolean);

                               private
                                      priv_Index               : integer;
                                      priv_OriginalIndex       : integer;                    // If framecollector merges interface lists, this holds the original ID from the capture file
                                      priv_Name                : string;
                                      priv_Description         : string;
                                      priv_IPv4AddressList     : tIPv4SimpleAddressList;
                                      priv_IPv6AddressList     : tIPv6SimpleAddressList;
                                      priv_MACAddress          : tMACAddress;
                                      priv_EUIAddress          : tEUIAddress;
                                      priv_SpeedBPS            : uInt64;
                                      priv_LinkType            : longword;
                                      priv_SnapLength          : longword;
                                      priv_TimeStampResolution : byte;
                                      priv_Timezone            : longword;
                                      priv_CaptureFilter       : tPCAPNGCaptureFilter;
                                      priv_OperatingSystem     : string;
                                      priv_FCSLength           : byte;
                                      priv_TimestampOffset     : Int64;
                                      priv_InterfaceStatistics : tStatisticsRecordList;                // list of tStatisticsRecord, see class_interfacestatistics
                                      priv_origin              : tInterfaceOrigin;
                                      priv_PacketLinkID        : tGUID;

                                      priv_OptMappedInterFace  : tCaptureInterface;                    // Used to map an interface to another, e.g for file merge operations

                                      procedure SetOptionalMappedInterFace (Entry : tCaptureInterface);
                               public
                                     property InterfaceID : integer read priv_Index write priv_Index;
                                     property OriginalID : integer read priv_OriginalIndex write priv_OriginalIndex;
                                     property Name : string read priv_Name write priv_Name;
                                     property Description : string read priv_Description write priv_Description;
                                     property IPv4Addresses : tIPv4SimpleAddressList read priv_IPv4AddressList write priv_IPv4AddressList;
                                     property IPv6Addresses : tIPv6SimpleAddressList read priv_IPv6AddressList write priv_IPv6AddressList;
                                     property MACAddress : tMACAddress read priv_MACAddress write priv_MACAddress;
                                     property EUIAddress : tEUIAddress read priv_EUIAddress write priv_EUIAddress;
                                     property SpeedBPS : uint64 read priv_SpeedBPS write priv_SpeedBPS;
                                     property LinkType : longword read priv_LinkType write priv_LinkType;
                                     property SnapLength : longword read priv_SnapLength write priv_SnapLength;
                                     property TimeStampResolution : byte read priv_TimeStampResolution write priv_TimeStampResolution;
                                     property TimeZone : longword read priv_Timezone write priv_Timezone;
                                     property CaptureFilter : tPCAPNGCaptureFilter read priv_CaptureFilter write priv_CaptureFilter;
                                     property OperatingSystem : string read priv_OperatingSystem write priv_OperatingSystem;
                                     property FCSLength : byte read priv_FCSLength write priv_FCSLength;
                                     property TimestampOffset : int64 read priv_TimestampOffset write priv_TimestampOffset;
                                     property InterfaceStatistics : tStatisticsRecordList read priv_InterfaceStatistics write priv_InterfaceStatistics;
                                     property Origin : tInterfaceOrigin read priv_origin write priv_origin;
                                     property PacketLinkID : tGUID read priv_PacketLinkID write priv_PacketLinkID;

                                     destructor Destroy; override;

                                     {$REGION 'Documentation'}
                                     /// <summary>
                                     ///   This property is used to map
                                     ///   interfaces to other interfaces. Used
                                     ///   by the merge task to specify what
                                     ///   target interface a source interface
                                     ///   will be written with (mapped to)
                                     /// </summary>
                                     {$ENDREGION}
                                     property MappedToInterface : tCaptureInterface read priv_OptMappedInterFace write SetOptionalMappedInterFace;

                                     {$REGION 'Documentation'}
                                     /// <summary>
                                     ///   Copies an interface entry to a new
                                     ///   entry. The new entry needs to be
                                     ///   created by calling the constructor
                                     ///   first.
                                     /// </summary>
                                     /// <param name="DestinationInterface">
                                     ///   The interface the values of the
                                     ///   current interface are copied to
                                     /// </param>
                                     /// <param name="KeepUniqueID">
                                     ///   If true the new interface will be an
                                     ///   exact copy, including the unique ID
                                     ///   that is used to match packets to
                                     ///   their interfaces. This is helpful
                                     ///   when creating interfaces in/for the
                                     ///   PCAPngWriter class. Default is FALSE.
                                     /// </param>
                                     {$ENDREGION}
                                     procedure copyTo (DestinationInterface : tCaptureInterface; const KeepUniqueID : boolean = false) ;
                         end;

     tQuickListEntry   = record
//                             index          : integer;
                               interfaceentry : tCaptureInterface;
                         end;

     tInterfaceQuickList = array [0..MaxQuickListEntries - 1] of tQuickListEntry;

     tCaptureInterfaceList = class (tObject)

                               {$REGION 'Documentation'}
                               /// <summary>
                               ///   If DiscardEntriesOnDestory is true the
                               ///   interface objects will be freed when the
                               ///   list is freed.
                               /// </summary>
                               {$ENDREGION}
                               constructor create (DiscardEntriesOnDestory : boolean);

                               private
                                      priv_interfacelist       : tObjectList;

                                      priv_quicklist           : tInterfaceQuickList;

                                      function getInterfaceCount: integer;

                               public
                                     property Count : integer read getInterfaceCount;

                                     destructor Destroy; override;

                                     procedure Add (NewInterface : tCaptureInterface);
                                     procedure Delete (Index : integer);
                                     procedure AddPseudoInterface (LinkType : longword);
                                     {$REGION 'Documentation'}
                                     /// <summary>
                                     ///   Returns the capture interface
                                     ///   specified by the index. Since the
                                     ///   interfaces are indexed starting at
                                     ///   zero in the PCAPng files the index
                                     ///   of the first interface is also zero.
                                     /// </summary>
                                     /// <param name="Index">
                                     ///   Index of the interface, starting at
                                     ///   zero
                                     /// </param>
                                     {$ENDREGION}
                                     function getInterfaceByIndex (Index : integer) : tCaptureInterface;
                                     function getInterfaceByOriginalIndex (Index : integer) : tCaptureInterface;
                                     function getInterfaceByName (SearchFor : string) : tCaptureInterface;
                                     {$REGION 'Documentation'}
                                     /// <summary>
                                     ///   Find an interface by the unique ID
                                     ///   that is used to link intefaces with
                                     ///   packets
                                     /// </summary>
                                     /// <param name="Identifier">
                                     ///   GUID used as a unique ID to link
                                     ///   interfaces and packets
                                     /// </param>
                                     {$ENDREGION}
                                     function getInterfaceByIdentifier (Identifier : tGUID) : tCaptureInterface;
                                     {$REGION 'Documentation'}
                                     /// <summary>
                                     ///   Clears the list of interfaces and
                                     ///   removes the interface objects if the
                                     ///   list was created with
                                     ///   DiscardEntriesOnDestroy
                                     /// </summary>
                                     {$ENDREGION}
                                     procedure clear;

                                     procedure copyTo (DestinationList : tCaptureInterfaceList; const KeepUniqueID : boolean = false; const clearListBeforeCopy : boolean = true);
                                     procedure mergeTo (DestinationList : tCaptureInterfaceList; const KeepUniqueID : boolean = false);
                         end;

     function GetInterfaceOriginAsString (Origin : tInterfaceOrigin): string;
     function InterfacesAreEqual (Entry1, Entry2: tCaptureInterface): boolean;


implementation

uses SysUtils,
     SiAuto;

(* ------------------------------------------- *)

function GetInterfaceOriginAsString (Origin : tInterfaceOrigin): string;

begin
     case Origin of
      ioUnknown       : result := 'unknown';
      ioFromFile      : result := 'From File';
      ioAutogenerated : result := 'AutoGenerated';
      ioUsercreated   : result := 'UserCreated';
     end;
end;


(* ------------------------------------------- *)

function InterfacesAreEqual (Entry1, Entry2: tCaptureInterface): boolean;

var Name1 : string;
    Name2 : string;

begin
     Name1 := lowercase (trim (Entry1.Name));
     Name2 := lowercase (trim (Entry2.Name));

     result := (Name1 = Name2) and
               (entry1.LinkType = entry2.LinkType) and
               (entry1.SnapLength = entry2.SnapLength) and
               (entry1.TimeStampResolution = entry2.TimeStampResolution);
end;

(* ------------------------------------------- *)
{ tCaptureInterface }
(* ------------------------------------------- *)

constructor tCaptureInterface.create (Index : integer; LinkType : longword; Origin : tInterfaceOrigin; OwnsItsStatisticsRecords : boolean);

begin
     inherited create;

     CreateGUID (priv_PacketLinkID);    // used for verification purposes when looking up interfaces for packets

     priv_Index := Index;
     priv_OriginalIndex := Index;

     priv_MACAddress := VALUE_ZERO_MAC;
     priv_EUIAddress := VALUE_ZERO_EUI;
     setLength (priv_IPv4AddressList.list, 0);
     setLength (priv_IPv6AddressList.list, 0);
     priv_SpeedBPS := 0;
     priv_LinkType := LinkType;
     priv_SnapLength := 65535;
     priv_TimeStampResolution := 6;
     priv_Timezone := 0;
     priv_CaptureFilter.initialize;
     priv_OperatingSystem := '';
     priv_FCSLength := 0;
     priv_TimestampOffset := 0;
     priv_InterfaceStatistics := tStatisticsRecordList.create (OwnsItsStatisticsRecords);

     priv_origin := Origin;
     priv_OptMappedInterFace := nil;
end;

(* ------------------------------------------- *)

destructor tCaptureInterface.Destroy;

begin
     priv_InterfaceStatistics.Free;

     inherited;
end;

(* ------------------------------------------- *)

procedure tCaptureInterface.SetOptionalMappedInterFace (Entry: tCaptureInterface);

begin
//   SiMain.EnterMethod ('tCaptureInterface.SetOptionalMappedInterFace');
{
     if Entry = nil then
        SiMain.LogWarning ('Setting optional mapping target interface for "' + priv_name + '" (LinkID: ' + priv_PacketLinkID.ToString + ') to NIL')
     else
         SiMain.LogVerbose ('Setting optional mapping target interface for "' + priv_name + '" (LinkID: ' + priv_PacketLinkID.ToString + ') to "' + Entry.Name + '" (LinkID: ' + Entry.PacketLinkID.ToString + ')');
}
     priv_OptMappedInterFace := Entry;

//   SiMain.LeaveMethod ('tCaptureInterface.SetOptionalMappedInterFace');
end;

(* ------------------------------------------- *)

procedure tCaptureInterface.copyTo (DestinationInterface: tCaptureInterface; const KeepUniqueID : boolean = false);

begin
//   SiMain.EnterMethod ('tCaptureInterface.copyTo');

     if KeepUniqueID then
        DestinationInterface.priv_PacketLinkID := priv_PacketLinkID;

     DestinationInterface.InterfaceID := priv_Index;
     DestinationInterface.OriginalID := priv_OriginalIndex;
     DestinationInterface.Name := priv_Name;
     DestinationInterface.Description := priv_Description;

     DestinationInterface.IPv4Addresses.copyFrom (priv_IPv4AddressList.list);
     DestinationInterface.IPv6Addresses.copyFrom (priv_IPv6AddressList.list);

     DestinationInterface.MACAddress := priv_MACAddress;
     DestinationInterface.EUIAddress := priv_EUIAddress;
     DestinationInterface.SpeedBPS := priv_SpeedBPS;
     DestinationInterface.LinkType := priv_LinkType;
     DestinationInterface.SnapLength := priv_SnapLength;
     DestinationInterface.TimeStampResolution := priv_TimeStampResolution;
     DestinationInterface.TimeZone := priv_Timezone;
     DestinationInterface.CaptureFilter := priv_CaptureFilter;
     DestinationInterface.OperatingSystem := priv_OperatingSystem;
     DestinationInterface.FCSLength := priv_FCSLength;
     DestinationInterface.TimestampOffset := priv_TimestampOffset;
     DestinationInterface.MappedToInterface := priv_OptMappedInterFace;

     priv_InterfaceStatistics.copyTo (DestinationInterface.InterfaceStatistics);

//   SiMain.LeaveMethod ('tCaptureInterface.copyTo');
end;

(* ------------------------------------------- *)
{ tInterfaceList }
(* ------------------------------------------- *)

constructor tCaptureInterfaceList.create (DiscardEntriesOnDestory : boolean);

var i : integer;

begin
     inherited create;

     priv_interfacelist := tObjectlist.Create (DiscardEntriesOnDestory);

     for i := 0 to MaxQuickListEntries - 1 do
         begin
//            priv_quicklist[i].index := -1;
              priv_quicklist[i].interfaceentry := nil;
         end;
end;

(* ------------------------------------------- *)

destructor tCaptureInterfaceList.Destroy;

begin
     priv_interfacelist.Free;

     inherited;
end;

(* ------------------------------------------- *)

function tCaptureInterfaceList.getInterfaceCount: integer;

begin
     result := priv_interfacelist.Count;
end;

(* ------------------------------------------- *)

procedure tCaptureInterfaceList.Add (NewInterface: tCaptureInterface);

begin
     assert (assigned (NewInterface));
     priv_interfacelist.Add (NewInterface);

     assert (NewInterface.priv_index < MaxQuickListEntries);
     priv_quicklist[NewInterface.priv_Index].interfaceentry := NewInterface;
end;

(* ------------------------------------------- *)

procedure tCaptureInterfaceList.Delete (Index: integer);

begin
     priv_interfacelist.Delete(Index);
end;

(* ------------------------------------------- *)

procedure tCaptureInterfaceList.AddPseudoInterface(LinkType: longword);

var entry : tCaptureInterface;

begin
     entry := tCaptureInterface.create (0, LinkType, ioAutoGenerated, true);

     entry.priv_OriginalIndex := 0;
     entry.priv_Name := 'unknown';
     entry.priv_Description := 'unknown';
end;

(* ------------------------------------------- *)

// Interface indexes start at 0, so no -1 here (this is because pcapng starts enumerating from 0)

function tCaptureInterfaceList.getInterfaceByIndex (Index: integer): tCaptureInterface;

begin
     if priv_quicklist[Index].interfaceentry <> nil then
        result := priv_quicklist[Index].interfaceentry
     else
         result := nil;
end;

(* ------------------------------------------- *)

function tCaptureInterfaceList.getInterfaceByOriginalIndex (Index: integer): tCaptureInterface;

var entry   : tCaptureInterface;
    boFound : boolean;
    i       : integer;

begin
     i := -1;

     boFound := false;

     repeat
           inc (i);

           entry := priv_quicklist[i].interfaceentry;

           if entry <> nil then
              boFound:= (entry.priv_OriginalIndex = Index);

     until (i > MaxQuickListEntries) or boFound;

     if boFound then
        result := entry
     else
         result := nil;
end;

(* ------------------------------------------- *)

function tCaptureInterfaceList.getInterfaceByName (SearchFor: string): tCaptureInterface;

var entry   : tCaptureInterface;
    boFound : boolean;
    i       : integer;

begin
     i := -1;

     boFound := false;

     repeat
           inc (i);

           entry := priv_quicklist[i].interfaceentry;

           if entry <> nil then
              boFound:= (lowercase (entry.priv_Name) = lowercase (SearchFor));

     until (i > MaxQuickListEntries) or boFound;

     if boFound then
        result := entry
     else
         result := nil;
end;

(* ------------------------------------------- *)

function tCaptureInterfaceList.getInterfaceByIdentifier (Identifier: tGUID): tCaptureInterface;

var entry   : tCaptureInterface;
    boFound : boolean;
    i       : integer;

begin
     i := -1;

     boFound := false;

     repeat
           inc (i);

           entry := priv_quicklist[i].interfaceentry;

           if entry <> nil then
              boFound:= (entry.priv_PacketLinkID = Identifier);

     until (i > MaxQuickListEntries) or boFound;

     if boFound then
        result := entry
     else
         result := nil;
end;

(* ------------------------------------------- *)

procedure tCaptureInterfaceList.clear;

begin
     priv_interfacelist.Clear;
end;

(* ------------------------------------------- *)

procedure tCaptureInterfaceList.copyTo (DestinationList: tCaptureInterfaceList; const KeepUniqueID : boolean = false; const clearListBeforeCopy : boolean = true);

var newEntry : tCaptureInterface;
    entry    : tCaptureInterface;
    i        : integer;

begin
     if clearListBeforeCopy then
        DestinationList.clear;

     for i := 1 to priv_interfacelist.Count do
         begin
              entry := (priv_interfacelist[i-1] as tCaptureInterface);

              newEntry := tCaptureInterface.create (entry.InterfaceID, entry.LinkType, entry.Origin, true);
              entry.copyTo (newEntry, KeepUniqueID);

              DestinationList.Add (newEntry);
         end;
end;

(* ------------------------------------------- *)

procedure tCaptureInterfaceList.mergeTo (DestinationList: tCaptureInterfaceList; const KeepUniqueID : boolean = false);

var newEntry : tCaptureInterface;
    chkEntry : tCaptureInterface;
    entry    : tCaptureInterface;
    found    : boolean;
    i,j      : integer;

begin
     for i := 1 to priv_interfacelist.Count do
         begin
              entry := (priv_interfacelist[i-1] as tCaptureInterface);

              // Check if we already have this one - since we're dealing with just a few interfaces we just do a simple slow search

              j := 0;
              found := false;

              repeat
                    if j < DestinationList.Count - 1 then
                       begin
                            chkEntry := DestinationList.priv_interfacelist[j] as tCaptureInterface;

                            if (chkEntry.priv_Name = entry.Name) and (chkEntry.priv_LinkType = entry.LinkType) then
                               found := true;
                       end;

                    inc (j);

              until found or (j >= DestinationList.Count);

              // Add the interface if not found

              if not found then
                 begin
                      newEntry := tCaptureInterface.create (entry.InterfaceID, entry.LinkType, entry.Origin, true);
                      entry.copyTo(newEntry, KeepUniqueID);
                 end;
         end;
end;

(* ------------------------------------------- *)

end.
