{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_loader_pcapng;

interface

uses base_structures,
     pcapng_structures,
     definition_ip_addresses,
     definition_simple_nameresolution,
     class_rawframe,
     class_interfacelist,
     class_interfacestatistics,
     class_traceloader;

type tSHBStringType = (sstComment, sstHardware, sstOperatingSystem, sstApplication);

     tPCAPNGLoader = class (tTraceLoader)

                           {$REGION 'Documentation'}
                           ///	<summary>
                           ///	  Constructor used to create a PCAPngLoader
                           ///	  instance
                           ///	</summary>
                           ///	<param name="Filename">
                           ///	  Filename of the tracefile that will be opened
                           ///	</param>
                           ///	<param name="FileIndex">
                           ///	  Index of the file if it is part of a filelist,
                           ///	  e.g. in a tTraceFileList object. Used to set
                           ///	  the according index in each frame read.
                           ///	</param>
                           ///	<param name="OwnsInterfacelist">
                           ///	  if true, the capture interface list will be
                           ///	  discarded when the loader is destroyed. This
                           ///	  must be set to false in cases where the
                           ///	  interfaces are stored and managed even after
                           ///	  the loader object is freed.
                           ///	</param>
                           {$ENDREGION}

                           constructor create (const Filename : string; const FileIndex : integer = 0; const OwnsInterfacelist : boolean = true);

                           private
                                  priv_ByteSwappedFile    : boolean;

                                  priv_SecHdrCount        : integer;
                                  priv_interfacemodifier  : integer;                      // necessary for multiple SHBs in a file to get the correct interface index
                                  priv_NRHdrCount         : integer;
                                  priv_skipsection        : boolean;

                                  priv_majorversion       : word;
                                  priv_minorversion       : word;
                                  priv_byteordermagic     : longword;
                                  priv_sectionlength      : uint64;

                                  priv_keepUTCTime        : boolean;

                                  priv_interfaceResList   : tNGInterfaceResolutionList;   // Keeps the timestamp resolution bytes we need for reading statistics

//                                function Get32BitAlignment (UnAlignedLength : integer) : integer;
                                  procedure DoSanityCheck (var WireDetails : tWireDetails; FilePosition : tFileFrameLocation);
{$ifndef unittest}
                                  function getTimestampUnits (TimeHigh, TimeLow : longword) : uint64;
                                  function getPCAPNGTiming (TimeResolution : byte; TimestampUnits : uint64) : tTimingDetails;

                                  // Single Major Block reading routines

                                  function readSectionBlockHeader (var Position : pByte; var Offset : integer) : tNGSHBParsedValues;
                                  function readInterfaceDescriptionHeader (var Position : pByte; var Offset : integer) : tCaptureInterface;
                                  function readNameResolutionBlockHeader (var Position: pByte; var Offset: integer) : tNGNRBParsedValues;
                                  function readInterfaceStatisticsBlock (var Position: pByte; var Offset: integer; InterfaceResolutionList : tNGInterfaceResolutionList) : tStatisticsRecord;
{$endif}
                                  function TreatAsRepeatingBlock (BlockTypeID : integer) : boolean;
                                  procedure AssignSHBValuesToLoader (SHBValueSet : tNGSHBParsedValues);
                                  procedure AssignNRBValuesToLoader (NRBValueSet : tNGNRBParsedValues);

                                  function readSectionBlockHeaderOptions (const Position : pByte; const Offset, OptionLen : integer) : tNGSHBParsedOptionValues;
                                  procedure ReadMasterBlocks (var Position: pByte; var Offset: integer);

                                  function readSimplePacketBlock (const FrameIndex : integer; var Position : pByte; var Offset : integer) : tRawFrame;
                                  function readPacketBlock (const FrameIndex : integer; var Position : pByte; var Offset : integer) : tRawFrame;
                                  function readEnhancedPacketBlock (const FrameIndex : integer; var Position : pByte; var Offset : integer) : tRawFrame;

                                  function getPacketBlockCountFromBlockChain (ChainRoot : pBlockChainEntry) : integer;
                                  function getSectionHeaderCountFromBlockChain (ChainRoot : pBlockChainEntry) : integer;
                                  function getInterfaceCountFromBlockChain (ChainRoot : pBlockChainEntry) : integer;
                                  procedure DetermineChainBlockDetails (var Entry : pBlockChainEntry);

                           protected
                                    function thisFileFormat : tCaptureFileType; override;

                           public
                                 property MajorVersion : word read priv_majorversion;
                                 property MinorVersion : word read priv_minorversion;
                                 property ByteOrderMagic : longword read priv_byteordermagic;
                                 property SectionLength : uint64 read priv_sectionlength;
                                 property DoNotAdjustUTCTimeStamps : boolean read priv_keepUTCTime write priv_keepUTCTime;

{$ifdef unittest}
                                  // -- Block moved from private to public for unittests

                                 function getTimestampUnits (TimeHigh, TimeLow : longword) : uint64;
                                 function getPCAPNGTiming (TimeResolution : byte; TimestampUnits : uint64) : tTimingDetails;

                                 // Single Major Block reading routines

                                 function readSectionBlockHeader (var Position : pByte; var Offset : integer) : tNGSHBParsedValues;
                                 function readInterfaceDescriptionHeader (var Position : pByte; var Offset : integer) : tCaptureInterface;
                                 function readNameResolutionBlockHeader (var Position: pByte; var Offset: integer) : tNGNRBParsedValues;
                                 function readInterfaceStatisticsBlock (var Position: pByte; var Offset: integer; InterfaceResolutionList : tNGInterfaceResolutionList) : tStatisticsRecord;

                                 // -- End of Block
{$endif}
                                 // Aggregated reading routines

                                 procedure readTraceFileheader; override;
                                 procedure readBatchFromDisk; override;
                                 function readSingleFrameFromDisk (const FrameIndex : integer) : tRawFrame; override;
                                 function readSingleFrameByOffset (const Offset : integer; FrameIndex : integer = -1) : tRawFrame; override;
                                 {$REGION 'Documentation'}
                                 /// <summary>
                                 ///   Reads the last frame from disk by
                                 ///   starting at the end of the file and
                                 ///   scanning backwards until an enhanced
                                 ///   packet block is found or the start of
                                 ///   the file is reached
                                 /// </summary>
                                 /// <returns>
                                 ///   Last frame in the file, or nil if not
                                 ///   found
                                 /// </returns>
                                 {$ENDREGION}
                                 function readLastFrameFromDisk : tRawFrame;
                                 function readFileStatistics (ReadFullStatistics : boolean = true) : tPCAPNGStatistic;
                                 function GetBlockChain : pBlockChainEntry;
                     end;


implementation

uses SiAuto,
     SysUtils,
     Math,
     Classes,
     Contnrs,
     DateUtils,
     toolkit;
//   class_framecomment,                  // Replaced by tStringlist in tRawFrame
//   class_definition_IP;


(* ------------------------------------------- *)
{ tPCAPLoader }
(* ------------------------------------------- *)

constructor tPCAPNGLoader.create (const Filename : string; const FileIndex : integer = 0; const OwnsInterfacelist : boolean = true);

begin
     inherited create (Filename, FileIndex, OwnsInterfaceList);

     priv_interfacemodifier := 0;
     priv_keepUTCTime := false;
     prot_LinkType := LINKTYPE_NULL;        // not needed here, as those are defined per IDB
end;

(* ------------------------------------------- *)

function tPCAPNGLoader.thisFileFormat : tCaptureFileType;

begin
     result := cfPCAPNG;
end;

(* ------------------------------------------- *)
{
function tPCAPNGLoader.Get32BitAlignment (UnAlignedLength: integer): integer;

begin
     if (UnAlignedLength mod 4) = 0 then
        result := UnAlignedLength
     else
         result := (4 - (UnAlignedLength MOD 4)) + UnAlignedLength;
end;
}
(* ------------------------------------------- *)

procedure tPCAPNGLoader.DoSanityCheck (var WireDetails: tWireDetails; FilePosition : tFileFrameLocation);

begin
     // check wiresize

     if WireDetails.WireSize < 54 then
        ReportMessage (msWarning, 'Bad Wiresize of ' + IntToStr (WireDetails.WireSize) + ' bytes detected for frame #' + IntToStr (Fileposition.FileFrameIndex), 'tPCAPNGLoader.DoSanityCheck', Fileposition.FileFrameIndex);

     // check capture size

     if WireDetails.CaptureSize > WireDetails.WireSize then
        begin
             if prot_fixloaderrors then
                begin
                     ReportMessage (msMajor, 'Fixing bad Capture size of ' + IntToStr (WireDetails.CaptureSize) + ' bytes larger than wire size (' + IntTostr (WireDetails.WireSize) + ' bytes) detected for frame #' + IntToStr (Fileposition.FileFrameIndex), 'tPCAPNGLoader.DoSanityCheck', Fileposition.FileFrameIndex);

                     WireDetails.CaptureSize := Wiredetails.WireSize;
                end
             else
                 ReportMessage (msCritical, 'Bad Capture size of ' + IntToStr (WireDetails.CaptureSize) + ' bytes larger than wire size (' + IntTostr (WireDetails.WireSize) + ' bytes) detected for frame #' + IntToStr (Fileposition.FileFrameIndex), 'tPCAPNGLoader.DoSanityCheck', Fileposition.FileFrameIndex);
        end;
end;

(* ------------------------------------------- *)

function tPCAPNGLoader.TreatAsRepeatingBlock (BlockTypeID: integer): boolean;

begin
     case BlockTypeID of
      BLOCKTYPE_PACKET,
      BLOCKTYPE_SIMPLE_PACKET,
      BLOCKTYPE_ENHANCED_PACKET,
      BLOCKTYPE_SYSDIG_FD_LIST,
      BLOCKTYPE_SYSDIG_EVENT    : result := true;
     else
         result := false;
     end;
end;

(* ------------------------------------------- *)

function tPCAPNGLoader.getTimestampUnits(TimeHigh, TimeLow: longword): uint64;

var high   : longword;
    low    : longword;
    swapit : longword;
    help   : uint64;

begin
     high := TimeHigh;
     low := TimeLow;
{
     // Workaround for hi/lo swap error in Wireshark pcapng format

     if high > low then
        begin
             ReportMessage (msWarning, 'trace file contains time stamps written in wrong order (lo/hi instead of hi/lo). tPCAPNGLoader.getTimestampUnits.', 'tPCAPNGLoader.getTimestampUnits');

             swapit := high;
             high := low;
             low := swapit;
        end;
}
     // Construct the 64 bit Timestamp value

     help := uint64 (High) shl 32;
     help := help + Low;

     result := help;
end;

(* ------------------------------------------- *)

// Calculates a tTimingDetail record from the uint64 value we find in the file

function tPCAPNGLoader.getPCAPNGTiming (TimeResolution : byte; TimestampUnits: uint64): tTimingDetails;

var TimingDetails    : tTimingDetails;
    ModifierSecs     : int64;
    ModifierTics     : int64;
    HelpDate         : tDateTime;
    HelpSeconds      : int64;
    HelpTicks        : int64;
    HelpByte         : byte;
    HelpLong         : longword;
    HelpInt64        : int64;
    HelpDouble       : double;
    SecondDiv        : extended;
    ExtSeconds       : extended;
    ExtTicks         : extended;
    ExtNano          : extended;
    ExtHelp          : extended;

begin
     // If UTCDelta is 0 we calculate the systems offset to UTC, otherwise we use what the user said

     if iUTCDelta = 0 then
        ModifierSecs := 0
     else
         begin
              ModifierSecs := iUTCDelta * 3600;
              ModifierTics := ModifierSecs * 1000 * 1000;
         end;

     if (TimeResolution and 128) = 128 then
        begin
             // If the Most Significant Bit is equal to one, the remaining bits indicates the resolution as as negative power of 2
             // (e.g. 10 means 1/1024 of second).

             HelpByte := TimeResolution and 127;
             SecondDiv := power (2, HelpByte);

             ExtSeconds := (TimestampUnits / SecondDiv);
             ExtNano := ExtSeconds * 1000000000;

             HelpSeconds := trunc (ExtSeconds);
             HelpDate := UnixToDateTime (HelpSeconds);

             if ((ModifierSecs = 0) and not priv_keepUTCTime) then
                begin
                     ModifierSecs := (TimeZoneHandler.GetBiasForDate (HelpDate) DIV 60 * -1) * 3600;
                     ModifierTics := ModifierSecs * 1000 * 1000;
                end;

             ExtSeconds := ExtSeconds + ModifierSecs;
             ExtTicks := ExtSeconds * 1000000;
             ExtNano := ExtSeconds * 1000000000;

             // get Ticks and Nanoticks

             HelpSeconds := trunc (ExtSeconds);

             TimingDetails.AbsoluteTime := UnixToDateTime (HelpSeconds);
             TimingDetails.TraceTicks := trunc (ExtTicks);

             HelpInt64 := trunc (ExtTicks) * 1000;
             HelpInt64 := trunc (ExtNano) - HelpInt64;

             if HelpInt64 > 0 then
                TimingDetails.NanoTicks := HelpInt64
             else
                 TimingDetails.NanoTicks := 0;      // TODO: check why nanoseconds are sometimes negative

             // Recalculate ticks to only include microseconds since midnight, not the whole daycount, too
             // So we take the absolute TraceDateTime and get Midnight of that day. Convert to ticks. Substract.

             HelpDate := recodeTime (TimingDetails.AbsoluteTime, 0, 0, 0, 0);
             HelpTicks := DateTimeToUnix (HelpDate) * 1000 * 1000;

             TimingDetails.TraceTicks := TimingDetails.TraceTicks - HelpTicks;
        end
     else
         begin
              // If the Most Significant Bit is equal to zero, the remaining bits indicates the resolution of the timestamp as
              // as a negative power of 10 (e.g. 6 means microsecond resolution, timestamps are the number of microseconds since 1/1/1970).

              case TimeResolution of
               0 : begin
                        // Second resolution, uh oh

                        try
                           HelpDate := UnixToDateTime (TimestampUnits);
                        except
                              raise EInOutError.Create ('Error: cannot convert UnixToDateTime for ' + IntToStr (TimeStampUnits) + ' units  (PCAPNGLoader.getPCAPNGTiming).');
                        end;

                        if ((ModifierSecs = 0) and not priv_keepUTCTime) then
                           begin
                                ModifierSecs := (TimeZoneHandler.GetBiasForDate (HelpDate) DIV 60 * -1) * 3600;
                                ModifierTics := ModifierSecs * 1000 * 1000;
                           end;

                        HelpSeconds := TimestampUnits + ModifierSecs;

                        TimingDetails.TraceTicks := (TimestampUnits * 1000 * 1000) + ModifierTics;
                        TimingDetails.NanoTicks := 0;
                   end;
               3 : begin
                        // Millisecond resolution, still not good

                        HelpDate := UnixToDateTime (TimestampUnits DIV 1000);

                        if ((ModifierSecs = 0) and not priv_keepUTCTime) then
                           begin
                                ModifierSecs := (TimeZoneHandler.GetBiasForDate (HelpDate) DIV 60 * -1) * 3600;
                                ModifierTics := ModifierSecs * 1000 * 1000;
                           end;

                        HelpSeconds := (TimestampUnits DIV 1000) + ModifierSecs;

                        TimingDetails.TraceTicks := (TimestampUnits * 1000) + ModifierTics;
                        TimingDetails.NanoTicks := 0;
                   end;
               6 : begin
                        // Microsecond resolution - we can work with this

                        HelpDate := UnixToDateTime (TimestampUnits DIV 1000000);

                        if ((ModifierSecs = 0) and not priv_keepUTCTime) then
                           begin
                                ModifierSecs := (TimeZoneHandler.GetBiasForDate (HelpDate) DIV 60 * -1) * 3600;
                                ModifierTics := ModifierSecs * 1000 * 1000;
                           end;

                        HelpSeconds := (TimestampUnits DIV 1000000) + ModifierSecs;

                        TimingDetails.TraceTicks := TimestampUnits + ModifierTics;
                        TimingDetails.NanoTicks := 0;
                   end;
               9 : begin
                        // Nanoseconds, niiiice...

                        HelpDate := UnixToDateTime (TimestampUnits DIV 1000000000);

                        if ((ModifierSecs = 0) and not priv_keepUTCTime) then
                           begin
                                ModifierSecs := (TimeZoneHandler.GetBiasForDate (HelpDate) DIV 60 * -1) * 3600;
                                ModifierTics := ModifierSecs * 1000 * 1000;
                           end;

                        HelpSeconds := (TimestampUnits DIV 1000000000) + ModifierSecs;

                        // get Ticks and Nanoticks

                        TimingDetails.TraceTicks := (TimestampUnits DIV 1000) + ModifierTics;
                        TimingDetails.NanoTicks := (TimestampUnits + (1000 * ModifierTics)) - (TimingDetails.TraceTicks * 1000);
                   end;
              else
                  raise EInOutError.Create ('Error: Unknown timestamp resolution encountered (PCAPNGLoader.readEnhancedPacketBlock).');
              end;

              TimingDetails.AbsoluteTime := UnixToDateTime (HelpSeconds);

              // Recalculate ticks to only include microseconds since midnight, not the whole daycount, too
              // So we take the absolute TraceDateTime and get Midnight of that day. Convert to ticks. Substract.

              HelpDate := recodeTime (TimingDetails.AbsoluteTime, 0, 0, 0, 0);
              HelpTicks := DateTimeToUnix (HelpDate) * 1000 * 1000;

              TimingDetails.TraceTicks := TimingDetails.TraceTicks - HelpTicks;
         end;

     result := TimingDetails;
end;

(* ------------------------------------------- *)

procedure tPCAPNGLoader.AssignSHBValuesToLoader (SHBValueSet: tNGSHBParsedValues);

begin
     priv_majorversion := SHBValueSet.MajorVersion;
     priv_minorversion := SHBValueSet.MinorVersion;
     priv_byteordermagic := SHBValueSet.ByteOrderMagic;
     priv_sectionlength := SHBValueSet.TotalSectionLength;

     // Assign optional values

     HeaderCommentList.Clear;
     HeaderCommentlist.AddStrings (SHBValueSet.OptionValues.HeaderCommentList);

     HardwareInfoList.Clear;
     HardwareInfoList.AddStrings (SHBValueSet.OptionValues.HardwareInfoList);

     OperatingSystemList.Clear;
     OperatingSystemList.AddStrings (SHBValueSet.OptionValues.OperatingSystemList);

     UserApplicationList.Clear;
     UserApplicationList.AddStrings (SHBValueSet.OptionValues.UserApplicationList);

     CaptureSystemGUID := SHBValueSet.OptionValues.CaptureSystemGUID;
end;

(* ------------------------------------------- *)

{
procedure tPCAPNGLoader.AssignNRBValuesToLoader (NRBValueSet: tNGNRBParsedValues);

var i           : integer;
    HelpString  : string;
    IPAddress   : tIPAddress;

begin
     for i := 1 to NRBValueSet.IPv4AddressList.Count do
         begin
              IPAddress := NRBValueSet.IPv4AddressList[i-1] as tIPv4Address;
              HelpString := NRBValueSet.IPv4NameList[i-1];

              IPv4NameResolutionMgr.addEntry (atIPv4, IPAddress, Helpstring, 'from PCAP-NG NRHdr');
         end;

     for i := 1 to NRBValueSet.IPv6AddressList.Count do
         begin
              IPAddress := NRBValueSet.IPv6AddressList[i-1] as tIPv6Address;
              HelpString := NRBValueSet.IPv6NameList[i-1];

              IPv6NameResolutionMgr.addEntry (atIPv6, IPAddress, Helpstring, 'from PCAP-NG NRHdr');
         end;
end;
}

(* ------------------------------------------- *)

procedure tPCAPNGLoader.AssignNRBValuesToLoader (NRBValueSet: tNGNRBParsedValues);

var i           : integer;
    HelpString  : string;
    count       : integer;
    IPv4Entry   : tSimpleIPv4NameResolutionEntry;
    IPv6Entry   : tSimpleIPv6NameResolutionEntry;

begin
     for i := 1 to NRBValueSet.IPv4AddressList.count do
         begin
              IPv4Entry := NRBValueSet.IPv4AddressList.GetEntry(i);

              IPv4NameResolutionMgr.AddEntry (IPv4Entry);
         end;

     for i := 1 to NRBValueSet.IPv6AddressList.count do
         begin
              IPv6Entry := NRBValueSet.IPv6AddressList.GetEntry(i);

              IPv6NameResolutionMgr.AddEntry (IPv6Entry);
         end;
end;

(* ------------------------------------------- *)

function tPCAPNGLoader.readSectionBlockHeaderOptions (const Position: pByte; const Offset, OptionLen: integer) : tNGSHBParsedOptionValues;

var OptionHdr        : pNGOptionCodeLen;
    OptionsProcessed : boolean;
    LocalOffset      : integer;
    Bit32Length      : integer;
    HelpChar         : pANSIChar;
    HelpString       : string;
    HelpGUID         : pGUID;
    ValueSet         : tNGSHBParsedOptionValues;

begin
     OptionsProcessed := false;
     LocalOffset := 0;

     ValueSet.initialize;

     // Process options

     repeat
           if LocalOffset < OptionLen then
              begin
                   if (Offset + LocalOffset > iFileSize)  then
                      begin
                           LoaderStatus := lsIncompleteFile;

                           ReportMessage (msCritical, 'Incomplete pcapng file, cannot read Section Block Header Options', 'tPCAPNGLoader.readSectionBlockHeaderOptions');
                           exit (ValueSet);
                      end;
              end;

           OptionHdr := pNGOptionCodeLen (Position + LocalOffset);
           LocalOffset := LocalOffset + sizeOf (NGOptionCodeLen);

           if (LocalOffset + OptionHdr.length) > OptionLen then
              ReportMessage (msWarning, 'Damaged pcapng file, option size is longer than the option block (Option Code #' + IntToStr (OptionHdr.code) + ')', 'tPCAPNGLoader.readSectionBlockHeaderOptions');

           case OptionHdr^.code of
            opt_endofopt :  OptionsProcessed := true;
            opt_comment  :  begin
                                 HelpChar := pANSIChar (Position + LocalOffset);
                                 HelpString := string (HelpChar);

                                 HelpString := copy (HelpString, 1, OptionHdr^.length);

                                 ValueSet.HeaderCommentList.Add(HelpString);
                            end;
            shb_hardware  : begin
                                 HelpChar := pANSIChar (Position + LocalOffset);
                                 HelpString := string (HelpChar);
                                 HelpString := copy (HelpString, 1, OptionHdr^.length);

                                 ValueSet.HardwareInfoList.Add(HelpString);
                            end;
            shb_os        : begin
                                 HelpChar := pANSIChar (Position + LocalOffset);
                                 HelpString := string (HelpChar);
                                 HelpString := copy (HelpString, 1, OptionHdr^.length);

                                 ValueSet.OperatingSystemList.Add (HelpString);

                            end;
            shb_userappl  :	begin
                                 HelpChar := pANSIChar (Position + LocalOffset);
                                 HelpString := string (HelpChar);
                                 HelpString := copy (HelpString, 1, OptionHdr^.length);

                                 ValueSet.UserApplicationList.Add(HelpString);
                            end;

            shb_hone_guid : begin
                                 HelpGUID := pGUID (Position + LocalOffset);
                                 ValueSet.CaptureSystemGUID := HelpGUID^;
                            end;
           end;

           // Options are variable length, but aligned to 32bits, so we need to find the aligned jump distance

           Bit32Length := Get32BitAlignment (OptionHdr^.length);
           LocalOffset := LocalOffset + Bit32Length;

           // Check end of options by looking if there are any bytes left (in case no opt_endofopt was written)

           if LocalOffset >= OptionLen then
              OptionsProcessed := true;

     until OptionsProcessed;

     result := ValueSet;
end;

(* ------------------------------------------- *)

function tPCAPNGLoader.readSectionBlockHeader (var Position : pByte; var Offset : integer) : tNGSHBParsedValues;

var SectionHdr       : pNGSectionHdrBlock;
    OptionLen        : integer;
    ValueSet         : tNGSHBParsedValues;

begin
     // Inititalize

     ValueSet.initialize;                     // the sub-OptionsSet will be initialized later when reading options
     priv_interfaceResList.initialize;

     // Length check

     if Offset + (SizeOf (NGSectionHdrBlock) + 4) > iFileSize then    // +4 for trailing TotalBlockLength
        begin
             LoaderStatus := lsIncompleteFile;

             ReportMessage (msCritical, 'incomplete pcapng file, cannot read Section Block Header', 'tPCAPNGLoader.readSectionBlockHeader');
             exit (ValueSet);
        end;

     // Prepare status variables

     inc (priv_SecHdrCount);
     priv_skipsection := false;

     priv_interfacemodifier := priv_interfacemodifier + InterfaceCount +1;   // This will usually 0, but in case of multiple SHBs it will make sure Interface indexes are correct
     InterfaceCount := -1;                                                   // Reset counter for this section which will have it's own IDBs

     // Read the header

     SectionHdr := pNGSectionHdrBlock (Position);

     // Length check for block content

     if (Offset + SectionHdr.BlockTotalLength) > iFileSize then
        begin
             LoaderStatus := lsIncompleteFile;

             ReportMessage (msCritical, 'incomplete pcapng file, cannot read Section Block Header', 'tPCAPNGLoader.readSectionBlockHeader');
             exit (ValueSet);
        end;

     ValueSet.BlockType := SectionHdr.BlockType;
     ValueSet.BlockTotalLength := SectionHdr.BlockTotalLength;
     ValueSet.ByteOrderMagic := SectionHdr.ByteOrderMagic;
     ValueSet.MajorVersion := SectionHdr.MajorVersion;
     ValueSet.MinorVersion := SectionHdr.MinorVersion;
     ValueSet.TotalSectionLength := SectionHdr.SectionLength;
{
     priv_majorversion := SectionHdr^.MayorVersion;
     priv_minorversion := SectionHdr^.MinorVersion;
     priv_byteordermagic := SectionHdr^.ByteOrderMagic;
     priv_sectionlength := SectionHdr^.SectionLength;
}
     // Check Version and do not process if we don't know it

     if (SectionHdr^.MajorVersion <> 1) or (SectionHdr^.MinorVersion <> 0) then
        begin
             priv_skipsection := true;

             ReportMessage (msWarning, 'Unknown PCAP-ng SHB version encoutered: ' + IntToStr (SectionHdr^.MajorVersion) + '.' + IntToStr (SectionHdr^.MinorVersion) + ', skipping section', 'tPCAPNGLoader.readSectionBlockHeader');
        end
     else
         begin
              // Check ByteOrder Little/Big Endian

              if SectionHdr^.ByteOrderMagic = SHB_BYTEORDERMAGIC then
                 priv_ByteSwappedFile := false
              else
                  if swap32 (SectionHdr^.ByteOrderMagic) = SHB_BYTEORDERMAGIC then
                     priv_ByteSwappedFile := true
                  else
                      begin
                           priv_skipsection := true;

                           ReportMessage (msWarning, 'Unknown PCAP-ng ByteOrderMagic value encountered: ' + IntToStr (SectionHdr^.ByteOrderMagic) + ', skipping section', 'tPCAPNGLoader.readSectionBlockHeader');
                      end;

              // Check block total length

              OptionLen := SectionHdr^.BlockTotalLength - (sizeOf (NGSectionHdrBlock) + 4);     // +4 because of the trailing BlockTotalLength

              if (OptionLen > 0) and not priv_skipsection then
                 begin
                      // Process options. They start at the 24th byte, after which follows the trailing TotalBlockLength for a second time

                      ValueSet.OptionValues :=  readSectionBlockHeaderOptions (Position + 24, Offset + 24, OptionLen);
                 end
              else
                  ValueSet.OptionValues.initialize;                           // Need to do this, otherwise it is going to crash when trying to read values from it
         end;

     // Move position pointer - this should work even if it has options

     inc (Offset, SectionHdr^.BlockTotalLength);
     inc (Position, SectionHdr^.BlockTotalLength);

     result := ValueSet;
end;

(* ------------------------------------------- *)

function tPCAPNGLoader.readInterfaceDescriptionHeader (var Position: pByte; var Offset : integer) : tCaptureInterface;

var InterfaceHdr     : pNGIntDescrBlock;
    OptionHdr        : pNGOptionCodeLen;
    InterfaceEntry   : tCaptureInterface;
    OptionLen        : integer;
    LocalOffset      : integer;
    OptionOffset     : integer;
    HelpChar         : pANSIChar;
    HelpString       : string;
    HelpByte         : pByte;
    HelpLongword     : pLongword;
    HelpUInt64       : pUInt64;
    HelpInt64        : pInt64;
    OptionsProcessed : boolean;
    Bit32Length      : integer;
    IPv6Struct       : pIPv6AddressByteStruct;
    IPv4Address      : tIPv4Address;
    IPv6Address      : tIPv6Address;
    MACAddress       : tMACAddress;
    MACBytes         : pMACBytes;
    EUIAddress       : tEUIAddress;
    EUIBytes         : pEUIBytes;
    CaptureFilter    : tPCAPNGCaptureFilter;
    count            : integer;

begin
     InterfaceEntry := nil;

     // Length check

     if Offset + (SizeOf (NGIntDescrBlock) + 4) > iFileSize then             // +4 for trailing TotalBlockLength
        begin
             LoaderStatus := lsIncompleteFile;

             ReportMessage (msCritical, 'incomplete pcapng file, cannot read Interface Description Header', 'tPCAPNGLoader.readInterfaceDescriptionHeader');
             exit (nil);
        end;

     inc (InterfaceCount);
     InterfaceHdr := pNGIntDescrBlock (Position);

     // Length check for block content

     if (Offset + InterfaceHdr.BlockTotalLength) > iFileSize then
        begin
             LoaderStatus := lsIncompleteFile;

             ReportMessage (msCritical, 'incomplete pcapng file, cannot read Interface Description Header', 'tPCAPNGLoader.readInterfaceDescriptionHeader');
             exit (nil);
        end;

     // Create new Interface

     InterfaceEntry := tCaptureInterface.create (InterfaceCount, InterfaceHdr.LinkType, ioFromFile, true);
     InterfaceEntry.TimeStampResolution := 6;                                // Default resolution 10^-6 (Microseconds), as specified in pcap-ng format specs
     InterfaceEntry.InterfaceID := InterfaceCount + priv_interfacemodifier;  // This will assign the correct interface id in case of multiple sections

     InterfaceEntry.CaptureFilter.initialize;

     OptionsProcessed := false;
     OptionLen := InterfaceHdr.BlockTotalLength - (sizeOf (NGIntDescrBlock) + 4);

     if OptionLen > 0 then
        begin
             LocalOffset := sizeOf (NGIntDescrBlock);

             // we have options

             repeat
                   // Check if the file is long enough if there should be options available

                   if LocalOffset < (sizeOf (NGIntDescrBlock) + OptionLen) then
                      begin
                           if (Offset + LocalOffset > iFileSize)  then
                              begin
                                   LoaderStatus := lsIncompleteFile;

                                   ReportMessage (msCritical, 'incomplete pcapng file, cannot read Interface Description Header Options', 'tPCAPNGLoader.readInterfaceDescriptionHeader');

                                   InterfaceEntry.Free;
                                   InterfaceEntry := nil;

                                   exit (InterfaceEntry);
                              end;
                      end;

                   // Read option header and move the local offset beyond it to get to the actual option data

                   OptionHdr := pNGOptionCodeLen (Position + LocalOffset);
                   LocalOffset := LocalOffset + sizeOf (NGOptionCodeLen);

                   // Calculate the offset we're at inside the option block and verify that we're not getting into trouble by reading an oversized option

                   OptionOffset := (LocalOffset - sizeOf (NGIntDescrBlock)) + OptionHdr.length;

                   if OptionOffset > OptionLen then
                      ReportMessage (msWarning, 'Damaged pcapng file, option size is longer than the option block (Option Code #' + IntToStr (OptionHdr.code) + ')', 'tPCAPNGLoader.readInterfaceDescriptionHeader');

                   case OptionHdr^.code of
                    opt_endofopt       : OptionsProcessed := true;
                    opt_if_name        : begin
                                              HelpChar := pANSIChar (Position + LocalOffset);
                                              HelpString := string (HelpChar);

                                              InterfaceEntry.Name := copy (HelpString, 1, OptionHdr^.length);
                                         end;
                    opt_if_description : begin
                                              HelpChar := pANSIChar (Position + LocalOffset);
                                              HelpString := string (HelpChar);

                                              InterfaceEntry.Description := copy (HelpString, 1, OptionHdr^.length);
                                         end;
                    opt_if_IPv4addr    : begin
                                              // Extract Address

                                              HelpLongword := pLongword (Position + LocalOffset);
                                              IPv4Address.ValueAsLong := HelpLongword^;
                                              IPv4Address.ValueAsLong := Swap32 (IPv4Address.ValueAsLong);

                                              // Extract Mask

                                              HelpLongword := pLongword (Position + LocalOffset + 4);
                                              IPv4Address.setMaskFromLong (swap32 (HelpLongword^));

                                              InterfaceEntry.IPv4Addresses.Add (IPv4Address);
                                         end;
                    opt_if_IPv6addr    : begin
                                              // Extract Address

                                              IPv6Struct := pIPv6AddressByteStruct (Position + LocalOffset);
                                              IPv6Address.SetValueFromByteStruct (IPv6Struct^);

                                              // Extract Mask

                                              HelpByte := pByte (Position + LocalOffset + sizeOf (tIPv6AddressWordStruct));
                                              IPv6Address.MaskBitCount := HelpByte^;

                                              InterfaceEntry.IPv6Addresses.Add(IPv6Address);
                                         end;
                    opt_if_MACaddr     : begin
                                              MACBytes := pMACBytes (Position + LocalOffset);
                                              MACAddress.RangeBytes := MACBytes^;
                                              InterfaceEntry.MACAddress := MACAddress;
                                         end;
                    opt_if_EUIaddr     : begin
                                              EUIBytes := pEUIBytes (Position + LocalOffset);
                                              EUIAddress.RangeBytes := EUIBytes^;
                                              InterfaceEntry.EUIAddress := EUIAddress;
                                         end;
                    opt_if_speed       : begin
                                              HelpUInt64 := pUINT64 (Position + LocalOffset);
                                              InterfaceEntry.SpeedBPS := HelpUInt64^;
                                         end;
                    opt_if_tsresol     : begin
                                              HelpByte := pByte (Position + LocalOffset);
                                              InterfaceEntry.TimeStampResolution := HelpByte^;

                                              inc (priv_interfaceResList.count);
                                              priv_interfaceResList.items[priv_interfaceResList.count] := HelpByte^;
                                         end;
                    opt_if_tzone       : begin
                                              HelpLongword := pLongword (Position + LocalOffset);
                                              InterfaceEntry.Timezone := HelpLongword^;
                                         end;
                    opt_if_filter      : begin
                                              // The first byte is the type of filter

                                              HelpByte := pByte (Position + LocalOffset);
                                              HelpChar := pANSIChar (Position + LocalOffset + 1);

                                              HelpString := string (HelpChar);

                                              // A bit of guesswork here since the PCAPng specs are not specifying this yet

                                              case HelpByte^ of
                                               0 : CaptureFilter.FilterType := cftLibPCAP;
                                               1 : CaptureFilter.FilterType := cftBPFByteCode;
                                              end;

                                              CaptureFilter.FilterString := copy (HelpString, 1, OptionHdr^.length - 1);

                                              InterfaceEntry.CaptureFilter := CaptureFilter;
                                         end;
                    opt_if_os          : begin
                                              HelpString := pANSIChar (Position + LocalOffset);
                                              InterfaceEntry.OperatingSystem := string (HelpString);
                                         end;
                    opt_if_fcslen      : begin
                                              HelpByte := pByte (Position + LocalOffset);
                                              InterfaceEntry.FCSLength := HelpByte^;
                                         end;
                    opt_if_tsoffset    : begin
                                              HelpInt64 := pINT64 (Position + LocalOffset);
                                              InterfaceEntry.TimestampOffset := HelpInt64^;
                                         end;
                   end;

                   // Options are variable length, but aligned to 32bits, so we need to find the aligned jump distance

                   Bit32Length := Get32BitAlignment (OptionHdr^.length);

                   LocalOffset := LocalOffset + Bit32Length;

                   // Check end of options by looking if there are any bytes left (in case no opt_endofopt was written)

                   if LocalOffset >= (sizeOf (NGIntDescrBlock) + OptionLen) then
                      OptionsProcessed := true;

             until OptionsProcessed;
        end;

     with InterfaceEntry do
          begin
               LinkType := InterfaceHdr.LinkType;
               SnapLength := InterfaceHdr.SnapLen;
          end;

     // Move position pointer - this should work even if it has options

     inc (Offset, InterfaceHdr^.BlockTotalLength);
     inc (Position, InterfaceHdr^.BlockTotalLength);

     result := InterfaceEntry;
end;

(* ------------------------------------------- *)

function tPCAPNGLoader.readNameResolutionBlockHeader (var Position: pByte; var Offset: integer) : tNGNRBParsedValues;

var NameResolutionHdr : pNGNameResolutionBlock;
    LocalOffset       : integer;
    RecordsetLen      : integer;
    RecordType        : pWord;
    RecordLength      : pWord;
    HelpLongword      : pLongword;
    HelpChar          : pANSIChar;
    HelpString        : string;
    IPv6Struct        : pIPv6AddressByteStruct;
    IPv4Address       : tIPv4Address;
    IPv6Address       : tIPv6Address;
    RecordsProcessed  : boolean;
    NSRecordSet       : tNGNRBParsedValues;

begin
     RecordsProcessed := false;

     NSRecordSet.initialize;

     // Length check

     if Offset + (SizeOf (NGNameResolutionBlock) + 4) > iFileSize then             // +4 for trailing TotalBlockLength
        begin
             LoaderStatus := lsIncompleteFile;

             ReportMessage (msCritical, 'incomplete pcapng file, cannot read Name Resolution Block Header', 'tPCAPNGLoader.readNameResolutionBlockHeader');
             exit (NSRecordSet);
        end;

     NameResolutionHdr := pNGNameResolutionBlock (Position);
     LocalOffset := sizeOf (NGNameResolutionBlock);

     // Length check for block content

     if (Offset + NameResolutionHdr.BlockTotalLength) > iFileSize then
        begin
             LoaderStatus := lsIncompleteFile;

             ReportMessage (msCritical, 'incomplete pcapng file, cannot read Name Resolution Block Header', 'tPCAPNGLoader.readNameResolutionBlockHeader');
             exit (NSRecordSet);
        end;

     RecordsetLen := NameResolutionHdr.BlockTotalLength - sizeOf (NGNameResolutionBlock);

     // Read Records

     repeat
           // Check if the file is long enough if there should be options available

           if LocalOffset < (sizeOf (NGNameResolutionBlock) + RecordsetLen) then
              begin
                   if (Offset + LocalOffset > iFileSize)  then
                      begin
                           LoaderStatus := lsIncompleteFile;

                           ReportMessage (msCritical, 'incomplete pcapng file, cannot read Name Resolution Block Options', 'tPCAPNGLoader.readNameResolutionBlockHeader');
                           exit (NSRecordSet);
                      end;
              end;

           // Read Record Type and Length

           RecordType := pWord (Position + LocalOffset);
           RecordLength := pWord (Position + LocalOffset + 2);

           // Move Offset to record value

           LocalOffset := LocalOffset + 4;

           case RecordType^ of
            nres_endofrecord : begin
                                    RecordsProcessed := true;
                               end;
            nres_ip4record   : begin
                                    HelpLongword := pLongword (Position + LocalOffset);

                                    IPv4Address.ValueAsLong := HelpLongword^;
                                    IPv4Address.ValueAsLong := Swap32 (IPv4Address.ValueAsLong);
                                    IPv4Address.MaskBitCount := 32;

                                    HelpChar := pANSIChar (Position + LocalOffset + 4);
                                    HelpString := string (HelpChar);

                                    HelpString := copy (HelpString, 1, RecordLength^);

                                    //TODO: According to specs, the string may contain multiple names separated by zero termination, so we might have to split them
{
                                    with NSRecordSet do
                                         begin
                                              IPv4AddressList.Add (IPv4Address);
                                              IPv4NameList.Add(HelpString);
                                         end;
}
                                    NSRecordSet.IPv4AddressList.AddIPandFQDN (IPv4Address, HelpString);

                                    // IPv4NameResolutionMgr.addEntry (atIPv4, IPv4Address, Helpstring, 'from PCAP-NG NRHdr');

                                    LocalOffset := LocalOffset + Get32BitAlignment (RecordLength^);
                               end;
            nres_ip6record   : begin
                                    // Extract Address

                                    IPv6Struct := pIPv6AddressByteStruct (Position + LocalOffset);
//                                  IPv6Address := tIPv6Address.create (IPv6Struct^);
                                    IPv6Address.SetValueFromByteStruct (IPv6Struct^);
                                    IPv6Address.MaskBitCount := 128;

                                    HelpChar := pANSIChar (Position + LocalOffset + 16);
                                    HelpString := copy (HelpString, 1, RecordLength^);

                                    HelpString := string (HelpChar);

                                    //TODO: According to specs, the string may contain multiple names separated by zero termination, so we might have to split them

                                    // IPv6NameResolutionMgr.addEntry (atIPv6, IPv6Address, HelpString, 'from PCAP-NG NRHdr');
{
                                    with NSRecordSet do
                                         begin
                                              IPv6AddressList.Add(IPv6Address);
                                              IPv6NameList.Add(HelpString);
                                         end;
}
                                    NSRecordSet.IPv6AddressList.AddIPandFQDN (IPv6Address, HelpString);

                                    LocalOffset := LocalOffset + Get32BitAlignment (RecordLength^);
                               end;
           end;

     until RecordsProcessed;

     // Test for options - if we find the total length value again we don't have any

     HelpLongword := pLongword (Position + LocalOffset);

     if (HelpLongword^ <> NameResolutionHdr^.BlockTotalLength) then
        begin
             // TODO: Process options

             raise EInOutError.Create ('PCAP-ng file has unparsed options in Name Resolution Header Block (PCAPNGLoader.readNameResolutionBlockHeader).');
        end;

     // Move position pointer - this should work even if it has options

     inc (Offset, NameResolutionHdr^.BlockTotalLength);
     inc (Position, NameResolutionHdr^.BlockTotalLength);

     result := NSRecordSet;
end;

(* ------------------------------------------- *)

procedure tPCAPNGLoader.ReadMasterBlocks (var Position: pByte; var Offset: integer);

var pBlockType      : pLongWord;
    pBlockTotalLen  : pLongWord;
    HeadersComplete : boolean;
    BlockCount      : integer;
    SHBValueSet     : tNGSHBParsedValues;
    NRBValueSet     : tNGNRBParsedValues;
    InterfaceEntry  : tCaptureInterface;
    UnknownCount    : integer;

begin
     BlockCount := 0;
     UnknownCount := 0;

     HeadersComplete := false;

     repeat
           // Check File Length

           if iFileSize < sizeOf (NGSectionHdrBlock) then
              begin
//                 raise EInOutError.Create ('Error: PCAPNG file too short (PCAPNGLoader.ReadMasterBlocks).');
                   LoaderStatus := lsIncompleteFile;

                   ReportMessage (msCritical, 'incomplete pcapng file, cannot read Section Block Header', 'tPCAPNGLoader.ReadMasterBlocks');
                   exit;
              end
            else
                begin
                     // Get Block Type and Block Length if not already at end of file

                     if (Offset + 8) > iFileSize then
                        HeadersComplete := true
                     else
                         begin
                              inc (BlockCount);

                              pBlockType := pLongWord (Position);
                              pBlockTotalLen := pLongWord (Position + 4);

                              if (BlockCount = 1) and (pBlockType^ <> BLOCKTYPE_SECTION_HEADER) then
                                 ReportMessage (msCritical, 'first block is not a Section Block Header', 'tPCAPNGLoader.ReadMasterBlocks');

                              case pBlockType^ of
                               BLOCKTYPE_SECTION_HEADER        : begin
                                                                      SHBValueSet := readSectionBlockHeader (Position, Offset);

                                                                      AssignSHBValuesToLoader (SHBValueSet);
                                                                      SHBValueSet.OptionValues.cleanUp;

                                                                      UnknownCount := 0;
                                                                 end;
                               BLOCKTYPE_INTERFACE_DESCRIPTION : begin
                                                                      if not priv_skipsection then
                                                                         begin
                                                                              InterfaceEntry := readInterfaceDescriptionHeader (Position, Offset);

                                                                              if InterfaceEntry <> nil then
                                                                                 InterfaceList.Add (InterfaceEntry);
                                                                         end;

                                                                      UnknownCount := 0;
                                                                 end;
                               BLOCKTYPE_PACKET,
                               BLOCKTYPE_SIMPLE_PACKET,
                               BLOCKTYPE_ENHANCED_PACKET       : begin
                                                                      if not priv_skipsection then
                                                                         HeadersComplete := true;

                                                                      UnknownCount := 0;
                                                                 end;

                               BLOCKTYPE_NAME_RESOLUTION       : if not priv_skipsection then
                                                                    begin
                                                                         NRBValueSet := readNameResolutionBlockHeader (Position, Offset);

                                                                         AssignNRBValuesToLoader (NRBValueSet);
//                                                                       NRBValueSet.cleanUp;

                                                                         UnknownCount := 0;
                                                                    end;

                               BLOCKTYPE_INTERFACE_STATISTICS  : begin
                                                                      if not priv_skipsection then
                                                                         HeadersComplete := true;

                                                                      UnknownCount := 0;
                                                                 end;
                               BLOCKTYPE_IRIG_TIMESTAMP        : begin
                                                                      if not priv_skipsection then
                                                                         begin
                                                                              ReportMessage (msWarning, 'Uncommon BlockType "IRIG_TIMESTAMP" found at offset ' + IntToStr (Offset), 'tPCAPNGLoader.ReadMasterBlocks');
                                                                              inc (Offset, pBlockTotalLen^);
                                                                         end;

                                                                      UnknownCount := 0;
                                                                 end;
                               BLOCKTYPE_ARINC_429_IN_AFDX     : begin
                                                                      if not priv_skipsection then
                                                                         begin
                                                                              ReportMessage (msWarning, 'Uncommon BlockType "ARINC_429_IN_AFDX" found at offset ' + IntToStr (Offset), 'tPCAPNGLoader.ReadMasterBlocks');
                                                                              inc (Offset, pBlockTotalLen^);
                                                                         end;

                                                                      UnknownCount := 0;
                                                                 end;
                               BLOCKTYPE_HONE_PROCESS_EVENT    : begin
                                                                      if not priv_skipsection then
                                                                         begin
                                                                              ReportMessage (msWarning, 'Uncommon BlockType "HONE_PROCESS_EVENT" found at offset ' + IntToStr (Offset), 'tPCAPNGLoader.ReadMasterBlocks');
                                                                              inc (Offset, pBlockTotalLen^);
                                                                         end;

                                                                      UnknownCount := 0;
                                                                 end;
                               BLOCKTYPE_HONE_CONNECTION_EVENT : begin
                                                                      if not priv_skipsection then
                                                                         begin
                                                                              ReportMessage (msWarning, 'Uncommon BlockType "HONE_CONNECTION_EVENT" found at offset ' + IntToStr (Offset), 'tPCAPNGLoader.ReadMasterBlocks');
                                                                              inc (Offset, pBlockTotalLen^);
                                                                         end;

                                                                      UnknownCount := 0;
                                                                 end;
                               BLOCKTYPE_SYSDIG_MACHINE_INFO   : begin
                                                                      if not priv_skipsection then
                                                                        begin
                                                                             ReportMessage (msWarning, 'Uncommon BlockType "BLOCKTYPE_SYSDIG_MACHINE_INFO" found at offset ' + IntToStr (Offset), 'tPCAPNGLoader.ReadMasterBlocks');
                                                                             inc (Offset, pBlockTotalLen^);
                                                                        end;

                                                                      UnknownCount := 0;
                                                                 end;
                               BLOCKTYPE_SYSDIG_PROCESS_LIST   : begin
                                                                      if not priv_skipsection then
                                                                         begin
                                                                              ReportMessage (msWarning, 'Uncommon BlockType "BLOCKTYPE_SYSDIG_PROCESS_LIST" found at offset ' + IntToStr (Offset), 'tPCAPNGLoader.ReadMasterBlocks');
                                                                              inc (Offset, pBlockTotalLen^);
                                                                         end;

                                                                      UnknownCount := 0;
                                                                 end;
                               BLOCKTYPE_SYSDIG_FD_LIST        : begin
                                                                      if not priv_skipsection then
                                                                         begin
                                                                              ReportMessage (msWarning, 'Uncommon BlockType "BLOCKTYPE_SYSDIG_FD_LIST" found at offset ' + IntToStr (Offset), 'tPCAPNGLoader.ReadMasterBlocks');
                                                                              inc (Offset, pBlockTotalLen^);
                                                                         end;

                                                                      UnknownCount := 0;
                                                                 end;
                               BLOCKTYPE_SYSDIG_EVENT          : begin
                                                                      if not priv_skipsection then
                                                                         begin
                                                                              ReportMessage (msWarning, 'Uncommon BlockType "BLOCKTYPE_SYSDIG_EVENT" found at offset ' + IntToStr (Offset), 'tPCAPNGLoader.ReadMasterBlocks');
                                                                              inc (Offset, pBlockTotalLen^);
                                                                         end;

                                                                      UnknownCount := 0;
                                                                 end;
                               BLOCKTYPE_SYSDIG_INTERFACE_LIST : begin
                                                                      if not priv_skipsection then
                                                                         begin
                                                                              ReportMessage (msWarning, 'Uncommon BlockType "BLOCKTYPE_SYSDIG_INTERFACE_LIST" found at offset ' + IntToStr (Offset), 'tPCAPNGLoader.ReadMasterBlocks');
                                                                              inc (Offset, pBlockTotalLen^);
                                                                         end;

                                                                      UnknownCount := 0;
                                                                 end;
                               BLOCKTYPE_SYSDIG_USER_LIST      : begin
                                                                      if not priv_skipsection then
                                                                         begin
                                                                              ReportMessage (msWarning, 'Uncommon BlockType "BLOCKTYPE_SYSDIG_USER_LIST" found at offset ' + IntToStr (Offset), 'tPCAPNGLoader.ReadMasterBlocks');
                                                                              inc (Offset, pBlockTotalLen^);
                                                                         end;

                                                                      UnknownCount := 0;
                                                                 end;
                              else
                                  begin
                                       // Protect against uncontrolled looping

                                       inc (UnknownCount);
                                       assert (UnknownCount < 10);

                                       ReportMessage (msWarning, 'Unknown BlockType $' + IntToHex (pBlockType^, 8) + ' found at offset ' + IntToStr (Offset), 'tPCAPNGLoader.ReadMasterBlocks');
                                  end;
                              end;
                         end;
                end;

     until HeadersComplete or (LoaderStatus >= lsGeneralError);
end;

(* ------------------------------------------- *)

procedure tPCAPNGLoader.readTraceFileheader;

begin
     iBatchFileOffset := 0;
     pBatchPosition := pByte (pData);

     priv_SecHdrCount := 0;
     priv_NRHdrCount := 0;

     bChannelCount := 1;

     ReadMasterBlocks (pBatchPosition, iBatchFileOffset);
end;

(* ------------------------------------------- *)

function tPCAPNGLoader.readPacketBlock (const FrameIndex: integer; var Position: pByte; var Offset: integer): tRawFrame;

var PacketHdr      : pNGPacketBlock;
    pFrameBuffer   : pRawBuffer;
    LocalOffset    : integer;
    helpBuf        : tRawBuffer;
    NewFrame       : tRawFrame;
    TimeStampUnits : uint64;
    TimeResolution : byte;
    ExpResolution  : integer;
    HelpValue      : int64;
    OptionSize     : integer;
    LenWithoutOpts : integer;
    OffsetModifier : integer;
    InterfaceEntry : tCaptureInterface;

    TimingDetails  : tTimingDetails;
    WireDetails    : tWireDetails;
    FilePosition   : tFileFrameLocation;

begin
     // Length check

     if Offset + (SizeOf (NGPacketBlock) + 4) > iFileSize then             // +4 for trailing TotalBlockLength
        begin
             LoaderStatus := lsIncompleteFile;

             ReportMessage (msCritical, 'incomplete pcapng file, cannot read Packet Block', 'tPCAPNGLoader.readPacketBlock', FrameIndex);
             exit (nil);
        end;

     PacketHdr := pNGPacketBlock (Position);
     LocalOffset := sizeOf (NGPacketBlock);

     // Length check for block content

     if (Offset + PacketHdr.BlockTotalLength) > iFileSize then
        begin
             LoaderStatus := lsIncompleteFile;

             ReportMessage (msCritical, 'incomplete pcapng file, cannot read Enhanced Packet Block', 'tPCAPNGLoader.readEnhancedPacketBlock', FrameIndex);
             exit (nil);
        end;

     if PacketHdr.BlockTotalLength <= PacketHdr.PacketLen then
        ReportMessage (msCritical, 'invalid header values found in frame #' + IntToStr (FrameIndex) + ' - total length shorter than packet length', 'tPCAPNGLoader.readPacketBlock', FrameIndex);

     WireDetails.WireSize := PacketHdr.PacketLen;
     WireDetails.CaptureSize := PacketHdr.CapturedLen;
     WireDetails.InterfaceID := PacketHdr.InterfaceID;
     WireDetails.LinkID := InterfaceList.getInterfaceByIndex(WireDetails.InterfaceID).PacketLinkID;
     WireDetails.FrameType := tftPCAPngDeprecated;
     WireDetails.FrameFlags.Value := 0;

     if PacketHdr.DropCount <> $ffff then
        WireDetails.Dropped := PacketHdr.DropCount
     else
         WireDetails.Dropped := 0;

     iFramesDropped := iFramesDropped + WireDetails.Dropped;
     iTotalFramesDropped := iTotalFramesDropped + WireDetails.Dropped;

     FilePosition.FileIndex := iFileNumber;                             // Taken from traceloader object - it knows what index the current file has
     FilePosition.FileFrameIndex := FrameIndex;
     FilePosition.FileOffset := Offset;

     // Calculate timings

     InterfaceEntry := InterfaceList.getInterfaceByIndex(PacketHdr.InterfaceID + priv_interfacemodifier);

     TimestampUnits := getTimestampUnits (PacketHdr.TimestampHigh, PacketHdr.TimestampLow);
     TimeResolution := InterfaceEntry.TimeStampResolution;
     TimingDetails := getPCAPNGTiming (TimeResolution, TimeStampUnits);

     WireDetails.LinkType := InterfaceEntry.LinkType;

     // Read frame bytes

     if WireDetails.CaptureSize > maxFrameSize then
        ReportMessage (msCritical, 'oversized frame encountered in frame #' + IntToStr (FrameIndex), 'tPCAPNGLoader.readPacketBlock', FrameIndex);


     if iFileSize < (Offset + WireDetails.CaptureSize) then
        begin
             raise EInOutError.Create ('Error: unexpected end of file in frame # ' + IntToStr (FrameIndex) + ' (tPCAPNGLoader.readPacketBlock).');

             exit (nil);
        end;

     pFrameBuffer := pRawBuffer (Position + LocalOffset);
     move (pFrameBuffer^, helpBuf, min (maxFrameSize, WireDetails.CaptureSize));

     // Check if there are options. We do that by checking if the Total Length includes
     // 1. the block header itself
     // 2. the captured bytes, aligned to 32 bit
     // 3. the additional total block size value at the end (4 bytes)
     //
     // If there is more, we have options.

     LenWithoutOpts := sizeOf (NGPacketBlock) + Get32BitAlignment (PacketHdr.CapturedLen) + 4;
     OptionSize := PacketHdr.BlockTotalLength - LenWithoutOpts;

     OffsetModifier := PacketHdr.BlockTotalLength;

     if OptionSize > 0 then
        begin
             ReportMessage (msWarning, 'options encountered but not parsed in frame #' + IntToStr (FrameIndex), 'tPCAPNGLoader.readPacketBlock', FrameIndex)

        end
     else
         if OptionSize < 0 then
            begin
                 // Workaround for broken Packet header length values

                 ReportMessage (msWarning, 'invalid packet header length found in frame #' + IntToStr (FrameIndex) + ': was ' + IntToStr (PacketHdr.BlockTotalLength) + ' but should be ' + IntToStr (LenWithoutOpts), 'tPCAPNGLoader.readPacketBlock', FrameIndex);

                 OffsetModifier := LenWithoutOpts;
            end;

     // Create new Frame Object that will hold the data

     NewFrame := tRawFrame.create (TimingDetails, WireDetails, FilePosition, helpBuf);

     // Move the offset

     Offset := Offset + OffsetModifier;
     Position := Position + OffsetModifier;

     // Give it back

     result := NewFrame;
end;

(* ------------------------------------------- *)

function tPCAPNGLoader.readSimplePacketBlock(const FrameIndex: integer; var Position: pByte; var Offset: integer): tRawFrame;

var SimplePacketHdr : pNGSimplePacketBlock;
    pFrameBuffer    : pRawBuffer;

    InterfaceEntry  : tCaptureInterface;

    LocalOffset     : integer;
    helpBuf         : tRawBuffer;
    NewFrame        : tRawFrame;

    TimingDetails   : tTimingDetails;
    WireDetails     : tWireDetails;
    FilePosition    : tFileFrameLocation;

begin
     // Length check

     if Offset + (SizeOf (NGSimplePacketBlock) + 4) > iFileSize then             // +4 for trailing TotalBlockLength
        begin
             LoaderStatus := lsIncompleteFile;

             ReportMessage (msCritical, 'incomplete pcapng file, cannot read Simple Packet Block', 'tPCAPNGLoader.readSimplePacketBlock', FrameIndex);
             exit (nil);
        end;

     SimplePacketHdr := pNGSimplePacketBlock (Position);
     LocalOffset := sizeOf (NGSimplePacketBlock);

     // Length check for block content

     if (Offset + SimplePacketHdr.BlockTotalLength) > iFileSize then
        begin
             LoaderStatus := lsIncompleteFile;

             ReportMessage (msCritical, 'incomplete pcapng file, cannot read Enhanced Packet Block', 'tPCAPNGLoader.readEnhancedPacketBlock', FrameIndex);
             exit (nil);
        end;

     if SimplePacketHdr.BlockTotalLength <= SimplePacketHdr.PacketLen then
        ReportMessage (msCritical, 'invalid header values found in frame #' + IntToStr (FrameIndex) + ' - total length shorter than packet length', 'tPCAPNGLoader.readSimplePacketBlock', FrameIndex);

     WireDetails.WireSize := SimplePacketHdr.PacketLen;
     WireDetails.CaptureSize := SimplePacketHdr.PacketLen;
     WireDetails.FrameType := tftPCAPngSimple;
     WireDetails.InterfaceID := 0;
     WireDetails.LinkID := InterfaceList.getInterfaceByIndex(WireDetails.InterfaceID).PacketLinkID;
     WireDetails.FrameFlags.Value := 0;

     InterfaceEntry := InterfaceList.getInterfaceByIndex(WireDetails.InterfaceID + priv_interfacemodifier);
     WireDetails.LinkType := InterfaceEntry.LinkType;

     FilePosition.FileIndex := iFileNumber;                             // Taken from traceloader object - it knows what index the current file has
     FilePosition.FileFrameIndex := FrameIndex;
     FilePosition.FileOffset := Offset;

     // Simple Block has no timings

     TimingDetails.AbsoluteTime := 0;
     TimingDetails.TraceTicks := 0;
     TimingDetails.NanoTicks := 0;

     // Read frame bytes

     if WireDetails.CaptureSize > maxFrameSize then
        ReportMessage (msCritical, 'oversized frame encountered in frame #' + IntToStr (FrameIndex), 'tPCAPNGLoader.readSimplePacketBlock', FrameIndex);

     if iFileSize < (Offset + SimplePacketHdr.PacketLen) then
        begin
             raise EInOutError.Create ('Error: unexpected end of file in frame # ' + IntToStr (FrameIndex) + ': expected ' + IntToStr (SimplePacketHdr.PacketLen) + ' bytes for frame (tPCAPNGLoader.readSimplePacketBlock).');

             exit (nil);
        end;

     pFrameBuffer := pRawBuffer (Position + LocalOffset);
     move (pFrameBuffer^, helpBuf, min (maxFrameSize, WireDetails.CaptureSize));

     // Create new Frame Object that will hold the data

     NewFrame := tRawFrame.create (TimingDetails, WireDetails, FilePosition, helpBuf);

     // TODO: simple frames CAN have options! Deal with them!

     // Move the offset

     Offset := Offset + SimplePacketHdr.BlockTotalLength;
     Position := Position + SimplePacketHdr.BlockTotalLength;

     // Give it back

     result := NewFrame;
end;

(* ------------------------------------------- *)

function tPCAPNGLoader.readEnhancedPacketBlock (const FrameIndex : integer; var Position: pByte; var Offset: integer) : tRawFrame;

var EnhPacketHdr     : pNGEnhPacketBlock;
    pFrameBuffer     : pRawBuffer;
    LocalOffset      : integer;
    OptionOffset     : integer;
    helpBuf          : tRawBuffer;
    NewFrame         : tRawFrame;
    DeltaTime        : tDeltaTickTime;
    TimeStampUnits   : uint64;
    TimingDetails    : tTimingDetails;
    WireDetails      : tWireDetails;
    FilePosition     : tFileFrameLocation;
    TimeResolution   : byte;
    ExpResolution    : integer;
    HelpDate         : tDateTime;
    HelpValue        : int64;
    HelpWord         : pWord;
    HelpLong         : pLongword;
    HelpUint64       : pUint64;
    HelpFlags        : longword;
    OptionSize       : integer;
    LenWithoutOpts   : integer;
    OptionHdr        : pNGOptionCodeLen;
    OptionsProcessed : boolean;
    HelpChar         : pAnsiChar;
    HelpString       : string;
    Bit32length      : integer;
    InterfaceEntry   : tCaptureInterface;
//  FrameComment     : tFrameComment;
    CommentList      : tStringlist;

begin
     // Length check for block header

     if Offset + (SizeOf (NGEnhPacketBlock) + 4) > iFileSize then             // +4 for trailing TotalBlockLength
        begin
             LoaderStatus := lsIncompleteFile;

             ReportMessage (msCritical, 'incomplete pcapng file, cannot read Enhanced Packet Block', 'tPCAPNGLoader.readEnhancedPacketBlock', FrameIndex);
             exit (nil);
        end;

     CommentList := tStringlist.Create;

     EnhPacketHdr := pNGEnhPacketBlock (Position);
     LocalOffset := sizeOf (NGEnhPacketBlock);

     // Length check for block content

     if (Offset + EnhPacketHdr.BlockTotalLength) > iFileSize then
        begin
             LoaderStatus := lsIncompleteFile;

             ReportMessage (msCritical, 'incomplete pcapng file, cannot read Enhanced Packet Block', 'tPCAPNGLoader.readEnhancedPacketBlock', FrameIndex);
             exit (nil);
        end;

     if EnhPacketHdr.BlockTotalLength <= EnhPacketHdr.PacketLen then
        ReportMessage (msCritical, 'invalid header values found in frame #' + IntToStr (FrameIndex) + ' - total length shorter than packet length', 'tPCAPNGLoader.readEnhancedPacketBlock', FrameIndex);

     WireDetails.WireSize := EnhPacketHdr.PacketLen;
     WireDetails.CaptureSize := EnhPacketHdr.CapturedLen;
     WireDetails.InterfaceID := EnhPacketHdr.InterfaceID;

     // Get Interface entry, if possible - if not, use the first interface we have
     // Reminder: Interfaces entries start at index 0

     try
        if WireDetails.InterfaceID > (InterfaceList.Count - 1) then
           begin
                ReportMessage (msCritical, 'Index ' + IntToStr (WireDetails.InterfaceID) + ' points to unknown interface in frame #' + IntToStr (FrameIndex) + ', using first interface instead', 'tPCAPNGLoader.readEnhancedPacketBlock', FrameIndex);

                InterfaceEntry := InterfaceList.getInterfaceByIndex(0);
                WireDetails.LinkID := InterfaceList.getInterfaceByIndex(0).PacketLinkID;
           end
        else
            begin
                 InterfaceEntry := InterfaceList.getInterfaceByIndex(EnhPacketHdr.InterfaceID + priv_interfacemodifier);
                 WireDetails.LinkID := InterfaceList.getInterfaceByIndex(WireDetails.InterfaceID).PacketLinkID;
            end;
     except
           ReportMessage (msCritical, 'Index ' + IntToStr (WireDetails.InterfaceID) + ' points to unknown interface in frame #' + IntToStr (FrameIndex) + ', cannot read Enhanced Packet Block', 'tPCAPNGLoader.readEnhancedPacketBlock', FrameIndex);
     end;

     WireDetails.FrameType := tftPCAPngEnhanced;
     WireDetails.FrameFlags.Value := 0;

     // Set File position stuff

     FilePosition.FileIndex := iFileNumber;                             // Taken from traceloader object - it knows what index the current file has
     FilePosition.FileFrameIndex := FrameIndex;
     FilePosition.FileOffset := Offset;

     // Do some sanity checks

     DoSanityCheck (WireDetails, FilePosition);

     // Calculate timings

     try
        TimestampUnits := getTimestampUnits (EnhPacketHdr.TimestampHigh, EnhPacketHdr.TimestampLow);
        TimeResolution := InterfaceEntry.TimeStampResolution;
        TimingDetails := getPCAPNGTiming (TimeResolution, TimeStampUnits);
     except
           SiMain.LogError ('Calculation of timings failed for frame #' + IntToStr (FrameIndex));
     end;

     WireDetails.LinkType := InterfaceEntry.LinkType;

     // Read frame bytes

     if WireDetails.CaptureSize > maxFrameSize then
        ReportMessage (msCritical, 'oversized frame encountered in frame #' + IntToStr (FrameIndex), 'tPCAPNGLoader.readEnhancedPacketBlock', FrameIndex);

     if iFileSize < (Offset + WireDetails.CaptureSize) then
        begin
             raise EInOutError.Create ('Error: unexpected end of file in frame # ' + IntToStr (FrameIndex) + ' (tPCAPNGLoader.readEnhancedPacketBlock).');

             exit (nil);
        end;

     pFrameBuffer := pRawBuffer (Position + LocalOffset);
     move (pFrameBuffer^, helpBuf, min (maxFrameSize, WireDetails.CaptureSize));

     LocalOffset := LocalOffset + Get32BitAlignment (EnhPacketHdr.CapturedLen);

     // Check if there are options. We do that by checking if the Total Length includes
     // 1. the block header itself
     // 2. the captured bytes, aligned to 32 bit
     // 3. the additional total block size value at the end (4 bytes)
     //
     // If there is more, we have options.

     LenWithoutOpts := sizeOf (NGEnhPacketBlock) + Get32BitAlignment (WireDetails.CaptureSize) + 4;
     OptionSize := EnhPacketHdr.BlockTotalLength - LenWithoutOpts;

     if OptionSize > 0 then
        begin
             OptionOffset := 0;

             repeat
                   OptionHdr := pNGOptionCodeLen (Position + LocalOffset);
                   LocalOffset := LocalOffset + sizeOf (NGOptionCodeLen);

                   // Calculate the offset we're at inside the option block and verify that we're not getting into trouble by reading an oversized option

                   OptionOffset := OptionOffset + sizeOf (NGOptionCodeLen);

                   if (OptionOffset + OptionHdr.length) > OptionSize then
                      ReportMessage (msWarning, 'Damaged pcapng file, option size is longer than the option block (Option Code #' + IntToStr (OptionHdr.code) + ') for enhanced frame #' + IntToStr (FrameIndex), 'tPCAPNGLoader.readEnhancedPacketBlock', FrameIndex);

                   // Process packet option

                   case OptionHdr^.code of
                    opt_endofopt       : OptionsProcessed := true;
                    opt_comment        : begin
                                              HelpChar := pANSIChar (Position + LocalOffset);
                                              HelpString := string (HelpChar);

                                              HelpString := copy (HelpString, 1, OptionHdr^.length);

                                              CommentList.Add(HelpString);
                                         end;
                    opt_epb_flags      : begin
                                              HelpLong := pLongword (Position + LocalOffset);
                                              HelpFlags := HelpLong^;

                                              WireDetails.FrameFlags.Present := true;
                                              WireDetails.FrameFlags.Value := HelpFlags;
                                         end;
                    opt_epb_hash       : begin
                                              ReportMessage (msWarning, 'enhanced packet hash option found in frame #' + IntToStr (FrameIndex) + ' - cannot parse option', 'tPCAPNGLoader.readEnhancedPacketBlock', FrameIndex);
                                         end;
                    opt_epb_dropcount  : begin
                                              HelpUint64 := pUint64 (Position + LocalOffset);
                                              WireDetails.Dropped := HelpUint64^;
                                         end;
                   end;

                   // Options are variable length, but aligned to 32bits, so we need to find the aligned jump distance

                   Bit32Length := Get32BitAlignment (OptionHdr^.length);

                   LocalOffset := LocalOffset + Bit32Length;
                   OptionOffset := OptionOffset + Bit32Length;

                   // Failsave in case there is no opt_endofopt

                   if OptionOffset >= OptionSize then
                      OptionsProcessed := true;

             until OptionsProcessed;
        end
     else
         if OptionSize < 0 then
            begin
                 ReportMessage (msWarning, 'invalid enhanced packet header length found in frame #' + IntToStr (FrameIndex) + ': was ' + IntToStr (EnhPacketHdr.BlockTotalLength) + ' but should be ' + IntToStr (LenWithoutOpts), 'tPCAPNGLoader.readEnhancedPacketBlock', FrameIndex);


            end;

     // Create new Frame Object that will hold the data

     NewFrame := tRawFrame.create (TimingDetails, WireDetails, FilePosition, helpBuf);

     // Move the offset

     Offset := Offset + EnhPacketHdr.BlockTotalLength;
     Position := Position + EnhPacketHdr.BlockTotalLength;

     // If no comment was found (and thus assigned to the tRawFrame, we need to discard it to avoid memory leaks
     // otherwise we assign it to the frame

     if CommentList.Count = 0 then
        FreeAndNil (CommentList)
     else
         NewFrame.Commentlist := CommentList;

     // Give it back

     result := NewFrame;
end;

(* ------------------------------------------- *)

function tPCAPNGLoader.readInterfaceStatisticsBlock (var Position: pByte; var Offset: integer; InterfaceResolutionList : tNGInterfaceResolutionList) : tStatisticsRecord;


var InterfaceStatHdr  : pNGInterfaceStatisticsBlock;
    LocalOffset       : integer;
    OptionHdr         : pNGOptionCodeLen;
    OptionSize        : integer;
    OptionOffset      : integer;
    OptionsProcessed  : boolean;
    OptionTimestamp   : pNGInterfaceStatisticsTimestamp;
    TimeResolution    : byte;
    TimestampUnits    : uint64;
    InterfaceEntry    : tCaptureInterface;
    StatRecord        : tStatisticsRecord;
    Bit32Length       : integer;
    HelpString        : pAnsiChar;
    HelpUint64        : pUint64;
    Comment           : string;
    TrySwapped        : boolean;

begin
     // Length check

     if Offset + (SizeOf (NGInterfaceStatisticsBlock) + 4) > iFileSize then             // +4 for trailing TotalBlockLength
        begin
             LoaderStatus := lsIncompleteFile;

             ReportMessage (msCritical, 'incomplete pcapng file, cannot read Interface Statistics Block', 'tPCAPNGLoader.readInterfaceStatisticsBlock');
             exit (nil);
        end;

     InterfaceStatHdr := pNGInterfaceStatisticsBlock (Position);
     LocalOffset := sizeOf (NGInterfaceStatisticsBlock);

     // Length check for block content

     if (Offset + InterfaceStatHdr.BlockTotalLength) > iFileSize then
        begin
             LoaderStatus := lsIncompleteFile;

             ReportMessage (msCritical, 'incomplete pcapng file, cannot read Interface Statistics Block', 'tPCAPNGLoader.readInterfaceStatisticsBlock');
             exit (nil);
        end;

     // Find the according Interface entry to get the timestamp resolution

     TimestampUnits := getTimestampUnits (InterfaceStatHdr.TimestampHigh, InterfaceStatHdr.TimestampLow);
     TimeResolution := InterfaceResolutionList.items[InterfaceStatHdr.InterfaceID];

     // Create statistics record entry

     StatRecord := tStatisticsRecord.create (InterfaceStatHdr.InterfaceID);
     StatRecord.ReferenceTime := getPCAPNGTiming (TimeResolution, TimestampUnits);

     // Test for options - if we find the total length value again we don't have any

     OptionSize := InterfaceStatHdr.BlockTotalLength - sizeOf (NGInterfaceStatisticsBlock) - 4;
     OptionOffset := 0;

     if (OptionSize > 0) then
        begin
             OptionsProcessed := false;

             repeat
                   OptionHdr := pNGOptionCodeLen (Position + LocalOffset);
                   LocalOffset := LocalOffset + sizeOf (NGOptionCodeLen);
                   OptionOffset  := OptionOffset + sizeOf (NGOptionCodeLen);

                   if (OptionOffset + OptionHdr.length) > OptionSize then
                      ReportMessage (msCritical, 'Damaged pcapng file, option size is longer than the option block (Option Code #' + IntToStr (OptionHdr.code) + ')', 'tPCAPNGLoader.readInterfaceStatisticsBlock');

                   case OptionHdr^.code of
                    opt_endofopt     : OptionsProcessed := true;
                    opt_comment      : begin
                                            HelpString := pANSIChar (Position + LocalOffset);
                                            Comment := string (HelpString);

                                            StatRecord.Comment := copy (Comment, 1, OptionHdr.length);
                                       end;
                    isb_starttime    : begin
                                            TrySwapped := False;
                                            OptionTimestamp := pNGInterfaceStatisticsTimestamp (Position + LocalOffset);

                                            // Problem: Wireshark write the timestamp in the wrong order, low first.

                                            try
                                               TimestampUnits := getTimestampUnits (OptionTimestamp^.TimestampHigh, OptionTimestamp^.TimestampLow);
                                               StatRecord.StartTime := getPCAPNGTiming (TimeResolution, TimeStampUnits);
                                            except
                                                  TrySwapped := True;
                                                  SiMain.LogError ('Reading interface statistics start time failed');
                                            end;

                                            if TrySwapped then
                                               try
                                                  TimestampUnits := getTimestampUnits (OptionTimestamp^.TimestampLow, OptionTimestamp^.TimestampHigh);
                                                  StatRecord.StartTime := getPCAPNGTiming (TimeResolution, TimeStampUnits);
                                               except
                                                     SiMain.LogError ('Reading interface statistics start time in swapped mode failed, too');
                                               end;
                                       end;
                    isb_endtime      : begin
                                            TrySwapped := False;
                                            OptionTimestamp := pNGInterfaceStatisticsTimestamp (Position + LocalOffset);

                                            try
                                               TimestampUnits := getTimestampUnits (OptionTimestamp^.TimestampHigh, OptionTimestamp^.TimestampLow);
                                               StatRecord.EndTime := getPCAPNGTiming (TimeResolution, TimeStampUnits);
                                            except
                                                  TrySwapped := True;
                                                  SiMain.LogError ('Reading interface statistics end time failed');
                                            end;

                                            if TrySwapped then
                                               try
                                                  TimestampUnits := getTimestampUnits (OptionTimestamp^.TimestampLow, OptionTimestamp^.TimestampHigh);
                                                  StatRecord.EndTime := getPCAPNGTiming (TimeResolution, TimeStampUnits);
                                               except
                                                     SiMain.LogError ('Reading interface statistics end time in swapped mode failed, too');
                                               end;
                                       end;
                    isb_ifrecv       : begin
                                            HelpUint64 := pUInt64 (Position + LocalOffset);

                                            // Problem: Number seems to be +4

                                            StatRecord.PacketsReceived := HelpUint64^;
                                       end;
                    isb_ifdrop       : begin
                                            HelpUint64 := pUInt64 (Position + LocalOffset);

                                            StatRecord.PacketsDropped := HelpUint64^;
                                       end;
                    isb_filteraccept : begin
                                            HelpUint64 := pUInt64 (Position + LocalOffset);

                                            StatRecord.FilterAccepted := HelpUint64^;
                                       end;
                    isb_osdrop       : begin
                                            HelpUint64 := pUInt64 (Position + LocalOffset);

                                            StatRecord.OSDropped := HelpUint64^;
                                       end;
                    isb_usrdeliv     : begin
                                            HelpUint64 := pUInt64 (Position + LocalOffset);

                                            StatRecord.UserDelivered := HelpUint64^;
                                       end;
                   end;

                   // Options are variable length, but aligned to 32bits, so we need to find the aligned jump distance

                   Bit32Length := Get32BitAlignment (OptionHdr^.length);

                   LocalOffset := LocalOffset + Bit32Length;
                   OptionOffset := OptionOffset + Bit32Length;

                   if OptionOffset >= OptionSize then
                      OptionsProcessed := true;

             until OptionsProcessed;
        end;

     result := StatRecord;
end;

(* ------------------------------------------- *)

procedure tPCAPNGLoader.readBatchFromDisk;

var pBlockType      : pLongWord;
    pBlockLength    : pLongWord;
    InterfaceEntry  : tCaptureInterface;
    StartTime       : tDateTime;
    iDuration       : integer;
    boReadDone      : boolean;
    NewFrame        : tRawFrame;
    StatRecord      : tStatisticsRecord;

begin
     // First we clear the frames we have in memory. but we keep the frame history

     FrameList_ValidFrames.Clear;

     // Some initialisations

     StartTime := now;

     iFramesAvailable := 0;
     iFramesTossed := 0;
     iFramesDropped := 0;

     boReadDone := false;

     if boEOF then
        exit;

     // Being here we should have read the file header and perhaps another batch of frames, so the iOffset needs to
     // be greater than 0

     if iBatchFileOffset = 0 then
        begin
             raise EInOutError.Create('Error: invalid file offset - file header not read? (PCAPNGLoader.readbatchfromdisk).');

             exit;
        end;

     if (iBatchFileOffset < iFileSize) then
        begin
             repeat
                   // Check File Length if a record header is available

                   if iFileSize < (iBatchFileOffset + 4) then
                      begin
                           raise EInOutError.Create ('Error: unexpected end of file (PCAPNGLoader.readbatchfromdisk).');

                           exit;
                      end;

                   // Read Header

                   pBlockType := pLongWord (pBatchPosition);
                   pBlockLength := pLongWord (pBatchPosition + 4);

                   case pBlockType^ of
                    BLOCKTYPE_SECTION_HEADER        : begin
                                                           ReportMessage (msWarning, 'found additional Section Block Header at offset ' + IntToStr (iBatchFileOffset), 'tPCAPNGLoader.readBatchFromDisk');

                                                           ReadMasterBlocks (pBatchPosition, iBatchFileOffset);
                                                      end;
                    BLOCKTYPE_INTERFACE_DESCRIPTION : begin
                                                           InterfaceEntry := readInterfaceDescriptionHeader (pBatchPosition, iBatchFileOffset);

                                                           if InterfaceEntry <> nil then
                                                              InterfaceList.Add (InterfaceEntry);
                                                      end;
                    BLOCKTYPE_PACKET                : begin
                                                           inc (iFramesAvailable);
                                                           inc (iTotalFramesRead);

                                                           NewFrame := readPacketBlock (iTotalFramesRead, pBatchPosition, iBatchFileOffset);

                                                           if NewFrame <> nil then
                                                              self.addRawFrame(NewFrame)
                                                           else
                                                               begin
                                                                    dec (iFramesAvailable);
                                                                    dec (iTotalFramesRead);
                                                               end;
                                                      end;
                    BLOCKTYPE_SIMPLE_PACKET         : begin
                                                           inc (iFramesAvailable);
                                                           inc (iTotalFramesRead);

                                                           NewFrame := readSimplePacketBlock (iTotalFramesRead, pBatchPosition, iBatchFileOffset);

                                                           if NewFrame <> nil then
                                                              self.addRawFrame(NewFrame)
                                                           else
                                                               begin
                                                                    dec (iFramesAvailable);
                                                                    dec (iTotalFramesRead);
                                                               end;
                                                      end;
                    BLOCKTYPE_ENHANCED_PACKET       : begin
                                                           inc (iFramesAvailable);
                                                           inc (iTotalFramesRead);

                                                           try
                                                              NewFrame := readEnhancedPacketBlock (iTotalFramesRead, pBatchPosition, iBatchFileOffset);
                                                           except
                                                                 SiMain.LogFatal ('Error reading EHB from disk for frame #' + IntToStr (iFramesAvailable));
                                                           end;

                                                           if NewFrame <> nil then
                                                              self.addRawFrame(NewFrame)
                                                           else
                                                               begin
                                                                    dec (iFramesAvailable);
                                                                    dec (iTotalFramesRead);
                                                               end;
                                                      end;
                    BLOCKTYPE_NAME_RESOLUTION       : readNameResolutionBlockHeader (pBatchPosition, iBatchFileOffset);
                    BLOCKTYPE_INTERFACE_STATISTICS  : begin
                                                           StatRecord := readInterfaceStatisticsBlock (pBatchPosition, iBatchFileOffset, priv_interfaceResList);

                                                           if assigned (StatRecord) then
                                                              begin
                                                                   // Add the new record to the according Interface

                                                                   InterfaceEntry := InterfaceList.getInterfaceByIndex (StatRecord.InterfaceIndex + priv_interfacemodifier);
                                                                   InterfaceEntry.InterfaceStatistics.addEntry (StatRecord);
                                                              end;

                                                           // Move position pointer - this should work even if it has options

                                                           inc (iBatchFileOffset, pBlockLength^);
                                                           inc (pBatchPosition, pBlockLength^);
                                                      end;
                   else
                       raise EInOutError.Create ('Warning: found unhandled blocktype ' + IntToStr (pBlockType^) + ', cannot process.');
                   end;

                   if boFrameMemLimited and (iFramesAvailable >= iMaxFramesinMem) then
                      boReadDone := true;

                   if (iBatchFileOffset >= iFileSize) or (LoaderStatus <> lsSuccess) then
                      begin
                           boReadDone := true;
                           boEOF := true;
                      end;

             until boReadDone;
        end
     else
         boEOF := true;

     iDuration := SecondsBetween (StartTime, now);

     if iDuration > 0 then
        iFPS := iFramesAvailable Div iDuration
     else
         iFPS := iFramesAvailable;
end;

(* ------------------------------------------- *)

function tPCAPNGLoader.readSingleFrameByOffset(const Offset: integer; FrameIndex: integer = -1): tRawFrame;

var pBlockType : pLongWord;
    NewFrame   : tRawFrame;
    pLocalPos  : pByte;
    HelpOffset : integer;

begin
     HelpOffset := Offset; // Necessary because ParseFrameFromOffset will need to modify it, but it will be discarded afterwards anyway

     // Set the local pointer to the required Offset

     pLocalPos := pByte (pData);
     inc (pLocalPos, Offset);

     // Check File Length if a record header is available

     if iFileSize < (Offset + 4) then
        begin
             raise EInOutError.Create ('Error: unexpected end of file (PCAPNGLoader.readSingleFrameByOffset).');

             exit;
        end;

     // Read Header

     pBlockType := pLongWord (pLocalPos);

     case pBlockType^ of
      BLOCKTYPE_PACKET                : begin
                                             inc (iFramesAvailable);

                                             NewFrame := readPacketBlock (FrameIndex, pLocalPos, HelpOffset);
                                        end;
      BLOCKTYPE_SIMPLE_PACKET         : begin
                                             inc (iFramesAvailable);

                                             NewFrame := readSimplePacketBlock (FrameIndex, pLocalPos, HelpOffset);
                                        end;
      BLOCKTYPE_ENHANCED_PACKET       : begin
                                             inc (iFramesAvailable);

                                             NewFrame := readEnhancedPacketBlock (FrameIndex, pLocalPos, HelpOffset);
                                        end;
     else
         raise EInOutError.Create ('Warning: found unhandled blocktype ' + IntToStr (pBlockType^) + ', cannot process (PCAPNGLoader.readSingleFrameByOffset).');
     end;

     result := newFrame;
end;

(* ------------------------------------------- *)

function tPCAPNGLoader.readSingleFrameFromDisk (const FrameIndex: integer): tRawFrame;

var pBlockType      : pLongWord;
    pBlockTotalLen  : pLongWord;
    pLocalPos       : pByte;
    iLocalOffset    : integer;
    boReadDone      : boolean;
    NewFrame        : tRawFrame;
    iFrameNumber    : integer;

begin
     boReadDone := false;

     iFrameNumber := 0;
     iLocalOffset := 0;
     pLocalPos := pByte (pData);
     NewFrame := nil;

     ReadMasterBlocks (pBatchPosition, iBatchFileOffset);

     // Being here we should have read the file header and perhaps another batch of frames, so the iOffset needs to
     // be greater than 0

     if iBatchFileOffset = 0 then
        begin
             raise EInOutError.Create('Error: invalid file offset - file header not read? (PCAPNGLoader.readSingleFrameFromDisk).');

             exit (nil);
        end;

     repeat
           // Check File Length if a record header is available

           if iFileSize < (iLocalOffset + 4) then
              begin
                   raise EInOutError.Create ('Error: unexpected end of file (PCAPNGLoader.readSingleFrameFromDisk).');

                   exit (nil);
              end;

           // Read Block Header

           pBlockType := pLongWord (pLocalPos);
           pBlockTotalLen := pLongWord (pLocalPos + 4);

           case pBlockType^ of
            BLOCKTYPE_PACKET                : begin
                                                   inc (iFrameNumber);

                                                   if iFrameNumber = FrameIndex then
                                                      begin
                                                           NewFrame := readPacketBlock (FrameIndex, pLocalPos, iLocalOffset);
                                                           boReadDone := true;
                                                      end;
                                              end;
            BLOCKTYPE_SIMPLE_PACKET         : begin
                                                   inc (iFrameNumber);

                                                   if iFrameNumber = FrameIndex then
                                                      begin
                                                           NewFrame := readSimplePacketBlock (FrameIndex, pLocalPos, iLocalOffset);
                                                           boReadDone := true;
                                                      end;

                                              end;
            BLOCKTYPE_ENHANCED_PACKET       : begin
                                                   inc (iFrameNumber);

                                                   if iFrameNumber = FrameIndex then
                                                      begin
                                                           NewFrame := readEnhancedPacketBlock (FrameIndex, pLocalPos, iLocalOffset);
                                                           boReadDone := true;
                                                      end;
                                              end;
           end;

           iLocalOffset := iLocalOffset + pBlockTotalLen^;
           pLocalPos := pLocalPos + pBlockTotalLen^;

           if (iLocalOffset >= iFileSize) then
              begin
                   boReadDone := true;
                   boEOF := true;
              end;

     until boReadDone;

     result := NewFrame;
end;

(* ------------------------------------------- *)

function tPCAPNGLoader.readLastFrameFromDisk: tRawFrame;

var pBlockType      : pLongWord;
    pBlockTotalLen  : pLongWord;
    pLocalPos       : pByte;
    iLocalOffset    : integer;
    NewFrame        : tRawFrame;
    found           : boolean;
    abort           : boolean;

begin
     NewFrame := nil;
     found := false;
     abort := false;

     // Position the offset to the last 4 bytes in the file. We know the filesize since
     // it is stored to iFileSize when opening the trace in class_traceloader

     iLocalOffset := iFileSize - 4;

     pLocalPos := pByte (pData) + iLocalOffset;

     repeat
           // Read the length of the block

           pBlockTotalLen := pLongWord (pLocalPos);

           // Do some sanity checks, normally the block sizes shouldn't exceed max frame size

           if pBlockTotalLen^ <= maxFrameSize then
              begin
                   // Jump to the beginning of the block

                   pLocalPos := pLocalPos - pBlockTotalLen^ + 4;
                   iLocalOffset := iLocalOffset - pBlockTotalLen^ + 4;

                   pBlockType := pLongWord (pLocalPos);

                   // If it is a statistics block we read it

                   if pBlockType^ = BLOCKTYPE_ENHANCED_PACKET then
                      begin
                           NewFrame := readEnhancedPacketBlock (1, pLocalPos, iLocalOffset);

                           if NewFrame <> nil then
                              found := true;
                      end
                   else
                       begin
                            // Position to the length indicator at the end of the preceeding block

                            pLocalPos := pLocalPos - 4;
                            iLocalOffset := iLocalOffset - 4;
                       end;
              end
           else
               abort := true;

     until found or abort or (iLocalOffset < 0);

     result := NewFrame;
end;

(* ------------------------------------------- *)

// Determines more detailed information about a block if available

procedure tPCAPNGLoader.DetermineChainBlockDetails (var Entry: pBlockChainEntry);

var SHBValues       : tNGSHBParsedValues;
    InterfaceEntry  : tCaptureInterface;
    StatRecord      : tStatisticsRecord;
    NRBValueSet     : tNGNRBParsedValues;
    pLocalPos       : pByte;
    iLocalOffset    : integer;            // TODO: deal with int64 values
    boHelp          : boolean;

begin
     // Store the settings we need to adjust for this routine; to be able to set it back afterwards

     boHelp := priv_keepUTCTime;

     priv_keepUTCTime := true;

     // Transfer offset values. They get moved around in the reading function, so
     // we do this to prevent changes to the values in "Entry".

     pLocalPos := Entry^.FirstBlockStartPointer;
     iLocalOffset := Entry^.FirstBlockStartOffset;

     case Entry^.BlockTypeID of
      BLOCKTYPE_SECTION_HEADER        : begin
                                             SHBValues := readSectionBlockHeader (pLocalPos, iLocalOffset);

                                             Entry^.SHBMajorVersion := SHBValues.MajorVersion;
                                             Entry^.SHBMinorVersion := SHBValues.MinorVersion;
                                             Entry^.SHBByteOrderMagic := SHBValues.ByteOrderMagic;
                                             Entry^.SHBSectionLength := SHBValues.TotalSectionLength;

                                             if SHBValues.OptionValues.HeaderCommentList.Count > 0 then
                                                Entry^.SHBComment := SHBValues.OptionValues.HeaderCommentList.CommaText;

                                             if SHBValues.OptionValues.HardwareInfoList.Count > 0 then
                                                Entry^.SHBHardware := SHBValues.OptionValues.HardwareInfoList.CommaText;

                                             if SHBValues.OptionValues.OperatingSystemList.Count > 0 then
                                                Entry^.SHBOS := SHBValues.OptionValues.OperatingSystemList.CommaText;

                                             if SHBValues.OptionValues.UserApplicationList.Count > 0 then
                                                Entry^.SHBUserAppl := SHBValues.OptionValues.UserApplicationList.CommaText;
{
                                             if SHBValues.OptionValues.CaptureSystemGUID <> VALUE_ZERO_GUID then
                                                Entry^.SHBGUID := SHBValues.OptionValues.CaptureSystemGUID;
}
                                             SHBValues.OptionValues.cleanUp;
                                        end;
      BLOCKTYPE_INTERFACE_DESCRIPTION : begin
                                             InterfaceEntry := readInterfaceDescriptionHeader (pLocalPos, iLocalOffset);

                                             Entry^.IDBIndex := InterfaceEntry.OriginalID;
                                             Entry^.IDBLinkType := InterfaceEntry.LinkType;
                                             Entry^.IDBSnapLen := InterfaceEntry.SnapLength;
                                             Entry^.IDBFCSLen := InterfaceEntry.FCSLength;
                                             Entry^.IDBName := InterfaceEntry.Name;
                                             Entry^.IDBDescription := InterfaceEntry.Description;
                                             Entry^.IDBTSResolution := InterfaceEntry.TimeStampResolution;
                                             Entry^.IDBFilter := InterfaceEntry.CaptureFilter;
                                             Entry^.IDBMACAddress := InterfaceEntry.MACAddress;
                                             Entry^.IDBIPv4AddressList := InterfaceEntry.IPv4Addresses;
                                             Entry^.IDBIPv6AddressList := InterfaceEntry.IPv6Addresses;

                                             InterfaceEntry.free;
                                             InterfaceEntry := nil;
                                        end;
      BLOCKTYPE_PACKET                : begin
                                        end;
      BLOCKTYPE_SIMPLE_PACKET         : begin
                                        end;
      BLOCKTYPE_ENHANCED_PACKET       : begin
                                        end;
      BLOCKTYPE_NAME_RESOLUTION       : begin
                                             NRBValueSet := readNameResolutionBlockHeader (pLocalPos, iLocalOffset);

                                             Entry^.NRBIPv4RecordCount := NRBValueSet.IPv4AddressList.Count;
                                             Entry^.NRBIPv6RecordCount := NRBValueSet.IPv6AddressList.Count;
                                             Entry^.NRBIPv4AddressList := NRBValueSet.IPv4AddressList;
                                             Entry^.NRBIPv6AddressList := NRBValueSet.IPv6AddressList;

//                                           NRBValueSet.cleanUp;
                                        end;
      BLOCKTYPE_INTERFACE_STATISTICS  : begin
                                             StatRecord := readInterfaceStatisticsBlock(pLocalPos, iLocalOffset, priv_interfaceResList);


                                             if assigned (StatRecord) then
                                                begin
                                                     Entry^.ISBInterfaceID := StatRecord.InterfaceIndex;
                                                     Entry^.ISBTimestamp := StatRecord.ReferenceTime.AbsoluteTime;
                                                     Entry^.ISBStartTime := StatRecord.StartTime.AbsoluteTime;
                                                     Entry^.ISBEndTime := StatRecord.EndTime.AbsoluteTime;
                                                     Entry^.ISBReceived := StatRecord.PacketsReceived;
                                                     Entry^.ISBDropped := StatRecord.PacketsDropped;

                                                     StatRecord.free;
                                                     StatRecord := nil;
                                                end;
                                        end;
      BLOCKTYPE_IRIG_TIMESTAMP        : begin
                                        end;
      BLOCKTYPE_ARINC_429_IN_AFDX     : begin
                                        end;
      BLOCKTYPE_HONE_PROCESS_EVENT    : begin
                                        end;
      BLOCKTYPE_HONE_CONNECTION_EVENT : begin
                                        end;
      BLOCKTYPE_SYSDIG_MACHINE_INFO   : begin
                                        end;
      BLOCKTYPE_SYSDIG_PROCESS_LIST   : begin
                                        end;
      BLOCKTYPE_SYSDIG_FD_LIST        : begin
                                        end;
      BLOCKTYPE_SYSDIG_EVENT          : begin
                                        end;
      BLOCKTYPE_SYSDIG_INTERFACE_LIST : begin
                                        end;
      BLOCKTYPE_SYSDIG_USER_LIST      : begin
                                        end;
     else
         raise EInOutError.Create ('Warning: found unhandled blocktype ' + IntToStr (Entry^.BlockTypeID) + ', cannot process.');
     end;

     priv_keepUTCTime := boHelp;
end;

(* ------------------------------------------- *)

function tPCAPNGLoader.GetBlockChain : pBlockChainEntry;

var pBlockType      : pLongWord;
    pBlockTotalLen  : pLongWord;
    pLocalPos       : pByte;
    iLocalOffset    : integer;
    boReadDone      : boolean;
    StartEntry      : pBlockChainEntry;
    CurrentEntry    : pBlockChainEntry;
    NewEntry        : pBlockChainEntry;
    CurrentSHBEntry : pBlockChainEntry;
    i               : integer;

begin
     iLocalOffset := 0;
     pLocalPos := pByte (pData);

     boReadDone := false;
     StartEntry := nil;

     repeat
           if (iLocalOffset + 8) > iFileSize then
              begin
                   LoaderStatus := lsIncompleteFile;

                   ReportMessage (msCritical, 'incomplete pcapng file, aborting BlockChain tracking', 'tPCAPNGLoader.GetBlockChain');
                   exit (nil);
              end;

           pBlockType := pLongWord (pLocalPos);
           pBlockTotalLen := pLongWord (pLocalPos + 4);

           if StartEntry = nil then
              begin
                   new (StartEntry);
                   StartEntry^.initialize;

                   CurrentEntry := StartEntry;

                   CurrentEntry^.BlockTypeID := pBlockType^;
                   CurrentEntry^.BlockTypeName := getBlockTypeName (CurrentEntry^.BlockTypeID);
                   CurrentEntry^.BlockLength := pBlockTotalLen^;
                   CurrentEntry^.FirstBlockStartOffset := iLocalOffset;
                   CurrentEntry^.FirstBlockStartPointer := pLocalPos;
                   CurrentEntry^.LastBlockEndOffset := iLocalOffset + pBlockTotalLen^ - 1;
                   CurrentEntry^.SectionStartOffset := 0;
                   CurrentEntry^.SectionEndOffset := 0;
                   CurrentEntry^.Count := 1;

                   CurrentEntry^.PreviousBlockType := nil;
                   CurrentEntry^.NextBlockType := nil;
              end
           else
               begin
                    if (CurrentEntry^.BlockTypeID = pBlockType^) and TreatAsRepeatingBlock (CurrentEntry^.BlockTypeID) then
                       begin
                            // If it is the same block type like the last AND a packet block, just increase the counter
                            // and note at what offset the block chain ends. For packet blocks that means it is the offset
                            // of the last byte of the last packet block

                            inc (CurrentEntry^.Count);
                            CurrentEntry^.LastBlockEndOffset := iLocalOffset + pBlockTotalLen^ - 1;
                       end
                    else
                        begin
                             // Create new entry

                             new (NewEntry);
                             NewEntry^.initialize;

                             CurrentEntry^.NextBlockType := NewEntry;
                             NewEntry^.PreviousBlockType := CurrentEntry;

                             CurrentEntry := NewEntry;

                             CurrentEntry^.BlockTypeID := pBlockType^;
                             CurrentEntry^.BlockTypeName := getBlockTypeName (CurrentEntry^.BlockTypeID);
                             CurrentEntry^.BlockLength := pBlockTotalLen^;
                             CurrentEntry^.FirstBlockStartOffset := iLocalOffset;
                             CurrentEntry^.FirstBlockStartPointer := pLocalPos;
                             CurrentEntry^.LastBlockEndOffset := iLocalOffset + pBlockTotalLen^ - 1;
                             CurrentEntry^.SectionStartOffset := 0;
                             CurrentEntry^.SectionEndOffset := 0;
                             CurrentEntry^.Count := 1;

                             CurrentEntry^.NextBlockType := nil;
                        end;
               end;

           // Update current SHB entry to keep track of the last block in the section

           if CurrentEntry.BlockTypeID = BLOCKTYPE_SECTION_HEADER then
              begin
                   CurrentSHBEntry := CurrentEntry;
                   CurrentSHBEntry^.SectionStartOffset := CurrentSHBEntry^.FirstBlockStartOffset;
                   CurrentSHBEntry^.SectionEndOffset := CurrentSHBEntry^.LastBlockEndOffset;
              end
           else
               CurrentSHBEntry^.SectionEndOffset := iLocalOffset + pBlockTotalLen^ - 1;

           // Get further details if available

           DetermineChainBlockDetails (CurrentEntry);

           // Move the pointers

           iLocalOffset := iLocalOffset + pBlockTotalLen^;
           pLocalPos := pLocalPos + pBlockTotalLen^;

           if (iLocalOffset >= iFileSize) then
              begin
                   boReadDone := true;
                   boEOF := true;
              end;

     until boReadDone;

     result := StartEntry;
end;

(* ------------------------------------------- *)

// Looks for packet blocks in the chain and adds their packet counts

function tPCAPNGLoader.getPacketBlockCountFromBlockChain (ChainRoot : pBlockChainEntry) : integer;

var currentEntry : pBlockChainEntry;
    count        : integer;

begin
     currentEntry := ChainRoot;
     count := 0;

     repeat
           if CurrentEntry <> nil then
              begin
                   // If the current entry is a packet block entry we add up all the packets
                   // it spanned

                   if TreatAsRepeatingBlock(CurrentEntry.BlockTypeID) then
                      count := count + CurrentEntry.Count;

                   CurrentEntry := CurrentEntry.NextBlockType;
              end;

     until (CurrentEntry = nil);

     result := count;
end;

(* ------------------------------------------- *)

// Counts SHB blocks in the chain

function tPCAPNGLoader.getSectionHeaderCountFromBlockChain (ChainRoot : pBlockChainEntry) : integer;

var currentEntry : pBlockChainEntry;
    count        : integer;

begin
     currentEntry := ChainRoot;
     count := 0;

     repeat
           if CurrentEntry <> nil then
              begin
                   if CurrentEntry.BlockTypeID = BLOCKTYPE_SECTION_HEADER then
                      count := count + 1;

                   CurrentEntry := CurrentEntry.NextBlockType;
              end;

     until (CurrentEntry = nil);

     result := count;
end;

(* ------------------------------------------- *)

function tPCAPNGLoader.getInterfaceCountFromBlockChain (ChainRoot: pBlockChainEntry): integer;

var currentEntry : pBlockChainEntry;
    count        : integer;

begin
     currentEntry := ChainRoot;
     count := 0;

     repeat
           if CurrentEntry <> nil then
              begin
                   if CurrentEntry.BlockTypeID = BLOCKTYPE_INTERFACE_DESCRIPTION then
                      count := count + 1;

                   CurrentEntry := CurrentEntry.NextBlockType;
              end;

     until (CurrentEntry = nil);

     result := count;
end;

(* ------------------------------------------- *)

// Keep in mind: the following function needs the trace file to be openend first
// TODO: implement reading file headers only (when ReadFullStatistics is false), skipping packets

function tPCAPNGLoader.readFileStatistics (ReadFullStatistics : boolean = true) : tPCAPNGStatistic;

var pBlockType      : pLongWord;
    pBlockTotalLen  : pLongWord;
    pLocalPos       : pByte;
    iLocalOffset    : integer;
    value           : tPCAPNGStatistic;
    CurrentEntry    : pBlockChainEntry;
    InterfaceEntry  : tCaptureInterface;
    StatisticEntry  : tStatisticsRecord;
    i,j             : integer;

begin
     // Clear Result Record

     value.initialize;

     // Get Block Chain

     Value.BlockChain := GetBlockChain;

     // Get Packet Block Count

     if value.BlockChain <> nil then
        begin
             value.FilePacketCount := GetPacketBlockCountFromBlockChain (value.BlockChain);
             value.SectionCount := getSectionHeaderCountFromBlockChain (value.BlockChain);
             value.FileInterfaceCount := getInterfaceCountFromBlockChain (value.BlockChain);







        end;

{
     // Read the interface statistics block from the end
     // TODO: this does not include blocks that are NOT at the end, for example when multiple SHBs exist

     iLocalOffset := iFileSize - 4;

     pLocalPos := pByte (pData) + iLocalOffset;

     repeat
           // Read the length of the block

           pBlockTotalLen := pLongWord (pLocalPos);

           // Jump to the beginning of the block

           pLocalPos := pLocalPos - pBlockTotalLen^ + 4;
           iLocalOffset := iLocalOffset - pBlockTotalLen^ + 4;

           pBlockType := pLongWord (pLocalPos);

           // If it is a statistics block we read it

           if pBlockType^ = BLOCKTYPE_INTERFACE_STATISTICS then
              readInterfaceStatisticsBlock (pLocalPos, iLocalOffset);

           pLocalPos := pLocalPos - 4;
           iLocalOffset := iLocalOffset - 4;

     until (pBlockType^ <> BLOCKTYPE_INTERFACE_STATISTICS) or (iLocalOffset < 0);

     // Get Interface results from Interface list (kept in tTraceFile)

     // TODO: count interfaces for all sections, then deduplicate them for the count

     for i := 0 to InterfaceList.Count - 1 do
        begin
             InterfaceEntry := InterfaceList.getInterfaceByIndex(i);

             inc (value.FileInterfaceCount);

             for j := 1 to InterfaceEntry.InterfaceStatistics.Count do
                 begin
                      inc (value.StatisticBlocKCount);

                      StatisticEntry := InterfaceEntry.InterfaceStatistics.getEntry (j);

                      value.StatPacketCount := value.StatPacketCount + StatisticEntry.PacketsReceived;
                      value.StatDropCount := value.StatDropCount + StatisticEntry.PacketsDropped;
                      value.StatFilterAcceptCount := value.StatFilterAcceptCount + StatisticEntry.FilterAccepted;
                      value.StatOsDropCount := value.StatOsDropCount + StatisticEntry.OSDropped;
                      value.StatUsrDeliveredCount := value.StatUsrDeliveredCount + StatisticEntry.UserDelivered;
                 end;
        end;
}

     result := value;
end;

(* ------------------------------------------- *)

end.
