{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_tracewriter;

// {$define usestream}       using this breaks at least the interface list, so it would require some debugging/troubleshooting if active

interface

uses Contnrs,
{$ifdef usestream}
     Classes,
{$endif}
     Sysutils,
     Diagnostics,
     TimeSpan,
     base_structures,
     class_timezonedetails,
     class_rawframe;

type
     EWriterException = class (Exception) // alle Exceptions beginnen per Konvention mit einem E; Die Klasse Exception ist Vorfahr aller Exceptions

                              private
                                     priv_FrameIndex   : int64;
                                     priv_MsgID        : integer;
                                     priv_ErrorMessage : string;
                              public
//                                  constructor Create (DiagHandle, Msg: string; FrameIndex : int64);
                                    constructor Create (const MsgID : integer; const Msg: string; FrameIndex : int64);
                                    property FrameIndex : Int64 read priv_FrameIndex write priv_FrameIndex;
//                                  property DiagnosticIndex : string read priv_DiagHandle write priv_DiagHandle;
                                    property MessageID : integer read priv_MSgID;
                                    property Message : string read priv_ErrorMessage;
                              end;

     tTraceWriter = class (tObject)

                          constructor create (const Filename : string; const DiscardFramesAfterWrite : boolean = false);
                          procedure free;

                          private
                                 priv_isOpen                  : boolean;

                                 function getFramesInMem      : integer;
                                 function AdjustFiletimeStamp : integer;

                          strict protected

                                   prot_Filename              : string;
                                   prot_WriteFrames           : tObjectList;
                                   prot_discardonwrite        : boolean;
{$ifdef usestream}
                                   prot_FileStream            : tBufferedFilestream;
{$else}
                                   prot_FileHandle            : file;
{$endif}
                                   prot_IOstatus              : integer;

                                   prot_maxframes             : integer;
                                   prot_flushcount            : integer;

                                   prot_lowesttimestamp       : tFrameTimeStamp;
                                   prot_highesttimestamp      : tFrameTimeStamp;
                                   prot_absolutetimestamp     : tDateTime;
                                   prot_timestampmode         : tFileTimestampMode;

                                   prot_StopWatch             : tStopWatch;
                                   prot_FlushDuration         : tTimeSpan;

                                   TimeZoneHandler            : tTimeZoneHandler;

                                   procedure DoFrameStatistics (Frame : tRawFrame);

                          public
                                property Filename : string read prot_Filename write prot_Filename;
                                property DiscardFramesAfterWrite : boolean read prot_discardonwrite;
                                property FramesInMem : integer read getFramesInMem;
                                property MaxFramesInMem : integer read prot_maxframes write prot_maxframes;
                                property IOStatus : integer read prot_IOstatus;
                                property FrameBuffer : TObjectList read prot_WriteFrames;

                                property FirstFrameTimestamp : tFrameTimestamp read prot_lowesttimestamp;
                                property LastFrameTimestamp : tFrameTimestamp read prot_highesttimestamp;
                                property AbsoluteTimestamp : tDateTime read prot_absolutetimestamp write prot_absolutetimestamp;
                                property TimeStampMode : tFileTimestampMode read prot_timestampmode write prot_timestampmode;
                                property LastFlushDuration : TTimeSpan read prot_FlushDuration;

                                procedure Open;
                                procedure Flush; virtual; abstract;
                                procedure Close;

                                procedure addFrameToList (NewFrame : tRawFrame);
                    end;


implementation

uses Windows,
     SiAuto;

(* ------------------------------------------- *)
{ EWriterException }
(* ------------------------------------------- *)

// constructor EWriterException.Create (DiagHandle, Msg: string; FrameIndex: int64);
constructor EWriterException.Create (const MsgID : integer; const Msg: string; FrameIndex : int64);

begin
     inherited Create (Msg);

     priv_FrameIndex := FrameIndex;
//   priv_DiagHandle := DiagHandle;
     priv_MsgID := MsgID;
end;

(* ------------------------------------------- *)
{ tTraceWriter }
(* ------------------------------------------- *)

constructor tTraceWriter.create (const Filename: string; const DiscardFramesAfterWrite : boolean = false);

begin
     inherited create;

     prot_Filename := Filename;
     prot_discardonwrite := DiscardFramesAfterWrite;

     prot_WriteFrames := tObjectList.Create (prot_discardonwrite);

     prot_maxframes := 10000;
     prot_flushcount := 0;
     priv_isOpen := false;

     prot_lowesttimestamp.initialize;
     prot_highesttimestamp.initialize;
     prot_absolutetimestamp := 0;

     prot_timestampmode := tsmNone;

     TimeZoneHandler := tTimeZoneHandler.create;
end;

(* ------------------------------------------- *)

procedure tTraceWriter.free;

begin
     prot_WriteFrames.Free;

     if priv_isOpen then
        close;

     TimeZoneHandler.free;
     TimeZoneHandler := nil;

     inherited free;
end;

(* ------------------------------------------- *)

procedure tTraceWriter.Open;

begin
{$ifdef usestream}
     try
        prot_FileStream := tBufferedFilestream.Create (prot_filename, fmCreate + fmOpenWrite);
        priv_isopen := true;
     except
           priv_isopen := false;
     end;
{$else}
     AssignFile(prot_FileHandle, prot_Filename);

     {$i-}
          FileMode := fmOpenWrite;
          Rewrite (prot_FileHandle, 1);
     {$i+}

     prot_IOstatus := IOResult;

     if IOstatus <> 0 then
        begin
//           raise EInOutError.Create ('Error: Could not create file ' + priv_Filename + '. IO result is ' + inttostr(IOstatus) + ' (tracewriter).');

             CloseFile (prot_FileHandle);
             exit;
        end
     else
         priv_isOpen := true;
{$endif}
end;

(* ------------------------------------------- *)

procedure tTraceWriter.Close;

begin
     SiMain.EnterMethod ('tTraceWriter.Close');

{$ifdef usestream}
//   prot_FileStream.FlushBuffer;
     prot_FileStream.Free;
     priv_isOpen := false;
{$else}
     CloseFile (prot_FileHandle);
     priv_isOpen := false;
{$endif}

     sleep (100);

     // adjust file time stamp if required

     AdjustFileTimeStamp;

     SiMain.LeaveMethod ('tTraceWriter.Close');
end;

(* ------------------------------------------- *)

function tTraceWriter.getFramesInMem: integer;

begin
     result := prot_WriteFrames.Count;
end;

(* ------------------------------------------- *)

function tTraceWriter.AdjustFiletimeStamp : integer;

var rcode     : integer;
    timestamp : tDatetime;

begin
     case prot_timestampmode of
      tsmOriginalFile,
      tsmSpecificDate : timestamp := prot_absolutetimestamp;
      tsmFirstFrame   : timestamp := prot_lowesttimestamp.datetime;
      tsmLastFrame    : timestamp := prot_highesttimestamp.datetime;
     else timestamp := 0;
     end;

     if timestamp <> 0 then
        rcode := FileSetDate (prot_Filename, DateTimeToFileDate(timestamp));

     result := rcode;
end;

(* ------------------------------------------- *)

procedure tTraceWriter.DoFrameStatistics(Frame: tRawFrame);

begin
     if prot_lowesttimestamp.datetime = 0 then
        prot_lowesttimestamp := Frame.Timestamp
     else
         if Frame.Timestamp < prot_lowesttimestamp then
            prot_lowesttimestamp := Frame.Timestamp;

     if prot_highesttimestamp.datetime = 0 then
        prot_highesttimestamp := Frame.Timestamp
     else
         if Frame.Timestamp > prot_highesttimestamp then
            prot_highesttimestamp := Frame.Timestamp;
end;

(* ------------------------------------------- *)

procedure tTraceWriter.addFrameToList(NewFrame: tRawFrame);

begin
     DoFrameStatistics (NewFrame);

     prot_WriteFrames.Add (NewFrame);

     if prot_WriteFrames.Count >= prot_maxframes then
        begin
             // Write gathered frames to disk

             Flush;
        end;
end;

(* ------------------------------------------- *)

end.
