{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_writer_pcapng;

// {$define usestream}

interface

uses Classes,
     Contnrs,
     base_structures,
     pcapng_structures,
     definition_ip_addresses,
     definition_simple_nameresolution,
     class_rawframe,
     class_dnsserverlist,
     class_interfacelist,
     class_interfacestatistics,
     class_timezonedetails,
     class_tracewriter,
     class_loader_pcapng;

type tPCAPngBlockHashMode = (bhmNone, bhmMD5);

     tWriteFrameEvent = procedure (Sender : tObject; const FrameIndex, FrameSize : integer) of object;
     tCopyInterfaceEvent = procedure (Sender : tObject; CaptureInterface : tCaptureInterface) of object;
     tCopySHBStringEvent = procedure (StringType : tSHBStringType; var Value : string) of object;

     tPCAPngWriter = class (tTraceWriter)

                           {$REGION 'Documentation'}
                           ///	<summary>
                           ///	  Constructor of the generic tracefile writer
                           ///	  class
                           ///	</summary>
                           ///	<param name="Filename">
                           ///	  Filename of the tracefile that will be
                           ///	  created. Will overwrite files if they exists,
                           ///	  so this needs to be handled in the code
                           ///	  calling this one.
                           ///	</param>
                           ///	<param name="DiscardFramesOnWrite">
                           ///	  If set to true, frames that have been written
                           ///	  to disk will be discarded. That means that the
                           ///	  tRawFrame object will be destroyed, so if the
                           ///	  writing process should not automatically do
                           ///	  that the constructor needs to be called with
                           ///	  DiscardFramesOnWrite=false.
                           ///	</param>
                           {$ENDREGION}
                           constructor create (const Filename : string; const DiscardFramesOnWrite : boolean = false);
                           procedure free;

                           private
                                  EventWriteFrame            : tWriteFrameEvent;
                                  EventBeforeWriteInterfaceBlock         : tCopyInterfaceEvent;
                                  EventCopySHBString         : tCopySHBStringEvent;

                                  priv_BlockWriteMode        : tBlockWriteMode;

                                  priv_SectionLength         : uint64;                                 // Tracks how much bytes were written after the section header block
                                  priv_WriteEndofopt         : boolean;                                // If set, and end of option marker will be written after each options block
                                  priv_WriteBigEndian        : boolean;                                // If set, all values will be written in little endian

                                  priv_InterfaceList         : tCaptureInterfaceList;
                                  priv_HeaderCommentList     : tStringList;
                                  priv_HardwareInfoList      : tStringList;
                                  priv_OperatingSystemList   : tStringList;
                                  priv_UserApplicationList   : tStringList;
                                  priv_IPv4NameResolutionMgr : tSimpleIPv4NameResolutionList;
                                  priv_IPv6NameResolutionMgr : tSimpleIPv6NameResolutionList;
                                  priv_DNSServerList         : tDNSServerList;
                                  priv_TimeZoneHandler       : tTimeZoneHandler;

                                  priv_BlockHashMode         : tPCAPngBlockHashMode;

                                  function Get32BitAlignment (UnAlignedLength : integer) : integer;
                                  function GetPaddedString (value : utf8string) : utf8string;
//                                function TimingDetailsToTimestamp (Timing : tTimingDetails) : NGInterfaceStatisticsTimestamp;
                                  function FrameTimestampToTimestamp (FrameTimestamp : tFrameTimeStamp; TimestampResolution : byte) : NGInterfaceStatisticsTimestamp;
                                  procedure WriteOptionHeader (OptionHeader : NGOptionCodeLen);
                                  function WriteOptionalStringList (const OptionCode : integer; Values : tStringList; const ZeroTerminated : boolean) : integer;
                                  function GetStringlistTotalPaddedLength (Stringlist : tStringlist; const ZeroTerminated : boolean) : integer;
                                  function DetermineSectionHeaderOptionblockLength : integer;
                                  function DetermineInterfaceOptionblockLength (Entry : tCaptureInterface) : integer;
                                  function DetermineNameResolutionOptionblockLength : integer;
                                  function DetermineInterfaceStatisticsBlockLength (StatisticsEntry : tStatisticsRecord) : integer;

                                  function WriteStringOption (const OptionCode : word; value : string; const ZeroTerminated : boolean) : integer;
                                  function WriteCaptureFilterOption (CaptureFilter : tPCAPNGCaptureFilter) : integer;
                                  function WriteIPv6AddressOption (const OptionCode : word; value : tIPv6AddressByteStruct) : integer;
                                  procedure WriteInterfaceDescriptionOptions (Entry : tCaptureInterface; OptionsSize : integer);
                                  function WriteIPv4AddressListOption (const OptionCode : word; AddressList : tIPv4SimpleAddressList) : integer;
                                  function WriteIPv6AddressListOption (const OptionCode : word; AddressList : tIPv6SimpleAddressList) : integer;
                                  function WriteMACAddressOption (const OptionCode : word; Address : tMACAddress) : integer;
                                  function WriteEUIAddressOption (const OptionCode : word; Address : tEUIAddress) : integer;
                                  function WriteLongOption (const OptionCode : word; value : longword; OptionLen : integer = 4) : integer;
                                  function WriteInt64Option (const OptionCode : word; value : uint64) : integer;
                                  function WriteTimestampOption (const OptionCode : word; value : NGInterfaceStatisticsTimestamp) : integer;
                                  function WriteNSRecord (TypeLenValue: NGRNameResolutionTypeLen; IPv4Address : longword; Name: string): integer; overload;
                                  function WriteNSRecord (TypeLenValue: NGRNameResolutionTypeLen; IPv6Address : tIPv6AddressByteStruct; Name: string): integer; overload;
                                  function WriteDNSEntryOption (Entry : tDNSServerEntry) : integer;
                                  function WriteFrameBytes (FrameBytes : tRawBuffer; FrameSize : longword) : integer;
                                  procedure WriteTrailingTotalLength (TotalBlockLen : longword);

                           protected


                           public
                                 property onWriteFrame : tWriteFrameEvent read EventWriteFrame write EventWriteFrame;
                                 property onBeforeWriteInterfaceBlock : tCopyInterfaceEvent read EventBeforeWriteInterfaceBlock write EventBeforeWriteInterfaceBlock;
                                 property onCopySHBString : tCopySHBStringEvent read EventCopySHBString write EventCopySHBString;

//                               property InterfaceList : tInterfaceList read priv_InterfaceList write priv_InterfaceList;       // should be read only because interfaces are added by writing blocks
                                 property InterfaceList : tCaptureInterfaceList read priv_InterfaceList;
                                 property HeaderCommentList : tStringlist read priv_HeaderCommentList write priv_HeaderCommentList;
                                 property HardwareInfoList : tStringlist read priv_HardwareInfoList write priv_HardwareInfoList;
                                 property OperatingSystemList : tStringlist read priv_OperatingSystemList write priv_OperatingSystemList;
                                 property UserApplicationList : tStringlist read priv_UserApplicationList write priv_UserApplicationList;
                                 property IPv4NameResolutionMgr : tSimpleIPv4NameResolutionList read priv_IPv4NameResolutionMgr write priv_IPv4NameResolutionMgr;
                                 property IPv6NameResolutionMgr : tSimpleIPv6NameResolutionList read priv_IPv6NameResolutionMgr write priv_IPv6NameResolutionMgr;
                                 property DNSServerList : tDNSServerList read priv_DNSServerList write priv_DNSServerList;

                                 property WriteEndOfOptionsMarker : boolean read priv_WriteEndofopt write priv_WriteEndofopt;
                                 property WriteBigEndian : boolean read priv_WriteBigEndian write priv_WriteBigEndian;
                                 property BlockWriteMode : tBlockWriteMode read priv_BlockWriteMode write priv_BlockWriteMode;
                                 property BlockHashMode : tPCAPngBlockHashMode read priv_BlockHashMode write priv_BlockHashMode;

                                  // -- Block moved from private to public for unittests

                                  function TimingDetailsToTimestamp (Timing : tTimingDetails) : NGInterfaceStatisticsTimestamp;

                                  procedure WriteSectionHeaderBlock;
                                  function WriteInterfaceDescriptionBlock (Entry : tCaptureInterface) : longword;
                                  function WriteNameResolutionBlock : longword;
                                  function WriteBlock (Frame : tRawFrame; const TimestampResolution : byte = 6) : longword;
                                  function WriteSimpleBlock (Frame : tRawFrame) : longword;
                                  function WriteEnhancedBlock (Frame : tRawFrame; TimestampResolution : byte = 6) : longword;
                                  function WriteInterfaceStatisticsBlock (StatisticsEntry : tStatisticsRecord) : longword;

                                  // -- End of block

                                 ///	<summary>
                                 ///	  This routine simply transfers the header
                                 ///	  values from the loader into the new file
                                 ///	</summary>
                                 ///	<param name="Loader">
                                 ///	  PCAPng Loader Object. This needs to be
                                 ///	  open and have the PCAPng headers read.
                                 ///	</param>
                                 ///	<remarks>
                                 ///	  This routine is a helper to allow faster
                                 ///	  copying of the PCAPng header structures
                                 ///	  when they are not relevant to the task
                                 ///	  at hand. Mostly used when just working
                                 ///	  on packets and nothing else
                                 ///	</remarks>
                                 procedure CopyPCAPngHeaderDetailsFromLoader (Loader : tPCAPNGLoader);
                                 procedure CopyPCAPngStatisticsDetailsFromLoader (Loader : tPCAPNGLoader);
                                 procedure WriteGenericHeader (const LinkType : longword); overload;
                                 procedure WriteGenericHeader (HeaderInterfaceList : tCaptureInterfaceList); overload;

                                 ///	<summary>
                                 ///	  Forces writing remaining frames from the
                                 ///	  buffer to disk.
                                 ///	</summary>
                                 ///	<remarks>
                                 ///	  This needs to be triggered externally if
                                 ///	  the buffer isn't going to be filled
                                 ///	  completely, which usually happens for
                                 ///	  the last batch of frames.
                                 ///	</remarks>
                                 procedure Flush; override;
                     end;

implementation

uses SysUtils,
     DateUtils,
     SiAuto,
     toolkit,
     base_messages;

const MayorVersion = 1;
      MinorVersion = 0;

(* ------------------------------------------- *)
{ tPCAPGNWriter }
(* ------------------------------------------- *)

constructor tPCAPngWriter.create (const Filename: string; const DiscardFramesOnWrite : boolean = false);

begin
     inherited create (Filename, DiscardFramesOnWrite);

     priv_InterfaceList := tCaptureInterfaceList.create (true);
     priv_TimeZoneHandler := tTimeZoneHandler.create;
     priv_HeaderCommentList := tStringList.create;
     priv_HardwareInfoList := tStringList.create;
     priv_OperatingSystemList := tStringList.create;
     priv_UserApplicationList := tStringList.create;
     priv_DNSServerList := tDNSServerList.create;

     priv_BlockWriteMode := bwmEnhancedPacketBlock;             // Default Write Mode
     priv_WriteEndofopt := true;                                // This might change to false by default since specs might be updated with a deprecation of this option type
     priv_WriteBigEndian := false;

     priv_BlockHashMode := bhmNone;
end;

(* ------------------------------------------- *)

procedure tPCAPngWriter.free;

begin
     SiMain.EnterMethod ('tPCAPngWriter.free');

     try
        priv_TimeZoneHandler.free;
        priv_TimeZoneHandler := nil;
     except
           SiMain.LogFatal ('Error: freeing the timezone handler failed');
     end;

     try
        priv_HeaderCommentList.Free;
        priv_HeaderCommentList := nil;
     except
           SiMain.LogFatal ('Error: freeing the header commentlist failed');
     end;

     try
        priv_HardwareInfoList.Free;
        priv_HardwareInfoList := nil;
     except
           SiMain.LogFatal ('Error: freeing the hardware infolist failed');
     end;

     try
        priv_OperatingSystemList.Free;
        priv_OperatingSystemList := nil;
     except
           SiMain.LogFatal ('Error: freeing the operatingsystem list failed');
     end;

     try
        priv_UserApplicationList.Free;
        priv_UserApplicationList := nil;
     except
           SiMain.LogFatal ('Error: freeing the userapplication list failed');
     end;

     try
        priv_InterfaceList.free;
        priv_InterfaceList := nil;
     except
           SiMain.LogFatal ('Error: freeing the interfacelist failed');
     end;

     try
        priv_DNSServerList.free;
        priv_DNSServerList := nil;
     except
           SiMain.LogFatal ('Error: freeing the DNS serverlist failed');
     end;

     inherited free;
     SiMain.LeaveMethod ('tPCAPngWriter.free');
end;

(* ------------------------------------------- *)

function tPCAPngWriter.Get32BitAlignment (UnAlignedLength: integer): integer;

begin
     if (UnAlignedLength mod 4) = 0 then
        result := UnAlignedLength
     else
         result := (4 - (UnAlignedLength MOD 4)) + UnAlignedLength;
end;

(* ------------------------------------------- *)

function tPCAPngWriter.GetPaddedString(value: utf8string): utf8string;

var Len        : integer;
    AlignedLen : integer;
    PadString  : utf8string;

begin
     // Check if padding is needed

     Len := length (value);
     AlignedLen := Get32BitAlignment (len);

     if AlignedLen > len then
        begin
             // pad the string

             PadString := StringOfChar (#0, AlignedLen - len);
             result := value + PadString;
        end
     else
         result := value;
end;

(* ------------------------------------------- *)

// Turns a Timingdetail record back into pcap-ng style Hi/Lo Timestamps

function tPCAPngWriter.TimingDetailsToTimestamp (Timing: tTimingDetails): NGInterfaceStatisticsTimestamp;

var value        : NGInterfaceStatisticsTimestamp;
    helpdate     : tDateTime;
    helpuint     : uint64;
    helphi       : uint64;
    helplo       : uint64;
    modifiersecs : int64;

begin
     // Get Date with 0:00 time and turn it into unix time (seconds since 1/1/1970)

     helpdate := recodetime (Timing.AbsoluteTime, 0, 0, 0, 0);
     helpuint := DateTimeToUnix (helpdate);

     ModifierSecs := (TimeZoneHandler.GetBiasForDate (HelpDate) DIV 60 * -1) * 3600;
     helpuint := helpuint - modifiersecs;

     helpuint := helpuint * 1000 * 1000;
     helpuint := helpuint + Timing.TraceTicks;

     if Timing.NanoTicks > 0 then
        begin
             helpuint := helpuint * 1000;
             helpuint := helpuint + Timing.NanoTicks;
        end;

     helphi := helpuint shr 32;
     helplo := helpuint and $ffffffff;

     value.TimestampHigh := helphi;
     value.TimestampLow := helplo;

     result := value;
end;

(* ------------------------------------------- *)

function tPCAPngWriter.FrameTimestampToTimestamp (FrameTimestamp: tFrameTimeStamp; TimestampResolution : byte): NGInterfaceStatisticsTimestamp;

var value        : NGInterfaceStatisticsTimestamp;
    ModifierSecs : int64;
    ModifierTics : int64;
    helpuint     : uint64;
    microsec     : uint64;
    helphi       : uint64;
    helplo       : uint64;

begin
     // Get timezone BIAS stuff

     ModifierSecs := (TimeZoneHandler.GetBiasForDate (FrameTimestamp.datetime) DIV 60 * -1) * 3600;

     // Get seconds since 1/1/1970 and adjust to UTC

     helpuint := DateTimeToUnix (FrameTimestamp.datetime) - ModifierSecs;

     // Add more exact timing details depending on the

     if TimestampResolution > 0 then
        helpuint := (helpuint * 1000) + MilliSecondOf (FrameTimestamp.datetime);

     // Get down to micro or nano seconds
     // TODO: deal with other Timestampresolutions?

     case TimeStampResolution of
      6 : begin
               microsec := FrameTimestamp.nanoseconds DIV 1000;

               helpuint := (helpuint * 1000) + microsec;
          end;
      9 : begin
               helpuint := (helpuint * 1000 * 1000) + FrameTimestamp.nanoseconds;
          end;
     end;

     helphi := helpuint shr 32;
     helplo := helpuint and $ffffffff;

     value.TimestampHigh := helphi;
     value.TimestampLow := helplo;

     result := value;
end;

(* ------------------------------------------- *)

procedure tPCAPngWriter.WriteOptionHeader (OptionHeader: NGOptionCodeLen);

var count : integer;

begin
     count := sizeOf (NGOptionCodeLen);

     // Flip some bytes if neccessary

     if priv_WriteBigEndian then
        begin
             OptionHeader.code := Swap16 (OptionHeader.code);
             OptionHeader.length := Swap16 (OptionHeader.length);
        end;

{$ifdef usestream}
     prot_FileStream.WriteBuffer (OptionHeader, Count);
{$else}
     try
        BlockWrite (prot_FileHandle, OptionHeader, count);
     except
           prot_IOstatus := IOResult;
           Exit;
     end;
{$endif}
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteStringOption (const OptionCode : word; value : string; const ZeroTerminated : boolean) : integer;

var OptionHdr    : NGOptionCodeLen;
    HelpUTF8     : UTF8String;
    count        : integer;

begin
     HelpUTF8 := UTF8String (value);
     HelpUTF8 := getPaddedString (HelpUTF8);
     count := length (HelpUTF8);

     // Write Option Header

     OptionHdr.code := OptionCode;
     OptionHdr.length := length (value);

     // Lets write

     WriteOptionHeader (OptionHdr);

{$ifdef usestream}
     prot_FileStream.WriteBuffer (HelpUTF8, Count);
{$else}
     try
        BlockWrite (prot_FileHandle, HelpUTF8[1], count);    // Cannot use OptionHdr.length because it might have been swapped
     except
           prot_IOstatus := IOResult;
           Exit;
     end;
{$endif}

     result := sizeOf (NGOptionCodeLen) + length (HelpUTF8);
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteCaptureFilterOption (CaptureFilter: tPCAPNGCaptureFilter): integer;

var value     : byte;
//  OptionHdr : NGOptionCodeLen;
//  HelpUTF8  : UTF8String;
    filter    : string;

begin
     case CaptureFilter.FilterType of
      cftUnknown     : value := 0; // TODO: verify that the capture filter types are correct when the specs define them
      cftLibPCAP     : value := 0;
      cftBPFByteCode : value := 1;
     end;

     // Set the first char in the string to hold the filter type

     filter := chr(value) + CaptureFilter.FilterString;

     result := WriteStringOption (opt_if_filter, filter, false);

{
     HelpUTF8 := UTF8String (CaptureFilter.FilterString);
     HelpUTF8 := getPaddedString (HelpUTF8);

     // Write Option Header

     OptionHdr.code := opt_if_filter;
     OptionHdr.length := 1 + length (HelpUTF8);

     // Lets write

     WriteOptionHeader (OptionHdr);

     BlockWrite (priv_FileHandle, value, 1);

     try
        BlockWrite (priv_FileHandle, HelpUTF8[1], length (HelpUTF8));
     except
           priv_IOstatus := IOResult;
           Exit;
     end;

     result := sizeOf (NGOptionCodeLen) + 1 + length (HelpUTF8);
}
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteIPv6AddressOption (const OptionCode: word; value: tIPv6AddressByteStruct): integer;

var OptionHdr    : NGOptionCodeLen;

begin
     OptionHdr.code := OptionCode;
     OptionHdr.length := 16;

     WriteOptionHeader (OptionHdr);

{$ifdef usestream}
     prot_FileStream.WriteBuffer (value, 16);
{$else}
     try
        BlockWrite (prot_FileHandle, value, 16);

     except
           prot_IOstatus := IOResult;
           Exit;
     end;
{$endif}

     result := sizeOf (NGOptionCodeLen) + sizeOf (tIPv6AddressByteStruct);
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteOptionalStringList (const OptionCode : integer; Values: tStringList; const ZeroTerminated : boolean) : integer;

var BytesWritten : integer;
    help         : string;
    i            : integer;

begin
     BytesWritten := 0;

     // Write Comments if not empty

     for i := 1 to Values.Count do
         begin
              help := Values[i-1];

              assert (help <> '', 'Assertion failed: string is empty (tPCAPngWriter.WriteOptionalStringList))');

              if help <> '' then
                 BytesWritten := BytesWritten + WriteStringOption (OptionCode, help, ZeroTerminated);
         end;

     result := BytesWritten;
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteTimestampOption (const OptionCode: word; value: NGInterfaceStatisticsTimestamp): integer;

var OptionHdr    : NGOptionCodeLen;
    timestamp    : NGInterfaceStatisticsTimestamp;
    count        : integer;

begin
     timestamp := value;
     count := sizeOf (timestamp);

     OptionHdr.code := OptionCode;
     OptionHdr.length := 8;

     WriteOptionHeader (OptionHdr);

     if priv_WriteBigEndian then
        begin
             timestamp.TimestampHigh := Swap32 (timestamp.TimestampHigh);
             timestamp.TimestampLow := Swap32 (timestamp.TimestampLow);
        end;

{$ifdef usestream}
     prot_FileStream.WriteBuffer (timestamp, count);
{$else}
     try
        BlockWrite (prot_FileHandle, timestamp, count);

     except
           prot_IOstatus := IOResult;
           Exit;
     end;
{$endif}

     result := sizeOf (NGOptionCodeLen) + sizeOf (NGInterfaceStatisticsTimestamp);
end;

(* ------------------------------------------- *)

procedure tPCAPngWriter.WriteTrailingTotalLength (TotalBlockLen: longword);

var lHelp : Longword;

begin
{$ifdef usestream}
     if priv_WriteBigEndian then
        lHelp := swap32 (TotalBlockLen)
     else
         lHelp := TotalBlockLen;

     prot_FileStream.Write (lHelp, 4);
{$else}
     try
        if priv_WriteBigEndian then
           begin
                lHelp := swap32 (TotalBlockLen);

                BlockWrite (prot_FileHandle, lHelp, 4)
           end
        else
            BlockWrite (prot_FileHandle, TotalBlockLen, 4);
     except
           prot_IOstatus := IOResult;
           Exit;
     end;
{$endif}
end;

(* ------------------------------------------- *)

function tPCAPngWriter.GetStringlistTotalPaddedLength (Stringlist: tStringlist; const ZeroTerminated: boolean): integer;

var i         : integer;
    OptionLen : integer;
    sHelp     : UTF8String;

begin
     OptionLen := 0;

     for i := 1 to Stringlist.Count do
         begin
              sHelp := UTF8String (Stringlist[i-1]);

              if ZeroTerminated then
                 sHelp := sHelp + #0;

              OptionLen := OptionLen + sizeOf (NGOptionCodeLen) + Get32BitAlignment (length (sHelp));
         end;

     result := OptionLen;
end;

(* ------------------------------------------- *)

function tPCAPngWriter.DetermineSectionHeaderOptionblockLength: integer;

var OptionLen : integer;

begin
     if priv_HeaderCommentList.Count > 0 then
        OptionLen := GetStringlistTotalPaddedLength (priv_HeaderCommentList, false)
     else
         OptionLen := 0;

     if priv_HardwareInfoList.Count > 0 then
        OptionLen := OptionLen + GetStringlistTotalPaddedLength (priv_HardwareInfoList, false);

     if priv_OperatingSystemList.Count > 0 then
        OptionLen := OptionLen + GetStringlistTotalPaddedLength (priv_OperatingSystemList, false);

     if priv_UserApplicationList.Count > 0 then
        OptionLen := OptionLen + GetStringlistTotalPaddedLength (priv_UserApplicationList, false);

     if priv_WriteEndofopt and (OptionLen > 0) then
        OptionLen := OptionLen + SizeOf (NGOptionCodeLen);

     result := OptionLen;
end;

(* ------------------------------------------- *)

procedure tPCAPngWriter.WriteSectionHeaderBlock;

var SectionHeader : NGSectionHdrBlock;
    OptionHdr     : NGOptionCodeLen;
    BlockTotalLen : longword;
    NumOptions    : integer;
    count         : integer;

begin
     priv_SectionLength := 0;

     BlockTotalLen := sizeOf (NGSectionHdrBlock) + DetermineSectionHeaderOptionblockLength + 4;      // +4 for trailing TotalBlockLength

     // Write Static Header Structure

     SectionHeader.BlockType := BLOCKTYPE_SECTION_HEADER;

     SectionHeader.SectionLength := $ffffffffffffffff;             // a.k.a. "I don't have a clue how much I'm going to write". This might be corrected later by overwriting the offset with SectionLength

     if priv_WriteBigEndian then
        begin
             SectionHeader.BlockTotalLength := swap32 (BlockTotalLen);
//           SectionHeader.SectionLength := swap64 (SectionHeader.SectionLength);   // doesn't make sense to do this, it's still the same lot of $ff bytes :-)
             SectionHeader.ByteOrderMagic := swap32 (SHB_BYTEORDERMAGIC);
             SectionHeader.MajorVersion := swap16 (MayorVersion);
             SectionHeader.MinorVersion := swap16 (MinorVersion);
        end
     else
         begin
              SectionHeader.BlockTotalLength := BlockTotalLen;
              SectionHeader.ByteOrderMagic := SHB_BYTEORDERMAGIC;
              SectionHeader.MajorVersion := MayorVersion;
              SectionHeader.MinorVersion := MinorVersion;
         end;

     count := sizeOf (SectionHeader);

{$ifdef usestream}
     prot_FileStream.WriteBuffer (SectionHeader, count);
{$else}
     try
        BlockWrite (prot_FileHandle, SectionHeader, count);
     except
           prot_IOstatus := IOResult;
           Exit;
     end;
{$endif}

     // Write Options - we ignore the returned length value because it is only counted AFTER the SHB has been written

     NumOptions := 0;

     if priv_HeaderCommentList.Count > 0 then
        begin
             WriteOptionalStringList (opt_comment, priv_HeaderCommentList, false);
             NumOptions := NumOptions + priv_HeaderCommentList.Count;
        end;

     if priv_HardwareInfoList.Count > 0 then
        begin
             WriteOptionalStringList (shb_hardware, priv_HardwareInfoList, false);
             NumOptions := NumOptions + priv_HardwareInfoList.Count;
        end;

     if priv_OperatingSystemList.Count > 0 then
        begin
             WriteOptionalStringList (shb_os, priv_OperatingSystemList, false);
             NumOptions := NumOptions + priv_OperatingSystemList.Count;
        end;

     if priv_UserApplicationList.Count > 0 then
        begin
             WriteOptionalStringList (shb_userappl, priv_UserApplicationList, false);
             NumOptions := NumOptions + priv_UserApplicationList.Count;
        end;

     if priv_WriteEndofopt and (NumOptions > 0) then
        begin
             OptionHdr.code := opt_endofopt;
             OptionHdr.length := 0;

             WriteOptionHeader (OptionHdr);
        end;

     // Write trailing total length

     WriteTrailingTotalLength (BlockTotalLen);
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteIPv4AddressListOption (const OptionCode : word; AddressList : tIPv4SimpleAddressList) : integer;

var i            : integer;
    OptionHdr    : NGOptionCodeLen;
    entry        : tIPv4Address;
    addr         : longword;
    mask         : longword;
    BytesWritten : integer;

begin
     BytesWritten := 0;

     OptionHdr.code := OptionCode;
     OptionHdr.length := 8;

     for i := 1 to AddressList.Count do
         begin
              entry := AddressList.GetEntry (i);

              addr := entry.ValueAsLong;
              mask := entry.MaskAsLong;

              if not priv_WriteBigEndian then   //
                 begin
                      addr := Swap32 (addr);
                      mask := Swap32 (mask);
                 end;

              WriteOptionHeader (OptionHdr);

{$ifdef usestream}
              prot_FileStream.Write (addr, 4);
              prot_FileStream.Write (mask, 4);
{$else}
              try
                 BlockWrite (prot_FileHandle, addr, 4);
                 BlockWrite (prot_FileHandle, mask, 4);

              except
                    prot_IOstatus := IOResult;
                    Exit;
              end;
{$endif}

              BytesWritten := BytesWritten + sizeOf (NGOptionCodeLen) + 8;
         end;

     result := BytesWritten;
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteIPv6AddressListOption (const OptionCode : word; AddressList: tIPv6SimpleAddressList) : integer;

var i         : integer;
    OptionHdr : NGOptionCodeLen;
    entry     : tIPv6Address;
    addr      : tIPv6AddressByteStruct;
    BytesWritten : integer;

begin
     BytesWritten := 0;

     OptionHdr.code := OptionCode;
     OptionHdr.length := 16;

     for i := 1 to AddressList.Count do
         begin
              entry := AddressList.GetEntry(i);

              addr := entry.ValueAsByteStruct;

              BytesWritten := BytesWritten + WriteIPv6AddressOption (OptionCode, addr);
         end;

     result := BytesWritten;
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteMACAddressOption(const OptionCode : word; Address: tMACAddress) : integer;

var OptionHdr : NGOptionCodeLen;
    Addr      : array [1..8] of byte;
    i         : integer;

begin
     OptionHdr.code := opt_if_MACaddr;
     OptionHdr.length := 6;

     for i  := 1 to 6 do
         Addr[i] := Address.RangeBytes[i];

     Addr[7] := 0;
     Addr[8] := 0;

     WriteOptionHeader (OptionHdr);

{$ifdef usestream}
     prot_FileStream.Write (addr, 8);
{$else}
     try
        BlockWrite (prot_FileHandle, addr, 8);

     except
           prot_IOstatus := IOResult;
           Exit;
     end;
{$endif}

     result := 8 + sizeOf (NGOptionCodeLen);
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteEUIAddressOption(const OptionCode : word; Address: tEUIAddress) : integer;

var OptionHdr : NGOptionCodeLen;
    Addr      : tEUIBytes;

begin
     OptionHdr.code := opt_if_EUIaddr;
     OptionHdr.length := 8;

     Addr := Address.RangeBytes;

     WriteOptionHeader (OptionHdr);

{$ifdef usestream}
     prot_FileStream.Write (addr, 8);
{$else}
     try
        BlockWrite (prot_FileHandle, addr, 8);

     except
           prot_IOstatus := IOResult;
           Exit;
     end;
{$endif}

     result := 8 + sizeOf (NGOptionCodeLen);
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteLongOption(const OptionCode : word; value: longword; OptionLen : integer = 4) : integer;

var OptionHdr : NGOptionCodeLen;
    longhelp  : longword;

begin
     OptionHdr.code := OptionCode;
     OptionHdr.length := OptionLen;

     longhelp := value;

     if priv_WriteBigEndian then
        longhelp := Swap32 (longhelp);

     WriteOptionHeader (OptionHdr);

{$ifdef usestream}
     prot_FileStream.Write (longhelp, 4);
{$else}
     try
        BlockWrite (prot_FileHandle, longhelp, 4);

     except
           prot_IOstatus := IOResult;
           Exit;
     end;
{$endif}

     result := 4 + sizeOf (NGOptionCodeLen);
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteInt64Option(const OptionCode : word; value: uint64) : integer;

var OptionHdr : NGOptionCodeLen;
    int64help : uint64;

begin
     OptionHdr.code := OptionCode;
     OptionHdr.length := 8;

     int64help := value;

     if priv_WriteBigEndian then
        int64help := Swap64 (int64help);

     WriteOptionHeader (OptionHdr);

{$ifdef usestream}
     prot_FileStream.Write (int64help, 8);
{$else}
     try
        BlockWrite (prot_FileHandle, int64help, 8);

     except
           prot_IOstatus := IOResult;
           Exit;
     end;
{$endif}

     result := 8 + sizeOf (NGOptionCodeLen);
end;

(* ------------------------------------------- *)

procedure tPCAPngWriter.WriteInterfaceDescriptionOptions (Entry: tCaptureInterface; OptionsSize : integer);

var OptionHdr  : NGOptionCodeLen;
    HelpString : string;

begin
     // Name

     if Entry.Name <> '' then
        WriteStringOption (opt_if_name, entry.Name, false);

     // Description

     if Entry.Description <> '' then
        WriteStringOption (opt_if_description, entry.Description, false);

     // IPv4 Addresses

     WriteIPv4AddressListOption (opt_if_ipv4addr, Entry.IPv4Addresses);

     // IPv6 Addresses

     WriteIPv6AddressListOption (opt_if_ipv6addr, Entry.IPv6Addresses);

     // Hardware Addresses

     if entry.MACAddress <> VALUE_ZERO_MAC then
        WriteMACAddressOption (opt_if_MACaddr, Entry.MACAddress);

     if entry.EUIAddress <> VALUE_ZERO_EUI then
        WriteEUIAddressOption (opt_if_EUIaddr, Entry.EUIAddress);

     // Speed

     if entry.SpeedBPS > 0 then
        WriteInt64Option (opt_if_speed, Entry.SpeedBPS);

     // Timestamp Resolution

     if entry.TimeStampResolution > 0 then
        WriteLongOption (opt_if_tsresol, Entry.TimeStampResolution, 1);

     // Timezone

     if entry.TimeZone > 0 then
        WriteLongOption (opt_if_tzone, Entry.TimeZone);

     // Capture Filter

     if (entry.CaptureFilter.FilterType <> cftUnknown) and (entry.CaptureFilter.FilterString <> '') then
        WriteCaptureFilterOption (Entry.CaptureFilter);

     // Operating System

     if entry.OperatingSystem <> '' then
        WriteStringOption (opt_if_os, Entry.OperatingSystem, false);

     // FCS Length

     if entry.FCSLength > 0 then
        WriteLongOption (opt_if_fcslen, Entry.FCSLength);

     // Timestamp Offset

     if entry.TimestampOffset > 0 then
        WriteInt64Option (opt_if_tsoffset, Entry.TimestampOffset);

     if (OptionsSize > 0) and priv_WriteEndofopt then
        begin
             OptionHdr.code := opt_endofopt;
             OptionHdr.length := 0;

             WriteOptionHeader (OptionHdr);
        end;
end;

(* ------------------------------------------- *)

// Determines how many bytes the options will have for an interface description block
// Since options need to be aligned to 32bit boundaries the length values may seem odd but they're not :)

function tPCAPngWriter.DetermineInterfaceOptionblockLength (Entry : tCaptureInterface): integer;

var len    : integer;
    i      : integer;

begin
     // Name and Description - if they are written, its aligned to 32 bits, and plus option header length

     len := 0;

     if Entry.Name <> '' then
        len := len + Get32BitAlignment (length (entry.Name)) + 4;

     if Entry.Description <> '' then
        len := len + Get32BitAlignment (length (entry.Description)) + 4;

     // IP Addresses

     len := len + (entry.IPv4Addresses.Count * 12) + (entry.IPv6Addresses.Count * 20);

     // Hardware Addresses

     if entry.MACAddress <> VALUE_ZERO_MAC then   // 6 Bytes, aligned to 8
        len := len + 8 + 4;

     if entry.EUIAddress <> VALUE_ZERO_EUI then
        len := len + 8 + 4;

     if entry.SpeedBPS > 0 then
        len := len + 8 + 4;

     if entry.TimeStampResolution > 0 then        // 1 Byte, aligned to 4
        len := len + 4 + 4;

     if entry.TimeZone > 0 then
        len := len + 4 + 4;

     if (entry.CaptureFilter.FilterType <> cftUnknown) and (entry.CaptureFilter.FilterString <> '') then
        len := len + Get32BitAlignment (length (chr(0) + entry.CaptureFilter.FilterString)) + 4;   // Add chr(0) as FilterType byte

     if entry.OperatingSystem <> '' then
        len := len + Get32BitAlignment (length (entry.OperatingSystem)) + 4;

     if entry.FCSLength > 0 then                  // 1 Byte, aligned to 4
        len := len + 4 + 4;

     if entry.TimestampOffset > 0 then
        len := len + 8 + 4;

     if (len > 0) and priv_WriteEndofopt then
        len := len + sizeOf (NGOptionCodeLen);

     result := len;
end;

(* ------------------------------------------- *)

function tPCAPngWriter.DetermineNameResolutionOptionblockLength: integer;

var IPv4Entry : tSimpleIPv4NameResolutionEntry;
    IPv6Entry : tSimpleIPv6NameResolutionEntry;
    DNSEntry  : tDNSServerEntry;
    sName     : string;
    len       : integer;
    i         : integer;

begin
     len := 4;         // End_of_Recs is written in any case

     // Count Name resolution records

     for i := 1 to priv_IPv4NameResolutionMgr.Count do
         begin
              IPv4Entry := priv_IPv4NameResolutionMgr.getEntry(i);

              len := len + 4; // Record Type and Record Length

              sName := IPv4Entry.FQDN;

              len := len + 4 + Get32BitAlignment (length (sName) + 1);    // +4 for the IP address, +1 for the 0-Termination
         end;

     for i := 1 to priv_IPv6NameResolutionMgr.Count do
         begin
              IPv6Entry := priv_IPv6NameResolutionMgr.getEntry(i);

              len := len + 4; // Record Type and Record Length

              sName := IPv6Entry.FQDN;

              len := len + 16 + Get32BitAlignment (length (sName) + 1);   // +16 for the IP address, +1 for the 0-Termination
         end;

     // Add options

     for i := 1 to priv_DNSServerList.Count do
         begin
              DNSEntry := priv_DNSServerList.getEntry(i);

              if DNSEntry.IPv4AddressAssigned then
                 len := len + 4 + Get32BitAlignment (length (DNSEntry.Name));

              if DNSEntry.IPv6AddressAssigned then
                 len := len + 16 + Get32BitAlignment (length (DNSEntry.Name));
         end;

     result := len;
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteInterfaceDescriptionBlock (Entry : tCaptureInterface) : longword;

var InterfaceHeader : NGIntDescrBlock;
    OptionLen       : integer;
    OptionHdr       : NGOptionCodeLen;
    BlockTotalLen   : longword;
    WriterInterface : tCaptureInterface;
    count           : integer;

begin
     // Add Interface to internal list first, because we need to keep it for writing packets later

     WriterInterface := tCaptureInterface.create (priv_interfacelist.Count, Entry.LinkType, Entry.Origin, true);

     Entry.copyTo (WriterInterface, true);

     // Fix weird time resolutions (while not having a different approach), and set it to nanoseconds deliberately

     if ((WriterInterface.TimeStampResolution <> 6) and (WriterInterface.TimeStampResolution <> 9)) then
        WriterInterface.TimeStampResolution := 9;

     // Allow modification to the entry before writing it if neccessary, e.g. when the linktype
     // needs to be changed in cases where an edit task cuts away layers

     if assigned (EventBeforeWriteInterfaceBlock) then
        EventBeforeWriteInterfaceBlock (self, WriterInterface);

     priv_interfacelist.Add(WriterInterface);

     // Write the entry

     OptionLen := DetermineInterfaceOptionblockLength (WriterInterface);          // This includes optional end_of_opts

     BlockTotalLen := sizeOf (NGIntDescrBlock) + OptionLen + 4;                   // +4 for trailing TotalBlockLength

     // Determine Static Header Structure

     InterfaceHeader.BlockType := BLOCKTYPE_INTERFACE_DESCRIPTION;

     if priv_WriteBigEndian then
        begin
             InterfaceHeader.BlockType := swap32 (InterfaceHeader.BlockType);
             InterfaceHeader.BlockTotalLength := swap32 (BlockTotalLen);

             InterfaceHeader.LinkType := Swap16 (WriterInterface.LinkType);
             InterfaceHeader.Reserved := 0;
             InterfaceHeader.SnapLen := Swap32 (WriterInterface.SnapLength);
        end
     else
         begin
              InterfaceHeader.BlockTotalLength := BlockTotalLen;

              InterfaceHeader.LinkType := WriterInterface.LinkType;
              InterfaceHeader.Reserved := 0;
              InterfaceHeader.SnapLen := WriterInterface.SnapLength;
         end;

     // Write Static Header Structure

     count := sizeOf (InterfaceHeader);

{$ifdef usestream}
     prot_FileStream.WriteBuffer (InterfaceHeader, count);
{$else}
     try
        BlockWrite (prot_FileHandle, InterfaceHeader, count);
     except
           prot_IOstatus := IOResult;
           Exit;
     end;
{$endif}

     // Write Options

     WriteInterfaceDescriptionOptions (WriterInterface, InterfaceHeader.BlockTotalLength);

     // Write trailing total length

     WriteTrailingTotalLength (BlockTotalLen);

     result := BlockTotalLen;
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteNSRecord (TypeLenValue: NGRNameResolutionTypeLen; IPv4Address : longword; Name: string): integer;

var UTF8Name : UTF8String;
    value    : longword;
    count    : integer;

begin
     if priv_WriteBigEndian then
        begin
             TypeLenValue.RecordType := Swap16 (TypeLenValue.RecordType);
             TypeLenValue.RecordLength := Swap16 (TypeLenValue.RecordLength);
        end;

     // Add 0 termination to the Name

     UTF8Name := GetPaddedString (Name + #0);

     value := Swap32 (IPv4Address);

{$ifdef usestream}
     count := sizeOf (NGRNameResolutionTypeLen);
     prot_FileStream.WriteBuffer (TypeLenValue, count);

     prot_FileStream.Write (Value, 4);

     count := length (UTF8Name);
     prot_FileStream.WriteBuffer (UTF8Name, count);
{$else}
     try
        count := sizeOf (NGRNameResolutionTypeLen);
        BlockWrite (prot_FileHandle, TypeLenValue, count);
        BlockWrite (prot_FileHandle, value, 4);
        count := length (UTF8Name);
        BlockWrite (prot_FileHandle, UTF8Name[1], count)
     except
           prot_IOstatus := IOResult;
           Exit;
     end;
{$endif}
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteNSRecord (TypeLenValue: NGRNameResolutionTypeLen; IPv6Address : tIPv6AddressByteStruct; Name: string): integer;

var UTF8Name : UTF8String;
    count    : integer;

begin
     if priv_WriteBigEndian then
        begin
             TypeLenValue.RecordType := Swap16 (TypeLenValue.RecordType);
             TypeLenValue.RecordLength := Swap16 (TypeLenValue.RecordLength);
        end;

     // Add 0 termination to the Name

     UTF8Name := GetPaddedString (Name + #0);

{$ifdef usestream}
     count := sizeOf (NGRNameResolutionTypeLen);
     prot_FileStream.Write (TypeLenValue, count);

     prot_FileStream.WriteBuffer (IPv6Address, 16);

     count := length (UTF8Name);
     prot_FileStream.WriteBuffer (UTF8Name, count);
{$else}
     try
        BlockWrite (prot_FileHandle, TypeLenValue, sizeOf (NGRNameResolutionTypeLen));
        BlockWrite (prot_FileHandle, IPv6Address, 16);
        BlockWrite (prot_FileHandle, UTF8Name[1], length (UTF8Name))
     except
           prot_IOstatus := IOResult;
           Exit;
     end;
{$endif}
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteDNSEntryOption (Entry: tDNSServerEntry): integer;

var OptionHdr : NGOptionCodeLen;

begin
     if Entry.IPv4AddressAssigned or Entry.IPv6AddressAssigned then
        WriteStringOption (ns_dnsname, Entry.Name, true);

     if Entry.IPv4AddressAssigned then
        WriteLongOption (ns_dnsIP4addr, Entry.IPv4Address.ValueAsLong);

     if Entry.IPv6AddressAssigned then
        WriteIPv6AddressOption (ns_dnsIP6addr, Entry.IPv6Address.ValueAsByteStruct);
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteNameResolutionBlock : longword;

var NameResolutionHeader : NGNameResolutionBlock;
    NameResolutionTL     : NGRNameResolutionTypeLen;
    IPv4NSEntry          : tSimpleIPv4NameResolutionEntry;
    IPv6NSEntry          : tSimpleIPv6NameResolutionEntry;
    IP4Address           : tIPv4Address;
    IP6Address           : tIPv6Address;
    DNSEntry             : tDNSServerEntry;
    OptionCodeLen        : NGOptionCodeLen;
    OptionTotalLen       : integer;
    BlockTotalLen        : longword;
    EndOfOpts            : longword;
    i                    : integer;
    count                : integer;

begin
     OptionTotalLen := DetermineNameResolutionOptionblockLength;
     BlockTotalLen := sizeOf (NGNameResolutionBlock) + OptionTotalLen + 4;

     NameResolutionHeader.BlockType := BLOCKTYPE_NAME_RESOLUTION;
     NameResolutionHeader.BlockTotalLength := BlockTotalLen;

     // Write Header

     if priv_WriteBigEndian then
        begin
             NameResolutionHeader.BlockType := Swap32 (NameResolutionHeader.BlockType);
             NameResolutionHeader.BlockTotalLength := Swap32 (NameResolutionHeader.BlockTotalLength);
        end;

     count := sizeOf (NGNameResolutionBlock);

{$ifdef usestream}
     prot_FileStream.WriteBuffer (NameResolutionHeader, count);
{$else}
     BlockWrite (prot_FileHandle, NameResolutionHeader, count);
{$endif}

     // Write IPv4 Name Resolution Addresses

     for i := 1 to priv_IPv4NameResolutionMgr.Count do
         begin
              IPv4NSEntry := priv_IPv4NameResolutionMgr.getEntry(i);

              NameResolutionTL.RecordType := nres_ip4record;
              NameResolutionTL.RecordLength := 4 + length (IPv4NSEntry.FQDN + chr(0));

              WriteNSRecord (NameResolutionTL, IPv4NSEntry.IPv4Address.ValueAsLong, IPv4NSEntry.FQDN);
         end;

     // Write IPv6 Name Resolution Addresses

     for i := 1 to priv_IPv6NameResolutionMgr.Count do
         begin
              IPv6NSEntry := priv_IPv6NameResolutionMgr.getEntry(i);

              NameResolutionTL.RecordType := nres_ip6record;
              NameResolutionTL.RecordLength := 16 + length (IPv6NSEntry.FQDN + chr(0));

              WriteNSRecord (NameResolutionTL, IP6Address.ValueAsByteStruct, IPv6NSEntry.FQDN);
         end;

     // Write End of Records

     EndOfOpts := 0;                                           // Record Type = 0 (16 bit) and Record Length = 0 (16 bit) -> Longword = 0

{$ifdef usestream}
     prot_FileStream.Write (EndOfOpts, 4);
{$else}
     BlockWrite (prot_Filehandle, EndOfOpts, 4);
{$endif}

     // Write Options

     if priv_DNSServerList.Count > 0 then
        begin
             for i := 1 to priv_DNSServerList.Count do
                 begin
                      DNSEntry := priv_DNSServerList.getEntry(i);

                      WriteDNSEntryOption (DNSEntry);
                 end;

             if priv_WriteEndofopt then
                begin
                     OptionCodeLen.code := opt_endofopt;
                     OptionCodeLen.length := 0;

                     WriteOptionHeader (OptionCodeLen);
                end;
        end;

     // Write trailing total length

     WriteTrailingTotalLength (BlockTotalLen);

     result := BlockTotalLen;
end;

(* ------------------------------------------- *)

function tPCAPngWriter.DetermineInterfaceStatisticsBlockLength (StatisticsEntry : tStatisticsRecord) : integer;

var len      : integer;

begin
     len := 0;

     if StatisticsEntry.Comment <> '' then
        len := len + Get32BitAlignment (length (StatisticsEntry.Comment));

     len := len + (7 * 8);                      // 7 options (starttime, endtime, recvd, drop, filteraccept, osdrop, usrdeliv), each 8 bytes long

     // TODO: maybe make writing some options optional, and not write all of them every time?

     result := len;
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteInterfaceStatisticsBlock (StatisticsEntry : tStatisticsRecord) : longword;

var InterfaceStatHdr : NGInterfaceStatisticsBlock;
    Timestamp        : NGInterfaceStatisticsTimestamp;
    OptionCodeLen    : NGOptionCodeLen;
    OptionTotalLen   : integer;
    BlockTotalLen    : longword;
    i                : integer;
    count            : integer;

begin
     OptionTotalLen := DetermineInterfaceStatisticsBlockLength (StatisticsEntry);
     BlockTotalLen := sizeOf (NGInterfaceStatisticsBlock) + OptionTotalLen + 4;

     InterfaceStatHdr.BlockType := BLOCKTYPE_INTERFACE_STATISTICS;
     InterfaceStatHdr.BlockTotalLength := BlockTotalLen;
     InterfaceStatHdr.InterfaceID := StatisticsEntry.InterfaceIndex;

     // Determine Header Timestamp

     Timestamp := TimingDetailsToTimestamp (StatisticsEntry.ReferenceTime);
     InterfaceStatHdr.TimestampHigh := Timestamp.TimestampHigh;
     InterfaceStatHdr.TimestampLow := Timestamp.TimestampLow;

     // Write Header

     if priv_WriteBigEndian then
        begin
             InterfaceStatHdr.BlockType := Swap32 (InterfaceStatHdr.BlockType);
             InterfaceStatHdr.BlockTotalLength := Swap32 (InterfaceStatHdr.BlockTotalLength);
             InterfaceStatHdr.InterfaceID := Swap32 (InterfaceStatHdr.InterfaceID);
             InterfaceStatHdr.TimestampHigh := Swap32 (InterfaceStatHdr.TimestampHigh);
             InterfaceStatHdr.TimestampLow := Swap32 (InterfaceStatHdr.TimestampLow);
        end;

     count := sizeOf (NGInterfaceStatisticsBlock);

{$ifdef usestream}
     prot_FileStream.WriteBuffer (InterfaceStatHdr, count);
{$else}
     BlockWrite (prot_FileHandle, InterfaceStatHdr, count);
{$endif}

     // Write Options

     if StatisticsEntry.Comment <> '' then
        WriteStringOption (opt_comment, StatisticsEntry.Comment, false);

     Timestamp := TimingDetailsToTimestamp (StatisticsEntry.StartTime);
     WriteTimestampOption (isb_starttime, Timestamp);

     Timestamp := TimingDetailsToTimestamp (StatisticsEntry.EndTime);
     WriteTimestampOption (isb_endtime, Timestamp);

     WriteInt64Option (isb_ifrecv, StatisticsEntry.PacketsReceived);
     WriteInt64Option (isb_ifdrop, StatisticsEntry.PacketsDropped);

     // TODO: the next three NSB Options were disabled for Wireshark 1.10.0rc1 compatibility, may need to be reenabled at some point
     //       also, OptionTotalLen seems to be off when adding these

{
     WriteInt64Option (isb_filteraccept, StatisticsEntry.FilterAccepted);
     WriteInt64Option (isb_osdrop, StatisticsEntry.OSDropped);
     WriteInt64Option (isb_usrdeliv, StatisticsEntry.UserDelivered);
}
     if priv_WriteEndofopt then
        begin
             OptionCodeLen.code := opt_endofopt;
             OptionCodeLen.length := 0;

             WriteOptionHeader (OptionCodeLen);
        end;

     // Write trailing total length

     WriteTrailingTotalLength (BlockTotalLen);

     result := BlockTotalLen;
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteFrameBytes (FrameBytes: tRawBuffer; FrameSize : longword): integer;

var FrameLenAligned : longword;
    PacketBytes     : tRawBuffer;
    i               : integer;

begin
     try
        FrameLenAligned := Get32BitAlignment (FrameSize);
        PacketBytes := FrameBytes;

        if FrameSize < FrameLenAligned then
           for i := (FrameSize + 1) to FrameLenAligned do
               PacketBytes[i] := 0;
     except
           SiMain.LogFatal('Error: preparing 32bit aligned frame data failed');
           raise;
     end;

{$ifdef usestream}
     prot_FileStream.WriteBuffer (PacketBytes, FrameLenAligned);
{$else}
     try
        BlockWrite (prot_FileHandle, PacketBytes, FrameLenAligned);
     except
           SiMain.LogFatal('Error: blockwriting 32bit aligned frame data failed');
           raise;
     end;
{$endif}
end;

(* ------------------------------------------- *)

// This block type is deprecated, but we might want to write it anyway

function tPCAPngWriter.WriteBlock (Frame : tRawFrame; const TimestampResolution : byte = 6) : longword;

var PacketBlockHdr  : NGPacketBlock;
    Timestamp       : NGInterfaceStatisticsTimestamp;
    OptionCodeLen   : NGOptionCodeLen;
    BlockTotalLen   : longword;
    HelpString      : string;
    i               : integer;
    count           : integer;

begin
     // Trigger the onFrameWrite event

     if Assigned(EventWriteFrame) then
        EventWriteFrame (Self,0, Frame.FrameSize);

     // Assign block type

     PacketBlockHdr.BlockType := BLOCKTYPE_PACKET;

     // Determine Header Length

     BlockTotalLen := sizeOf (NGPacketBlock) + Get32BitAlignment (Frame.FrameSize) + 4;             // 4 for secondary total length
     BlockTotalLen := BlockTotalLen + sizeOf (NGOptionCodeLen) + 4;                                 // Flags Option

     if priv_WriteEndOfOpt then
        BlockTotalLen := BlockTotalLen + sizeOf (NGOptionCodeLen);

     if Frame.Commentlist.Count > 0 then
        begin
             for i := 1 to Frame.Commentlist.Count do
                 begin
                      HelpString := Frame.Commentlist[i-1];

                      BlockTotalLen := BlockTotalLen + sizeOf (NGOptionCodeLen) + Get32BitAlignment (length (HelpString));
                 end;
        end;

     PacketBlockHdr.BlockTotalLength := BlockTotalLen;

     // TODO: determine how to handle hashes and count the length

     // Assign static header values

     PacketBlockHdr.InterfaceID := Frame.InterfaceID;            // Should work even though we assign a 32bit value to a 16bit value
     PacketBlockHdr.DropCount := Frame.DropCount;                // Same here, but 64bit to 16bit

     Timestamp := FrameTimestampToTimestamp (Frame.Timestamp, TimestampResolution);

     PacketBlockHdr.TimestampHigh := Timestamp.TimestampHigh;
     PacketBlockHdr.TimestampLow := Timestamp.TimestampLow;

     PacketBlockHdr.CapturedLen := Frame.FrameSize;
     PacketBlockHdr.PacketLen := Frame.WireSize;

     if priv_WriteBigEndian then
        begin
             PacketBlockHdr.BlockType := Swap32 (PacketBlockHdr.BlockType);
             PacketBlockHdr.BlockTotalLength := Swap32 (PacketBlockHdr.BlockTotalLength);
             PacketBlockHdr.InterfaceID := Swap16 (PacketBlockHdr.InterfaceID);
             PacketBlockHdr.DropCount := Swap16 (PacketBlockHdr.DropCount);
             PacketBlockHdr.TimestampHigh := Swap32 (PacketBlockHdr.TimestampHigh);
             PacketBlockHdr.TimestampLow := Swap32 (PacketBlockHdr.TimestampLow);
             PacketBlockHdr.CapturedLen := Swap32 (PacketBlockHdr.CapturedLen);
             PacketBlockHdr.PacketLen := Swap32 (PacketBlockHdr.PacketLen);
        end;

     // Write the block header

     count := sizeOf (NGPacketBlock);

{$ifdef usestream}
     prot_FileStream.WriteBuffer (PacketBlockHdr, count);
{$else}
     BlockWrite (prot_FileHandle, PacketBlockHdr, count);
{$endif}

     // Write frame bytes, but make sure the 32bit alignment will be padded with zeros

     WriteFrameBytes (Frame.Buffer, Frame.FrameSize);

     // Write options

     if Frame.Commentlist.Count > 0 then
        begin
             for i := 1 to Frame.Commentlist.Count do
                 begin
                      HelpString := Frame.Commentlist[i-1];

                      WriteStringOption (opt_comment, HelpString, false);
                 end;
        end;

     WriteLongOption (pack_flags, Frame.FrameFlags.Value);

     // TODO: determine how to handle hashes and write them to file

     if priv_WriteEndofopt then
        begin
             OptionCodeLen.code := opt_endofopt;
             OptionCodeLen.length := 0;

             WriteOptionHeader (OptionCodeLen);
        end;

     // Write trailing total length

     WriteTrailingTotalLength (BlockTotalLen);

     result := BlockTotalLen;
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteSimpleBlock (Frame : tRawFrame) : longword;

var PacketBlockHdr  : NGSimplePacketBlock;
    Timestamp       : NGInterfaceStatisticsTimestamp;
    BlockTotalLen   : longword;
    OptionCodeLen   : NGOptionCodeLen;
    OptionLen       : integer;
    HelpString      : string;
    count           : integer;
    i               : integer;

begin
     PacketBlockHdr.BlockType := BLOCKTYPE_SIMPLE_PACKET;

     // Determine Header Length

     OptionLen := 0;

     if Frame.Commentlist.Count > 0 then
        begin
             for i := 1 to Frame.Commentlist.Count do
                 begin
                      HelpString := Frame.Commentlist[i-1];

                      OptionLen := OptionLen + sizeOf (NGOptionCodeLen) + Get32BitAlignment (length (HelpString));
                 end;
        end;

     if (OptionLen > 0) and priv_WriteEndOfOpt then
//      OptionLen :=  OptionLen + BlockTotalLen + sizeOf (NGOptionCodeLen);
        OptionLen :=  OptionLen + sizeOf (NGOptionCodeLen);

     BlockTotalLen := sizeOf (NGSimplePacketBlock) + OptionLen + Get32BitAlignment (Frame.FrameSize) + 4;             // 4 for secondary total length

     PacketBlockHdr.BlockTotalLength := BlockTotalLen;
     PacketBlockHdr.PacketLen := Frame.FrameSize;

     if priv_WriteBigEndian then
        begin
             PacketBlockHdr.BlockType := Swap32 (PacketBlockHdr.BlockType);
             PacketBlockHdr.BlockTotalLength := Swap32 (PacketBlockHdr.BlockTotalLength);
             PacketBlockHdr.PacketLen := Swap32 (PacketBlockHdr.PacketLen);
        end;

     // Write the block header

     count := sizeOf (NGSimplePacketBlock);

{$ifdef usestream}
     prot_FileStream.WriteBuffer (PacketBlockHdr, count);
{$else}
     BlockWrite (prot_FileHandle, PacketBlockHdr, count);
{$endif}

     // Write options

     if Frame.Commentlist.Count > 0 then
        begin
             for i := 1 to Frame.Commentlist.Count do
                 begin
                      HelpString := Frame.Commentlist[i-1];

                      WriteStringOption (opt_comment, HelpString, false);
                 end;
        end;

     if (OptionLen > 0) and priv_WriteEndofopt then
        begin
             OptionCodeLen.code := opt_endofopt;
             OptionCodeLen.length := 0;

             WriteOptionHeader (OptionCodeLen);
        end;

     // Write frame bytes, but make sure the 32bit alignment will be padded with zeros

     WriteFrameBytes (Frame.Buffer, Frame.FrameSize);

     // Write trailing total length

     WriteTrailingTotalLength (BlockTotalLen);

     result := BlockTotalLen;
end;

(* ------------------------------------------- *)

function tPCAPngWriter.WriteEnhancedBlock (Frame : tRawFrame; TimestampResolution : byte = 6) : longword;

var PacketBlockHdr  : NGEnhPacketBlock;
    Timestamp       : NGInterfaceStatisticsTimestamp;
    OptionCodeLen   : NGOptionCodeLen;
    OptionLen       : integer;
    BlockTotalLen   : longword;
    HelpString      : string;
    count           : integer;
    i               : integer;

begin
     PacketBlockHdr.BlockType := BLOCKTYPE_ENHANCED_PACKET;

     // Determine Header Length

     OptionLen := 0;

     if Frame.FrameFlags.Present then
        OptionLen := sizeOf (NGOptionCodeLen) + 4;

     if Frame.Commentlist.Count > 0 then
        begin
             for i := 1 to Frame.Commentlist.Count do
                 begin
                      HelpString := Frame.Commentlist[i-1];

                      OptionLen := OptionLen + sizeOf (NGOptionCodeLen) + Get32BitAlignment (length (HelpString));
                 end;
        end;

     if (OptionLen > 0) and priv_WriteEndOfOpt then
        OptionLen :=  OptionLen + sizeOf (NGOptionCodeLen);

     BlockTotalLen := sizeOf (NGEnhPacketBlock) + OptionLen + Get32BitAlignment (Frame.FrameSize) + 4;             // 4 for secondary total length

     PacketBlockHdr.BlockTotalLength := BlockTotalLen;

     // Assign static header values

     PacketBlockHdr.InterfaceID := Frame.InterfaceID;

     Timestamp := FrameTimestampToTimestamp (Frame.Timestamp, TimestampResolution);

     PacketBlockHdr.BlockTotalLength := BlockTotalLen;
     PacketBlockHdr.TimestampHigh := Timestamp.TimestampHigh;
     PacketBlockHdr.TimestampLow := Timestamp.TimestampLow;
     PacketBlockHdr.CapturedLen := Frame.FrameSize;
     PacketBlockHdr.PacketLen := Frame.WireSize;

     if priv_WriteBigEndian then
        begin
             PacketBlockHdr.BlockType := Swap32 (PacketBlockHdr.BlockType);
             PacketBlockHdr.BlockTotalLength := Swap32 (PacketBlockHdr.BlockTotalLength);
             PacketBlockHdr.InterfaceID := Swap32 (PacketBlockHdr.InterfaceID);
             PacketBlockHdr.TimestampHigh := Swap32 (PacketBlockHdr.TimestampHigh);
             PacketBlockHdr.TimestampLow := Swap32 (PacketBlockHdr.TimestampLow);
             PacketBlockHdr.CapturedLen := Swap32 (PacketBlockHdr.CapturedLen);
             PacketBlockHdr.PacketLen := Swap32 (PacketBlockHdr.PacketLen);
        end;

     // Write the block header

     count := sizeOf (NGEnhPacketBlock);

{$ifdef usestream}
     prot_FileStream.WriteBuffer (PacketBlockHdr, count);
{$else}
     BlockWrite (prot_FileHandle, PacketBlockHdr, count);
{$endif}

     // Write frame bytes, but make sure the 32bit alignment will be padded with zeros

     WriteFrameBytes (Frame.Buffer, Frame.FrameSize);

     // Write options

     if Frame.Commentlist.Count > 0 then
        begin
             for i := 1 to Frame.Commentlist.Count do
                 begin
                      HelpString := Frame.Commentlist[i-1];

                      WriteStringOption (opt_comment, HelpString, false);
                 end;
        end;

     if Frame.FrameFlags.Present then
        WriteLongOption (pack_flags, Frame.FrameFlags.Value);

     // TODO: determine how to handle hashes and dropcount for enhanced packet blocks and write them to file. Don't forget the BlockTotalLength

     if (OptionLen > 0) and priv_WriteEndofopt then
        begin
             OptionCodeLen.code := opt_endofopt;
             OptionCodeLen.length := 0;

             WriteOptionHeader (OptionCodeLen);
        end;

     // Write trailing total length

     WriteTrailingTotalLength (BlockTotalLen);

     result := BlockTotalLen;
end;

(* ------------------------------------------- *)

procedure tPCAPngWriter.CopyPCAPngHeaderDetailsFromLoader (Loader: tPCAPNGLoader);

var InterfaceEntry : tCaptureInterface;
    NewInterface   : tCaptureInterface;
    NREntryIPv4    : tSimpleIPv4NameResolutionEntry;
    NREntryIPv6    : tSimpleIPv6NameResolutionEntry;
    HelpString     : string;
    i              : integer;

begin
     // Transfer SHB optionlist values

     // Header Comments

     if Loader.HeaderComments.Count = 0 then
        begin
             if assigned (EventCopySHBString) then
                begin
                     HelpString := '';

                     EventCopySHBString (sstComment, HelpString);

                     if HelpString <> '' then
                        priv_HeaderCommentList.Add(HelpString);
                end;
        end
     else
         for i := 1 to Loader.HeaderComments.Count do
             begin
                  HelpString := Loader.HeaderComments[i-1];

                  if assigned (EventCopySHBString) then
                     EventCopySHBString (sstComment, HelpString);

                  priv_HeaderCommentList.Add(HelpString);
             end;

     // Hardware

     if Loader.Hardware.Count = 0 then
        begin
             if assigned (EventCopySHBString) then
                begin
                     HelpString := '';

                     EventCopySHBString (sstHardware, HelpString);

                     if HelpString <> '' then
                        priv_HardwareInfoList.Add(HelpString);
                end;
        end
     else
         for i := 1 to Loader.Hardware.Count do
             begin
                  HelpString := Loader.Hardware[i-1];

                  if assigned (EventCopySHBString) then
                     EventCopySHBString (sstHardware, HelpString);

                  priv_HardwareInfoList.Add(HelpString);
             end;

     // OperatingSystem

     if Loader.OperatingSystem.Count = 0 then
        begin
             if assigned (EventCopySHBString) then
                begin
                     HelpString := '';

                     EventCopySHBString (sstOperatingSystem, HelpString);

                     if HelpString <> '' then
                        priv_OperatingSystemList.Add(HelpString);
                end;
        end
     else
         for i := 1 to Loader.OperatingSystem.Count do
             begin
                  HelpString := Loader.OperatingSystem[i-1];

                  if assigned (EventCopySHBString) then
                     EventCopySHBString (sstOperatingSystem, HelpString);

                  priv_OperatingSystemList.Add(HelpString);
             end;

     // User Application

     if Loader.Userapplication.Count = 0 then
        begin
             if assigned (EventCopySHBString) then
                begin
                     HelpString := '';

                     EventCopySHBString (sstApplication, HelpString);

                     if HelpString <> '' then
                        priv_UserApplicationList.Add(HelpString);
                end;
        end
     else
         for i := 1 to Loader.Userapplication.Count do
             begin
                  HelpString := Loader.Userapplication[i-1];

                  if assigned (EventCopySHBString) then
                     EventCopySHBString (sstApplication, HelpString);

                  priv_UserApplicationList.Add(HelpString);
             end;

     WriteSectionHeaderBlock;

     // Transfer and write IDB values

     for i := 1 to Loader.CaptureInterfaces.Count do
         begin
              InterfaceEntry := Loader.CaptureInterfaces.getInterfaceByIndex(i-1);

              WriteInterfaceDescriptionBlock (InterfaceEntry);
         end;

     // Transfer NRB values, if any

     if (Loader.NameResolutionManagerIPv4.count > 0) or (Loader.NameResolutionManagerIPv6.count > 0) then
        begin
             for i := 1 to Loader.NameResolutionManagerIPv4.count do
                 begin
                      NREntryIPv4 := Loader.NameResolutionManagerIPv4.GetEntry(i);

                      priv_IPv4NameResolutionMgr.AddEntry(NREntryIPv4);
                 end;

             for i := 1 to Loader.NameResolutionManagerIPv6.count do
                 begin
                      NREntryIPv6 := Loader.NameResolutionManagerIPv6.GetEntry(i);

                      priv_IPv6NameResolutionMgr.AddEntry(NREntryIPv6);
                 end;

             WriteNameResolutionBlock;
        end;
end;

(* ------------------------------------------- *)

procedure tPCAPngWriter.CopyPCAPngStatisticsDetailsFromLoader (Loader: tPCAPNGLoader);

var InterfaceEntry : tCaptureInterface;
    StatRec        : tStatisticsRecord;
    i,j            : integer;

begin
     for i := 1 to Loader.CaptureInterfaces.Count do
         begin
              InterfaceEntry := Loader.CaptureInterfaces.getInterfaceByIndex(i-1);

              for j := 1 to InterfaceEntry.InterfaceStatistics.Count do
                  begin
                       StatRec := InterfaceEntry.InterfaceStatistics.getEntry(j);

                       WriteInterfaceStatisticsBlock (StatRec);
                  end;
         end;
end;

(* ------------------------------------------- *)

procedure tPCAPngWriter.WriteGenericHeader (const LinkType : longword);

var InterfaceEntry : tCaptureInterface;

begin
     InterfaceEntry := tCaptureInterface.create (0, LinkType, ioAutoGenerated ,true);

     WriteSectionHeaderBlock;
     WriteInterfaceDescriptionBlock (InterfaceEntry);
end;

(* ------------------------------------------- *)

procedure tPCAPngWriter.WriteGenericHeader (HeaderInterfaceList : tCaptureInterfaceList);

var InterfaceEntry : tCaptureInterface;
    i              : integer;

begin
     WriteSectionHeaderBlock;
     priv_interfacelist.clear;
{
     HeaderInterfaceList.copyTo (priv_interfacelist);           // copy is done per interface on writing the entry now
}
     for i := 1 to HeaderInterfaceList.Count do
         begin
              InterfaceEntry := HeaderInterfaceList.getInterfaceByIndex(i-1);

              WriteInterfaceDescriptionBlock (InterfaceEntry);
         end;
end;

(* ------------------------------------------- *)

procedure tPCAPngWriter.Flush;

var WriteFrame : tRawFrame;
    i          : integer;
    frameindex : integer;
    Entry      : tCaptureInterface;
    TimeRes    : byte;

begin
     SiMain.EnterMethod ('tPCAPngWriter.Flush');

     if priv_InterfaceList.Count = 0 then
        begin
             SiMain.LogFatal('Error: not interfaces defined in writer object on tPCAPngWriter.flush');
             raise EWriterException.Create (MSG_WRITER_INTERFACE_UNDEFINED, 'Error: not interfaces defined in writer object on tPCAPngWriter.flush', 0);
        end;

//   SiMain.LogVerbose ('PCAPngWriter - Flushing ' + IntToStr (priv_WriteFrames.Count) + ' to disk (Total Flushes: ' + IntToStr (prot_flushcount) + ')');

     prot_Stopwatch.StartNew;

     for i := 1 to prot_WriteFrames.Count do
         begin
              frameindex := (prot_flushcount * prot_maxframes) + i;

              WriteFrame := prot_WriteFrames[i-1] as tRawFrame;

              try
                 Entry := priv_InterfaceList.getInterfaceByIndex (WriteFrame.InterfaceID);
              except
                    SiMain.LogFatal ('Error: could not get a valid interface for the frame to write');
              end;

              // Check if the interface in the packet is consistent with the interface in the list

              if WriteFrame.InterfaceLinkID <> Entry.PacketLinkID then
                 begin
                      raise EWriterException.Create (MSG_WRITER_LINKID_MISMATCH, 'Error: Frame and associated interface link ID mismatch', frameindex);
                 end;

              // Get the timestamp resolution from the interface

              if Entry <> nil then
                 TimeRes := Entry.TimeStampResolution
              else
                  begin
                       SiMain.LogFatal ('Error writing frame ' + IntToStr (prot_flushcount) + '/' + IntToStr (i) + ' to file "' + prot_filename + '"');
                       raise EWriterException.Create(MSG_WRITER_ERROR_WRITING_FRAME, 'Error writing frame ' + IntToStr (frameindex) + ' to file "' + prot_filename + '"', frameindex);
                  end;

              if assigned (onWriteFrame) then
                 onWriteFrame (self, frameindex, WriteFrame.FrameSize);

              try
                 case priv_BlockWriteMode of
                  bwmPacketBlock          : WriteBlock (WriteFrame, TimeRes);
                  bwmSimplePacketBlock    : WriteSimpleBlock (WriteFrame);
                  bwmEnhancedPacketBlock  : WriteEnhancedBlock (WriteFrame, TimeRes);
                 end;
              except
                    SiMain.LogFatal ('Error writing frame ' + IntToStr (prot_flushcount) + '/' + IntToStr (i) + ' to file "' + prot_filename + '"');
                    raise;
              end;
         end;

     prot_FlushDuration := prot_Stopwatch.Elapsed;
     prot_StopWatch.Stop;

//   SiMain.LogVerbose ('PCAPngWriter - Flush complete');

     inc (prot_flushcount);

     // free the framelist for the next batch. If the constructor was called with the
     // DiscardFramesAfterWrite=true parameter, then this will NOT free the RawFrame objects,
     // just clear the list!

     prot_WriteFrames.Clear;
     SiMain.LeaveMethod ('tPCAPngWriter.Flush');
end;

(* ------------------------------------------- *)

end.
