{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit pcapng_structures;

interface

uses classes,
     contnrs,
     base_structures,
     definition_ip_addresses,
     definition_simple_nameresolution;

     function GetBlockTypeName (BlockTypeID : integer) : string;


const
     // Block Type Codes

     BLOCKTYPE_RESERVED              = $00000000;
     BLOCKTYPE_INTERFACE_DESCRIPTION = $00000001;
     BLOCKTYPE_PACKET                = $00000002;
     BLOCKTYPE_SIMPLE_PACKET         = $00000003;
     BLOCKTYPE_NAME_RESOLUTION       = $00000004;
     BLOCKTYPE_INTERFACE_STATISTICS  = $00000005;
     BLOCKTYPE_ENHANCED_PACKET       = $00000006;
     BLOCKTYPE_IRIG_TIMESTAMP        = $00000007;
     BLOCKTYPE_ARINC_429_IN_AFDX     = $00000008;
     BLOCKTYPE_HONE_PROCESS_EVENT    = $00000101;
     BLOCKTYPE_HONE_CONNECTION_EVENT = $00000102;
     BLOCKTYPE_SYSDIG_MACHINE_INFO   = $00000201;
     BLOCKTYPE_SYSDIG_PROCESS_LIST   = $00000202;
     BLOCKTYPE_SYSDIG_FD_LIST        = $00000203;
     BLOCKTYPE_SYSDIG_EVENT          = $00000204;
     BLOCKTYPE_SYSDIG_INTERFACE_LIST = $00000205;
     BLOCKTYPE_SYSDIG_USER_LIST      = $00000206;
     BLOCKTYPE_SECTION_HEADER        = $0A0D0D0A;

     // Byte-Order Magic

     SHB_BYTEORDERMAGIC = $1A2B3C4D;

     // Option block codes - General Codes

     opt_endofopt       =	0;             // It delimits the end of the optional fields. This block cannot be repeated within a given list of options.
     opt_comment 	      = 1;             // A UTF-8 string containing a comment that is associated to the current block.

     // Option block codes - Section Header

     shb_hardware =	2; 	                 // An UTF-8 string containing the description of the hardware used to create this section. 	"x86 Personal Computer" / "Sun Sparc Workstation" / ...
     shb_os =	3;                         // An UTF-8 string containing the name of the operating system used to create this section. 	"Windows XP SP2" / "openSUSE 10.2" / ...
     shb_userappl =	4;                   // An UTF-8 string containing the name of the application used to create this section. 	"dumpcap V0.99.7" / ...
     shb_hone_guid = 257;                // Hone Project - GUID which uniquely identifies the machine.

     // Option block codes - Interface Description specific

     opt_if_name        =	2;             // A UTF-8 string containing the name of the device used to capture data.
     opt_if_description =	3;             // A UTF-8 string containing the description of the device used to capture data.
     opt_if_IPv4addr 	  = 4;             // Interface network address and netmask. This option can be repeated multiple times within the same Interface Description Block when multiple IPv4 addresses are assigned to the interface.
     opt_if_IPv6addr 	  = 5;             // Interface network address and prefix length (stored in the last byte). This option can be repeated multiple times within the same Interface Description Block when multiple IPv6 addresses are assigned to the interface.
     opt_if_MACaddr 	   = 6;             // Interface Hardware MAC address (48 bits)
     opt_if_EUIaddr 	   = 7;             // Interface Hardware EUI address (64 bits), if available.
     opt_if_speed 	     = 8;             // Interface speed (in bps).
     opt_if_tsresol 	   = 9;             // Resolution of timestamps. If the Most Significant Bit is equal to zero, the remaining bits indicates the resolution of the timestamp as as a negative power of 10
     opt_if_tzone 	     = 10;            // Time zone for GMT support
     opt_if_filter 	    = 11;            // The filter (e.g. "capture only TCP traffic") used to capture traffic. The first byte of the Option Data keeps a code of the filter used
     opt_if_os 	        = 12;            // A UTF-8 string containing the name of the operating system of the machine in which this interface is installed
     opt_if_fcslen 	    = 13;            // An integer value that specified the length of the Frame Check Sequence (in bits) for this interface
     opt_if_tsoffset 	  = 14;            // A 64 bits integer value that specifies an offset (in seconds) that must be added to the timestamp of each packet to obtain the absolute timestamp of a packet.

     // Option block codes - Enhanced Packet Block

     opt_epb_flags 	    = 2;             // A flags word containing link-layer information
     opt_epb_hash 	     = 3;             // This option contains a hash of the packet. The first byte specifies the hashing algorithm, while the following bytes contain the actual hash, whose size depends on the hashing algorithm,
     opt_epb_dropcount 	= 4;             // A 64bit integer value specifying the number of packets lost (by the interface and the operating system) between this packet and the preceding one.

     // Option block codes - deprecated Packet Block

     pack_flags         = 2;
     pack_hash          = 3;

     // Name Resolution Options

     ns_dnsname =	2; 	                   // Variable 	A UTF-8 string containing the name of the machine (DNS server) used to perform the name resolution.
     ns_dnsIP4addr =	3;                  //	The IPv4 address of the DNS server. 	192 168 0 1
     ns_dnsIP6addr =	4; 	                // The IPv6 address of the DNS server.

     // Interface Description Options

     isb_starttime =	2;                  // Time in which the capture started; time will be stored in two blocks of four bytes each. The format of the timestamp is the same already defined in the Enhanced Packet Block (Section 3.3). 	TODO: give a good example
     isb_endtime =	3;                    // Time in which the capture ended; ; time will be stored in two blocks of four bytes each. The format of the timestamp is the same already defined in the Enhanced Packet Block (Section 3.3). 	TODO: give a good example
     isb_ifrecv =	4;                     // 	Number of packets received from the physical interface starting from the beginning of the capture. 	100
     isb_ifdrop =	5;                     //	Number of packets dropped by the interface due to lack of resources starting from the beginning of the capture. 	0
     isb_filteraccept =	6;               //	Number of packets accepted by filter starting from the beginning of the capture. 	100
     isb_osdrop =	7;                     // 	Number of packets dropped by the operating system starting from the beginning of the capture. 	0
     isb_usrdeliv =	8;                   //	Number of packets delivered to the user starting from the beginning of the capture. The value contained in this field can be different from the value 'isb_filteraccept - isb_osdrop' because some packets could still lay in the OS buffers when the capture ended.

     // Name Resolution Block Record Types

     nres_endofrecord =	0;               // It delimits the end of name resolution records. This record is needed to determine when the list of name resolution records has ended and some options (if any) begin.
     nres_ip4record =	1;	                // Variable 	Specifies an IPv4 address (contained in the first 4 bytes), followed by one or more zero-terminated strings containing the DNS entries for that address. 	127 0 0 1 "localhost"
     nres_ip6record =	2;                 // Specifies an IPv6 address (contained in the first 16 bytes), followed by one or more zero-terminated strings containing the DNS entries for that address.



type tBlockWriteMode = (bwmPacketBlock, bwmSimplePacketBlock, bwmEnhancedPacketBlock);

     pNGOptionCodeLen = ^NGOptionCodeLen;
     NGOptionCodeLen = packed record
                                    code        : word;
                                    length      : word;
                              end;

     pNGSectionHdrBlock = ^NGSectionHdrBlock;
     NGSectionHdrBlock   = packed record
                                        BlockType        : longword;      // The block type of the Section Header Block is the integer corresponding to the 4-char string "\r\n\n\r" (0x0A0D0D0A)
                                        BlockTotalLength : longword;      // total size of this block
                                        ByteOrderMagic   : longword;      // magic number, whose value is the hexadecimal number 0x1A2B3C4D.
                                        MajorVersion     : word;          // number of the current mayor version of the format. Current value is 1.
                                        MinorVersion     : word;          // number of the current minor version of the format. Current value is 0
                                        SectionLength    : uint64;        // unsigned 64-bit value specifying the length in bytes of the following section, excluding the Section Header Block itself
                                end;

     pNGIntDescrBlock = ^NGIntDescrBlock;
     NGIntDescrBlock     = packed record
                                        BlockType        : longword;      // The block type of the Interface Description Block is 1.
                                        BlockTotalLength : longword;      // total size of this block
                                        LinkType         : word;          // a value that defines the link layer type of this interface
                                        Reserved         : word;
                                        SnapLen          : longword;      // maximum number of bytes dumped from each packet
                                  end;

     pNGPacketBlock    = ^NGPacketBlock;
     NGPacketBlock       = packed record
                                        BlockType        : longword;      // The block type of the Enhanced Packet Block is 6
                                        BlockTotalLength : longword;      // total size of this block
                                        InterfaceID      : word;          // it specifies the interface this packet comes from; the correct interface will be the one whose Interface Description Block (within the current Section of the file) is identified by the same number of this field
                                        DropCount        : word;          // a local drop counter. It specifies the number of packets lost (by the interface and the operating system) between this packet and the preceding one. The value xFFFF (in hexadecimal) is reserved for those systems in which this information is not available.
                                        TimestampHigh    : longword;      // high 32-bits of a 64-bit quantity representing the timestamp. The timestamp is a single 64-bit unsigned integer representing the number of units since 1/1/1970
                                        TimestampLow     : longword;      // low 32-bits of a 64-bit quantity representing the timestamp. The way to interpret this field is specified by the 'if_tsresol' option of the Interface Description block referenced by this packet
                                        CapturedLen      : longword;      // number of bytes captured from the packet (i.e. the length of the Packet Data field). It will be the minimum value among the actual Packet Length and the snapshot length
                                        PacketLen        : longword;      // actual length of the packet when it was transmitted on the network
                                  end;

     pNGEnhPacketBlock = ^NGEnhPacketBlock;
     NGEnhPacketBlock    = packed record
                                        BlockType        : longword;      // The block type of the Enhanced Packet Block is 6
                                        BlockTotalLength : longword;      // total size of this block
                                        InterfaceID      : longword;      // it specifies the interface this packet comes from; the correct interface will be the one whose Interface Description Block (within the current Section of the file) is identified by the same number of this field
                                        TimestampHigh    : longword;      // high 32-bits of a 64-bit quantity representing the timestamp. The timestamp is a single 64-bit unsigned integer representing the number of units since 1/1/1970
                                        TimestampLow     : longword;      // low 32-bits of a 64-bit quantity representing the timestamp. The way to interpret this field is specified by the 'if_tsresol' option of the Interface Description block referenced by this packet
                                        CapturedLen      : longword;      // number of bytes captured from the packet (i.e. the length of the Packet Data field). It will be the minimum value among the actual Packet Length and the snapshot length
                                        PacketLen        : longword;      // actual length of the packet when it was transmitted on the network
                                  end;

     pNGSimplePacketBlock = ^NGSimplePacketBlock;
     NGSimplePacketBlock = packed record
                                        BlockType        : longword;      // The block type of the Enhanced Packet Block is 3
                                        BlockTotalLength : longword;      // total size of this block
                                        PacketLen        : longword;      // actual length of the packet when it was transmitted on the network
                                  end;

     pNGNameResolutionBlock = ^NGNameResolutionBlock;
     NGNameResolutionBlock = packed record
                                          BlockType        : longword;    // The block type of the Name Resolution Block is 4
                                          BlockTotalLength : longword;    // total size of this block
                                    end;

     pNGRNameResolutionTypeLen = ^NGRNameResolutionTypeLen;
     NGRNameResolutionTypeLen = packed record
                                             RecordType    : word;
                                             RecordLength  : word;
                                       end;

     pNGInterfaceStatisticsBlock = ^NGInterfaceStatisticsBlock;
     NGInterfaceStatisticsBlock = packed record
                                               BlockType        : longword;    // The block type of the Name Resolution Block is 5
                                               BlockTotalLength : longword;    // total size of this block
                                               InterfaceID      : longword;    // it specifies the interface these statistics refers to; the correct interface will be the one whose Interface Description Block (within the current Section of the file) is identified by same number
                                               TimestampHigh    : longword;    // Timestamp: time this statistics refers to. The format of the timestamp is the same already defined in the Enhanced Packet Block
                                               TimestampLow     : longword;    // Timestamp: time this statistics refers to. The format of the timestamp is the same already defined in the Enhanced Packet Block
                                         end;

     pNGInterfaceStatisticsTimestamp = ^NGInterfaceStatisticsTimestamp;
     NGInterfaceStatisticsTimestamp = packed record
                                                   TimestampHigh    : longword;    // Timestamp: time this statistics refers to. The format of the timestamp is the same already defined in the Enhanced Packet Block
                                                   TimestampLow     : longword;    // Timestamp: time this statistics refers to. The format of the timestamp is the same already defined in the Enhanced Packet Block
                                             end;

     pNGHoneProcessEventBlock = ^tNGHoneProcessEventBlock;
     tNGHoneProcessEventBlock = packed record
                                             BlockType        : longword;    // The block type of the Name Resolution Block is 5
                                             BlockTotalLength : longword;    // total size of this block
                                             ProcessID        : longword;    // the process ID (PID) as given by the OS.
                                             TimeStampHigh    : longword;    // high 32-bits of a 64-bit quantity representing the timestamp, as described in Section 3.3 (Enhanced Packet Block).
                                             TimeStampLow     : longword;    // low 32-bits of a 64-bit quantity representing the timestamp, as described in Section 3.3 (Enhanced Packet Block).
                                end;

     pNGHoneConnectionEventBlock = ^tNGHoneProcessEventBlock;
     tNGHoneConnectionEventBlock = packed record
                                                BlockType        : longword;    // The block type of the Name Resolution Block is 5
                                                BlockTotalLength : longword;    // total size of this block
                                                ConnectionID     : longword;    // the connection ID of the connection. This ID is unique only over the lifetime of the connection.
                                                ProcessID        : longword;    // the process ID (PID) of the process.
                                                TimeStampHigh    : longword;    // high 32-bits of a 64-bit quantity representing the timestamp, as described in Section 3.3 (Enhanced Packet Block).
                                                TimeStampLow     : longword;    // low 32-bits of a 64-bit quantity representing the timestamp, as described in Section 3.3 (Enhanced Packet Block).
                                   end;

     // Additional structures for parsing pcap-ng files

     tNGSHBParsedOptionValues    = record
                                         HeaderCommentList       : tStringList;
                                         HardwareInfoList        : tStringList;
                                         OperatingSystemList     : tStringList;
                                         UserApplicationList     : tStringList;

                                         CaptureSystemGUID       : tGUID;   // Used by the Hone Project to associate programs/ports/packets to a system that took the capture

                                         procedure initialize;
                                         procedure cleanUp;
                                   end;

     tNGSHBParsedValues          = record
                                         BlockType               : longword;
                                         BlockTotalLength        : longword;
                                         ByteOrderMagic          : longword;
                                         MajorVersion            : word;
                                         MinorVersion            : word;
                                         TotalSectionLength      : uint64;

                                         OptionValues            : tNGSHBParsedOptionValues;

                                         procedure initialize;
                                   end;

     tNGNRBParsedValues          = record
                                         IPv4AddressList : tSimpleIPv4NameResolutionList;
                                         IPv6AddressList : tSimpleIPv6NameResolutionList;

                                         procedure initialize;
                                   end;

     tNGInterfaceResolutionList  = record
                                         count : integer;
                                         items : array [0..255] of byte;

                                         procedure initialize;
                                   end;

     tPCAPNGCaptureFilterType = (cftUnknown, cftLibPCAP, cftBPFByteCode);

     tPCAPNGCaptureFilter        = record
                                         FilterType    : tPCAPNGCaptureFilterType;
                                         FilterString  : string;

                                         procedure initialize;
                                         function FilterTypeName : string;
                                   end;

     tPCAPNGPacketComment     = record
                                      FrameNo        : integer;
                                      Comment        : string;
                                end;

     tPCAPNGPacketCommentList = array of tPCAPNGPacketComment;

     // Statistics structures

     pBlockChainEntry = ^tBlockChainEntry;
     tBlockChainEntry = record
                              BlockTypeID            : integer;
                              BlockTypeName          : string;
                              SectionStartOffset     : int64;
                              SectionEndOffset       : int64;
                              FirstBlockStartOffset  : int64;
                              FirstBlockStartPointer : pByte;
                              LastBlockEndOffset     : int64;
                              BlockLength            : integer;
                              Count                  : int64;

                              // SHB Stats

                              SHBMajorVersion       : word;
                              SHBMinorVersion       : word;
                              SHBByteOrderMagic     : longword;
                              SHBSectionLength      : uint64;
                              SHBComment            : string;
                              SHBHardware           : string;
                              SHBOS                 : string;
                              SHBUserAppl           : string;
                              SHBGUID               : tGUID;

                              // IDB Stats

                              IDBIndex              : longword;
                              IDBLinkType           : word;
                              IDBSnapLen            : longword;
                              IDBFCSLen             : byte;
                              IDBName               : string;
                              IDBDescription        : string;
                              IDBMACAddress         : tMACAddress;
                              IDBIPv4AddressList    : tIPv4SimpleAddressList;
                              IDBIPv6AddressList    : tIPv6SimpleAddressList;
                              IDBTSResolution       : byte;
                              IDBFilter             : tPCAPNGCaptureFilter;

                              // ISB Stats

                              ISBInterfaceID        : longword;
                              ISBTimestamp          : tDateTime;
                              ISBStartTime          : tDateTime;
                              ISBEndTime            : tDateTime;
                              ISBReceived           : longword;
                              ISBDropped            : longword;

                              // NRB Stats

                              NRBIPv4RecordCount    : integer;
                              NRBIPv6RecordCount    : integer;
                              NRBIPv4AddressList    : tSimpleIPv4NameResolutionList;
                              NRBIPv6AddressList    : tSimpleIPv6NameResolutionList;

                              // Pointers

                              PreviousBlockType     : pBlockChainEntry;
                              NextBlockType         : pBlockChainEntry;

                              procedure initialize;
                              procedure FreeSubChain;
                        end;

     tPCAPNGStatistic = record
                              SectionCount          : integer;
                              FilePacketCount       : integer;
                              FileInterfaceCount    : integer;
                              StatisticBlocKCount   : integer;
                              StatPacketCount       : uint64;
                              StatDropCount         : uint64;
                              StatFilterAcceptCount : uint64;
                              StatOsDropCount       : uint64;
                              StatUsrDeliveredCount : uint64;

                              BlockChain            : pBlockChainEntry;

                              ErrorCode             : integer;
                              ErrorText             : string;

                              FirstFrameTime        : tDateTime;  // Not used yet
                              LastFrameTime         : tDateTime;  // Not used yet

                              procedure initialize;
                        end;

implementation

uses SysUtils,
     DateUtils;


function GetBlockTypeName (BlockTypeID : integer) : string;

begin
     case BlockTypeID of
      BLOCKTYPE_SECTION_HEADER        : result := 'Section Header';
      BLOCKTYPE_INTERFACE_DESCRIPTION : result := 'Interface Description';
      BLOCKTYPE_PACKET                : result := 'Packet (Deprecated)';
      BLOCKTYPE_SIMPLE_PACKET         : result := 'Simple Packet';
      BLOCKTYPE_ENHANCED_PACKET       : result := 'Enhanced Packet';
      BLOCKTYPE_NAME_RESOLUTION       : result := 'Name Resolution';
      BLOCKTYPE_INTERFACE_STATISTICS  : result := 'Interface Statistics';
      BLOCKTYPE_IRIG_TIMESTAMP        : result := 'IRIG Timestamp';
      BLOCKTYPE_ARINC_429_IN_AFDX     : result := 'ARICN 429 in AFDX';
      BLOCKTYPE_HONE_PROCESS_EVENT    : result := 'HoneProject Process Event';
      BLOCKTYPE_HONE_CONNECTION_EVENT : result := 'HoneProject Connection Event';
      BLOCKTYPE_SYSDIG_MACHINE_INFO   : result := 'SysDig Machine Info';
      BLOCKTYPE_SYSDIG_PROCESS_LIST   : result := 'SysDig Process List';
      BLOCKTYPE_SYSDIG_FD_LIST        : result := 'SysDig FD List';
      BLOCKTYPE_SYSDIG_EVENT          : result := 'SysDig Event';
      BLOCKTYPE_SYSDIG_INTERFACE_LIST : result := 'SysDig Interface List';
      BLOCKTYPE_SYSDIG_USER_LIST      : result := 'SysDig User List';

     else
         result := 'Unknown';
     end;
end;

(* ------------------------------------------- *)
{ tPCAPNGCaptureFilter }
(* ------------------------------------------- *)

procedure tPCAPNGCaptureFilter.initialize;

begin
     FilterType := cftUnknown;
     FilterString := '';
end;

(* ------------------------------------------- *)

function tPCAPNGCaptureFilter.FilterTypeName: string;

begin
     case FilterType of
      cftUnknown     : result := 'Unknown';
      cftLibPCAP     : result := 'LibPCAP';
      cftBPFByteCode : result := 'BPFByteCode';
     end;
end;

(* ------------------------------------------- *)
{ tBlockChainEntry }
(* ------------------------------------------- *)

procedure tBlockChainEntry.initialize;

begin
     BlockTypeID            := 0;
     BlockTypeName          := '';
     SectionStartOffset     := 0;
     SectionEndOffset       := 0;
     FirstBlockStartOffset  := 0;
     FirstBlockStartPointer := nil;
     LastBlockEndOffset     := 0;
     BlockLength            := 0;
     Count                  := 0;

     // SHB Stats

     SHBMajorVersion       := 0;
     SHBMinorVersion       := 0;
     SHBByteOrderMagic     := 0;
     SHBSectionLength      := 0;
     SHBComment            := '';
     SHBHardware           := '';
     SHBOS                 := '';
     SHBUserAppl           := '';
     SHBGUID               := tGUID.Empty;

     // IDB Stats

     IDBIndex              := 0;
     IDBLinkType           := 0;
     IDBSnapLen            := 0;
     IDBFCSLen             := 0;
     IDBName               := '';
     IDBDescription        := '';
     IDBTSResolution       := 0;
     IDBMACAddress.initialize;
     setLength (IDBIPv4AddressList.list, 0);
     setLength (IDBIPv6AddressList.list, 0);

     IDBFilter.initialize;

     // ISB Stats

     ISBInterfaceID        := 0;
     ISBTimestamp          := 0;
     ISBStartTime          := 0;
     ISBEndTime            := 0;
     ISBReceived           := 0;
     ISBDropped            := 0;

     // NRB Stats

     NRBIPv4RecordCount    := 0;
     NRBIPv6RecordCount    := 0;

     NRBIPv4AddressList.initialize;
     NRBIPv6AddressList.initialize;

     // Pointers

     PreviousBlockType     := nil;
     NextBlockType         := nil;
end;

(* ------------------------------------------- *)

procedure tBlockChainEntry.FreeSubChain;

begin
     // Go recursively to the last item

     if not (NextBlockType = nil) then
        NextBlockType.FreeSubChain;

     // dispose child items if any

     if NextBlockType <> nil then
        dispose (NextBlockType);
end;

(* ------------------------------------------- *)
{ tNGSHBParsedOptionValues }
(* ------------------------------------------- *)

procedure tNGSHBParsedOptionValues.initialize;

begin
     HeaderCommentList := tStringList.Create;
     HardwareInfoList := tStringList.Create;
     OperatingSystemList := tStringList.Create;
     UserApplicationList := tStringList.Create;

     CaptureSystemGUID := tGUID.Empty;
end;

(* ------------------------------------------- *)

procedure tNGSHBParsedOptionValues.cleanUp;

begin
     freeAndNil (HeaderCommentList);
     freeAndNil (HardwareInfoList);
     freeAndNil (OperatingSystemList);
     freeAndNil (UserApplicationList);
end;

(* ------------------------------------------- *)
{ tNGSHBParsedValues }
(* ------------------------------------------- *)

procedure tNGSHBParsedValues.initialize;

begin
     BlockType := BLOCKTYPE_RESERVED;
     BlockTotalLength := 0;
     ByteOrderMagic := 0;
     MajorVersion := 0;
     MinorVersion := 0;
     TotalSectionLength := 0;
end;

(* ------------------------------------------- *)
{ tNGNRBParsedValues }
(* ------------------------------------------- *)
{
procedure tNGNRBParsedValues.initialize;

begin
//   IPv4AddressList := tObjectList.create (false);
//   IPv6AddressList := tObjectList.create (false);

     IPv4NameList := tStringList.Create;
     IPv6NameList := tStringList.Create;
end;

(* ------------------------------------------- *)

procedure tNGNRBParsedValues.cleanUp;

begin
//   IPv4AddressList.Free;
//   IPv6AddressList.Free;
     freeAndNil (IPv4NameList);
     freeAndNil (IPv6NameList);
end;
}
(* ------------------------------------------- *)
{ tPCAPGNStatistic }
(* ------------------------------------------- *)

procedure tPCAPNGStatistic.initialize;

begin
     SectionCount          := 0;
     FilePacketCount       := 0;
     FileInterfaceCount    := 0;
     StatisticBlockCount   := 0;
     StatPacketCount       := 0;
     StatDropCount         := 0;
     StatFilterAcceptCount := 0;
     StatOsDropCount       := 0;
     StatUsrDeliveredCount := 0;

     ErrorCode := 0;
     ErrorText := 'Success';

     FirstFrameTime        := 0;
     LastFrameTime         := 0;
end;

(* ------------------------------------------- *)
{ tNGInterfaceResolutionList }
(* ------------------------------------------- *)

procedure tNGInterfaceResolutionList.initialize;

var i : integer;

begin
     for i := 0 to 255 do
         items[i] := 6;            // Microsecond resolution is the default

     count := -1;
end;

(* ------------------------------------------- *)
{ tNGNRBParsedValues }
(* ------------------------------------------- *)

procedure tNGNRBParsedValues.initialize;

begin
     IPv4AddressList.initialize;
     IPv6AddressList.initialize;
end;

(* ------------------------------------------- *)

end.
