{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_parser_802dot1q;

interface

uses base_structures,
     common_parse_helpers,
     class_parser_baseclass;


type t802dot1qParser  = class (tFrameParser)

                              private
                                     priv_priority    : byte;
                                     priv_cfi         : boolean;
                                     priv_vlanid      : integer;
                                     priv_ethertype   : word;

                              public
                                    property Priority : byte read priv_priority;
                                    property CFI : boolean read priv_cfi;
                                    property VLANID : integer read priv_vlanid;
                                    property EtherType : word read priv_ethertype;

                                    function checkIsPlausible : boolean; override;
                                    function parse : tParseResult; override;
                        end;


implementation

uses Toolkit;

(* ------------------------------------------- *)
{ t802dot1qParser }
(* ------------------------------------------- *)

function t802dot1qParser.checkIsPlausible: boolean;

begin
     result := true;              // good enough for now
end;

(* ------------------------------------------- *)

function t802dot1qParser.parse: tParseResult;

var pData  : pWord;
    status : tParseResult;
    help   : word;
    data   : t802dot1qData;

begin
     priv_osilayer := 2;     // Set Layer this parser is for
     prot_layersize := 4;

     status := prSuccess;

     if priv_framelength < priv_frameoffset + 4 then
        status := prFailUndersize
     else
         try
            pData := pWord (priv_framedata + priv_frameoffset);
            help := pData^;

            help := swap16 (help);

            data := Parse802dot1qData (help);

            priv_priority := data.priority;
            priv_cfi := data.cfi;
            priv_vlanid := data.vlanid;

            // Get next anchor

            pData := pWord (priv_framedata + priv_frameoffset + 2);

            help := pData^;
            priv_ethertype := swap16 (help);

            priv_nextAnchor.HeaderType := priv_ethertype;
            priv_nextAnchor.Offset := priv_frameoffset + 4;

         except
               status := prFailInvalidValues;
         end;

     result := status;
end;

(* ------------------------------------------- *)

end.
