{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_parser_dns;

interface

uses classes,
     base_structures,
     definition_ip_addresses,
     common_parse_helpers,
     class_parser_baseclass;


type tDNSQueryRecord = record
                             Name                  : tFQDN;
                             RecordType            : tDNSRecordType;
                             RecordClass           : tDNSRecordClass;
                             ParseStatus           : integer;

                             procedure initialize;
                       end;

     tDNSQueryRecordSet = array of tDNSQueryRecord;

     tDNSAnswerRecord = record
                              Name                 : tFQDN;
                              RecordType           : tDNSRecordType;
                              RecordClass          : tDNSRecordClass;
                              TTL                  : longword;
                              Length               : SmallInt;
                              Preference           : word;
                              Answer               : tNameResolutionAnswer;
                              Txt                  : tStringList;
                              NameServer           : tFQDN;
                              Mailbox              : tFQDN;

                              SerialNumber         : longword;
                              RefreshInterval      : longword;
                              RetryInterval        : longword;
                              ExpirationLimit      : longword;
                              MinimumTTL           : longword;
                              ServicePriority      : word;
                              ServiceWeight        : word;
                              ServicePort          : word;
                              ServiceTarget        : tFQDN;
                              ParseStatus          : tParseResult;

                              procedure initialize;
                              procedure Free;
                        end;

     tDNSAnswerRecordSet = array of tDNSAnswerRecord;
{
     tFQDNResult = record
                         Name                      : string;
                         OffsetIncrement           : integer;
                   end;
}
     tDNSParser   = class (tFrameParser)

                          constructor create (FrameData : pointer; const FrameOffset, DataLength, WireLength, DNSSectionLength : integer; const FrameIndex : integer = 0); reintroduce;
                          procedure free;

                          private
                                 priv_sectionlen             : integer;
                                 priv_lastoffset             : integer;

                                 priv_transactionID          : word;

                                 priv_isResponse             : boolean;
                                 priv_opcode                 : tDNSOpCode;
                                 priv_isAuthoritative        : boolean;
                                 priv_isTruncated            : boolean;
                                 priv_RecursionDesired       : boolean;
                                 priv_RecursionAvailable     : boolean;
                                 priv_Z                      : boolean;
                                 priv_AnswerAuthenticated    : boolean;
                                 priv_AcceptNonAuthenticated : boolean;
                                 priv_ReplyCode              : tDNSReturnCode;

                                 priv_querycount             : word;
                                 priv_answercount            : word;
                                 priv_authoritycount         : word;
                                 priv_additionalinfocount    : word;

                                 priv_zonecount              : word;
                                 priv_prerequesitescount     : word;
                                 priv_updatescount           : word;

                                 // Normal DNS Stuff

                                 priv_QueryRecords           : tDNSQueryRecordSet;
                                 priv_AnswerRecords          : tDNSAnswerRecordSet;
                                 priv_NSRecords              : tDNSAnswerRecordSet;
                                 priv_AdditionalRecords      : tDNSAnswerRecordSet;

                                 // DDNS Stuff

                                 priv_ZoneRecords            : tDNSQueryRecordSet;
                                 priv_prerequisitesRecords   : tDNSAnswerRecordSet;
                                 priv_UpdateRecords          : tDNSAnswerRecordSet;

                                 // Settings stuff

                                 priv_SkipUnknownRecordTypeDiag : boolean;

                                 procedure DiscardAnswerRecords;
                                 procedure InitRecordCounters;

                                 function getOpCodeFromValue (const OpCode : word) : tDNSOpCode;
                                 function getDNSRecordTypeFromValue (const TypeValue : word) : tDNSRecordType;
                                 function getDNSRecordTypeAsText (const TypeValue : tDNSRecordType) : string;
                                 function getDNSRecordClassFromValue (const ClassValue : word) : tDNSRecordClass;
                                 procedure DissectDNSHeaderFlags (const FlagValue : word);
                                 function parseQueryRecordFromOffset (var Offset : integer) : tDNSQueryRecord;
                                 function parseAnswerRecordFromOffset (var Offset : integer) : tDNSAnswerRecord;
                                 function parseRessouceRecords (Offset : integer) : tParseResult;

                          public
                                property TransactionID : word read priv_transactionID;
                                property isResponse : boolean read priv_isResponse;
                                property OpCode : tDNSOpCode read priv_opcode;
                                property IsAuthoritative : boolean read priv_isAuthoritative;
                                property RecursionDesired : boolean read priv_RecursionDesired;
                                property RecursionAvailable : boolean read priv_RecursionAvailable;
                                property ZFlag : boolean read priv_Z;
                                property AnswerAuthenticated : boolean read priv_AnswerAuthenticated;
                                property ReplyCode : tDNSReturnCode read priv_ReplyCode;
                                property QueryCount : word read priv_querycount;
                                property AnswerCount : word read priv_answercount;
                                property AuthorityCount : word read priv_authoritycount;
                                property AdditionalInfoCount : word read priv_additionalinfocount;

                                property QueryRecordSet : tDNSQueryRecordSet read priv_QueryRecords;
                                property AnswerRecordSet : tDNSAnswerRecordSet read priv_AnswerRecords;
                                property NSRecordSet : tDNSAnswerRecordSet read priv_NSRecords;
                                property AdditionalRecordSet : tDNSAnswerRecordSet read priv_AdditionalRecords;
                                property ZoneRecordSet : tDNSQueryRecordSet read priv_ZoneRecords;
                                property PrerequisitesRecordSet : tDNSAnswerRecordSet read priv_prerequisitesRecords;
                                property UpdateRecordSet : tDNSAnswerRecordSet read priv_UpdateRecords;

                                property SkipWarningAboutUnknownRecTypes : boolean read priv_SkipUnknownRecordTypeDiag write priv_SkipUnknownRecordTypeDiag;

                                function checkIsPlausible : boolean; override;
                                function parse : tParseResult; override;
                    end;


implementation

uses SiAuto,
     base_messages,
     SysUtils,
     toolkit;

type pDNSStruct = ^tDNSStruct;
     tDNSStruct = packed record
                               TransactionID         : word;
                               Flags                 : word;
                               QueryCount            : word;
                               AnswerCount           : word;
                               AuthorityCount        : word;
                               AdditionalInfoCount   : word;
                          end;

// const MaxPointerLoopCount = 16;

(* ------------------------------------------- *)
{ tDNSQueryRecord }
(* ------------------------------------------- *)

procedure tDNSQueryRecord.initialize;

begin
     Name.initialize;
     RecordType := dnsrtUnknown;
     RecordClass := dnsrcUnknown;
     ParseStatus := ParseStatusNoError;
end;

(* ------------------------------------------- *)
{ tDNSAnswerRecord }
(* ------------------------------------------- *)

procedure tDNSAnswerRecord.initialize;

begin
     Preference := 0;
{
     IPVersion := 0;
     IPv4Address.ValueAsLong := 0;
     IPv4Address.MaskBitCount := 32;
     IPv6Address.SetValueFromByteStruct (IPv6_NODE_UNSPECIFIED_BYTES);
     IPv6Address.MaskBitCount := 64;
     NameInfo.initialize;
}
     Txt := tStringList.Create;
     NameServer.initialize;
     Mailbox.initialize;

     Answer.initialize;
     SerialNumber := 0;
     RefreshInterval := 0;
     RetryInterval := 0;
     ExpirationLimit := 0;
     MinimumTTL := 0;
     ServiceTarget.initialize;

     ParseStatus := prNotParsed;
end;

(* ------------------------------------------- *)

procedure tDNSAnswerRecord.Free;

begin
     FreeAndNil (Txt);

     ParseStatus := prNotParsed;
end;

(* ------------------------------------------- *)
{ DNSParser }
(* ------------------------------------------- *)

constructor tDNSParser.create (FrameData: pointer; const FrameOffset, DataLength, WireLength, DNSSectionLength, FrameIndex: integer);

begin
     inherited create (FrameData, FrameOffset, DataLength, WireLength, FrameIndex);

     priv_sectionlen := DNSSectionLength;

     // calculate the end offset we can parse up to

     if (priv_frameoffset + priv_sectionlen) > priv_framelength  then
        priv_lastoffset := priv_framelength
     else
         priv_lastoffset := priv_frameoffset + priv_sectionlen - 1;
end;

(* ------------------------------------------- *)

procedure tDNSParser.free;

begin
     DiscardAnswerRecords;

     inherited;
end;

(* ------------------------------------------- *)

function tDNSParser.checkIsPlausible: boolean;

var help   : word;
    pHelp  : pWord;
    tcplen : word;
    pData  : pDNSStruct;
    boOkay : boolean;

begin
     boOkay := true;

     if priv_parentAnchor.HeaderType = NEXT_HEADER_TCP then
        begin
             boOkay := false;          // TODO: once we can to TCP reassembly, parse DNS over TCP

             // We don't do TCP yet because for parsing pointers (and all the rest) we need the full payload, and skip over the length bytes
{
             pHelp := pWord (priv_framedata + priv_frameoffset);
             help := pHelp^;
             tcplen := Swap16 (help);

             // Check if the length is at least that of the DNS header

             boOkay := (tcplen >= sizeOf (tDNSStruct));

             if boOkay then
                pData := pDNSStruct (priv_framedata + priv_frameoffset + 2);
}
        end
     else
         begin
              tcplen := 0;
              pData := pDNSStruct (priv_framedata + priv_frameoffset);
         end;

     if boOkay then
        begin
             // Flags

             help := pData^.Flags;
             DissectDNSHeaderFlags (swap16 (help));

             if boOkay then
                boOkay := not (priv_opcode = dnsocUnknown);
        end;

     result := boOkay;
end;

(* ------------------------------------------- *)

procedure tDNSParser.DiscardAnswerRecords;

var i    : integer;
    help : tStringList;

begin
     for i := 1 to length (priv_AnswerRecords) do
         begin
              help := priv_AnswerRecords[i-1].Txt;
              help.Free;
         end;

     for i := 1 to length (priv_NSRecords) do
         begin
              help := priv_NSRecords[i-1].Txt;
              help.Free;
         end;

     for i := 1 to length (priv_AdditionalRecords) do
         begin
              help := priv_AdditionalRecords[i-1].Txt;
              help.Free;
         end;

     for i := 1 to length (priv_UpdateRecords) do
         begin
              help := priv_UpdateRecords[i-1].Txt;
              help.Free;
         end;

     for i := 1 to length (priv_prerequisitesRecords) do
         begin
              help := priv_prerequisitesRecords[i-1].Txt;
              help.Free;
         end;
end;

(* ------------------------------------------- *)

procedure tDNSParser.InitRecordCounters;

begin
     priv_querycount := 0;
     priv_answercount := 0;
     priv_authoritycount := 0;
     priv_additionalinfocount := 0;

     priv_zonecount := 0;
     priv_prerequesitescount := 0;
     priv_updatescount := 0;
end;

(* ------------------------------------------- *)

function tDNSParser.getOpCodeFromValue (const OpCode: word) : tDNSOpCode;

begin
     case OpCode of
      0 : result := dnsocQuery;
      1 : result := dnsocInverseQuery;
      2 : result := dnsocStatus;
      3 : result := dnsocAvailable;
      4 : result := dnsocNotify;
      5 : result := dnsocUpdate;
      6..15 : result := dnsocUnknown;
     end;
end;

(* ------------------------------------------- *)
{
function tDNSParser.getReplyCodeFromValue (const ReplyCode: word): tDNSReturnCode;

begin
     case ReplyCode of
      0 : result := dnsrcNoError;
      1 : result := dnsrcFormErr;
      2 : result := dnsrcServFail;
      3 : result := dnsrcNXDomain;
      4 : result := dnsrcNotImp;
      5 : result := dnsrcRefused;
      6 : result := dnsrcYXDomain;
      7 : result := dnsrcYXRRSet;
      8 : result := dnsrcNXRRSet;
      9 : result := dnsrcNotAuth;
      10 : result := dnsrcNotZone;
      11..15 : result := dnsrcAvailable;
      16 : result := dnsrcBADSIG;
      17 : result := dnsrcBADKEY;
      18 : result := dnsrcBADTIME;
      19 : result := dnsrcBADMODE;
      20 : result := dnsrcBADNAME;
      21 : result := dnsrcBADALG;
      22..3840 : result := dnsrcAvailable;
      3851..4095 : result := dnsRCPrivateUse;
      4096..65535 : result := dnsrcAvailable;
     end;
end;
}
(* ------------------------------------------- *)

function tDNSParser.getDNSRecordTypeFromValue (const TypeValue : word) : tDNSRecordType;

begin
     case TypeValue of
       1 : result := dnsrtA;
       2 : result := dnsrtNS;
       5 : result := dnsrtCNAME;
       6 : result := dnsrtSOA;
      12 : result := dnsrtPTR;
      15 : result := dnsrtMX;
      16 : result := dnsrtTXT;
      18 : result := dnsrtAFSDB;
      24 : result := dnsrtSIG;
      25 : result := dnsrtKEY;
      28 : result := dnsrtAAAA;
      29 : result := dnsrtLOC;
      33 : result := dnsrtSRV;
      35 : result := dnsrtNAPTR;
      36 : result := dnsrtKX;
      37 : result := dnsrtCERT;
      39 : result := dnsrtDNAME;
      41 : result := dnsrtOPT;
      42 : result := dnsrtAPL;
      43 : result := dnsrtDS;
      44 : result := dnsrtSSHFP;
      45 : result := dnsrtIPSECKEY;
      46 : result := dnsrtRRSIG;
      47 : result := dnsrtNSEC;
      48 : result := dnsrtDNSKEY;
      49 : result := dnsrtDHCID;
      50 : result := dnsrtNSEC3;
      51 : result := dnsrtNSEC3PARAM;
      52 : result := dnsrtTLSA;
      55 : result := dnsrtHIP;
      99 : result := dnsrtSPF;
      249 : result := dnsrtTKEY;
      250 : result := dnsrtTSIG;
      251 : result := dnsrtIXFR;
      252 : result := dnsrtAFXR;
      255 : result := dnsrtAny;
      32769 : result := dnsrtDLV;
     else
         result := dnsrtUnknown;
     end;
end;

(* ------------------------------------------- *)

function tDNSParser.getDNSRecordTypeAsText (const TypeValue : tDNSRecordType) : string;

begin
     case TypeValue of
      dnsrtA          : result := 'A';
      dnsrtNS         : result := 'NS';
      dnsrtCNAME      : result := 'CNAME';
      dnsrtSOA        : result := 'SOA';
      dnsrtPTR        : result := 'PTR';
      dnsrtMX         : result := 'MX';
      dnsrtTXT        : result := 'TXT';
      dnsrtAFSDB      : result := 'AFSDB';
      dnsrtSIG        : result := 'SIG';
      dnsrtKEY        : result := 'KEY';
      dnsrtAAAA       : result := 'AAAA';
      dnsrtLOC        : result := 'LOC';
      dnsrtSRV        : result := 'SRV';
      dnsrtNAPTR      : result := 'NAPTR';
      dnsrtKX         : result := 'KX';
      dnsrtCERT       : result := 'CERT';
      dnsrtDNAME      : result := 'DNAME';
      dnsrtOPT        : result := 'OPT';
      dnsrtAPL        : result := 'APL';
      dnsrtDS         : result := 'DS';
      dnsrtSSHFP      : result := 'SSHFP';
      dnsrtIPSECKEY   : result := 'IPSECKEY';
      dnsrtRRSIG      : result := 'RRSIG';
      dnsrtNSEC       : result := 'NSEC';
      dnsrtDNSKEY     : result := 'DNSKEY';
      dnsrtDHCID      : result := 'DHCID';
      dnsrtNSEC3      : result := 'NSEC3';
      dnsrtNSEC3PARAM : result := 'NSEC3PARAM';
      dnsrtTLSA       : result := 'TLSA';
      dnsrtHIP        : result := 'HIP';
      dnsrtSPF        : result := 'SPF';
      dnsrtTKEY       : result := 'TKEY';
      dnsrtTSIG       : result := 'TSIG';
      dnsrtIXFR       : result := 'IXFR';
      dnsrtAFXR       : result := 'AFXR';
      dnsrtAny        : result := 'Any';
      dnsrtDLV        : result := 'DLV';
      dnsrtUnknown    : result := 'Unknown';
     else
         result := 'Unknown';
     end;
end;

(* ------------------------------------------- *)

function tDNSParser.getDNSRecordClassFromValue (const ClassValue : word) : tDNSRecordClass;

begin
     case ClassValue of
        1 : result := dnsrcInternet;
        3 : result := dnsrcChaos;
      254 : result := dnssrcNONE;
      255 : result := dnssrcANY;
     else
         result := dnsrcUnknown;
     end;
end;

(* ------------------------------------------- *)

procedure tDNSParser.DissectDNSHeaderFlags (const FlagValue: word);

var help : word;

begin
{
     Flags: 0x8403 Standard query response, No such name
        1... .... .... .... = Response: Message is a response
        .000 0... .... .... = Opcode: Standard query (0)
        .... .1.. .... .... = Authoritative: Server is an authority for domain
        .... ..0. .... .... = Truncated: Message is not truncated
        .... ...0 .... .... = Recursion desired: Don't do query recursively
        .... .... 0... .... = Recursion available: Server can't do recursive queries
        .... .... .0.. .... = Z: reserved (0)
        .... .... ..0. .... = Answer authenticated: Answer/authority portion was not authenticated by the server
        .... .... ...0 .... = Non-authenticated data: Unacceptable
        .... .... .... 0011 = Reply code: No such name (3)
}
     priv_isResponse := (FlagValue and $8000) = $8000;

     help := (FlagValue and $7800);
     help := help shr 11;
     priv_opcode := getOpCodeFromValue (help);

     priv_isAuthoritative := (FlagValue and $400) = $400;
     priv_isTruncated := (FlagValue and $200) = $200;
     priv_RecursionDesired := (FlagValue and $100) = $100;
     priv_RecursionAvailable := (FlagValue and $80) = $80;
     priv_Z := (FlagValue and $40) = $40;
     priv_AnswerAuthenticated := (FlagValue and $20) = $20;
     priv_AcceptNonAuthenticated := (FlagValue and $10) = $10;

     help := (FlagValue and $f);

     priv_ReplyCode := DNSReplyCodeFromWord (help);
end;

(* ------------------------------------------- *)

function tDNSParser.parseQueryRecordFromOffset (var Offset: integer): tDNSQueryRecord;

var QueryRecord  : tDNSQueryRecord;
    RecordLength : pByte;
    NewOffset    : pByte;
    pHelpWord    : pWord;
    HelpWord     : word;
    ResultSet    : tFQDNResult;

begin
     QueryRecord.initialize;

     ResultSet := readFQDNFromOffset (priv_framedata, priv_sectionlen, priv_frameoffset, Offset);

     if ResultSet.ParseResult = prSuccess then
        begin
             QueryRecord.Name := ResultSet.Name;
             inc (Offset, ResultSet.OffsetIncrement);

             // Record Type

             if (Offset + 2) <= priv_lastOffset then
                begin
                     pHelpWord := pWord (priv_framedata + Offset);
                     HelpWord := pHelpWord^;
                     QueryRecord.RecordType := getDNSRecordTypeFromValue (swap16 (HelpWord));

                     inc (Offset, 2);

                     if (Offset + 1) <= priv_lastOffset then
                        begin
                             // Record Class

                             HelpWord := pWord (priv_framedata + Offset)^;
                             QueryRecord.RecordClass := getDNSRecordClassFromValue (swap16 (HelpWord));

                             inc (Offset, 2);
                        end
                     else
                         QueryRecord.ParseStatus := FQDNParseErrorParsingOutOfBounds;
                end
             else
                 QueryRecord.ParseStatus := FQDNParseErrorParsingOutOfBounds;
        end
     else
         begin
              QueryRecord.ParseStatus := FQDNParseErrorParsingOutOfBounds;

              addDiagnosticMsg (ResultSet.Msg);
         end;

     result := QueryRecord;
end;

(* ------------------------------------------- *)

function tDNSParser.parseAnswerRecordFromOffset (var Offset: integer) : tDNSAnswerRecord;

var AnswerRecord : tDNSAnswerRecord;
    RecordLength : pByte;
    ResultSet    : tFQDNResult;
    NewOffset    : pByte;
    pHelpByte    : pByte;
    pHelpWord    : pWord;
    pHelpLong    : pLongword;
    pHelpInt     : pSmallInt;
    HelpByte     : Byte;
    HelpWord     : word;
    HelpLong     : longword;
    HelpSmallInt : SmallInt;
    HelpChar     : pANSIChar;
    HelpString   : string;
    HelpAddress  : tIPAddress;
    pHelpIPv6Add : pIPv6AddressByteStruct;
    TotalTxtLen  : integer;

begin
     // Init Answer fields

     AnswerRecord.initialize;

     ResultSet := readFQDNFromOffset (priv_framedata, priv_sectionlen, priv_frameoffset, Offset);
     AnswerRecord.ParseStatus := ResultSet.ParseResult;

     if ResultSet.ParseResult = prSuccess then
        begin
             AnswerRecord.Name := ResultSet.Name;
             inc (Offset, ResultSet.OffsetIncrement);

             // Record Type

             pHelpWord := pWord (priv_framedata + Offset);
             HelpWord := pHelpWord^;
             HelpWord := Swap16 (HelpWord);
             AnswerRecord.RecordType := getDNSRecordTypeFromValue (HelpWord);

             inc (Offset, 2);

             // Record Class

             pHelpWord := pWord (priv_framedata + Offset);
             HelpWord := pHelpWord^;
             AnswerRecord.RecordClass := getDNSRecordClassFromValue (swap16 (HelpWord));

             inc (Offset, 2);

             // Time to Live

             pHelpLong := pLongword (priv_framedata + Offset);
             HelpLong := pHelpLong^;
             AnswerRecord.TTL := Swap32 (HelpLong);

             inc (Offset, 4);

             // Data Length

             pHelpInt := pSmallInt (priv_framedata + Offset);
             HelpSmallInt := pHelpInt^;
             AnswerRecord.Length := Swap16 (HelpSmallInt);

             inc (Offset, 2);

             // Check Data Length

             if Offset + AnswerRecord.Length > priv_lastOffset + 1 then
                begin
                     SiMain.LogFatal ('Answer length exceeds payload size for frame #' + intToStr (priv_frameindex) + ': ' + IntToStr (Offset) + ' (DNSParser.parseAnswerRecordFromOffset)');
                     addDiagnosticMsg (msCritical, MSG_DNS_ANSWER_PARSE_ERROR, (Offset + AnswerRecord.length).ToString, priv_lastOffset.ToString);

        //           raise EParsingException.Create (MSG_DNS_ANSWER_PARSE_ERROR, 'Answer length exceeds payload size for frame #' + intToStr (priv_frameindex) + ' (DNSParser.parseAnswerRecordFromOffset)', priv_frameindex);
                end;

             if AnswerRecord.Length > 0 then
                begin
                     // Parse answer based on record type

                     case AnswerRecord.RecordType of
                      dnsrtA      : begin
                                         if AnswerRecord.Length <> 4 then
                                            addDiagnosticMsg (msWarning, MSG_DNS_BROKEN_IPV4_RECORD);

                                         // Check if we have enough bytes left and parse the IP if we do, no matter what

                                         if (Offset + 4) <= priv_lastOffset then
                                            begin
                                                 // IPv4 address record

                                                 pHelpLong := pLongword (priv_framedata + Offset);
                                                 HelpLong := pHelpLong^;
                                                 HelpAddress.initialize (Swap32 (HelpLong));
                                                 AnswerRecord.Answer.initialize (HelpAddress);
                                            end;

                                         inc (Offset, 4);
                                    end;
                       dnsrtNS    : begin
                                         ResultSet := readFQDNFromOffset (priv_framedata, priv_sectionlen, priv_frameoffset, Offset);

                                         if ResultSet.ParseResult = prSuccess then
                                            AnswerRecord.Answer.initialize (ResultSet.Name)
                                         else
                                             AddDiagnosticMsg (ResultSet.Msg);

                                         inc (Offset, ResultSet.OffsetIncrement);
                                    end;
                       dnsrtSOA   : begin
                                         // Primary Name Server

                                         ResultSet := readFQDNFromOffset (priv_framedata, priv_sectionlen, priv_frameoffset, Offset);

                                         // TODO: handle sliced frames, which lead to empty result sets. Deal with it

                                         if ResultSet.ParseResult = prSuccess then
                                            begin
                                                 AnswerRecord.NameServer := ResultSet.Name;
                                                 AnswerRecord.Answer.initialize (ResultSet.Name);
                                            end
                                         else
                                             AddDiagnosticMsg (ResultSet.Msg);

                                         inc (Offset, ResultSet.OffsetIncrement);

                                         // Responsible authorities mailbox

                                         ResultSet := readFQDNFromOffset (priv_framedata, priv_sectionlen, priv_frameoffset, Offset);

                                         if ResultSet.ParseResult = prSuccess then
                                            AnswerRecord.Mailbox := ResultSet.Name
                                         else
                                             AddDiagnosticMsg (ResultSet.Msg);

                                         inc (Offset, ResultSet.OffsetIncrement);

                                         // Serial Number

                                         pHelpLong := pLongword (priv_framedata + Offset);
                                         HelpLong := pHelpLong^;
                                         AnswerRecord.SerialNumber := Swap32 (HelpLong);
                                         inc (Offset, 4);

                                         // Refresh Interval

                                         pHelpLong := pLongword (priv_framedata + Offset);
                                         HelpLong := pHelpLong^;
                                         AnswerRecord.RefreshInterval := Swap32 (HelpLong);
                                         inc (Offset, 4);

                                         // Retry Interval

                                         pHelpLong := pLongword (priv_framedata + Offset);
                                         HelpLong := pHelpLong^;
                                         AnswerRecord.RetryInterval := Swap32 (HelpLong);
                                         inc (Offset, 4);

                                         // Expiration Limit

                                         pHelpLong := pLongword (priv_framedata + Offset);
                                         HelpLong := pHelpLong^;
                                         AnswerRecord.ExpirationLimit := Swap32 (HelpLong);
                                         inc (Offset, 4);

                                         // Minimum TTL

                                         pHelpLong := pLongword (priv_framedata + Offset);
                                         HelpLong := pHelpLong^;
                                         AnswerRecord.MinimumTTL := Swap32 (HelpLong);
                                         inc (Offset, 4);
                                    end;
                       dnsrtMX    : begin
                                         // MX Preference

                                         pHelpWord := pWord (priv_framedata + Offset);
                                         HelpWord := pHelpWord^;
                                         AnswerRecord.Preference := Swap16 (HelpWord);

                                         inc (Offset, 2);

                                         // FQDN

                                         ResultSet := readFQDNFromOffset (priv_framedata, priv_sectionlen, priv_frameoffset, Offset);

                                         if ResultSet.ParseResult = prSuccess then
                                            AnswerRecord.Answer.initialize (ResultSet.Name)
                                         else
                                             AddDiagnosticMsg (ResultSet.Msg);

                                         inc (Offset, ResultSet.OffsetIncrement);
                                    end;
                       dnsrtTXT   : begin
                                         // Despite what RFC 1035 says in section 3.3.14 TXT records seem to have a length byte before the actual text starts, and can contain multiple text values

                                         TotalTxtLen := 0;

                                         while TotalTxtLen < AnswerRecord.Length do
                                               begin
                                                    pHelpByte := pByte (priv_framedata + Offset);
                                                    HelpByte := pHelpByte^;
                                                    inc (Offset);

                                                    if TotalTxtLen + HelpByte > AnswerRecord.Length then
                                                       begin
                                                            addDiagnosticMsg (msCritical, MSG_DNS_TXT_RECORD_LENGTH_INVALID);
        //                                                  raise EParsingException.Create (MSG_DNS_TXT_RECORD_LENGTH_INVALID, 'TXT Record Length failure for frame #' + intToStr (priv_frameindex) + ' (DNSParser.parseAnswerRecordFromOffset)', priv_frameindex);
                                                       end;

                                                    // Read the TXT value

                                                    HelpChar := pANSIChar (priv_framedata + Offset);
                                                    HelpString := string (HelpChar);
                                                    HelpString := copy (HelpString, 1, HelpByte);

                                                    AnswerRecord.Txt.Add (HelpString);
                                                    inc (Offset, HelpByte);

                                                    TotalTxtLen := TotalTxtLen + HelpByte + 1;      // +1 for the length byte
                                               end;
                                    end;
                       dnsrtCNAME,
                       dnsrtPTR   : begin
                                         // we need to store a string

                                         ResultSet := readFQDNFromOffset (priv_framedata, priv_sectionlen, priv_frameoffset, Offset);

                                         if ResultSet.ParseResult = prSuccess then
                                            AnswerRecord.Answer.initialize (ResultSet.Name)
                                         else
                                             AddDiagnosticMsg (ResultSet.Msg);

                                         inc (Offset, ResultSet.OffsetIncrement);
                                    end;
                       dnsrtSRV   : begin
                                         // Service Priority

                                         pHelpWord := pWord (priv_framedata + Offset);
                                         HelpWord := pHelpWord^;
                                         AnswerRecord.ServicePriority := Swap16 (HelpWord);

                                         inc (Offset, 2);

                                         // Service Weight

                                         pHelpWord := pWord (priv_framedata + Offset);
                                         HelpWord := pHelpWord^;
                                         AnswerRecord.ServiceWeight := Swap16 (HelpWord);

                                         inc (Offset, 2);

                                         // Service Port

                                         pHelpWord := pWord (priv_framedata + Offset);
                                         HelpWord := pHelpWord^;
                                         AnswerRecord.ServicePort := Swap16 (HelpWord);

                                         inc (Offset, 2);

                                         // Service Target

                                         ResultSet := readFQDNFromOffset (priv_framedata, priv_sectionlen, priv_frameoffset, Offset);

                                         if ResultSet.ParseResult = prSuccess then
                                            AnswerRecord.ServiceTarget := ResultSet.Name
                                         else
                                             AddDiagnosticMsg (ResultSet.Msg);

                                         inc (Offset, ResultSet.OffsetIncrement);
                                    end;
                       dnsrtAAAA  : begin
                                         if AnswerRecord.Length <> 16 then
                                            addDiagnosticMsg (msWarning, MSG_DNS_BROKEN_IPV6_RECORD);

                                         // Check if we have enough bytes left and parse the IP if we do, no matter what

                                         if (Offset + 16) <= priv_lastoffset then
                                            begin
                                                 // IPv6 address record

        //                                       AnswerRecord.IPVersion := 6;

                                                 pHelpIPv6Add := pIPv6AddressByteStruct (priv_framedata + Offset);
        //                                       AnswerRecord.IPv6Address.SetValueFromByteStruct (pHelpIPv6Add^);
                                                 HelpAddress.initialize (pHelpIPv6Add^);
                                                 AnswerRecord.Answer.initialize (HelpAddress);
                                            end;

                                         inc (Offset, 16);
                                    end;
                       dnsrtIXFR,
                       dnsrtAFXR,
                       dnsrtOPT,
                       dnsrtNSEC3,
                       dnsrtRRSIG,
                       dnsrtSPF,
                       dnsrtUnknown : begin
                                           // we skip these suckers for now

                                           inc (Offset, AnswerRecord.Length);

                                           if not priv_SkipUnknownRecordTypeDiag then
                                              addDiagnosticMsg (msWarning, MSG_DNS_RECORD_TYPE_SKIPPED, QueryTypeToString (AnswerRecord.RecordType));
                                      end;
                      else
                          begin
                               // we skip these suckers for now

                               inc (Offset, AnswerRecord.Length);

                               if not priv_SkipUnknownRecordTypeDiag then
                                  addDiagnosticMsg (msWarning, MSG_DNS_RECORD_TYPE_SKIPPED, QueryTypeToString (AnswerRecord.RecordType));
                          end;
                     end;
                end;
        end
     else
         begin
              addDiagnosticMsg (ResultSet.Msg);

         end;

     result := AnswerRecord;
end;

(* ------------------------------------------- *)

function tDNSParser.parseRessouceRecords (Offset: integer) : tParseResult;

var i            : integer;
    QueryRecord  : tDNSQueryRecord;
    AnswerRecord : tDNSAnswerRecord;
    RecCount     : integer;
    abortParse   : boolean;

begin
     // Make sure we do not parse in uncharted waters

     if Offset > priv_lastoffset + 1 then
        begin
//           addDiagnosticMsg (msCritical, MSG_DNS_PARSE_ERROR);          not required here

//           raise EParsingException.Create (MSG_DNS_PARSE_ERROR, 'Offset out of scope of payload for frame #' + intToStr (priv_frameindex) + ' (DNSParser.parseRessouceRecords)', priv_frameindex);
             exit (prFailUndersize);
        end;

     // Parse Query records

     abortParse := false;

     for i := 1 to priv_querycount do
         begin
              if (Offset <= priv_lastoffset) then
                 begin
                      QueryRecord := parseQueryRecordFromOffset (Offset);

                      if QueryRecord.ParseStatus = ParseStatusNoError then
                         begin
                              // Store Record

                              RecCount := length (priv_QueryRecords) + 1;
                              setLength (priv_QueryRecords, RecCount);
                              priv_QueryRecords[RecCount-1] := QueryRecord;
                         end
                      else
                          begin
                               addDiagnosticMsg (msCritical, MSG_DNS_PARSE_ERROR, 'Query Records');
                               exit (prFailUndersize);
                          end;
                 end
              else
                  begin
                       addDiagnosticMsg (msCritical, MSG_DNS_PARSE_ERROR);
                       exit (prFailUndersize);
                  end;
         end;

     // Parse Answer Records

     i := 0;
     abortParse := false;

     while (i < priv_answercount) and not abortParse do
           begin
                inc (i);

                if (Offset <= priv_lastoffset) then
                   begin
                        AnswerRecord := parseAnswerRecordFromOffset (Offset);

                        if AnswerRecord.ParseStatus = prSuccess then
                           begin
                                // Store Record

                                RecCount := length (priv_AnswerRecords) + 1;
                                setLength (priv_AnswerRecords, RecCount);
                                priv_AnswerRecords[RecCount - 1] := AnswerRecord;
                           end
                        else
                            begin
                                 AnswerRecord.Free;
                                 abortParse := true;
                            end;
                   end
                else
                    begin
                         addDiagnosticMsg (msCritical, MSG_DNS_PARSE_ERROR, 'Answer Records');
                         exit (prFailUndersize);
                    end;
           end;
{
     for i := 1 to priv_answercount do
         begin
              if (Offset <= priv_lastoffset) then
                 begin
                      AnswerRecord := parseAnswerRecordFromOffset (Offset);

                      if AnswerRecord.ParseStatus = prSuccess then
                         begin
                              // Store Record

                              RecCount := length (priv_AnswerRecords) + 1;
                              setLength (priv_AnswerRecords, RecCount);
                              priv_AnswerRecords[RecCount - 1] := AnswerRecord;
                         end
                      else
                          AnswerRecord.Free;
                 end
              else
                  begin
                       addDiagnosticMsg (msCritical, MSG_DNS_PARSE_ERROR, 'Answer Records');
                       exit (prFailUndersize);
                  end;
         end;
}
     // Parse NS Records

     for i := 1 to priv_authoritycount do
         begin
              if (Offset <= priv_lastoffset) then
                 begin
                      AnswerRecord := parseAnswerRecordFromOffset (Offset);

                      if AnswerRecord.ParseStatus = prSuccess then
                         begin
                              // Store Record

                              RecCount := length (priv_NSRecords) + 1;
                              setLength (priv_NSRecords, RecCount);
                              priv_NSRecords[RecCount - 1] := AnswerRecord;
                         end
                      else
                          AnswerRecord.Free;
                 end
              else
                  begin
                       addDiagnosticMsg (msCritical, MSG_DNS_PARSE_ERROR, 'Authority Records');
                       exit (prFailUndersize);
                  end;
         end;

     // DDNS Stuff

     if priv_opcode = dnsocUpdate then
        begin
             // Parse Zone records

             for i := 1 to priv_zonecount do
                 begin
                      if (Offset <= priv_lastoffset) then
                         begin
                              QueryRecord := parseQueryRecordFromOffset (Offset);

                              if AnswerRecord.ParseStatus = prSuccess then
                                 begin
                                      // Store Record

                                      RecCount := length (priv_ZoneRecords) + 1;
                                      setLength (priv_ZoneRecords, RecCount);
                                      priv_ZoneRecords[RecCount-1] := QueryRecord;
                                 end
                              else
                                  AnswerRecord.Free;
                         end
                      else
                          begin
                               addDiagnosticMsg (msCritical, MSG_DNS_PARSE_ERROR, 'Zone Records');
                               exit (prFailUndersize);
                          end;
                 end;

             // Parse Prerequisites Records

             for i := 1 to priv_prerequesitescount do
                 begin
                      if (Offset <= priv_lastoffset) then
                         begin
                              AnswerRecord := parseAnswerRecordFromOffset (Offset);

                              if AnswerRecord.ParseStatus = prSuccess then
                                 begin
                                      // Store Record

                                      RecCount := length (priv_prerequisitesRecords) + 1;
                                      setLength (priv_prerequisitesRecords, RecCount);
                                      priv_prerequisitesRecords[RecCount - 1] := AnswerRecord;
                                 end
                              else
                                  AnswerRecord.Free;
                         end
                      else
                          begin
                               addDiagnosticMsg (msCritical, MSG_DNS_PARSE_ERROR, 'Prerequisite Records');
                               exit (prFailUndersize);
                          end;
                 end;

             // Parse Update Records

             for i := 1 to priv_updatescount do
                 begin
                      if (Offset <= priv_lastoffset) then
                         begin
                              AnswerRecord := parseAnswerRecordFromOffset (Offset);

                              if AnswerRecord.ParseStatus = prSuccess then
                                 begin
                                      // Store Record

                                      RecCount := length (priv_UpdateRecords) + 1;
                                      setLength (priv_UpdateRecords, RecCount);
                                      priv_UpdateRecords[RecCount - 1] := AnswerRecord;
                                 end
                              else
                                  AnswerRecord.Free;
                         end
                      else
                          begin
                               addDiagnosticMsg (msCritical, MSG_DNS_PARSE_ERROR, 'Update Records');
                               exit (prFailUndersize);
                          end;
                 end;
        end;

     // Parse Additional Records

     for i := 1 to priv_additionalinfocount do
         begin
              if (Offset <= priv_lastoffset) then
                 begin
                      AnswerRecord := parseAnswerRecordFromOffset (Offset);

                      if AnswerRecord.ParseStatus = prSuccess then
                         begin
                              // Store Record

                              RecCount := length (priv_AdditionalRecords) + 1;
                              setLength (priv_AdditionalRecords, RecCount);
                              priv_AdditionalRecords[RecCount - 1] := AnswerRecord;
                         end
                      else
                          AnswerRecord.Free;
                 end
              else
                  begin
                       addDiagnosticMsg (msCritical, MSG_DNS_PARSE_ERROR, 'Additional Info');
                       exit (prFailUndersize);
                  end;
         end;
end;

(* ------------------------------------------- *)

function tDNSParser.parse: tParseResult;

var pData  : pDNSStruct;
    status : tParseResult;
    tcplen : word;
    help   : word;
    pHelp  : pWord;

begin
     priv_osilayer := 5;     // Set Layer this parser is for

     status := prSuccess;

     if priv_lastoffset < priv_frameoffset + sizeOf (tDNSStruct) then
        status := prFailUndersize
     else
         begin
              if priv_parentAnchor.HeaderType = NEXT_HEADER_TCP then
                 begin
                      pHelp := pWord (priv_framedata + priv_frameoffset);
                      help := pHelp^;
                      tcplen := Swap16 (help);

                      pData := pDNSStruct (priv_framedata + priv_frameoffset + 2);
                 end
              else
                  begin
                       tcplen := 0;
                       pData := pDNSStruct (priv_framedata + priv_frameoffset);
                  end;

              // Init Counters

              InitRecordCounters;

              // TransactionID

              help := pData^.TransactionID;
              priv_transactionID := Swap16 (help);

              // Flags

              help := pData^.Flags;
              DissectDNSHeaderFlags (swap16 (help));

              if priv_opcode = dnsocUpdate then
                 begin
                      // Dynamic Update

                       // Zone Count

                       help := pData^.QueryCount;
                       priv_zonecount := Swap16 (help);

                       // Prerequisites Count

                       help := pData^.AnswerCount;
                       priv_prerequesitescount := Swap16 (help);

                       // Update Count

                       help := pData^.AuthorityCount;
                       priv_updatescount := Swap16 (help);
                 end
              else
                  begin
                       // "Normal" Record

                       // Query Count

                       help := pData^.QueryCount;
                       priv_querycount := Swap16 (help);

                       // Answer Count

                       help := pData^.AnswerCount;
                       priv_answercount := Swap16 (help);

                       // Authority Count

                       help := pData^.AuthorityCount;
                       priv_authoritycount := Swap16 (help);
                  end;

              // Additional Info Count

              help := pData^.AdditionalInfoCount;
              priv_additionalinfocount := Swap16 (help);

              // Read ressource records - TCP has a 2 byte length value that pushes the DNS structure back

              if tcplen = 0 then
                 begin
                      if priv_lastoffset > priv_frameoffset + sizeOf (tDNSStruct) then
                         status := parseRessouceRecords (priv_frameoffset + sizeOf (tDNSStruct))
                      else
                          status := prFailTruncated;
                 end
              else
                  begin
                      if priv_lastoffset > priv_frameoffset + 2 + sizeOf (tDNSStruct) then
                         status := parseRessouceRecords (priv_frameoffset + 2 + sizeOf (tDNSStruct))
                      else
                          status := prFailTruncated;
                  end;

              // Prepare next anchor

              priv_nextAnchor.HeaderType := NEXT_HEADER_NONE;
              priv_nextAnchor.Offset := priv_lastoffset + 1;
         end;

     result := status;
end;

(* ------------------------------------------- *)


end.
