{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_parser_erspan;

{

                     ERSPAN Type II header (8 octets [42:49])
       0                   1                   2                   3
       0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
      |  Ver  |          VLAN         | COS | En|T|    Session ID     |
      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
      |      Reserved         |                  Index                |
      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
}

interface

uses base_structures,
     definition_ip_addresses,
     common_parse_helpers,
     class_parser_baseclass;

type tTrunkEncapsulationType = (tetWithoutVLAN, tetISLEncapsulated, tet8021qEncapsulated, tetTagPreserved);

     tERSPANHdrStruct = packed record
                                     VersionVLAN     : word;
                                     ClassEncTruncID : word;
                                     PortIndex       : longword;
                       end;
     pERSPANHdrStruct = ^tERSPANHdrStruct;

     tERSPANParser  = class (tFrameParser)


                           private
                                  priv_version      : byte;
                                  priv_VLAN         : tVLANID;
                                  priv_ClassOfSvc   : byte;
                                  priv_EncType      : tTrunkEncapsulationType;
                                  priv_truncated    : boolean;
                                  priv_ERSPANID     : word;
                                  priv_portindex    : longword;

                           public
                                 property Version : byte read priv_version;
                                 property VLAN : tVLANID read priv_VLAN;
                                 property ClassOfService : byte read priv_ClassOfSvc;
                                 property EncapsulationType : tTrunkEncapsulationType read priv_EncType;
                                 property Truncated : boolean read priv_truncated;
                                 property ERSPANID : word read priv_ERSPANID;
                                 property PortIndex : longword read priv_portindex;

                                 function checkIsPlausible : boolean; override;
                                 function parse : tParseResult; override;
                       end;

implementation

uses Toolkit;

(* ------------------------------------------- *)
{ tGeneveParser }
(* ------------------------------------------- *)

function tERSPANParser.checkIsPlausible: boolean;

var pData   : pERSPANHdrStruct;
    value   : boolean;
    version : word;

begin
     value := true;

     if priv_framelength < priv_frameoffset + sizeOf (tERSPANHdrStruct) then
        value := false
     else
         try
            pData := pERSPANHdrStruct (priv_framedata + priv_frameoffset);

            version := swap16 (pData^.VersionVLAN);
            version := (version and $f000) shr 12;                    // highest nibble

            value := (Version = 1);                                   // Type 2

         except
               value := false;
         end;

     result := value;
end;

(* ------------------------------------------- *)

function tERSPANParser.parse: tParseResult;

var status    : tParseResult;
    pData     : pERSPANHdrStruct;
    bHelp     : byte;
    wHelp     : word;
    wHelp2    : word;

begin
     priv_osilayer := 2;     // Set Layer this parser is for
     prot_layersize := sizeOf (tERSPANHdrStruct);

     status := prSuccess;

     if priv_framelength < priv_frameoffset + sizeOf (tERSPANHdrStruct) then
        status := prFailUndersize
     else
         try
            pData := pERSPANHdrStruct (priv_framedata + priv_frameoffset);

            // Version and VLAN ID

            wHelp := swap16 (pData^.VersionVLAN);
            wHelp := (wHelp and $f000) shr 12;                    // highest nibble

            case wHelp of
             0 : priv_version := 1;
             1 : priv_version := 2;
            end;

            wHelp := swap16 (pData^.VersionVLAN);
            priv_VLAN := (wHelp and $0fff);

            // ClassOfService, EncapsulationType, Trunc Flag, ERSPAN ID

            wHelp := swap16 (pData^.ClassEncTruncID);

            wHelp2 := (wHelp and $e000) shr 13;
            priv_ClassOfSvc := wHelp2;

            wHelp2 := (wHelp and $1800) shr 11;

            case wHelp2 of
             0 : priv_EncType := tetWithoutVLAN;
             1 : priv_EncType := tetISLEncapsulated;
             2 : priv_EncType := tet8021qEncapsulated;
             3 : priv_EncType := tetTagPreserved;
            end;

            priv_truncated := (wHelp and $400) = $400;

            wHelp2 := (wHelp and $3ff);
            priv_ERSPANID := wHelp2;

            // Port Index

            priv_portindex := swap32 (pData^.PortIndex);

            // Determine next anchor

            priv_nextAnchor.HeaderType := NEXT_HEADER_ETHERNET;
            priv_nextAnchor.Offset := priv_frameoffset + sizeOf (tERSPANHdrStruct);

         except
               status := prFailInvalidValues;
         end;

     result := status;
end;

(* ------------------------------------------- *)

end.
