{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_parser_geneve;

{
   Geneve Header:

      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
      |Ver|  Opt Len  |O|C|    Rsvd.  |          Protocol Type        |
      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
      |        Virtual Network Identifier (VNI)       |    Reserved   |
      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
      |                    Variable Length Options                    |
      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
}

interface

uses base_structures,
     common_parse_helpers,
     class_parser_baseclass;

type tGenevaFlags = (gfOAMFrame, gfCriticalOptions);
     tGenevaFlagSet = set of tGenevaFlags;

     pGenevaHdrStruct = ^tGenevaHdrStruct;
     tGenevaHdrStruct = packed record
                                     VersionOptionLen : byte;
                                     Flags            : byte;
                                     ProtocolType     : word;
                                     VNIdentifier     : longword;
                        end;

     pGenevaOptStruct = ^tGenevaOptStruct;
     tGenevaOptStruct = packed record
                                     OptionClass      : word;
                                     OptionType       : byte;
                                     OptionLength     : byte;
                        end;

     tGeneveParser  = class (tFrameParser)

                            private
                                   priv_Version      : byte;
                                   priv_OptionLength : word;
                                   priv_Flags        : tGenevaFlagSet;
                                   priv_ProtocolType : word;
                                   priv_VNI          : longword;

                                   function ParseOptions (const Offset : integer) : tParseResult;

                            public
                                  property Version : byte read priv_Version;
                                  property OptionLength : word read priv_OptionLength;
                                  property Flags : tGenevaFlagSet read priv_Flags;
                                  property ProtocolType : word read priv_ProtocolType;
                                  property VirtualNetworkIndicator : longword read priv_VNI;

                                  function checkIsPlausible : boolean; override;
                                  function parse : tParseResult; override;
                        end;

implementation

uses Toolkit;

(* ------------------------------------------- *)
{ tGeneveParser }
(* ------------------------------------------- *)

function tGeneveParser.checkIsPlausible: boolean;

var value : boolean;
    pData : pGenevaHdrStruct;
    help  : byte;

begin
     value := true;

     if priv_framelength < priv_frameoffset + sizeOf (tGenevaHdrStruct) then
        value := false
     else
         begin
              pData := pGenevaHdrStruct (priv_framedata + priv_frameoffset);

              help := pData^.VersionOptionLen and $c0;

              if help > 0 then
                 value := false;
         end;

     result := value;
end;

(* ------------------------------------------- *)

function tGeneveParser.ParseOptions (const Offset: integer) : tParseResult;

begin
     result := prSuccess;




end;

(* ------------------------------------------- *)

function tGeneveParser.parse: tParseResult;

var status    : tParseResult;
    pData     : pGenevaHdrStruct;
    bHelp     : byte;
    wHelp     : word;
    lHelp     : longword;

begin
     priv_osilayer := 3;     // Set Layer this parser is for
     prot_layersize := sizeOf (tGenevaHdrStruct);

     status := prSuccess;

     if priv_framelength < priv_frameoffset + sizeOf (tGenevaHdrStruct) then
        status := prFailUndersize
     else
         try
            pData := pGenevaHdrStruct (priv_framedata + priv_frameoffset);

            // Version and Length (length as 32 bit value count)

            priv_Version := (pData^.VersionOptionLen and $c0) shr 6;
            bHelp := pData^.VersionOptionLen and $3f;

            priv_OptionLength := bHelp * 4;

            // Flags

            bHelp := pData^.Flags and $c0;

            case bHelp of
             0 : priv_Flags := [];
             1 : priv_Flags := [gfOAMFrame];
             2 : priv_Flags := [gfCriticalOptions];
             3 : priv_Flags := [gfOAMFrame, gfCriticalOptions];
            end;

            // Protocol Type

            wHelp := pData^.ProtocolType;
            priv_ProtocolType := swap16 (wHelp);

            // Virtual Network Identifier (VNI)

            lHelp := pData^.VNIdentifier and $ffffff;
            priv_VNI := swap32 (lHelp);

            if priv_OptionLength > 0 then
               status := ParseOptions (priv_frameoffset + sizeOf (tGenevaHdrStruct));

            // Determine next anchor

            priv_nextAnchor.HeaderType := priv_ProtocolType;
            priv_nextAnchor.Offset := priv_frameoffset + sizeOf (tGenevaHdrStruct) + priv_OptionLength;

         except
               status := prFailInvalidValues;
         end;

     result := status;
end;

(* ------------------------------------------- *)

end.
