{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_parser_llc;

interface

uses base_structures,
     base_checksums,
     common_parse_helpers,
     definition_ip_addresses,
     definition_tcp_structures,
     class_parser_baseclass;

type tSNAPVendorCode = array [0..2] of byte;

     pStruct_802dot3 = ^tStruct_802dot3;
     tStruct_802dot3 = packed record
                              DSAP           : byte;                  // Destination Service Access Point
                              SSAP           : byte;                  // Source Service Access Point, $AA for SNAP following the LLC
                              Control        : byte;                  // LLC Field Type
                       end;

     pStruct_SNAP = ^tStruct_SNAP;
     tStruct_SNAP = packed record
                                 VendorCode   : tSNAPVendorCode;  // Organization code
                                 ProtocolType : word;
                    end;


     tLLCParser = class (tFrameParser)

                        private
                               // 802.2 (not extracting IG Bit and CR Bit and other detail stuff here, maybe later)

                               priv_dsap            : byte;
                               priv_ssap            : byte;
                               priv_control         : byte;

                               // SNAP

                               priv_vendorcode      : tSNAPVendorCode;
                               priv_protocoltype    : word;

                               function getSNAPProtocolType (const DSAP, SSAP : byte) : tHeaderType;

                        public
                              property DSAP : byte read priv_dsap;
                              property SSAP : byte read priv_ssap;
                              property Control : byte read priv_control;
                              property VendorCode : tSNAPVendorCode read priv_vendorcode;
                              property ProtocolType : word read priv_protocoltype;

                              function checkIsPlausible : boolean; override;
                              function parse : tParseResult; override;
                  end;


implementation

uses Toolkit;

(* ------------------------------------------- *)
{ tLLCParser }
(* ------------------------------------------- *)

function tLLCParser.checkIsPlausible: boolean;

begin
     result := true;
end;

(* ------------------------------------------- *)

function tLLCParser.getSNAPProtocolType (const DSAP, SSAP : byte) : tHeaderType;

begin
     result := NEXT_HEADER_PAYLOAD;
end;

(* ------------------------------------------- *)

function tLLCParser.parse : tParseResult;

var status    : tParseResult;
    pData     : pStruct_802dot3;
    pSNAPData : pStruct_SNAP;

begin
     priv_osilayer := 2;     // Set Layer this parser is for
     prot_LayerSize := 4;

     status := prSuccess;

     if priv_framelength < (priv_frameoffset + sizeOf (tStruct_802dot3)) then
        status := prFailUndersize
     else
         try
            pData := pStruct_802dot3 (priv_framedata + priv_frameoffset);

            // 802.2

            priv_dsap := pData^.DSAP;
            priv_ssap := pData^.SSAP;
            priv_control := pData^.Control;

            if priv_dsap = $AA then
               begin
                    prot_LayerSize := 8;

                    pSNAPData := pStruct_SNAP (priv_framedata + priv_frameoffset + sizeOf (tStruct_802dot3));

                    priv_vendorcode := pSNAPData^.VendorCode;
                    priv_protocoltype := pSNAPData^.ProtocolType;
                    priv_protocoltype := Swap16 (priv_protocoltype);
{
                    if priv_vendorcode = SNAP_VENDOR_ENCAPSULATEDETHERNET then
                       priv_nextAnchor.HeaderType := priv_protocoltype
                    else
                        priv_nextAnchor.HeaderType := getSNAPFrameType;
}
                    priv_nextAnchor.HeaderType := priv_protocoltype;
                    priv_nextAnchor.Offset := priv_frameoffset + sizeOf (tStruct_802dot3) + sizeOf (tStruct_SNAP);
               end
            else
                begin
                    priv_nextAnchor.HeaderType := getSNAPProtocolType (priv_dsap, priv_ssap);
                    priv_nextAnchor.Offset := priv_frameoffset + sizeOf (tStruct_802dot3);
                end;

         except
               status := prFailInvalidValues;
         end;

     result := status;
end;

(* ------------------------------------------- *)

end.
