{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_parser_mpls;

interface

uses base_structures,
     common_parse_helpers,
     class_parser_baseclass;


type tMPLSParser  = class (tFrameParser)

                          private
                                 priv_label       : longword;
                                 priv_tos         : byte;
                                 priv_bottom      : boolean;
                                 priv_ttl         : byte;

                          public
                                property MPLSLabel : longword read priv_label;
                                property ToSExperimental : byte read priv_tos;
                                property BottomOfStack : boolean read priv_bottom;
                                property TTL : byte read priv_ttl;

                                function checkIsPlausible : boolean; override;
                                function parse : tParseResult; override;
                    end;


implementation

uses Toolkit;

(* ------------------------------------------- *)
{ tMPLSParser }
(* ------------------------------------------- *)

function tMPLSParser.checkIsPlausible: boolean;

begin
     result := true;
end;

(* ------------------------------------------- *)

function tMPLSParser.parse: tParseResult;

var pData  : pLongword;
    pProt  : pByte;
    prot   : byte;
    status : tParseResult;
    help   : longword;
    help2  : longword;
    help3  : pWord;
    etype  : word;

begin
     priv_osilayer := 3;     // Set Layer this parser is for
     prot_layersize := 4;

     status := prSuccess;

     if priv_framelength < priv_frameoffset + 4 + 1 then      // +4 for MPLS label, +20 for minimum IP header length of 1 byte to determine IP protocol type
        status := prFailUndersize
     else
         try
            // Get full shim (32 bits)

            pData := pLongWord (priv_framedata + priv_frameoffset);
            help := pData^;
            help := swap32 (help);

            // get Label

            priv_label := (help and $fffff000);
            priv_label := priv_label shr 12;

            // get Experimental ToS Bits

            help2 := (help and $e00);
            help2 := help2 shr 9;
            priv_tos := help2;

            // get Bottom of Stack flag

            priv_bottom := (help and $100) = $100;

            // get TTL

            priv_ttl := help and $ff;

            // Read 1 byte ahead to determine IP protocol type

            pProt := pByte (priv_framedata + priv_frameoffset + 4);
            prot := pProt^;

            prot := prot and $f0;
            prot := prot shr 4;

            // Get next anchor

            priv_nextAnchor.Offset := priv_frameoffset + 4;

            if not priv_bottom then
               priv_nextAnchor.HeaderType := NEXT_HEADER_MPLS_UNICAST
            else
                begin
                     case prot of
                      4 : priv_nextAnchor.HeaderType := NEXT_HEADER_IPv4;
                      6 : priv_nextAnchor.HeaderType := NEXT_HEADER_TYPE_IPV6;
                     else
                         if priv_framelength >= (priv_frameoffset + 4 + 14) then
                            begin
                                 // this may be Ethernet over MPLS. Check the ControlWord first,
                                 // if it's zero we got Ethernet

                                 help2 := pLongWord (priv_framedata + priv_frameoffset + 4)^;

                                 if help2 = 0 then
                                    begin
                                         // Looks like we do, so it's MPLS + 4 Bytes Control now

                                         priv_nextAnchor.Offset := priv_frameoffset + 8;
                                         priv_nextAnchor.HeaderType := NEXT_HEADER_ETHERNET
                                    end
                                 else
                                     begin
                                          // let's check some realistic Ethertypes

                                          help3 := pWord (priv_framedata + priv_frameoffset + 4 + 12);
                                          etype := swap16 (help3^);

                                          case etype of
                                           $0800,
                                           $0806,
                                           $8035,
                                           $8100,
                                           $86dd : priv_nextAnchor.HeaderType := NEXT_HEADER_ETHERNET;
                                          end;
                                     end;
                            end
                         else
                             priv_nextAnchor.HeaderType := NEXT_HEADER_PAYLOAD;
                     end
                end;
         except
               status := prFailInvalidValues;
         end;

     result := status;
end;

(* ------------------------------------------- *)

end.
