{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_parser_vxlan;

{
   VXLAN Header:

    0                   1                   2                   3
    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |R|R|R|R|I|R|R|R|            Reserved                           |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |                VXLAN Network Identifier (VNI) |   Reserved    |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
}

interface

uses base_structures,
     common_parse_helpers,
     class_parser_baseclass;

type tVXLANHdrStruct = packed record
                                    Flags            : byte;
                                    Reserved         : array [0..2] of byte;
                                    NetworkIndicator : longword;
                       end;
     pVXLANHdrStruct = ^tVXLANHdrStruct;

     tVXLANParser  = class (tFrameParser)


                           private
                                  priv_VXLANIDPresent   : boolean;
                                  priv_NetworkIndicator : longword;

                           public
                                 property VXLANIDPresent : boolean read priv_VXLANIDPresent;
                                 property NetworkIndicator : longword read priv_NetworkIndicator;

                                 function checkIsPlausible : boolean; override;
                                 function parse : tParseResult; override;
                       end;

implementation

uses Toolkit;

(* ------------------------------------------- *)
{ tVXLANParser }
(* ------------------------------------------- *)

function tVXLANParser.checkIsPlausible: boolean;

var pData : pVXLANHdrStruct;
    value : boolean;

begin
     value := true;

     if priv_framelength < priv_frameoffset + sizeOf (tVXLANHdrStruct) then
        value := false
     else
         try
            pData := pVXLANHdrStruct (priv_framedata + priv_frameoffset);

            value := (pData^.Flags = 8);

         except
               value := false;
         end;

     result := value;
end;

(* ------------------------------------------- *)

function tVXLANParser.parse: tParseResult;

var status    : tParseResult;
    pData     : pVXLANHdrStruct;
    bHelp     : byte;
    wHelp     : word;
    lHelp     : longword;

begin
     priv_osilayer := 2;     // Set Layer this parser is for
     prot_layersize := sizeOf (tVXLANHdrStruct);

     status := prSuccess;

     if priv_framelength < priv_frameoffset + sizeOf (tVXLANHdrStruct) then
        status := prFailUndersize
     else
         try
            pData := pVXLANHdrStruct (priv_framedata + priv_frameoffset);

            priv_VXLANIDPresent := (pData.Flags = 8);

            // Virtual Network Identifier (VNI)

            lHelp := pData^.NetworkIndicator;
            priv_NetworkIndicator := swap32 (lHelp) shr 8;

            // Determine next anchor

            priv_nextAnchor.HeaderType := NEXT_HEADER_ETHERNET;
            priv_nextAnchor.Offset := priv_frameoffset + sizeOf (tVXLANHdrStruct);

         except
               status := prFailInvalidValues;
         end;

     result := status;
end;

(* ------------------------------------------- *)

end.
