program pcapscan;

{$APPTYPE CONSOLE}

{$R *.res}

{$R *.dres}

uses
  System.SysUtils,
  {$i base\defaultincludes.inc}
  {$i base\firedacincludes.inc}
  class_dirparser in 'base\class_dirparser.pas',
  SiAuto;

const DefaultMergedDBFile = '.pcapintel.db';
      Version = '0.3';

(* ------------------------------------------- *)

function getDatabaseConnection (Filename : string) : tFDConnection;

var DiskDB : tFDConnection;

begin
     DiskDB := TFDConnection.Create(nil);
     DiskDB.DriverName := 'SQLite';
     DiskDB.Params.Values['DriverID'] := 'SQLite';
     DiskDB.Params.Values['Database'] := Filename;
     DiskDB.Params.Values['OpenMode'] := 'ReadOnly';
     DiskDB.Open;

     result := DiskDB;
end;

(* ------------------------------------------- *)

procedure mergeScans (Path : string);

var scanner         : tDirParser;
    TempFilename    : string;
    MergeFilename   : string;
    CaptureFileName : string;
    MergeIntelDBMgr : tIntelDBManager;
    TempIntelDbMgr  : tIntelDBManager;
    Database        : tFDConnection;
    mergestore      : tConversationStorage;
    tempstore       : tConversationStorage;
    FileDetails     : tFileDetails;
    generalstats    : tGeneralTraceStats;
    PCAPngStats     : tPCAPNGStatistic;
    First           : tFrameTimeStamp;
    Last            : tFrameTimeStamp;
    FileID          : integer;
    Query           : tFDQuery;
    count           : integer;
    percentage      : integer;
    help            : integer;
    i               : integer;

begin
     FindCmdLineSwitch ('d', MergeFilename);

     path := includeTrailingPathDelimiter (path);

     if MergeFilename = '' then
        MergeFilename := DefaultMergedDBFile;

     MergeFilename := Path + MergeFilename;

     scanner := tDirParser.create (path, '*.db');
     scanner.execute;

     count := 0;

     if scanner.FileCount > 0 then
        begin
             write ('Merging ' + Scanner.FileCount.ToString + ' DB files into "' + MergeFilename + '" (' + format ('%3d', [percentage]) + '%)');

             Query := tFDQuery.Create (nil);

             for i := 1 to scanner.FileCount do
                 begin
                      TempFilename := scanner.getFileName (i);

                      mergestore := tConversationStorage.create;

                      if lowercase (TempFileName) <> lowercase (MergeFilename) then
                         begin
                              inc (count);

                              Database := getDatabaseConnection (TempFilename);

                              // First step: get first file from the temporary database

                              Query.Connection := Database;
                              Query.SQL.Text := 'SELECT * FROM filelist;';
                              Query.Open;

                              CaptureFileName := Query.FieldByName ('filename').AsString;
                              FileDetails.FileSize := Query.FieldByName ('filesize').AsLargeInt;
                              FileDetails.FileModified := Query.FieldByName ('filedate').AsDateTime;
                              First.datetime := Query.FieldByName ('firstframetimestamp').AsDateTime;
                              First.nanoseconds := Query.FieldByName ('firstframenanoseconds').AsInteger;
                              Last.datetime := Query.FieldByName ('lastframetimestamp').AsDateTime;
                              Last.nanoseconds := Query.FieldByName ('lastframenanoseconds').AsInteger;

                              TempIntelDbMgr := tIntelDBManager.create (TempFilename, CaptureFilename, FileDetails.FileSize, FileDetails.FileModified, First, Last, true, false);

                              if count = 1 then
                                 begin
                                      // Create the target conversation store

                                      MergeIntelDBMgr := tIntelDBManager.create (MergeFilename, CaptureFileName, FileDetails.FileSize, FileDetails.FileModified, First, Last, true, false);
                                 end
                              else
                                  MergeIntelDBMgr.AssignTraceFile (CaptureFileName, FileDetails.FileSize, FileDetails.FileModified, First, Last);

                              generalstats := TempIntelDBMgr.GeneralStatistics;
                              MergeIntelDBMgr.GeneralStatistics := generalstats;

                              if TempIntelDbMgr.PCAPNGStructureScanned then
                                 begin
                                      PCAPngStats := TempIntelDBMgr.readPCAPngStatisticsFromDB;
                                      MergeIntelDBMgr.writePCAPngStatisticsToDB (PCAPngStats);
                                 end;

                              // Second step: transfer conversations

                              tempStore := tConversationStorage.create;
                              tempStore.LoadFirstFileConversationsFromDatabase (Database);

                              mergestore.AddConversationsFromStore (tempstore);
                              mergestore.StoreConversationsToDatabase (MergeIntelDBMgr.Database, CaptureFilename, filedetails.FileSize, filedetails.FileModified, first, last);

                              Database.Close;
                              Database.Free;

                              // Report progress

                              help := trunc (100 * count / scanner.FileCount);

                              if help > percentage then
                                 begin
                                      percentage := help;
                                      write (#8 + #8 + #8 + #8 + #8 + format ('%3d', [percentage]) + '%)');
                                 end;
                         end;
                 end;

             writeln (#8 + #8 + #8 + #8 + #8 + '100%)');

             mergestore.free;

             MergeIntelDBMgr.free;

             Query.Free;
        end
     else
         writeln ('No traceintel DB files found at "' + path + '".');
end;

(* ------------------------------------------- *)

procedure scanfile (Filename : string);

var loader       : tTraceLoader;
    frame        : tRawFrame;
    store        : tConversationStorage;
    IntelDBMgr   : tIntelDBManager;
    DBFilename   : string;
    filedetails  : tFileDetails;
    generalstats : tGeneralTraceStats;
    count        : integer;
    percentage   : integer;
    help         : integer;
    i            : integer;

begin
     write ('Scanning "' + Filename + '"' );

     filedetails.Determine (Filename);
     DBFilename := changeFileExt (Filename, '.db');

     if fileexists (DBFileName) then
        deletefile (DBFileName);

     loader := createTraceLoader (Filename);
     percentage := 0;
     count := 0;
     generalstats.initialize;

     write (' (' + format ('%3d', [percentage]) + '%)');

     if assigned (loader) then
        begin
             store := tConversationStorage.create;

             loader.MaxFramesInMem := 10000000;

             loader.openFile;
             Loader.readTraceFileheader;

             repeat
                   Loader.readBatchFromDisk;

                   for i := 1 to Loader.FramesAvailable do
                       begin
                            Frame := Loader.getRawFrame (i);
                            inc (count);

                            if count = 1 then
                               generalstats.FirstFrameTime := Frame.Timestamp
                            else
                                generalstats.LastFrameTime := Frame.Timestamp;

                            // Do frame statistics

                            AddFrameStatistics (generalstats, Frame);

                            // Do endpoint and conversation scan

                            store.AddFrame (Frame);

                            Frame.free;

                            // Report progress

                            if (count > 0) and ((count mod 1000) = 0) then
                               begin
                                    help := trunc (100 * count / Loader.FramesAvailable);

                                    if help > percentage then
                                       begin
                                            percentage := help;
                                            write (#8 + #8 + #8 + #8 + #8 + format ('%3d', [percentage]) + '%)');
                                       end;
                               end;
                       end;

             until Loader.FramesAvailable = 0;

             generalstats.FrameCount := count;

             FinalizeFrameStatistics (generalstats);

             writeln (#8 + #8 + #8 + #8 + #8 + '100%)');

             IntelDBMgr := tIntelDBManager.create (DBFilename, Filename, FileDetails.FileSize, FileDetails.FileModified, generalstats.FirstFrameTime, generalstats.LastFrameTime, true, false);
             IntelDBMgr.GeneralStatistics := generalstats;

             store.StoreConversationsToDatabase (IntelDBMgr.Database, Filename, filedetails.FileSize, filedetails.FileModified, generalstats.FirstFrameTime, generalstats.LastFrameTime);
             store.free;

             IntelDBMgr.free;

             Loader.Free;
             Loader := nil;
        end;
end;

(* ------------------------------------------- *)

procedure main;

var command     : string;
    filename    : string;
    filepath    : string;
    dbfilename  : string;
    scanner     : tDirParser;
    boRecursive : boolean;
    boForceScan : boolean;
    boAutoMerge : boolean;
    i           : Integer;

begin
     command := ParamStr(1);
     filename := ParamStr (2);

     boRecursive := FindCmdLineSwitch ('r');
     boForceScan := FindCmdLineSwitch ('f');
     boAutoMerge := FindCmdLineSwitch ('m');

     if command = 'scan' then
        begin
             if fileexists (filename) then
                scanFile (filename)
             else
                 begin
                      if pos ('*', filename) > 0 then
                         begin
                              filepath := extractFilePath (filename);
                              filename := extractFilename (filename);

                              if not DirectoryExists (filepath) then
                                 filepath := GetCurrentDir;

                              scanner := tDirParser.create (filepath, filename, boRecursive);
                              scanner.execute;

                              writeln ('Scanning ' + Scanner.FileCount.ToString + ' files...');

                              for i := 1 to scanner.FileCount do
                                  begin
                                       filename := scanner.getFileName (i);
                                       dbfilename := changeFileExt (filename, '.db');

                                       if not fileexists (dbfilename) or boForceScan then
                                          scanFile (filename);
                                  end;

                              scanner.Free;

                              // Merge, if required

                              if boAutoMerge then
                                 mergeScans (filepath);
                         end
                      else
                          writeln ('Error: file not found');
                 end;
        end
     else
         if command = 'merge' then
            begin
                 if filename = '' then
                    filename := GetCurrentDir;

                 if directoryexists (filename) then
                    mergeScans (filename);
            end
         else
             writeln ('Error: command not recognized');
end;

(* ------------------------------------------- *)

procedure ShowHelp;

begin
     writeln ('PCAPscan version ' + version);
     writeln;
     writeln ('Usage: pcapscan <command> <path> [options]');
     writeln;
     writeln ('Commands');
     writeln ('scan     scan trace files into conversation db');
     writeln ('merge    merges per file DBs into the folder DB');
     writeln;
     writeln ('Options');
     writeln ('/r                 recursive scan');
     writeln ('/f                 force scan, overwriting existing DB files');
     writeln ('/m                 merge after complete scan');
     writeln ('/d=<filename>      set name for merged DB file (default: ".pcapintel.db")');
end;

(* ------------------------------------------- *)

begin
     try
        if FindCmdLineSwitch ('h')  or (ParamCount = 0) then
           ShowHelp
        else
            main;

     except
       on E: Exception do
         Writeln(E.ClassName, ': ', E.Message);
     end;
end.
