{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_dnsserverlist;

// TODO: Test this unit

interface

uses Contnrs,
//   class_definition_IP;
     definition_ip_addresses;

type tDNSServerEntry = class (tObject)

                             constructor create;
                             procedure free;

                             private
                                    priv_name         : string;
                                    priv_IPv4Address  : tIPv4Address;
                                    priv_IPv6Address  : tIPv6Address;
                                    priv_IPv4Assigned : boolean;         // Tells if priv_IPv4Address is set
                                    priv_IPv6Assigned : boolean;         // Tells if priv_IPv6Address is set

                             protected

                             public
                                   property Name : string read priv_name write priv_name;
                                   property IPv4Address : tIPv4Address read priv_IPv4Address write priv_IPv4Address;
                                   property IPv4AddressAssigned : boolean read priv_IPv4Assigned write priv_IPv4Assigned;
                                   property IPv6Address : tIPv6Address read priv_IPv6Address write priv_IPv6Address;
                                   property IPv6AddressAssigned : boolean read priv_IPv6Assigned write priv_IPv6Assigned;
                       end;

     tDNSServerList  = class (tObject)

                             constructor create;
                             procedure free;

                             private
                                    priv_serverlist : tObjectList;

                                    function getItemCount : integer;

                             protected

                             public
                                   property Count : integer read getItemCount;

                                   procedure addEntry (Entry : tDNSServerEntry);
                                   function getEntry (index : integer) : tDNSServerEntry;
                                   procedure clear;
                       end;

implementation

(* ------------------------------------------- *)
{ tDNSServerEntry }
(* ------------------------------------------- *)

constructor tDNSServerEntry.create;

begin
     inherited create;

     priv_IPv4Address.ValueAsLong := 0;
     priv_IPv4Address.MaskBitCount := 32;
     priv_IPv4Assigned := false;

     priv_IPv6Address.ValueAsWordStruct := IPv6_NODE_UNSPECIFIED;
     priv_IPv6Address.MaskBitCount := 64;
     priv_IPv6Assigned := false;
end;

(* ------------------------------------------- *)

procedure tDNSServerEntry.free;

begin
     inherited free;
end;

(* ------------------------------------------- *)
{ tDNSServerList }
(* ------------------------------------------- *)

constructor tDNSServerList.create;

begin
     inherited create;

     priv_serverlist := tObjectList.Create (true);
end;

(* ------------------------------------------- *)

procedure tDNSServerList.free;

begin
     priv_serverlist.Free;

     inherited free;
end;

(* ------------------------------------------- *)

procedure tDNSServerList.addEntry(Entry: tDNSServerEntry);

begin
     priv_serverlist.Add(Entry);
end;

(* ------------------------------------------- *)

procedure tDNSServerList.clear;

begin
     priv_serverlist.Clear;
end;

(* ------------------------------------------- *)

function tDNSServerList.getEntry(index: integer): tDNSServerEntry;

begin
     result := priv_serverlist[index -1] as tDNSServerEntry;
end;

(* ------------------------------------------- *)

function tDNSServerList.getItemCount: integer;

begin
     result := priv_serverlist.Count;
end;

(* ------------------------------------------- *)

end.
