{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

 unit class_flowhandler_baseclass;

interface

uses contnrs,
     class_rawframe,
     class_diagmessagehandler,
     class_parsemaster;

type tFlowHandler = class (tObject)

                          constructor create (Frame : tRawFrame);
                          procedure free;

                          private
                                 priv_diagnosticmsghandler : tDiagnosticMsgHandler;

                                 priv_currentframe         : integer;

                          protected
                                   prot_parsemaster             : tParseMaster;

                                   prot_framelist_filesequence  : tObjectlist;      // List of Rawframes as loaded from file

                                   procedure processFrame; virtual; abstract;

                          public
                                procedure addFrame (Frame : tRawFrame);

                    end;

implementation

uses base_messages,
     base_structures;

(* ------------------------------------------- *)
{ tFlowHandler }
(* ------------------------------------------- *)

constructor tFlowHandler.create (Frame : tRawFrame);

begin
     inherited create;

     priv_diagnosticmsghandler := tDiagnosticMsgHandler.create;

     try
        prot_parsemaster := tParseMaster.create (Frame, ppmParseAll);
     except
           priv_diagnosticmsghandler.addDiagnosticMsg (msCritical, MSG_FLOWHNDLR_PARSEFAILED);
     end;

     prot_framelist_filesequence := TObjectList.Create (false);
     prot_framelist_filesequence.Add (Frame);

     priv_currentframe := 0;
end;

(* ------------------------------------------- *)

procedure tFlowHandler.free;

begin
     prot_framelist_filesequence.free;

     inherited;
end;

(* ------------------------------------------- *)

procedure tFlowHandler.addFrame(Frame: tRawFrame);

begin
     prot_framelist_filesequence.Add (Frame);

     priv_currentframe := prot_framelist_filesequence.Count;

     // Call the main processing routine

     processFrame;
end;

(* ------------------------------------------- *)

end.
