{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_simpleframestorage;

interface

uses Contnrs,
     System.Generics.Collections,
     class_rawframe;

type tSimpleFrameStorage = class (tObject)

                                 constructor create (Const OwnsFrames : boolean = true);
                                 procedure free;

                                 private
                                        priv_framelist       : TObjectList<tRawFrame>;
                                        priv_sortbytime      : boolean;
                                        priv_currentindex    : integer;

                                        function getFrameCount: integer;

                                 protected

                                 public
                                       property Count : integer read getFrameCount;
                                       property SortByTime : boolean read priv_sortbytime write priv_sortbytime;

                                       procedure AddFrame (NewFrame : tRawFrame);
                                       procedure Clear;
                                       procedure DeleteFrame (Index : integer);
                                       function getFrame (Index : integer) : tRawFrame;
                                       function getFirst : tRawFrame;
                                       function getNext : tRawFrame;
                           end;

implementation

uses SysUtils,
     SiAuto;

(* ------------------------------------------- *)
{ tSimpleFrameStorage }
(* ------------------------------------------- *)

constructor tSimpleFrameStorage.create (Const OwnsFrames : boolean = true);

begin
     inherited create;

     priv_currentindex := 0;

     priv_sortbytime := false;
     priv_framelist := TObjectList<tRawFrame>.Create (OwnsFrames);
end;

(* ------------------------------------------- *)

procedure tSimpleFrameStorage.free;

begin
     priv_framelist.Free;

     inherited free;
end;

(* ------------------------------------------- *)

procedure tSimpleFrameStorage.AddFrame (NewFrame: tRawFrame);

var compareframe : tRawFrame;
    i            : integer;

begin
     assert (assigned (NewFrame));

     if priv_sortbytime and (priv_framelist.Count > 0) then
        begin
             i := priv_framelist.Count;          // start at one higher than there are items

             repeat
                   dec (i);

                   compareframe := priv_framelist[i];

                   // Check if the new frame is earlier than the previous one

             until (i=0) or (NewFrame.Timestamp >= compareframe.Timestamp);

             // put into the right spot

             if (NewFrame.Timestamp >= compareframe.Timestamp) then
                priv_framelist.Insert(i + 1, NewFrame)
             else
                 priv_framelist.Insert(0, NewFrame);
        end
     else
         priv_framelist.Add (NewFrame);
end;

(* ------------------------------------------- *)

procedure tSimpleFrameStorage.Clear;

begin
     priv_framelist.Clear;
end;

(* ------------------------------------------- *)

procedure tSimpleFrameStorage.DeleteFrame (Index: integer);

begin
     if (Index > 0) and (Index <= priv_framelist.Count) then
        try
           priv_framelist.Delete(Index - 1);
        except
              raise Exception.Create('Error deleting frame #' + IntToStr (Index) + ' (tSimpleFrameStorage.deleteFrame)');
        end
end;

(* ------------------------------------------- *)

function tSimpleFrameStorage.getFrame (Index: integer): tRawFrame;

var frame : tRawFrame;

begin
     if (Index > 0) and (Index <= priv_framelist.Count) then
        try
           frame := priv_framelist[Index-1];
        except
              raise Exception.Create('Error returning frame #' + IntToStr (Index) + ' (tSimpleFrameStorage.getFrame)');
        end
     else
         frame := nil;

     result := frame;
end;

(* ------------------------------------------- *)

function tSimpleFrameStorage.getFrameCount: integer;

begin
     result := priv_framelist.Count;
end;

(* ------------------------------------------- *)

function tSimpleFrameStorage.getFirst: tRawFrame;

var frame : tRawFrame;

begin
     priv_currentindex := 0;

     if priv_framelist.Count > 0 then
        frame := priv_framelist[priv_currentindex]
     else
         frame := nil;

     result := frame;
end;

(* ------------------------------------------- *)

function tSimpleFrameStorage.getNext: tRawFrame;

var frame : tRawFrame;

begin
     inc (priv_currentindex);

     if priv_currentindex < priv_framelist.Count then
        frame := priv_framelist[priv_currentindex]
     else
         frame := nil;

     result := frame;
end;

(* ------------------------------------------- *)

end.
