{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_formstorage_anontask;

interface

uses Forms,
     Classes,

{$ifdef VER230}

     // AnyDAC

     uADStanIntf, uADStanOption, uADStanError, uADGUIxIntf,
     uADPhysIntf, uADStanDef, uADStanPool,
     uADStanAsync, uADPhysManager, uADStanParam, uADDatSManager, uADDAptIntf,
     uADDAptManager, uADStanExprFuncs, Vcl.StdCtrls, uADPhysSQLite, Data.DB,
     uADCompDataSet, uADCompClient, uADGUIxFormsWait, uADPhysSQLiteWrapper,

{$else}

     // FireDAC

     FireDAC.Stan.Intf, FireDAC.Stan.Option, FireDAC.Stan.Error, FireDAC.UI.Intf,
     FireDAC.Phys.Intf, FireDAC.Stan.Def, FireDAC.Stan.Pool, FireDAC.dAPT,
     FireDAC.Stan.Async, FireDAC.Phys, FireDAC.Stan.ExprFuncs,
     FireDAC.VCLUI.Wait, FireDAC.Comp.UI, FireDAC.Phys.SQLite, FireDAC.Phys.SQLiteWrapper,
     Data.DB, FireDAC.Comp.Client, FireDAC.Stan.Param,
{$endif}

     // Smartinspect

     VirtualTrees,
     ValEdit,

     SiAuto,
     definition_ip_addresses,
     class_formstorage;

type tAnonTaskFormStorageManager = class (tFormStorageManager)

                                         constructor create (AssociatedForm : tForm);
                                         procedure free;

                                         private

                                         protected
                                                  function getDBStructure : string; override;

                                                  procedure StoreAdditionalComponentsToDB (Component : TComponent); override;
                                                  procedure RetrieveAdditionalComponentsFromDB (Component : TComponent); override;

                                         public
                                               procedure CheckAddOnTablesBeforeLoad;

                                               procedure StoreMACAddressList (Tree : TVirtualStringTree);
                                               procedure StoreIPv4AddressList (Tree : TVirtualStringTree);
                                               procedure StoreIPv4SubnetList (Tree : TVirtualStringTree);
                                               procedure StoreIPv6AddressList (Tree : TVirtualStringTree);
                                               procedure StoreIPv6SubnetList (Tree : TVirtualStringTree);
                                               procedure StoreICMPv4TypeList (Tree : TVirtualStringTree);
                                               procedure StoreVLANIDList (Tree : TVirtualStringTree);
                                               procedure StoreTCPSocketList (Tree : TVirtualStringTree);
                                               procedure StoreTCPPortList (Tree : TVirtualStringTree);
                                               procedure StoreUDPSocketList (Tree : TVirtualStringTree);
                                               procedure StoreUDPPortList (Tree : TVirtualStringTree);
                                               procedure StorePayloadReplacementStrings (Tree: TVirtualStringTree);
                                               procedure Store802dot11SSIDList (Tree : TVirtualStringTree);

                                               procedure LoadMACAddressList (Tree : TVirtualStringTree);
                                               procedure LoadIPv4AddressList (Tree : TVirtualStringTree);
                                               procedure LoadIPv4SubnetList (Tree : TVirtualStringTree);
                                               procedure LoadIPv6AddressList (Tree : TVirtualStringTree);
                                               procedure LoadIPv6SubnetList (Tree : TVirtualStringTree);
                                               procedure LoadICMPv4TypeList (Tree : TVirtualStringTree);
                                               procedure LoadVLANIDList (Tree : TVirtualStringTree);
                                               procedure LoadTCPSocketList (Tree : TVirtualStringTree);
                                               procedure LoadTCPPortList (Tree : TVirtualStringTree);
                                               procedure LoadUDPSocketList (Tree : TVirtualStringTree);
                                               procedure LoadUDPPortList (Tree : TVirtualStringTree);
                                               procedure LoadPayloadReplacementStrings (Tree: TVirtualStringTree);
                                               procedure Load802dot11SSIDList (Tree : TVirtualStringTree);

                                               function storeFormToDB (DatabaseFile : string; const Comment : string = '') : integer; virtual;
                                               function retrieveFormFromDB (DatabaseFile : string) : integer; virtual;

                                   end;

     tSocketListEntry = record
                              Socket      : tIPSocketPair;
                              isPlausible : boolean;
                        end;
     pSocketListEntry = ^tSocketListEntry;


implementation

uses SysUtils,
     base_structures,
     vstedit_payloadreplacement;

const CR = chr(13);
      LF = chr(10);
      CRLF = CR + LF;
      AnonPairTable = 'CREATE TABLE "formAnonPairs" ("id" INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL UNIQUE, "elementid" INTEGER NOT NULL, "ipversion" INTEGER DEFAULT 4, "original" VARCHAR NOT NULL, "replacement" VARCHAR NOT NULL, "misc" VARCHAR );' + CRLF +
                      'CREATE TABLE "formAnonIMCPv4Types" ("id" INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL UNIQUE, "type" INTEGER NOT NULL, "code" INTEGER NOT NULL, "checked" BOOLEAN DEFAULT 0);';

      MACAddressListID = 100;
      VLANIDListID = 101;

      IPv4AddressListID = 103;
      IPv4SubnetListID = 104;
      IPv6AddressListID = 105;
      IPv6SubnetListID = 106;
      TCPPortListID = 107;
      UDPPortListID = 108;
      PayLoadStringListID = 109;
      TCPSocketListID = 110;
      UDPSocketListID = 111;
      WiFiSSIDListID = 112;


(* -------------------------------------------- *)
{ tAnonTaskFormStorageManager }
(* -------------------------------------------- *)

constructor tAnonTaskFormStorageManager.create(AssociatedForm: TForm);

begin
     inherited create (AssociatedForm);
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.free;

begin
     inherited free;
end;

(* -------------------------------------------- *)

function tAnonTaskFormStorageManager.getDBStructure: string;

var struct : string;

begin
     struct := AnonPairTable + inherited;

     result := struct;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.CheckAddOnTablesBeforeLoad;

begin
     priv_query.SQL.Text := 'SELECT * FROM sqlite_master WHERE name="formAnonIMCPv4Types";';
     priv_query.Open;

     if priv_query.RecordCount = 0 then
        begin
             priv_query.SQL.Text := 'CREATE TABLE "formAnonIMCPv4Types" ("id" INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL UNIQUE, "type" INTEGER NOT NULL, "code" INTEGER NOT NULL, "checked" BOOLEAN DEFAULT 0);';
             priv_query.ExecSQL;
        end;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.RetrieveAdditionalComponentsFromDB (Component: TComponent);

begin

end;

(* -------------------------------------------- *)

function tAnonTaskFormStorageManager.retrieveFormFromDB (DatabaseFile: string): integer;

begin
     inherited;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.LoadMACAddressList (Tree: TVirtualStringTree);

var Node        : PVirtualNode;
    Data        : pMACAddressPair;
    Original    : ANSIString;
    Replacement : ANSIString;
    i           : integer;

begin
     priv_query.SQL.Text := 'SELECT * FROM formAnonPairs WHERE elementid=:elementid;)';
     priv_query.ParamByName ('elementid').AsInteger := MACAddressListID;
     priv_query.Open;
     priv_query.FetchAll;

     Tree.Clear;
     Tree.BeginUpdate;

     for i := 1 to priv_query.RecordCount do
         begin
              Node := Tree.AddChild (nil);
              Data := Tree.GetNodeData (Node);

              Original := priv_query.FieldByName ('original').AsAnsiString;
              Replacement := priv_query.FieldByName ('replacement').AsAnsiString;

              Data^.Source.setfromString (Original, ':');
              Data^.Destination.setfromString (Replacement, ':');

              priv_query.Next;
         end;

     Tree.EndUpdate;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.LoadTCPSocketList (Tree: TVirtualStringTree);

var Node        : PVirtualNode;
    Data        : pSocketListEntry;
    Original    : string;
    Replacement : string;
    i           : integer;

begin
     priv_query.SQL.Text := 'SELECT * FROM formAnonPairs WHERE elementid=:elementid;)';
     priv_query.ParamByName ('elementid').AsInteger := TCPSocketListID;
     priv_query.Open;
     priv_query.FetchAll;

     Tree.Clear;
     Tree.BeginUpdate;

     for i := 1 to priv_query.RecordCount do
         begin
              Node := Tree.AddChild (nil);
              Data := Tree.GetNodeData (Node);

              Original := priv_query.FieldByName ('original').AsString;
              Replacement := priv_query.FieldByName ('replacement').AsString;

              Data^.Socket.Source.IPAddress.IPVersion := priv_query.FieldByName ('ipversion').AsInteger;

              Data^.Socket.Source.initialize (Original);
              Data^.Socket.Destination.initialize (Replacement);
              Data^.isPlausible := true;          // will be checked after load in form

              priv_query.Next;
         end;

     Tree.EndUpdate;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.LoadTCPPortList (Tree: TVirtualStringTree);

var Node        : PVirtualNode;
    Data        : pPortPair;
    Original    : string;
    Replacement : string;
    i           : integer;

begin
     priv_query.SQL.Text := 'SELECT * FROM formAnonPairs WHERE elementid=:elementid;)';
     priv_query.ParamByName ('elementid').AsInteger := TCPPortListID;
     priv_query.Open;
     priv_query.FetchAll;

     Tree.Clear;
     Tree.BeginUpdate;

     for i := 1 to priv_query.RecordCount do
         begin
              Node := Tree.AddChild (nil);
              Data := Tree.GetNodeData (Node);

              Original := priv_query.FieldByName ('original').AsString;
              Replacement := priv_query.FieldByName ('replacement').AsString;

              Data^.Value1 := StrToInt (Original);
              Data^.Value2 := StrToInt (Replacement);

              priv_query.Next;
         end;

     Tree.EndUpdate;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.LoadUDPSocketList (Tree: TVirtualStringTree);

var Node        : PVirtualNode;
    Data        : pSocketListEntry;
    Original    : string;
    Replacement : string;
    i           : integer;

begin
     priv_query.SQL.Text := 'SELECT * FROM formAnonPairs WHERE elementid=:elementid;)';
     priv_query.ParamByName ('elementid').AsInteger := UDPSocketListID;
     priv_query.Open;
     priv_query.FetchAll;

     Tree.Clear;
     Tree.BeginUpdate;

     for i := 1 to priv_query.RecordCount do
         begin
              Node := Tree.AddChild (nil);
              Data := Tree.GetNodeData (Node);

              Original := priv_query.FieldByName ('original').AsString;
              Replacement := priv_query.FieldByName ('replacement').AsString;

              Data^.Socket.Source.IPAddress.IPVersion := priv_query.FieldByName ('ipversion').AsInteger;

              Data^.Socket.Source.initialize (Original);
              Data^.Socket.Destination.initialize (Replacement);
              Data^.isPlausible := true;          // will be checked after load in form

              priv_query.Next;
         end;

     Tree.EndUpdate;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.LoadUDPPortList (Tree: TVirtualStringTree);

var Node        : PVirtualNode;
    Data        : pPortPair;
    Original    : string;
    Replacement : string;
    i           : integer;

begin
     priv_query.SQL.Text := 'SELECT * FROM formAnonPairs WHERE elementid=:elementid;)';
     priv_query.ParamByName ('elementid').AsInteger := UDPPortListID;
     priv_query.Open;
     priv_query.FetchAll;

     Tree.Clear;
     Tree.BeginUpdate;

     for i := 1 to priv_query.RecordCount do
         begin
              Node := Tree.AddChild (nil);
              Data := Tree.GetNodeData (Node);

              Original := priv_query.FieldByName ('original').AsString;
              Replacement := priv_query.FieldByName ('replacement').AsString;

              Data^.Value1 := StrToInt (Original);
              Data^.Value2 := StrToInt (Replacement);

              priv_query.Next;
         end;

     Tree.EndUpdate;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.LoadVLANIDList (Tree: TVirtualStringTree);

var Node        : PVirtualNode;
    Data        : pVLANPair;
    Original    : string;
    Replacement : string;
    i           : integer;

begin
     priv_query.SQL.Text := 'SELECT * FROM formAnonPairs WHERE elementid=:elementid;)';
     priv_query.ParamByName ('elementid').AsInteger := VLANIDListID;
     priv_query.Open;
     priv_query.FetchAll;

     Tree.Clear;
     Tree.BeginUpdate;

     for i := 1 to priv_query.RecordCount do
         begin
              Node := Tree.AddChild (nil);
              Data := Tree.GetNodeData (Node);

              Original := priv_query.FieldByName ('original').AsString;
              Replacement := priv_query.FieldByName ('replacement').AsString;

              Data^.Value1 := StrToInt (Original);
              Data^.Value2 := StrToInt (Replacement);

              priv_query.Next;
         end;

     Tree.EndUpdate;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.LoadIPv4AddressList (Tree: TVirtualStringTree);

var Node        : PVirtualNode;
    Data        : pIPv4AddressPair;
    Original    : string;
    Replacement : string;
    i           : integer;

begin
     priv_query.SQL.Text := 'SELECT * FROM formAnonPairs WHERE elementid=:elementid;)';
     priv_query.ParamByName ('elementid').AsInteger := IPv4AddressListID;
     priv_query.Open;
     priv_query.FetchAll;

     Tree.Clear;
     Tree.BeginUpdate;

     for i := 1 to priv_query.RecordCount do
         begin
              Node := Tree.AddChild (nil);
              Data := Tree.GetNodeData (Node);

              Original := priv_query.FieldByName ('original').AsString;
              Replacement := priv_query.FieldByName ('replacement').AsString;

              Data^.Source.setValueFromString (Original, 32);
              Data^.Destination.setValueFromString (Replacement, 32);

              priv_query.Next;
         end;

     Tree.EndUpdate;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.LoadIPv4SubnetList (Tree: TVirtualStringTree);

var Node        : PVirtualNode;
    Data        : pIPv4AddressPair;
    Original    : string;
    Replacement : string;
    i           : integer;

begin
     priv_query.SQL.Text := 'SELECT * FROM formAnonPairs WHERE elementid=:elementid;)';
     priv_query.ParamByName ('elementid').AsInteger := IPv4SubnetListID;
     priv_query.Open;
     priv_query.FetchAll;

     Tree.Clear;
     Tree.BeginUpdate;

     for i := 1 to priv_query.RecordCount do
         begin
              Node := Tree.AddChild (nil);
              Data := Tree.GetNodeData (Node);

              Original := priv_query.FieldByName ('original').AsString;
              Replacement := priv_query.FieldByName ('replacement').AsString;

              Data^.Source.setValueFromCIDRString (Original);
              Data^.Destination.setValueFromCIDRString (Replacement);

              priv_query.Next;
         end;

     Tree.EndUpdate;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.LoadIPv6AddressList (Tree: TVirtualStringTree);

var Node        : PVirtualNode;
    Data        : pIPv6AddressPair;
    Original    : string;
    Replacement : string;
    i           : integer;

begin
     priv_query.SQL.Text := 'SELECT * FROM formAnonPairs WHERE elementid=:elementid;)';
     priv_query.ParamByName ('elementid').AsInteger := IPv6AddressListID;
     priv_query.Open;
     priv_query.FetchAll;

     Tree.Clear;
     Tree.BeginUpdate;

     for i := 1 to priv_query.RecordCount do
         begin
              Node := Tree.AddChild (nil);
              Data := Tree.GetNodeData (Node);

              Original := priv_query.FieldByName ('original').AsString;
              Replacement := priv_query.FieldByName ('replacement').AsString;

              Data^.Source.setValueFromString (Original, 64);
              Data^.Destination.setValueFromString (Replacement, 64);

              priv_query.Next;
         end;

     Tree.EndUpdate;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.LoadIPv6SubnetList (Tree: TVirtualStringTree);

var Node        : PVirtualNode;
    Data        : pIPv6AddressPair;
    Original    : string;
    Replacement : string;
    i           : integer;

begin
     priv_query.SQL.Text := 'SELECT * FROM formAnonPairs WHERE elementid=:elementid;)';
     priv_query.ParamByName ('elementid').AsInteger := IPv6SubnetListID;
     priv_query.Open;
     priv_query.FetchAll;

     Tree.Clear;
     Tree.BeginUpdate;

     for i := 1 to priv_query.RecordCount do
         begin
              Node := Tree.AddChild (nil);
              Data := Tree.GetNodeData (Node);

              Original := priv_query.FieldByName ('original').AsString;
              Replacement := priv_query.FieldByName ('replacement').AsString;

              Data^.Source.setValueFromStringWithMask (Original);
              Data^.Destination.setValueFromStringWithMask (Replacement);

              priv_query.Next;
         end;

     Tree.EndUpdate;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.LoadICMPv4TypeList (Tree: TVirtualStringTree);

var Node  : PVirtualNode;
    Data  : pICMPv4TreeEntry;
    iType : integer;
    iCode : integer;
    found : boolean;
    i     : Integer;

begin
     // Unselect all nodes

     Node := Tree.GetFirst;

     repeat
           Node.CheckState := csUncheckedNormal;

           Node := Tree.GetNext (node);

     until (Node = nil) ;


     priv_query.SQL.Text := 'SELECT * FROM formAnonIMCPv4Types WHERE checked=1;)';
     priv_query.Open;
     priv_query.FetchAll;

     for i := 1 to priv_query.RecordCount do
         begin
              iType := priv_query.FieldByName ('type').AsInteger;
              iCode := priv_query.FieldByName ('code').AsInteger;

              // Scan the tree for the according node

              Node := Tree.GetFirst;
              found := false;

              repeat
                    data := Tree.GetNodeData (node);

                    if (data^.TypeCode.ICMPv4Type = iType) and (data^.TypeCode.ICMPv4Code = iCode) then
                       begin
                            Node.CheckState := csCheckedNormal;
                            found := true;
                       end;

                    Node := Tree.GetNext (node);

              until (Node = nil) or found;

              priv_query.Next;
         end;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.LoadPayloadReplacementStrings (Tree: TVirtualStringTree);

var Help : tANSIStringPair;
    Node : PVirtualNode;
    Data : pReplacementEditorData;
    i    : integer;

begin
     priv_query.SQL.Text := 'SELECT * FROM formAnonPairs WHERE elementid=:elementid;)';
     priv_query.ParamByName ('elementid').AsInteger := PayLoadStringListID;
     priv_query.Open;
     priv_query.FetchAll;

     Tree.BeginUpdate;
     Tree.Clear;

     for i := 1 to priv_query.RecordCount do
         begin
              Help.Original := priv_query.FieldByName ('original').AsString;
              Help.Replacement := priv_query.FieldByName ('replacement').AsString;

              Node := Tree.AddChild (nil);
              Data := Tree.GetNodeData (Node);

              Data^.original := Help.Original;
              Data^.replacement := Help.Replacement;
              Data^.deltalen := Help.getLengthDelta;

              priv_query.Next;
         end;

     Tree.EndUpdate;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.Load802dot11SSIDList (Tree: TVirtualStringTree);

var Help : tANSIStringPair;
    Node : PVirtualNode;
    Data : pReplacementEditorData;
    i    : integer;

begin
     priv_query.SQL.Text := 'SELECT * FROM formAnonPairs WHERE elementid=:elementid;)';
     priv_query.ParamByName ('elementid').AsInteger := WiFiSSIDListID;
     priv_query.Open;
     priv_query.FetchAll;

     Tree.BeginUpdate;
     Tree.Clear;

     for i := 1 to priv_query.RecordCount do
         begin
              Help.Original := priv_query.FieldByName ('original').AsString;
              Help.Replacement := priv_query.FieldByName ('replacement').AsString;

              Node := Tree.AddChild (nil);
              Data := Tree.GetNodeData (Node);

              Data^.original := Help.Original;
              Data^.replacement := Help.Replacement;
              Data^.deltalen := Help.getLengthDelta;

              priv_query.Next;
         end;

     Tree.EndUpdate;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.StoreMACAddressList (Tree: TVirtualStringTree);

var Node : PVirtualNode;
    Data : pMACAddressPair;

begin
     Node := Tree.GetFirst;

     while Node <> nil do
          begin
               Data := Tree.GetNodeData (Node);

               priv_query.SQL.Text := 'INSERT INTO formAnonPairs (elementid,original,replacement) VALUES (:elementid,:original,:replacement);';
               priv_query.ParamByName ('elementid').AsInteger := MACAddressListID;
               priv_query.ParamByName ('original').AsString := Data^.Source.asString(':');
               priv_query.ParamByName ('replacement').AsString := Data^.Destination.asString(':');
               priv_query.ExecSQL;

               Node := Tree.GetNext(Node);
          end;
end;


(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.StoreTCPSocketList (Tree: TVirtualStringTree);

var Node : PVirtualNode;
//  Data : pSocketPortPair;
    Data : pIPSocketPair;

begin
     Node := Tree.GetFirst;

     while Node <> nil do
          begin
               Data := Tree.GetNodeData (Node);

               priv_query.SQL.Text := 'INSERT INTO formAnonPairs (elementid,ipversion,original,replacement) VALUES (:elementid,:ipversion,:original,:replacement);';
               priv_query.ParamByName ('elementid').AsInteger := TCPSocketListID;
//             priv_query.ParamByName ('ipversion').AsInteger := Data^.IPVersion;
               priv_query.ParamByName ('ipversion').AsInteger := Data^.Source.IPAddress.IPVersion;

               priv_query.ParamByName ('original').AsString := Data^.Source.ValueAsString;
               priv_query.ParamByName ('replacement').AsString := Data^.Destination.ValueAsString;
{
               case Data^.IPVersion of
                4 : priv_query.ParamByName ('original').AsString := Data^.OriginalIPv4Socket.ValueAsString;
                6 : priv_query.ParamByName ('original').AsString := Data^.OriginalIPv6Socket.ValueAsString;
               end;

               priv_query.ParamByName ('replacement').AsString := IntToStr (Data^.NewPort);
}
               priv_query.ExecSQL;

               Node := Tree.GetNext(Node);
          end;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.StoreTCPPortList (Tree: TVirtualStringTree);

var Node : PVirtualNode;
    Data : pPortPair;

begin
     Node := Tree.GetFirst;

     while Node <> nil do
          begin
               Data := Tree.GetNodeData (Node);

               priv_query.SQL.Text := 'INSERT INTO formAnonPairs (elementid,original,replacement) VALUES (:elementid,:original,:replacement);';
               priv_query.ParamByName ('elementid').AsInteger := TCPPortListID;
               priv_query.ParamByName ('original').AsString := IntToStr (Data^.Value1);
               priv_query.ParamByName ('replacement').AsString := IntToStr (Data^.Value2);
               priv_query.ExecSQL;

               Node := Tree.GetNext(Node);
          end;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.StoreUDPSocketList (Tree: TVirtualStringTree);

var Node : PVirtualNode;
//  Data : pSocketPortPair;
    Data : pIPSocketPair;

begin
     Node := Tree.GetFirst;

     while Node <> nil do
          begin
               Data := Tree.GetNodeData (Node);

               priv_query.SQL.Text := 'INSERT INTO formAnonPairs (elementid,ipversion,original,replacement) VALUES (:elementid,:ipversion,:original,:replacement);';
               priv_query.ParamByName ('elementid').AsInteger := UDPSocketListID;

               priv_query.ParamByName ('ipversion').AsInteger := Data^.Source.IPAddress.IPVersion;
               priv_query.ParamByName ('original').AsString := Data^.Source.ValueAsString;
               priv_query.ParamByName ('replacement').AsString := Data^.Destination.ValueAsString;

{
               priv_query.ParamByName ('ipversion').AsInteger := Data^.IPVersion;

               case Data^.IPVersion of
                4 : priv_query.ParamByName ('original').AsString := Data^.OriginalIPv4Socket.ValueAsString;
                6 : priv_query.ParamByName ('original').AsString := Data^.OriginalIPv6Socket.ValueAsString;
               end;

               priv_query.ParamByName ('replacement').AsString := IntToStr (Data^.NewPort);
}
               priv_query.ExecSQL;

               Node := Tree.GetNext(Node);
          end;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.StoreUDPPortList (Tree: TVirtualStringTree);

var Node : PVirtualNode;
    Data : pPortPair;

begin
     Node := Tree.GetFirst;

     while Node <> nil do
          begin
               Data := Tree.GetNodeData (Node);

               priv_query.SQL.Text := 'INSERT INTO formAnonPairs (elementid,original,replacement) VALUES (:elementid,:original,:replacement);';
               priv_query.ParamByName ('elementid').AsInteger := UDPPortListID;
               priv_query.ParamByName ('original').AsString := IntToStr (Data^.Value1);
               priv_query.ParamByName ('replacement').AsString := IntToStr (Data^.Value2);
               priv_query.ExecSQL;

               Node := Tree.GetNext(Node);
          end;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.StoreVLANIDList (Tree: TVirtualStringTree);

var Node : PVirtualNode;
    Data : pVLANPair;

begin
     Node := Tree.GetFirst;

     while Node <> nil do
          begin
               Data := Tree.GetNodeData (Node);

               priv_query.SQL.Text := 'INSERT INTO formAnonPairs (elementid,original,replacement) VALUES (:elementid,:original,:replacement);';
               priv_query.ParamByName ('elementid').AsInteger := VLANIDListID;
               priv_query.ParamByName ('original').AsString := IntToStr (Data^.Value1);
               priv_query.ParamByName ('replacement').AsString := IntToStr (Data^.Value2);
               priv_query.ExecSQL;

               Node := Tree.GetNext(Node);
          end;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.StoreIPv4AddressList (Tree: TVirtualStringTree);

var Node : PVirtualNode;
    Data : pIPv4AddressPair;

begin
     Node := Tree.GetFirst;

     while Node <> nil do
          begin
               Data := Tree.GetNodeData (Node);

               priv_query.SQL.Text := 'INSERT INTO formAnonPairs (elementid,original,replacement) VALUES (:elementid,:original,:replacement);';
               priv_query.ParamByName ('elementid').AsInteger := IPv4AddressListID;
               priv_query.ParamByName ('original').AsString := Data^.Source.ValueAsString;
               priv_query.ParamByName ('replacement').AsString := Data^.Destination.ValueAsString;
               priv_query.ExecSQL;

               Node := Tree.GetNext(Node);
          end;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.StoreIPv4SubnetList (Tree: TVirtualStringTree);

var Node : PVirtualNode;
    Data : pIPv4AddressPair;

begin
     Node := Tree.GetFirst;

     while Node <> nil do
          begin
               Data := Tree.GetNodeData (Node);

               priv_query.SQL.Text := 'INSERT INTO formAnonPairs (elementid,original,replacement) VALUES (:elementid,:original,:replacement);';
               priv_query.ParamByName ('elementid').AsInteger := IPv4SubnetListID;
               priv_query.ParamByName ('original').AsString := Data^.Source.ValueAsCIDRString;
               priv_query.ParamByName ('replacement').AsString := Data^.Destination.ValueAsCIDRString;
               priv_query.ExecSQL;

               Node := Tree.GetNext(Node);
          end;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.StoreIPv6AddressList (Tree: TVirtualStringTree);

var Node : PVirtualNode;
    Data : pIPv6AddressPair;

begin
     Node := Tree.GetFirst;

     while Node <> nil do
          begin
               Data := Tree.GetNodeData (Node);

               priv_query.SQL.Text := 'INSERT INTO formAnonPairs (elementid,original,replacement) VALUES (:elementid,:original,:replacement);';
               priv_query.ParamByName ('elementid').AsInteger := IPv6AddressListID;
               priv_query.ParamByName ('original').AsString := Data^.Source.ValueAsString;
               priv_query.ParamByName ('replacement').AsString := Data^.Destination.ValueAsString;
               priv_query.ExecSQL;

               Node := Tree.GetNext(Node);
          end;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.StoreIPv6SubnetList (Tree: TVirtualStringTree);

var Node : PVirtualNode;
    Data : pIPv6AddressPair;

begin
     Node := Tree.GetFirst;

     while Node <> nil do
          begin
               Data := Tree.GetNodeData (Node);

               priv_query.SQL.Text := 'INSERT INTO formAnonPairs (elementid,original,replacement) VALUES (:elementid,:original,:replacement);';
               priv_query.ParamByName ('elementid').AsInteger := IPv6SubnetListID;
               priv_query.ParamByName ('original').AsString := Data^.Source.ValueAsSlashedNotationString;
               priv_query.ParamByName ('replacement').AsString := Data^.Destination.ValueAsSlashedNotationString;
               priv_query.ExecSQL;

               Node := Tree.GetNext(Node);
          end;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.StoreICMPv4TypeList (Tree: TVirtualStringTree);

var Node : PVirtualNode;
    Data : pICMPv4TreeEntry;

begin
     Node := Tree.GetFirst;

     while Node <> nil do
          begin
               Data := Tree.GetNodeData (Node);

               priv_query.SQL.Text := 'INSERT INTO formAnonIMCPv4Types (type,code,checked) VALUES (:type,:code,:checked);';
               priv_query.ParamByName ('type').AsInteger := Data^.TypeCode.ICMPv4Type;
               priv_query.ParamByName ('code').AsInteger := Data^.TypeCode.ICMPv4Code;
               priv_query.ParamByName ('checked').AsBoolean := (Node.CheckState = csCheckedNormal);
               priv_query.ExecSQL;

               Node := Tree.GetNext(Node);
          end;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.StorePayLoadReplacementStrings (Tree: TVirtualStringTree);

var Node : PVirtualNode;
    Data : pReplacementEditorData;

begin
     Node := Tree.GetFirst;

     while (Node <> nil) do
          begin
               Data := Tree.GetNodeData (Node);

               if (Data^.original <> '') or (Data^.replacement <> '') then
                  begin
                       priv_query.SQL.Create.Text := 'INSERT INTO formAnonPairs (elementid,original,replacement) VALUES (:elementid,:original,:replacement);';
                       priv_query.ParamByName ('elementid').AsInteger := PayLoadStringListID;
                       priv_query.ParamByName ('original').AsAnsiString := Data^.original;
                       priv_query.ParamByName ('replacement').AsAnsiString := Data^.replacement;
                       priv_query.ExecSQL;
                  end;

               Node := Tree.GetNext (Node);
         end;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.Store802dot11SSIDList (Tree: TVirtualStringTree);

var Node : PVirtualNode;
    Data : pReplacementEditorData;

begin
     Node := Tree.GetFirst;

     while Node <> nil do
          begin
               Data := Tree.GetNodeData (Node);

               if (Data^.original <> '') or (Data^.replacement <> '') then
                  begin
                       priv_query.SQL.Create.Text := 'INSERT INTO formAnonPairs (elementid,original,replacement) VALUES (:elementid,:original,:replacement);';
                       priv_query.ParamByName ('elementid').AsInteger := WiFiSSIDListID;
                       priv_query.ParamByName ('original').AsAnsiString := Data^.original;
                       priv_query.ParamByName ('replacement').AsAnsiString := Data^.replacement;
                       priv_query.ExecSQL;
                  end;

               Node := Tree.GetNext (Node);
         end;
end;

(* -------------------------------------------- *)

procedure tAnonTaskFormStorageManager.StoreAdditionalComponentsToDB (Component: TComponent);

// var TreeName : string;

begin
     // For some reason tVirtualStringTrees do not seem to make it here... no idea why
{
     if Component.ClassType = tVirtualStringTree then
        begin
             TreeName := Component.Name;

             if TreeName = 'VSTMACAddressList' then
                StoreMACAddressList (Component as TVirtualStringTree);

        end;
}
end;

(* -------------------------------------------- *)

function tAnonTaskFormStorageManager.storeFormToDB (DatabaseFile: string; const Comment: string): integer;

begin
     inherited;
end;

(* -------------------------------------------- *)

end.
