{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_tasklistmanager;

interface

uses Classes,
     Contnrs;

type tTaskType = (ttNone, ttAnonymize, ttExtract, ttEdit, ttMerge, ttSeparate);
     tTaskStatus = (tsNew, tsPending, tsRunning, tsAborted, tsFailed, tsCompleted);

     tTaskEntry       = class (tObject)

                              constructor create (const TaskName : string; const TaskType : tTaskType);
                              procedure free;

                              private
                                     priv_index             : integer;
                                     priv_taskname          : string;
                                     priv_tasktype          : tTaskType;
                                     priv_parameterfile     : string;
                                     priv_taskstatus        : tTaskStatus;
                                     priv_description       : string;

                              public
                                    property TaskIndex : integer read priv_index;
                                    property TaskName : string read priv_taskname write priv_taskname;
                                    property TaskType : tTaskType read priv_tasktype write priv_tasktype;
                                    property ParameterFile : string read priv_parameterfile write priv_parameterfile;
                                    property Status : tTaskStatus read priv_taskstatus write priv_taskstatus;

                        end;

     tTaskListManager = class (tObject)

                              constructor create (DefaultPathToTaskData : string);
                              procedure free;

                              private
                                     priv_tasklist     : tObjectList;
                                     priv_taskdatapath : string;

                              public
                                    procedure addTask (NewTask : tTaskEntry);
                                    procedure clear;
                                    function count : integer;
                                    function getTask (const Index : integer) : tTaskEntry;
                                    procedure removeTask (const Index : integer);
                                    procedure ExchangeTasks (const Index1, Index2 : integer);
                                    procedure MoveTaskToIndex (const CurIndex, NewIndex : integer);

                        end;

implementation

uses SysUtils;

(* -------------------------------------- *)
{ tTaskEntry }
(* -------------------------------------- *)

constructor tTaskEntry.create (const TaskName : string; const TaskType : tTaskType);

begin
     inherited create;

     priv_taskname := TaskName;
     priv_tasktype := TaskType;
     priv_taskstatus := tsNew;
     priv_parameterfile := '';
end;

(* -------------------------------------- *)

procedure tTaskEntry.free;

begin
     inherited;
end;

(* -------------------------------------- *)
{ tTaskListManager }
(* -------------------------------------- *)

constructor tTaskListManager.create (DefaultPathToTaskData : string);

begin
     inherited create;

     priv_taskdatapath := DefaultPathToTaskData;
     priv_tasklist := tObjectList.Create (true);
end;

(* -------------------------------------- *)

procedure tTaskListManager.free;

begin
     priv_tasklist.Free;

     inherited;
end;

(* -------------------------------------- *)

procedure tTaskListManager.addTask(NewTask: tTaskEntry);

begin
     priv_tasklist.Add(NewTask);

     NewTask.priv_index := priv_tasklist.Count;
end;

(* -------------------------------------- *)

procedure tTaskListManager.removeTask(const Index: integer);

var task : tTaskEntry;
    path : string;

begin
     task := priv_tasklist[Index-1] as tTaskEntry;

     if fileexists (task.ParameterFile) then
        begin
             path := ExtractFilePath (task.ParameterFile);

             if path = priv_taskdatapath then
                try
                   deleteFile (task.ParameterFile);
                except
                      // TODO: handle error
                end;

             priv_tasklist.Delete(Index-1);
        end;
end;

(* -------------------------------------- *)

procedure tTaskListManager.clear;

begin
     priv_tasklist.Clear;
end;

(* -------------------------------------- *)

function tTaskListManager.count: integer;

begin
     result := priv_tasklist.Count;
end;

(* -------------------------------------- *)

procedure tTaskListManager.ExchangeTasks(const Index1, Index2: integer);

begin
     priv_tasklist.Exchange(Index1, Index2);
end;

(* -------------------------------------- *)

function tTaskListManager.getTask(const Index: integer): tTaskEntry;

begin
     if (Index > 0) and (Index <= priv_tasklist.Count) then
        result := priv_tasklist[Index-1] as tTaskEntry
     else
         result := nil;
end;

(* -------------------------------------- *)

procedure tTaskListManager.MoveTaskToIndex(const CurIndex, NewIndex: integer);

begin
     priv_tasklist.Move(CurIndex, NewIndex);
end;

(* -------------------------------------- *)

end.
