{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit form_task_separate;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls;

type
  TFormTaskSeparation = class(TForm)
    GroupBoxOutputLocation: TGroupBox;
    RadioButtonFileOutputSameDirectory: TRadioButton;
    RadioButtonFileOutputToSubdirectory: TRadioButton;
    LabeledEditOutputToSubdirectory: TLabeledEdit;
    RadioButtonFileOutputToAbsoluteDirectory: TRadioButton;
    LabeledEditOutputToAbsoluteDirectory: TLabeledEdit;
    ButtonSelectOutputDirectory: TButton;
    GroupBoxFileOutputNamingOptions: TGroupBox;
    EditOutputFilename: TEdit;
    MemoOutputHint: TMemo;
    ButtonOkay: TButton;
    ButtonCancel: TButton;
    LabelOutputFilename: TLabel;
    procedure onChangeOutputSettings(Sender: TObject);
    procedure ButtonSelectOutputDirectoryClick(Sender: TObject);
    procedure onCreate(Sender: TObject);
  private
    { Private declarations }

         function BrowseDialog (const Title: string; const Flag: integer): string;
  public
    { Public declarations }

        procedure LoadSettings (const Filename : string);
        procedure SaveSettings (const Filename : string);

  end;

var
  FormTaskSeparation: TFormTaskSeparation;

const TaskTypeSeparate = 'TaskSeparate';

implementation

{$R *.dfm}

uses ShlObj,
     class_formstorage;

(* ------------------------------------------- *)

procedure TFormTaskSeparation.onCreate(Sender: TObject);

begin
     // Workaround for scaling problems: look for /noscale command line option

     self.Scaled := FindCmdLineSwitch ('noscale');
end;

(* ------------------------------------------- *)

function TFormTaskSeparation.BrowseDialog (const Title: string; const Flag: integer): string;

var lpItemID    : PItemIDList;
    BrowseInfo  : TBrowseInfo;
    DisplayName : array[0..MAX_PATH] of char;
    TempPath    : array[0..MAX_PATH] of char;

begin
     Result:='';
     FillChar(BrowseInfo, sizeof(TBrowseInfo), #0);

     with BrowseInfo do
          begin
               hwndOwner := Application.Handle;
               pszDisplayName := @DisplayName;
               lpszTitle := PChar(Title);
               ulFlags := Flag;
          end;

  lpItemID := SHBrowseForFolder(BrowseInfo);

  if lpItemId <> nil then
     begin
          SHGetPathFromIDList(lpItemID, TempPath);
          Result := TempPath;
          GlobalFreePtr(lpItemID);
     end;
end;

(* ------------------------------------------- *)

procedure TFormTaskSeparation.ButtonSelectOutputDirectoryClick (Sender: TObject);

var sPath : string;

begin
     sPath := BrowseDialog ('Select Path', BIF_RETURNONLYFSDIRS);

     if DirectoryExists (sPath) then
        LabeledEditOutputToAbsoluteDirectory.Text := sPath;
end;

(* ------------------------------------------- *)

procedure TFormTaskSeparation.LoadSettings (const Filename: string);

var StorMan : tFormStorageManager;

begin
     StorMan := tFormStorageManager.create (self);

     StorMan.retrieveFormFromDB (Filename);
     StorMan.free;
end;

(* ------------------------------------------- *)

procedure TFormTaskSeparation.SaveSettings (const Filename: string);

var StorMan : tFormStorageManager;

begin
     StorMan := tFormStorageManager.create (self);

     StorMan.storeFormToDB (Filename, TaskTypeSeparate);
     StorMan.free;
end;

(* ------------------------------------------- *)

procedure TFormTaskSeparation.onChangeOutputSettings(Sender: TObject);

begin
     LabeledEditOutputToSubdirectory.Enabled := RadioButtonFileOutputToSubdirectory.Checked;
     LabeledEditOutputToAbsoluteDirectory.Enabled := RadioButtonFileOutputToAbsoluteDirectory.Checked;
end;

(* ------------------------------------------- *)

end.
