{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}
 
unit Testclass_assembly_tcp;
{

  Delphi DUnit Test Case
  ----------------------
  This unit contains a skeleton test case class generated by the Test Case Wizard.
  Modify the generated code to correctly setup and call the methods from the unit 
  being tested.

}

interface

uses
  TestFramework, class_assembly_tcp, class_assembly_baseclass, class_parser_tcp,
  base_structures, base_checksums;

type
  // Test methods for class tTCPAssembler

  TesttTCPAssembler = class(TTestCase)
  strict private
    FtTCPAssembler: tTCPAssembler;
  public
        procedure SetUp;
        procedure TearDown;

  published

    procedure TestAssemble;
  end;

implementation

uses class_rawframe,
     class_loader_pcapng,
     class_parser_ipv4,
     class_parsemaster;


(* ------------------------------------------- *)

procedure TesttTCPAssembler.SetUp;

begin
     FtTCPAssembler := tTCPAssembler.Create;
end;

(* ------------------------------------------- *)

procedure TesttTCPAssembler.TearDown;

begin
     FtTCPAssembler.Free;
     FtTCPAssembler := nil;
end;

(* ------------------------------------------- *)

procedure TesttTCPAssembler.TestAssemble;

var FtParseMaster : tParseMaster;
    FtLoader      : tPCAPNGLoader;
    FtIPParser    : tIPv4Parser;
    FtTCPParser   : tTCPParser;
    OpenResult    : boolean;
    Frame         : tRawFrame;
    PseudoHdr     : tIPv4PseudoHeader;
    testbuf       : tBufferWords;
    layerdata     : tFrameChunk;

begin
     FtLoader := tPCAPNGLoader.create ('d:\traces\TCP-ShortSampleConversation.pcapng');
     OpenResult := FtLoader.openFile;

     CheckEquals (true, Openresult, 'PCAPNGLoader.OpenFile Failed');

     FtLoader.readTraceFileheader;
     FtLoader.readBatchFromDisk;

     Frame := FtLoader.getRawFrame(1);

     FtParseMaster := tParseMaster.create(Frame, ppmParseAll);

     // Get IP Parser for the Pseudo header

     FtIPParser := (FtParseMaster.getParserByIndex(2) as tIPv4Parser);
     FtTCPParser := (FtParseMaster.getParserByIndex(3) as tTCPParser);
     PseudoHdr := FtIPParser.PseudoHeader;

     // Set parameters for assembly

     if FtParseMaster.HasRemainingPayload then
        begin
             // test

             testbuf := getWordsStructFromOffset (FtParseMaster.RemainingPayloadCopy.Data, 0, FtParseMaster.RemainingPayloadCopy.Len);

             FtTCPAssembler := tTCPAssembler.create (PseudoHdr, FtParseMaster.RemainingPayloadCopy);
        end
     else
         FtTCPAssembler := tTCPAssembler.create (PseudoHdr);

{
     FtTCPAssembler.SourcePort := 1269;
     FtTCPAssembler.DestinationPort := 80;
     FtTCPAssembler.SequenceNumber := 632049139;
     FtTCPAssembler.AcknowledgeNumber := 1442553351;
     FtTCPAssembler.Flags := [tcpACK, tcpPSH];
     FtTCPAssembler.WindowSize := 64240;
     FtTCPAssembler.UrgentPointer := 0;
}
     FtTCPAssembler.CopyFromParser (FtTCPParser);

     // Assemble

     layerdata := FtTCPAssembler.GetAssembly;

     // Clean Up

     layerdata.free;                         // This crashes after CopyFromParser but not when setting by Hand without options

     FtTCPAssembler.Free;
     FtParseMaster.free;

     FtLoader.Free;
     FtLoader := nil;
end;

(* ------------------------------------------- *)

initialization
              // Register any test cases with the test runner

              RegisterTest(TesttTCPAssembler.Suite);
end.

