﻿{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit Testclass_conversationstore_tcpcheck;

interface

uses
  TestFramework,
  class_conversationstore_tcpcheck,
  definition_tcp_structures,
  definition_ip_addresses,
  class_rawframe,
  class_loader_pcapng,
  class_parsemaster;

type
{
    tTCPFlightPlanEntry = record
                                 FrameIndex    : integer;
                                 SequenceNo    : longword;
                                 AckNo         : longword;
                                 Windowsize    : word;
                                 Flags         : tTCPFlagSet;
                           end;
     tTCPFlightPlan = array of tTCPFlightPlanEntry;
}
     // create plans like this:
     // tshark -r "HTTP Sample.pcapng" -Tfields -Eseparator=, -e frame.number -e tcp.seq -e tcp.ack -e tcp.window_size_value -e tcp.flags.str > "http sample.plan"

     // Test methods for class tConversationTCPChecker

    TesttConversationTCPChecker = class (TTestCase)
                                        strict private
                                                      TCPChecker : tConversationTCPChecker;
                                                      Loader     : tPCAPngLoader;

//                                                    function LoadFlightPlan (const Filename : string) : tTCPFlightPlan;

                                                      function getState (const Filename : string) : tTCPBasicConversationState;
//                                                    function getStateWithPlanChecks (const Filename : string) : tTCPBasicConversationState;
                                        public
                                              procedure SetUp; override;
                                              procedure TearDown; override;

                                        published
                                                 procedure TestHTTPSample;
                                                 procedure TestSocketRefused1;
                                                 procedure TestSingleSidedFIN;
                                                 procedure TestSlicedLocalCapture;
                                                 procedure TestTripleSYNRST;
                                                 procedure TestHalfOpenForgedReset;
                                                 procedure TestSocketCloseInactive;
                                                 procedure TestConnectionAfterMultiSYNReset;
                                                 procedure TestConnectionWithSequenceWrap;
                                                 procedure TestForgedResetsWithFollowUp;
                                                 procedure TestSYNWithPayload;
                                                 procedure TestVLANDuplicates;
                                                 procedure TestTrueDuplicates;
                                                 procedure TestHandshakeThirdFIN;
                                                 procedure TestPortReusedGoodSample;
                                                 procedure TestPortReuseFailAndWorks;
                                                 procedure TestMultiPortDuplicates;
                                                 procedure TestMultiPortFDXExample;
                                                 procedure TestZeroSYNSequence;
                                                 procedure TestHalfOpenTimestampReset;
                                                 procedure TestMSSViolationAndTrueCRCErrors;
                                                 procedure TestYoutubeResetParty;
                                                 procedure TestMegalodon;

                                  end;

implementation


uses SysUtils,
     Classes,
     Toolkit,
     base_structures,
     definition_symptoms,
     definition_5tuple,
     class_tcpconversation_manager;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.SetUp;

begin
     TCPChecker := tConversationTCPChecker.Create;
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TearDown;

begin
     TCPChecker.Free;
     TCPChecker := nil;
end;

(* -------------------------------------- *)
{
function TesttConversationTCPChecker.LoadFlightPlan (const Filename: string): tTCPFlightPlan;

var help  : string;
    lines : tStringlist;
    line  : string;
    entry : tTCPFlightPlanEntry;
    plan  : tTCPFlightPlan;
    i     : integer;
    dummy : integer;

begin
     help := changeFileExt (Filename, '.plan');

     CheckTrue (FileExists (help), 'Loading Flightplan for "' + Filename + '" failed');

     if fileExists (help) then
        begin
             lines := tStringlist.Create;
             lines.LoadFromFile (help);
             setLength (plan, lines.Count);

             // Parse lines into entries

             for i := 1 to lines.Count do
                 begin
                      line := lines[i-1];

                      help := getEntry (line, ',', 1);
                      val (help, entry.FrameIndex, dummy);

                      help := getEntry (line, ',', 2);
                      val (help, entry.SequenceNo, dummy);

                      help := getEntry (line, ',', 3);
                      val (help, entry.AckNo, dummy);

                      help := getEntry (line, ',', 4);
                      val (help, entry.Windowsize, dummy);

                      help := getEntry (line, ',', 5);
                      help := stringReplace (help, '·', TCPFlagSetPlaceHolder, [rfReplaceAll]);

                      if length (help) = 12 then
                         help := copy (help, 4, 9);

                      entry.Flags := TCPDumpToTCPFlagSet (help);

                      plan[i-1] := entry;
                 end;

             lines.Free;
        end
     else
         setLength (plan, 0);

     result := plan;
end;
}
(* -------------------------------------- *)

function TesttConversationTCPChecker.getState (const Filename: string) : tTCPBasicConversationState;

var Frame    : tRawFrame;
    Location : tFileFrameLocation;
    i        : integer;

begin
     loader := tPCAPngLoader.create (Filename);

     CheckTrue (Loader.openFile, 'Opening file "' + Filename + '" failed');
     Loader.readTraceFileheader;

     repeat
           Loader.readBatchFromDisk;

           for i := 1 to Loader.FramesAvailable do
               begin
                    Frame := Loader.getRawFrame(i);

                    Location.initialize (0, Frame.FileFrameID, Frame.FileOffset);

                    TCPChecker.AddFrame (Frame);

                    Frame.free;
               end;
     until Loader.FramesAvailable = 0;

     Loader.Free;
     Loader := nil;

     TCPChecker.FinalizeAnalysis;

     // Check result

     result := TCPChecker.LastState;
end;

(* -------------------------------------- *)
{
function TesttConversationTCPChecker.getStateWithPlanChecks (const Filename : string): tTCPBasicConversationState;

var Frame    : tRawFrame;
    Location : tFileFrameLocation;
    plan     : tTCPFlightPlan;
    state    : tTCPBasicConversationState;
    i        : integer;

begin
     // Load the flight plan (containing sequence numbers per frame)

     plan := LoadFlightPlan (Filename);

     // Load the pcapng file

     loader := tPCAPngLoader.create (Filename);

     CheckTrue (Loader.openFile, 'Opening file "' + Filename + '" failed');
     Loader.readTraceFileheader;
     Loader.readBatchFromDisk;

     for i := 1 to Loader.FramesAvailable do
         begin
              Frame := Loader.getRawFrame(i);

              CheckEquals (plan[i-1].FrameIndex, Frame.FileFrameID, 'Frame Index mismatch');

              // check if the resulting state matches the flight plan

              if i > 1 then
                 begin
                      State := TCPChecker.LastState;

                      if State.MostRecentFromClient then
                         CheckEquals (plan[i-1].SequenceNo, State.NextExpectedClientSeq.Value, 'Next Expected Client Seq mismatch')
                      else
                          CheckEquals (plan[i-1].SequenceNo, State.NextExpectedServerSeq.Value, 'Next Expected Server Seq mismatch');
                 end;

              Location.initialize (0, Frame.FileFrameID, Frame.FileOffset);

              TCPChecker.AddFrame (Frame, Location);

              Frame.free;
         end;

     Loader.Free;
     Loader := nil;

     // Check result

     result := TCPChecker.LastState;
end;
}
(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestHTTPSample;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\HTTP Sample.pcapng');

     // Final state checks

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdFINACked, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdFINACked, 'Teardown Server State');
     CheckTrue (state.Reset = rsNone, 'Reset State');
     CheckTrue (state.ClientTCPSYNOptions = [tcpNOP, tcpMSS, tcpScaleFactor, tcpSACKPermitted], 'Client TCP SYN Options');
     CheckTrue (state.ServerTCPSYNOptions = [tcpNOP, tcpMSS, tcpScaleFactor, tcpSACKPermitted], 'Server TCP SYN Options');
     CheckEquals (17456000, state.InitialRTTns, 'Initial RTT');
     CheckEquals (19, state.TotalPackets, 'Total Packets');
     CheckEquals (9, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (9, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckTrue (state.Location = slClientLocal, 'Capture Location');
     CheckTrue (state.Symptoms = [], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (100.0, state.GapRatio, 'GapRatio');
     CheckEquals (0, state.SymptomScore, 'SymptomScore');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestSocketRefused1;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\TCP - Socket refused by Server.pcapng');

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdNone, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdFINACked, 'Teardown Server State');
     CheckTrue (state.Reset = rsClient, 'Reset State');
     CheckTrue (state.ClientTCPSYNOptions = [tcpNOP, tcpMSS, tcpScaleFactor, tcpSACKPermitted], 'Client TCP SYN Options');
     CheckTrue (state.ServerTCPSYNOptions = [tcpNOP, tcpMSS, tcpScaleFactor, tcpSACKPermitted], 'Server TCP SYN Options');
     CheckEquals (33267000, state.InitialRTTns, 'Initial RTT');
     CheckEquals (6, state.TotalPackets, 'Total Packets');
     CheckEquals (3, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (2, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckTrue (state.Location = slSPANClient, 'Capture Location');
     CheckTrue (state.Symptoms = [tsSocketRefused, tsOneSidedServerFIN], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (100.0, state.GapRatio, 'GapRatio');
     CheckEquals (60, state.SymptomScore, 'SymptomScore');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestSingleSidedFIN;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\TraceWrangler\FIN and RST analysis FW tcp 58049.pcapng');

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdFINAcked, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdNone, 'Teardown Server State');
     CheckTrue (state.Reset = rsNone, 'Reset State');
     CheckTrue (state.ClientTCPSYNOptions = [tcpEOL, tcpMSS, tcpScaleFactor, tcpSACKPermitted, tcpTimestamp], 'Client TCP SYN Options');
     CheckTrue (state.ServerTCPSYNOptions = [tcpEOL, tcpMSS, tcpNOP, tcpSACKPermitted, tcpTimestamp], 'Server TCP SYN Options');
     CheckEquals (32, state.ClientFINCount, 'Client FIN count');
     CheckEquals (0, state.ServerFINCount, 'Server FIN count');
     CheckEquals (545, state.TotalPackets, 'Total Packets');
     CheckEquals (28338000, state.InitialRTTns, 'Initial RTT');
     CheckEquals (171, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (341, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckTrue (state.Location = slSPAN, 'Capture Location');
     CheckTrue (state.Symptoms = [tsIPAllowsFragmentation, tsNoServerScaleOption, tsClientFINRetransmit, tsOneSidedClientFIN], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (94, state.GapRatio, 'GapRatio');
     CheckEquals (55, state.SymptomScore, 'SymptomScore');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestSlicedLocalCapture;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\SlicedTest-WiztowerLocal.pcapng');

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdFINACked, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdFINACked, 'Teardown Server State');
     CheckTrue (state.Reset = rsNone, 'Reset State');
     CheckTrue (state.ClientTCPSYNOptions = [tcpMSS, tcpScaleFactor, tcpNOP, tcpOptionsTruncated], 'Client TCP SYN Options');
     CheckTrue (state.ServerTCPSYNOptions = [tcpMSS, tcpScaleFactor, tcpNOP, tcpOptionsTruncated], 'Server TCP SYN Options');
     CheckEquals (1, state.ClientFINCount, 'Client FIN count');
     CheckEquals (1, state.ServerFINCount, 'Server FIN count');
     CheckEquals (12, state.TotalPackets, 'Total Packets');
     CheckEquals (23476000, state.InitialRTTns, 'Initial RTT');
     CheckEquals (6, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (5, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckTrue (state.Location = slClientLocal, 'Capture Location');
     CheckTrue (state.Symptoms = [], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (0, state.SymptomScore, 'SymptomScore');
     CheckEquals (100, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestTripleSYNRST;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\TCP - triple SYN RST.pcapng');

     CheckTrue (state.Handshake = hsSYNReset, 'Handshake State');
     CheckTrue (state.TeardownClient = tdNone, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdNone, 'Teardown Server State');
     CheckTrue (state.Reset = rsServer, 'Reset State');
     CheckTrue (state.ClientTCPSYNOptions = [tcpNOP, tcpMSS, tcpScaleFactor, tcpSACKPermitted], 'Client TCP SYN Options');
     CheckTrue (state.ServerTCPSYNOptions = [], 'Server TCP SYN Options');
     CheckEquals (6, state.TotalPackets, 'Total Packets');
     CheckEquals (0, state.InitialRTTns, 'Initial RTT');
     CheckEquals (0, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (0, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (3, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slSPANServer, 'Capture Location');
     CheckTrue (state.Symptoms = [tsSYNRetransmit, tsMultiTCPClientSYNOptionSets], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (35, state.SymptomScore, 'SymptomScore');
     CheckEquals (0, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestHalfOpenForgedReset;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\TCP - Half Open and Reset with different TTL.pcapng');

     CheckTrue (state.Handshake = hsHalfOpenReset, 'Handshake State');
     CheckTrue (state.TeardownClient = tdNone, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdNone, 'Teardown Server State');
     CheckEquals (3, state.TotalPackets, 'Total Packets');
     CheckEquals (0, state.InitialRTTns, 'Initial RTT');
     CheckEquals (1, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (1, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slSPAN, 'Capture Location');
     CheckTrue (state.Symptoms = [tsIPAllowsFragmentation, tsNoClientMSSOption, tsNoServerScaleOption, tsNoClientScaleOption, tsZeroServerWindow, tsForgedReset, tsMultiClientTTL, tsZeroWindowOnSYNACK, tsServerUsesMinMSS], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (255, state.SymptomScore, 'SymptomScore');

     CheckEquals (100, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestSocketCloseInactive;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\TCP - Server Socket close due to Client inactivity after Handshake.pcapng');

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdFINAcked, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdFINAcked, 'Teardown Server State');
     CheckEquals (7, state.TotalPackets, 'Total Packets');
     CheckEquals (50777000, state.InitialRTTns, 'Initial RTT');
     CheckEquals (3, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (3, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slSPANServerLocation, 'Capture Location');
     CheckTrue (state.Symptoms = [tsClosedInactive], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (25, state.SymptomScore, 'SymptomScore');
     CheckEquals (100, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestConnectionAfterMultiSYNReset;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\TCP - Session established after multi SYN reject.pcapng');

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdFINAcked, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdNone, 'Teardown Server State');
     CheckEquals (41, state.TotalPackets, 'Total Packets');
     CheckEquals (558000, state.InitialRTTns, 'Initial RTT');
     CheckEquals (15, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (20, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (2, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slSPANServer, 'Capture Location');
     CheckTrue (state.Symptoms = [tsSYNRetransmit, tsNoServerScaleOption, tsOneSidedClientFIN, tsEstablishAfterRefuse, tsMultiTCPClientSYNOptionSets, tsCumbersomeHandshake], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (130, state.SymptomScore, 'SymptomScore');
     CheckEquals (88, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestConnectionWithSequenceWrap;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\TCP - Sequence Wrapped.pcapng');

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdNone, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdNone, 'Teardown Server State');
     CheckTrue (state.Reset = rsClient, 'Reset State');
     CheckEquals (142, state.TotalPackets, 'Total Packets');
     CheckEquals (4372000, state.InitialRTTns, 'Initial RTT');
     CheckEquals (59, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (82, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slSPAN, 'Capture Location');
     CheckTrue (state.Symptoms = [tsIPAllowsFragmentation, tsTCPSequenceWrapped], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (15, state.SymptomScore, 'SymptomScore');
     CheckEquals (100, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestForgedResetsWithFollowUp;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\Security\TCP - Forged Resets (probably).pcapng');

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdNone, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdNone, 'Teardown Server State');
     CheckEquals (9, state.TotalPackets, 'Total Packets');
     CheckEquals (11529000, state.InitialRTTns, 'Initial RTT');
     CheckEquals (2, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (3, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slClientLocal, 'Capture Location');
     CheckTrue (state.Symptoms = [tsSYNACKRetransmit, tsMultiServerTTL, tsRoutingDuplicate, tsPacketsAfterReset], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (46, state.SymptomScore, 'SymptomScore');
     CheckEquals (63, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestSYNWithPayload;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\TCP - SYN packet carrying data.pcapng');

     CheckEquals (state.InitialSequenceNo.Value, 677738411, 'Initial Sequence Number');
     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdNone, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdNone, 'Teardown Server State');
     CheckTrue (state.Reset = rsClient, 'Reset State');
     CheckEquals (7, state.TotalPackets, 'Total Packets');
     CheckEquals (0, state.InitialRTTns, 'Initial RTT');
     CheckEquals (3, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (2, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slSPANClientServer, 'Capture Location');
     CheckTrue (state.Symptoms = [tsIPAllowsFragmentation, tsSYNACKnotMatchingSYN, tsSYNFlagWithPayload, tsUnseenPacket, tsHandshakeCompleteDetermined, tsCumbersomeHandshake], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (71, state.SymptomScore, 'SymptomScore');
     CheckEquals (83, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestVLANDuplicates;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\TCP - Routing Duplicates on different VLANs.pcapng');

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdFINAcked, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdFINAcked, 'Teardown Server State');
     CheckEquals (74, state.TotalPackets, 'Total Packets');
     CheckEquals (7095100, state.InitialRTTns, 'Initial RTT');
     CheckEquals (26, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (47, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slSPANClient, 'Capture Location');
     CheckTrue (state.Symptoms = [tsNoServerScaleOption, tsMultiServerTTL, tsSegmentExceedsMSS], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (60, state.SymptomScore, 'SymptomScore');
     CheckEquals (100, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestTrueDuplicates;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\TCP - True Duplicates on same VLAN.pcapng');

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdNone, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdNone, 'Teardown Server State');
     CheckEquals (30, state.TotalPackets, 'Total Packets');
     CheckEquals (1299600, state.InitialRTTns, 'Initial RTT');
     CheckEquals (8, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (7, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slSPANClientServer, 'Capture Location');
     CheckTrue (state.Symptoms = [tsFrameDuplicate, tsThirdACKWithPayload], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (1, state.SymptomScore, 'SymptomScore');
     CheckEquals (52, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestHandshakeThirdFIN;

var state : tTCPBasicConversationState;

begin
     // Tricky trace... out of order arrival of third ACK

     state := getState ('P:\Traces\Interesting\TCP - FIN as part of the handshake.pcapng');

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdFINAcked, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdFINAcked, 'Teardown Server State');
     CheckEquals (7, state.TotalPackets, 'Total Packets');
     CheckEquals (29752800, state.InitialRTTns, 'Initial RTT');
     CheckEquals (1, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (3, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slSPAN, 'Capture Location');
     CheckTrue (state.Symptoms = [tsHandshakeOutOfOrder, tsClientAbort], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (51, state.SymptomScore, 'SymptomScore');
     CheckEquals (67, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestPortReusedGoodSample;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\TCP - Port reused without problem.pcapng');

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdFINAcked, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdNone, 'Teardown Server State');
     CheckTrue (state.Reset = rsClient, 'Reset State');
     CheckEquals (36, state.TotalPackets, 'Total Packets');
     CheckEquals (13236000, state.InitialRTTns, 'Initial RTT');
     CheckEquals (14, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (21, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slSPANClient, 'Capture Location');
     CheckTrue (state.Symptoms = [tsSocketsReused, tsOneSidedClientFIN], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (20, state.SymptomScore, 'SymptomScore');
     CheckEquals (100, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestPortReuseFailAndWorks;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\TCP - Port reused two times, once too fast, second works.pcapng');

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdFINAcked, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdFINAcked, 'Teardown Server State');
     CheckEquals (56, state.TotalPackets, 'Total Packets');
     CheckEquals (35801400, state.InitialRTTns, 'Initial RTT');
     CheckEquals (24, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (31, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slSPAN, 'Capture Location');
     CheckTrue (state.Symptoms = [tsSACKEdgesSeen], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (10, state.SymptomScore, 'SymptomScore');
     CheckEquals (100, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestMultiPortDuplicates;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\TCP - MultiInterfaceDuplicateTest.pcapng');

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdFINAcked, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdFINAcked, 'Teardown Server State');
     CheckEquals (19, state.TotalPackets, 'Total Packets');
     CheckEquals (8576000, state.InitialRTTns, 'Initial RTT');
     CheckEquals (9, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (9, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slClientLocal, 'Capture Location');
     CheckTrue (state.Symptoms = [], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (0, state.SymptomScore, 'SymptomScore');
     CheckEquals (100, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestMultiPortFDXExample;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\TCP - MultiInterfaceFDXTAPTest.pcapng');

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdFINAcked, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdFINAcked, 'Teardown Server State');
     CheckEquals (12, state.TotalPackets, 'Total Packets');
     CheckEquals (26829271, state.InitialRTTns, 'Initial RTT');
     CheckEquals (6, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (5, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slFDXTAPClient, 'Capture Location');
     CheckTrue (state.Symptoms = [], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (0, state.SymptomScore, 'SymptomScore');
     CheckEquals (100, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestZeroSYNSequence;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\TCP - SYN packet with 2 byte trailer plus port number reused.pcapng');

     CheckTrue (state.Handshake = hsSYNSent, 'Handshake State');
     CheckTrue (state.TeardownClient = tdNone, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdNone, 'Teardown Server State');
     CheckTrue (state.Reset = rsClient, 'Reset State');
     CheckEquals (5, state.TotalPackets, 'Total Packets');
     CheckEquals (0, state.InitialRTTns, 'Initial RTT');
     CheckEquals (0, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (0, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slSPANClient, 'Capture Location');
     CheckTrue (state.Symptoms = [tsIPAllowsFragmentation, tsSYNRetransmit, tsNoClientScaleOption, tsUrgentPointerWithoutFlag, tsZeroISN], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (95, state.SymptomScore, 'SymptomScore');
     CheckEquals (0, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestHalfOpenTimestampReset;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\ASK Cases\TCP - SYN SYN-ACK RST by Timestamp Error.pcapng');

     CheckTrue (state.Handshake = hsHalfOpenReset, 'Handshake State');
     CheckTrue (state.TeardownClient = tdNone, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdNone, 'Teardown Server State');
     CheckTrue (state.Reset = rsClient, 'Reset State');
     CheckEquals (12, state.TotalPackets, 'Total Packets');
     CheckEquals (0, state.InitialRTTns, 'Initial RTT');
     CheckEquals (4, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (1, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slClientLocal, 'Capture Location');
     CheckTrue (state.Symptoms = [tsNoServerScaleOption, tsPortAndISNReusedAfterReset, tsTimestampValueMismatch, tsHandshakeFailByTimestamp], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (150, state.SymptomScore, 'SymptomScore');
     CheckEquals (45, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestMSSViolationAndTrueCRCErrors;

var state : tTCPBasicConversationState;

begin
     state := getState ('D:\Development\Jays Utilities\Trace Wrangler\test\traces\TCP-MSSViolation.pcapng');

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdNone, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdNone, 'Teardown Server State');
     CheckTrue (state.Reset = rsNone, 'Reset State');
     CheckEquals (1210, state.TotalPackets, 'Total Packets');
     CheckEquals (1735000, state.InitialRTTns, 'Initial RTT');
     CheckEquals (151, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (908, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slClientLocal, 'Capture Location');
//   CheckTrue (state.Symptoms = [tsNoServerScaleOption, tsServerUsesMinMSS, tsSegmentExceedsMSS, tsTrueCRCErrors], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));        // there are true CRCs, but diagnosis has to wait for 2nd pass
     CheckTrue (state.Symptoms = [tsNoServerScaleOption, tsServerUsesMinMSS, tsSegmentExceedsMSS], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (60, state.SymptomScore, 'SymptomScore');
     CheckEquals (88, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestYoutubeResetParty;

var state : tTCPBasicConversationState;

begin
     state := getState ('P:\Traces\Interesting\TCP - Reset Party - Youtube STFU!.pcapng');

     CheckTrue (state.Handshake = hsComplete, 'Handshake State');
     CheckTrue (state.TeardownClient = tdNone, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdNone, 'Teardown Server State');
     CheckTrue (state.Reset = rsClient, 'Reset State');
     CheckEquals (61, state.TotalPackets, 'Total Packets');
     CheckEquals (10031000, state.InitialRTTns, 'Initial RTT');
     CheckEquals (28, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (32, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slSPANClient, 'Capture Location');
     CheckTrue (state.Symptoms = [tsFrameDuplicate, tsServerPacketsAfterRST], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (26, state.SymptomScore, 'SymptomScore');
     CheckEquals (100, state.GapRatio, 'GapRatio');
end;

(* -------------------------------------- *)

procedure TesttConversationTCPChecker.TestMegalodon;

var state : tTCPBasicConversationState;
{
    i     : integer;
    flow  : tTCPFlow;
    j     : integer;
}

begin
     state := getState ('D:\Traces\Megalodon\Content Server no Firewall.pcapng');

     CheckEquals (23050, TCPChecker.FlowCount, 'Total Flows identified');

     CheckTrue (state.Handshake = hsNone, 'Handshake State');
     CheckTrue (state.TeardownClient = tdNone, 'Teardown Client State');
     CheckTrue (state.TeardownServer = tdNone, 'Teardown Server State');
     CheckTrue (state.Reset = rsNone, 'Reset State');
     CheckEquals (353558, state.TotalPackets, 'Total Packets');
     CheckEquals (0, state.InitialRTTns, 'Initial RTT');
     CheckEquals (141432, state.ClientSequenceMatch, 'Client Sequence Match');
     CheckEquals (212066, state.ServerSequenceMatch, 'Server Sequence Match');
     CheckEquals (0, state.SYNResetCount, 'SYNReset Count');
     CheckTrue (state.Location = slSPANServer, 'Capture Location');
     CheckTrue (state.Symptoms = [tsSACKEdgesSeen], 'Symptoms found: ' + TCPSymptomSetToString (state.Symptoms));
     CheckEquals (10, state.SymptomScore, 'SymptomScore');
     CheckEquals (100, state.GapRatio, 'GapRatio');
{
     j := 0;

     flow := TCPChecker.GetFlowByAbsoluteIndex(16);

     for i := 1 to TCPChecker.FlowCount do
         begin
              flow := TCPChecker.GetFlowByAbsoluteIndex(i);


              if flow.BasicState.SymptomScore > j then
                 j := flow.BasicState.SymptomScore;
         end;
}
end;

(* -------------------------------------- *)

initialization
              // Register any test cases with the test runner

              RegisterTest(TesttConversationTCPChecker.Suite);
end.

