{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}
 
unit Testclass_definition_ip_addresses;
{

  Delphi DUnit Test Case
  ----------------------
  This unit contains a skeleton test case class generated by the Test Case Wizard.
  Modify the generated code to correctly setup and call the methods from the unit 
  being tested.

}

interface

uses
  TestFramework, Types, definition_ip_addresses;

type
  // Test methods for class

  TestAddresses = class (TTestCase)

                        strict private

                        public

                        published
                                 procedure TestisValidIPv6String;
                                 procedure TestIPV6Address;
                                 procedure TestCheckRoutines;
                  end;

implementation

uses Classes,
     SysUtils,
     Toolkit;

(* -------------------------------------- *)
{ TestAddresses }
(* -------------------------------------- *)

procedure TestAddresses.TestisValidIPv6String;

var ReturnValue : Boolean;
    testlist    : tStringlist;
    ip          : string;
    mustfail    : boolean;
    msg         : string;
    i           : integer;

begin
     testlist := tStringlist.Create;
     testlist.LoadFromFile('D:\Software Development\Jays Utilities\Trace Wrangler\test\ipv6addresstest.txt');

     for  i := 1 to testlist.Count do
          begin
               if pos ('ipv6test(', testlist[i-1]) = 1 then
                  begin
                       ip := getEntry (testlist[i-1], '"', 2);;

                       mustfail := pos ('ipv6test(!1,"', testlist[i-1]) = 1;

                       if mustfail then
                          msg := 'Testcase in line #' + IntToStr (i) + ': "' + ip + '" should fail'
                       else
                           msg := 'Testcase in line #' + IntToStr (i) + ': "' + ip + '" should work';

                       CheckEquals (not mustfail, isValidIPv6String (ip), msg);
                  end;
          end;
end;

(* -------------------------------------- *)

procedure TestAddresses.TestIPV6Address;

var Address1 : tIPv6Address;
    Address2 : tIPv6Address;
    Address3 : tIPv6Address;
    MACAdr   : tMACAddress;

begin
     Address1.initialize(false);

     CheckEquals ('::', Address1.ValueAsString, 'Zero Address');
     CheckEquals (128, Address1.MaskBitCount, 'Mask Bits should be 128');

     Address1.setValueFromString ('2001:470:b45e:30::10');
     CheckEquals ('2001:470:b45e:30::10', Address1.ValueAsString, 'Address should be 2001:470:b45e:30::10');
     CheckEquals ('2001:470:b45e:30:0:0:0:10', Address1.getExpandedValue, 'Address should be 2001:470:b45e:30:0:0:0:10');
     CheckEquals ('ff02::1:ff00:10', Address1.getSolicitedNodeAddress.ValueAsString, 'Solicited Node MCast should be ff02::1:ff00:10');
     CheckFalse (Address1.IsEUI64Address, 'Is not a EUI64 address');

     Address1.setValueFromString ('2001:470:b45e:30:1234:5678:9abc:def1');
     CheckEquals ('2001:470:b45e:30:1234:5678:9abc:def1', Address1.ValueAsString, 'Address should be 2001:470:b45e:30:1234:5678:9abc:def1');
     CheckEquals ('ff02::1:ffbc:def1', Address1.getSolicitedNodeAddress.ValueAsString, 'Solicited Node MCast should be ff02::1:ff78:abcd');

     Address2.initialize (true);
     Address2.setValueFromString('2001:db8::');
     CheckEquals ('2001:db8::', Address2.ValueAsString, 'Address should be 2001:db8::');

     Address1.CopyPrefixFrom (Address2, 24);
     CheckEquals ('2001:d70:b45e:30:1234:5678:9abc:def1', Address1.ValueAsString, 'Address should be 2001:d70:b45e:30:1234:5678:9abc:def1');

     MacAdr.setfromString ('00:12:7f:eb:6b:40', ':');
     Address3.setValueFromString('fe80::');

     Address2.setValueFromMAC (Address3, MacAdr);
     CheckEquals ('fe80::212:7fff:feeb:6b40', Address2.ValueAsString, 'EUI 64 Address set correctly');
     CheckTrue (Address2.IsEUI64Address);

     MacAdr := Address2.getEUI64baseMAC;
     CheckEquals ('00:12:7f:eb:6b:40', MacAdr.asString(':', true), 'MAC Address of EUI64 IP');

end;

(* -------------------------------------- *)

procedure TestAddresses.TestCheckRoutines;

var help : string;

begin
     help := '192.168.0.1:80';
     CheckTrue (isValidIPv4Socket (help), 'Valid IPv4 Socket');
     help := '192.168.0.1.80';
     CheckFalse (isValidIPv4Socket (help), 'Not a valid IPv4 Socket');
     help := '192.168.0.1:80:80';
     CheckFalse (isValidIPv4Socket (help), 'Not a valid IPv4 Socket');

     help := '[2001:470:b45e:30::10]:80';
     CheckTrue (isValidIPv6Socket (help), 'Valid IPv6 Socket');
     help := '2001:470:b45e:30::10]:80';
     CheckFalse (isValidIPv6Socket (help), 'Not a valid IPv6 Socket');
     help := '2001:470:b45e:30::10:80';
     CheckFalse (isValidIPv6Socket (help), 'Not a valid IPv6 Socket');
     help := '[2001:470:b45e:30::10:80';
     CheckFalse (isValidIPv6Socket (help), 'Not a valid IPv6 Socket');
end;

(* -------------------------------------- *)

initialization
              // Register any test cases with the test runner

              RegisterTest(TestAddresses.Suite);
end.

