{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}
 
unit Testclass_loader_general;
{

  Delphi DUnit Test Case
  ----------------------
  This unit contains a skeleton test case class generated by the Test Case Wizard.
  Modify the generated code to correctly setup and call the methods from the unit
  being tested.

}

// {$define createfile}

interface

uses
  TestFramework, class_interfacelist, base_structures, class_interfacestatistics,
  definition_simple_nameresolution, pcapng_structures, class_traceloader,
  class_rawframe, definition_ip_addresses, class_loader_pcapng, class_dirparser;

type tTraceCharacteristics = record
                                   Filename           : string;
                                   Interfacecount     : integer;
                                   FirstInterfaceName : string;
                                   Framecount         : int64;
                                   FirstFrameTime     : tFrameTimestamp;
                                   FirstFrameMD5      : string;
                                   LastFrameTime      : tFrameTimestamp;
                                   LastFrameMD5       : string;

                                   class operator equal (a, b : tTraceCharacteristics) : boolean;

                                   procedure initialize (const NewFilename : string);
                                   procedure setFromCSV (const csvline : string);
                                   function asCSV : string;
                             end;

     // Test methods for class tPCAPNGLoader

     TestLoaderGeneral = class (TTestCase)

                               strict private

                                      function getFileCharacteristics (const Filename : string) : tTraceCharacteristics;


                               public

                               published
                                        procedure VerifyTestFiles;
                         end;

implementation

uses SysUtils,
     classes,
     toolkit,
     tracefile_utils;

(* ------------------------------------------- *)
{ tTraceCharacteristics }
(* ------------------------------------------- *)

class operator tTraceCharacteristics.equal (a,  b: tTraceCharacteristics): boolean;

begin
     result := (lowercase (a.Filename) = lowercase (b.Filename)) and
               (a.Interfacecount = b.Interfacecount) and
               (a.FirstInterfaceName = b.FirstInterfaceName) and
               (a.Framecount = b.Framecount) and
               (a.FirstFrameTime = b.FirstFrameTime) and
               (a.FirstFrameMD5 = b.FirstFrameMD5) and
               (a.LastFrameTime = b.LastFrameTime) and
               (a.LastFrameMD5 = b.LastFrameMD5);
end;

(* ------------------------------------------- *)

procedure tTraceCharacteristics.initialize (const NewFilename : string);

begin
     Filename := NewFilename;
     Interfacecount := 0;
     FirstInterfaceName := '';
     FrameCount := 0;
     FirstFrameTime.initialize;
     FirstFrameMD5 := '';
     LastFrameTime.initialize;
     LastFrameMD5 := '';
end;

(* ------------------------------------------- *)

procedure tTraceCharacteristics.setFromCSV(const csvline: string);

var help  : string;
    dummy : integer;

begin
     help := getEntry (csvline, ',', 1);
     filename := help;

     help := getEntry (csvline, ',', 2);
     val (help, InterfaceCount, dummy);

     FirstInterfaceName := getEntry (csvline, ',' , 3);

     help := getEntry (csvline, ',', 4);
     val (help, FrameCount, dummy);

     help := getEntry (csvline, ',', 5);
     FirstFrameTime.setFromString (help);

     FirstFrameMD5 := getEntry (csvline, ',', 6);

     help := getEntry (csvline, ',', 7);
     LastFrameTime.setFromString (help);

     LastFrameMD5 := getEntry (csvline, ',', 8);
end;

(* ------------------------------------------- *)

function tTraceCharacteristics.asCSV: string;

begin
     result := filename + ',' + IntToStr (Interfacecount) + ',' + FirstInterfaceName + ',' + IntToStr (FrameCount) + ',' + FirstFrameTime.asFullString + ',' + FirstFrameMD5 + ',' + LastFrameTime.asFullString + ',' + LastFrameMD5;
end;

(* ------------------------------------------- *)

function TestLoaderGeneral.getFileCharacteristics (const Filename: string): tTraceCharacteristics;

var loader : tTraceloader;
    iface  : tCaptureInterface;
    frame  : tRawFrame;
    value  : tTraceCharacteristics;
    index  : int64;
    i      : Integer;

begin
     value.initialize (Filename);

     loader := createTraceLoader (value.Filename);
     loader.openFile;
     loader.readTraceFileheader;

     iface := loader.CaptureInterfaces.getInterfaceByIndex(0);
     value.FirstInterfaceName := iface.Name;

     value.Interfacecount := loader.CaptureInterfaces.Count;

     loader.readBatchFromDisk;

     index := 0;

     while loader.FramesAvailable > 0 do
           begin
                for i := 1 to loader.FramesAvailable do
                    begin
                         inc (index);

                         frame := loader.getRawFrame (i);

                         if index = 1 then
                            begin
                                 value.FirstFrameTime := frame.Timestamp;
                                 value.FirstFrameMD5 := frame.MD5Hash;
                            end;

                         if loader.EOF and (i = loader.FramesAvailable) then
                            begin
                                 value.LastFrameTime := frame.Timestamp;
                                 value.LastFrameMD5 := frame.MD5Hash;
                            end;

                         frame.free;
                         frame := nil;
                    end;

                loader.readBatchFromDisk;
           end;

     value.Framecount := index;

     loader.closeFile;
     loader.Free;
     loader := nil;

     result := value;
end;

(* ------------------------------------------- *)

procedure TestLoaderGeneral.VerifyTestFiles;

var dirparser : tDirParser;
    filename  : string;
    testentry : tTraceCharacteristics;
    expected  : tTraceCharacteristics;
    testlines : tStringList;
    i         : Integer;

begin
{$ifdef createfile}
     // Create Testresults file

     dirparser := tDirParser.create;
     dirparser.BaseDirectory := 'D:\Traces\LoaderTest\';

     dirparser.execute;

     testlines := tStringlist.Create;

     for i := 1 to dirparser.FileCount do
         begin
              filename := dirparser.getFileName (i);

              testentry := getFileCharacteristics (filename);

              testlines.Add (testentry.asCSV);
         end;

     testlines.SaveToFile ('D:\Traces\loadertest.csv');

     testlines.Free;

{$else}

     // Verify Testresults

     testlines := tStringList.Create;
     testlines.LoadFromFile ('D:\Traces\loadertest.csv');

     for i := 1 to testlines.Count do
         begin
              expected.setFromCSV (testlines[i-1]);

              testentry := getFileCharacteristics (expected.Filename);

              CheckEquals (expected.Filename, testentry.Filename, 'Filename');
              CheckEquals (expected.Interfacecount, testentry.Interfacecount, 'Interface Count');
              CheckEquals (expected.Framecount, testentry.Framecount, 'Frame Count');
              CheckEquals (expected.FirstFrameTime.asFullString, testentry.FirstFrameTime.asFullString, 'Filename');
              CheckEquals (expected.FirstFrameMD5, testentry.FirstFrameMD5, 'Filename');
              CheckEquals (expected.LastFrameTime.asFullString, testentry.LastFrameTime.asFullString, 'Filename');
              CheckEquals (expected.LastFrameMD5, testentry.LastFrameMD5, 'Filename');
         end;
{$endif}
end;

(* ------------------------------------------- *)

initialization
              // Register any test cases with the test runner

              RegisterTest(TestLoaderGeneral.Suite);
end.

