{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}
 
unit Testclass_loader_naicap;
{

  Delphi DUnit Test Case
  ----------------------
  This unit contains a skeleton test case class generated by the Test Case Wizard.
  Modify the generated code to correctly setup and call the methods from the unit 
  being tested.

}

interface

uses
  TestFramework, class_rawframe, class_traceloader, class_loader_naicap,
  base_structures, class_DirParser, class_tracefilelist;

type
  // Test methods for class tCAPLoader

  TesttCAPLoader = class(TTestCase)

                         strict private
                                       CAPLoader : tCAPLoader;
                                       DirParser : tDirParser;
                         public

                         published
                                  procedure TestMultiFileReading;
                                  procedure TestMultiFileScanning;
                                  procedure TestFrameTime;
                   end;

implementation

uses class_inteldb_manager;

(* ------------------------------------------- *)

procedure TesttCAPLoader.TestMultiFileReading;

var filename : string;
    frame    : tRawFrame;
    i,j      : Integer;

begin
     DirParser := tDirParser.create;
     DirParser.BaseDirectory := 'p:\Traces\Merlin\ICA\S6040';
     DirParser.FileMask := '*.cap';
     DirParser.IncludeSubDirs := false;

     DirParser.execute;

     for i := 1 to DirParser.FileCount do
         begin
              filename := DirParser.getFileName (i);

              CAPLoader := tCAPLoader.create (filename);

              CAPLoader.openFile;
              CAPLoader.readTraceFileheader;

              repeat
                    CAPLoader.readBatchFromDisk;

                    // Read all frames from the batch and discard them

                    for j := 1 to CAPLoader.FramesAvailable do
                        begin
                             frame := CAPLoader.getRawFrame(j);

                             frame.free;
                        end;

              until CAPLoader.EOF;

              CAPLoader.closeFile;
              CAPLoader.Free;
              CAPLoader := nil;
         end;

end;

(* ------------------------------------------- *)

procedure TesttCAPLoader.TestMultiFileScanning;

var filename      : string;
    fileentry     : tTracefileEntry;
    i,j           : Integer;
    IntelDBParams : tIntelDBParameters;

begin
     DirParser := tDirParser.create;
     DirParser.BaseDirectory := 'p:\Traces\Merlin\ICA\S6040';
     DirParser.FileMask := '*.cap';
     DirParser.IncludeSubDirs := false;

     DirParser.execute;

     for i := 1 to DirParser.FileCount do
         begin
              filename := DirParser.getFileName (i);

//            fileentry := tTracefileEntry.create (filename, IntelDBParams, 0);
              fileentry := tTracefileEntry.create (filename, 0);
         end;
end;

(* ------------------------------------------- *)

procedure TesttCAPLoader.TestFrameTime;

var frame : tRawFrame;
    i     : integer;

begin
     CAPLoader := tCAPLoader.create ('P:\Traces\Merlin\ICA\S6040\Merlin_001jqa.cap');

     CAPLoader.openFile;
     CAPLoader.readTraceFileheader;

     repeat
           CAPLoader.readBatchFromDisk;

           // Read all frames from the batch and discard them

           for i := 1 to CAPLoader.FramesAvailable do
               begin
                    frame := CAPLoader.getRawFrame(i);

                    case frame.FileFrameID of
                     1 : CheckEquals (0, frame.Timestamp.nanoseconds, 'NanoSeconds of first frame');
                     2 : CheckEquals (150693300, frame.Timestamp.nanoseconds, 'NanoSeconds of first frame');
                    end;

                    frame.free;
               end;

     until CAPLoader.EOF;

     CAPLoader.closeFile;
     CAPLoader.Free;
     CAPLoader := nil;
end;

(* ------------------------------------------- *)

initialization
              // Register any test cases with the test runner

              RegisterTest(TesttCAPLoader.Suite);
end.

