{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}
 
unit Testclass_parser_dhcpv4;
{

  Delphi DUnit Test Case
  ----------------------
  This unit contains a skeleton test case class generated by the Test Case Wizard.
  Modify the generated code to correctly setup and call the methods from the unit
  being tested.

}

interface

uses
  TestFramework, class_parser_baseclass, definition_ip_addresses, base_structures,
  class_parser_dhcpv4, class_parsemaster;

type // Test methods for class tDHCPv4Parser

     TesttDHCPv4Parser = class (TTestCase)

                               strict private

                               private
                                      Parsemaster : tParseMaster;

                                      function PrepareTest (Tracefile : string) : tDHCPv4Parser;

                               public

                               published
                                        procedure TestparseDHCPDiscover;
                                        procedure TestparseDHCPOffer;
                                        procedure TestparseDHCPRequest;
                                        procedure TestparseDHCPAcknowledge;
                                        procedure TestparseOverloadOptionWithEnds;
                         end;

implementation


uses SysUtils,
     class_rawframe,
     class_loader_pcapng;

(* ------------------------------------------- *)

function TesttDHCPv4Parser.PrepareTest(Tracefile: string) : tDHCPv4Parser;

var Loader      : tPCAPNGLoader;
    Frame       : tRawFrame;
    DHCPParser  : tDHCPv4Parser;
    OpenResult  : boolean;
    i           : integer;

begin
     Loader := tPCAPNGLoader.create (Tracefile);
     OpenResult := Loader.openFile;

     Loader.readTraceFileheader;
     Loader.readBatchFromDisk;

     Frame := Loader.getRawFrame(1);

     ParseMaster := tParseMaster.create (Frame, ppmParseAll);

     Loader.Free;
     Loader := nil;

     CheckEquals (true, Openresult, 'PCAPNGLoader.OpenFile Failed');

     i := 0;

     // Find the ICMP Parser

     repeat
           inc (i)
     until (Parsemaster.getParserTypeByIndex (i) = NEXT_HEADER_DHCPv4) or (i >= Parsemaster.ParserCount);

     if (Parsemaster.getParserTypeByIndex (i) = NEXT_HEADER_DHCPv4) then
        DHCPParser := (Parsemaster.getParserByIndex (i) as tDHCPv4Parser)
     else
         DHCPParser := nil;

     result := DHCPParser;
end;

(* ------------------------------------------- *)

procedure TesttDHCPv4Parser.TestparseDHCPDiscover;

var DHCPParser : tDHCPv4Parser;

begin
     DHCPParser := PrepareTest ('d:\Software Development\Jays Utilities\Trace Wrangler\test\traces\DHCP-Discover.pcapng');

     if DHCPParser <> nil then
        begin
             CheckTrue (DHCPParser.MessageOpCode = docBootRequest);
             CheckEquals (5, DHCPParser.PaddingSize, 'Padding Size');
             CheckEquals (300, DHCPParser.LayerSize, 'Layer Size');
             CheckEquals (1, DHCPParser.HardwareAddressType, 'Hardwaretype Ethernet');
             CheckEquals (6, DHCPParser.HardwareAddressLength, 'Hardware Address Length');
             CheckEquals (0, DHCPParser.Hops, 'Hops');
             CheckEquals ($a2a24eb9, DHCPParser.TransactionID, 'Transaction ID');
             CheckEquals (13, DHCPParser.Seconds, 'Seconds Elapsed');
             CHeckTrue (DHCPParser.Flags = dfUnicast);
             CheckEquals ('0.0.0.0', DHCPParser.ClientIPAddress.ValueAsString, 'Client IP address');
             CheckEquals ('0.0.0.0', DHCPParser.YourClientIPAddress.ValueAsString, 'Your Client IP address');
             CheckEquals ('0.0.0.0', DHCPParser.ServerIPAddress.ValueAsString, 'Server IP address');
             CheckEquals ('0.0.0.0', DHCPParser.RelayIPAddress.ValueAsString, 'Relay IP address');
             CheckEquals ('00:0E:A6:84:19:C1', DHCPParser.ClientHWAddress.asString, 'Client MAC Address');
             CheckEquals ('', DHCPParser.ServerHostName, 'Server Hostname');
             CheckEquals ('', DHCPParser.BootFileName, 'Boot file name');
             CheckEquals ('192.168.123.1', DHCPParser.DHCPRequestedIPAddress.ValueAsString, 'Requested IPAddress');
             CheckEquals ('00:0E:A6:84:19:C1', DHCPParser.DHCPClientIdentifier.asString, 'Client Identifier');
             CheckEquals ('obsidian', DHCPParser.DHCPHostName.FullFQDN, 'Hostname');
             CheckTrue (DHCPParser.DHCPMessageType = dmtDiscover);
        end;
end;

(* ------------------------------------------- *)

procedure TesttDHCPv4Parser.TestparseDHCPOffer;

var DHCPParser : tDHCPv4Parser;

begin
     DHCPParser := PrepareTest ('d:\Software Development\Jays Utilities\Trace Wrangler\test\traces\DHCP-Offer.pcapng');

     if DHCPParser <> nil then
        begin
             CheckTrue (DHCPParser.MessageOpCode = docBootReply);
             CheckEquals (270, DHCPParser.PaddingSize, 'Padding Size');
             CheckEquals (548, DHCPParser.LayerSize, 'Layer Size');
             CheckEquals (1, DHCPParser.HardwareAddressType, 'Hardwaretype Ethernet');
             CheckEquals (6, DHCPParser.HardwareAddressLength, 'Hardware Address Length');
             CheckEquals (0, DHCPParser.Hops, 'Hops');
             CheckEquals ($a2a24eb9, DHCPParser.TransactionID, 'Transaction ID');
             CheckEquals (0, DHCPParser.Seconds, 'Seconds Elapsed');
             CHeckTrue (DHCPParser.Flags = dfUnicast);
             CheckEquals ('0.0.0.0', DHCPParser.ClientIPAddress.ValueAsString, 'Client IP address');
             CheckEquals ('192.168.123.106', DHCPParser.YourClientIPAddress.ValueAsString, 'Your Client IP address');
             CheckEquals ('192.168.123.254', DHCPParser.ServerIPAddress.ValueAsString, 'Server IP address');
             CheckEquals ('0.0.0.0', DHCPParser.RelayIPAddress.ValueAsString, 'Relay IP address');
             CheckEquals ('00:0e:a6:84:19:c1', lowercase (DHCPParser.ClientHWAddress.asString(':')), 'Client MAC Address');
             CheckEquals ('', DHCPParser.ServerHostName, 'Server Hostname');
             CheckEquals ('', DHCPParser.BootFileName, 'Boot file name');
             CheckTrue (DHCPParser.DHCPMessageType = dmtOffer);
             CheckEquals ('192.168.123.254', DHCPParser.DHCPServerIdentifier.ValueAsString, 'Server Identifier');
             CheckEquals (86400, DHCPParser.DHCPLeaseTime, 'Lease Time');
             CheckEquals ('255.255.255.0', DHCPParser.DHCPSubnetMask.ValueAsString, 'Subnet Mask');
             CheckEquals ('192.168.123.254', DHCPParser.DHCPRouter.ValueAsString, 'Router');
             CheckEquals ('217.237.150.97', DHCPParser.DHCPNameServers[1].ValueAsString, 'DNS Server 1');
             CheckEquals ('217.237.149.161', DHCPParser.DHCPNameServers[2].ValueAsString, 'DNS Server 2');
        end;
end;

(* ------------------------------------------- *)

procedure TesttDHCPv4Parser.TestparseDHCPRequest;

var DHCPParser : tDHCPv4Parser;

begin
     DHCPParser := PrepareTest ('d:\Software Development\Jays Utilities\Trace Wrangler\test\traces\DHCP-Request.pcapng');

     if DHCPParser <> nil then
        begin
             CheckTrue (DHCPParser.MessageOpCode = docBootRequest);
             CheckEquals (0, DHCPParser.PaddingSize, 'Padding Size');
             CheckEquals (312, DHCPParser.LayerSize, 'Layer Size');
             CheckEquals (1, DHCPParser.HardwareAddressType, 'Hardwaretype Ethernet');
             CheckEquals (6, DHCPParser.HardwareAddressLength, 'Hardware Address Length');
             CheckEquals (0, DHCPParser.Hops, 'Hops');
             CheckEquals ($a2a24eb9, DHCPParser.TransactionID, 'Transaction ID');
             CheckEquals (13, DHCPParser.Seconds, 'Seconds Elapsed');
             CHeckTrue (DHCPParser.Flags = dfUnicast);
             CheckEquals ('0.0.0.0', DHCPParser.ClientIPAddress.ValueAsString, 'Client IP address');
             CheckEquals ('0.0.0.0', DHCPParser.YourClientIPAddress.ValueAsString, 'Your Client IP address');
             CheckEquals ('0.0.0.0', DHCPParser.ServerIPAddress.ValueAsString, 'Server IP address');
             CheckEquals ('0.0.0.0', DHCPParser.RelayIPAddress.ValueAsString, 'Relay IP address');
             CheckEquals ('00:0e:a6:84:19:c1', lowercase (DHCPParser.ClientHWAddress.asString(':')), 'Client MAC Address');
             CheckEquals ('', DHCPParser.ServerHostName, 'Server Hostname');
             CheckEquals ('', DHCPParser.BootFileName, 'Boot file name');
             CheckTrue (DHCPParser.DHCPMessageType = dmtRequest);
             CheckEquals ('00:0E:A6:84:19:C1', DHCPParser.ClientHWAddress.asString, 'Client MAC Address');
             CheckEquals ('', DHCPParser.ServerHostName, 'Server Hostname');
             CheckEquals ('', DHCPParser.BootFileName, 'Boot file name');
             CheckEquals ('192.168.123.254', DHCPParser.DHCPServerIdentifier.ValueAsString, 'Server Identifier');
             CheckEquals ('192.168.123.106', DHCPParser.DHCPRequestedIPAddress.ValueAsString, 'Requested IPAddress');
             CheckEquals ('00:0E:A6:84:19:C1', DHCPParser.DHCPClientIdentifier.asString, 'Client Identifier');
             CheckEquals ('obsidian', DHCPParser.DHCPHostName.FullFQDN, 'Hostname');
        end;
end;

(* ------------------------------------------- *)

procedure TesttDHCPv4Parser.TestparseDHCPAcknowledge;

var DHCPParser : tDHCPv4Parser;

begin
     DHCPParser := PrepareTest ('d:\Software Development\Jays Utilities\Trace Wrangler\test\traces\DHCP-Acknowledge.pcapng');

     if DHCPParser <> nil then
        begin
             CheckTrue (DHCPParser.MessageOpCode = docBootReply);
             CheckEquals (270, DHCPParser.PaddingSize, 'Padding Size');
             CheckEquals (548, DHCPParser.LayerSize, 'Layer Size');
             CheckEquals (1, DHCPParser.HardwareAddressType, 'Hardwaretype Ethernet');
             CheckEquals (6, DHCPParser.HardwareAddressLength, 'Hardware Address Length');
             CheckEquals (0, DHCPParser.Hops, 'Hops');
             CheckEquals ($a2a24eb9, DHCPParser.TransactionID, 'Transaction ID');
             CheckEquals (0, DHCPParser.Seconds, 'Seconds Elapsed');
             CHeckTrue (DHCPParser.Flags = dfUnicast);
             CheckEquals ('0.0.0.0', DHCPParser.ClientIPAddress.ValueAsString, 'Client IP address');
             CheckEquals ('192.168.123.106', DHCPParser.YourClientIPAddress.ValueAsString, 'Your Client IP address');
             CheckEquals ('192.168.123.254', DHCPParser.ServerIPAddress.ValueAsString, 'Server IP address');
             CheckEquals ('0.0.0.0', DHCPParser.RelayIPAddress.ValueAsString, 'Relay IP address');
             CheckEquals ('00:0e:a6:84:19:c1', lowercase (DHCPParser.ClientHWAddress.asString(':')), 'Client MAC Address');
             CheckEquals ('', DHCPParser.ServerHostName, 'Server Hostname');
             CheckEquals ('', DHCPParser.BootFileName, 'Boot file name');
             CheckTrue (DHCPParser.DHCPMessageType = dmtAcknowledge);
             CheckEquals ('192.168.123.254', DHCPParser.DHCPServerIdentifier.ValueAsString, 'Server Identifier');
             CheckEquals (86400, DHCPParser.DHCPLeaseTime, 'Lease Time');
             CheckEquals ('255.255.255.0', DHCPParser.DHCPSubnetMask.ValueAsString, 'Subnet Mask');
             CheckEquals ('192.168.123.254', DHCPParser.DHCPRouter.ValueAsString, 'Router');
             CheckEquals ('217.237.150.97', DHCPParser.DHCPNameServers[1].ValueAsString, 'DNS Server 1');
             CheckEquals ('217.237.149.161', DHCPParser.DHCPNameServers[2].ValueAsString, 'DNS Server 2');
        end;
end;

(* ------------------------------------------- *)

procedure TesttDHCPv4Parser.TestparseOverloadOptionWithEnds;

var DHCPParser : tDHCPv4Parser;

begin
     DHCPParser := PrepareTest ('d:\Software Development\Jays Utilities\Trace Wrangler\test\traces\DHCP-OverloadOption.pcapng');

     if DHCPParser <> nil then
        begin
             CheckTrue (DHCPParser.MessageOpCode = docBootRequest);
             CheckEquals (0, DHCPParser.PaddingSize, 'Padding Size');
             CheckEquals (282, DHCPParser.LayerSize, 'Layer Size');
             CheckEquals (1, DHCPParser.HardwareAddressType, 'Hardwaretype Ethernet');
             CheckEquals (6, DHCPParser.HardwareAddressLength, 'Hardware Address Length');
             CheckEquals (0, DHCPParser.Hops, 'Hops');
             CheckEquals ($ac2effff, DHCPParser.TransactionID, 'Transaction ID');
             CheckEquals (0, DHCPParser.Seconds, 'Seconds Elapsed');
             CHeckTrue (DHCPParser.Flags = dfUnicast);
             CheckEquals ('0.0.0.0', DHCPParser.ClientIPAddress.ValueAsString, 'Client IP address');
             CheckEquals ('0.0.0.0', DHCPParser.YourClientIPAddress.ValueAsString, 'Your Client IP address');
             CheckEquals ('0.0.0.0', DHCPParser.ServerIPAddress.ValueAsString, 'Server IP address');
             CheckEquals ('0.0.0.0', DHCPParser.RelayIPAddress.ValueAsString, 'Relay IP address');
             CheckEquals ('00:00:6C:82:DC:4E', DHCPParser.ClientHWAddress.asString, 'Client MAC Address');
             CheckEquals (590, DHCPParser.DHCPMaxMessageSize, 'Max MessageSize');
             CheckEquals ('', DHCPParser.ServerHostName, 'Server Hostname');
             CheckEquals ('', DHCPParser.BootFileName, 'Boot file name');
             CheckTrue (DHCPParser.DHCPMessageType = dmtDiscover);
             CheckEquals ('00:00:6C:82:DC:4E', DHCPParser.ClientHWAddress.asString, 'Client MAC Address');
             CheckEquals ('', DHCPParser.ServerHostName, 'Server Hostname');
             CheckEquals ('', DHCPParser.BootFileName, 'Boot file name');
        end;
end;

(* ------------------------------------------- *)

initialization
              // Register any test cases with the test runner

              RegisterTest(TesttDHCPv4Parser.Suite);
end.

