{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}
 
unit Testclass_parser_ipv4;
{

  Delphi DUnit Test Case
  ----------------------
  This unit contains a skeleton test case class generated by the Test Case Wizard.
  Modify the generated code to correctly setup and call the methods from the unit 
  being tested.

}

interface

uses
  TestFramework, class_parser_baseclass, class_parser_ipv4, class_parsemaster, base_structures,
  base_checksums;

type
  // Test methods for class tIPv4Parser

  TesttIPv4Parser = class (TTestCase)
                          strict private
                                        Parsemaster : tParseMaster;

                                        function PrepareTest (Tracefile : string) : tIPv4Parser;
                          public

                          published
                                   procedure TestparseFragment;
                                   procedure TestHardSlicing;
                                   procedure TestEthernetTrailer;
                          end;

implementation

uses class_rawframe,
     class_loader_pcapng;

(* -------------------------------------- *)

function TesttIPv4Parser.PrepareTest (Tracefile: string) : tIPv4Parser;

var Loader      : tPCAPNGLoader;
    Frame       : tRawFrame;
    IPParser    : tIPv4Parser;
    OpenResult  : boolean;
    i           : integer;

begin
     Loader := tPCAPNGLoader.create (Tracefile);
     OpenResult := Loader.openFile;

     Loader.readTraceFileheader;
     Loader.readBatchFromDisk;

     Frame := Loader.getRawFrame(1);

     ParseMaster := tParseMaster.create (Frame, ppmParseAll);

     Loader.Free;
     Loader := nil;

     CheckEquals (true, Openresult, 'PCAPNGLoader.OpenFile Failed');

     i := 0;

     // Find the IP Parser

     repeat
           inc (i)
     until (Parsemaster.getParserTypeByIndex (i) = NEXT_HEADER_IPv4) or (i >= Parsemaster.ParserCount);

     if (Parsemaster.getParserTypeByIndex (i) = NEXT_HEADER_IPv4) then
        IPParser := (Parsemaster.getParserByIndex (i) as tIPv4Parser)
     else
         IPParser := nil;

     result := IPParser;
end;

(* -------------------------------------- *)

procedure TesttIPv4Parser.TestparseFragment;

var IPParser : tIPv4Parser;

begin
     IPParser := PrepareTest ('d:\Software Development\Jays Utilities\Trace Wrangler\test\traces\IPv4-FragmentWithPingPayload.pcapng');

     if IPParser <> nil then
        begin
             CheckEquals ('192.168.124.100', IPParser.Source.ValueAsString, 'Source IP');
             CheckEquals ('192.168.122.2', IPParser.Destination.ValueAsString, 'Destination IP');
             CheckEquals (1500, IPParser.TotalLength, 'Total Length');
             CheckEquals (2451, IPParser.Identification, 'IP Identification');
             CheckEquals (true, IPParser.IsFragment, 'Is Fragment');
             CheckEquals (128, IPParser.TTL, 'TTL');
             CheckEquals (1, IPParser.Protocol, 'Protocol');
             CheckEquals (NEXT_HEADER_PAYLOAD, IPParser.NextAnchor.HeaderType, 'Next Anchor');
             CheckEquals (1480, Parsemaster.RemainingPayloadCopy.Len, 'Remaining Payload Length');
             CheckEquals (0, IPParser.FramePayloadLengthDelta, 'Frame Payload Length Delta');
        end;

     IPParser.free;
end;

(* -------------------------------------- *)

procedure TesttIPv4Parser.TestHardSlicing;

var IPParser : tIPv4Parser;

begin
     IPParser := PrepareTest ('D:\Software Development\Jays Utilities\Trace Wrangler\test\traces\HardSlicedSampleFrame.pcapng');

     if IPParser <> nil then
        begin
             // This one is hard sliced, containing 1450 bytes less than IP says it does

             CheckEquals (-1450, IPParser.FramePayloadLengthDelta, 'Frame Payload Length Delta');
        end;

     IPParser.free;
end;

(* -------------------------------------- *)

procedure TesttIPv4Parser.TestEthernetTrailer;

var IPParser : tIPv4Parser;

begin
     IPParser := PrepareTest ('D:\Software Development\Jays Utilities\Trace Wrangler\test\traces\TCP-ACKwithPaddingAndFCS.pcapng');

     if IPParser <> nil then
        begin
             // This one has 6 bytes padding and 4 bytes FCS

             CheckEquals (10, IPParser.FramePayloadLengthDelta, 'Frame Payload Length Delta');
        end;

     IPParser.free;
end;

(* -------------------------------------- *)

initialization
              // Register any test cases with the test runner
              RegisterTest(TesttIPv4Parser.Suite);
end.

