{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}
 
unit Testclass_replacement_manager;
{

  Delphi DUnit Test Case
  ----------------------
  This unit contains a skeleton test case class generated by the Test Case Wizard.
  Modify the generated code to correctly setup and call the methods from the unit
  being tested.

}

interface

uses
  TestFramework,
  Generics.Collections,

{$ifdef VER230}

     // AnyDAC

     uADStanIntf, uADStanOption, uADStanError, uADGUIxIntf,
     uADPhysIntf, uADStanDef, uADStanPool,
     uADStanAsync, uADPhysManager, uADStanParam, uADDatSManager, uADDAptIntf,
     uADDAptManager, uADStanExprFuncs, Vcl.StdCtrls, uADPhysSQLite, Data.DB,
     uADCompDataSet, uADCompClient, uADGUIxFormsWait, uADPhysSQLiteWrapper,

{$else}

     // FireDAC

     FireDAC.Stan.Intf, FireDAC.Stan.Option, FireDAC.Stan.Error, FireDAC.UI.Intf,
     FireDAC.Phys.Intf, FireDAC.Stan.Def, FireDAC.Stan.Pool, FireDAC.dAPT,
     FireDAC.Stan.Async, FireDAC.Phys, FireDAC.Stan.ExprFuncs,
     FireDAC.VCLUI.Wait, FireDAC.Comp.UI, FireDAC.Phys.SQLite, FireDAC.Phys.SQLiteWrapper,
     Data.DB, FireDAC.Comp.Client, FireDAC.Stan.Param,
{$endif}

  base_structures,
  definition_ip_addresses,
  class_replacement_manager;

type
  // Test methods for class tReplacementManager

  TesttReplacementManager = class (TTestCase)
                                  strict private
                                                Manager: tReplacementManager;
                                  public
                                        procedure SetUp; override;
                                        procedure TearDown; override;

                                  published
                                          procedure TestAddReplacementMAC;
                                          procedure TestAddReplacementIPv4Address;
                                          procedure TestgetReplacementForMACAddress;
                                          procedure TestgetReplacementForIPv4Address;
                                          procedure TestgetReplacementForIPv6Address;
                                          procedure TestSequenceDeltaMarkers;
                                  end;

implementation

uses SysUtils,
     definition_anonymization_struct,
     definition_tcp_structures,
     definition_5tuple,
     class_ipv4subnethandler,
     class_ipv6subnethandler;

(* ------------------------------------------- *)

procedure TesttReplacementManager.SetUp;

var AnonParams : tAnonymizeParameters;

begin
     if fileexists ('d:\dummy.db') then
        deletefile ('d:\dummy.db');

     Manager := tReplacementManager.create ('d:\dummy.db', AnonParams);
end;

(* ------------------------------------------- *)

procedure TesttReplacementManager.TearDown;

begin
     Manager.Free;
     Manager := nil;
end;

(* ------------------------------------------- *)

procedure TesttReplacementManager.TestAddReplacementMAC;

var NewMAC: tMACAddress;
    OldMAC: tMACAddress;

begin
     OldMAC.setfromString ('00:00:11:00:00:11', ':');
     NewMAC.setfromString ('00:00:99:00:00:99', ':');

     // TODO: Setup method call parameters
     Manager.AddReplacementPair(OldMAC, NewMAC);
     // TODO: Validate method results
end;

(* ------------------------------------------- *)

procedure TesttReplacementManager.TestAddReplacementIPv4Address;

var NewIP: tIPv4Address;
    OldIP: tIPv4Address;

begin
     OldIP.setValueFromString('81.209.179.1');
     NewIP.setValueFromString('192.168.0.1');

     // TODO: Setup method call parameters
     Manager.AddReplacementPair(OldIP, NewIP);
     // TODO: Validate method results
end;

(* ------------------------------------------- *)

procedure TesttReplacementManager.TestgetReplacementForMACAddress;

var MAC    : tMACAddress;
    Value1 : tMACAddress;
    Value2 : tMACAddress;

begin
     MAC.setfromString ('00:0c:29:00:00:11', ':');

//   FtReplacementManager.KeepMACVendorIntact := true;
     Value1 := Manager.getReplacementForAddress(MAC);
     Value2 := Manager.getReplacementForAddress(MAC);

     CheckEquals (Value1.asString (':'), Value2.asString (':'), 'Replacement MAC incorrect');
end;

(* ------------------------------------------- *)

procedure TesttReplacementManager.TestgetReplacementForIPv4Address;

var Value1    : tIPv4Address;
    Value2    : tIPv4Address;
    IPAddress : tIPv4Address;
    subnet    : tIPv4SubnetDefinition;

begin
     IPAddress.initialize;
     IPAddress.setValueFromString('81.209.147.14');

     subnet := tIPv4SubnetDefinition.create ('81.209.147.0', '255.255.255.240', 'FNHNet');
     subnet.ReplacementNetAddress.setValueFromString('192.168.0.0', 24);
     Manager.IPv4SubnetManager.AddSubnet(Subnet);

     // Test static replacement based on subnet

     Value1 := Manager.getReplacementForAddress(IPAddress);
     CheckEquals ('192.168.0.14', Value1.ValueAsString, 'Replacement IPv4 Address incorrect');

     // Test random replacement based on subnet

//   FtReplacementManager.KeepIPv4HostPartIntact := false;
     Value1 := Manager.getReplacementForAddress(IPAddress);
     Value2 := Manager.getReplacementForAddress(IPAddress);

     CheckEquals (Value1.ValueAsLong, Value2.ValueAsLong, 'Random replacement was not stored to database');
end;

(* ------------------------------------------- *)

procedure TesttReplacementManager.TestgetReplacementForIPv6Address;

var ReturnValue : tIPv6Address;
    IPAddress   : tIPv6Address;
    subnet      : tIPv6SubnetDefinition;
    NewIP: tIPv6Address;
    OldIP: tIPv6Address;

begin
     // Add test replacement IP

     OldIP.setValueFromString('2001:4dd0:b45e::1');
     NewIP.setValueFromString('fe80:4dd0:b45e::1');
     Manager.AddReplacementPair(OldIP, NewIP);

     // Add a test net with replacement net

     IPAddress.initialize;
     IPAddress.setValueFromString ('2001:470:b45e:10::', 64);

     subnet := tIPv6SubnetDefinition.create (IPAddress, 'Testnet');
     subnet.ReplacementNetAddress.setValueFromString ('fd00:470:1234:10::',64);
     Manager.IPv6SubnetManager.AddSubnet (Subnet);

     // Test replacement of IP Addresses

     IPAddress.setValueFromString('2001:4dd0:b45e::1');
     ReturnValue := Manager.getReplacementForAddress(IPAddress);
     CheckEquals (ReturnValue.ValueAsString, 'fe80:4dd0:b45e::1', 'Replacement IPv6 Address incorrect');

//   FtReplacementManager.KeepIPv6HostPartIntact := true;
     IPAddress.setValueFromString('2001:470:b45e:10::1');
     ReturnValue := Manager.getReplacementForAddress(IPAddress);
     CheckEquals (ReturnValue.ValueAsString, 'fd00:470:1234:10::1', 'Replacement IPv6 Address incorrect');
end;

(* ------------------------------------------- *)

procedure TesttReplacementManager.TestSequenceDeltaMarkers;

var SocketPair : tIPv4SocketPair;
    Tuple      : t5Tuple;
    Delta      : integer;
    seq        : tTCPSequence;

begin
     SocketPair.initialize;
     SocketPair.Source.IPAddress.setValueFromString('192.168.0.1');
     SocketPair.Source.Port := 49152;
     SocketPair.Destination.IPAddress.setValueFromString('192.168.0.1');
     SocketPair.Destination.Port := 80;

     Tuple.initialize (SocketPair, L4TCP);

     seq.initialize;

     seq.Value := 12345;
     Manager.AddTCPSequenceDelta (Tuple, seq, -1);
     seq.Value := 23456;
     Manager.AddTCPSequenceDelta (Tuple, seq, +2);
     seq.Value := 1234;
     Manager.AddTCPSequenceDelta (Tuple, seq, -3);
     seq.Value := 12456;
     Manager.AddTCPSequenceDelta (Tuple, seq, +4);

     seq.Value := 1234;
     delta := Manager.getTCPSequenceDelta (Tuple, seq);
     CheckEquals (Delta, 0, 'Delta value needs to be 0');

     seq.Value := 12345;
     delta := Manager.getTCPSequenceDelta (Tuple, seq);
     CheckEquals (Delta, -3, 'Delta value needs to be -3');

     seq.Value := 12457;
     delta := Manager.getTCPSequenceDelta (Tuple, seq);
     CheckEquals (Delta, 0, 'Delta value needs to be 0');

     seq.Value := 34567;
     delta := Manager.getTCPSequenceDelta (Tuple, seq);
     CheckEquals (Delta, 2, 'Delta value needs to be 2');
end;

(* ------------------------------------------- *)

initialization
              // Register any test cases with the test runner
              RegisterTest(TesttReplacementManager.Suite);
end.

