{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}
 
unit Testclass_writer_pcapng;
{

  Delphi DUnit Test Case
  ----------------------
  This unit contains a skeleton test case class generated by the Test Case Wizard.
  Modify the generated code to correctly setup and call the methods from the unit
  being tested.

}

interface

uses
  TestFramework, class_interfacelist, class_writer_pcapng, base_structures,
  class_interfacestatistics, class_dnsserverlist, Classes, pcapng_structures,
  definition_simple_nameresolution, class_rawframe, Contnrs,
  definition_ip_addresses, class_tracewriter, class_loader_pcapng,
  class_timezonedetails;

type
  // Test methods for class tPCAPngWriter

  TesttPCAPngWriter = class(TTestCase)
  strict private
                procedure onCopyInterface (CaptureInterface : tCaptureInterface);

  public
  published
    procedure TestCopyPCAPngStatisticsDetailsFromLoader;
    procedure TestCreateTestfile;
    procedure TestRawFrameMemoryHandling;
  end;

implementation

uses SiAuto,
     SysUtils,
     class_simpleframestorage,
     class_frameprocessor_baseclass,
     class_parsemaster;

(* ------------------------------------------- *)

procedure TesttPCAPngWriter.onCopyInterface (CaptureInterface: tCaptureInterface);

var IPv4address : tIPv4Address;

begin
     CaptureInterface.MACAddress.setfromString('00:01:02:00:ca:5e', ':');

     IPv4address.setValueFromString('1.2.3.4');
     CaptureInterface.IPv4Addresses.Add(IPv4Address);
     CaptureInterface.Name := 'Testinterface';
end;

(* ------------------------------------------- *)

procedure TesttPCAPngWriter.TestCopyPCAPngStatisticsDetailsFromLoader;

var Writer      : tPCAPngWriter;
    Loader      : tPCAPNGLoader;
    frame       : tRawFrame;
//  framebuffer : tSimpleFrameStorage;
    i           : integer;

begin
     Loader := tPCAPNGLoader.create ('D:\Traces\Capture Frames Out of Order Sample with Name Resolution.pcapng');

     // open Loader

     Loader.openFile;                        // TODO: handle failure of opening trace files
     Loader.readTraceFileheader;

     // Open writer

     Writer := tPCAPngWriter.create ('D:\Traces\Capture Frames Out of Order Sample with Name Resolution (Test).pcapng', true);
//   Writer.BlockWriteMode := bwmSimplePacketBlock;
     Writer.onCopyInterface := onCopyInterface;
     Writer.Open;

     (Writer as tPCAPNGWriter).CopyPCAPngHeaderDetailsFromLoader (Loader as tPCAPNGLoader);

     Loader.MaxFramesInMem := 10000;

     // Read batches of frames and add them to the framebuffer
     // which will write them to file every now and then

     repeat
           Loader.readBatchFromDisk;

           for i := 1 to Loader.FramesAvailable do
               begin
                    Frame := Loader.getRawFrame(i);

                    Writer.addFrameToList(Frame);
               end;

     until Loader.FramesAvailable = 0;;

     // Write remaining frames to disk if any

     if Writer.FramesInMem > 0 then
        (Writer as tPCAPNGWriter).Flush;

     // Copy statistics

     (Writer as tPCAPNGWriter).CopyPCAPngStatisticsDetailsFromLoader (loader as tPCAPNGLoader);

     // Close the loader/writer

     Writer.Close;
     Writer.free;
     Loader.free;
end;

(* ------------------------------------------- *)

procedure TesttPCAPngWriter.TestCreateTestfile;

var Writer      : tPCAPngWriter;
    Loader      : tPCAPNGLoader;
    loadframe   : tRawFrame;
    writeframe  : tRawFrame;
//  framebuffer : tSimpleFrameStorage;
    i           : integer;
    totalbytes  : integer;

const targetsize = 800000000;

begin
{
     Loader:= tPCAPNGLoader.create ('D:\Software Development\Jays Utilities\Trace Wrangler\test\traces\GTPv1-PingSample.pcapng');

     // open Loader

     Loader.openFile;                        // TODO: handle failure of opening trace files
     Loader.readTraceFileheader;

     // Open writer

     Writer := tPCAPngWriter.create ('D:\Traces\FrameBatch01.pcapng', true);
     Writer.onCopyInterface := onCopyInterface;
     Writer.Open;

     (Writer as tPCAPNGWriter).CopyPCAPngHeaderDetailsFromLoader (Loader as tPCAPNGLoader);

     // Do the work

     Loader.readBatchFromDisk;

     loadframe := Loader.getRawFrame(1);
     totalbytes := 0;

     repeat
           writeframe := tRawFrame.create (loadframe);  // This create statement proved to be evil and was removed from tRawFrame again

           Writer.addFrameToList(writeframe);

           inc (totalbytes, writeframe.FrameSize);

     until totalbytes > targetsize;


     // Write remaining frames to disk if any

     if Writer.FramesInMem > 0 then
        (Writer as tPCAPNGWriter).Flush;

     // Copy statistics

//   (Writer as tPCAPNGWriter).CopyPCAPngStatisticsDetailsFromLoader (loader as tPCAPNGLoader);

     // Close the loader/writer

     Writer.Close;
     Writer.free;
     Loader.free;
}
end;

(* ------------------------------------------- *)

procedure TesttPCAPngWriter.TestRawFrameMemoryHandling;

var Loader      : tPCAPNGLoader;
    Writer      : tPCAPngWriter;
    loadframe   : tRawFrame;
    TestFrame   : tRawFrame;
    TestIf      : tCaptureInterface;
    i,j         : integer;

    TimeDetails  : tTimingDetails;
    WireDetails  : tWireDetails;
    FilePosition : tFileFrameLocation;
    FrameBuffer  : tRawBuffer;
    ParseMaster  : tParseMaster;

begin
     SiMain.EnterMethod ('TesttPCAPngWriter.TestRawFrameMemoryHandling');

     Loader := tPCAPNGLoader.create ('D:\Software Development\Jays Utilities\Trace Wrangler\test\traces\TCP-Jumboframe.pcapng');

     // open Loader

     Loader.openFile;
     Loader.readTraceFileheader;
     Loader.readBatchFromDisk;
     loadframe := Loader.getRawFrame(1);

     Writer := tPCAPngWriter.create ('D:\Software Development\Jays Utilities\Trace Wrangler\test\traces\memtest.pcapng', true);
     Writer.Open;
     Writer.CopyPCAPngHeaderDetailsFromLoader(Loader);

     TimeDetails.initialize;
     WireDetails.initialize;
     FilePosition.initialize;

     WireDetails.WireSize := 9014;
     WireDetails.CaptureSize := 9014;

     FrameBuffer := loadframe.Buffer;

     repeat
           testframe := tRawFrame.create (TimeDetails, WireDetails, FilePosition, FrameBuffer);

           ParseMaster := tParseMaster.create (TestFrame, ppmParseAll);
           ParseMaster.free;
           ParseMaster := nil;

//         Writer.addFrameToList (TestFrame);             // THIS

           TestFrame.free;                                // OR THIS
           TestFrame := nil;

     until false;

     loadframe.free;

     Writer.Close;
     Writer.free;

     Loader.closeFile;
     Loader.free;

     SiMain.LeaveMethod ('TesttPCAPngWriter.TestRawFrameMemoryHandling');
end;

(* ------------------------------------------- *)

initialization
              // Register any test cases with the test runner

              RegisterTest(TesttPCAPngWriter.Suite);
end.

