{ TraceWrangler - Network Capture File Toolkit
  By Jasper Bongertz <jasper@packet-foo.com>
  Copyright 2016 Jasper Bongertz

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

unit class_conversationstorage;

interface

uses Classes,
     System.Generics.Collections,

     FireDAC.Stan.Intf, FireDAC.Stan.Option, FireDAC.Stan.Error, FireDAC.UI.Intf,
     FireDAC.Phys.Intf, FireDAC.Stan.Def, FireDAC.Stan.Pool, FireDAC.dAPT,
     FireDAC.Stan.Async, FireDAC.Phys, FireDAC.Stan.ExprFuncs,
     FireDAC.VCLUI.Wait, FireDAC.Comp.UI, FireDAC.Phys.SQLite,
     Data.DB, FireDAC.Comp.Client,

     base_structures,
     definition_ip_addresses,
     definition_interface_mapping_types,
     definition_conversation_types,
     definition_filter_structures,
     class_diagmessagehandler,
     class_rawframe,
     class_parser_ethernet,
     class_parser_arp,
     class_parser_ipv4,
     class_parser_ipv6,
     class_parser_tcp,
     class_parser_udp,
     class_parser_dns,
     class_FQDNManager;

type tTCPConversationSortMode = (csmStandard, csmIPPair);

     tEventOnDiagnosticMessages = procedure (Sender : tObject; Messagelist : tDiagnosticMsgList) of object;

     tFrameDetails = record
                           FileFrameID    : int64;
                           WireSize       : longword;
                           FileOffset     : int64;
                           FrameTimestamp : tFrameTimestamp;
                           Attributes     : tConversationAttributeSet;

                           procedure initialize; overload;
                           procedure initialize (const Index : int64; const Size : longword; const Offset : int64; const Timestamp : tFrameTimestamp); overload;
                     end;

     tNameResolutionDetails = record
                                    FQDN      : tFQDN;
                                    isAnswer  : boolean;
                                    IPVersion : byte;
                                    Answer    : tNameResolutionAnswer;
                                    Marker    : tMarkerSet;
                                    QueryType : tDNSRecordType;
                                    QueryOnly : boolean;
                                    ReplyCode : tDNSReturnCode;

                                    procedure initialize; overload;
                                    procedure initialize (const NewFQDN : tFQDN; const NewIsAnswer : boolean; const NewIPVersion : byte; const NewAnswer : tNameResolutionAnswer; const NewMarker : tMarkerSet; NewQueryType : tDNSRecordType; const NewQueryOnly : boolean; const NewReturnCode : tDNSReturnCode); overload;
                             end;

     tConversationStorage = class (tObject)

                                  constructor create; overload;
                                  constructor create (const Tracefilename : string); overload;
                                  procedure free;

                                  private
                                         priv_EventonDiagnosticsMessages : tEventOnDiagnosticMessages;

                                         priv_tracefilename              : string;
                                         priv_filedbindex                : integer;
                                         priv_database                   : tFDConnection;
                                         priv_currentfilter              : tConversationFilter;     // not in use yet
                                         priv_OverallComplexity          : integer;

                                         priv_dictFirstFrames            : tDictionary<string, tConversationStoreRecTypeSet>;
                                         priv_dictLastFrames             : tDictionary<string, tConversationStoreRecTypeSet>;
                                         priv_FirstLastSources           : tConversationStoreRecTypeSet;

                                         priv_dictMACEndpoints           : tDictionary<string, tMACEndpoint>;
                                         priv_dictMACConversations       : tDictionary<string, tMACConversation>;
                                         priv_dictIPv4Endpoints          : tDictionary<string, tIPv4Endpoint>;
                                         priv_dictIPv4Conversations      : tDictionary<string, tIPv4Conversation>;
                                         priv_dictIPv6Endpoints          : tDictionary<string, tIPv6Endpoint>;
                                         priv_dictIPv6Conversations      : tDictionary<string, tIPv6Conversation>;
                                         priv_dictTCPConversations       : tDictionary<string, tTCPConversation>;
                                         priv_dictUDPConversations       : tDictionary<string, tUDPConversation>;
                                         priv_dictNameResolutions        : tDictionary<string, tNameResolutionEntry>;
                                         priv_dictTCPSockets             : tDictionary<string, tSocketDetails>;
                                         priv_dictUDPSockets             : tDictionary<string, tSocketDetails>;

                                         procedure PrepareDictionaries;

                                         procedure AddEthernetDetails (parser : tEthernetParser; InterfaceIndex : tAbsoluteFileInterface; FrameDetails : tFrameDetails);
                                         procedure AddARPDetails (parser : tARPParser; InterfaceIndex : tAbsoluteFileInterface; FrameDetails : tFrameDetails);
                                         procedure AddIPv4Details (parser : tIPv4Parser; InterfaceIndex : tAbsoluteFileInterface; FrameDetails : tFrameDetails; const isICMPQuote : boolean = false);
                                         procedure AddIPv6Details (parser : tIPv6Parser; InterfaceIndex : tAbsoluteFileInterface; FrameDetails : tFrameDetails; const isICMPQuote : boolean = false);
                                         procedure AddTCPDetails (parser : tTCPParser; IPv4Pair : tIPv4AddressPair; InterfaceIndex : tAbsoluteFileInterface; FrameDetails : tFrameDetails; TunnelFlags : tTunnelFlagSet); overload;
                                         procedure AddTCPDetails (parser : tTCPParser; IPv6Pair : tIPv6AddressPair; InterfaceIndex : tAbsoluteFileInterface; FrameDetails : tFrameDetails; TunnelFlags : tTunnelFlagSet); overload;
                                         procedure AddUDPDetails (parser : tUDPParser; IPv4Pair : tIPv4AddressPair; InterfaceIndex : tAbsoluteFileInterface; FrameDetails : tFrameDetails; TunnelFlags : tTunnelFlagSet); overload;
                                         procedure AddUDPDetails (parser : tUDPParser; IPv6Pair : tIPv6AddressPair; InterfaceIndex : tAbsoluteFileInterface; FrameDetails : tFrameDetails; TunnelFlags : tTunnelFlagSet); overload;
                                         procedure AddNameResolutionDetails (const NRDetails : tNameResolutionDetails; const InterfaceIndex : tAbsoluteFileInterface; const FrameDetails : tFrameDetails);

                                         procedure StoreMACEndpointsToDB;
                                         procedure StoreMACConversationsToDB;
                                         procedure StoreIPv4EndpointsToDB;
                                         procedure StoreIPv4ConversationsToDB;
                                         procedure StoreIPv6EndpointsToDB;
                                         procedure StoreIPv6ConversationsToDB;
                                         procedure StoreTCPConversationsToDB;
                                         procedure StoreUDPConversationsToDB;
                                         procedure StoreNameResolutionsToDB;

                                         procedure ReadMACEndpointsFromDB;
                                         procedure ReadMACConversationsFromDB;
                                         procedure ReadIPv4EndpointsFromDB;
                                         procedure ReadIPv4ConversationsFromDB;
                                         procedure ReadIPv6EndpointsFromDB;
                                         procedure ReadIPv6ConversationsFromDB;
                                         procedure ReadTCPConversationsFromDB;
                                         procedure ReadUDPConversationsFromDB;
                                         procedure ReadNameResolutionsFromDB;

//                                       function GetTracefileDBIndex : integer;

                                         procedure setDBFileIndex (Value : integer);

                                  public
                                        property CurrentFileIndex : integer read priv_filedbindex write setDBFileIndex;
                                        property OverallComplexity : integer read priv_OverallComplexity;
                                        property OnDiagnosticMessageList : tEventOnDiagnosticMessages read priv_EventonDiagnosticsMessages write priv_EventonDiagnosticsMessages;
                                        property Filter : tConversationFilter read priv_currentfilter write priv_currentfilter;

                                        procedure AddMACEndpoint (Value : tMACEndpoint);
                                        procedure AddIPv4Endpoint (Value : tIPv4Endpoint);
                                        procedure AddIPv6Endpoint (Value : tIPv6Endpoint);
                                        procedure AddMACConversation (Value : tMACConversation);
                                        procedure AddIPv4Conversation (Value : tIPv4Conversation);
                                        procedure AddIPv6Conversation (Value : tIPv6Conversation);
                                        procedure AddTCPConversation (value : tTCPConversation);
                                        procedure AddUDPConversation (value : tUDPConversation);
                                        procedure AddNameResolution (Value : tNameResolutionEntry);

                                        function GetMACEndpointList : tMACEndpointList;
                                        function GetMACConversationList : tMACConversationList;
                                        function GetIPv4EndpointsList : tIPv4EndpointList;
                                        function GetIPv4ConversationList : tIPv4ConversationList;
                                        function GetIPv6EndpointsList : tIPv6EndpointList;
                                        function GetIPv6ConversationList : tIPv6ConversationList;
                                        function GetTCPConversationList (const SortMode : tTCPConversationSortMode = csmStandard) : tTCPConversationList;
                                        function getUDPConversationList : tUDPConversationList;
                                        function GetNameResolutionList : tNameResolutionList;

                                        function GetMACEndpointCount : integer;
                                        function GetMACConversationCount : integer;
                                        function GetIPv4EndpointCount : integer;
                                        function GetIPv4ConversationCount : integer;
                                        function GetIPv6EndpointCount : integer;
                                        function GetIPv6ConversationCount : integer;
                                        function GetTCPConversationCount : integer;
                                        function getUDPConversationCount : integer;
                                        function GetCountTable : tConversationCountTable;

                                        function GetMACEndpointEntry (const Value : tMACAddress) : tMACEndpoint;
                                        function GetIPv4EndpointEntry (const Value : tIPv4Address) : tIPv4Endpoint;
                                        function GetIPV6EndpointEntry (const Value : tIPv6Address) : tIPv6Endpoint;
                                        function GetMACConversationEntry (const Value : tMACAddressPair) : tMACConversation;
                                        function GetIPv4ConversationEntry (const Value : tIPv4AddressPair) : tIPv4Conversation;
                                        function GetIPv6ConversationEntry (const Value : tIPv6AddressPair) : tIPv6Conversation;
                                        function GetTCPConversationEntry (const Value: tIPv4SocketPair): tTCPConversation; overload;
                                        function GetTCPConversationEntry (const Value: tIPv6SocketPair): tTCPConversation; overload;
                                        function GetUDPConversationEntry (const Value : tIPv4SocketPair) : tUDPConversation; overload;
                                        function GetUDPConversationEntry (const Value : tIPv6SocketPair) : tUDPConversation; overload;

                                        function GetSocketEntry (const Protocol : byte; const Socket : tIPSocket) : tSocketDetails;

                                        /// <summary>
                                        ///   Allows determining if a frame is
                                        ///   the last of it's kind in the
                                        ///   current task being processed.
                                        ///   This is usually used to close
                                        ///   MultiWriter objects as soon as
                                        ///   possible.
                                        /// </summary>
                                        /// <param name="FramePosition">
                                        ///   Position of the Frame
                                        /// </param>
                                        function LastFrameStatusType (const FramePosition : tFileFrameLocation) : tConversationStoreRecTypeSet;

                                        {$REGION 'Documentation'}
                                        /// <summary>
                                        ///   This routine is used to update
                                        ///   the file index of interface
                                        ///   mappings. This is necessary
                                        ///   because the file index can change
                                        ///   when assembling files from
                                        ///   multiple traceintel dbs.
                                        /// </summary>
                                        /// <param name="InterfaceList">
                                        ///   List of interfaces in use by
                                        ///   endpoints or conversations
                                        /// </param>
                                        /// <param name="FileID">
                                        ///   current file index the endpoint
                                        ///   or conversation entry belongs to
                                        /// </param>
                                        {$ENDREGION}
                                        procedure ForceFileIDOnInterfaceList (var InterfaceList : tInterfaceFileIDList; const FileID: integer);
                                        function HasRecords : boolean;

                                        procedure Clear;

                                        {$REGION 'Documentation'}
                                        /// <summary>
                                        ///   Determines the interfaces used by
                                        ///   the conversations relevant to the
                                        ///   filter set. This is used when
                                        ///   extracting flows to know which
                                        ///   interfaces are in use
                                        /// </summary>
                                        {$ENDREGION}
                                        function GetInterfaceMapListForFilterSet (const FilterSet : tFilterSet) : tInterfaceFileIDList;

                                        procedure StoreConversationsToDatabase (Database : tFDConnection; Filename: string; const FileSize : integer; FileDate : tDateTime; const FirstFrameTime, LastFrameTime : tFrameTimeStamp);
                                        procedure LoadConversationsFromDatabase (Database : tFDConnection; Filename: string; const FileSize : integer; FileDate : tDateTime; const FirstFrameTime, LastFrameTime : tFrameTimeStamp);
                                        /// <summary>
                                        ///   Special version of the load
                                        ///   routine that loads the details of
                                        ///   the first file in the filelist.
                                        ///   This is used by pcapscan to merge
                                        ///   files.
                                        /// </summary>
                                        procedure LoadFirstFileConversationsFromDatabase (Database : tFDConnection);
                                        /// <summary>
                                        ///   Fill the FQDN Manager with the
                                        ///   values from the convesationstore
                                        ///   name resolution dictionary
                                        /// </summary>
                                        procedure InitFQDNManager (var FQDNManager : tFQDNManager);
                                        {$REGION 'Documentation'}
                                        /// <summary>
                                        ///   Merges a conversation store of a
                                        ///   single file entry into another,
                                        ///   usually to create a glocal store
                                        ///   for multiple trace files. It can
                                        ///   also be used to merge multi-file
                                        ///   stores, but then the FileID value
                                        ///   must be zero, because otherwise
                                        ///   the conversation records will be
                                        ///   modified by mistake.<br />
                                        /// </summary>
                                        /// <param name="Store">
                                        ///   Store to retrieve conversation
                                        ///   details from
                                        /// </param>
                                        /// <param name="FileID">
                                        ///   Index of the trace file in the
                                        ///   list. If zero the file id from
                                        ///   each conversation detail record
                                        ///   will be used instead
                                        /// </param>
                                        {$ENDREGION}
                                        procedure AddConversationsFromStore (Store : tConversationStorage; const FileID : integer = 0);
                                        /// <summary>
                                        ///   Fills the dictionaries that keep
                                        ///   track of which frames are the
                                        ///   first and the last in endpoints
                                        ///   or conversations. Those are
                                        ///   mostly used to know when a
                                        ///   loader/writer object can be
                                        ///   released. Before each task is run
                                        ///   the ForceClean must be set to
                                        ///   build the dictionaries from
                                        ///   scratch.
                                        /// </summary>
                                        /// <param name="WriteMode">
                                        ///   Specifies the write mode, to help
                                        ///   shorten the update process if
                                        ///   possible
                                        /// </param>
                                        /// <param name="ForceClean">
                                        ///   Discards the existing
                                        ///   dictionaries and builds them from
                                        ///   scratch
                                        /// </param>
                                        procedure UpdateFirstAndLastDictionaries (const WriteMode : tMultiWriteMode = mwmNone; const ForceClean : boolean = false);
                                        /// <summary>
                                        ///   This method calculates meta
                                        ///   values, e.g. min/max/average RTT
                                        ///   and the filtermatch strings after
                                        ///   all conversations have been
                                        ///   merged (so it should be called
                                        ///   after the merge loop for the
                                        ///   final converstation store)
                                        /// </summary>
                                        procedure UpdateCalculatedValues;
                                        /// <summary>
                                        ///   This determines what kind of
                                        ///   filter is transmitted, finds the
                                        ///   according record in the
                                        ///   conversation store and sets
                                        ///   First/Last frame and conversation
                                        ///   index helpers of the filter
                                        /// </summary>
                                        /// <param name="Filter">
                                        ///   Filter to be updated
                                        /// </param>
                                        procedure UpdateFilterHelpers (var Filter : tFrameFilter);

                                        {$REGION 'Documentation'}
                                        /// <summary>
                                        ///   Takes the provided frame and adds
                                        ///   its meta data to the endpoint and
                                        ///   conversation tables
                                        /// </summary>
                                        /// <param name="Frame">
                                        ///   Frame to be parsed; should not
                                        ///   yet have been aded to the store
                                        /// </param>
                                        {$ENDREGION}
                                        procedure AddFrame (Frame : tRawFrame);
                            end;


implementation

uses SysUtils,
     DateUtils,
     SiAuto,
     class_parser_baseclass,
     definition_tcp_structures,
     inteldb_tools,
     class_parsemaster;


(* ------------------------------------------- *)
{ tFrameDetails }
(* ------------------------------------------- *)

procedure tFrameDetails.initialize;

begin
     FileFrameID := 0;
     WireSize := 0;
     FileOffset := 0;
     FrameTimestamp.initialize;
     Attributes := [];
end;

(* ------------------------------------------- *)

procedure tFrameDetails.initialize (const Index: int64; const size: longword; const offset: int64; const timestamp: tFrameTimestamp);

begin
     FileFrameID := Index;
     WireSize := Size;
     FileOffset := Offset;
     FrameTimestamp := Timestamp;
     Attributes := [];
end;

(* ------------------------------------------- *)
{ tNameResolutionDetails }
(* ------------------------------------------- *)

procedure tNameResolutionDetails.initialize;

begin
     FQDN.initialize;
     isAnswer := false;
     IPVersion := 0;
     Answer.initialize;
     Marker := [];
     QueryType := dnsrtUnknown;
     QueryOnly := true;
     ReplyCode := dnsrcAvailable;
end;

(* ------------------------------------------- *)

procedure tNameResolutionDetails.initialize (const NewFQDN: tFQDN; const NewIsAnswer : boolean; const NewIPVersion: byte; const NewAnswer: tNameResolutionAnswer; const NewMarker: tMarkerSet; NewQueryType: tDNSRecordType; const NewQueryOnly: boolean; const NewReturnCode: tDNSReturnCode);

begin
     FQDN := NewFQDN;
     isAnswer := NewIsAnswer;
     IPVersion := NewIPVersion;
     Answer := NewAnswer;
     Marker := NewMarker;
     QueryType := NewQueryType;
     QueryOnly := NewQueryOnly;
     ReplyCode := NewReturnCode;
end;

(* ------------------------------------------- *)
{ tConversationStorage }
(* ------------------------------------------- *)

constructor tConversationStorage.create;

begin
//   SiMain.EnterMethod ('tConversationStorage.create');

     inherited create;

     priv_tracefilename := '';
     priv_filedbindex := 0;
     priv_OverallComplexity := ComplexityUnknown;

     priv_currentfilter.initialize;

     prepareDictionaries;

//   SiMain.LeaveMethod ('tConversationStorage.create');
end;

(* ------------------------------------------- *)

constructor tConversationStorage.create (const Tracefilename: string);

begin
//   SiMain.EnterMethod ('tConversationStorage.create ("' + Tracefilename + '")');

     inherited create;

     priv_tracefilename := Tracefilename;
     priv_filedbindex := 0;
     priv_OverallComplexity := ComplexityUnknown;

     prepareDictionaries;

//   SiMain.LeaveMethod ('tConversationStorage.create ("' + Tracefilename + '")');
end;

(* ------------------------------------------- *)

procedure tConversationStorage.free;

begin
     SiMain.EnterMethod ('tConversationStorage.free');

     clear;

     priv_dictFirstFrames.Free;
     priv_dictLastFrames.Free;

     priv_dictMACEndpoints.Free;
     priv_dictIPv4Endpoints.Free;
     priv_dictIPv6Endpoints.Free;
     priv_dictMACConversations.Free;
     priv_dictIPv4Conversations.Free;
     priv_dictIPv6Conversations.Free;
     priv_dictTCPConversations.Free;
     priv_dictUDPConversations.Free;
     priv_dictNameResolutions.Free;
     priv_dictTCPSockets.Free;
     priv_dictUDPSockets.Free;

     inherited free;

     SiMain.LeaveMethod ('tConversationStorage.free');
end;

(* ------------------------------------------- *)

procedure tConversationStorage.setDBFileIndex (Value: integer);

begin
//   SiMain.LogVerbose ('Updating Current DB File Index for Conversation Store, setting it to ' + IntToStr (Value));

     priv_filedbindex := Value;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.PrepareDictionaries;

begin
     priv_dictFirstFrames := tDictionary<string, tConversationStoreRecTypeSet>.create;
     priv_dictLastFrames := tDictionary<string, tConversationStoreRecTypeSet>.create;

     priv_dictMACEndpoints := tDictionary<string, tMACEndpoint>.create;
     priv_dictIPv4Endpoints := tDictionary<string, tIPv4Endpoint>.create;
     priv_dictIPv6Endpoints := tDictionary<string, tIPv6Endpoint>.create;
     priv_dictMACConversations := tDictionary<string, tMACConversation>.create;
     priv_dictIPv4Conversations := tDictionary<string, tIPv4Conversation>.create;
     priv_dictIPv6Conversations := tDictionary<string, tIPv6Conversation>.create;
     priv_dictTCPConversations := tDictionary<string, tTCPConversation>.create;
     priv_dictUDPConversations := tDictionary<string, tUDPConversation>.create;
     priv_dictNameResolutions := tDictionary<string, tNameResolutionEntry>.create;
     priv_dictTCPSockets := tDictionary<string, tSocketDetails>.create;
     priv_dictUDPSockets := tDictionary<string, tSocketDetails>.create;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddMACEndpoint (Value: tMACEndpoint);

var help   : tMACEndpoint;
    marker : tMarker;
    key    : string;

begin
     key := value.DictionaryKey;

     if not priv_dictMACEndpoints.ContainsKey (key) then
        begin
             priv_dictMACEndpoints.Add (key, value);
        end
     else
         begin
              // Get existing entry for merging

              help := priv_dictMACEndpoints.Items[key];

              if caSingleFrame in value.Attributes then
                 begin
                      // Check if we have an frame out of order situation if it isn't already

                      if not (caTimeOutOfOrder in help.attributes) then
                         begin
                              if (value.OriginFileIndex = help.LastFileID) and (value.OriginTimestamp < help.LastFrameTime) then
                                 help.Attributes := help.Attributes + [caTimeOutOfOrder];
                         end;

                      // Remove the SingleFrame attribute to avoid merging it later

                      value.Attributes := value.Attributes - [caSingleFrame];
                 end;

              // Check some other attributes, if we're still in the same file

              if (help.LastFileID = value.OriginFileIndex) and (help.InterfaceList[0].FileIndex = value.InterfaceList[0].FileIndex) then
                 begin
                      // Check if there is more than one interface used

                      if (help.InterfaceList[0].InterfaceFlags <> value.InterfaceList[0].InterfaceFlags) then
                         help.Attributes := help.Attributes + [caMultipleInterfaces];
                 end;

              // Make sure to clear the one sided attribute if at least one of both is has two way communication

              if caOneSidedCommunication in help.Attributes then
                 begin
                      if not (caOneSidedCommunication in value.Attributes) then
                         help.Attributes := help.Attributes - [caOneSidedCommunication];
                 end
              else
                  value.Attributes := value.Attributes - [caOneSidedCommunication];

              // Merge Attributesets

              help.Attributes := help.Attributes + value.Attributes;

              // If the existing entry was from a single frame, it won't be anymore after this merge

              if caSingleFrame in help.Attributes then
                 begin
                      help.Attributes := help.Attributes - [caSingleFrame];

                      // Clear helpers

                      help.OriginTimestamp.initialize;
                 end;

              // Always merge interface lists

              help.InterfaceList := mergeInterfaceLists (help.InterfaceList, value.InterfaceList);

              if not (caSpansMultipleFiles in help.Attributes) and (length (help.InterfaceList) > 1) then
                 help.Attributes := help.Attributes + [caSpansMultipleFiles];

              // Merge everything else

              help.FramesReceived := help.FramesReceived + value.FramesReceived;
              help.FramesSent := help.FramesSent + value.FramesSent;
              help.BytesReceived := help.BytesReceived + value.BytesReceived;
              help.BytesSent := help.BytesSent + value.BytesSent;

              // Update first frame values if the new entry is from the same or earlier file

              if (value.FirstFileID <= help.FirstFileID) then
                 begin
                      if (value.FirstFileID < help.FirstFileID) then
                         help.FirstFileID := value.FirstFileID;

                      if (value.FirstFrameIndex < help.FirstFrameIndex) then
                         help.FirstFrameIndex := value.FirstFrameIndex;

                      if (value.FirstFrameOffset < help.FirstFrameOffset) then
                         help.FirstFrameOffset := value.FirstFrameOffset;

                      if value.FirstFrameTime < help.FirstFrameTime then
                         help.FirstFrameTime := value.FirstFrameTime;
                 end;

              // Update last frame values if the new entry is from the same or later file

              if (value.LastFileID >= help.LastFileID) then
                 begin
                      if value.LastFileID > help.LastFileID then
                         help.LastFileID := value.LastFileID;

                      if (value.LastFrameIndex > help.LastFrameIndex) then
                         help.LastFrameIndex := value.LastFrameIndex;

                      if (value.LastFrameOffset > help.LastFrameOffset) then
                         help.LastFrameOffset := value.LastFrameOffset;

                      if value.LastFrameTime > help.LastFrameTime then
                         help.LastFrameTime := value.LastFrameTime;
                 end;

              for marker in value.Markers do
                  help.Markers := help.Markers + [marker];

              priv_dictMACEndpoints.Items[key] := help;
         end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddIPv4Endpoint (Value: tIPv4Endpoint);

var help   : tIPv4Endpoint;
    marker : tMarker;
    key    : string;

begin
     key := Value.DictionaryKey;

     if not priv_dictIPv4Endpoints.ContainsKey (key) then
        begin
             priv_dictIPv4Endpoints.Add (key, Value);
        end
     else
         begin
              help := priv_dictIPv4Endpoints.Items[key];

              if caSingleFrame in value.Attributes then
                 begin
                      // Check if we have an frame out of order situation if it isn't already

                      if not (caTimeOutOfOrder in help.attributes) then
                         begin
                              if (value.OriginFileIndex = help.LastFileID) and (value.OriginTimestamp < help.LastFrameTime) then
                                 help.Attributes := help.Attributes + [caTimeOutOfOrder];
                         end;

                      // Remove the SingleFrame attribute to avoid merging it later

                      value.Attributes := value.Attributes - [caSingleFrame];
                 end;

              // Check some other attributes, if we're still in the same file

              if (help.LastFileID = value.OriginFileIndex) and (help.InterfaceList[0].FileIndex = value.InterfaceList[0].FileIndex) then
                 begin
                      // Check if there is more than one interface used

                      if (help.InterfaceList[0].InterfaceFlags <> value.InterfaceList[0].InterfaceFlags) then
                         help.Attributes := help.Attributes + [caMultipleInterfaces];
                 end;

              // Make sure to clear the one sided attribute if at least one of both is has two way communication

              if caOneSidedCommunication in help.Attributes then
                 begin
                      if not (caOneSidedCommunication in value.Attributes) then
                         help.Attributes := help.Attributes - [caOneSidedCommunication];
                 end
              else
                  value.Attributes := value.Attributes - [caOneSidedCommunication];

              // Merge Attributesets

              help.Attributes := help.Attributes + value.Attributes;

              // If the existing entry was from a single frame, it won't be anymore after this merge

              if caSingleFrame in help.Attributes then
                 begin
                      help.Attributes := help.Attributes - [caSingleFrame];

                      // Clear helpers

                      help.OriginTimestamp.initialize;
                 end;

              // Always merge interface lists

              help.InterfaceList := mergeInterfaceLists (help.InterfaceList, value.InterfaceList);

              if not (caSpansMultipleFiles in help.Attributes) and (length (help.InterfaceList) > 1) then
                 help.Attributes := help.Attributes + [caSpansMultipleFiles];

              // Merge everything else

              help.FramesReceived := help.FramesReceived + value.FramesReceived;
              help.FramesSent := help.FramesSent + value.FramesSent;
              help.BytesReceived := help.BytesReceived + value.BytesReceived;
              help.BytesSent := help.BytesSent + value.BytesSent;

              // Update first frame values if the new entry is from the same or earlier file

              if (value.FirstFileID <= help.FirstFileID) then
                 begin
                      if (value.FirstFileID < help.FirstFileID) then
                         help.FirstFileID := value.FirstFileID;

                      if (value.FirstFrame < help.FirstFrame) then
                         help.FirstFrame := value.FirstFrame;

                      if (value.FirstFrameOffset < help.FirstFrameOffset) then
                         help.FirstFrameOffset := value.FirstFrameOffset;

                      if value.FirstFrameTime < help.FirstFrameTime then
                         help.FirstFrameTime := value.FirstFrameTime;
                 end;

              // Update last frame values if the new entry is from the same or later file

              if (value.LastFileID >= help.LastFileID) then
                 begin
                      if value.LastFileID > help.LastFileID then
                         help.LastFileID := value.LastFileID;

                      if (value.LastFrame > help.LastFrame) then
                         help.LastFrame := value.LastFrame;

                      if (value.LastFrameOffset > help.LastFrameOffset) then
                         help.LastFrameOffset := value.LastFrameOffset;

                      if value.LastFrameTime > help.LastFrameTime then
                         help.LastFrameTime := value.LastFrameTime;
                 end;

              // Set TTL, if not set yet, and if from an IPv4 layer not quoted on ICMP

              if (value.Markers = [mFoundByIPv4]) and (value.TTL > 0) then
                 begin
                      if (help.TTL = 0) then
                         help.TTL := value.TTL
                      else
                          if help.TTL <> value.TTL then
                             help.Attributes := help.Attributes + [caMultiTTL];
                 end;

              for marker in value.Markers do
                  help.Markers := help.Markers + [marker];

              priv_dictIPv4Endpoints.Items[key] := help;
         end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddIPv6Endpoint (Value: tIPv6Endpoint);

var help   : tIPv6Endpoint;
    marker : tMarker;
    key    : string;

begin
     key := Value.DictionaryKey;

     if not priv_dictIPv6Endpoints.ContainsKey (key) then
        begin
             priv_dictIPv6Endpoints.Add (key, Value);
        end
     else
         begin
              help := priv_dictIPv6Endpoints.Items[key];

              if caSingleFrame in value.Attributes then
                 begin
                      // Check if we have an frame out of order situation if it isn't already

                      if not (caTimeOutOfOrder in help.attributes) then
                         begin
                              if (value.OriginFileIndex = help.LastFileID) and (value.OriginTimestamp < help.LastFrameTime) then
                                 help.Attributes := help.Attributes + [caTimeOutOfOrder];
                         end;

                      // Remove the SingleFrame attribute to avoid merging it later

                      value.Attributes := value.Attributes - [caSingleFrame];
                 end;

              // Check some other attributes, if we're still in the same file

              if (help.LastFileID = value.OriginFileIndex) and (help.InterfaceList[0].FileIndex = value.InterfaceList[0].FileIndex) then
                 begin
                      // Check if there is more than one interface used

                      if (help.InterfaceList[0].InterfaceFlags <> value.InterfaceList[0].InterfaceFlags) then
                         help.Attributes := help.Attributes + [caMultipleInterfaces];
                 end;

              // Make sure to clear the one sided attribute if at least one of both is has two way communication

              if caOneSidedCommunication in help.Attributes then
                 begin
                      if not (caOneSidedCommunication in value.Attributes) then
                         help.Attributes := help.Attributes - [caOneSidedCommunication];
                 end
              else
                  value.Attributes := value.Attributes - [caOneSidedCommunication];

              // Merge Attributesets

              help.Attributes := help.Attributes + value.Attributes;

              // If the existing entry was from a single frame, it won't be anymore after this merge

              if caSingleFrame in help.Attributes then
                 begin
                      help.Attributes := help.Attributes - [caSingleFrame];

                      // Clear helpers

                      help.OriginTimestamp.initialize;
                 end;

              // Always merge interface lists

              help.InterfaceList := mergeInterfaceLists (help.InterfaceList, value.InterfaceList);

              if not (caSpansMultipleFiles in help.Attributes) and (length (help.InterfaceList) > 1) then
                 help.Attributes := help.Attributes + [caSpansMultipleFiles];

              // Merge everything else

              help.FramesReceived := help.FramesReceived + value.FramesReceived;
              help.FramesSent := help.FramesSent + value.FramesSent;
              help.BytesReceived := help.BytesReceived + value.BytesReceived;
              help.BytesSent := help.BytesSent + value.BytesSent;

              // Update first frame values if the new entry is from the same or earlier file

              if (value.FirstFileID <= help.FirstFileID) then
                 begin
                      if (value.FirstFileID < help.FirstFileID) then
                         help.FirstFileID := value.FirstFileID;

                      if (value.FirstFrame < help.FirstFrame) then
                         help.FirstFrame := value.FirstFrame;

                      if (value.FirstFrameOffset < help.FirstFrameOffset) then
                         help.FirstFrameOffset := value.FirstFrameOffset;

                      if value.FirstFrameTime < help.FirstFrameTime then
                         help.FirstFrameTime := value.FirstFrameTime;
                 end;

              // Update last frame values if the new entry is from the same or later file

              if (value.LastFileID >= help.LastFileID) then
                 begin
                      if value.LastFileID > help.LastFileID then
                         help.LastFileID := value.LastFileID;

                      if (value.LastFrame > help.LastFrame) then
                         help.LastFrame := value.LastFrame;

                      if (value.LastFrameOffset > help.LastFrameOffset) then
                         help.LastFrameOffset := value.LastFrameOffset;

                      if value.LastFrameTime > help.LastFrameTime then
                         help.LastFrameTime := value.LastFrameTime;
                 end;

              for marker in value.Markers do
                  help.Markers := help.Markers + [marker];

              // Set TTL, if not set yet, and if from an IPv4 layer not quoted on ICMP

              if (value.Markers = [mFoundByIPv6]) and (value.HopLimit > 0) then
                 begin
                      if (help.HopLimit = 0) then
                         help.HopLimit := value.HopLimit
                      else
                          if help.HopLimit <> value.HopLimit then
                             help.Attributes := help.Attributes + [caMultiTTL];
                 end;

              priv_dictIPv6Endpoints.Items[key] := help;
         end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddMACConversation (Value: tMACConversation);

var help   : tMACConversation;
    marker : tMarker;
    key    : string;

begin
     key := value.DictionaryKey;

     if not priv_dictMACConversations.ContainsKey (key) then
        begin
             priv_dictMACConversations.Add (key, value);
        end
     else
         begin
              help := priv_dictMACConversations.Items[key];

              // Since we already have another (previous) packet, the flipped value is irrelevant as that only interest us for the FIRST packet

              value.Attributes := value.Attributes - [caAddressPairFlipped];

              if caSingleFrame in value.Attributes then
                 begin
                      // Check if we have an frame out of order situation if it isn't already

                      if not (caTimeOutOfOrder in help.attributes) then
                         begin
                              if (value.OriginFileIndex = help.LastFileID) and (value.OriginTimestamp < help.LastFrameTime) then
                                 help.Attributes := help.Attributes + [caTimeOutOfOrder];
                         end;

                      // Check if we have a two way communication

                      if (caOneSidedCommunication in help.Attributes) then
                         begin
                              // compare source addresses, because comparing the pairs does not care about source/destination swaps

                              if not (value.OriginAddressPair.Source = help.OriginAddressPair.Source) then
                                 help.Attributes := help.Attributes - [caOneSidedCommunication];
                         end;

                      // Remove the SingleFrame attribute to avoid merging it later

                      value.Attributes := value.Attributes - [caSingleFrame];
                 end;

              // Check some other attributes, if we're still in the same file

              if (help.LastFileID = value.OriginFileIndex) and (help.InterfaceList[0].FileIndex = value.InterfaceList[0].FileIndex) then
                 begin
                      // Check if there is more than one interface used

                      if (help.InterfaceList[0].InterfaceFlags <> value.InterfaceList[0].InterfaceFlags) then
                         help.Attributes := help.Attributes + [caMultipleInterfaces];
                 end;

              // Make sure to clear the one sided attribute if at least one of both is has two way communication

              if caOneSidedCommunication in help.Attributes then
                 begin
                      if not (caOneSidedCommunication in value.Attributes) then
                         help.Attributes := help.Attributes - [caOneSidedCommunication];
                 end
              else
                  value.Attributes := value.Attributes - [caOneSidedCommunication];

              // Merge Attributesets

              help.Attributes := help.Attributes + value.Attributes;

              // If the existing entry was from a single frame, it won't be anymore after this merge

              if caSingleFrame in help.Attributes then
                 begin
                      help.Attributes := help.Attributes - [caSingleFrame];

                      // Clear helpers

                      help.OriginTimestamp.initialize;
                 end;

              // Always merge interface lists

              help.InterfaceList := mergeInterfaceLists (help.InterfaceList, value.InterfaceList);

              if not (caSpansMultipleFiles in help.Attributes) and (length (help.InterfaceList) > 1) then
                 help.Attributes := help.Attributes + [caSpansMultipleFiles];

              // Merge everything else

              help.FramesSrcToDst := help.FramesSrcToDst + value.FramesSrcToDst;
              help.FramesDstToSrc := help.FramesDstToSrc + value.FramesDstToSrc;
              help.BytesSrcToDst := help.BytesSrcToDst + value.BytesSrcToDst;
              help.BytesDstToSrc := help.BytesDstToSrc + value.BytesDstToSrc;

              // Update first frame values if the new entry is from the same or earlier file

              if (value.FirstFileID <= help.FirstFileID) then
                 begin
                      if (value.FirstFileID < help.FirstFileID) then
                         help.FirstFileID := value.FirstFileID;

                      if (value.FirstFrame < help.FirstFrame) then
                         help.FirstFrame := value.FirstFrame;

                      if (value.FirstFrameOffset < help.FirstFrameOffset) then
                         help.FirstFrameOffset := value.FirstFrameOffset;

                      if value.FirstFrameTime < help.FirstFrameTime then
                         help.FirstFrameTime := value.FirstFrameTime;
                 end;

              // Update last frame values if the new entry is from the same or later file

              if (value.LastFileID >= help.LastFileID) then
                 begin
                      if value.LastFileID > help.LastFileID then
                         help.LastFileID := value.LastFileID;

                      if (value.LastFrame > help.LastFrame) then
                         help.LastFrame := value.LastFrame;

                      if (value.LastFrameOffset > help.LastFrameOffset) then
                         help.LastFrameOffset := value.LastFrameOffset;

                      if value.LastFrameTime > help.LastFrameTime then
                         help.LastFrameTime := value.LastFrameTime;
                 end;

              for marker in value.Markers do
                  help.Markers := help.Markers + [marker];

              priv_dictMACConversations.Items[key] := help;
         end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddIPv4Conversation (Value: tIPv4Conversation);

var help   : tIPv4Conversation;
    marker : tMarker;
    key    : string;

begin
     key := Value.DictionaryKey;

     if not priv_dictIPv4Conversations.ContainsKey (key) then
        begin
             priv_dictIPv4Conversations.Add(key, value);
        end
     else
         begin
              help := priv_dictIPv4Conversations.Items[key];

              // Since we already have another (previous) packet, the flipped value is irrelevant as that only interest us for the FIRST packet

              value.Attributes := value.Attributes - [caAddressPairFlipped];

              if caSingleFrame in value.Attributes then
                 begin
                      // Check if we have an frame out of order situation if it isn't already

                      if not (caTimeOutOfOrder in help.attributes) then
                         begin
                              if (value.OriginFileIndex = help.LastFileID) and (value.OriginTimestamp < help.LastFrameTime) then
                                 help.Attributes := help.Attributes + [caTimeOutOfOrder];
                         end;

                      // Check if we have a two way communication

                      if (caOneSidedCommunication in help.Attributes) then
                         begin
                              // compare source addresses, because comparing the pairs does not care about source/destination swaps

                              if not (value.OriginAddressPair.Source = help.OriginAddressPair.Source) then
                                 help.Attributes := help.Attributes - [caOneSidedCommunication];
                         end;

                      // Remove the SingleFrame attribute to avoid merging it later

                      value.Attributes := value.Attributes - [caSingleFrame];
                 end;

              // Check some other attributes, if we're still in the same file

              if (help.LastFileID = value.OriginFileIndex) and (help.InterfaceList[0].FileIndex = value.InterfaceList[0].FileIndex) then
                 begin
                      // Check if there is more than one interface used

                      if (help.InterfaceList[0].InterfaceFlags <> value.InterfaceList[0].InterfaceFlags) then
                         help.Attributes := help.Attributes + [caMultipleInterfaces];
                 end;

              // Make sure to clear the one sided attribute if at least one of both is has two way communication

              if caOneSidedCommunication in help.Attributes then
                 begin
                      if not (caOneSidedCommunication in value.Attributes) then
                         help.Attributes := help.Attributes - [caOneSidedCommunication];
                 end
              else
                  value.Attributes := value.Attributes - [caOneSidedCommunication];

              // Merge Attributesets

              help.Attributes := help.Attributes + value.Attributes;

              // If the existing entry was from a single frame, it won't be anymore after this merge

              if caSingleFrame in help.Attributes then
                 begin
                      help.Attributes := help.Attributes - [caSingleFrame];

                      // Clear helpers

                      help.OriginTimestamp.initialize;
                 end;

              // Always merge interface lists

              help.InterfaceList := mergeInterfaceLists (help.InterfaceList, value.InterfaceList);

              if not (caSpansMultipleFiles in help.Attributes) and (length (help.InterfaceList) > 1) then
                 help.Attributes := help.Attributes + [caSpansMultipleFiles];

              // Merge everything else

              help.FramesSrcToDst := help.FramesSrcToDst + value.FramesSrcToDst;
              help.FramesDstToSrc := help.FramesDstToSrc + value.FramesDstToSrc;
              help.BytesSrcToDst := help.BytesSrcToDst + value.BytesSrcToDst;
              help.BytesDstToSrc := help.BytesDstToSrc + value.BytesDstToSrc;

              // Update first frame values if the new entry is from the same or earlier file

              if (value.FirstFileID <= help.FirstFileID) then
                 begin
                      if (value.FirstFileID < help.FirstFileID) then
                         help.FirstFileID := value.FirstFileID;

                      if (value.FirstFrame < help.FirstFrame) then
                         help.FirstFrame := value.FirstFrame;

                      if (value.FirstFrameOffset < help.FirstFrameOffset) then
                         help.FirstFrameOffset := value.FirstFrameOffset;

                      if value.FirstFrameTime < help.FirstFrameTime then
                         help.FirstFrameTime := value.FirstFrameTime;
                 end;

              // Update last frame values if the new entry is from the same or later file

              if (value.LastFileID >= help.LastFileID) then
                 begin
                      if value.LastFileID > help.LastFileID then
                         help.LastFileID := value.LastFileID;

                      if (value.LastFrame > help.LastFrame) then
                         help.LastFrame := value.LastFrame;

                      if (value.LastFrameOffset > help.LastFrameOffset) then
                         help.LastFrameOffset := value.LastFrameOffset;

                      if value.LastFrameTime > help.LastFrameTime then
                         help.LastFrameTime := value.LastFrameTime;
                 end;

              for marker in value.Markers do
                  help.Markers := help.Markers + [marker];

              help.FlowCount := help.FlowCount + Value.FlowCount;

              priv_dictIPv4Conversations.Items[key] := help;
         end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddIPv6Conversation (Value: tIPv6Conversation);

var help   : tIPv6Conversation;
    marker : tMarker;
    key    : string;

begin
     key := Value.DictionaryKey;

     if not priv_dictIPv6Conversations.ContainsKey (key) then
        begin
             priv_dictIPv6Conversations.Add(key, value);
        end
     else
         begin
              help := priv_dictIPv6Conversations.Items[key];

              // Since we already have another (previous) packet, the flipped value is irrelevant as that only interest us for the FIRST packet

              value.Attributes := value.Attributes - [caAddressPairFlipped];

              if caSingleFrame in value.Attributes then
                 begin
                      // Check if we have an frame out of order situation if it isn't already

                      if not (caTimeOutOfOrder in help.attributes) then
                         begin
                              if (value.OriginFileIndex = help.LastFileID) and (value.OriginTimestamp < help.LastFrameTime) then
                                 help.Attributes := help.Attributes + [caTimeOutOfOrder];
                         end;

                      // Check if we have a two way communication

                      if (caOneSidedCommunication in help.Attributes) then
                         begin
                              // compare source addresses, because comparing the pairs does not care about source/destination swaps

                              if not (value.OriginAddressPair.Source = help.OriginAddressPair.Source) then
                                 help.Attributes := help.Attributes - [caOneSidedCommunication];
                         end;

                      // Remove the SingleFrame attribute to avoid merging it later

                      value.Attributes := value.Attributes - [caSingleFrame];
                 end;

              // Check some other attributes, if we're still in the same file

              if (help.LastFileID = value.OriginFileIndex) and (help.InterfaceList[0].FileIndex = value.InterfaceList[0].FileIndex) then
                 begin
                      // Check if there is more than one interface used

                      if (help.InterfaceList[0].InterfaceFlags <> value.InterfaceList[0].InterfaceFlags) then
                         help.Attributes := help.Attributes + [caMultipleInterfaces];
                 end;

              // Make sure to clear the one sided attribute if at least one of both is has two way communication

              if caOneSidedCommunication in help.Attributes then
                 begin
                      if not (caOneSidedCommunication in value.Attributes) then
                         help.Attributes := help.Attributes - [caOneSidedCommunication];
                 end
              else
                  value.Attributes := value.Attributes - [caOneSidedCommunication];

              // Merge Attributesets

              help.Attributes := help.Attributes + value.Attributes;

              // If the existing entry was from a single frame, it won't be anymore after this merge

              if caSingleFrame in help.Attributes then
                 begin
                      help.Attributes := help.Attributes - [caSingleFrame];

                      // Clear helpers

                      help.OriginTimestamp.initialize;
                 end;

              // Always merge interface lists

              help.InterfaceList := mergeInterfaceLists (help.InterfaceList, value.InterfaceList);

              if not (caSpansMultipleFiles in help.Attributes) and (length (help.InterfaceList) > 1) then
                 help.Attributes := help.Attributes + [caSpansMultipleFiles];

              // Merge everything else

              help.FramesSrcToDst := help.FramesSrcToDst + value.FramesSrcToDst;
              help.FramesDstToSrc := help.FramesDstToSrc + value.FramesDstToSrc;
              help.BytesSrcToDst := help.BytesSrcToDst + value.BytesSrcToDst;
              help.BytesDstToSrc := help.BytesDstToSrc + value.BytesDstToSrc;

              // Update first frame values if the new entry is from the same or earlier file

              if (value.FirstFileID <= help.FirstFileID) then
                 begin
                      if (value.FirstFileID < help.FirstFileID) then
                         help.FirstFileID := value.FirstFileID;

                      if (value.FirstFrame < help.FirstFrame) then
                         help.FirstFrame := value.FirstFrame;

                      if (value.FirstFrameOffset < help.FirstFrameOffset) then
                         help.FirstFrameOffset := value.FirstFrameOffset;

                      if value.FirstFrameTime < help.FirstFrameTime then
                         help.FirstFrameTime := value.FirstFrameTime;
                 end;

              // Update last frame values if the new entry is from the same or later file

              if (value.LastFileID >= help.LastFileID) then
                 begin
                      if value.LastFileID > help.LastFileID then
                         help.LastFileID := value.LastFileID;

                      if (value.LastFrame > help.LastFrame) then
                         help.LastFrame := value.LastFrame;

                      if (value.LastFrameOffset > help.LastFrameOffset) then
                         help.LastFrameOffset := value.LastFrameOffset;

                      if value.LastFrameTime > help.LastFrameTime then
                         help.LastFrameTime := value.LastFrameTime;
                 end;

              for marker in value.Markers do
                  help.Markers := help.Markers + [marker];

              help.FlowCount := help.FlowCount + Value.FlowCount;

              priv_dictIPv6Conversations.Items[key] := help;
         end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddTCPConversation (value: tTCPConversation);

var help            : tTCPConversation;
    marker          : tMarker;
    key             : string;
    ValueFroMClient : boolean;
    ValueFromServer : boolean;
    IPMatch         : boolean;
    delay           : integer;
    secs            : uint64;

begin
     key := value.DictionaryKey;

     if not priv_dictTCPConversations.ContainsKey (key) then
        begin
             // So this is new. Check if a SYN flag is set, because if not, we're missing the handshake apparently
             // in which case we try to determine who's the client from the ports used

             if  not (tcpSYN in value.Flags) then
                 value.ClientIP := DetermineProbableClientAddress (value.SocketPair);

             priv_dictTCPConversations.Add (key, Value);
        end
     else
         begin
              help := priv_dictTCPConversations.Items[key];

              // For some processing we need to know if the new value is from the client or the server
              // Since the socket pair was normalized it may have been flipped

              if not help.ClientIP.isZeroAddress then
                 begin
                      if (caAddressPairFlipped in Value.Attributes) then
                         IPMatch := (help.ClientIP = value.SocketPair.Destination.IPAddress)
                      else
                          IPMatch := (help.ClientIP = value.SocketPair.Source.IPAddress);

                      ValueFromClient := IPMatch;
                      ValueFromServer := not IPMatch;

                      if ValueFromClient then
                         help.ClientPayloadBytes := help.ClientPayloadBytes + value.PayloadBytes
                      else
                          help.ServerPayloadBytes := help.ServerPayloadBytes + value.PayloadBytes
                 end
              else
                  begin
                       ValueFromClient := false;
                       ValueFromServer := false;
                  end;

              // Since we already have another (previous) packet, the flipped value is irrelevant as that only interest us for the FIRST packet

              value.Attributes := value.Attributes - [caAddressPairFlipped];

              // Some tests that are only relevant if adding a new single frame (and not merging multiples)

              if caSingleFrame in value.Attributes then
                 begin
                      // Check if we have an frame out of order situation if it isn't already

                      if not (caTimeOutOfOrder in help.attributes) then
                         begin
                              if (value.OriginFileIndex = help.LastFileID) and (value.OriginTimestamp < help.LastFrameTime) then
                                 help.Attributes := help.Attributes + [caTimeOutOfOrder];
                         end;

                      // Check if we have a two way communication

                      if (caOneSidedCommunication in help.Attributes) then
                         begin
                              // compare source addresses, because comparing the pairs does not care about source/destination swaps

                              if not (value.OriginAddressPair.Source = help.OriginAddressPair.Source) then
                                 help.Attributes := help.Attributes - [caOneSidedCommunication];
                         end;

                      // Remove the SingleFrame attribute to avoid merging it later

                      value.Attributes := value.Attributes - [caSingleFrame];
                 end;

              // Check some other attributes, if we're still in the same file

              if (help.LastFileID = value.OriginFileIndex) and (help.InterfaceList[0].FileIndex = value.InterfaceList[0].FileIndex) then
                 begin
                      // Check if there is more than one interface used

                      if (help.InterfaceList[0].InterfaceFlags <> value.InterfaceList[0].InterfaceFlags) then
                         help.Attributes := help.Attributes + [caMultipleInterfaces];
                 end;

              // Make sure to clear the one sided attribute if at least one of both is has two way communication

              if caOneSidedCommunication in help.Attributes then
                 begin
                      if not (caOneSidedCommunication in value.Attributes) then
                         help.Attributes := help.Attributes - [caOneSidedCommunication];
                 end
              else
                  value.Attributes := value.Attributes - [caOneSidedCommunication];

              // Merge Attributesets

              help.Attributes := help.Attributes + value.Attributes;

              // If the existing entry was from a single frame, it won't be anymore after this merge

              if caSingleFrame in help.Attributes then
                 begin
                      help.Attributes := help.Attributes - [caSingleFrame];

                      // Clear helpers

                      help.OriginTimestamp.initialize;
                 end;

              // Determine if conversation spans across multiple files

              if not (caSpansMultipleFiles in help.Attributes) then
                 begin
                      if not InterfaceListsAreIdentical (help.InterfaceList, Value.InterfaceList) then
                         help.Attributes := help.Attributes + [caSpansMultipleFiles];
                 end;

              // Always merge interface lists

              help.InterfaceList := mergeInterfaceLists (help.InterfaceList, value.InterfaceList);

              // Merge everything else

              help.FramesSrcToDst := help.FramesSrcToDst + value.FramesSrcToDst;
              help.FramesDstToSrc := help.FramesDstToSrc + value.FramesDstToSrc;
              help.BytesSrcToDst := help.BytesSrcToDst + value.BytesSrcToDst;
              help.BytesDstToSrc := help.BytesDstToSrc + value.BytesDstToSrc;

              if not value.ClientIP.isZeroAddress and help.ClientIP.isZeroAddress then
                 help.ClientIP := value.ClientIP;

              // Update first frame values if the new entry is from the same or earlier file

              if (value.FirstFileID <= help.FirstFileID) then
                 begin
                      if (value.FirstFileID < help.FirstFileID) then
                         help.FirstFileID := value.FirstFileID;

                      if (value.FirstFrame < help.FirstFrame) then
                         help.FirstFrame := value.FirstFrame;

                      if (value.FirstFrameOffset < help.FirstFrameOffset) then
                         help.FirstFrameOffset := value.FirstFrameOffset;

                      if value.FirstFrameTime < help.FirstFrameTime then
                         help.FirstFrameTime := value.FirstFrameTime;
                 end;

              // Update last frame values if the new entry is from the same or later file

              if (value.LastFileID >= help.LastFileID) then
                 begin
                      if value.LastFileID > help.LastFileID then
                         help.LastFileID := value.LastFileID;

                      if (value.LastFrame > help.LastFrame) then
                         help.LastFrame := value.LastFrame;

                      if (value.LastFrameOffset > help.LastFrameOffset) then
                         help.LastFrameOffset := value.LastFrameOffset;

                      if value.LastFrameTime > help.LastFrameTime then
                         help.LastFrameTime := value.LastFrameTime;
                 end;

              // Update TCP things

              if value.SynSeen then
                 help.SynSeen := true;

              if value.FinSeen then
                 help.FinSeen := true;

              if value.RstSeen then
                 help.RstSeen := true;

              for marker in value.Markers do
                  help.Markers := help.Markers + [marker];

              // Three way handshake stuff and iRTT calculation

              if (help.SessionStatus = TCPHandshakeSYNSent) then
                 begin
                      // If this is a packet from the server do some handshake checks

                      if value.ClientIP <> help.ClientIP then
                         begin
                              // See if we have a initial SYN coming from the other guy, too

                              if (value.SessionStatus = TCPHandshakeSYNSent) then
                                 help.SessionStatus := TCPHandshakeSYNTwoWay;

                              // See if this is the SYN/ACK we are waiting for

                              if (value.SessionStatus = TCPHandshakeSYNACKSent) then
                                 help.SessionStatus := TCPHandshakeHalfOpen;

                              // Check for Session Refuse

                              if value.RstSeen then
                                 begin
                                      help.SessionStatus := TCPHandshakeSYNReset;
                                      help.Attributes := help.Attributes + [caTCPSYNReset];
                                 end;
                         end
                      else
                          begin
                               // Check if we have a SYN retry

                               if (value.SessionStatus = TCPHandshakeSYNSent) and (value.InitialSequenceNo = help.InitialSequenceNo) then
                                  begin
                                       inc (help.SynRetries);
                                       help.Attributes := help.Attributes + [caTCPMultiSYNRetry];
                                  end;
                          end;
                 end
              else
                  // Maybe this is port reuse - only check if we have complete conversations

                  if (help.SessionStatus = TCPHandshakeAndTeardown) and (value.SynSeen and (Value.SynTimestamp <> help.SynTimestamp)) then
                     begin
                          help.Attributes := help.Attributes + [caTCPPortReused];

                     end;

              // Check if we have a complete handshake and a session teardown, no matter if complete or not

              if (value.FinSeen or value.RstSeen) then
                 begin
                      help.FinTimestamp := value.FirstFrameTime;

                      case help.SessionStatus of
                       TCPHandshakeNone        : help.SessionStatus := TCPTeardown;
                       TCPHandshakeHalfOpen    : help.SessionStatus := TCPHalfOpenReset;
                       TCPHandshakeComplete    : begin
                                                      // Check if the session is torn down without any payload transfer (socket abort)

                                                      if (help.ClientPayloadBytes = 0) and (help.ServerPayloadBytes = 0) and ValueFromServer then
                                                         begin
                                                              // This is either a socket refuse or a teardown due to inactivity. We look at timings to decide which one it is
                                                              // We may not always have the time stamp of the third packet (when loading from DB), so skip it in that case

                                                              if not (help.HandshakeThirdTime.datetime = 0) then
                                                                 begin
                                                                      delay := secondsbetween (value.FirstFrameTime.datetime, help.HandshakeThirdTime.datetime);

                                                                      // The delay must be at least 1 second (arbitrary value), otherwise this is not a timeout

                                                                      if delay > 0 then
                                                                         begin
                                                                              secs := (help.InitialRTTns div 1000000000);

                                                                              // Check against twice the RTT

                                                                              if (delay > (2 * secs)) then
                                                                                 help.SessionStatus := TCPSocketCloseByInactivity
                                                                              else
                                                                                  help.SessionStatus := TCPSocketRefused;
                                                                         end
                                                                      else
                                                                          help.SessionStatus := TCPSocketRefused;
                                                                 end
                                                              else
                                                                  help.SessionStatus := TCPHandshakeAndTeardown;
                                                         end
                                                      else
                                                          help.SessionStatus := TCPHandshakeAndTeardown;
                                                 end;
                       TCPHandshakeAndTeardown : help.SessionStatus := TCPHandshakeAndTeardown;
                      end;
                 end;

              // Calculate InitialRTT

              if (help.InitialRTTns = 0) then
                 begin
                      // If this is a retransmitted SYN we update it parameters for RTT calculation
                      // If this is just a SYN/ACK the InitialSequenceNo is 0, so we ignore it

                      if value.SynSeen and (value.InitialSequenceNo > 0) then
                         begin
                              help.InitialSequenceNo := value.InitialSequenceNo;
                              help.SynTimestamp := value.SynTimestamp;
                         end;

                      // If this is the first packet without a SYN flag and we have seen the syn flag we try to determine iRTT

                      if (not (value.SynSeen or value.FinSeen or value.RstSeen)) and (help.InitialSequenceNo > 0) then
                         begin
                              // Check if our handshake is complete by comparing sequence numbers

                              if (value.LastSeenSequence = help.HandshakeThird) then
                                 begin
                                      help.InitialRTTns := nanosecondsbetween (value.FirstFrameTime, help.SynTimestamp);
                                      help.SessionStatus := TCPHandshakeComplete;

                                      // Note the handshake complete absolute timestamp

                                      help.HandshakeThirdTime := value.FirstFrameTime;

                                      inc (help.HandshakeCount);

                                      if help.HandshakeCount > 1 then
                                         help.Attributes := help.Attributes + [caTCPMultiSession];
                                 end;
                         end;
                 end;

              // Merge Flags

              help.Flags := help.Flags + Value.Flags;

              priv_dictTCPConversations.Items[key] := help;
         end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddUDPConversation (value: tUDPConversation);

var help   : tUDPConversation;
    marker : tMarker;
    key    : string;

begin
     key := Value.DictionaryKey;

     if not priv_dictUDPConversations.ContainsKey (key) then
        begin
             priv_dictUDPConversations.Add (key, Value);
        end
     else
         begin
              help := priv_dictUDPConversations.Items[key];

              // Since we already have another (previous) packet, the flipped value is irrelevant as that only interest us for the FIRST packet

              value.Attributes := value.Attributes - [caAddressPairFlipped];

              if caSingleFrame in value.Attributes then
                 begin
                      // Check if we have an frame out of order situation if it isn't already

                      if not (caTimeOutOfOrder in help.attributes) then
                         begin
                              if (value.OriginFileIndex = help.LastFileID) and (value.OriginTimestamp < help.LastFrameTime) then
                                 help.Attributes := help.Attributes + [caTimeOutOfOrder];
                         end;

                      // Check if we have a two way communication

                      if (caOneSidedCommunication in help.Attributes) then
                         begin
                              // compare source addresses, because comparing the pairs does not care about source/destination swaps

                              if not (value.OriginAddressPair.Source = help.OriginAddressPair.Source) then
                                 help.Attributes := help.Attributes - [caOneSidedCommunication];
                         end;

                      // Remove the SingleFrame attribute to avoid merging it later

                      value.Attributes := value.Attributes - [caSingleFrame];
                 end;

              // Check some other attributes, if we're still in the same file

              if (help.LastFileID = value.OriginFileIndex) and (help.InterfaceList[0].FileIndex = value.InterfaceList[0].FileIndex) then
                 begin
                      // Check if there is more than one interface used

                      if (help.InterfaceList[0].InterfaceFlags <> value.InterfaceList[0].InterfaceFlags) then
                         help.Attributes := help.Attributes + [caMultipleInterfaces];
                 end;

              // Make sure to clear the one sided attribute if at least one of both is has two way communication

              if caOneSidedCommunication in help.Attributes then
                 begin
                      if not (caOneSidedCommunication in value.Attributes) then
                         help.Attributes := help.Attributes - [caOneSidedCommunication];
                 end
              else
                  value.Attributes := value.Attributes - [caOneSidedCommunication];

              // Merge Attributesets

              help.Attributes := help.Attributes + value.Attributes;

              // If the existing entry was from a single frame, it won't be anymore after this merge

              if caSingleFrame in help.Attributes then
                 begin
                      help.Attributes := help.Attributes - [caSingleFrame];

                      // Clear helpers

                      help.OriginTimestamp.initialize;
                 end;

              // Always merge interface lists

              help.InterfaceList := mergeInterfaceLists (help.InterfaceList, value.InterfaceList);

              if not (caSpansMultipleFiles in help.Attributes) and (length (help.InterfaceList) > 1) then
                 help.Attributes := help.Attributes + [caSpansMultipleFiles];

              // Merge everything else

              help.FramesSrcToDst := help.FramesSrcToDst + value.FramesSrcToDst;
              help.FramesDstToSrc := help.FramesDstToSrc + value.FramesDstToSrc;
              help.BytesSrcToDst := help.BytesSrcToDst + value.BytesSrcToDst;
              help.BytesDstToSrc := help.BytesDstToSrc + value.BytesDstToSrc;

              // Update first frame values if the new entry is from the same or earlier file

              if (value.FirstFileID <= help.FirstFileID) then
                 begin
                      if (value.FirstFileID < help.FirstFileID) then
                         help.FirstFileID := value.FirstFileID;

                      if (value.FirstFrame < help.FirstFrame) then
                         help.FirstFrame := value.FirstFrame;

                      if (value.FirstFrameOffset < help.FirstFrameOffset) then
                         help.FirstFrameOffset := value.FirstFrameOffset;

                      if value.FirstFrameTime < help.FirstFrameTime then
                         help.FirstFrameTime := value.FirstFrameTime;
                 end;

              // Update last frame values if the new entry is from the same or later file

              if (value.LastFileID >= help.LastFileID) then
                 begin
                      if value.LastFileID > help.LastFileID then
                         help.LastFileID := value.LastFileID;

                      if (value.LastFrame > help.LastFrame) then
                         help.LastFrame := value.LastFrame;

                      if (value.LastFrameOffset > help.LastFrameOffset) then
                         help.LastFrameOffset := value.LastFrameOffset;

                      if value.LastFrameTime > help.LastFrameTime then
                         help.LastFrameTime := value.LastFrameTime;
                 end;

              for marker in value.Markers do
                  help.Markers := help.Markers + [marker];

              priv_dictUDPConversations.Items[key] := help;
         end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddNameResolution (Value: tNameResolutionEntry);

var help   : tNameResolutionEntry;
    marker : tMarker;
    key    : string;

begin
     key := value.DictionaryKey;

     if not priv_dictNameResolutions.ContainsKey (key) then
        begin
             priv_dictNameResolutions.Add (key, Value);
        end
     else
         begin
              help := priv_dictNameResolutions.Items[key];

              help.FrameCount := help.FrameCount + value.FrameCount;

              // Update first frame values if the new entry is from the same or earlier file

              if (value.FirstFileID <= help.FirstFileID) then
                 begin
                      if (value.FirstFileID < help.FirstFileID) then
                         help.FirstFileID := value.FirstFileID;

                      if (value.FirstFrame < help.FirstFrame) then
                         help.FirstFrame := value.FirstFrame;

                      if (value.FirstFrameOffset < help.FirstFrameOffset) then
                         help.FirstFrameOffset := value.FirstFrameOffset;

                      if value.FirstFrameTime < help.FirstFrameTime then
                         help.FirstFrameTime := value.FirstFrameTime;
                 end;

              // Update last frame values if the new entry is from the same or later file

              if (value.LastFileID >= help.LastFileID) then
                 begin
                      if value.LastFileID > help.LastFileID then
                         help.LastFileID := value.LastFileID;

                      if (value.LastFrame > help.LastFrame) then
                         help.LastFrame := value.LastFrame;

                      if (value.LastFrameOffset > help.LastFrameOffset) then
                         help.LastFrameOffset := value.LastFrameOffset;

                      if value.LastFrameTime > help.LastFrameTime then
                         help.LastFrameTime := value.LastFrameTime;
                 end;

              help.QueryTypes := help.QueryTypes + value.QueryTypes;

              help.Attributes := help.Attributes + value.Attributes;
              help.Markers := help.Markers + value.Markers;

              priv_dictNameResolutions.Items[key] := help;
         end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddEthernetDetails (parser: tEthernetParser; InterfaceIndex : tAbsoluteFileInterface; FrameDetails : tFrameDetails);

var endpoint     : tMACEndpoint;
    conversation : tMACConversation;

begin
     // Source MAC

     endpoint.initialize (InterfaceIndex.FileIndex);

     setlength (endpoint.InterfaceList, 1);
//   endpoint.InterfaceList[0].initialize (0, InterfaceIndex);              // don't know the index here, will be determined on writing to DB
     endpoint.InterfaceList[0].initialize (InterfaceIndex);

     endpoint.MACAddress := Parser.SourceAddress;
     endpoint.FramesSent := 1;
     endpoint.BytesSent := FrameDetails.WireSize;
     endpoint.FirstFrameIndex := FrameDetails.FileFrameID;
     endpoint.LastFrameIndex := FrameDetails.FileFrameID;
     endpoint.FirstFrameOffset := FrameDetails.FileOffset;
     endpoint.LastFrameOffset := FrameDetails.FileOffset;
     endpoint.FirstFrameTime := FrameDetails.FrameTimestamp;
     endpoint.LastFrameTime := FrameDetails.FrameTimestamp;
     endpoint.Markers := [mFoundByEthernet];
//   endpoint.Attributes := [caSingleFrame];
     endpoint.Attributes := FrameDetails.Attributes;

     endpoint.OriginTimestamp := FrameDetails.FrameTimestamp;

     AddMACEndpoint (endpoint);

     // Destination MAC

     endpoint.initialize (InterfaceIndex.FileIndex);

     setlength (endpoint.InterfaceList, 1);
//   endpoint.InterfaceList[0].initialize (0, InterfaceIndex);              // don't know the index here, will be determined on writing to DB
     endpoint.InterfaceList[0].initialize (InterfaceIndex);

     endpoint.MACAddress := Parser.DestinationAddress;
     endpoint.FramesReceived := 1;
     endpoint.BytesReceived := FrameDetails.WireSize;
     endpoint.FirstFrameIndex := FrameDetails.FileFrameID;
     endpoint.LastFrameIndex := FrameDetails.FileFrameID;
     endpoint.FirstFrameOffset := FrameDetails.FileOffset;
     endpoint.LastFrameOffset := FrameDetails.FileOffset;
     endpoint.FirstFrameTime := FrameDetails.FrameTimestamp;
     endpoint.LastFrameTime := FrameDetails.FrameTimestamp;
     endpoint.Markers := [mFoundByEthernet];
//   endpoint.Attributes := [caSingleFrame];
     endpoint.Attributes := FrameDetails.Attributes;

     endpoint.OriginTimestamp := FrameDetails.FrameTimestamp;

     AddMACEndpoint (endpoint);

     // Conversation

     conversation.initialize (InterfaceIndex.FileIndex);

     setlength (conversation.InterfaceList, 1);
//   conversation.InterfaceList[0].initialize (0, InterfaceIndex);              // don't know the index here, will be determined on writing to DB
     conversation.InterfaceList[0].initialize (InterfaceIndex);

     conversation.MACPair.Source := parser.SourceAddress;
     conversation.MACPair.Destination := parser.DestinationAddress;
     conversation.OriginAddressPair := conversation.MACPair;
     conversation.MACPair.normalize;

     if conversation.MACPair.Source = Parser.SourceAddress then
        begin
             conversation.BytesSrcToDst := FrameDetails.WireSize;
             conversation.FramesSrcToDst := 1;
        end
     else
         begin
              conversation.BytesDstToSrc := FrameDetails.WireSize;
              conversation.FramesDstToSrc := 1;
         end;

     conversation.FirstFrame := FrameDetails.FileFrameID;
     conversation.LastFrame := FrameDetails.FileFrameID;
     conversation.FirstFrameOffset := FrameDetails.FileOffset;
     conversation.LastFrameOffset := FrameDetails.FileOffset;
     conversation.FirstFrameTime := FrameDetails.FrameTimestamp;
     conversation.LastFrameTime := FrameDetails.FrameTimestamp;
     conversation.Markers := [mFoundByEthernet];
     conversation.Attributes := FrameDetails.Attributes + [caOneSidedCommunication];
//   conversation.Attributes := [caSingleFrame, caOneSidedCommunication];

     if not (conversation.MACPair.Source = parser.SourceAddress) then
        conversation.Attributes := conversation.Attributes + [caAddressPairFlipped];

     conversation.OriginTimestamp := FrameDetails.FrameTimestamp;

     AddMACConversation (conversation);
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddARPDetails (parser: tARPParser; InterfaceIndex : tAbsoluteFileInterface; FrameDetails : tFrameDetails);

var MACEndpoint     : tMACEndpoint;
    MACConversation : tMACConversation;
    IPEndpoint      : tIPv4Endpoint;
    IPConversation  : tIPV4Conversation;

begin
     // Sender MAC

     MACEndpoint.initialize (InterfaceIndex.FileIndex);

     setlength (MACEndpoint.InterfaceList, 1);
//   MACEndpoint.InterfaceList[0].initialize (0, InterfaceIndex);              // don't know the index here, will be determined on writing to DB
     MACEndpoint.InterfaceList[0].initialize (InterfaceIndex);

     MACEndpoint.MACAddress := Parser.SenderMAC;
     MACEndpoint.FirstFrameIndex := FrameDetails.FileFrameID;
     MACEndpoint.LastFrameIndex := FrameDetails.FileFrameID;
     MACEndpoint.FirstFrameOffset := FrameDetails.FileOffset;
     MACEndpoint.LastFrameOffset := FrameDetails.FileOffset;
     MACEndpoint.FirstFrameTime := FrameDetails.FrameTimestamp;
     MACEndpoint.LastFrameTime := FrameDetails.FrameTimestamp;
     MACEndpoint.Markers := [mFoundByARP];
     MACEndpoint.OriginTimestamp := FrameDetails.FrameTimestamp;
     MACEndpoint.Attributes := FrameDetails.Attributes;

     AddMACEndpoint (MACEndpoint);

     // Target MAC

     MACEndpoint.initialize (InterfaceIndex.FileIndex);

     setlength (MACEndpoint.InterfaceList, 1);
//   MACEndpoint.InterfaceList[0].initialize (0, InterfaceIndex);              // don't know the index here, will be determined on writing to DB
     MACEndpoint.InterfaceList[0].initialize (InterfaceIndex);

     MACEndpoint.MACAddress := Parser.TargetMAC;
     MACEndpoint.FirstFrameIndex := FrameDetails.FileFrameID;
     MACEndpoint.LastFrameIndex := FrameDetails.FileFrameID;
     MACEndpoint.FirstFrameOffset := FrameDetails.FileOffset;
     MACEndpoint.LastFrameOffset := FrameDetails.FileOffset;
     MACEndpoint.FirstFrameTime := FrameDetails.FrameTimestamp;
     MACEndpoint.LastFrameTime := FrameDetails.FrameTimestamp;
     MACEndpoint.Markers := [mFoundByARP];
     MACEndpoint.OriginTimestamp := FrameDetails.FrameTimestamp;
     MACEndpoint.Attributes := FrameDetails.Attributes;

     AddMACEndpoint (MACEndpoint);

     // MAC Conversation (as zero transmission since its just ARP)

     MACConversation.initialize (InterfaceIndex.FileIndex);

     setlength (MACConversation.InterfaceList, 1);
//   MACConversation.InterfaceList[0].initialize (0, InterfaceIndex);              // don't know the index here, will be determined on writing to DB
     MACConversation.InterfaceList[0].initialize (InterfaceIndex);

     MACConversation.MACPair.Source := parser.SenderMAC;
     MACConversation.MACPair.Destination := parser.TargetMAC;
     MACConversation.OriginAddressPair := MACConversation.MACPair;
     MACConversation.MACPair.normalize;
     MACConversation.FirstFrame := FrameDetails.FileFrameID;
     MACConversation.LastFrame := FrameDetails.FileFrameID;
     MACConversation.FirstFrameOffset := FrameDetails.FileOffset;
     MACConversation.LastFrameOffset := FrameDetails.FileOffset;
     MACConversation.FirstFrameTime := FrameDetails.FrameTimestamp;
     MACConversation.LastFrameTime := FrameDetails.FrameTimestamp;
     MACConversation.Markers := [mFoundByARP];
//   MACConversation.Attributes := [caSingleFrame, caOneSidedCommunication];
     MACConversation.OriginTimestamp := FrameDetails.FrameTimestamp;
     MACConversation.Attributes := FrameDetails.Attributes + [caOneSidedCommunication];

     AddMACConversation (MACConversation);

     // Sender IP

     IPEndpoint.initialize (InterfaceIndex.FileIndex);

     setlength (IPEndpoint.InterfaceList, 1);
//   IPEndpoint.InterfaceList[0].initialize (0, InterfaceIndex);              // don't know the index here, will be determined on writing to DB
     IPEndpoint.InterfaceList[0].initialize (InterfaceIndex);

     IPEndpoint.IPv4Address := Parser.SenderIP;
     IPEndpoint.FirstFrame := FrameDetails.FileFrameID;
     IPEndpoint.LastFrame := FrameDetails.FileFrameID;
     IPEndpoint.FirstFrameOffset := FrameDetails.FileOffset;
     IPEndpoint.LastFrameOffset := FrameDetails.FileOffset;
     IPEndpoint.FirstFrameTime := FrameDetails.FrameTimestamp;
     IPEndpoint.LastFrameTime := FrameDetails.FrameTimestamp;
     IPEndpoint.Markers := [mFoundByARP];
     IPEndpoint.OriginTimestamp := FrameDetails.FrameTimestamp;
     IPEndpoint.Attributes := FrameDetails.Attributes;

     AddIPv4Endpoint (IPEndpoint);

     // Sender IP

     IPEndpoint.initialize (InterfaceIndex.FileIndex);

     setlength (IPEndpoint.InterfaceList, 1);
//   IPEndpoint.InterfaceList[0].initialize (0, InterfaceIndex);              // don't know the index here, will be determined on writing to DB
     IPEndpoint.InterfaceList[0].initialize (InterfaceIndex);

     IPEndpoint.IPv4Address := Parser.TargetIP;
     IPEndpoint.FirstFrame := FrameDetails.FileFrameID;
     IPEndpoint.LastFrame := FrameDetails.FileFrameID;
     IPEndpoint.FirstFrameOffset := FrameDetails.FileOffset;
     IPEndpoint.LastFrameOffset := FrameDetails.FileOffset;
     IPEndpoint.FirstFrameTime := FrameDetails.FrameTimestamp;
     IPEndpoint.LastFrameTime := FrameDetails.FrameTimestamp;
     IPEndpoint.Markers := [mFoundByARP];
     IPEndpoint.OriginTimestamp := FrameDetails.FrameTimestamp;
     IPEndpoint.Attributes := FrameDetails.Attributes;

     AddIPv4Endpoint (IPEndpoint);

     // IP Conversation

     IPConversation.initialize (InterfaceIndex.FileIndex);

     setlength (IPConversation.InterfaceList, 1);
//   IPConversation.InterfaceList[0].initialize (0, InterfaceIndex);          // don't know the index here, will be determined on writing to DB
     IPConversation.InterfaceList[0].initialize (InterfaceIndex);

     IPConversation.IPv4AddressPair.Source := Parser.SenderIP;
     IPConversation.IPv4AddressPair.Destination := Parser.TargetIP;
     IPConversation.OriginAddressPair := IPConversation.IPv4AddressPair;
     IPConversation.IPv4AddressPair.normalize;

     IPConversation.FirstFrame := FrameDetails.FileFrameID;
     IPConversation.LastFrame := FrameDetails.FileFrameID;
     IPConversation.FirstFrameOffset := FrameDetails.FileOffset;
     IPConversation.LastFrameOffset := FrameDetails.FileOffset;
     IPConversation.FirstFrameTime := FrameDetails.FrameTimestamp;
     IPConversation.LastFrameTime := FrameDetails.FrameTimestamp;
     IPConversation.Markers := [mFoundByARP];
     IPConversation.Attributes := FrameDetails.Attributes + [caOneSidedCommunication];

     IPConversation.OriginTimestamp := FrameDetails.FrameTimestamp;

     if not (IPConversation.IPv4AddressPair.Source = Parser.SenderIP) then
        IPConversation.Attributes := IPConversation.Attributes + [caAddressPairFlipped];

     AddIPv4Conversation (IPConversation);
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddIPv4Details (parser: tIPv4Parser; InterfaceIndex : tAbsoluteFileInterface; FrameDetails : tFrameDetails; const isICMPQuote : boolean = false);

var IPEndpoint      : tIPv4Endpoint;
    IPConversation  : tIPV4Conversation;

begin
     // Sender IP

     IPEndpoint.initialize (InterfaceIndex.FileIndex);

     setlength (IPEndpoint.InterfaceList, 1);
     IPEndpoint.InterfaceList[0].initialize (InterfaceIndex);

     IPEndpoint.IPv4Address := Parser.Source;
     IPEndpoint.TTL := Parser.TTL;                                  // Only have TTL coming from the sender
     IPEndpoint.BytesSent := FrameDetails.WireSize;
     IPEndpoint.FramesSent := 1;
     IPEndpoint.FirstFrame := FrameDetails.FileFrameID;
     IPEndpoint.LastFrame := FrameDetails.FileFrameID;
     IPEndpoint.FirstFrameOffset := FrameDetails.FileOffset;
     IPEndpoint.LastFrameOffset := FrameDetails.FileOffset;
     IPEndpoint.FirstFrameTime := FrameDetails.FrameTimestamp;
     IPEndpoint.LastFrameTime := FrameDetails.FrameTimestamp;
     IPEndpoint.Attributes := FrameDetails.Attributes;

     if isICMPQuote then
        IPEndpoint.Markers := [mFoundByICMP]
     else
         IPEndpoint.Markers := [mFoundByIPv4];

     IPEndpoint.OriginTimestamp := FrameDetails.FrameTimestamp;

     AddIPv4Endpoint (IPEndpoint);

     // Sender IP

     IPEndpoint.initialize (InterfaceIndex.FileIndex);

     setlength (IPEndpoint.InterfaceList, 1);
     IPEndpoint.InterfaceList[0].initialize (InterfaceIndex);

     IPEndpoint.IPv4Address := Parser.Destination;
     IPEndpoint.BytesReceived := FrameDetails.WireSize;
     IPEndpoint.FramesReceived := 1;
     IPEndpoint.FirstFrame := FrameDetails.FileFrameID;
     IPEndpoint.LastFrame := FrameDetails.FileFrameID;
     IPEndpoint.FirstFrameOffset := FrameDetails.FileOffset;
     IPEndpoint.LastFrameOffset := FrameDetails.FileOffset;
     IPEndpoint.FirstFrameTime := FrameDetails.FrameTimestamp;
     IPEndpoint.LastFrameTime := FrameDetails.FrameTimestamp;
     IPEndpoint.Attributes := FrameDetails.Attributes;

     if isICMPQuote then
        IPEndpoint.Markers := [mFoundByICMP]
     else
         IPEndpoint.Markers := [mFoundByIPv4];

     IPEndpoint.OriginTimestamp := FrameDetails.FrameTimestamp;

     AddIPv4Endpoint (IPEndpoint);

     // IP Conversation

     IPConversation.initialize (InterfaceIndex.FileIndex);

     setlength (IPConversation.InterfaceList, 1);
     IPConversation.InterfaceList[0].initialize (InterfaceIndex);

     IPConversation.IPv4AddressPair.Source := Parser.Source;
     IPConversation.IPv4AddressPair.Destination := Parser.Destination;
     IPConversation.OriginAddressPair := IPConversation.IPv4AddressPair;
     IPConversation.IPv4AddressPair.normalize;

     if IPConversation.IPv4AddressPair.Source = Parser.Source then
        begin
             IPConversation.BytesSrcToDst := FrameDetails.WireSize;
             IPConversation.FramesSrcToDst := 1;
        end
     else
         begin
              IPConversation.BytesDstToSrc := FrameDetails.WireSize;
              IPConversation.FramesDstToSrc := 1;
         end;

     IPConversation.FirstFrame := FrameDetails.FileFrameID;
     IPConversation.LastFrame := FrameDetails.FileFrameID;
     IPConversation.FirstFrameOffset := FrameDetails.FileOffset;
     IPConversation.LastFrameOffset := FrameDetails.FileOffset;
     IPConversation.FirstFrameTime := FrameDetails.FrameTimestamp;
     IPConversation.LastFrameTime := FrameDetails.FrameTimestamp;
     IPConversation.Attributes := FrameDetails.Attributes + [caOneSidedCommunication];

     if isICMPQuote then
        IPConversation.Markers := [mFoundByICMP]
     else
         IPConversation.Markers := [mFoundByIPv4];

//   IPConversation.Attributes := [caSingleFrame, caOneSidedCommunication];

     if not (IPConversation.IPv4AddressPair.Source = Parser.Source) then
        IPConversation.Attributes := IPConversation.Attributes + [caAddressPairFlipped];

     IPConversation.OriginTimestamp := FrameDetails.FrameTimestamp;

     AddIPv4Conversation (IPConversation);
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddIPv6Details (parser: tIPv6Parser; InterfaceIndex : tAbsoluteFileInterface; FrameDetails : tFrameDetails; const isICMPQuote : boolean = false);

var IPEndpoint      : tIPv6Endpoint;
    IPConversation  : tIPV6Conversation;

begin
     // Sender IP

     IPEndpoint.initialize (InterfaceIndex.FileIndex);

     setlength (IPEndpoint.InterfaceList, 1);
     IPEndpoint.InterfaceList[0].initialize (InterfaceIndex);

     IPEndpoint.IPv6Address := Parser.Source;
     IPEndpoint.BytesSent := FrameDetails.WireSize;
     IPEndpoint.FramesSent := 1;
     IPEndpoint.FirstFrame := FrameDetails.FileFrameID;
     IPEndpoint.LastFrame := FrameDetails.FileFrameID;
     IPEndpoint.FirstFrameOffset := FrameDetails.FileOffset;
     IPEndpoint.LastFrameOffset := FrameDetails.FileOffset;
     IPEndpoint.FirstFrameTime := FrameDetails.FrameTimestamp;
     IPEndpoint.LastFrameTime := FrameDetails.FrameTimestamp;
     IPEndpoint.Attributes := FrameDetails.Attributes;

     if isICMPQuote then
        IPEndpoint.Markers := [mFoundByICMP]
     else
         IPEndpoint.Markers := [mFoundByIPv6];

     IPEndpoint.OriginTimestamp := FrameDetails.FrameTimestamp;

     AddIPv6Endpoint (IPEndpoint);

     // Sender IP

     IPEndpoint.initialize (InterfaceIndex.FileIndex);

     setlength (IPEndpoint.InterfaceList, 1);
     IPEndpoint.InterfaceList[0].initialize (InterfaceIndex);

     IPEndpoint.IPv6Address := Parser.Destination;
     IPEndpoint.HopLimit := Parser.HopLimit;
     IPEndpoint.BytesReceived := FrameDetails.WireSize;
     IPEndpoint.FramesReceived := 1;
     IPEndpoint.FirstFrame := FrameDetails.FileFrameID;
     IPEndpoint.LastFrame := FrameDetails.FileFrameID;
     IPEndpoint.FirstFrameOffset := FrameDetails.FileOffset;
     IPEndpoint.LastFrameOffset := FrameDetails.FileOffset;
     IPEndpoint.FirstFrameTime := FrameDetails.FrameTimestamp;
     IPEndpoint.LastFrameTime := FrameDetails.FrameTimestamp;
     IPEndpoint.Attributes := FrameDetails.Attributes;

     if isICMPQuote then
        IPEndpoint.Markers := [mFoundByICMP]
     else
         IPEndpoint.Markers := [mFoundByIPv6];

     IPEndpoint.OriginTimestamp := FrameDetails.FrameTimestamp;

     AddIPv6Endpoint (IPEndpoint);

     // IP Conversation

     IPConversation.initialize (InterfaceIndex.FileIndex);

     setlength (IPConversation.InterfaceList, 1);
     IPConversation.InterfaceList[0].initialize (InterfaceIndex);

     IPConversation.IPv6AddressPair.Source := Parser.Source;
     IPConversation.IPv6AddressPair.Destination := Parser.Destination;
     IPConversation.OriginAddressPair := IPConversation.IPv6AddressPair;
     IPConversation.IPv6AddressPair.normalize;

     if IPConversation.IPv6AddressPair.Source = Parser.Source then
        begin
             IPConversation.BytesSrcToDst := FrameDetails.WireSize;
             IPConversation.FramesSrcToDst := 1;
        end
     else
         begin
              IPConversation.BytesDstToSrc := FrameDetails.WireSize;
              IPConversation.FramesDstToSrc := 1;
         end;

     IPConversation.FirstFrame := FrameDetails.FileFrameID;
     IPConversation.LastFrame := FrameDetails.FileFrameID;
     IPConversation.FirstFrameOffset := FrameDetails.FileOffset;
     IPConversation.LastFrameOffset := FrameDetails.FileOffset;
     IPConversation.FirstFrameTime := FrameDetails.FrameTimestamp;
     IPConversation.LastFrameTime := FrameDetails.FrameTimestamp;
     IPConversation.Attributes := FrameDetails.Attributes + [caOneSidedCommunication];

     if isICMPQuote then
        IPConversation.Markers := [mFoundByICMP]
     else
         IPConversation.Markers := [mFoundByIPv6];

//   IPConversation.Attributes := [caSingleFrame, caOneSidedCommunication];

     // Check if the socket pair was normalized

     if not (IPConversation.IPv6AddressPair.Source = Parser.Source) then
        IPConversation.Attributes := IPConversation.Attributes + [caAddressPairFlipped];

     IPConversation.OriginTimestamp := FrameDetails.FrameTimestamp;

     AddIPv6Conversation (IPConversation);
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddTCPDetails (parser: tTCPParser; IPv4Pair : tIPv4AddressPair; InterfaceIndex : tAbsoluteFileInterface; FrameDetails : tFrameDetails; TunnelFlags : tTunnelFlagSet);

var Conversation : tTCPConversation;

begin
     Conversation.initialize (InterfaceIndex.FileIndex);

     setlength (Conversation.InterfaceList, 1);
     Conversation.InterfaceList[0].initialize (InterfaceIndex);

     Conversation.SocketPair.Source.IPAddress.initialize (IPv4Pair.Source);
     Conversation.SocketPair.Source.Port := Parser.SourcePort;
     Conversation.SocketPair.Destination.IPAddress.initialize (IPv4Pair.Destination);
     Conversation.SocketPair.Destination.Port := Parser.DestinationPort;
     Conversation.OriginAddressPair := Conversation.SocketPair;
     Conversation.SocketPair.normalize;

     if Conversation.SocketPair.Source.IPAddress.IPv4Address = IPv4Pair.Source then
        begin
             Conversation.FramesSrcToDst := 1;
             Conversation.BytesSrcToDst := FrameDetails.WireSize;
        end
     else
         begin
              Conversation.FramesDstToSrc := 1;
              Conversation.BytesDstToSrc := FrameDetails.WireSize;
         end;

     Conversation.FirstFrame := FrameDetails.FileFrameID;
     Conversation.LastFrame := FrameDetails.FileFrameID;
     Conversation.FirstFrameOffset := FrameDetails.FileOffset;
     Conversation.LastFrameOffset := FrameDetails.FileOffset;
     Conversation.FirstFrameTime := FrameDetails.FrameTimestamp;
     Conversation.LastFrameTime := FrameDetails.FrameTimestamp;

     Conversation.SynSeen := tcpSYN in Parser.Flags;
     Conversation.FinSeen := tcpFIN in Parser.Flags;
     Conversation.RstSeen := tcpRST in Parser.Flags;

     Conversation.Markers := [mFoundByIPv4];
     Conversation.Attributes := FrameDetails.Attributes + [caOneSidedCommunication];

     // Check if the socket pair was normalized

     if not (IPv4Pair.Source = Conversation.SocketPair.Source.IPAddress.IPv4Address) then
        Conversation.Attributes := Conversation.Attributes + [caAddressPairFlipped];

     // TODO: figure out how to mark a conversation onesided

     if Parser.isSYNACKPacket then
        begin
             Conversation.SessionStatus := TCPHandshakeSYNACKSent;
             Conversation.ClientIP.initialize (IPv4Pair.Destination);
        end;

     if Parser.isSYNPacket then
        begin
             Conversation.ClientIP.initialize (IPv4Pair.Source);

             // Record what we need to determine Handshake status

             Conversation.SessionStatus := TCPHandshakeSYNSent;

             Conversation.SynTimestamp := FrameDetails.FrameTimestamp;
             Conversation.InitialSequenceNo := Parser.SequenceNumber.Value;
             Conversation.HandshakeThird := Parser.SequenceNumber.Value + Parser.PayloadLength + 1;

             Conversation.ClientPayloadBytes := Parser.PayloadLength;
{
             // Add SYN blocked state, which will be cleared as soon as we see a matching SYN/ACK - does not work as long as we have ports reused situations

             Conversation.Attributes := Conversation.Attributes + [caTCPSYNBlocked];
}
        end
     else
         Conversation.LastSeenSequence := Parser.SequenceNumber.Value;

     Conversation.OriginTimestamp := FrameDetails.FrameTimestamp;
     Conversation.PayloadBytes := Parser.PayloadLength;
     Conversation.Flags := parser.Flags;
     Conversation.Tunnelflags := TunnelFlags;

     // Add to store

     AddTCPConversation (Conversation);
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddTCPDetails (parser: tTCPParser; IPv6Pair : tIPv6AddressPair; InterfaceIndex : tAbsoluteFileInterface; FrameDetails : tFrameDetails; TunnelFlags : tTunnelFlagSet);

var Conversation : tTCPConversation;

begin
     Conversation.initialize (InterfaceIndex.FileIndex);

     setlength (Conversation.InterfaceList, 1);
     Conversation.InterfaceList[0].initialize (InterfaceIndex);

     Conversation.SocketPair.Source.IPAddress.initialize (IPv6Pair.Source);
     Conversation.SocketPair.Source.Port := Parser.SourcePort;
     Conversation.SocketPair.Destination.IPAddress.initialize (IPv6Pair.Destination);
     Conversation.SocketPair.Destination.Port := Parser.DestinationPort;
     Conversation.OriginAddressPair := Conversation.SocketPair;
     Conversation.SocketPair.normalize;

     if Conversation.SocketPair.Source.IPAddress.IPv6Address = IPv6Pair.Source then
        begin
             Conversation.FramesSrcToDst := 1;
             Conversation.BytesSrcToDst := FrameDetails.WireSize;
        end
     else
         begin
              Conversation.FramesDstToSrc := 1;
              Conversation.BytesDstToSrc := FrameDetails.WireSize;
         end;

     Conversation.FirstFrame := FrameDetails.FileFrameID;
     Conversation.LastFrame := FrameDetails.FileFrameID;
     Conversation.FirstFrameOffset := FrameDetails.FileOffset;
     Conversation.LastFrameOffset := FrameDetails.FileOffset;
     Conversation.FirstFrameTime := FrameDetails.FrameTimestamp;
     Conversation.LastFrameTime := FrameDetails.FrameTimestamp;

     Conversation.SynSeen := tcpSYN in Parser.Flags;
     Conversation.FinSeen := tcpFIN in Parser.Flags;
     Conversation.RstSeen := tcpRST in Parser.Flags;

     Conversation.Markers := [mFoundByIPv6];
     Conversation.Attributes := FrameDetails.Attributes + [caOneSidedCommunication];

     // Check if the socket pair was normalized

     if not (IPv6Pair.Source = Conversation.SocketPair.Source.IPAddress.IPv6Address) then
        Conversation.Attributes := Conversation.Attributes + [caAddressPairFlipped];

     if Parser.isSYNACKPacket then
        begin
             Conversation.SessionStatus := TCPHandshakeSYNACKSent;
             Conversation.ClientIP.initialize (IPv6Pair.Destination);
        end;

     if Parser.isSYNPacket then
        begin
             Conversation.ClientIP.initialize (IPv6Pair.Source);

             // Record what we need to determine Handshake status

             Conversation.SessionStatus := TCPHandshakeSYNSent;

             Conversation.SynTimestamp := FrameDetails.FrameTimestamp;
             Conversation.InitialSequenceNo := Parser.SequenceNumber.Value;
             Conversation.HandshakeThird := Parser.SequenceNumber.Value + Parser.PayloadLength + 1;

             Conversation.ClientPayloadBytes := Parser.PayloadLength;

{
             // Add SYN blocked state, which will be cleared as soon as we see a matching SYN/ACK - does not work as long as we have ports reused situations

             Conversation.Attributes := Conversation.Attributes + [caTCPSYNBlocked];
}
        end
     else
         Conversation.LastSeenSequence := Parser.SequenceNumber.Value;

     Conversation.OriginTimestamp := FrameDetails.FrameTimestamp;
     Conversation.PayloadBytes := Parser.PayloadLength;
     Conversation.Flags := parser.Flags;
     Conversation.Tunnelflags := TunnelFlags;

     AddTCPConversation (Conversation);
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddUDPDetails (parser: tUDPParser; IPv4Pair : tIPv4AddressPair; InterfaceIndex : tAbsoluteFileInterface; FrameDetails : tFrameDetails; TunnelFlags : tTunnelFlagSet);

var Conversation : tUDPConversation;

begin
     Conversation.initialize (InterfaceIndex.FileIndex);

     setlength (Conversation.InterfaceList, 1);
     Conversation.InterfaceList[0].initialize (InterfaceIndex);

     Conversation.SocketPair.Source.IPAddress.initialize (IPv4Pair.Source);
     Conversation.SocketPair.Source.Port := Parser.SourcePort;
     Conversation.SocketPair.Destination.IPAddress.initialize (IPv4Pair.Destination);
     Conversation.SocketPair.Destination.Port := Parser.DestinationPort;
     Conversation.OriginAddressPair := Conversation.SocketPair;
     Conversation.SocketPair.normalize;

     if Conversation.SocketPair.Source.IPAddress.IPv4Address = IPv4Pair.Source then
        begin
             Conversation.FramesSrcToDst := 1;
             Conversation.BytesSrcToDst := FrameDetails.WireSize;
        end
     else
         begin
              Conversation.FramesDstToSrc := 1;
              Conversation.BytesDstToSrc := FrameDetails.WireSize;
         end;

     Conversation.FirstFrame := FrameDetails.FileFrameID;
     Conversation.LastFrame := FrameDetails.FileFrameID;
     Conversation.FirstFrameOffset := FrameDetails.FileOffset;
     Conversation.LastFrameOffset := FrameDetails.FileOffset;
     Conversation.FirstFrameTime := FrameDetails.FrameTimestamp;
     Conversation.LastFrameTime := FrameDetails.FrameTimestamp;
     Conversation.Markers := [mFoundByIPv4];
     Conversation.Attributes := FrameDetails.Attributes + [caOneSidedCommunication];

     Conversation.OriginTimestamp := FrameDetails.FrameTimestamp;

     // Check if the socket pair was normalized

     if not (IPv4Pair.Source = Conversation.SocketPair.Source.IPAddress.IPv4Address) then
        Conversation.Attributes := Conversation.Attributes + [caAddressPairFlipped];

     Conversation.Tunnelflags := TunnelFlags;

     AddUDPConversation (Conversation);
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddUDPDetails (parser: tUDPParser; IPv6Pair : tIPv6AddressPair; InterfaceIndex : tAbsoluteFileInterface; FrameDetails : tFrameDetails; TunnelFlags : tTunnelFlagSet);

var Conversation : tUDPConversation;

begin
     Conversation.initialize (InterfaceIndex.FileIndex);

     setlength (Conversation.InterfaceList, 1);
     Conversation.InterfaceList[0].initialize (InterfaceIndex);

     Conversation.SocketPair.Source.IPAddress.initialize (IPv6Pair.Source);
     Conversation.SocketPair.Source.Port := Parser.SourcePort;
     Conversation.SocketPair.Destination.IPAddress.initialize (IPv6Pair.Destination);
     Conversation.SocketPair.Destination.Port := Parser.DestinationPort;
     Conversation.OriginAddressPair := Conversation.SocketPair;
     Conversation.SocketPair.normalize;

     if Conversation.SocketPair.Source.IPAddress.IPv6Address = IPv6Pair.Source then
        begin
             Conversation.FramesSrcToDst := 1;
             Conversation.BytesSrcToDst := FrameDetails.WireSize;
        end
     else
         begin
              Conversation.FramesDstToSrc := 1;
              Conversation.BytesDstToSrc := FrameDetails.WireSize;
         end;

     Conversation.FirstFrame := FrameDetails.FileFrameID;
     Conversation.LastFrame := FrameDetails.FileFrameID;
     Conversation.FirstFrameOffset := FrameDetails.FileOffset;
     Conversation.LastFrameOffset := FrameDetails.FileOffset;
     Conversation.FirstFrameTime := FrameDetails.FrameTimestamp;
     Conversation.LastFrameTime := FrameDetails.FrameTimestamp;
     Conversation.Markers := [mFoundByIPv6];
     Conversation.Attributes := FrameDetails.Attributes + [caOneSidedCommunication];

     Conversation.OriginTimestamp := FrameDetails.FrameTimestamp;

     // Check if the socket pair was normalized

     if not (IPv6Pair.Source = Conversation.SocketPair.Source.IPAddress.IPv6Address) then
        Conversation.Attributes := Conversation.Attributes + [caAddressPairFlipped];

     Conversation.Tunnelflags := TunnelFlags;

     AddUDPConversation (Conversation);
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddNameResolutionDetails (const NRDetails : tNameResolutionDetails; const InterfaceIndex : tAbsoluteFileInterface; const FrameDetails : tFrameDetails);

var NameResolution : tNameResolutionEntry;

begin
     assert (NRDetails.FQDN.FullFQDN <> '', 'Empty FQDN (tConversationStorage.AddNameResolutionDetails)');

     NameResolution.initialize (InterfaceIndex.FileIndex);
     setlength (NameResolution.InterfaceList, 1);
     NameResolution.InterfaceList[0].initialize (InterfaceIndex);

     NameResolution.FQDN := NrDetails.FQDN;
     NameResolution.isAnswer := NRDetails.isAnswer;
     NameResolution.Answer := NrDetails.Answer;
     NameResolution.FrameCount := 1;
     NameResolution.FirstFrame := FrameDetails.FileFrameID;
     NameResolution.LastFrame := FrameDetails.FileFrameID;
     NameResolution.FirstFrameOffset := FrameDetails.FileOffset;
     NameResolution.LastFrameOffset := FrameDetails.FileOffset;
     NameResolution.FirstFrameTime := FrameDetails.FrameTimestamp;
     NameResolution.LastFrameTime := FrameDetails.FrameTimestamp;

     NameResolution.QueryTypes := [NrDetails.QueryType];
     NameResolution.QueryOnly := NrDetails.QueryOnly;

     case NrDetails.IPVersion of
      4 : NameResolution.Markers := [mFoundByIPv4];
      6 : NameResolution.Markers := [mFoundByIPv6];
     end;

     NameResolution.Attributes := FrameDetails.Attributes;
     NameResolution.Markers := NrDetails.Marker;
     NameResolution.ReplyCode := NrDetails.ReplyCode;

     AddNameResolution (NameResolution);
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddFrame (Frame: tRawFrame);

var parsemaster   : tParseMaster;
    parsetype     : integer;
    parser        : tFrameParser;
    parseinfo     : tParseEntry;
    DNSParser     : tDNSParser;
    IPVersion     : byte;
    AbsIntIndex   : tAbsoluteFileInterface;
    isICMPQuote   : boolean;
    IPv4Pair      : tIPv4AddressPair;
    IPv6Pair      : tIPv6AddressPair;
    FrameDetails  : tFrameDetails;
    NRDetails     : tNameResolutionDetails;
    PayloadLen    : integer;
    PayloadStart  : integer;
    DNSZeroAnswer : tNameResolutionAnswer;
    Complexity    : integer;
    MessageList   : tDiagnosticMsgList;
    count         : integer;
    i,j           : integer;

begin
//   SiMain.EnterMethod('tConversationStorage.AddFrame');

{$ifdef debug}
     SiMain.LogDebug ('Adding frame to conversation storage: #' + IntToStr (Frame.FileFrameID));
{$endif}

//{$ifdef debug}
     parsemaster := tParseMaster.create (Frame, ppmParseAll);
//{$else}
//     parsemaster := tParseMaster.create (Frame, ppmUpToLayer4);
//{$endif}

     // Handle diagnostics

     if assigned (priv_EventonDiagnosticsMessages) then
        begin
             count := parsemaster.getDiagnosticMsgCount;
             setLength (MessageList, count);

             for i := 1 to count do
                 MessageList[i-1] := parsemaster.getDiagnosticMsg(i);

             priv_EventonDiagnosticsMessages (self, MessageList);
        end;

     // Do the actual stuff

     DNSZeroAnswer.initialize;
     isICMPQuote := false;
     Complexity := 0;

     FrameDetails.initialize (Frame.FileFrameID, Frame.WireSize, Frame.FileOffset, Frame.Timestamp);
     FrameDetails.Attributes := [caSingleFrame];

     NRDetails.initialize;

     // Check if we can apply quick IPv4 conversation filtering

     if (parsemaster.IPLayerCount > 1) then
        FrameDetails.Attributes := FrameDetails.Attributes + [caNoQuickFilter]
     else
         if parsemaster.ParserCount > 2 then
            begin
                 parseType := parsemaster.getParserTypeByIndex (1);

                 // First layer needs to be ethernet

                 if not ((parsetype = NEXT_HEADER_ETHERNET) or (parsetype = NEXT_HEADER_TRANS_ETHER_BRIDGE)) then
                    FrameDetails.Attributes := FrameDetails.Attributes + [caNoQuickFilter];

                 parseType := parsemaster.getParserTypeByIndex (2);

                 // Second layer needs to be IP of fixed size

                 if not ((parsetype = NEXT_HEADER_IPv4) or (parsetype = NEXT_HEADER_IPv4_ENCAPSULATION) or (parsetype = NEXT_HEADER_IPv6) or (parsetype = NEXT_HEADER_TYPE_IPV6)) then
                    FrameDetails.Attributes := FrameDetails.Attributes + [caNoQuickFilter]
                 else
                     begin
                          if ((parsetype = NEXT_HEADER_IPv4) or (parsetype = NEXT_HEADER_IPv4_ENCAPSULATION)) then
                             begin
                                  parser := parsemaster.getParserByIndex(2);

                                  // Make sure IPv4 header is 20 bytes in size

                                  if (parser as tIPv4Parser).IHL <> 20 then
                                     FrameDetails.Attributes := FrameDetails.Attributes + [caNoQuickFilter]
                             end;
                     end;

                 parseType := parsemaster.getParserTypeByIndex (3);

                 // Third layer needs to be TCP

                 if not (parsetype = NEXT_HEADER_TCP) then
                    FrameDetails.Attributes := FrameDetails.Attributes + [caNoQuickFilter];
            end;

     AbsIntIndex.initialize (priv_filedbindex, Frame.InterfaceID);

     for i := 1 to parsemaster.ParserCount do
         begin
              parseInfo := parsemaster.getParserParseInfoByIndex (i);
              parser := parsemaster.getParserByIndex(i);

              case parseInfo.Anchor.HeaderType of
               NEXT_HEADER_ETHERNET,
               NEXT_HEADER_TRANS_ETHER_BRIDGE : AddEthernetDetails (parser as tEthernetParser, AbsIntIndex, FrameDetails);
               NEXT_HEADER_ARP                : AddARPDetails (parser as tARPParser, AbsIntIndex, FrameDetails);
               NEXT_HEADER_IPv4_ENCAPSULATION,
               NEXT_HEADER_IPv4               : begin
                                                     PayloadLen := (parser as tIPv4Parser).TotalLength;
                                                     PayloadStart := Parser.FrameOffset;

                                                     // Check if there is an ethernet trailer - 4 bytes for Ethernet FCS plus IPv4 Total Length

                                                     if ((parser.ParentAnchor.HeaderType = NEXT_HEADER_ETHERNET) and (frame.WireSize > 64)) and
                                                        ((PayloadLen + PayloadStart + 4) < Frame.WireSize) then
                                                        FrameDetails.Attributes := FrameDetails.Attributes + [caEthernetTrailerPresent];

                                                     // Check if this is a IP fragment

                                                     if (Parser as tIPv4Parser).IsFragment then
                                                        FrameDetails.Attributes := FrameDetails.Attributes + [caIPFragmented];

                                                     // Check if we have a hard sliced frame (meaning, wiresize and capturesize are shorter than the real wiresize

                                                     if (frame.WireSize = frame.FrameSize) and // ((frame.WireSize = 64) or (frame.WireSize = 128) or (frame.WireSize = 256)then
                                                        (frame.WireSize < PayloadLen) and not isICMPQuote then
                                                        FrameDetails.Attributes := FrameDetails.Attributes + [caHardSliced];

                                                     AddIPv4Details (parser as tIPv4Parser, AbsIntIndex, FrameDetails, isICMPQuote);

                                                     isICMPQuote := false;    // Reset
                                                     IPVersion := 4;

                                                     IPv4Pair.Source := (parser as tIPV4Parser).Source;
                                                     IPv4Pair.Destination := (parser as tIPV4Parser).Destination;
                                                end;
               NEXT_HEADER_TYPE_IPV6,
               NEXT_HEADER_IPv6               : begin
                                                     PayloadLen := (parser as tIPv6Parser).PayloadLength;
                                                     PayloadStart := Parser.FrameOffset;

                                                     // Check if there is an ethernet trailer - 4 bytes for Ethernet FCS plus IPv6 Total Length

                                                     if ((parser.ParentAnchor.HeaderType = NEXT_HEADER_ETHERNET) and (frame.WireSize > 64)) and
                                                        ((PayloadLen + 40 + PayloadStart + 4) < Frame.WireSize) then
                                                        FrameDetails.Attributes := FrameDetails.Attributes + [caEthernetTrailerPresent];

                                                     // Check if we have a hard sliced frame (meaning, wiresize and capturesize are shorter than the real wiresize

                                                     if (frame.WireSize = frame.FrameSize) and // ((frame.WireSize = 64) or (frame.WireSize = 128) or (frame.WireSize = 256)then
                                                        (frame.WireSize < 40 + PayloadLen) and not isICMPQuote then
                                                        FrameDetails.Attributes := FrameDetails.Attributes + [caHardSliced];

                                                     AddIPv6Details (parser as tIPv6Parser, AbsIntIndex, FrameDetails, isICMPQuote);

                                                     isICMPQuote := false;    // Reset
                                                     IPVersion := 6;

                                                     IPv6Pair.Source := (parser as tIPV6Parser).Source;
                                                     IPv6Pair.Destination := (parser as tIPV6Parser).Destination;
                                                end;
               NEXT_HEADER_IPV6_FRAGMENTATION : begin
                                                     FrameDetails.Attributes := FrameDetails.Attributes + [caIPFragmented];
                                                end;
               NEXT_HEADER_TCP                : begin
                                                     if IPVersion = 4 then
                                                        AddTCPDetails (parser as tTCPParser, IPv4Pair, AbsIntIndex, FrameDetails, parseInfo.TunnelFlags)
                                                     else
                                                         AddTCPDetails (parser as tTCPParser, IPv6Pair, AbsIntIndex, FrameDetails, parseInfo.TunnelFlags)
                                                end;
               NEXT_HEADER_UDP                : begin
                                                     if IPVersion = 4 then
                                                        AddUDPDetails (parser as tUDPParser, IPv4Pair, AbsIntIndex, FrameDetails, parseInfo.TunnelFlags)
                                                     else
                                                         AddUDPDetails (parser as tUDPParser, IPv6Pair, AbsIntIndex, FrameDetails, parseInfo.TunnelFlags);
                                                end;
               NEXT_HEADER_ICMP,
               NEXT_HEADER_IPV6_ICMP          : begin
                                                     // Mark that the next IP layer following this one is an ICMP quote

                                                     isICMPQuote := true;
                                                end;
               NEXT_HEADER_DNS                : begin
                                                     DNSParser := (parser as tDNSParser);

                                                     // Queries

                                                     for j := 1 to DNSParser.QueryCount do
                                                         begin
                                                              NRDetails.initialize (DNSParser.QueryRecordSet[j-1].Name, DNSParser.isResponse, IPVersion, DNSZeroAnswer, [mFoundByDNS], DNSParser.QueryRecordSet[j-1].RecordType, true, DNSParser.ReplyCode);

                                                              if NRDetails.FQDN.FullFQDN <> '' then
                                                                 AddNameResolutionDetails (NRDetails, AbsIntIndex, FrameDetails);
                                                         end;

                                                     // Answers

                                                     for j := 1 to DNSParser.AnswerCount do
                                                         begin
                                                              NRDetails.initialize (DNSParser.AnswerRecordSet[j-1].Name, DNSParser.isResponse, IPVersion, DNSParser.AnswerRecordSet[j-1].Answer, [mFoundByDNS], DNSParser.AnswerRecordSet[j-1].RecordType, false, DNSParser.ReplyCode);

                                                              if NRDetails.FQDN.FullFQDN <> '' then
                                                                 AddNameResolutionDetails (NRDetails, AbsIntIndex, FrameDetails);
                                                         end;
                                                end;
              end;
         end;

     Complexity := parsemaster.ComplexityRating;

     if Complexity > priv_OverallComplexity then
        priv_OverallComplexity := Complexity;

     parsemaster.free;

//   if not botcpfound then
//      SiMain.LogFatal ('Error: did not process TCP layer in frame #' + IntToStr (frame.FileFrameID));

//   SiMain.LeaveMethod('tConversationStorage.AddFrame');
end;

(* ------------------------------------------- *)

procedure tConversationStorage.StoreMACEndpointsToDB;

var Query : tFDQuery;
    entry : tMACEndpoint;
    Key   : string;
    count : integer;
    i,j   : integer;

begin
     count := priv_dictMACEndpoints.Count;

     if count > 0 then
        begin
             Query := tFDQuery.Create(nil);
             Query.Connection := priv_database;

             Query.SQL.Text := 'INSERT INTO MACEndpoints (fileid,MACAddress,FramesReceived,FramesSent,BytesReceived,BytesSent,FirstFrame,LastFrame,FirstFrameOffset,LastFrameOffset,FirstFrameTime,FirstFrameTimeNano,LastFrameTime,LastFrameTimeNano,FlagEthernet,FlagARP,' +
                                                         'Attributes,InterfaceIDs)' +
                               ' VALUES (:fileid,:MACAddress,:FramesReceived,:FramesSent,:BytesReceived,:BytesSent,:FirstFrame,:LastFrame,:FirstFrameOffset,:LastFrameOffset,:FirstFrameTime,:FirstFrameTimeNano,:LastFrameTime,:LastFrameTimeNano,:FlagEthernet,:FlagARP,' +
                                        ':Attributes,:InterfaceIDs)';

             Query.Params.ArraySize := count;

             i := 0;

             for key in priv_dictMACEndpoints.Keys do
                 begin
                      entry := priv_dictMACEndpoints.Items[key];
{
                      // Set FileIndex for Interface list since we only know it now

                      for j := 1 to length (entry.InterfaceList) do
                          entry.InterfaceList[j-1].FileIndex := priv_filedbindex;
}
                      Query.ParamByName ('fileid').AsIntegers[i] := priv_filedbindex;
                      Query.ParamByName ('MACAddress').AsStrings[i] := entry.MACAddress.asString(':', true);
                      Query.ParamByName ('FramesReceived').AsIntegers[i] := entry.FramesReceived;
                      Query.ParamByName ('FramesSent').AsIntegers[i] := entry.FramesSent;
                      Query.ParamByName ('BytesReceived').AsLargeInts[i] := entry.BytesReceived;
                      Query.ParamByName ('BytesSent').AsLargeInts[i] := entry.BytesSent;
                      Query.ParamByName ('FirstFrame').AsLargeInts[i] := entry.FirstFrameIndex;
                      Query.ParamByName ('LastFrame').AsLargeInts[i] := entry.LastFrameIndex;
                      Query.ParamByName ('FirstFrameOffset').AsLargeInts[i] := entry.FirstFrameOffset;
                      Query.ParamByName ('LastFrameOffset').AsLargeInts[i] := entry.LastFrameOffset;
                      Query.ParamByName ('FirstFrameTime').AsDateTimes[i] := entry.FirstFrameTime.datetime;
                      Query.ParamByName ('FirstFrameTimeNano').AsIntegers[i] := entry.FirstFrameTime.nanoseconds;
                      Query.ParamByName ('LastFrameTime').AsDateTimes[i] := entry.LastFrameTime.datetime;
                      Query.ParamByName ('LastFrameTimeNano').AsIntegers[i] := entry.LastFrameTime.nanoseconds;
                      Query.ParamByName ('FlagEthernet').AsBooleans[i] := (mFoundByEthernet in entry.Markers);
                      Query.ParamByName ('FlagARP').AsBooleans[i] := (mFoundByARP in entry.Markers);
                      Query.ParamByName ('InterfaceIDs').AsStrings[i] := InterfaceListToStr (entry.InterfaceList);
                      Query.ParamByName ('Attributes').AsLongwords[i] := AttributesToLong (entry.Attributes);

                      inc (i);
                 end;

             Query.Execute (count,0);
             Query.Free;
        end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.StoreMACConversationsToDB;

var Query : tFDQuery;
    entry : tMACConversation;
    Key   : string;
    count : integer;
    i,j   : integer;

begin
     count := priv_dictMACConversations.Count;

     if count > 0 then
        begin
             Query := tFDQuery.Create(nil);
             Query.Connection := priv_database;

             Query.SQL.Text := 'INSERT INTO MACConversations (fileid,SourceMAC,DestinationMAC,FramesSrcToDst,FramesDstToSrc,BytesSrcToDst,BytesDstToSrc,FirstFrame,LastFrame,FirstFrameOffset,LastFrameOffset,' +
                                                             'FirstFrameTime,FirstFrameTimeNano,LastFrameTime,LastFrameTimeNano,FlagEthernet,FlagARP,Attributes,InterfaceIDs)' +
                               ' VALUES (:fileid,:SourceMAC,:DestinationMAC,:FramesSrcToDst,:FramesDstToSrc,:BytesSrcToDst,:BytesDstToSrc,:FirstFrame,:LastFrame,:FirstFrameOffset,:LastFrameOffset,' +
                                        ':FirstFrameTime,:FirstFrameTimeNano,:LastFrameTime,:LastFrameTimeNano,:FlagEthernet,:FlagARP,:Attributes,:InterfaceIDs)';

             Query.Params.ArraySize := count;

             i := 0;

             for key in priv_dictMACConversations.Keys do
                 begin
                      entry := priv_dictMACConversations.Items[key];
{
                      // Set FileIndex for Interface list since we only know it now

                      for j := 1 to length (entry.InterfaceList) do
                          entry.InterfaceList[j-1].FileIndex := priv_filedbindex;
}
                      Query.ParamByName ('fileid').AsIntegers[i] := priv_filedbindex;
                      Query.ParamByName ('SourceMAC').AsStrings[i] := entry.MACPair.Source.asString(':', true);
                      Query.ParamByName ('DestinationMAC').AsStrings[i] := entry.MACPair.Destination.asString(':', true);
                      Query.ParamByName ('FramesSrcToDst').AsIntegers[i] := entry.FramesSrcToDst;
                      Query.ParamByName ('FramesDstToSrc').AsIntegers[i] := entry.FramesDstToSrc;
                      Query.ParamByName ('BytesSrcToDst').AsLargeInts[i] := entry.BytesSrcToDst;
                      Query.ParamByName ('BytesDstToSrc').AsLargeInts[i] := entry.BytesDstToSrc;
                      Query.ParamByName ('FirstFrame').AsLargeInts[i] := entry.FirstFrame;
                      Query.ParamByName ('LastFrame').AsLargeInts[i] := entry.LastFrame;
                      Query.ParamByName ('FirstFrameOffset').AsLargeInts[i] := entry.FirstFrameOffset;
                      Query.ParamByName ('LastFrameOffset').AsLargeInts[i] := entry.LastFrameOffset;
                      Query.ParamByName ('FirstFrameTime').AsDateTimes[i] := entry.FirstFrameTime.datetime;
                      Query.ParamByName ('FirstFrameTimeNano').AsIntegers[i] := entry.FirstFrameTime.nanoseconds;
                      Query.ParamByName ('LastFrameTime').AsDateTimes[i] := entry.LastFrameTime.datetime;
                      Query.ParamByName ('LastFrameTimeNano').AsIntegers[i] := entry.LastFrameTime.nanoseconds;
                      Query.ParamByName ('FlagEthernet').AsBooleans[i] := (mFoundByEthernet in entry.Markers);
                      Query.ParamByName ('FlagARP').AsBooleans[i] := (mFoundByARP in entry.Markers);
                      Query.ParamByName ('InterfaceIDs').AsStrings[i] := InterfaceListToStr (entry.InterfaceList);
                      Query.ParamByName ('Attributes').AsLongwords[i] := AttributesToLong (entry.Attributes);

                      inc (i);
                 end;

             Query.Execute (count,0);
             Query.Free;
        end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.StoreIPv4EndpointsToDB;

var Query : tFDQuery;
    entry : tIPv4Endpoint;
    Key   : string;
    count : integer;
    i,j   : integer;

begin
     count := priv_dictIPv4Endpoints.Count;

     if count > 0 then
        begin
             Query := tFDQuery.Create(nil);
             Query.Connection := priv_database;

             Query.SQL.Text := 'INSERT INTO IPv4Endpoints (fileid,IPAddress,FramesReceived,FramesSent,BytesReceived,BytesSent,FirstFrame,LastFrame,FirstFrameOffset,LastFrameOffset,' +
                                                          'FirstFrameTime,FirstFrameTimeNano,LastFrameTime,LastFrameTimeNano,TTL,FlagARP,FlagIPv4,FlagDNS,FlagICMP,FlagNRB,Attributes,InterfaceIDs)' +
                                                ' VALUES (:fileid,:IPAddress,:FramesReceived,:FramesSent,:BytesReceived,:BytesSent,:FirstFrame,:LastFrame,:FirstFrameOffset,:LastFrameOffset,' +
                                                         ':FirstFrameTime,:FirstFrameTimeNano,:LastFrameTime,:LastFrameTimeNano,:TTL,:FlagARP,:FlagIPv4,:FlagDNS,:FlagICMP,:FlagNRB,:Attributes,:InterfaceIDs)';

             Query.Params.ArraySize := count;

             i := 0;

             for key in priv_dictIPv4Endpoints.Keys do
                 begin
                      entry := priv_dictIPv4Endpoints.Items[key];
{
                      // Set FileIndex for Interface list since we only know it now

                      for j := 1 to length (entry.InterfaceList) do
                          entry.InterfaceList[j-1].FileIndex := priv_filedbindex;
}
                      Query.ParamByName ('fileid').AsIntegers[i] := priv_filedbindex;
                      Query.ParamByName ('IPAddress').AsStrings[i] := entry.IPv4Address.ValueAsString;
                      Query.ParamByName ('FramesReceived').AsIntegers[i] := entry.FramesReceived;
                      Query.ParamByName ('FramesSent').AsIntegers[i] := entry.FramesSent;
                      Query.ParamByName ('BytesReceived').AsLargeInts[i] := entry.BytesReceived;
                      Query.ParamByName ('BytesSent').AsLargeInts[i] := entry.BytesSent;
                      Query.ParamByName ('FirstFrame').AsLargeInts[i] := entry.FirstFrame;
                      Query.ParamByName ('LastFrame').AsLargeInts[i] := entry.LastFrame;
                      Query.ParamByName ('FirstFrameOffset').AsLargeInts[i] := entry.FirstFrameOffset;
                      Query.ParamByName ('LastFrameOffset').AsLargeInts[i] := entry.LastFrameOffset;
                      Query.ParamByName ('FirstFrameTime').AsDateTimes[i] := entry.FirstFrameTime.datetime;
                      Query.ParamByName ('FirstFrameTimeNano').AsIntegers[i] := entry.FirstFrameTime.nanoseconds;
                      Query.ParamByName ('LastFrameTime').AsDateTimes[i] := entry.LastFrameTime.datetime;
                      Query.ParamByName ('LastFrameTimeNano').AsIntegers[i] := entry.LastFrameTime.nanoseconds;
                      Query.ParamByName ('TTL').AsBytes[i] := entry.TTL;
                      Query.ParamByName ('FlagARP').AsBooleans[i] := (mFoundByARP in entry.Markers);
                      Query.ParamByName ('FlagIPv4').AsBooleans[i] := (mFoundByIPv4 in entry.Markers);
                      Query.ParamByName ('FlagDNS').AsBooleans[i] := (mFoundByDNS in entry.Markers);
                      Query.ParamByName ('FlagICMP').AsBooleans[i] := (mFoundByICMP in entry.Markers);
                      Query.ParamByName ('FlagNRB').AsBooleans[i] := (mFoundByNRB in entry.Markers);
                      Query.ParamByName ('InterfaceIDs').AsStrings[i] := InterfaceListToStr (entry.InterfaceList);
                      Query.ParamByName ('Attributes').AsLongwords[i] := AttributesToLong (entry.Attributes);

                      inc (i);
                 end;

             Query.Execute (count,0);
             Query.Free;
        end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.StoreIPv4ConversationsToDB;

var Query : tFDQuery;
    entry : tIPv4Conversation;
    Key   : string;
    count : integer;
    i,j   : integer;

begin
     count := priv_dictIPv4Conversations.Count;

     if count > 0 then
        begin
             Query := tFDQuery.Create(nil);
             Query.Connection := priv_database;

             Query.SQL.Text := 'INSERT INTO IPv4Conversations (fileid,SourceIP,DestinationIP,FramesSrcToDst,FramesDstToSrc,BytesSrcToDst,BytesDstToSrc,FirstFrame,LastFrame,FirstFrameOffset,LastFrameOffset,FirstFrameTime,FirstFrameTimeNano,LastFrameTime,LastFrameTimeNano' +
                               ',FlagARP,FlagIPv4,FlagDNS,FlagICMP,FlagNRB,Attributes,InterfaceIDs)' +
                               ' VALUES (:fileid,:SourceIP,:DestinationIP,:FramesSrcToDst,:FramesDstToSrc,:BytesSrcToDst,:BytesDstToSrc,:FirstFrame,:LastFrame,:FirstFrameOffset,:LastFrameOffset,:FirstFrameTime,:FirstFrameTimeNano,:LastFrameTime,:LastFrameTimeNano,' +
                               ':FlagARP,:FlagIPv4,:FlagDNS,:FlagICMP,:FlagNRB,:Attributes,:InterfaceIDs)';

             Query.Params.ArraySize := count;

             i := 0;

             for key in priv_dictIPv4Conversations.Keys do
                 begin
                      entry := priv_dictIPv4Conversations.Items[key];
{
                      // Set FileIndex for Interface list since we only know it now

                      for j := 1 to length (entry.InterfaceList) do
                          entry.InterfaceList[j-1].FileIndex := priv_filedbindex;
}
                      Query.ParamByName ('fileid').AsIntegers[i] := priv_filedbindex;
                      Query.ParamByName ('SourceIP').AsStrings[i] := entry.IPv4AddressPair.Source.ValueAsString;
                      Query.ParamByName ('DestinationIP').AsStrings[i] := entry.IPv4AddressPair.Destination.ValueAsString;
                      Query.ParamByName ('FramesSrcToDst').AsIntegers[i] := entry.FramesSrcToDst;
                      Query.ParamByName ('FramesDstToSrc').AsIntegers[i] := entry.FramesDstToSrc;
                      Query.ParamByName ('BytesSrcToDst').AsLargeInts[i] := entry.BytesSrcToDst;
                      Query.ParamByName ('BytesDstToSrc').AsLargeInts[i] := entry.BytesDstToSrc;
                      Query.ParamByName ('FirstFrame').AsLargeInts[i] := entry.FirstFrame;
                      Query.ParamByName ('LastFrame').AsLargeInts[i] := entry.LastFrame;
                      Query.ParamByName ('FirstFrameOffset').AsLargeInts[i] := entry.FirstFrameOffset;
                      Query.ParamByName ('LastFrameOffset').AsLargeInts[i] := entry.LastFrameOffset;
                      Query.ParamByName ('FirstFrameTime').AsDateTimes[i] := entry.FirstFrameTime.datetime;
                      Query.ParamByName ('FirstFrameTimeNano').AsIntegers[i] := entry.FirstFrameTime.nanoseconds;
                      Query.ParamByName ('LastFrameTime').AsDateTimes[i] := entry.LastFrameTime.datetime;
                      Query.ParamByName ('LastFrameTimeNano').AsIntegers[i] := entry.LastFrameTime.nanoseconds;
                      Query.ParamByName ('FlagARP').AsBooleans[i] := (mFoundByARP in entry.Markers);
                      Query.ParamByName ('FlagIPv4').AsBooleans[i] := (mFoundByIPv4 in entry.Markers);
                      Query.ParamByName ('FlagDNS').AsBooleans[i] := (mFoundByDNS in entry.Markers);
                      Query.ParamByName ('FlagICMP').AsBooleans[i] := (mFoundByICMP in entry.Markers);
                      Query.ParamByName ('FlagNRB').AsBooleans[i] := (mFoundByNRB in entry.Markers);
                      Query.ParamByName ('InterfaceIDs').AsStrings[i] := InterfaceListToStr (entry.InterfaceList);
                      Query.ParamByName ('Attributes').AsLongwords[i] := AttributesToLong (entry.Attributes);

                      inc (i);
                 end;

             Query.Execute (count,0);
             Query.Free;
        end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.StoreIPv6EndpointsToDB;

var Query : tFDQuery;
    entry : tIPv6Endpoint;
    Key   : string;
    count : integer;
    i,j   : integer;

begin
     count := priv_dictIPv6Endpoints.Count;

     if count > 0 then
        begin
             Query := tFDQuery.Create(nil);
             Query.Connection := priv_database;

             Query.SQL.Text := 'INSERT INTO IPv6Endpoints (fileid,IPAddress,FramesReceived,FramesSent,BytesReceived,BytesSent,FirstFrame,LastFrame,FirstFrameOffset,LastFrameOffset,FirstFrameTime,FirstFrameTimeNano,LastFrameTime,LastFrameTimeNano,HopLimit,' +
                                                          'FlagIPv6,FlagDNS,FlagICMP,FlagNRB,Attributes,InterfaceIDs)' +
                                                ' VALUES (:fileid,:IPAddress,:FramesReceived,:FramesSent,:BytesReceived,:BytesSent,:FirstFrame,:LastFrame,:FirstFrameOffset,:LastFrameOffset,:FirstFrameTime,:FirstFrameTimeNano,:LastFrameTime,:LastFrameTimeNano,:HopLimit,' +
                                                         ':FlagIPv6,:FlagDNS,:FlagICMP,:FlagNRB,:Attributes,:InterfaceIDs)';

             Query.Params.ArraySize := count;

             i := 0;

             for key in priv_dictIPv6Endpoints.Keys do
                 begin
                      entry := priv_dictIPv6Endpoints.Items[key];
{
                      // Set FileIndex for Interface list since we only know it now

                      for j := 1 to length (entry.InterfaceList) do
                          entry.InterfaceList[j-1].FileIndex := priv_filedbindex;
}
                      Query.ParamByName ('fileid').AsIntegers[i] := priv_filedbindex;
                      Query.ParamByName ('IPAddress').AsStrings[i] := entry.IPv6Address.ValueAsString;
                      Query.ParamByName ('FramesReceived').AsIntegers[i] := entry.FramesReceived;
                      Query.ParamByName ('FramesSent').AsIntegers[i] := entry.FramesSent;
                      Query.ParamByName ('BytesReceived').AsLargeInts[i] := entry.BytesReceived;
                      Query.ParamByName ('BytesSent').AsLargeInts[i] := entry.BytesSent;
                      Query.ParamByName ('FirstFrame').AsLargeInts[i] := entry.FirstFrame;
                      Query.ParamByName ('LastFrame').AsLargeInts[i] := entry.LastFrame;
                      Query.ParamByName ('FirstFrameOffset').AsLargeInts[i] := entry.FirstFrameOffset;
                      Query.ParamByName ('LastFrameOffset').AsLargeInts[i] := entry.LastFrameOffset;
                      Query.ParamByName ('FirstFrameTime').AsDateTimes[i] := entry.FirstFrameTime.datetime;
                      Query.ParamByName ('FirstFrameTimeNano').AsIntegers[i] := entry.FirstFrameTime.nanoseconds;
                      Query.ParamByName ('LastFrameTime').AsDateTimes[i] := entry.LastFrameTime.datetime;
                      Query.ParamByName ('LastFrameTimeNano').AsIntegers[i] := entry.LastFrameTime.nanoseconds;
                      Query.ParamByName ('HopLimit').AsBytes[i] := entry.HopLimit;
                      Query.ParamByName ('FlagIPv6').AsBooleans[i] := (mFoundByIPv6 in entry.Markers);
                      Query.ParamByName ('FlagDNS').AsBooleans[i] := (mFoundByDNS in entry.Markers);
                      Query.ParamByName ('FlagICMP').AsBooleans[i] := (mFoundByICMP in entry.Markers);
                      Query.ParamByName ('FlagNRB').AsBooleans[i] := (mFoundByNRB in entry.Markers);
                      Query.ParamByName ('InterfaceIDs').AsStrings[i] := InterfaceListToStr (entry.InterfaceList);
                      Query.ParamByName ('Attributes').AsLongwords[i] := AttributesToLong (entry.Attributes);

                      inc (i);
                 end;

             Query.Execute (count,0);
             Query.Free;
        end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.StoreIPv6ConversationsToDB;

var Query : tFDQuery;
    entry : tIPv6Conversation;
    Key   : string;
    count : integer;
    i,j   : integer;

begin
     count := priv_dictIPv6Conversations.Count;

     if count > 0 then
        begin
             Query := tFDQuery.Create(nil);
             Query.Connection := priv_database;

             Query.SQL.Text := 'INSERT INTO IPv6Conversations (fileid,SourceIP,DestinationIP,FramesSrcToDst,FramesDstToSrc,BytesSrcToDst,BytesDstToSrc,FirstFrame,LastFrame,FirstFrameOffset,LastFrameOffset,FirstFrameTime,FirstFrameTimeNano,LastFrameTime,LastFrameTimeNano,' +
                               'FlagIPv6,FlagDNS,FlagICMP,FlagNRB,Attributes,InterfaceIDs)' +
                                                ' VALUES (:fileid,:SourceIP,:DestinationIP,:FramesSrcToDst,:FramesDstToSrc,:BytesSrcToDst,:BytesDstToSrc,:FirstFrame,:LastFrame,:FirstFrameOffset,:LastFrameOffset,:FirstFrameTime,:FirstFrameTimeNano,:LastFrameTime,:LastFrameTimeNano,' +
                                                         ':FlagIPv6,:FlagDNS,:FlagICMP,:FlagNRB,:Attributes,:InterfaceIDs)';

             Query.Params.ArraySize := count;

             i := 0;

             for key in priv_dictIPv6Conversations.Keys do
                 begin
                      entry := priv_dictIPv6Conversations.Items[key];
{
                      // Set FileIndex for Interface list since we only know it now

                      for j := 1 to length (entry.InterfaceList) do
                          entry.InterfaceList[j-1].FileIndex := priv_filedbindex;
}
                      Query.ParamByName ('fileid').AsIntegers[i] := priv_filedbindex;
                      Query.ParamByName ('SourceIP').AsStrings[i] := entry.IPv6AddressPair.Source.ValueAsString;
                      Query.ParamByName ('DestinationIP').AsStrings[i] := entry.IPv6AddressPair.Destination.ValueAsString;
                      Query.ParamByName ('FramesSrcToDst').AsIntegers[i] := entry.FramesSrcToDst;
                      Query.ParamByName ('FramesDstToSrc').AsIntegers[i] := entry.FramesDstToSrc;
                      Query.ParamByName ('BytesSrcToDst').AsLargeInts[i] := entry.BytesSrcToDst;
                      Query.ParamByName ('BytesDstToSrc').AsLargeInts[i] := entry.BytesDstToSrc;
                      Query.ParamByName ('FirstFrame').AsLargeInts[i] := entry.FirstFrame;
                      Query.ParamByName ('LastFrame').AsLargeInts[i] := entry.LastFrame;
                      Query.ParamByName ('FirstFrameOffset').AsLargeInts[i] := entry.FirstFrameOffset;
                      Query.ParamByName ('LastFrameOffset').AsLargeInts[i] := entry.LastFrameOffset;
                      Query.ParamByName ('FirstFrameTime').AsDateTimes[i] := entry.FirstFrameTime.datetime;
                      Query.ParamByName ('FirstFrameTimeNano').AsIntegers[i] := entry.FirstFrameTime.nanoseconds;
                      Query.ParamByName ('LastFrameTime').AsDateTimes[i] := entry.LastFrameTime.datetime;
                      Query.ParamByName ('LastFrameTimeNano').AsIntegers[i] := entry.LastFrameTime.nanoseconds;
                      Query.ParamByName ('FlagIPv6').AsBooleans[i] := (mFoundByIPv6 in entry.Markers);
                      Query.ParamByName ('FlagDNS').AsBooleans[i] := (mFoundByDNS in entry.Markers);
                      Query.ParamByName ('FlagICMP').AsBooleans[i] := (mFoundByICMP in entry.Markers);
                      Query.ParamByName ('FlagNRB').AsBooleans[i] := (mFoundByNRB in entry.Markers);
                      Query.ParamByName ('InterfaceIDs').AsStrings[i] := InterfaceListToStr (entry.InterfaceList);
                      Query.ParamByName ('Attributes').AsLongwords[i] := AttributesToLong (entry.Attributes);

                      inc (i);
                 end;

             Query.Execute (count,0);
             Query.Free;
        end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.StoreTCPConversationsToDB;

var Query  : tFDQuery;
    entry  : tTCPConversation;
    Key    : string;
    count  : integer;
    i,j    : integer;

begin
     // IPv4

     count := priv_dictTCPConversations.Count;

     if count > 0 then
        begin
             Query := tFDQuery.Create(nil);
             Query.Connection := priv_database;

             Query.SQL.Text := 'INSERT INTO TCPConversations (fileid,IPVersion,SourceIP,SourcePort,DestinationIP,DestinationPort,ClientIP,FramesSrcToDst,FramesDstToSrc,BytesSrcToDst,BytesDstToSrc,FirstFrame,LastFrame,FirstFrameOffset,LastFrameOffset,' +
                                                             'FirstFrameTime,FirstFrameTimeNano,LastFrameTime,LastFrameTimeNano,SessionStatus,SynFlagSeen,FinFlagSeen,RstFlagSeen,TCPFlags,InitialRTTns,FlagIPv4,FlagIPv6,FlagICMP,Attributes,InterfaceIDs,TunnelFlags,MiscInfo)' +
                                                    ' VALUES (:fileid,:IPVersion,:SourceIP,:SourcePort,:DestinationIP,:DestinationPort,:ClientIP,:FramesSrcToDst,:FramesDstToSrc,:BytesSrcToDst,:BytesDstToSrc,:FirstFrame,:LastFrame,:FirstFrameOffset,:LastFrameOffset,' +
                                                             ':FirstFrameTime,:FirstFrameTimeNano,:LastFrameTime,:LastFrameTimeNano,:SessionStatus,:SynFlagSeen,:FinFlagSeen,:RstFlagSeen,:TCPFlags,:InitialRTTns,:FlagIPv4,:FlagIPv6,:FlagICMP,:Attributes,:InterfaceIDs,:TunnelFlags,:MiscInfo)';

             Query.Params.ArraySize := count;

             i := 0;

             for key in priv_dictTCPConversations.Keys do
                 begin
                      entry := priv_dictTCPConversations.Items[key];

                      Query.ParamByName ('fileid').AsIntegers[i] := priv_filedbindex;
                      Query.ParamByName ('IPVersion').AsIntegers[i] := entry.SocketPair.Source.IPAddress.IPVersion;
                      Query.ParamByName ('SourceIP').AsStrings[i] := entry.SocketPair.Source.IPAddress.ValueAsString;
                      Query.ParamByName ('SourcePort').AsWords[i] := entry.SocketPair.Source.Port;
                      Query.ParamByName ('DestinationIP').AsStrings[i] := entry.SocketPair.Destination.IPAddress.ValueAsString;
                      Query.ParamByName ('DestinationPort').AsWords[i] := entry.SocketPair.Destination.Port;
                      Query.ParamByName ('ClientIP').AsStrings[i] := entry.ClientIP.ValueAsString;
                      Query.ParamByName ('FramesSrcToDst').AsIntegers[i] := entry.FramesSrcToDst;
                      Query.ParamByName ('FramesDstToSrc').AsIntegers[i] := entry.FramesDstToSrc;
                      Query.ParamByName ('BytesSrcToDst').AsLargeInts[i] := entry.BytesSrcToDst;
                      Query.ParamByName ('BytesDstToSrc').AsLargeInts[i] := entry.BytesDstToSrc;
                      Query.ParamByName ('FirstFrame').AsLargeInts[i] := entry.FirstFrame;
                      Query.ParamByName ('LastFrame').AsLargeInts[i] := entry.LastFrame;
                      Query.ParamByName ('FirstFrameOffset').AsLargeInts[i] := entry.FirstFrameOffset;
                      Query.ParamByName ('LastFrameOffset').AsLargeInts[i] := entry.LastFrameOffset;
                      Query.ParamByName ('FirstFrameTime').AsDateTimes[i] := entry.FirstFrameTime.datetime;
                      Query.ParamByName ('FirstFrameTimeNano').AsIntegers[i] := entry.FirstFrameTime.nanoseconds;
                      Query.ParamByName ('LastFrameTime').AsDateTimes[i] := entry.LastFrameTime.datetime;
                      Query.ParamByName ('LastFrameTimeNano').AsIntegers[i] := entry.LastFrameTime.nanoseconds;
                      Query.ParamByName ('SessionStatus').AsIntegers[i] := entry.SessionStatus;
                      Query.ParamByName ('SynFlagSeen').AsBooleans[i] := entry.SynSeen;
                      Query.ParamByName ('FinFlagSeen').AsBooleans[i] := entry.FinSeen;
                      Query.ParamByName ('RstFlagSeen').AsBooleans[i] := entry.RstSeen;
                      Query.ParamByName ('TCPFlags').AsIntegers[i] := getWordFromTCPFlagSet (entry.Flags);
                      Query.ParamByName ('InitialRTTns').AsLargeInts[i] := entry.InitialRTTns;
                      Query.ParamByName ('FlagIPv4').AsBooleans[i] := (mFoundByIPv4 in entry.Markers);
                      Query.ParamByName ('FlagIPv6').AsBooleans[i] := (mFoundByIPv6 in entry.Markers);
                      Query.ParamByName ('FlagICMP').AsBooleans[i] := (mFoundByICMP in entry.Markers);
                      Query.ParamByName ('MiscInfo').AsString := ''; // unused for now
                      Query.ParamByName ('InterfaceIDs').AsStrings[i] := InterfaceListToStr (entry.InterfaceList);
                      Query.ParamByName ('Attributes').AsLongwords[i] := AttributesToLong (entry.Attributes);
                      Query.ParamByName ('TunnelFlags').AsWords[i] := TunnelFlagsToWord (entry.TunnelFlags);

                      inc (i);
                 end;

             Query.Execute (count,0);
             Query.Free;
        end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.StoreUDPConversationsToDB;

var Query  : tFDQuery;
    entry  : tUDPConversation;
    Key    : string;
    count  : integer;
    i,j    : integer;

begin
     count := priv_dictUDPConversations.Count;

     if count > 0 then
        begin
             Query := tFDQuery.Create(nil);
             Query.Connection := priv_database;

             // IPv4

             Query.SQL.Text := 'INSERT INTO UDPConversations (fileid,IPVersion,SourceIP,SourcePort,DestinationIP,DestinationPort,FramesSrcToDst,FramesDstToSrc,BytesSrcToDst,BytesDstToSrc,FirstFrame,LastFrame,FirstFrameOffset,LastFrameOffset,' +
                                                             'FirstFrameTime,FirstFrameTimeNano,LastFrameTime,LastFrameTimeNano,FlagIPv4,FlagIPv6,FlagICMP,Attributes,InterfaceIDs,TunnelFlags)' +
                                                    ' VALUES (:fileid,:IPVersion,:SourceIP,:SourcePort,:DestinationIP,:DestinationPort,:FramesSrcToDst,:FramesDstToSrc,:BytesSrcToDst,:BytesDstToSrc,:FirstFrame,:LastFrame,:FirstFrameOffset,:LastFrameOffset,' +
                                                             ':FirstFrameTime,:FirstFrameTimeNano,:LastFrameTime,:LastFrameTimeNano,:FlagIPv4,:FlagIPv6,:FlagICMP,:Attributes,:InterfaceIDs,:TunnelFlags)';

             Query.Params.ArraySize := count;

             i := 0;

             for key in priv_dictUDPConversations.Keys do
                 begin
                      entry := priv_dictUDPConversations.Items[key];

                      Query.ParamByName ('fileid').AsIntegers[i] := priv_filedbindex;
                      Query.ParamByName ('IPVersion').AsIntegers[i] := entry.SocketPair.Source.IPAddress.IPVersion;
                      Query.ParamByName ('SourceIP').AsStrings[i] := entry.SocketPair.Source.IPAddress.ValueAsString;
                      Query.ParamByName ('SourcePort').AsWords[i] := entry.SocketPair.Source.Port;
                      Query.ParamByName ('DestinationIP').AsStrings[i] := entry.SocketPair.Destination.IPAddress.ValueAsString;
                      Query.ParamByName ('DestinationPort').AsWords[i] := entry.SocketPair.Destination.Port;
                      Query.ParamByName ('FramesSrcToDst').AsIntegers[i] := entry.FramesSrcToDst;
                      Query.ParamByName ('FramesDstToSrc').AsIntegers[i] := entry.FramesDstToSrc;
                      Query.ParamByName ('BytesSrcToDst').AsLargeInts[i] := entry.BytesSrcToDst;
                      Query.ParamByName ('BytesDstToSrc').AsLargeInts[i] := entry.BytesDstToSrc;
                      Query.ParamByName ('FirstFrame').AsLargeInts[i] := entry.FirstFrame;
                      Query.ParamByName ('LastFrame').AsLargeInts[i] := entry.LastFrame;
                      Query.ParamByName ('FirstFrameOffset').AsLargeInts[i] := entry.FirstFrameOffset;
                      Query.ParamByName ('LastFrameOffset').AsLargeInts[i] := entry.LastFrameOffset;
                      Query.ParamByName ('FirstFrameTime').AsDateTimes[i] := entry.FirstFrameTime.datetime;
                      Query.ParamByName ('FirstFrameTimeNano').AsIntegers[i] := entry.FirstFrameTime.nanoseconds;
                      Query.ParamByName ('LastFrameTime').AsDateTimes[i] := entry.LastFrameTime.datetime;
                      Query.ParamByName ('LastFrameTimeNano').AsIntegers[i] := entry.LastFrameTime.nanoseconds;
                      Query.ParamByName ('FlagIPv4').AsBooleans[i] := (mFoundByIPv4 in entry.Markers);
                      Query.ParamByName ('FlagIPv6').AsBooleans[i] := (mFoundByIPv6 in entry.Markers);
                      Query.ParamByName ('FlagICMP').AsBooleans[i] := (mFoundByICMP in entry.Markers);
                      Query.ParamByName ('InterfaceIDs').AsStrings[i] := InterfaceListToStr (entry.InterfaceList);
                      Query.ParamByName ('Attributes').AsLongwords[i] := AttributesToLong (entry.Attributes);
                      Query.ParamByName ('TunnelFlags').AsWords[i] := TunnelFlagsToWord (entry.TunnelFlags);

                      inc (i);
                 end;

             Query.Execute (count,0);
             Query.Free;
        end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.StoreNameResolutionsToDB;

var Query : tFDQuery;
    entry : tNameResolutionEntry;
    Key   : string;
    count : integer;
    i,j   : integer;

begin
     count := priv_dictNameResolutions.Count;

     if count > 0 then
        begin
             Query := tFDQuery.Create(nil);
             Query.Connection := priv_database;

             Query.SQL.Text := 'INSERT INTO NameResolution (fileid,ipversion,fqdn,answertype,answer,ReplyCode,FrameCount,FirstFrame,LastFrame,FirstFrameOffset,LastFrameOffset,FirstFrameTime,FirstFrameTimeNano,LastFrameTime,LastFrameTimeNano,' +
                                                           'QueryTypes,FlagIsAnswer,FlagQueryOnly,FlagNRB,FlagDNS,FlagNBNS,FlagDHCP,FlagExternalResearch,InterfaceIDs)' +
                               ' VALUES (:fileid,:ipversion,:fqdn,:answertype,:answer,:ReplyCode,:FrameCount,:FirstFrame,:LastFrame,:FirstFrameOffset,:LastFrameOffset,:FirstFrameTime,:FirstFrameTimeNano,:LastFrameTime,:LastFrameTimeNano,' +
                                        ':QueryTypes,:FlagIsAnswer,:FlagQueryOnly,:FlagNRB,:FlagDNS,:FlagNBNS,:FlagDHCP,:FlagExternalResearch,:InterfaceIDs)';

             Query.Params.ArraySize := count;

             i := 0;

             for key in priv_dictNameResolutions.Keys do
                 begin
                      entry := priv_dictNameResolutions.Items[key];

                      Query.ParamByName ('fileid').AsIntegers[i] := priv_filedbindex;
                      Query.ParamByName ('ipversion').AsIntegers[i] := entry.Answer.IPAddress.IPVersion;
                      Query.ParamByName ('fqdn').AsStrings[i] := entry.FQDN.FullFQDN;
                      Query.ParamByName ('answertype').AsIntegers[i] := AnswerTypeToInt (entry.Answer.answertype);
                      Query.ParamByName ('answer').AsStrings[i] := entry.Answer.asString;
                      Query.ParamByName ('replycode').AsIntegers[i] := DNSReplyCodeToWord (Entry.ReplyCode);
                      Query.ParamByName ('FrameCount').AsIntegers[i] := entry.FrameCount;
                      Query.ParamByName ('FirstFrame').AsLargeInts[i] := entry.FirstFrame;
                      Query.ParamByName ('LastFrame').AsLargeInts[i] := entry.LastFrame;
                      Query.ParamByName ('FirstFrameOffset').AsLargeInts[i] := entry.FirstFrameOffset;
                      Query.ParamByName ('LastFrameOffset').AsLargeInts[i] := entry.LastFrameOffset;
                      Query.ParamByName ('FirstFrameTime').AsDateTimes[i] := entry.FirstFrameTime.datetime;
                      Query.ParamByName ('FirstFrameTimeNano').AsIntegers[i] := entry.FirstFrameTime.nanoseconds;
                      Query.ParamByName ('LastFrameTime').AsDateTimes[i] := entry.LastFrameTime.datetime;
                      Query.ParamByName ('LastFrameTimeNano').AsIntegers[i] := entry.LastFrameTime.nanoseconds;
                      Query.ParamByName ('QueryTypes').AsLargeInts[i] := QueryTypesToInt64 (entry.QueryTypes);
                      Query.ParamByName ('FlagIsAnswer').AsBooleans[i] := entry.isAnswer;
                      Query.ParamByName ('FlagQueryOnly').AsBooleans[i] := entry.QueryOnly;
                      Query.ParamByName ('FlagNRB').AsBooleans[i] := (mFoundByNRB in entry.Markers);
                      Query.ParamByName ('FlagDNS').AsBooleans[i] := (mFoundByDNS in entry.Markers);
                      Query.ParamByName ('FlagNBNS').AsBooleans[i] := (mFoundByNBNS in entry.Markers);
                      Query.ParamByName ('FlagDHCP').AsBooleans[i] := (mFoundByDHCP in entry.Markers);
                      Query.ParamByName ('FlagExternalResearch').AsBooleans[i] := (mFoundByExternalResearch in entry.Markers);
                      Query.ParamByName ('InterfaceIDs').AsStrings[i] := InterfaceListToStr (entry.InterfaceList);

                      inc (i);
                 end;

             Query.Execute (count, 0);
             Query.Free;
        end;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.ReadMACEndpointsFromDB;

var Query   : tFDQuery;
    entry   : tMACEndpoint;
    help    : string;
    lhelp   : longword;
    i       : Integer;

begin
     Query := tFDQuery.Create(nil);
     Query.Connection := priv_database;

     Query.SQL.Text := 'SELECT * FROM MACEndpoints WHERE fileid=:fileid ORDER BY FirstFrame ASC;';
     Query.ParamByName ('fileid').AsInteger := priv_filedbindex;
     Query.Open;
     Query.FetchAll;

     for i := 1 to Query.RecordCount do
         begin
              entry.initialize (priv_filedbindex);

              help := Query.FieldByName ('MACAddress').AsString;
              entry.MACAddress.setfromString (help,':');
              entry.FramesReceived := Query.FieldByName ('FramesReceived').AsInteger;
              entry.FramesSent := Query.FieldByName ('FramesSent').AsInteger;
              entry.BytesReceived := Query.FieldByName ('BytesReceived').AsLargeInt;
              entry.BytesSent := Query.FieldByName ('BytesSent').AsLargeInt;
              entry.FirstFrameIndex := Query.FieldByName ('FirstFrame').AsLargeInt;
              entry.LastFrameIndex := Query.FieldByName ('LastFrame').AsLargeInt;
              entry.FirstFrameOffset := Query.FieldByName ('FirstFrameOffset').AsLargeInt;
              entry.LastFrameOffset := Query.FieldByName ('LastFrameOffset').AsLargeInt;
              entry.FirstFrameTime.datetime := Query.FieldByName ('FirstFrameTime').AsDateTime;
              entry.FirstFrameTime.nanoseconds := Query.FieldByName ('FirstFrameTimeNano').AsInteger;
              entry.LastFrameTime.datetime := Query.FieldByName ('LastFrameTime').AsDateTime;
              entry.LastFrameTime.nanoseconds := Query.FieldByName ('LastFrameTimeNano').AsInteger;

              help := IntToStr (priv_filedbindex) + '/' + Query.FieldByName ('InterfaceIDs').AsString;
              entry.InterfaceList := StrToInterfaceList (Help);

              lhelp := Query.FieldByName ('Attributes').AsLongWord;
              entry.Attributes := LongToAttributes (lhelp);

              if Query.FieldByName ('FlagEthernet').AsBoolean then
                 entry.Markers := [mFoundByEthernet]
              else
                  entry.Markers := [];

              if Query.FieldByName ('FlagARP').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByARP];

              AddMACEndpoint (entry);

              Query.Next;
         end;

     Query.Free;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.ReadMACConversationsFromDB;

var Query   : tFDQuery;
    entry   : tMACConversation;
    help    : string;
    lhelp   : longword;
    i       : Integer;

begin
     Query := tFDQuery.Create(nil);
     Query.Connection := priv_database;

     Query.SQL.Text := 'SELECT * FROM MACConversations WHERE fileid=:fileid ORDER BY FirstFrame ASC;';
     Query.ParamByName ('fileid').AsInteger := priv_filedbindex;
     Query.Open;
     Query.FetchAll;

     for i := 1 to Query.RecordCount do
         begin
              entry.initialize (priv_filedbindex);

              help := Query.FieldByName ('SourceMAC').AsString;
              entry.MACPair.Source.setfromString (help,':');
              help := Query.FieldByName ('DestinationMAC').AsString;
              entry.MACPair.Destination.setfromString (help,':');

              entry.FramesSrcToDst := Query.FieldByName ('FramesSrcToDst').AsInteger;
              entry.FramesDstToSrc := Query.FieldByName ('FramesDstToSrc').AsInteger;
              entry.BytesSrcToDst := Query.FieldByName ('BytesSrcToDst').AsLargeInt;
              entry.BytesDstToSrc := Query.FieldByName ('BytesDstToSrc').AsLargeInt;
              entry.FirstFrame := Query.FieldByName ('FirstFrame').AsLargeInt;
              entry.LastFrame := Query.FieldByName ('LastFrame').AsLargeInt;
              entry.FirstFrameOffset := Query.FieldByName ('FirstFrameOffset').AsLargeInt;
              entry.LastFrameOffset := Query.FieldByName ('LastFrameOffset').AsLargeInt;
              entry.FirstFrameTime.datetime := Query.FieldByName ('FirstFrameTime').AsDateTime;
              entry.FirstFrameTime.nanoseconds := Query.FieldByName ('FirstFrameTimeNano').AsInteger;
              entry.LastFrameTime.datetime := Query.FieldByName ('LastFrameTime').AsDateTime;
              entry.LastFrameTime.nanoseconds := Query.FieldByName ('LastFrameTimeNano').AsInteger;

              help := IntToStr (priv_filedbindex) + '/' + Query.FieldByName ('InterfaceIDs').AsString;
              entry.InterfaceList := StrToInterfaceList (Help);

              lhelp := Query.FieldByName ('Attributes').AsLongWord;
              entry.Attributes := LongToAttributes (lhelp);

              if Query.FieldByName ('FlagEthernet').AsBoolean then
                 entry.Markers := [mFoundByEthernet]
              else
                  entry.Markers := [];

              if Query.FieldByName ('FlagARP').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByARP];

              AddMACConversation (entry);

              Query.Next;
         end;

     Query.Free;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.ReadIPv4EndpointsFromDB;

var Query   : tFDQuery;
    entry   : tIPv4Endpoint;
    help    : string;
    lhelp   : longword;
    i       : Integer;

begin
     Query := tFDQuery.Create(nil);
     Query.Connection := priv_database;

     Query.SQL.Text := 'SELECT * FROM IPv4Endpoints WHERE fileid=:fileid ORDER BY FirstFrame ASC;';
     Query.ParamByName ('fileid').AsInteger := priv_filedbindex;
     Query.Open;
     Query.FetchAll;

     for i := 1 to Query.RecordCount do
         begin
              entry.initialize (priv_filedbindex);

              help := Query.FieldByName ('IPAddress').AsString;
              entry.IPv4Address.setValueFromString (help);

              entry.FramesReceived := Query.FieldByName ('FramesReceived').AsInteger;
              entry.FramesSent := Query.FieldByName ('FramesSent').AsInteger;
              entry.BytesReceived := Query.FieldByName ('BytesReceived').AsLargeInt;
              entry.BytesSent := Query.FieldByName ('BytesSent').AsLargeInt;
              entry.FirstFrame := Query.FieldByName ('FirstFrame').AsLargeInt;
              entry.LastFrame := Query.FieldByName ('LastFrame').AsLargeInt;
              entry.FirstFrameOffset := Query.FieldByName ('FirstFrameOffset').AsLargeInt;
              entry.LastFrameOffset := Query.FieldByName ('LastFrameOffset').AsLargeInt;
              entry.FirstFrameTime.datetime := Query.FieldByName ('FirstFrameTime').AsDateTime;
              entry.FirstFrameTime.nanoseconds := Query.FieldByName ('FirstFrameTimeNano').AsInteger;
              entry.LastFrameTime.datetime := Query.FieldByName ('LastFrameTime').AsDateTime;
              entry.LastFrameTime.nanoseconds := Query.FieldByName ('LastFrameTimeNano').AsInteger;
              entry.TTL := Query.FieldByName ('TTL').AsInteger;

              help := IntToStr (priv_filedbindex) + '/' + Query.FieldByName ('InterfaceIDs').AsString;
              entry.InterfaceList := StrToInterfaceList (Help);

              lhelp := Query.FieldByName ('Attributes').AsLongWord;
              entry.Attributes := LongToAttributes (lhelp);

              if Query.FieldByName ('FlagARP').AsBoolean then
                 entry.Markers := [mFoundByARP]
              else
                  entry.Markers := [];

              if Query.FieldByName ('FlagIPv4').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByIPv4];

              if Query.FieldByName ('FlagDNS').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByDNS];

              if Query.FieldByName ('FlagICMP').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByICMP];

              if Query.FieldByName ('FlagNRB').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByNRB];

              AddIPv4Endpoint (entry);

              Query.Next;
         end;

     Query.Free;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.ReadIPv4ConversationsFromDB;

var Query   : tFDQuery;
    entry   : tIPv4Conversation;
    help    : string;
    lhelp   : longword;
    i       : Integer;

begin
     Query := tFDQuery.Create(nil);
     Query.Connection := priv_database;

     Query.SQL.Text := 'SELECT * FROM IPv4Conversations WHERE fileid=:fileid ORDER BY FirstFrame ASC;';
     Query.ParamByName ('fileid').AsInteger := priv_filedbindex;
     Query.Open;
     Query.FetchAll;

     for i := 1 to Query.RecordCount do
         begin
              entry.initialize (priv_filedbindex);

              help := Query.FieldByName ('SourceIP').AsString;
              entry.IPv4AddressPair.Source.setValueFromString(help);

              help := Query.FieldByName ('DestinationIP').AsString;
              entry.IPv4AddressPair.Destination.setValueFromString(help);

              entry.FramesSrcToDst := Query.FieldByName ('FramesSrcToDst').AsInteger;
              entry.FramesDstToSrc := Query.FieldByName ('FramesDstToSrc').AsInteger;
              entry.BytesSrcToDst := Query.FieldByName ('BytesSrcToDst').AsLargeInt;
              entry.BytesDstToSrc := Query.FieldByName ('BytesDstToSrc').AsLargeInt;
              entry.FirstFrame := Query.FieldByName ('FirstFrame').AsLargeInt;
              entry.LastFrame := Query.FieldByName ('LastFrame').AsLargeInt;
              entry.FirstFrameOffset := Query.FieldByName ('FirstFrameOffset').AsLargeInt;
              entry.LastFrameOffset := Query.FieldByName ('LastFrameOffset').AsLargeInt;
              entry.FirstFrameTime.datetime := Query.FieldByName ('FirstFrameTime').AsDateTime;
              entry.FirstFrameTime.nanoseconds := Query.FieldByName ('FirstFrameTimeNano').AsInteger;
              entry.LastFrameTime.datetime := Query.FieldByName ('LastFrameTime').AsDateTime;
              entry.LastFrameTime.nanoseconds := Query.FieldByName ('LastFrameTimeNano').AsInteger;

              help := IntToStr (priv_filedbindex) + '/' + Query.FieldByName ('InterfaceIDs').AsString;
              entry.InterfaceList := StrToInterfaceList (Help);

              lhelp := Query.FieldByName ('Attributes').AsLongWord;
              entry.Attributes := LongToAttributes (lhelp);

              if Query.FieldByName ('FlagARP').AsBoolean then
                 entry.Markers := [mFoundByARP]
              else
                  entry.Markers := [];

              if Query.FieldByName ('FlagIPv4').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByIPv4];

              if Query.FieldByName ('FlagDNS').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByDNS];

              if Query.FieldByName ('FlagICMP').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByICMP];

              if Query.FieldByName ('FlagNRB').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByNRB];

              AddIPv4Conversation (entry);

              Query.Next;
         end;

     Query.Free;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.ReadIPv6EndpointsFromDB;

var Query   : tFDQuery;
    entry   : tIPv6Endpoint;
    help    : string;
    lhelp   : longword;
    i       : Integer;

begin
     Query := tFDQuery.Create(nil);
     Query.Connection := priv_database;

     Query.SQL.Text := 'SELECT * FROM IPv6Endpoints WHERE fileid=:fileid ORDER BY FirstFrame ASC;';
     Query.ParamByName ('fileid').AsInteger := priv_filedbindex;
     Query.Open;
     Query.FetchAll;

     for i := 1 to Query.RecordCount do
         begin
              entry.initialize (priv_filedbindex);

              help := Query.FieldByName ('IPAddress').AsString;
              entry.IPv6Address.setValueFromString (help);

              entry.FramesReceived := Query.FieldByName ('FramesReceived').AsInteger;
              entry.FramesSent := Query.FieldByName ('FramesSent').AsInteger;
              entry.BytesReceived := Query.FieldByName ('BytesReceived').AsLargeInt;
              entry.BytesSent := Query.FieldByName ('BytesSent').AsLargeInt;
              entry.FirstFrame := Query.FieldByName ('FirstFrame').AsLargeInt;
              entry.LastFrame := Query.FieldByName ('LastFrame').AsLargeInt;
              entry.FirstFrameOffset := Query.FieldByName ('FirstFrameOffset').AsLargeInt;
              entry.LastFrameOffset := Query.FieldByName ('LastFrameOffset').AsLargeInt;
              entry.FirstFrameTime.datetime := Query.FieldByName ('FirstFrameTime').AsDateTime;
              entry.FirstFrameTime.nanoseconds := Query.FieldByName ('FirstFrameTimeNano').AsInteger;
              entry.LastFrameTime.datetime := Query.FieldByName ('LastFrameTime').AsDateTime;
              entry.LastFrameTime.nanoseconds := Query.FieldByName ('LastFrameTimeNano').AsInteger;
              entry.HopLimit := Query.FieldByName ('HopLimit').AsInteger;

              help := IntToStr (priv_filedbindex) + '/' + Query.FieldByName ('InterfaceIDs').AsString;
              entry.InterfaceList := StrToInterfaceList (Help);

              lhelp := Query.FieldByName ('Attributes').AsLongWord;
              entry.Attributes := LongToAttributes (lhelp);

              if Query.FieldByName ('FlagIPv6').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByIPv4]
              else
                  entry.Markers := [];

              if Query.FieldByName ('FlagDNS').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByDNS];

              if Query.FieldByName ('FlagICMP').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByICMP];

              if Query.FieldByName ('FlagNRB').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByNRB];

              AddIPv6Endpoint (entry);

              Query.Next;
         end;

     Query.Free;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.ReadIPv6ConversationsFromDB;

var Query   : tFDQuery;
    entry   : tIPv6Conversation;
    help    : string;
    lhelp   : longword;
    i       : Integer;

begin
     Query := tFDQuery.Create(nil);
     Query.Connection := priv_database;

     Query.SQL.Text := 'SELECT * FROM IPv6Conversations WHERE fileid=:fileid ORDER BY FirstFrame ASC;';
     Query.ParamByName ('fileid').AsInteger := priv_filedbindex;
     Query.Open;
     Query.FetchAll;

     for i := 1 to Query.RecordCount do
         begin
              entry.initialize (priv_filedbindex);

              help := Query.FieldByName ('SourceIP').AsString;
              entry.IPv6AddressPair.Source.setValueFromString(help);

              help := Query.FieldByName ('DestinationIP').AsString;
              entry.IPv6AddressPair.Destination.setValueFromString(help);

              entry.FramesSrcToDst := Query.FieldByName ('FramesSrcToDst').AsInteger;
              entry.FramesDstToSrc := Query.FieldByName ('FramesDstToSrc').AsInteger;
              entry.BytesSrcToDst := Query.FieldByName ('BytesSrcToDst').AsLargeInt;
              entry.BytesDstToSrc := Query.FieldByName ('BytesDstToSrc').AsLargeInt;
              entry.FirstFrame := Query.FieldByName ('FirstFrame').AsLargeInt;
              entry.LastFrame := Query.FieldByName ('LastFrame').AsLargeInt;
              entry.FirstFrameOffset := Query.FieldByName ('FirstFrameOffset').AsLargeInt;
              entry.LastFrameOffset := Query.FieldByName ('LastFrameOffset').AsLargeInt;
              entry.FirstFrameTime.datetime := Query.FieldByName ('FirstFrameTime').AsDateTime;
              entry.FirstFrameTime.nanoseconds := Query.FieldByName ('FirstFrameTimeNano').AsInteger;
              entry.LastFrameTime.datetime := Query.FieldByName ('LastFrameTime').AsDateTime;
              entry.LastFrameTime.nanoseconds := Query.FieldByName ('LastFrameTimeNano').AsInteger;

              help := IntToStr (priv_filedbindex) + '/' + Query.FieldByName ('InterfaceIDs').AsString;
              entry.InterfaceList := StrToInterfaceList (Help);

              lhelp := Query.FieldByName ('Attributes').AsLongWord;
              entry.Attributes := LongToAttributes (lhelp);

              if Query.FieldByName ('FlagIPv6').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByIPv4]
              else
                  entry.Markers := [];

              if Query.FieldByName ('FlagDNS').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByDNS];

              if Query.FieldByName ('FlagICMP').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByICMP];

              if Query.FieldByName ('FlagNRB').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByNRB];

              AddIPv6Conversation (entry);

              Query.Next;
         end;

     Query.Free;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.ReadTCPConversationsFromDB;

var Query   : tFDQuery;
    entry   : tTCPConversation;
    test    : uint64;
    help    : string;
    lhelp   : longword;
    i       : Integer;

begin
     Query := tFDQuery.Create(nil);
     Query.Connection := priv_database;

     Query.SQL.Text := 'SELECT * FROM TCPConversations WHERE fileid=:fileid ORDER BY FirstFrame ASC;';
     Query.ParamByName ('fileid').AsInteger := priv_filedbindex;
     Query.Open;
     Query.FetchAll;

     for i := 1 to Query.RecordCount do
         begin
              entry.initialize (priv_filedbindex);

              help := Query.FieldByName ('SourceIP').AsString;
              entry.SocketPair.Source.IPAddress.setValueFromString (help);
              entry.SocketPair.Source.Port := Query.FieldByName('SourcePort').AsInteger;

              help := Query.FieldByName ('DestinationIP').AsString;
              entry.SocketPair.Destination.IPAddress.setValueFromString (help);
              entry.SocketPair.Destination.Port := Query.FieldByName('DestinationPort').AsInteger;
              help := Query.FieldByName ('ClientIP').AsString;
              entry.ClientIP.setValueFromString (help);

              entry.FramesSrcToDst := Query.FieldByName ('FramesSrcToDst').AsInteger;
              entry.FramesDstToSrc := Query.FieldByName ('FramesDstToSrc').AsInteger;
              entry.BytesSrcToDst := Query.FieldByName ('BytesSrcToDst').AsLargeInt;
              entry.BytesDstToSrc := Query.FieldByName ('BytesDstToSrc').AsLargeInt;
              entry.FirstFrame := Query.FieldByName ('FirstFrame').AsLargeInt;
              entry.LastFrame := Query.FieldByName ('LastFrame').AsLargeInt;
              entry.FirstFrameOffset := Query.FieldByName ('FirstFrameOffset').AsLargeInt;
              entry.LastFrameOffset := Query.FieldByName ('LastFrameOffset').AsLargeInt;
              entry.FirstFrameTime.datetime := Query.FieldByName ('FirstFrameTime').AsDateTime;
              entry.FirstFrameTime.nanoseconds := Query.FieldByName ('FirstFrameTimeNano').AsInteger;
              entry.LastFrameTime.datetime := Query.FieldByName ('LastFrameTime').AsDateTime;
              entry.LastFrameTime.nanoseconds := Query.FieldByName ('LastFrameTimeNano').AsInteger;

              // Need to add the file index to the interface ID list to make sure it ends up in the list

              Help := IntToStr (priv_filedbindex) + '/' + Query.FieldByName ('InterfaceIDs').AsString;
              entry.InterfaceList := StrToInterfaceList (Help);

              lhelp := Query.FieldByName ('Attributes').AsLongWord;
              entry.Attributes := LongToAttributes (lhelp);

              entry.SynSeen := Query.FieldByName ('SynFlagSeen').AsBoolean;
              entry.FinSeen := Query.FieldByName ('FinFlagSeen').AsBoolean;
              entry.RstSeen := Query.FieldByName ('RstFlagSeen').AsBoolean;
              entry.SessionStatus := Query.FieldByName ('SessionStatus').AsInteger;

              if Query.FieldByName ('FlagIPv4').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByIPv4]
              else
                  entry.Markers := [];

              if Query.FieldByName ('FlagIPv6').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByIPv6];

              if Query.FieldByName ('FlagICMP').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByICMP];

              entry.Flags := getTCPFlagSetFromWord (Query.FieldByName ('TCPFlags').AsInteger);
              entry.InitialRTTns := Query.FieldByName ('InitialRTTns').AsLargeInt;

              entry.Tunnelflags := WordToTunnelFlags (Query.FieldByName ('Tunnelflags').AsInteger);

              AddTCPConversation (entry);

              Query.Next;
         end;

     Query.Free;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.ReadUDPConversationsFromDB;

var Query   : tFDQuery;
    entry   : tUDPConversation;
    help    : string;
    lhelp   : longword;
    i       : Integer;

begin
     Query := tFDQuery.Create(nil);
     Query.Connection := priv_database;

     Query.SQL.Text := 'SELECT * FROM UDPConversations WHERE fileid=:fileid ORDER BY FirstFrame ASC;';
     Query.ParamByName ('fileid').AsInteger := priv_filedbindex;
     Query.Open;
     Query.FetchAll;

     for i := 1 to Query.RecordCount do
         begin
              entry.initialize (priv_filedbindex);

              help := Query.FieldByName ('SourceIP').AsString;
              entry.SocketPair.Source.IPAddress.setValueFromString (help);
              entry.SocketPair.Source.Port := Query.FieldByName('SourcePort').AsInteger;

              help := Query.FieldByName ('DestinationIP').AsString;
              entry.SocketPair.Destination.IPAddress.setValueFromString (help);
              entry.SocketPair.Destination.Port := Query.FieldByName('DestinationPort').AsInteger;

              entry.FramesSrcToDst := Query.FieldByName ('FramesSrcToDst').AsInteger;
              entry.FramesDstToSrc := Query.FieldByName ('FramesDstToSrc').AsInteger;
              entry.BytesSrcToDst := Query.FieldByName ('BytesSrcToDst').AsLargeInt;
              entry.BytesDstToSrc := Query.FieldByName ('BytesDstToSrc').AsLargeInt;
              entry.FirstFrame := Query.FieldByName ('FirstFrame').AsLargeInt;
              entry.LastFrame := Query.FieldByName ('LastFrame').AsLargeInt;
              entry.FirstFrameOffset := Query.FieldByName ('FirstFrameOffset').AsLargeInt;
              entry.LastFrameOffset := Query.FieldByName ('LastFrameOffset').AsLargeInt;
              entry.FirstFrameTime.datetime := Query.FieldByName ('FirstFrameTime').AsDateTime;
              entry.FirstFrameTime.nanoseconds := Query.FieldByName ('FirstFrameTimeNano').AsInteger;
              entry.LastFrameTime.datetime := Query.FieldByName ('LastFrameTime').AsDateTime;
              entry.LastFrameTime.nanoseconds := Query.FieldByName ('LastFrameTimeNano').AsInteger;

              Help := IntToStr (priv_filedbindex) + '/' + Query.FieldByName ('InterfaceIDs').AsString;
              entry.InterfaceList := StrToInterfaceList (Help);

              lhelp := Query.FieldByName ('Attributes').AsLongWord;
              entry.Attributes := LongToAttributes (lhelp);

              if Query.FieldByName ('FlagIPv4').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByIPv4]
              else
                  entry.Markers := [];

              if Query.FieldByName ('FlagIPv6').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByIPv6];

              if Query.FieldByName ('FlagICMP').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByICMP];

              entry.Tunnelflags := WordToTunnelFlags (Query.FieldByName ('Tunnelflags').AsInteger);

              AddUDPConversation (entry);
              Query.Next;
         end;

     Query.Free;
end;

(* ------------------------------------------- *)

procedure tConversationStorage.ReadNameResolutionsFromDB;

var Query   : tFDQuery;
    entry   : tNameResolutionEntry;
    help    : string;
//  lhelp   : longword;
    i64Help : uint64;
    ipver   : integer;
    ipv4    : tIPv4Address;
    ipv6    : tIPv6Address;
    fqdn    : tFQDN;
    i       : Integer;

begin
     Query := tFDQuery.Create(nil);
     Query.Connection := priv_database;

     Query.SQL.Text := 'SELECT * FROM NameResolution WHERE fileid=:fileid ORDER BY FirstFrame ASC;';
     Query.ParamByName ('fileid').AsInteger := priv_filedbindex;
     Query.Open;
     Query.FetchAll;

     for i := 1 to Query.RecordCount do
         begin
              entry.initialize (priv_filedbindex);

              ipver := Query.FieldByName ('ipversion').AsInteger;
              entry.FQDN.setFQDN (Query.FieldByName ('fqdn').AsString);

              entry.Answer.AnswerType := IntegerToAnswerType (Query.FieldByName ('answertype').AsInteger);

              case entry.Answer.AnswerType of
               nraUnknown   : entry.Answer.initialize;
               nraIPAddress : begin
                                   if ipver = 4 then
                                      begin
                                           ipv4.setValueFromString (Query.FieldByName('answer').AsString);
                                           entry.Answer.IPAddress.initialize (ipv4);
                                      end
                                   else
                                       begin
                                            ipv6.setValueFromString (Query.FieldByName('answer').AsString);
                                            entry.Answer.IPAddress.initialize (ipv6);
                                       end;
                              end;
               nraFQDN      : begin
                                   fqdn.initialize;
                                   fqdn.setFQDN (Query.FieldByName('answer').AsString);
                                   entry.Answer.initialize (fqdn);
                              end;
               nraText      : entry.Answer.initialize (Query.FieldByName('answer').AsString);
              end;

              entry.ReplyCode := DNSReplyCodeFromWord (Query.FieldByName ('replycode').AsInteger);
              entry.FrameCount := Query.FieldByName ('FrameCount').AsInteger;
              entry.FirstFrame := Query.FieldByName ('FirstFrame').AsLargeInt;
              entry.LastFrame := Query.FieldByName ('LastFrame').AsLargeInt;
              entry.FirstFrameOffset := Query.FieldByName ('FirstFrameOffset').AsLargeInt;
              entry.LastFrameOffset := Query.FieldByName ('LastFrameOffset').AsLargeInt;
              entry.FirstFrameTime.datetime := Query.FieldByName ('FirstFrameTime').AsDateTime;
              entry.FirstFrameTime.nanoseconds := Query.FieldByName ('FirstFrameTimeNano').AsInteger;
              entry.LastFrameTime.datetime := Query.FieldByName ('LastFrameTime').AsDateTime;
              entry.LastFrameTime.nanoseconds := Query.FieldByName ('LastFrameTimeNano').AsInteger;

              i64Help := Query.FieldByName ('QueryTypes').AsLargeInt;

              entry.QueryTypes := Int64ToQueryTypes (i64Help);
              entry.QueryOnly := Query.FieldByName ('FlagQueryOnly').AsBoolean;
              entry.isAnswer := Query.FieldByName ('FlagIsAnswer').AsBoolean;

              if Query.FieldByName ('FlagNRB').AsBoolean then
                 entry.Markers := [mFoundByNRB]
              else
                  entry.Markers := [];

              if Query.FieldByName ('FlagDNS').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByDNS];

              if Query.FieldByName ('FlagNBNS').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByNBNS];

              if Query.FieldByName ('FlagDHCP').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByDHCP];

              if Query.FieldByName ('FlagExternalResearch').AsBoolean then
                 entry.Markers := entry.Markers + [mFoundByExternalResearch];

              // TODO: Special block, please remove try/except after working through GW traces

              try
                 Help := IntToStr (priv_filedbindex) + '/' + Query.FieldByName ('InterfaceIDs').AsString;
                 entry.InterfaceList := StrToInterfaceList (Help);
              except
                    setLength (entry.InterfaceList, 0);
              end;

              AddNameResolution (entry);

              Query.Next;
         end;

     Query.Free;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetMACEndpointCount: integer;

begin
     result := priv_dictMACEndpoints.Count;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetMACConversationCount: integer;

begin
     result := priv_dictMACConversations.Count;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetIPv4EndpointCount: integer;

begin
     result := priv_dictIPv4Endpoints.Count;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetIPv4ConversationCount: integer;

begin
     result := priv_dictIPv4Conversations.Count;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetIPv6EndpointCount: integer;

begin
     result := priv_dictIPv6Endpoints.Count;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetIPv6ConversationCount: integer;

begin
     result := priv_dictIPv6Conversations.Count;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetTCPConversationCount: integer;

begin
     result := priv_dictTCPConversations.Count;
end;

(* ------------------------------------------- *)

function tConversationStorage.getUDPConversationCount: integer;

begin
     result := priv_dictUDPConversations.Count;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetCountTable: tConversationCountTable;

var value : tConversationCountTable;

begin
     value.EtherAddressCount := priv_dictMACEndpoints.Count;
     value.IPv4AddressCount  := priv_dictIPv4Endpoints.Count;
     value.IPv6AddressCount  := priv_dictIPv6Endpoints.Count;
     value.EtherPairCount    := priv_dictMACConversations.Count;
     value.IPv4PairCount     := priv_dictIPv4Conversations.Count;
     value.IPv6PairCount     := priv_dictIPv6Conversations.Count;
     value.TCPPairCount      := priv_dictTCPConversations.Count;
     value.UDPPairCount      := priv_dictUDPConversations.Count;

     result := value;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetMACEndpointEntry (const Value: tMACAddress): tMACEndpoint;

var key : string;

begin
     key := value.DictionaryKey;

     if priv_dictMACEndpoints.ContainsKey (key) then
        result := priv_dictMACEndpoints.Items[key]
     else
         result.initialize;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetIPv4EndpointEntry (const Value: tIPv4Address): tIPv4Endpoint;

var key : string;

begin
     key := value.DictionaryKey;

     if priv_dictIPv4Endpoints.ContainsKey (key) then
        result := priv_dictIPv4Endpoints.Items[key]
     else
         result.initialize;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetIPV6EndpointEntry (const Value: tIPv6Address): tIPv6Endpoint;

var key : string;

begin
     key := value.DictionaryKey;

     if priv_dictIPv6Endpoints.ContainsKey (key) then
        result := priv_dictIPv6Endpoints.Items[key]
     else
         result.initialize;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetMACConversationEntry (const Value: tMACAddressPair): tMACConversation;

var key : string;

begin
     key := value.DictionaryKey;

     if priv_dictMACConversations.ContainsKey (key) then
        result := priv_dictMACConversations.Items[key]
     else
         result.initialize;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetIPv4ConversationEntry (const Value: tIPv4AddressPair): tIPv4Conversation;

var key : string;

begin
     key := Value.DictionaryKey;

     if priv_dictIPv4Conversations.ContainsKey (key) then
        result := priv_dictIPv4Conversations.Items[key]
     else
         result.initialize;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetIPv6ConversationEntry (const Value: tIPv6AddressPair): tIPv6Conversation;

var key : string;

begin
     key := Value.DictionaryKey;

     if priv_dictIPv6Conversations.ContainsKey (key) then
        result := priv_dictIPv6Conversations.Items[key]
     else
         result.initialize;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetTCPConversationEntry (const Value: tIPv4SocketPair): tTCPConversation;

var key : string;

begin
     key := value.DictionaryKey;

     if priv_dictTCPConversations.ContainsKey (key) then
        result := priv_dictTCPConversations.Items[key]
     else
         result.initialize;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetTCPConversationEntry (const Value: tIPv6SocketPair): tTCPConversation;

var key : string;

begin
     key := value.DictionaryKey;

     if priv_dictTCPConversations.ContainsKey (key) then
        result := priv_dictTCPConversations.Items[key]
     else
         result.initialize;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetUDPConversationEntry (const Value: tIPv4SocketPair): tUDPConversation;

var key : string;

begin
     key := value.DictionaryKey;

     if priv_dictUDPConversations.ContainsKey (key) then
        result := priv_dictUDPConversations.Items[key]
     else
         result.initialize;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetUDPConversationEntry (const Value: tIPv6SocketPair): tUDPConversation;

var key : string;

begin
     key := value.DictionaryKey;

     if priv_dictUDPConversations.ContainsKey (key) then
        result := priv_dictUDPConversations.Items[key]
     else
         result.initialize;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetSocketEntry (const Protocol : byte; const Socket: tIPSocket): tSocketDetails;

var key : string;

begin
     key := Protocol.ToString + '-' + Socket.DictionaryKey;

     if priv_dictTCPSockets.ContainsKey (key) then
        result := priv_dictTCPSockets.Items[key]
     else
         result.initialize;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetMACEndpointList : tMACEndpointList;

var list : tMACEndpointList;
    key  : string;
    i    : integer;

begin
     SiMain.EnterMethod ('tConversationStorage.GetMACEndpointList');

     setlength (list, priv_dictMACEndpoints.Count);
     i := 0;

     for key in priv_dictMACEndpoints.keys do
         begin
              list[i] := priv_dictMACEndpoints.Items[key];

              inc (i);
         end;

     result := list;

     SiMain.LeaveMethod ('tConversationStorage.GetMACEndpointList');
end;

(* ------------------------------------------- *)

function tConversationStorage.GetMACConversationList : tMACConversationList;

var list : tMACConversationList;
    key  : string;
    i    : integer;

begin
     SiMain.EnterMethod ('tConversationStorage.GetMACConversationList');

     setlength (list, priv_dictMACConversations.Count);
     i := 0;

     for key in priv_dictMACConversations.keys do
         begin
              list[i] := priv_dictMACConversations.Items[key];

              inc (i);
         end;

     result := list;

     SiMain.LeaveMethod ('tConversationStorage.GetMACConversationList');
end;

(* ------------------------------------------- *)

function tConversationStorage.GetIPv4EndpointsList: tIPv4EndpointList;

var list : tIPv4EndpointList;
    key  : string;
    i    : integer;

begin
     SiMain.EnterMethod ('tConversationStorage.GetIPv4EndpointsList');

     setlength (list, priv_dictIPv4Endpoints.Count);
     i := 0;

     for key in priv_dictIPv4Endpoints.keys do
         begin
              list[i] := priv_dictIPv4Endpoints.Items[key];

              inc (i);
         end;

     result := list;

     SiMain.LeaveMethod ('tConversationStorage.GetIPv4EndpointsList');
end;

(* ------------------------------------------- *)

function tConversationStorage.GetIPv4ConversationList: tIPv4ConversationList;

var list : tIPv4ConversationList;
    key  : string;
    i    : integer;

begin
     SiMain.EnterMethod ('tConversationStorage.GetIPv4ConversationList');

     setlength (list, priv_dictIPv4Conversations.Count);
     i := 0;

     for key in priv_dictIPv4Conversations.keys do
         begin
              list[i] := priv_dictIPv4Conversations.Items[key];

              // To keep the display consistent with Wireshark, we show un-normalized pairs

              if caAddressPairFlipped in list[i].Attributes then
                 list[i].IPv4AddressPair.swap;

              inc (i);
         end;

     result := list;

     SiMain.LeaveMethod ('tConversationStorage.GetIPv4ConversationList');
end;

(* ------------------------------------------- *)

function tConversationStorage.GetIPv6EndpointsList: tIPv6EndpointList;

var list : tIPv6EndpointList;
    key  : string;
    i    : integer;

begin
     SiMain.EnterMethod ('tConversationStorage.GetIPv6EndpointsList');

     setlength (list, priv_dictIPv6Endpoints.Count);
     i := 0;

     for key in priv_dictIPv6Endpoints.keys do
         begin
              list[i] := priv_dictIPv6Endpoints.Items[key];

              inc (i);
         end;

     result := list;

     SiMain.EnterMethod ('tConversationStorage.GetIPv6EndpointsList');
end;

(* ------------------------------------------- *)

function tConversationStorage.GetIPv6ConversationList : tIPv6ConversationList;

var list : tIPv6ConversationList;
    key  : string;
    i    : integer;

begin
     SiMain.EnterMethod ('tConversationStorage.GetIPv6ConversationList');

     setlength (list, priv_dictIPv6Conversations.Count);
     i := 0;

     for key in priv_dictIPv6Conversations.keys do
         begin
              list[i] := priv_dictIPv6Conversations.Items[key];

              // To keep the display consistent with Wireshark, we show un-normalized pairs

              if caAddressPairFlipped in list[i].Attributes then
                 list[i].IPv6AddressPair.swap;

              inc (i);
         end;

     result := list;

     SiMain.LeaveMethod ('tConversationStorage.GetIPv6ConversationList');
end;

(* ------------------------------------------- *)

function tConversationStorage.GetTCPConversationList (const SortMode : tTCPConversationSortMode = csmStandard) : tTCPConversationList;

var list    : tTCPConversationList;
    entry   : tTCPConversation;
    sorter  : tStringlist;
    sortkey : string;
    key     : string;
    i       : integer;

begin
     SiMain.EnterMethod ('tConversationStorage.GetTCPConversationList');

     setlength (list, priv_dictTCPConversations.Count);
     sorter := tStringlist.Create;

     for key in priv_dictTCPConversations.keys do
         begin
              entry := priv_dictTCPConversations.Items[key];

              // Get sort key and use it to sort the key list

              case SortMode of
               csmStandard : sortkey := Format ('%.6d-%.12d', [entry.FirstFileID, entry.FirstFrame]);
               csmIPPair   : sortkey := entry.SocketPair.IPAddressPair.ValueAsString (true) + '-' + Format ('%.6d-%.12d', [entry.FirstFileID, entry.FirstFrame]);
              end;

              sorter.Add (sortkey + '=' + key);
         end;

     sorter.Sort;

     for i := 1 to sorter.Count do
         begin
              key := sorter.ValueFromIndex[i-1];

              list[i-1] := priv_dictTCPConversations.Items[key];

              // To keep the display consistent with Wireshark, we show un-normalized pairs

              if caAddressPairFlipped in list[i-1].Attributes then
                 list[i-1].SocketPair.swap;

              // Set the string that we use to filter against in the conversation display list

              list[i-1].UpdateFilterMatchText;
         end;

     sorter.Free;

     result := list;

     SiMain.LeaveMethod ('tConversationStorage.GetTCPConversationList');
end;

(* ------------------------------------------- *)

function tConversationStorage.GetUDPConversationList: tUDPConversationList;

var list    : tUDPConversationList;
    entry   : tUDPConversation;
    sorter  : tStringlist;
    sortkey : string;
    key     : string;
    i       : integer;

begin
     SiMain.EnterMethod ('tConversationStorage.GetUDPConversationList');

     setlength (list, priv_dictUDPConversations.Count);
     sorter := tStringlist.Create;

     for key in priv_dictUDPConversations.keys do
         begin
              entry := priv_dictUDPConversations.Items[key];

              // Get sort key and use it to sort the key list

              sortkey := Format('%.6d-%.12d', [entry.FirstFileID, entry.FirstFrame]);
              sorter.Add (sortkey + '=' + key);
         end;

     sorter.Sort;

     for i := 1 to sorter.Count do
         begin
              key := sorter.ValueFromIndex[i-1];

              list[i-1] := priv_dictUDPConversations.Items[key];

              // To keep the display consistent with Wireshark, we show un-normalized pairs

              if caAddressPairFlipped in list[i-1].Attributes then
                 list[i-1].SocketPair.swap;
         end;

     sorter.Free;

     result := list;
     SiMain.LeaveMethod ('tConversationStorage.GetUDPConversationList');
end;

(* ------------------------------------------- *)

function tConversationStorage.GetNameResolutionList : tNameResolutionList;

var sortarray : tArray<string>;
    sortlist  : tNameResolutionList;
    key       : string;
    count     : integer;

begin
     SiMain.EnterMethod ('tConversationStorage.GetNameResolutionList');

     sortarray := priv_dictNameResolutions.Keys.ToArray;
     tArray.sort<string>(sortarray);

     setlength (sortlist, priv_dictNameResolutions.Count);
     count := 0;

     for key in sortarray  do
         begin
              sortlist[count] := priv_dictNameResolutions.Items[key];
              inc (count);
         end;

     result := sortlist;

     SiMain.LeaveMethod ('tConversationStorage.GetNameResolutionList');
end;

(* ------------------------------------------- *)

procedure tConversationStorage.Clear;

begin
     priv_OverallComplexity := ComplexityUnknown;

     priv_dictMACEndpoints.Clear;
     priv_dictIPv4Endpoints.Clear;
     priv_dictIPv6Endpoints.Clear;
     priv_dictMACConversations.Clear;
     priv_dictIPv4Conversations.Clear;
     priv_dictIPv6Conversations.Clear;
     priv_dictTCPConversations.Clear;
     priv_dictUDPConversations.Clear;
     priv_dictNameResolutions.Clear;
end;

(* ------------------------------------------- *)

function tConversationStorage.HasRecords: boolean;

begin
     result := (priv_dictMACEndpoints.Count > 0) or
               (priv_dictMACConversations.Count > 0) or
               (priv_dictIPv4Endpoints.Count > 0) or
               (priv_dictIPv4Conversations.Count > 0) or
               (priv_dictIPv6Endpoints.Count > 0) or
               (priv_dictIPv6Conversations.Count > 0) or
               (priv_dictTCPConversations.Count > 0) or
               (priv_dictUDPConversations.Count > 0) or
               (priv_dictNameResolutions.Count > 0);
end;

(* ------------------------------------------- *)

procedure tConversationStorage.ForceFileIDOnInterfaceList (var InterfaceList : tInterfaceFileIDList; const FileID: integer);

var i : integer;

begin
     for i := 1 to length (Interfacelist) do
         Interfacelist[i-1].FileIndex := FileID;
end;

(* ------------------------------------------- *)

function tConversationStorage.GetInterfaceMapListForFilterSet (const FilterSet: tFilterSet): tInterfaceFileIDList;

var maplist     : tInterfaceFileIDList;
    socketpair4 : tIPv4SocketPair;
    socketpair6 : tIPv6SocketPair;
    addrpair4   : tIPv4AddressPair;
    addrpair6   : tIPv6AddressPair;
    ipv4conv    : tIPv4Conversation;
    ipv6conv    : tIPv6Conversation;
    tcpconv     : tTCPConversation;
    udpconv     : tUDPConversation;
    filter      : tFrameFilter;
    i           : integer;

begin
     SiMain.EnterMethod ('tConversationStorage.GetInterfaceMapListForFilterSet');

     setlength (maplist, 0);

     filter := FilterSet.getFirst;

     while not filter.isUnspecified do
           begin
{
     for i := 1 to length (FilterSet) do
         begin
              filter := FilterSet[i-1];
}
                socketpair4.initialize;
                socketpair6.initialize;

                // Build socket so we can find the conversation for it

                if filter.SectionCount > 0 then
                   begin
                        if filter.IPv4Filter.AddressFilterType <> ftIgnore then
                           begin
                                socketpair4.Source.IPAddress := filter.IPv4Filter.SourceAddress;
                                socketpair4.Destination.IPAddress := filter.IPv4Filter.DestinationAddress;

                                if (filter.PortFilter.Layer4Protocol = l4tcp) or (filter.PortFilter.Layer4Protocol = l4udp) then
                                   begin
                                        socketpair4.Source.Port := filter.PortFilter.SourcePort;
                                        socketpair4.Destination.Port := filter.PortFilter.DestinationPort;

                                        // find the tcp or udp conversation entry

                                        if (filter.PortFilter.Layer4Protocol = l4tcp) then
                                           begin
                                                tcpconv := GetTCPConversationEntry (socketpair4);

                                                // check if we got something back, and if so, merge interface lists

                                                if tcpconv.SocketPair.Source.IPAddress.IPVersion = 4 then
                                                   maplist := mergeInterfaceLists (maplist, tcpconv.InterfaceList)
                                                else
                                                    SiMain.LogVerbose ('No TCP conversation found for Socket "' + socketpair4.ValueAsString(false) + '" in tConversationStorage.GetInterfaceMapListForFilterSet');
                                           end
                                        else
                                            begin
                                                 udpconv := GetUDPConversationEntry (socketpair4);

                                                 if udpconv.SocketPair.Source.IPAddress.IPVersion = 4 then
                                                    maplist := mergeInterfaceLists (maplist, udpconv.InterfaceList)
                                                 else
                                                     SiMain.LogVerbose ('No UDP conversation found for Socket "' + socketpair4.ValueAsString(false) + '" in tConversationStorage.GetInterfaceMapListForFilterSet');
                                            end;
                                   end
                                else
                                    begin
                                         // no ports available, so we filter on IP only instead

                                         addrpair4.Source := filter.IPv4Filter.SourceAddress;
                                         addrpair4.Destination := filter.IPv4Filter.DestinationAddress;

                                         ipv4conv := GetIPv4ConversationEntry (addrpair4);

                                         maplist := mergeInterfaceLists (maplist, ipv4conv.InterfaceList);
                                    end;
                           end;

                        if filter.IPv6Filter.AddressFilterType <> ftIgnore then
                           begin
                                socketpair6.Source.IPAddress := filter.IPv6Filter.SourceAddress;
                                socketpair6.Destination.IPAddress := filter.IPv6Filter.DestinationAddress;

                                if (filter.PortFilter.Layer4Protocol = l4tcp) or (filter.PortFilter.Layer4Protocol = l4udp) then
                                   begin
                                        socketpair6.Source.Port := filter.PortFilter.SourcePort;
                                        socketpair6.Destination.Port := filter.PortFilter.DestinationPort;

                                        // find the tcp or udp conversation entry

                                        if (filter.PortFilter.Layer4Protocol = l4tcp) then
                                           begin
                                                tcpconv := GetTCPConversationEntry (socketpair6);

                                                if tcpconv.SocketPair.Source.IPAddress.IPVersion = 6 then
                                                   maplist := mergeInterfaceLists (maplist, tcpconv.InterfaceList)
                                                else
                                                    SiMain.LogVerbose ('No TCP conversation found for Socket "' + socketpair4.ValueAsString(false) + '" in tConversationStorage.GetInterfaceMapListForFilterSet');
                                           end
                                        else
                                            begin
                                                 udpconv := GetUDPConversationEntry (socketpair6);

                                                 if udpconv.SocketPair.Source.IPAddress.IPVersion = 6 then
                                                    maplist := mergeInterfaceLists (maplist, udpconv.InterfaceList)
                                                 else
                                                     SiMain.LogVerbose ('No UDP conversation found for Socket "' + socketpair4.ValueAsString(false) + '" in tConversationStorage.GetInterfaceMapListForFilterSet');
                                            end;
                                   end
                                else
                                    begin
                                         // no ports available, so we filter on IP only instead

                                         addrpair6.Source := filter.IPv6Filter.SourceAddress;
                                         addrpair6.Destination := filter.IPv6Filter.DestinationAddress;

                                         ipv6conv := GetIPv6ConversationEntry (addrpair6);

                                         maplist := mergeInterfaceLists (maplist, ipv6conv.InterfaceList);
                                    end;
                           end;
                   end;

                filter := FilterSet.getNext;
           end;

     result := maplist;

     SiMain.LeaveMethod ('tConversationStorage.GetInterfaceMapListForFilterSet');
end;

(* ------------------------------------------- *)

procedure tConversationStorage.InitFQDNManager (var FQDNManager: tFQDNManager);

var IgnoreCase : boolean;
    NREntry    : tNameResolutionEntry;
    FQDNEntry  : tFQDNDetailNode;
    key        : string;

begin
     SiMain.EnterMethod ('tConversationStorage.InitFQDNManager');

     // Clear the FQDNManager first by destroying and creating a new one

     IgnoreCase := FQDNManager.IgnoreCase;

     FQDNManager.Free;
     FQDNManager := tFQDNManager.create (IgnoreCase);

     // Initialize it from NameResolutions, which are one entry each for FQDN + Type + Answer

     for key in priv_dictNameResolutions.Keys do
         begin
              NREntry := priv_dictNameResolutions.Items[key];

              // Add the Entry, or get the existing entry back

              if FQDNManager.FQDNExists (NREntry.FQDN) then
                 FQDNEntry := FQDNManager.GetFDQNDetailNode (NREntry.FQDN)
              else
                  FQDNEntry := FQDNManager.AddFQDN (NREntry.FQDN);

              // Add the answer record

              if assigned (FQDNEntry) and not NREntry.QueryOnly then
                 FQDNEntry.AddAnswer (NREntry.Answer);
         end;

     SiMain.LeaveMethod ('tConversationStorage.InitFQDNManager');
end;

(* ------------------------------------------- *)

procedure tConversationStorage.AddConversationsFromStore (Store: tConversationStorage; const FileID: integer = 0);

var key                 : string;
    MACEndpoint         : tMACEndpoint;
    MACConversation     : tMACConversation;
    IPv4Endpoint        : tIPv4Endpoint;
    IPv4Conversation    : tIPv4Conversation;
    IPv6Endpoint        : tIPv6Endpoint;
    IPv6Conversation    : tIPv6Conversation;
    TCPConversation     : tTCPConversation;
    UDPConversation     : tUDPConversation;
    NameResolution      : tNameResolutionEntry;

begin
     SiMain.EnterMethod ('tConversationStorage.AddConversationsFromStore');

     for key in Store.priv_dictMACEndpoints.Keys do
         begin
              MACEndpoint := Store.priv_dictMACEndpoints.Items[key];

              if FileID <> 0 then
                 begin
                      MACEndpoint.FirstFileID := FileID;
                      MACEndpoint.LastFileID := FileID;

                      ForceFileIDOnInterfaceList (MACEndpoint.InterfaceList, FileID);
                 end;

              AddMACEndpoint (MACEndpoint);
         end;

     for key in Store.priv_dictMACConversations.Keys do
         begin
              MACConversation := Store.priv_dictMACConversations.Items[key];

              if FileID <> 0 then
                 begin
                      MACConversation.FirstFileID := FileID;
                      MACConversation.LastFileID := FileID;

                      ForceFileIDOnInterfaceList (MACConversation.InterfaceList, FileID);
                 end;

              AddMACConversation (MACConversation);
         end;

     for key in Store.priv_dictIPv4Endpoints.Keys do
         begin
              IPv4Endpoint := Store.priv_dictIPv4Endpoints.Items[key];

              if FileID <> 0 then
                 begin
                      IPv4Endpoint.FirstFileID := FileID;
                      IPv4Endpoint.LastFileID := FileID;

                      ForceFileIDOnInterfaceList (IPv4Endpoint.InterfaceList, FileID);
                 end;

              AddIPv4Endpoint (IPv4Endpoint);
         end;

     for key in Store.priv_dictIPv4Conversations.Keys do
         begin
              IPv4Conversation := Store.priv_dictIPv4Conversations.Items[key];

              if FileID <> 0 then
                 begin
                      IPv4Conversation.FirstFileID := FileID;
                      IPv4Conversation.LastFileID := FileID;

                      ForceFileIDOnInterfaceList (IPv4Conversation.InterfaceList, FileID);
                 end;

              AddIPv4Conversation (IPv4Conversation);
         end;

     for key in Store.priv_dictIPv6Endpoints.Keys do
         begin
              IPv6Endpoint := Store.priv_dictIPv6Endpoints.Items[key];

              if FileID <> 0 then
                 begin
                      IPv6Endpoint.FirstFileID := FileID;
                      IPv6Endpoint.LastFileID := FileID;

                      ForceFileIDOnInterfaceList (IPv6Endpoint.InterfaceList, FileID);
                 end;

              AddIPv6Endpoint (IPv6Endpoint);
         end;

     for key in Store.priv_dictIPv6Conversations.Keys do
         begin
              IPv6Conversation := Store.priv_dictIPv6Conversations.Items[key];

              if FileID <> 0 then
                 begin
                      IPv6Conversation.FirstFileID := FileID;
                      IPv6Conversation.LastFileID := FileID;

                      ForceFileIDOnInterfaceList (IPv6Conversation.InterfaceList, FileID);
                 end;

              AddIPv6Conversation (IPv6Conversation);
         end;

     for key in Store.priv_dictTCPConversations.Keys do
         begin
              TCPConversation := Store.priv_dictTCPConversations.Items[key];

              if FileID <> 0 then
                 begin
                      TCPConversation.FirstFileID := FileID;
                      TCPConversation.LastFileID := FileID;

                      ForceFileIDOnInterfaceList (TCPConversation.InterfaceList, FileID);
                 end;

              AddTCPConversation (TCPConversation);
         end;

     for key in Store.priv_dictUDPConversations.Keys do
         begin
              UDPConversation := Store.priv_dictUDPConversations.Items[key];

              if FileID <> 0 then
                 begin
                      UDPConversation.FirstFileID := FileID;
                      UDPConversation.LastFileID := FileID;

                      ForceFileIDOnInterfaceList (UDPConversation.InterfaceList, FileID);
                 end;

              AddUDPConversation (UDPConversation);
         end;

     for key in Store.priv_dictNameResolutions.Keys do
         begin
              NameResolution := Store.priv_dictNameResolutions.Items[key];

              if FileID <> 0 then
                 begin
                      NameResolution.FirstFileID := FileID;
                      NameResolution.LastFileID := FileID;

                      ForceFileIDOnInterfaceList (NameResolution.InterfaceList, FileID);
                 end;

              AddNameResolution (NameResolution);
         end;

     // Clear the dictionaries for first and last frame as they are now invalid/incomplete

     priv_dictFirstFrames.Clear;
     priv_dictLastFrames.Clear;
     priv_FirstLastSources := [];

     SiMain.LeaveMethod ('tConversationStorage.AddConversationsFromStore');
end;

(* ------------------------------------------- *)

procedure tConversationStorage.UpdateCalculatedValues;

var key                 : string;
    conversationkey     : string;
    endpointkey         : string;
    socketkey           : string;
    MACEndpoint         : tMACEndpoint;
    MACConversation     : tMACConversation;
    IPv4Endpoint        : tIPv4Endpoint;
    IPv4Conversation    : tIPv4Conversation;
    IPv6Endpoint        : tIPv6Endpoint;
    IPv6Conversation    : tIPv6Conversation;
    TCPConversation     : tTCPConversation;
    UDPConversation     : tUDPConversation;
    IPSocketDetails     : tSocketDetails;

begin
     SiMain.EnterMethod ('tConversationStorage.UpdateCalculatedValues');

     // Loop through all TCP conversations and count flows as well as gathering RTT info

     for key in priv_dictTCPConversations.Keys do
         begin
              TCPConversation := priv_dictTCPConversations.Items[key];

              if TCPConversation.SocketPair.Source.IPAddress.IPVersion = 4 then
                 begin
                      // Get the matching IPv4 conversation entry. To build the dictionary key, we use the default method
                      // to avoid problems if it ever changes. So build a dummy first, then get the real one with it.

                      IPv4Conversation.IPv4AddressPair.Source := TCPConversation.SocketPair.Source.IPAddress.IPv4Address;
                      IPv4Conversation.IPv4AddressPair.Destination := TCPConversation.SocketPair.Destination.IPAddress.IPv4Address;
                      conversationkey := IPv4Conversation.DictionaryKey;

                      IPv4Conversation := priv_dictIPv4Conversations.Items[conversationkey];

                      // Increase general flow count, first for the IPv4 conversation, then for both IPv4 endpoints

                      IPv4Conversation.FlowCount := IPv4Conversation.FlowCount + 1;

                      // Source IP

                      IPv4Endpoint.IPv4Address := IPv4Conversation.IPv4AddressPair.Source;
                      endpointkey := IPv4Endpoint.DictionaryKey;

                      IPv4Endpoint := priv_dictIPv4Endpoints.Items[endpointkey];
                      IPv4Endpoint.FlowCount := IPv4Endpoint.FlowCount + 1;

                      // Check if this IP was sending SYNs

                      if IPv4Endpoint.IPv4Address = TCPConversation.ClientIP.IPv4Address then
                         inc (IPv4Endpoint.ClientCount);

                      // write Endpoint back

                      priv_dictIPv4Endpoints.Items[endpointkey] := IPv4Endpoint;

                      // Destination IP

                      IPv4Endpoint.IPv4Address := IPv4Conversation.IPv4AddressPair.Destination;
                      endpointkey := IPv4Endpoint.DictionaryKey;

                      IPv4Endpoint := priv_dictIPv4Endpoints.Items[endpointkey];
                      IPv4Endpoint.FlowCount := IPv4Endpoint.FlowCount + 1;

                      // Check if this IP was sending SYNs

                      if IPv4Endpoint.IPv4Address = TCPConversation.ClientIP.IPv4Address then
                         inc (IPv4Endpoint.ClientCount);

                      // write Endpoint back

                      priv_dictIPv4Endpoints.Items[endpointkey] := IPv4Endpoint;

                      // Calculate RTT stuff

                      if (TCPConversation.InitialRTTns <> 0) then
                         begin
                              if (IPv4Conversation.MinRTTns = 0) or (TCPConversation.InitialRTTns < IPv4Conversation.MinRTTns) then
                                 IPv4Conversation.MinRTTns := TCPConversation.InitialRTTns;

                              if (IPv4Conversation.MaxRTTns = 0) or (TCPConversation.InitialRTTns > IPv4Conversation.MaxRTTns) then
                                 IPv4Conversation.MaxRTTns := TCPConversation.InitialRTTns;

                              // Add to total

                              IPv4Conversation.AggregatedRTTns := IPv4Conversation.AggregatedRTTns + TCPConversation.InitialRTTns;
                              IPv4Conversation.RTTSampleCount := IPv4Conversation.RTTSampleCount + 1;
                         end;

                      // Write IPv4 conversation entry back to storage

                      priv_dictIPv4Conversations.Items[conversationkey] := IPv4Conversation;
                 end
              else
                  begin
                       // Get the matching IPv4 conversation entry. To build the dictionary key, we use the default method
                       // to avoid problems if it ever changes. So build a dummy first, then get the real one with it.

                       IPv6Conversation.IPv6AddressPair.Source := TCPConversation.SocketPair.Source.IPAddress.IPv6Address;
                       IPv6Conversation.IPv6AddressPair.Destination := TCPConversation.SocketPair.Destination.IPAddress.IPv6Address;
                       conversationkey := IPv6Conversation.DictionaryKey;

                       IPv6Conversation := priv_dictIPv6Conversations.Items[conversationkey];

                       // Increase general flow count, first for the IPv4 conversation, then for both IPv4 endpoints

                       IPv6Conversation.FlowCount := IPv6Conversation.FlowCount + 1;

                       // Source IP

                       IPv6Endpoint.IPv6Address := IPv6Conversation.IPv6AddressPair.Source;
                       endpointkey := IPv6Endpoint.DictionaryKey;

                       IPv6Endpoint := priv_dictIPv6Endpoints.Items[endpointkey];
                       IPv6Endpoint.FlowCount := IPv6Endpoint.FlowCount + 1;

                      // Check if this IP was sending SYNs

                      if IPv6Endpoint.IPv6Address = TCPConversation.ClientIP.IPv6Address then
                         inc (IPv6Endpoint.ClientCount);

                      // write endpoint back

                      priv_dictIPv6Endpoints.Items[endpointkey] := IPv6Endpoint;

                       // Destination IP

                       IPv6Endpoint.IPv6Address := IPv6Conversation.IPv6AddressPair.Destination;
                       endpointkey := IPv6Endpoint.DictionaryKey;

                       IPv6Endpoint := priv_dictIPv6Endpoints.Items[endpointkey];
                       IPv6Endpoint.FlowCount := IPv6Endpoint.FlowCount + 1;

                      // Check if this IP was sending SYNs

                      if IPv6Endpoint.IPv6Address = TCPConversation.ClientIP.IPv6Address then
                         inc (IPv6Endpoint.ClientCount);

                      // write endpoint back

                      priv_dictIPv6Endpoints.Items[endpointkey] := IPv6Endpoint;

                       // Calculate RTT stuff

                       if (TCPConversation.InitialRTTns <> 0) then
                          begin
                               if (IPv6Conversation.MinRTTns = 0) or (TCPConversation.InitialRTTns < IPv6Conversation.MinRTTns) then
                                  IPv6Conversation.MinRTTns := TCPConversation.InitialRTTns;

                               if (IPv6Conversation.MaxRTTns = 0) or (TCPConversation.InitialRTTns > IPv6Conversation.MaxRTTns) then
                                  IPv6Conversation.MaxRTTns := TCPConversation.InitialRTTns;

                               // Add to total

                               IPv6Conversation.AggregatedRTTns := IPv6Conversation.AggregatedRTTns + TCPConversation.InitialRTTns;
                               IPv6Conversation.RTTSampleCount := IPv6Conversation.RTTSampleCount + 1;
                          end;

                       // Write IPv6 conversation entry back to storage

                       priv_dictIPv6Conversations.Items[conversationkey] := IPv6Conversation;
                  end;

              // Update TCP socket list

              // Source sockets first

              socketkey := IntToStr (protTCP) + '-' + TCPConversation.SocketPair.Source.DictionaryKey;

              if not priv_dictTCPSockets.ContainsKey (socketkey) then
                 begin
                      IPSocketDetails.IPSocket := TCPConversation.SocketPair.Source;
                      IPSocketDetails.UseCount := 1;
                      IPSocketDetails.NumPacketsIn := TCPConversation.FramesDstToSrc;
                      IPSocketDetails.NumPacketsOut := TCPConversation.FramesSrcToDst;
                      IPSocketDetails.Firstseen := TCPConversation.FirstFrameTime;
                      IPSocketDetails.LastSeen := TCPCOnversation.LastFrameTime;

                      priv_dictTCPSockets.Add (socketkey, IPSocketDetails);
                 end
              else
                  begin
                       IPSocketDetails := priv_dictTCPSockets.Items [socketkey];

                       IPSocketDetails.UseCount := IPSocketDetails.UseCount + 1;
                       IPSocketDetails.NumPacketsIn := IPSocketDetails.NumPacketsIn + TCPConversation.FramesDstToSrc;
                       IPSocketDetails.NumPacketsOut := IPSocketDetails.NumPacketsOut + TCPConversation.FramesSrcToDst;

                       if TCPConversation.FirstFrameTime < IPSocketDetails.Firstseen then
                          IPSocketDetails.Firstseen := TCPConversation.FirstFrameTime;

                       if TCPCOnversation.LastFrameTime > IPSocketDetails.LastSeen then
                          IPSocketDetails.LastSeen := TCPCOnversation.LastFrameTime;

                       priv_dictTCPSockets.Items [socketkey] := IPSocketDetails;
                  end;

              // Destination sockets second

              socketkey := IntToStr (protTCP) + '-' + TCPConversation.SocketPair.Destination.DictionaryKey;

              if not priv_dictTCPSockets.ContainsKey (socketkey) then
                 begin
                      IPSocketDetails.IPSocket := TCPConversation.SocketPair.Destination;
                      IPSocketDetails.UseCount := 1;
                      IPSocketDetails.NumPacketsIn := TCPConversation.FramesDstToSrc;
                      IPSocketDetails.NumPacketsOut := TCPConversation.FramesSrcToDst;
                      IPSocketDetails.Firstseen := TCPConversation.FirstFrameTime;
                      IPSocketDetails.LastSeen := TCPCOnversation.LastFrameTime;

                      priv_dictTCPSockets.Add (socketkey, IPSocketDetails);
                 end
              else
                  begin
                       IPSocketDetails := priv_dictTCPSockets.Items [socketkey];

                       IPSocketDetails.UseCount := IPSocketDetails.UseCount + 1;
                       IPSocketDetails.NumPacketsIn := IPSocketDetails.NumPacketsIn + TCPConversation.FramesDstToSrc;
                       IPSocketDetails.NumPacketsOut := IPSocketDetails.NumPacketsOut + TCPConversation.FramesSrcToDst;

                       if TCPConversation.FirstFrameTime < IPSocketDetails.Firstseen then
                          IPSocketDetails.Firstseen := TCPConversation.FirstFrameTime;

                       if TCPCOnversation.LastFrameTime > IPSocketDetails.LastSeen then
                          IPSocketDetails.LastSeen := TCPCOnversation.LastFrameTime;

                       priv_dictTCPSockets.Items [socketkey] := IPSocketDetails;
                  end;
         end;

     // Loop through all UDP conversations

     for key in priv_dictUDPConversations.Keys do
         begin
              UDPConversation := priv_dictUDPConversations.Items[key];

              // Update TCP socket list

              // Source sockets first

              socketkey := IntToStr (protTCP) + '-' + UDPConversation.SocketPair.Source.DictionaryKey;

              if not priv_dictUDPSockets.ContainsKey (socketkey) then
                 begin
                      IPSocketDetails.IPSocket := UDPConversation.SocketPair.Source;
                      IPSocketDetails.UseCount := 1;
                      IPSocketDetails.NumPacketsIn := UDPConversation.FramesDstToSrc;
                      IPSocketDetails.NumPacketsOut := UDPConversation.FramesSrcToDst;
                      IPSocketDetails.Firstseen := UDPConversation.FirstFrameTime;
                      IPSocketDetails.LastSeen := UDPConversation.LastFrameTime;

                      priv_dictUDPSockets.Add (socketkey, IPSocketDetails);
                 end
              else
                  begin
                       IPSocketDetails := priv_dictUDPSockets.Items [socketkey];

                       IPSocketDetails.UseCount := IPSocketDetails.UseCount + 1;
                       IPSocketDetails.NumPacketsIn := IPSocketDetails.NumPacketsIn + UDPConversation.FramesDstToSrc;
                       IPSocketDetails.NumPacketsOut := IPSocketDetails.NumPacketsOut + UDPConversation.FramesSrcToDst;

                       if UDPConversation.FirstFrameTime < IPSocketDetails.Firstseen then
                          IPSocketDetails.Firstseen := UDPConversation.FirstFrameTime;

                       if UDPConversation.LastFrameTime > IPSocketDetails.LastSeen then
                          IPSocketDetails.LastSeen := UDPConversation.LastFrameTime;

                       priv_dictUDPSockets.Items [socketkey] := IPSocketDetails;
                  end;

              // Destination sockets second

              socketkey := IntToStr (protTCP) + '-' + UDPConversation.SocketPair.Destination.DictionaryKey;

              if not priv_dictUDPSockets.ContainsKey (socketkey) then
                 begin
                      IPSocketDetails.IPSocket := UDPConversation.SocketPair.Destination;
                      IPSocketDetails.UseCount := 1;
                      IPSocketDetails.NumPacketsIn := UDPConversation.FramesDstToSrc;
                      IPSocketDetails.NumPacketsOut := UDPConversation.FramesSrcToDst;
                      IPSocketDetails.Firstseen := UDPConversation.FirstFrameTime;
                      IPSocketDetails.LastSeen := UDPConversation.LastFrameTime;

                      priv_dictUDPSockets.Add (socketkey, IPSocketDetails);
                 end
              else
                  begin
                       IPSocketDetails := priv_dictUDPSockets.Items [socketkey];

                       IPSocketDetails.UseCount := IPSocketDetails.UseCount + 1;
                       IPSocketDetails.NumPacketsIn := IPSocketDetails.NumPacketsIn + UDPConversation.FramesDstToSrc;
                       IPSocketDetails.NumPacketsOut := IPSocketDetails.NumPacketsOut + UDPConversation.FramesSrcToDst;

                       if UDPConversation.FirstFrameTime < IPSocketDetails.Firstseen then
                          IPSocketDetails.Firstseen := UDPConversation.FirstFrameTime;

                       if UDPConversation.LastFrameTime > IPSocketDetails.LastSeen then
                          IPSocketDetails.LastSeen := UDPConversation.LastFrameTime;

                       priv_dictUDPSockets.Items [socketkey] := IPSocketDetails;
                  end;
         end;

     // Loop through IPv4 conversations to update partner counter for endpoints

     for key in priv_dictIPv4Conversations.Keys do
         begin
              IPv4Conversation := priv_dictIPv4Conversations.Items[key];

              IPv4Endpoint.IPv4Address := IPv4Conversation.IPv4AddressPair.Source;
              endpointkey := IPv4Endpoint.DictionaryKey;

              IPv4Endpoint := priv_dictIPv4Endpoints.Items[endpointkey];
              IPv4Endpoint.PartnerCount := IPv4Endpoint.PartnerCount + 1;
              priv_dictIPv4Endpoints.Items[endpointkey] := IPv4Endpoint;

              IPv4Endpoint.IPv4Address := IPv4Conversation.IPv4AddressPair.Destination;
              endpointkey := IPv4Endpoint.DictionaryKey;

              IPv4Endpoint := priv_dictIPv4Endpoints.Items[endpointkey];
              IPv4Endpoint.PartnerCount := IPv4Endpoint.PartnerCount + 1;
              priv_dictIPv4Endpoints.Items[endpointkey] := IPv4Endpoint;
         end;

     // Loop through IPv4 conversations to update partner counter for endpoints

     for key in priv_dictIPv6Conversations.Keys do
         begin
              IPv6Conversation := priv_dictIPv6Conversations.Items[key];

              IPv6Endpoint.IPv6Address := IPv6Conversation.IPv6AddressPair.Source;
              endpointkey := IPv6Endpoint.DictionaryKey;

              IPv6Endpoint := priv_dictIPv6Endpoints.Items[endpointkey];
              IPv6Endpoint.PartnerCount := IPv6Endpoint.PartnerCount + 1;
              priv_dictIPv6Endpoints.Items[endpointkey] := IPv6Endpoint;

              IPv6Endpoint.IPv6Address := IPv6Conversation.IPv6AddressPair.Destination;
              endpointkey := IPv6Endpoint.DictionaryKey;

              IPv6Endpoint := priv_dictIPv6Endpoints.Items[endpointkey];
              IPv6Endpoint.PartnerCount := IPv6Endpoint.PartnerCount + 1;
              priv_dictIPv6Endpoints.Items[endpointkey] := IPv6Endpoint;
         end;

     SiMain.LeaveMethod ('tConversationStorage.UpdateCalculatedValues');
end;

(* ------------------------------------------- *)

procedure tConversationStorage.UpdateFirstAndLastDictionaries (const WriteMode : tMultiWriteMode = mwmNone; const ForceClean : boolean = false);

var key                 : string;
    filepos             : tFileFrameLocation;
    fileposkey          : string;
    MACEndpoint         : tMACEndpoint;
    MACConversation     : tMACConversation;
    IPv4Endpoint        : tIPv4Endpoint;
    IPv4Conversation    : tIPv4Conversation;
    IPv6Endpoint        : tIPv6Endpoint;
    IPv6Conversation    : tIPv6Conversation;
    TCPConversation     : tTCPConversation;
    UDPConversation     : tUDPConversation;
    NameResolution      : tNameResolutionEntry;

begin
     SiMain.EnterMethod ('tConversationStorage.UpdateFirstAndLastDictionaries');

     // If necessary, remove all entries first (usually this happens when running a task)

     if ForceClean then
        begin
             priv_dictFirstFrames.Clear;
             priv_dictLastFrames.Clear;

             priv_FirstLastSources := [];
        end;

     // Add markers for frames which are the last of their kind

     if ((WriteMode = mwmEtherEndpoint) or (WriteMode = mwmNone)) and not (csrMACEndpoint in priv_FirstLastSources) then
        begin
             if priv_dictMACEndpoints.Count > 0 then
                begin
                     for key in priv_dictMACEndpoints.Keys do
                         begin
                              MacEndPoint := priv_dictMACEndpoints.Items[key];

                              filepos.initialize (MACEndPoint.FirstFileID, MACEndPoint.FirstFrameIndex, MACEndPoint.FirstFrameOffset);
                              fileposkey := filepos.DictionaryKey;

                              if priv_dictFirstFrames.ContainsKey (fileposkey) then
                                 priv_dictFirstFrames.Items[fileposkey] := priv_dictFirstFrames.Items[fileposkey] + [csrMACEndpoint]
                              else
                                  priv_dictFirstFrames.Add (fileposkey, [csrMACEndpoint]);

                              filepos.initialize (MACEndPoint.LastFileID, MACEndPoint.LastFrameIndex, MACEndPoint.LastFrameOffset);
                              fileposkey := filepos.DictionaryKey;

                              if priv_dictLastFrames.ContainsKey (fileposkey) then
                                 priv_dictLastFrames.Items[fileposkey] := priv_dictLastFrames.Items[fileposkey] + [csrMACEndpoint]
                              else
                                  priv_dictLastFrames.Add (fileposkey, [csrMACEndpoint]);
                         end;

                         // Note that we have added those to the dictionary

                         priv_FirstLastSources := priv_FirstLastSources + [csrMACEndpoint];
                end;
        end;

     if ((WriteMode = mwmIPEndpoint) or (WriteMode = mwmNone)) and not (csrIPv4Endpoint in priv_FirstLastSources) then
        begin
             if priv_dictIPv4Endpoints.Count > 0 then
                begin
                     for key in priv_dictIPv4Endpoints.Keys do
                         begin
                              IPv4EndPoint := priv_dictIPv4Endpoints.Items[key];

                              filepos.initialize (IPv4EndPoint.FirstFileID, IPv4EndPoint.FirstFrame, IPv4EndPoint.FirstFrameOffset);
                              fileposkey := filepos.DictionaryKey;

                              if priv_dictFirstFrames.ContainsKey (fileposkey) then
                                 priv_dictFirstFrames.Items[fileposkey] := priv_dictFirstFrames.Items[fileposkey] + [csrIPv4Endpoint]
                              else
                                  priv_dictFirstFrames.Add (fileposkey, [csrIPv4Endpoint]);

                              filepos.initialize (IPv4EndPoint.LastFileID, IPv4EndPoint.LastFrame, IPv4EndPoint.LastFrameOffset);
                              fileposkey := filepos.DictionaryKey;

                              if priv_dictLastFrames.ContainsKey (fileposkey) then
                                 priv_dictLastFrames.Items[fileposkey] := priv_dictLastFrames.Items[fileposkey] + [csrIPv4Endpoint]
                              else
                                  priv_dictLastFrames.Add (fileposkey, [csrIPv4Endpoint]);
                         end;

                     // Note that we have added those to the dictionary

                     priv_FirstLastSources := priv_FirstLastSources + [csrIPv4Endpoint];
                end;
        end;

     if ((WriteMode = mwmIPEndpoint) or (WriteMode = mwmNone)) and not (csrIPv6Endpoint in priv_FirstLastSources) then
        begin
             if priv_dictIPv6Endpoints.Count > 0 then
                begin
                     for key in priv_dictIPv6Endpoints.Keys do
                         begin
                              IPv6EndPoint := priv_dictIPv6Endpoints.Items[key];

                              filepos.initialize (IPv6EndPoint.FirstFileID, IPv6EndPoint.LastFrame, IPv6EndPoint.LastFrameOffset);
                              fileposkey := filepos.DictionaryKey;

                              if priv_dictFirstFrames.ContainsKey (fileposkey) then
                                 priv_dictFirstFrames.Items[fileposkey] := priv_dictFirstFrames.Items[fileposkey] + [csrIPv6Endpoint]
                              else
                                  priv_dictFirstFrames.Add (fileposkey, [csrIPv6Endpoint]);

                              filepos.initialize (IPv6EndPoint.LastFileID, IPv6EndPoint.LastFrame, IPv6EndPoint.LastFrameOffset);
                              fileposkey := filepos.DictionaryKey;

                              if priv_dictLastFrames.ContainsKey (fileposkey) then
                                 priv_dictLastFrames.Items[fileposkey] := priv_dictLastFrames.Items[fileposkey] + [csrIPv6Endpoint]
                              else
                                  priv_dictLastFrames.Add (fileposkey, [csrIPv6Endpoint]);
                         end;

                     // Note that we have added those to the dictionary

                     priv_FirstLastSources := priv_FirstLastSources + [csrIPv6Endpoint];
                end;
        end;

     if ((WriteMode = mwmEtherConversation) or (WriteMode = mwmNone)) and not (csrMACConversation in priv_FirstLastSources) then
        begin
             if priv_dictMACConversations.Count > 0 then
                begin
                     for key in priv_dictMACConversations.Keys do
                         begin
                              MACConversation := priv_dictMACConversations.Items[key];

                              filepos.initialize (MACConversation.FirstFileID, MACConversation.FirstFrame, MACConversation.FirstFrameOffset);
                              fileposkey := filepos.DictionaryKey;

                              if priv_dictFirstFrames.ContainsKey (fileposkey) then
                                 priv_dictFirstFrames.Items[fileposkey] := priv_dictFirstFrames.Items[fileposkey] + [csrMACConversation]
                              else
                                  priv_dictFirstFrames.Add (fileposkey, [csrMACConversation]);

                              filepos.initialize (MACConversation.LastFileID, MACConversation.LastFrame, MACConversation.LastFrameOffset);
                              fileposkey := filepos.DictionaryKey;

                              if priv_dictLastFrames.ContainsKey (fileposkey) then
                                 priv_dictLastFrames.Items[fileposkey] := priv_dictLastFrames.Items[fileposkey] + [csrMACConversation]
                              else
                                  priv_dictLastFrames.Add (fileposkey, [csrMACConversation]);
                         end;

                     // Note that we have added those to the dictionary

                     priv_FirstLastSources := priv_FirstLastSources + [csrMACConversation];
                end;
        end;

     if ((WriteMode = mwmIPConversation) or (WriteMode = mwmNone)) and not (csrIPv4Conversation in priv_FirstLastSources) then
        begin
             if priv_dictIPv4Conversations.Count > 0 then
                begin
                     for key in priv_dictIPv4Conversations.Keys do
                         begin
                              IPv4Conversation := priv_dictIPv4Conversations.Items[key];

                              filepos.initialize (IPv4Conversation.FirstFileID, IPv4Conversation.FirstFrame, IPv4Conversation.FirstFrameOffset);
                              fileposkey := filepos.DictionaryKey;

                              if priv_dictFirstFrames.ContainsKey (fileposkey) then
                                 priv_dictFirstFrames.Items[fileposkey] := priv_dictFirstFrames.Items[fileposkey] + [csrIPv4Conversation]
                              else
                                  priv_dictFirstFrames.Add (fileposkey, [csrIPv4Conversation]);

                              filepos.initialize (IPv4Conversation.LastFileID, IPv4Conversation.LastFrame, IPv4Conversation.LastFrameOffset);
                              fileposkey := filepos.DictionaryKey;

                              if priv_dictLastFrames.ContainsKey (fileposkey) then
                                 priv_dictLastFrames.Items[fileposkey] := priv_dictLastFrames.Items[fileposkey] + [csrIPv4Conversation]
                              else
                                  priv_dictLastFrames.Add (fileposkey, [csrIPv4Conversation]);
                         end;

                     // Note that we have added those to the dictionary

                     priv_FirstLastSources := priv_FirstLastSources + [csrIPv4Conversation];
                end;
        end;

     if ((WriteMode = mwmIPConversation) or (WriteMode = mwmNone)) and not (csrIPv6conversation in priv_FirstLastSources) then
        begin
             if priv_dictIPv6Conversations.Count > 0 then
                begin
                     for key in priv_dictIPv6Conversations.Keys do
                         begin
                              IPv6Conversation := priv_dictIPv6Conversations.Items[key];

                              filepos.initialize (IPv6Conversation.FirstFileID, IPv6Conversation.FirstFrame, IPv6Conversation.FirstFrameOffset);
                              fileposkey := filepos.DictionaryKey;

                              if priv_dictFirstFrames.ContainsKey (fileposkey) then
                                 priv_dictFirstFrames.Items[fileposkey] := priv_dictFirstFrames.Items[fileposkey] + [csrIPv6Conversation]
                              else
                                  priv_dictFirstFrames.Add (fileposkey, [csrIPv6Conversation]);

                              filepos.initialize (IPv6Conversation.LastFileID, IPv6Conversation.LastFrame, IPv6Conversation.LastFrameOffset);
                              fileposkey := filepos.DictionaryKey;

                              if priv_dictLastFrames.ContainsKey (fileposkey) then
                                 priv_dictLastFrames.Items[fileposkey] := priv_dictLastFrames.Items[fileposkey] + [csrIPv6Conversation]
                              else
                                  priv_dictLastFrames.Add (fileposkey, [csrIPv6Conversation]);
                         end;

                     // Note that we have added those to the dictionary

                     priv_FirstLastSources := priv_FirstLastSources + [csrIPv6Conversation];
                end;
        end;

     if ((WriteMode = mwmL4Conversation) or (WriteMode = mwmNone)) and not (csrTCPConversation in priv_FirstLastSources) then
        begin
             if priv_dictTCPConversations.Count > 0 then
                begin
                     for key in priv_dictTCPConversations.Keys do
                         begin
                              TCPConversation := priv_dictTCPConversations.Items[key];

                              filepos.initialize (TCPConversation.FirstFileID, TCPConversation.FirstFrame, TCPConversation.FirstFrameOffset);
                              fileposkey := filepos.DictionaryKey;

                              if priv_dictFirstFrames.ContainsKey (fileposkey) then
                                 priv_dictFirstFrames.Items[fileposkey] := priv_dictFirstFrames.Items[fileposkey] + [csrTCPConversation]
                              else
                                  priv_dictFirstFrames.Add (fileposkey, [csrTCPConversation]);

                              filepos.initialize (TCPConversation.LastFileID, TCPConversation.LastFrame, TCPConversation.LastFrameOffset);
                              fileposkey := filepos.DictionaryKey;

                              if priv_dictLastFrames.ContainsKey (fileposkey) then
                                 priv_dictLastFrames.Items[fileposkey] := priv_dictLastFrames.Items[fileposkey] + [csrTCPConversation]
                              else
                                  priv_dictLastFrames.Add (fileposkey, [csrTCPConversation]);
                         end;

                     // Note that we have added those to the dictionary

                     priv_FirstLastSources := priv_FirstLastSources + [csrTCPConversation];
                end;
        end;

     if ((WriteMode = mwmL4Conversation) or (WriteMode = mwmNone)) and not (csrUDPConversation in priv_FirstLastSources) then
        begin
             if priv_dictUDPConversations.count > 0 then
                begin
                     for key in priv_dictUDPConversations.Keys do
                         begin
                              UDPConversation := priv_dictUDPConversations.Items[key];

                              filepos.initialize (UDPConversation.FirstFileID, UDPConversation.FirstFrame, UDPConversation.FirstFrameOffset);
                              fileposkey := filepos.DictionaryKey;

                              if priv_dictFirstFrames.ContainsKey (fileposkey) then
                                 priv_dictFirstFrames.Items[fileposkey] := priv_dictFirstFrames.Items[fileposkey] + [csrUDPConversation]
                              else
                                  priv_dictFirstFrames.Add (fileposkey, [csrUDPConversation]);

                              filepos.initialize (UDPConversation.LastFileID, UDPConversation.LastFrame, UDPConversation.LastFrameOffset);
                              fileposkey := filepos.DictionaryKey;

                              if priv_dictLastFrames.ContainsKey (fileposkey) then
                                 priv_dictLastFrames.Items[fileposkey] := priv_dictLastFrames.Items[fileposkey] + [csrUDPConversation]
                              else
                                  priv_dictLastFrames.Add (fileposkey, [csrUDPConversation]);
                         end;

                     // Note that we have added those to the dictionary

                     priv_FirstLastSources := priv_FirstLastSources + [csrUDPConversation];
                end;
        end;

     SiMain.LeaveMethod ('tConversationStorage.UpdateFirstAndLastDictionaries');
end;

(* ------------------------------------------- *)

procedure tConversationStorage.UpdateFilterHelpers (var Filter: tFrameFilter);

var MACEndp   : tMACEndpoint;
    IPv4Endp  : tIPv4Endpoint;
    IPv6Endp  : tIPv6Endpoint;
    MACConv   : tMACConversation;
    IPv4Conv  : tIPv4Conversation;
    IPv6Conv  : tIPv6Conversation;
    TCPConv   : tTCPConversation;
    UDPConv   : tUDPConversation;
    MACPair   : tMACAddressPair;
    IPv4Pair  : tIPv4AddressPair;
    IPv6Pair  : tIPv6AddressPair;
    IPv4Sock  : tIPv4SocketPair;
    IPv6Sock  : tIPv6SocketPair;
    IPVersion : byte;

begin
     if Filter.HelpRecType = csrNone then
        Filter.updateRecType;

     case filter.HelpRecType of
      csrMACEndpoint       : begin
                                  MACEndp := GetMACEndpointEntry (Filter.MACFilter.SingleAddress);

                                  if not (MACEndp.MACAddress = VALUE_ZERO_MAC) then
                                     begin
                                          Filter.HelpFirstSeen.initialize (MACEndp.FirstFileID, MACEndp.FirstFrameIndex, MACEndp.FirstFrameOffset);
                                          Filter.HelpLastSeen.initialize (MACEndp.LastFileID, MACEndp.LastFrameIndex, MACEndp.LastFrameOffset);
                                     end
                                  else
                                      begin
                                           Filter.HelpFirstSeen.initialize;
                                           Filter.HelpLastSeen.initialize;
                                      end;
                             end;
      csrIPv4Endpoint      : begin
                                  IPv4Endp := GetIPv4EndpointEntry (Filter.IPv4Filter.SingleAddress);

                                  if not IPv4Endp.IPv4Address.isZeroAddress then
                                     begin
                                          Filter.HelpFirstSeen.initialize (IPv4Endp.FirstFileID, IPv4Endp.FirstFrame, IPv4Endp.FirstFrameOffset);
                                          Filter.HelpLastSeen.initialize (IPv4Endp.LastFileID, IPv4Endp.LastFrame, IPv4Endp.LastFrameOffset);
                                     end
                                  else
                                      begin
                                           Filter.HelpFirstSeen.initialize;
                                           Filter.HelpLastSeen.initialize;
                                      end;
                             end;
      csrIPv6Endpoint      : begin
                                  IPv6Endp := GetIPv6EndpointEntry (Filter.IPv6Filter.SingleAddress);

                                  if not IPv6Endp.IPv6Address.IsUnspecifiedAddress then
                                     begin
                                          Filter.HelpFirstSeen.initialize (IPv6Endp.FirstFileID, IPv6Endp.FirstFrame, IPv6Endp.FirstFrameOffset);
                                          Filter.HelpLastSeen.initialize (IPv6Endp.LastFileID, IPv6Endp.LastFrame, IPv6Endp.LastFrameOffset);
                                     end
                                  else
                                      begin
                                           Filter.HelpFirstSeen.initialize;
                                           Filter.HelpLastSeen.initialize;
                                      end;
                             end;
      csrMACConversation   : begin
                                  MACPair.initialize;
                                  MACPair.Source := filter.MACFilter.SourceAddress;
                                  MACPair.Destination := filter.MACFilter.DestinationAddress;
                                  MACPair.normalize;

                                  MACConv := GetMACConversationEntry (MACPair);

                                  if not (MACConv.MACPair.Source = VALUE_ZERO_MAC) then
                                     begin
                                          Filter.HelpFirstSeen.initialize (MACConv.FirstFileID, MACConv.FirstFrame, MACConv.FirstFrameOffset);
                                          Filter.HelpLastSeen.initialize (MACConv.LastFileID, MACConv.LastFrame, MACConv.LastFrameOffset);
                                     end
                                  else
                                      begin
                                           Filter.HelpFirstSeen.initialize;
                                           Filter.HelpLastSeen.initialize;
                                      end;
                             end;
      csrIPv4Conversation  : begin
                                  IPv4Pair.initialize;
                                  IPv4Pair.Source := filter.IPv4Filter.SourceAddress;
                                  IPv4Pair.Destination := filter.IPv4Filter.DestinationAddress;
                                  IPv4Pair.normalize;

                                  IPv4Conv := GetIPv4ConversationEntry (IPv4Pair);

                                  if not IPv4Conv.IPv4AddressPair.Source.isZeroAddress then
                                     begin
                                          Filter.HelpFirstSeen.initialize (IPv4Conv.FirstFileID, IPv4Conv.FirstFrame, IPv4Conv.FirstFrameOffset);
                                          Filter.HelpLastSeen.initialize (IPv4Conv.LastFileID, IPv4Conv.LastFrame, IPv4Conv.LastFrameOffset);
                                     end
                                  else
                                      begin
                                           Filter.HelpFirstSeen.initialize;
                                           Filter.HelpLastSeen.initialize;
                                      end;
                             end;
      csrIPv6conversation  : begin
                                  IPv6Pair.initialize;
                                  IPv6Pair.Source := filter.IPv6Filter.SourceAddress;
                                  IPv6Pair.Destination := filter.IPv6Filter.DestinationAddress;
                                  IPv6Pair.normalize;

                                  IPv6Conv := GetIPv6ConversationEntry (IPv6Pair);

                                  if not IPv6Conv.IPv6AddressPair.Source.IsUnspecifiedAddress then
                                     begin
                                          Filter.HelpFirstSeen.initialize (IPv6Conv.FirstFileID, IPv6Conv.FirstFrame, IPv6Conv.FirstFrameOffset);
                                          Filter.HelpLastSeen.initialize (IPv6Conv.LastFileID, IPv6Conv.LastFrame, IPv6Conv.LastFrameOffset);
                                     end
                                  else
                                      begin
                                           Filter.HelpFirstSeen.initialize;
                                           Filter.HelpLastSeen.initialize;
                                      end;
                             end;
      csrTCPConversation,
      csrUDPConversation,
      csrL4Conversation    : begin
                                  Filter.HelpFirstSeen.initialize;
                                  Filter.HelpLastSeen.initialize;

                                  if filter.IPv6Filter.AddressFilterType = ftSingle then
                                     IPVersion := 6
                                  else
                                      IPVersion := 4;

                                  if IPversion = 4 then
                                     begin
                                          IPv4Sock.initialize;
                                          IPv4Sock.Source.IPAddress := filter.IPv4Filter.SourceAddress;
                                          IPv4Sock.Destination.IPAddress := filter.IPv4Filter.DestinationAddress;
                                          IPv4Sock.Source.Port := filter.PortFilter.SourcePort;
                                          IPv4Sock.Destination.Port := filter.PortFilter.DestinationPort;
                                          IPv4Sock.normalize;

                                          TCPConv := getTCPConversationEntry (IPv4Sock);
                                          UDPConv := getUDPConversationEntry (IPv4Sock);
                                     end
                                  else
                                      begin
                                          IPv6Sock.initialize;
                                          IPv6Sock.Source.IPAddress := filter.IPv6Filter.SourceAddress;
                                          IPv6Sock.Destination.IPAddress := filter.IPv6Filter.DestinationAddress;
                                          IPv6Sock.Source.Port := filter.PortFilter.SourcePort;
                                          IPv6Sock.Destination.Port := filter.PortFilter.DestinationPort;
                                          IPv6Sock.normalize;

                                          TCPConv := getTCPConversationEntry (IPv6Sock);
                                          UDPConv := getUDPConversationEntry (IPv6Sock);
                                      end;

                                  if not TCPConv.SocketPair.Source.IPAddress.isZeroAddress then
                                     begin
                                          Filter.HelpFirstSeen.initialize (TCPConv.FirstFileID, TCPConv.FirstFrame, TCPConv.FirstFrameOffset);
                                          Filter.HelpLastSeen.initialize (TCPConv.LastFileID, TCPConv.LastFrame, TCPConv.LastFrameOffset);
                                     end
                                  else
                                      if not UDPConv.SocketPair.Source.IPAddress.isZeroAddress then
                                         begin
                                              Filter.HelpFirstSeen.initialize (UDPConv.FirstFileID, UDPConv.FirstFrame, UDPConv.FirstFrameOffset);
                                              Filter.HelpLastSeen.initialize (UDPConv.LastFileID, UDPConv.LastFrame, UDPConv.LastFrameOffset);
                                         end;
                             end;
     end;
end;

(* ------------------------------------------- *)

function tConversationStorage.LastFrameStatusType (const FramePosition: tFileFrameLocation): tConversationStoreRecTypeSet;

var key : string;

begin
     // Make sure the records have been updated

     if self.HasRecords and (priv_dictLastFrames.Count = 0) then
        begin
             SiMain.LogWarning ('Warning: accessing LastFrameStatusType without having initialized the directory for it first (tConversationStorage.LastFrameStatusType)');
             UpdateFirstAndLastDictionaries;
        end;

     key := FramePosition.DictionaryKey;

     if priv_dictLastFrames.ContainsKey (key) then
        result := priv_dictLastFrames.Items[key]
     else
         result := [];
end;

(* ------------------------------------------- *)

procedure tConversationStorage.StoreConversationsToDatabase (Database: tFDConnection; Filename: string; const FileSize : integer; FileDate : tDateTime; const FirstFrameTime, LastFrameTime : tFrameTimeStamp);

var FileStatus : tFileIntelStatus;
    Query      : tFDQuery;

begin
     SiMain.EnterMethod ('tConversationStorage.StoreConversationsToDatabase');

     priv_database := Database;

     FileStatus := getIntelDBFileStatus (priv_database, Filename, FileSize, FileDate, FirstFrameTime, LastFrameTime);

     if DBFileEntryMatchFound (FileStatus) then
        begin
             priv_filedbindex := FileStatus.DBIndex;

             StoreMACEndpointsToDB;
             StoreMACConversationsToDB;
             StoreIPv4EndpointsToDB;
             StoreIPv4ConversationsToDB;
             StoreIPv6EndpointsToDB;
             StoreIPv6ConversationsToDB;
             StoreTCPConversationsToDB;
             StoreUDPConversationsToDB;
             StoreNameResolutionsToDB;

             // Mark file as "conversations scanned"

             Query := tFDQuery.Create (nil);
             Query.Connection := Database;

             Query.SQL.Text := 'UPDATE filelist SET conversationsscanned=:conversationsscanned WHERE id=:id;';
             Query.ParamByName ('id').AsInteger := priv_filedbindex;
             Query.ParamByName ('conversationsscanned').AsBoolean := true;
             Query.ExecSQL;

             Query.Free;
        end;

     SiMain.LeaveMethod ('tConversationStorage.StoreConversationsToDatabase');
end;

(* ------------------------------------------- *)

procedure tConversationStorage.LoadConversationsFromDatabase (Database: tFDConnection; Filename: string; const FileSize : integer; FileDate : tDateTime; const FirstFrameTime, LastFrameTime : tFrameTimeStamp);

var FileStatus : tFileIntelStatus;

begin
     SiMain.EnterMethod ('tConversationStorage.LoadConversationsFromDatabase');

     priv_database := Database;

     FileStatus := getIntelDBFileStatus (priv_database, Filename, FileSize, FileDate, FirstFrameTime, LastFrameTime);

     if DBFileEntryMatchFound (FileStatus) then
        begin
             priv_filedbindex := FileStatus.DBIndex;

             ReadMACEndpointsFromDB;
             ReadMACConversationsFromDB;
             ReadIPv4EndpointsFromDB;
             ReadIPv6EndpointsFromDB;
             ReadIPv4ConversationsFromDB;
             ReadIPv6EndpointsFromDB;
             ReadIPv6ConversationsFromDB;
             ReadTCPConversationsFromDB;
             ReadUDPConversationsFromDB;
             ReadNameResolutionsFroMDB;
        end;

     SiMain.LeaveMethod ('tConversationStorage.LoadConversationsFromDatabase');
end;

(* ------------------------------------------- *)

procedure tConversationStorage.LoadFirstFileConversationsFromDatabase (Database: tFDConnection);

var Query          : tFDQuery;
    Filename       : string;
    FileSize       : int64;
    FileDate       : tDateTime;
    FirstFrameTime : tFrameTimeStamp;
    LastFrameTime  : tFrameTimeStamp;

begin
     assert (assigned (Database) and Database.Connected, 'Database not assigned or not open (tConversationStorage.LoadConversationsFromDatabase)');

     // get the first file in the filelist (there should be only one anyway

     Query := tFDQuery.Create (nil);
     Query.Connection := Database;

     Query.SQL.Text := 'SELECT * FROM filelist;';
     Query.Open;

     if Query.RecordCount > 0 then
        begin
             Filename := Query.FieldByName ('filename').AsString;
             FileSize := Query.FieldByName ('filesize').AsLargeInt;
             FileDate := Query.FieldByName ('filedate').AsDateTime;
             FirstFrameTime.datetime := Query.FieldByName ('firstframetimestamp').AsDateTime;
             FirstFrameTime.nanoseconds := Query.FieldByName ('firstframenanoseconds').AsInteger;
             LastFrameTime.datetime := Query.FieldByName ('lastframetimestamp').AsDateTime;
             LastFrameTime.nanoseconds := Query.FieldByName ('lastframenanoseconds').AsInteger;

             LoadConversationsFromDatabase (Database, Filename, FileSize, FileDate, FirstFrameTime, LastFrameTime);
        end;

     Query.Free;
end;

(* ------------------------------------------- *)

end.

